/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <stdafx.h>
#include "BootSpine.h"
#include "MBasics.h"
#include "toolbars.h"

STDMETHODIMP BootSpine::InitializeOther(IUnknown *montage) {

	CComPtr<IMoniker> mkToolLayer;
	ItemMoniker mkToolbars(L"Toolbars");

	MBasics mbMain;
	RUN(mbMain.init(montage));

    CComPtr<IGroup> toolGroup;
	RUN(mkToolbars->BindToObject(mbMain.bctx,mbMain.mkG,riidret(&toolGroup)));

	MBasics mbTools;
	RUN(mbTools.init(toolGroup));

	ItemMoniker mkToolDrawer(L"Toolbar Drawer");
	ItemMoniker mkToolLayerP(L"Tool Layer");
	RUN(mkToolLayerP->ComposeWith(mbTools.goSubject,false,&mkToolLayer));
	ItemMoniker mkOTo(L"Tool Site Ordering");

	CComPtr<IMCCSiteLayoutEvents> toolDrawer;
	RUN(mkToolDrawer->BindToObject(mbTools.bctx,mbTools.mkG,riidret(&toolDrawer)));
	CComPtr<IMCCLayer> toolLayer;
	RUN(mkToolLayer->BindToObject(mbTools.bctx,mbTools.mkG,riidret(&toolLayer)));

	Toolness toolness(mbTools);

	CComPtr<IMCCButtonSet> playButtons;
	RUN(playButtons.CoCreateInstance(CLSID_MCCButtonSet));
	CComPtr<IMoniker> mkPlayButtons;
	RUN(mbTools.add(L"Play Buttons",playButtons,keep_configuration,&mkPlayButtons));
	RECT rcPlay = {200,0,296,32};
	RUN(toolness.createToolbar(L"Play Buttons",rcPlay,playButtons,mkPlayButtons));

	CComPtr<IMontageContainer> document;
    CComPtr<IMoniker> mkUp;
    RUN(mbTools.g->Up(&mkUp));
	RUN(mbTools.bind(ComposeMk(mkUp,mbTools.mk),&document));
	CComPtr<IMCCPlayControls> playButtonBrain;
	RUN(playButtonBrain.CoCreateInstance(CLSID_MCCPlayControls));
	RUN(playButtonBrain->Initialize(playButtons));

	CComPtr<IHoldSomePointers> hsp;
	RUN(hsp.CoCreateInstance(CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkPlayButtonBrain;
	RUN(mbTools.add(L"Play Button Brain",hsp,keep_configuration,&mkPlayButtonBrain));
	RUN(hsp->Push(mkPlayButtons));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,playButtonBrain));

	ItemMoniker mkDynagraph(L"Graph Layout Engine");
    CComPtr<IDGLayout> dynagraph;
	RUN(mkDynagraph->BindToObject(mbMain.bctx,mbMain.mkG,riidret(&dynagraph)));
    CComPtr<IDGIncrClient> incrClient;
	RUN(incrClient.CoCreateInstance(CLSID_DGIncrClient));
	CComPtr<IMoniker> mkIncrClient;
	RUN(mbMain.add(L"IncrClient",incrClient,keep_configuration,&mkIncrClient));

	CComPtr<IDGMSpine> spine;
	RUN(spine.CoCreateInstance(CLSID_DGMSpine));
	RUN(spine->Initialize(mbMain.c,dynagraph,incrClient,playButtonBrain));

	hsp.Release();
	RUN(hsp.CoCreateInstance(CLSID_SomePointersPersistor));
	RUN(mbMain.add(L"DGM Spine",hsp,keep_configuration,0));
	RUN(hsp->Push(mbMain.mk));
	RUN(hsp->Push(mkDynagraph));
	RUN(hsp->Push(mkIncrClient));
	RUN(hsp->Push(ComposeMk(ComposeMk(mkToolbars,mkPlayButtonBrain),mbMain.goSubject)));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,spine));

	return S_OK;
}