/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// DGMBoot.cpp : Implementation of DGMBoot and DLL registration.

#include "stdafx.h"
#include "DGMBoot.h"

#include <strstream>
#include "RegCategory.h"
#include "dgm_ids.h"
#include "ChooseDGEngine.h"
#include "CvtMetrics.h"
#include "MBasics.h"
#include "installMode.h"


LPCTSTR g_szMiscKey = "OleMisc";
LPCOLESTR g_szCatDesc  = L"Montage Application";
STDMETHODIMP BootDG::InitializeOther(IUnknown *munk) {
	METHTRACE(BootDG,InitializeOther);

	MBasics mb;
	RUN(mb.init(munk));

	RUN(mb.c->put_ClassID(CLSID_DynagraphControl)); // re-tag
	RUN(mb.p->put_Class(CLSID_DynagraphControl)); // re-tag

	// create monikers for the things we expect to be already loaded
	CComPtr<IMoniker> mkMainLayer,mkPalette,mkNodeSource,mkTemp;
	ItemMoniker mkReader(L"DataObject Reader");
	RUN(ItemMoniker(L"Main Layer")->ComposeWith(mb.goSubject,false,&mkMainLayer));

	CComPtr<IMCCDataObjectUnpacker> reader;
	RUN(mkReader->BindToObject(mb.bctx,mb.mkG,riidret(&reader)));

	// look for local node source then look for palette...  really these specifics should be parameters
	mkNodeSource = ItemMoniker(L"Node Source");
	CComPtr<IDataObject> nodeSource;
	if(FAILED(mkNodeSource->BindToObject(mb.bctx,mb.mkG,riidret(&nodeSource)))) {
		mkNodeSource.Release();
        mkNodeSource = ComposeMk(ComposeMk(ItemMoniker(L"Toolbars"),ItemMoniker(L"Palette")),ItemMoniker(L"Data Buffer"));
		if(FAILED(mkNodeSource->BindToObject(mb.bctx,mb.mkG,riidret(&nodeSource))))
			ERR(E_PENDING); // whatever
	}

    CComBSTR engines;
    CComPtr<IDGStringAttributes> layoutChoices;
    RUN(layoutChoices.CoCreateInstance(CLSID_DGLayoutChoices));
    CComBSTR defaultLayout;
    RUN(layoutChoices->get_Name(&defaultLayout));
    if(!defaultLayout.Length())
        defaultLayout = "DynaDAG";
    RUN(layoutChoices->GetAttr(defaultLayout,&engines));
	CComPtr<IDGLayout> engine;
	RUN(CREATE(engine,CLSID_DynagraphEngine));
    RUN(CComQIPtr<IDGStringAttributes>(engine)->SetAttr(CComBSTR("engines"),engines));
    RUN(engine->put_Resolution(pointd(0.1,0.25)));
    RUN(engine->put_Separation(pointd(0.5,1.0)));
    if(m_initEng)
        RUN(m_initEng->InitializeOther(engine));
	RUN(engine->Initialize());

	// make eng persistent
	CComPtr<IMoniker> mkEngine;
	RUN(mb.add(L"Graph Layout Engine",engine,keep_configuration,&mkEngine));

	// default edge (in lieu of real data transfer)
	CComPtr<DIMCCDataObject> edgeDO;
	RUN(CREATE(edgeDO,CLSID_MCCDataObjectPersistent));
	// persist data object
	CComPtr<IMoniker> mkEdgeSource;
	RUN(mb.add(L"Edge Source",edgeDO,dont_keep,&mkEdgeSource,true));
	// get SimpleEdge into data object.
	CComPtr<IUnknown> edgeProto;
	RUN(CREATE(edgeProto,CLSID_DGMSimpleEdge));
	RUN(edgeDO->CreateEmbeddedObject(edgeProto));

	// default label
	CComPtr<DIMCCDataObject> labelDO;
	RUN(CREATE(labelDO,CLSID_MCCDataObjectPersistent));
	// persist data object
	CComPtr<IMoniker> mkLabelSource;
	RUN(mb.add(L"Label Source",labelDO,dont_keep,&mkLabelSource,true));
	// create control
	CComPtr<IMCCTextEdit> textEdit;
	RUN(CREATE(textEdit,CLSID_MCCTextEdit));
	RUN(labelDO->CreateEmbeddedObject(textEdit));

	// DGMDrawer
	CComPtr<IDGMDrawer> drawer;
	RUN(CREATE(drawer,CLSID_DGMDrawer));
	RUN(drawer->Initialize(mb.c,engine,nodeSource,CComQIPtr<IDataObject>(edgeDO),
		CComQIPtr<IDataObject>(labelDO),reader,mkLabelSource,mkReader));

	// drawer's
	CComPtr<IHoldSomePointers> hsp;
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkDrawerP, mkDrawer;
	RUN(mb.add(L"Graph Drawer",hsp,keep_configuration,&mkDrawerP));
	RUN(mkDrawerP->ComposeWith(mb.goSubject,false,&mkDrawer));
	RUN(hsp->Push(mb.mk));
	RUN(hsp->Push(mkEngine));
	RUN(hsp->Push(mkNodeSource));
	RUN(hsp->Push(mkEdgeSource));
	RUN(hsp->Push(mkLabelSource));
	RUN(hsp->Push(mkReader));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,drawer));
	hsp.Release();

	// connect engine to montage so that the events of each get translated to the other
	// (currently this consists of the GroupOp event holding the engine)
	CComPtr<IDGMEngineConnector> dgmc;
	RUN(CREATE(dgmc,CLSID_DGMEngineConnector));
	RUN(dgmc->Initialize(mb.c,engine,drawer));
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Dynagraph-Montage Connector",hsp,keep_configuration,0));
	RUN(hsp->Push(mb.mk));
	RUN(hsp->Push(mkEngine));
	RUN(hsp->Push(mkDrawer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,dgmc));
	hsp.Release();

	return S_OK;
}
HRESULT BootShapeNodeNodeSource::InitializeOther(IUnknown *mper) {
    MBasics mb;
    RUN(mb.init(mper));
	// give it shapenode proto for its drawer
	CComPtr<DIMCCDataObject> nodeDO;
	RUN(CREATE(nodeDO,CLSID_MCCDataObjectPersistent));
	// persist data object
	CComPtr<IMoniker> mkNodeSource;
	RUN(mb.add(L"Node Source",nodeDO,dont_keep,&mkNodeSource,true));
	// get ShapeNode into data object.
	CComPtr<IUnknown> nodeProto;
	RUN(CREATE(nodeProto,CLSID_ShapeNode));
	RUN(CComQIPtr<IPersistStreamInit>(nodeProto)->InitNew());
	RUN(nodeDO->CreateEmbeddedObject(nodeProto));
    return S_OK;
}
struct InitDGPalette : CComObjectRoot, IInitializeOther {
    BEGIN_COM_MAP(InitDGPalette)
        COM_INTERFACE_ENTRY(IInitializeOther)
    END_COM_MAP()
    STDMETHODIMP InitializeOther(IUnknown *other) {
        CComQIPtr<IDGStringAttributes> sattrs = other;
        // dynadag is a decent way to lay out the palette: tends to put in row with okay spacing
        RUN(sattrs->SetAttr(CComBSTR("engines"),CComBSTR("shapegen,dynadag,labels")));
        return S_OK;
    }
};
HRESULT DGMBoot::InitializeOther(IUnknown *munk) {
    METHTRACE(DGMBoot,InitializeOther);
	// start with manual layout; add dynagraph
	CComPtr<IInitializeOther> ioManual;
	RUN(CREATE(ioManual,CLSID_MCCBootStandard));
	RUN(ioManual->InitializeOther(munk));
	ioManual.Release();
	CComPtr<IInitializeOther> ioDynagraph;
	RUN(CREATE(ioDynagraph,CLSID_DGMBootDynagraph));
	RUN(ioDynagraph->InitializeOther(munk));

	MBasics mb;
	RUN(mb.init(munk));

	// add dynagraph to palette
	CComPtr<IMoniker> mkPalette = ComposeMk(ItemMoniker(L"Toolbars"),ItemMoniker(L"Palette"));
	CComPtr<IMontagePersistor> palper;
	RUN(ComposeMk(mkPalette,mb.mkP)->BindToObject(mb.bctx,mb.mkG,riidret(&palper)));
	MBasics mbPalette;
	RUN(mbPalette.init(palper));

    CComPtr<IInitializeOther> bootNodeSource;
    RUN(bootNodeSource.CoCreateInstance(CLSID_DGMBootShapeNodeNodeSource));
    RUN(bootNodeSource->InitializeOther(palper));

    CComPtr<IInitializeOther> initDGPalette = new CComObject<InitDGPalette>;
    RUN(CComQIPtr<IDGMBootDynagraph>(ioDynagraph)->put_EngineInitializer(initDGPalette));
	RUN(ioDynagraph->InitializeOther(palper));

	CComPtr<IMoniker> mkDrawer,mkPaletteSelection,mkMainLayer,mkModeLayer,
		mkActiveLayer,mkPaletteDataSet,mkActivator,
		mkCEM,mkPaletteContainer,mkPaletteMainLayer;
	RUN(ItemMoniker(L"Graph Drawer")->ComposeWith(mbPalette.goSubject,false,&mkDrawer));
	CComPtr<IDGMDrawer> drawer;
	RUN(mkDrawer->BindToObject(mb.bctx,mb.mkG,riidret(&drawer)));
	ItemMoniker mkSelection(L"Selection");
	CComPtr<IItemSet> selection;
	RUN(mkSelection->BindToObject(mb.bctx,mb.mkG,riidret(&selection)));

	RUN(ItemMoniker(L"Main Layer")->ComposeWith(mb.goSubject,false,&mkMainLayer));

	RUN(mkPalette->ComposeWith(mb.mk,false,&mkPaletteContainer));
	RUN(mkPalette->ComposeWith(mkMainLayer,false,&mkPaletteMainLayer));

	RUN(ItemMoniker(L"Mode Layer")->ComposeWith(mb.goSubject,false,&mkModeLayer));
	RUN(ItemMoniker(L"Active Layer")->ComposeWith(mb.goSubject,false,&mkActiveLayer));
	CComPtr<IMCCLayer> mainl,model,activel,mainlPalette;
	RUN(mkMainLayer->BindToObject(mb.bctx,mb.mkG,riidret(&mainl)));
	RUN(mkModeLayer->BindToObject(mb.bctx,mb.mkG,riidret(&model)));
	RUN(mkActiveLayer->BindToObject(mb.bctx,mb.mkG,riidret(&activel)));
	RUN(mkPaletteMainLayer->BindToObject(mb.bctx,mb.mkG,riidret(&mainlPalette)));

	RUN(mkPalette->ComposeWith(mkSelection,false,&mkPaletteSelection));
	CComPtr<IItemSet> paletteSelection;
	RUN(mkPaletteSelection->BindToObject(mb.bctx,mb.mkG,riidret(&paletteSelection)));
	CComPtr<IGroup> paletteGroup;
	RUN(mkPalette->BindToObject(mb.bctx,mb.mkG,riidret(&paletteGroup)));
	CComPtr<IMontageContainer> paletteContainer;
	RUN(mkPaletteContainer->BindToObject(mb.bctx,mb.mkG,riidret(&paletteContainer)));

	RUN(ItemMoniker(L"Selection Activator")->ComposeWith(mb.goSubject,false,&mkActivator));
	CComPtr<IMCCItemSetActivator> activator;
	if(FAILED(mkActivator->BindToObject(mb.bctx,mb.mkG,riidret(&activator))))
		mkActivator.Release();

	RUN(ItemMoniker(L"Edit Mode")->ComposeWith(mb.goSubject,false,&mkCEM));
	CComPtr<IContainerEditMode> editMode;
	RUN(mkCEM->BindToObject(mb.bctx,mb.mkG,riidret(&editMode)));

	ItemMoniker mkSwitcher(L"Mode Switcher");
	CComPtr<IMCCModeSwitcher> switcher;
	RUN(mkSwitcher->BindToObject(mb.bctx,mb.mkG,riidret(&switcher)));

	// install DOT import
	CComPtr<IHoldSomePointers> hsp;
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"DOT File Format",hsp,keep_configuration,0));
	RUN(hsp->Push(mkDrawer));
	RUN(hsp->Push(mkMainLayer));
	CComPtr<IMoniker> mkFP;
	RUN(mb.p->get_FilePortMk(&mkFP));
	RUN(hsp->Push(mkFP));
	CComPtr<IMCCFileFormat> dotmat; // has no init method right now so use Realize This
	RUN(CREATE(dotmat,CLSID_DGMDOTFileFormat));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(false,dotmat));
	RUN(CComQIPtr<IGroupItem>(hsp)->Realize(mb.bctx));
	hsp.Release();

	// graph draw mode.  man this is cryptic.
	CComPtr<IDGMGraphDrawMode> drawm;
	RUN(CREATE(drawm,CLSID_DGMGraphDrawMode));
	RUN(drawm->Initialize(selection,activator,drawer,editMode));
	RUN(drawm->AddLayer(activel));
	RUN(drawm->AddLayer(mainl));

	// mode's persistor
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkGDMP,mkGDM;
	RUN(mb.add(L"Graph Draw Mode",hsp,dont_keep,&mkGDMP));
	RUN(mkGDMP->ComposeWith(mb.goSubject,false,&mkGDM));
	RUN(hsp->Push(mkSelection));
	RUN(hsp->Push(mkActivator));
	RUN(hsp->Push(mkDrawer));
	RUN(hsp->Push(mkCEM));
	RUN(hsp->Push(mkActiveLayer));
	RUN(hsp->Push(mkMainLayer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,drawm));
	hsp.Release();

    RUN(installMode(mb,drawm,mkGDMP,model,mkModeLayer,switcher,mkSwitcher,
        CComBSTR("Draw"),CComBSTR("Graph Draw Mode"),true));

	CComPtr<IDGMDrawer> palDrawer;
	RUN(mkDrawer->BindToObject(mbPalette.bctx,mbPalette.mkG,riidret(&palDrawer)));
	CComPtr<IDGLayout> eng;
	RUN(palDrawer->get_Engine(&eng));

	double w = 1.0, h = 1.0;
	CComPtr<IDGNode> n;
	RUN(eng->new_Node(0,&n));
	CComQIPtr<DIDGShapeDefinition> shape = n;

	RUN(shape->SetOutside(w,h));
	RUN(shape->put_IsEllipse(VARIANT_TRUE));
	CComPtr<IMCCSitePersistor> sp;
	RUN(palDrawer->NewNode(CComQIPtr<IDGDrawnNode>(n),0,0,0,&sp));
	// put this one in the selection
	CComPtr<IUnknown> us;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&us));
	CComPtr<IMoniker> mkSP;
	RUN(CComQIPtr<IMCCSite>(us)->get_RelMoniker(&mkSP));
	RUN(paletteSelection->Add(mkSP));
	n.Release();

	RUN(eng->new_Node(0,&n));
	shape = n;
	RUN(shape->SetOutside(w,h));
	RUN(shape->put_IsEllipse(VARIANT_FALSE));
	RUN(shape->put_Sides(4));
	RUN(palDrawer->NewNode(CComQIPtr<IDGDrawnNode>(n),0,0,0,0));
	n.Release();

	RUN(eng->new_Node(0,&n));
	shape = n;
	RUN(shape->SetOutside(w,h));
	RUN(shape->put_IsEllipse(VARIANT_FALSE));
	RUN(shape->put_Sides(4));
	RUN(shape->put_Skew(0.05f));
	RUN(palDrawer->NewNode(CComQIPtr<IDGDrawnNode>(n),0,0,0,0));
	n.Release();

	RUN(eng->new_Node(0,&n));
	shape = n;
	RUN(shape->SetOutside(w,h));
	RUN(shape->put_IsEllipse(VARIANT_TRUE));
	RUN(shape->put_Distortion(0.4f));
	RUN(shape->put_Rotation(3.1415926536f/2.0f));
	RUN(palDrawer->NewNode(CComQIPtr<IDGDrawnNode>(n),0,0,0,0));
	n.Release();

	// replace the Save As file description
	CComPtr<IMCCFileFormat> fport;
	RUN(mb.c->get_FilePort(&fport));
	RUN(CComQIPtr<IMCCFilePort>(fport)->RegisterNativeFileFormat(MFP_IMPORT,L"dgm:Dynagraph Diagram"));
	RUN(CComQIPtr<IMCCFilePort>(fport)->RegisterNativeFileFormat(MFP_EXPORT,L"dgm:Dynagraph Diagram"));

	/*
	// add play controls and incr response stuff
	CComPtr<IInitializeOther> bootSpine;
	RUN(bootSpine.CoCreateInstance(CLSID_DGMBootSpine));
	RUN(bootSpine->InitializeOther(munk));
	*/

	return S_OK;
}
