/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DOTFileFormat.h"
#include "createAffixMoniker.h"

#include <fstream>
#include <map>

extern "C" {
#include "..\..\graphviz\lib\agraph\agraph.h"
};

#include "allocDGShapes.h"


DOTFileFormat::DOTFileFormat() {
	CONSTRUCTORTRACE(DOTFileFormat);
}
DOTFileFormat::~DOTFileFormat() {
	DESTRUCTORTRACE(DOTFileFormat);
}
STDMETHODIMP DOTFileFormat::TakePointers(IEnumUnknown *eu,IEnumMoniker *ms) {
    METHTRACE(DOTFileFormat,TakePointers);
	CComPtr<IUnknown> u;
	RUN(eu->Next(1,&u,0));
	RUN(u->QueryInterface(riidret(&m_drawer)));
	u.Release();
	RUN(eu->Next(1,&u,0));
	RUN(u->QueryInterface(riidret(&m_mainLayer)));
	u.Release();
	RUN(eu->Next(1,&u,0));
	CComPtr<IMCCFilePort> port;
	RUN(u->QueryInterface(riidret(&port)));
	RUN(port->RegisterFileFormat(MFP_IMPORT,L"dot:Dot Graph Description",this));
	RUN(port->RegisterFileFormat(MFP_EXPORT,L"dot:Dot Graph Description",this));
	return S_OK;
}

STDMETHODIMP DOTFileFormat::Import(LPCOLESTR name) {
    METHTRACE(DOTFileFormat,Import);
    USES_CONVERSION;
    FILE *f = fopen(OLE2A(name),"r");
    fseek(f,0,SEEK_END);
    long size = ftell(f);
    fseek(f,0,SEEK_SET);
    HGLOBAL dat = GlobalAlloc(GHND,size);
    void *p = GlobalLock(dat);
    fread(p,size,1,f);
    GlobalUnlock(dat);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RUN(eng->LoadDotfile(dat,VARIANT_FALSE,VARIANT_FALSE));
    GlobalFree(dat);
    fclose(f);
    return S_OK;
}
STDMETHODIMP DOTFileFormat::Export(LPCOLESTR name) {
    METHTRACE(DOTFileFormat,Export);
    USES_CONVERSION;
    FILE *f = fopen(OLE2A(name),"w");
    if(!f)
        ERR(E_FAIL);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
    HGLOBAL dat;
	RUN(eng->SaveDotfile(&dat));
    void *p = GlobalLock(dat);
    fwrite(p,GlobalSize(dat),1,f);
    GlobalUnlock(dat);
    GlobalFree(dat);
    fclose(f);
    return S_OK;
}
