/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Drawer.h"
#include "createAffixMoniker.h"
#include "combinePos.h"
#include "siteControl.h"
#include "labelNodeSite.h"

Drawer::Drawer() : m_labelLength(-1) {
	CONSTRUCTORTRACE(Drawer);
}
Drawer::~Drawer() {
	DESTRUCTORTRACE(Drawer);
}
CComPtr<IItemSet> Drawer::dataSet() {
    METHTRACE(Drawer,dataSet);
	CComPtr<IItemSet> dataset;
	RUN(CComQIPtr<IOleItemContainer>(mc())->GetObject(L"Data",BINDSPEED_INDEFINITE,BindCtx(),riidret(&dataset)));
	return dataset;
}

// IDGMDrawer
STDMETHODIMP Drawer::Initialize(IMontageContainer *mc,IDGLayout *engine,IDataObject *nodeSource,
								IDataObject *edgeSource,IDataObject *labelSource,IMCCDataObjectUnpacker *reader,
								IMoniker *mkLabelSource, IMoniker *mkReader) {
	METHTRACE(Drawer,Initialize);
    RUN(mc->get_RefObject(&m_mref));
	DWORD macaroon; // no need to disconnect EVER
	RUN(APCAdvise(mc,(IMCCSiteLayoutEvents*)this,IID_IMCCSiteLayoutEvents,&macaroon));
	m_engine = engine;
	m_nodeSource = nodeSource;
	m_edgeSource = edgeSource;
	m_labelSource = labelSource;
	m_reader = reader;
	m_mkLabelSource = mkLabelSource;
	m_mkReader = mkReader;
	return S_OK;
}
STDMETHODIMP Drawer::get_Engine(IDGLayout **ret) {
    METHTRACE(Drawer,get_Engine);
	m_engine.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Drawer::get_Container(IMontageContainer **ret) {
    METHTRACE(Drawer,get_Container);
    RUN(mc().CopyTo(ret));
	return S_OK;
}
STDMETHODIMP Drawer::get_NodeSource(IDataObject **ret) {
    METHTRACE(Drawer,get_NodeSource);
	m_nodeSource.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Drawer::get_LabelLength(int *ret) {
    *ret = m_labelLength;
	return S_OK;
}
STDMETHODIMP Drawer::put_LabelLength(int val) {
    m_labelLength = val;
	return S_OK;
}

CComPtr<IMoniker> persistLayoutNode(IGroup *g,IDGNode *node,LPOLESTR baseName) {
    FUNTRACE(persistLayoutNode);
	CComPtr<IDGNodePersistor> np;
	SRUN(CREATE(np,CLSID_DGNodePersistor));
	CComPtr<IMoniker> mkNP0,mkNP;
	if(baseName)
		SRUN(createAffixMoniker(baseName,L"Node",&mkNP0));
	SRUN(g->Paste(BindCtx(),GP_SrcIsObj,np,mkNP0,IID_NULL,0,&mkNP));
	// HACK!!!  Drawer should certainly NOT know the name of the engine!
	// but there's this endless problem of the one-way pointer from mk->
	// persistor->subject.
	SRUN(np->put_MkEngine(ItemMoniker(L"Graph Layout Engine")));
	SRUN(CComQIPtr<DIPersistor>(np)->SetSubject(true,node));
	return ComposeMk(mkNP,GetSubjectMk());
}
HRESULT Drawer::annotateNode(IBindCtx *bctx, IMCCSitePersistor *sp, IDGDrawnNode *node, LPOLESTR baseName) {
	METHTRACE(Drawer,annotateNode);
	// Create connector & init.
	CComPtr<DIDGMConnector> connector;
	RUN(CREATE(connector,CLSID_DGMNodeConnector));
	RUN(CComQIPtr<INeedOnePointer>(connector)->TakePointer(node,0));  // not strictly necessary

	CComPtr<IMoniker> mkM,mkE,mkNP0,mkNP,mkN,mkConnectorP0,mkConnectorP,mkSite;

	CComPtr<IGroup> g;
	RUN(mc()->GetMainGroup(riidret(&g)));

	// create anon. moniker for connector
	CComPtr<IHoldOnePointer> hop;
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Node Connector",&mkConnectorP0));
	RUN(g->Paste(bctx,GP_SrcIsObj,hop,mkConnectorP0,IID_NULL,0,&mkConnectorP));
	RUN(hop->put_Moniker(persistLayoutNode(g,node,baseName)));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,connector));

	// add connector to init list of site persistor, after any initialization
	CComPtr<IOrderedItemList> oil;
	RUN(sp->get_InitList(&oil));
	RUN(oil->SetMoniker(20,ComposeMk(mkConnectorP,GetSubjectMk()))); // 20 is magic layout order
	return S_OK;
}
HRESULT Drawer::addLabelMode(IBindCtx *bctx,IMCCSitePersistor *sp,IUnknown *layout,LPOLESTR baseName,IDGMLabelMode **ret) {
    METHTRACE(Drawer,addLabelMode);
	CComPtr<IUnknown> us;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&us));
	CComQIPtr<IMCCSite> site(us);
	CComPtr<IDGMLabelMode> mode = labelModeFromSite(site);
	if(mode) {
		RUN(mode->put_LayoutObject(layout));
		mode.CopyTo(ret);
		return S_FALSE;
	}
	CComPtr<IGroup> g;
	RUN(mc()->GetMainGroup(riidret(&g)));

	CComPtr<IMCCLayer> layer;
	RUN(site->get_Layer(&layer));
	RECTL rc;
	bool paused = FAILED(site->get_VirtRect(&rc)); // site should be visible unless engine is paused

    RUN(CREATE(mode,CLSID_DGMLabelMode));
	RUN(mode->Initialize(layout,site,m_labelSource,m_reader));

	// persist mode+create site
	CComPtr<DIPersistor> modePer;
	RUN(CREATE(modePer,CLSID_DGMLabelModePersistor));
	CComPtr<IMoniker> mkModePer0,mkModePer,goSubject,mkMode;
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Label Mode",&mkModePer0));
	RUN(g->Paste(bctx,GP_SrcIsObj,modePer,mkModePer0,IID_NULL,0,&mkModePer));
	RUN(CComQIPtr<IDGMLabelModePersistor>(modePer)->Initialize(m_mkLabelSource,m_mkReader));
	RUN(modePer->SetSubject(true,mode));
	RUN(CREATE(goSubject,CLSID_GetSubjectMoniker));
	RUN(mkModePer->ComposeWith(goSubject,false,&mkMode));
	CComPtr<IMoniker> mkModeSite,mkSP;
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Label Mode Site",&mkModeSite));
	MCC_POS_INFO mpi;
	mpi.postype = MCC_POS_CANVASRECT;
	mpi.rcl = rc;
	RUN(siteControl(bctx,mkMode,mkModeSite,layer,paused?0:&mpi,paused?MCCS_HIDDEN:MCCS_SHOWN,0,0,0,&mkSP,0));

	// add label mode's *site* to init list of content's site, so that mode will
	// always get realized with content.
	CComPtr<IMoniker> mkSite = ComposeMk(mkSP,goSubject);
	CComPtr<IOrderedItemList> initContent;
	RUN(sp->get_InitList(&initContent));
	RUN(initContent->SetMoniker(21,mkSite)); // just after layout

	// go!
	CComPtr<IMCCSite> now;
	RUN(mkSite->BindToObject(bctx,PointerMoniker(g),riidret(&now)));
	return S_OK;
}

STDMETHODIMP Drawer::NewNode(IDGDrawnNode *node, MCC_POS_INFO *mpi, LPOLESTR baseName, LPOLESTR caption,IMCCSitePersistor **ret) {
	METHTRACE(Drawer,NewNode);

    CComPtr<IMontageContainer> MC = mc();
	RUN(groupOp(MC,true));
	// get central data transfer unit to extract a site persistor from data object
	CComPtr<IMCCSitePersistor> sp;
	if(S_FALSE==m_reader->UnpackOne(m_nodeSource,0,baseName,dataSet(),&sp)) {
		// A node with this name's already here, so just reposition
		if(mpi && mpi->postype!=MCC_POS_NONE) {
			CComPtr<IUnknown> uS;
			RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&uS));
			RUN(fireSiteEvent(CComQIPtr<IMCCSite>(uS),fireMove(mpi)));
		}
		if(ret)
			sp.CopyTo(ret);
		RUN(groupOp(MC,false));
		return S_FALSE;
	}
	if(!sp) {
		RUN(groupOp(MC,false));
		return E_FAIL;
	}
	// we leave all of the data we've just extracted "frozen" while we fit it to our needs.
	// that is to say, all the objects are still just saved data.  we bind a few monikers
	// and otherwise realize objects, but only persistors and not the actual sites
#ifdef _DEBUG
	// make sure it's not too late to change the data
	CComPtr<IUnknown> badSite;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&badSite));
	_ASSERTE(!badSite);
#endif
    // combine position
	MCCSITEPERSISTORDATA data;
	RUN(sp->get_Data(&data));
	if(mpi) {
		CComPtr<IMCCTransform> t;
		RUN(MC->get_Transform(&t));
		RUN(combine_pos(t,*mpi,data.pos));
	}
	else
		data.pos.postype = MCC_POS_NONE;
	RUN(sp->put_Data(&data));

	CComPtr<IGroup> group;
	RUN(MC->GetMainGroup(riidret(&group)));
	PointerMoniker mkG(group);
	BindCtx bctx;

	// add or replace dgnode
	CComPtr<IDGDrawnNode> n2;
	CComQIPtr<DIDGMConnector> conn;
	if(CComPtr<DIPersistor> conper = connectorPersistorFromMkConnector(mkConnectorFromSitePersistor(sp),PointerMoniker(group))) {
		// the data came with a dgnode
		CComPtr<IUnknown> uc;
		RUN(conper->get_Subject(&uc));
        conn = uc;
		if(node) {
			// replace the dgnode that it came with with the node parameter
			// VERY IMPORTANT to replace all moniker references.  hopefully all the
			// ReleaseStgRefs should cause the excess dgnode to be erased
			// some sort of Group service to replace all occurences of one moniker w/ another
			// would be really nice (in a 2.0 version with persistors rolled into the Group)

			// at this point it would probably be a good idea to merge the data from
			// the two dgnodes rather than just replacing one with another
			n2 = node;
			// one reference is in the node connector & its persistor
			RUN(conn->put_LayoutObject(node));
			CComPtr<IMoniker> mkNode = persistLayoutNode(group,node,baseName);
			RUN(CComQIPtr<IHoldOnePointer>(conper)->put_Moniker(mkNode));
            conn.Release();
			// another reference is in the labelmode & its
			CComPtr<IOrderedItemList> olist;
			RUN(sp->get_InitList(&olist));
			CComPtr<IMoniker> mkLabelModeSite;
			RUN(olist->GetMoniker(21,&mkLabelModeSite)); // as Drawer::addLabelMode specified it.
			CComPtr<IMoniker> anti;
			RUN(CreateAntiMoniker(&anti)); // strip off GetSubjectMoniker
			CComPtr<IMCCSitePersistor> sp;
			RUN(ComposeMk(mkLabelModeSite,anti)->BindToObject(bctx,mkG,riidret(&sp)));
			CComPtr<IMoniker> mkLabelMode;
			RUN(sp->get_ControlMk(&mkLabelMode));
			CComPtr<IDGMLabelModePersistor> labelModePer;
			RUN(ComposeMk(mkLabelMode,anti)->BindToObject(bctx,mkG,riidret(&labelModePer)));
#ifdef _DEBUG
			CComPtr<IUnknown> badMode;
			RUN(CComQIPtr<DIPersistor>(labelModePer)->get_Subject(&badMode));
			_ASSERTE(!badMode);
#endif
			RUN(labelModePer->put_LayoutMk(mkNode));
		}
        // else use the dgnode that came with.
	}
	else {
		// doesn't yet have dgnode
		// if no node passed in, create now.
		if(node)
			n2 = node;
		else {
			CComPtr<IDGNode> n;
			RUN(m_engine->new_Node(CComBSTR(baseName),&n));
			n2 = n;
		}
		RUN(annotateNode(bctx,sp,n2,baseName));
	}
	// and realize
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(bctx));
    if(!n2 && conn) { // couldn't get dgnode until after realize
		CComPtr<IUnknown> un;
		RUN(conn->get_LayoutObject(&un));
		RUN(un.QueryInterface(&n2));
    }

	CComPtr<IDGMLabelMode> labelMode;
	RUN(addLabelMode(bctx,sp,n2,baseName,&labelMode));
	CComQIPtr<IDGStringAttributes> attrs = n2;
	CComBSTR label;
	if(FAILED(attrs->GetAttr(CComBSTR("label"),&label))
		|| (label.m_str[0]=='\\' && label.m_str[1]=='N')) {
		RUN(attrs->get_Name(&label));
		if(label.m_str[0]=='n' && label.m_str[1]=='_')
			label.Empty(); // sheesh, you'd think they'd call this Clear?!
	}
	if(label) {
		CComPtr<IUnknown> su;
		RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&su));
        CComBSTR label2;
        if(m_labelLength) // <0 any length
            if(m_labelLength>0 && int(label.Length())>m_labelLength) {
                label2 = CComBSTR(m_labelLength-1,label);
                label2 += L"";
            }
            else label2 = label;
		RUN(labelNodeSite(su,label2,0));
	}
	if(ret)
		(*ret = sp)->AddRef();
	RUN(groupOp(MC,false));

	return S_OK;
}
STDMETHODIMP Drawer::NewEdge(IDGDrawnEdge *edge, LPOLESTR baseName, LPOLESTR caption, IMCCSitePersistor **ret) {
	METHTRACE(Drawer,NewEdge);
	if(!edge) // required (as it is not for NewNode) because otherwise how do we know the end-nodes?
		ERR(E_INVALIDARG);

    CComPtr<IMontageContainer> MC = mc();
	RUN(groupOp(MC,true));

	// get central data transfer unit to extract a site persistor from data object
	CComPtr<IMCCSitePersistor> sp;
	RUN(m_reader->UnpackOne(m_edgeSource,0,baseName,dataSet(),&sp));

	CComPtr<IMoniker> mkM,mkTailSite,mkHeadSite,
		mkEdgeP0,mkEdgeP,mkEdge,mkConnectorP0,mkConnectorP,mkConnector,mkSite;

	// standard BS
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<IGroup> g;
	RUN(MC->GetMainGroup(riidret(&g)));

	// get monikers of montage, layer, engine
	RUN(MC->get_RelMoniker(&mkM));
	ItemMoniker mkEng(L"Graph Layout Engine");

	// good ol' good ol.
	CComPtr<IMoniker> goSubject;
	RUN(CREATE(goSubject,CLSID_GetSubjectMoniker));

	// persist the layout edge
	CComPtr<IDGEdgePersistor> ep;
	RUN(CREATE(ep,CLSID_DGEdgePersistor));
	if(baseName) {
		RUN(createAffixMoniker(baseName,L"Edge",&mkEdgeP0));
		RUN(ep->put_Name(baseName));
	}
	RUN(g->Paste(bctx,GP_SrcIsObj,ep,mkEdgeP0,IID_NULL,0,&mkEdgeP));
	RUN(mkEdgeP->ComposeWith(goSubject,false,&mkEdge));
	RUN(ep->put_MkEngine(mkEng));

	CComPtr<IDGNode> tail,head;
	RUN(edge->get_TailNode(&tail));
	CComPtr<IMCCSite> tailSite = siteFromNode(CComQIPtr<IDGDrawnNode>(tail));
	RUN(tailSite->get_RelMoniker(&mkTailSite));
	RUN(ep->put_MkTailNode(mkLayoutFromSite(tailSite)));

	RUN(edge->get_HeadNode(&head));
	CComPtr<IMCCSite> headSite = siteFromNode(CComQIPtr<IDGDrawnNode>(head));
	RUN(headSite->get_RelMoniker(&mkHeadSite));
	RUN(ep->put_MkHeadNode(mkLayoutFromSite(headSite)));

	RUN(CComQIPtr<DIPersistor>(ep)->SetSubject(true,edge));

	// create connector, persist that
	CComPtr<DIDGMConnector> connect;
	RUN(CREATE(connect,CLSID_DGMEdgeConnector  ));
	RUN(connect->put_LayoutObject(edge));
	CComPtr<IHoldSomePointers> hsp;
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Edge Connector",&mkConnectorP0));
	RUN(g->Paste(bctx,GP_SrcIsObj,hsp,mkConnectorP0,IID_NULL,0,&mkConnectorP));
	RUN(mkConnectorP->ComposeWith(goSubject,false,&mkConnector));
	RUN(hsp->Push(mkEdge));
	// The connector doesn't actually use these but this makes sure that the node sites are
	// realized (thus inserted into the layout) before the edge
	RUN(hsp->Push(mkTailSite));
	RUN(hsp->Push(mkHeadSite));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,connect));

	// put connector on init list
	CComPtr<IOrderedItemList> oil;
	RUN(sp->get_InitList(&oil));
	RUN(oil->SetMoniker(20,mkConnector)); // 20?

	// bang!
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(bctx));

	RUN(addLabelMode(bctx,sp,edge,baseName,0));

	if(ret)
		(*ret = sp)->AddRef();
	RUN(groupOp(MC,false));
	return S_OK;
}
// DIDGMDrawer
STDMETHODIMP Drawer::Initialize(DIMontageContainer *mc,DIDGLayout *engine,
		IUnknown *nodeSource,IUnknown *edgeSource,IUnknown *labelSource,IUnknown *reader,
		IUnknown *mkLabelSource,IUnknown *mkReader) {
    METHTRACE(Drawer,Initialize);
	CComPtr<IMontageContainer> mc2; if(mc) RUNRET(mc->QueryInterface(riidret(&mc2)));
	CComPtr<IDGLayout> engine2; if(engine) RUNRET(engine->QueryInterface(riidret(&engine2)));
	CComPtr<IDataObject> nodeSource2; if(nodeSource) RUNRET(nodeSource->QueryInterface(riidret(&nodeSource2)));
	CComPtr<IDataObject> edgeSource2; if(edgeSource) RUNRET(edgeSource->QueryInterface(riidret(&edgeSource2)));
	CComPtr<IDataObject> labelSource2; if(labelSource) RUNRET(labelSource->QueryInterface(riidret(&labelSource2)));
	CComPtr<IMCCDataObjectUnpacker> reader2; if(reader) RUNRET(reader->QueryInterface(riidret(&reader2)));
	CComPtr<IMoniker> mkLS2; if(mkLabelSource) RUNRET(mkLabelSource->QueryInterface(riidret(&mkLS2)));
	CComPtr<IMoniker> mkR2; if(mkReader) RUNRET(mkReader->QueryInterface(riidret(&mkR2)));
	return Initialize(mc2,engine2,nodeSource2,edgeSource2,labelSource2,reader2,mkLS2,mkR2);
}
STDMETHODIMP Drawer::NewNode(DIDGDrawnNode *node, BSTR baseName, BSTR caption, DIMCCPosInfo *pos,DIMCCSitePersistor **ret) {
    METHTRACE(Drawer,NewNoe);
	CComPtr<IDGDrawnNode> node2; if(node) RUNRET(node->QueryInterface(riidret(&node2)));
	MCC_POS_INFO mpi;
	if(pos)
		RUN(CComQIPtr<IMCCPosInfo>(pos)->GetStuff(&mpi));
	CComPtr<IMCCSitePersistor> sp;
	RUNRET(NewNode(node2,&mpi,baseName,caption,&sp));
	return sp.QueryInterface(ret);
}
STDMETHODIMP Drawer::NewEdge(DIDGDrawnEdge *edge, BSTR baseName, BSTR caption, DIMCCSitePersistor **ret) {
    METHTRACE(Drawer,NewEdge);
	CComPtr<IDGDrawnEdge> edge2; if(edge) RUNRET(edge->QueryInterface(riidret(&edge2)));
	CComPtr<IMCCSitePersistor> sp;
	RUN(NewEdge(edge2,baseName,caption,&sp));
	return sp.QueryInterface(ret);
}

// IMCCSiteLayoutEvents
STDMETHODIMP Drawer::Process(IMCCSitePersistor *pers, LPOLESTR baseName) {
	METHTRACE(Drawer,Process);
	if(!mkConnectorFromSitePersistor(pers)) { // skip thems that got layout already
		CComPtr<IDGNode> node;
		RUN(m_engine->new_Node(0,&node));
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		return annotateNode(bctx,pers,CComQIPtr<IDGDrawnNode>(node),baseName);
	}
	// modify the node?
	return S_OK;
}
// INeedSomePointers
STDMETHODIMP Drawer::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(Drawer,TakePointers);
	// stupid ass code.  But I'm glad I don't have to write a whole persistor at least!
	CComPtr<IUnknown> temp;
	RUNO(os->Next(1,&temp,0));
	CComQIPtr<IMontageContainer> mc = temp;
	temp.Release();
	RUNO(os->Next(1,&temp,0));
	CComQIPtr<IDGLayout> engine = temp;
	temp.Release();
	RUNO(os->Next(1,&temp,0));
	CComQIPtr<IDataObject> nsource = temp,
		esource,lsource;
	temp.Release();
	RUNO(os->Next(1,&temp,0));
	esource = temp;
	temp.Release();
	RUNO(os->Next(1,&temp,0));
	lsource = temp;
	temp.Release();
	CComQIPtr<IMCCDataObjectUnpacker> reader;
	RUNO(os->Next(1,&temp,0));
	reader = temp;
	// one of those scary all-too common incidents where an object doesn't want to be confined
	// to names or objects alone
	RUNO(ms->Skip(4));
	CComPtr<IMoniker> mkLabelSource,mkReader;
	RUNO(ms->Next(1,&mkLabelSource,0));
	RUNO(ms->Next(1,&mkReader,0));
	return Initialize(mc,engine,nsource,esource,lsource,reader,mkLabelSource,mkReader);
}
// IPersistStream
STDMETHODIMP Drawer::Load(IStream *pStm) {
    METHTRACE(Drawer,Load);
	RUN(pStm->Read(&m_labelLength,sizeof m_labelLength,0));
	return S_OK;
}
STDMETHODIMP Drawer::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(Drawer,Save);
	RUN(pStm->Write(&m_labelLength,sizeof m_labelLength,0));
	return S_OK;
}
