/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "EngineConnector.h"

EngineConnector::EngineConnector() {
	CONSTRUCTORTRACE(EngineConnector);
}
EngineConnector::~EngineConnector() {
	DESTRUCTORTRACE(EngineConnector);
}
// IDGMEngineConnector
STDMETHODIMP EngineConnector::Initialize(IMontageContainer *montage, IDGLayout *engine,IDGMDrawer *drawer){
	METHTRACE(EngineConnector,Initialize);
    RUN(montage->get_RefObject(&m_mref));
	m_engine = engine;
	m_drawer = drawer;
	RUN(AtlAdvise(m_engine,(IDGLayoutShadow *)this,IID_IDGLayoutShadow,&m_chocolateChip));
	DWORD chocograham;
	return AtlAdvise(montage,(IMontageContainerEvents*)this,IID_IMontageContainerEvents,&chocograham);
}
// DIDGMEngineConnector
STDMETHODIMP EngineConnector::Initialize(DIMontageContainer *montage, DIDGLayout *engine, DIDGMDrawer *drawer) {
    METHTRACE(EngineConnector,Initialize);
	CComPtr<IMontageContainer> mc2;
	if(montage)
		RUNRET(montage->QueryInterface(riidret(&mc2)));
	CComPtr<IDGLayout> e2;
	if(engine)
		RUNRET(engine->QueryInterface(riidret(&e2)));
	CComPtr<IDGMDrawer> d2;
	if(drawer)
		RUNRET(drawer->QueryInterface(riidret(&d2)));
	return Initialize(mc2,e2,d2);
}
// INeedSomePointers
STDMETHODIMP EngineConnector::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(EngineConnector,TakePointers);
	CComPtr<IUnknown> u;
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMontageContainer> mc = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IDGLayout> e = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IDGMDrawer> d = u;
	return Initialize(mc,e,d);
}


// IDGLayoutShadow
HRESULT EngineConnector::OnBoundsChanged(PRECTD pNewBounds) {
	METHTRACE(EngineConnector,OnBoundsChanged);
	return S_OK;
}
HRESULT EngineConnector::OnChangeRect(PRECTD chrect) {
    CComPtr<IUnknown> umc;
    RUN(m_mref->get_Montage(&umc));
    if(!umc)
        return S_FALSE;
	VARIANT stc;
	if(SUCCEEDED(CComDispatchDriver(umc).GetPropertyByName(L"ScrollToChanges",&stc))) {
		if(stc.boolVal) {
			RECTL r = rectl(*chrect);
			RUN(fireEvent(umc,fireOneParam<IMontageContainerEvents,LPCRECTL>(&IMontageContainerEvents::ShowArea,&r)));
		}
	}
	return S_OK;
}
STDMETHODIMP EngineConnector::OnLinesChanged() {
    METHTRACE(EngineConnector,OnLinesChanged);
	return S_OK;
}

STDMETHODIMP EngineConnector::OnNewNode(IDGDrawnNode *node) {
    METHTRACE(EngineConnector,OnNewNode);
	CComBSTR name;
	RUN(CComQIPtr<IDGStringAttributes>(node)->get_Name(&name));
	return m_drawer->NewNode(node,0,0,name,0);
}
STDMETHODIMP EngineConnector::OnNewEdge(IDGDrawnEdge *edge) {
    METHTRACE(EngineConnector,OnNewEdge);
	CComBSTR name;
	RUN(CComQIPtr<IDGStringAttributes>(edge)->get_Name(&name));
	return m_drawer->NewEdge(edge,0,name,0);
}
STDMETHODIMP EngineConnector::OnStringsChanged() {
    return S_OK;
}

// IMontageContainerEvents
STDMETHODIMP EngineConnector::Shutdown() {
    METHTRACE(EngineConnector,Shutdown);
	return S_OK;
}
HRESULT EngineConnector::Resize(SIZE) {
    METHTRACE(EngineConnector,Resize);
	return S_FALSE;
}
HRESULT EngineConnector::NewExtent(SIZEL) {
    METHTRACE(EngineConnector,NewExtent);
	return S_FALSE;
}
STDMETHODIMP EngineConnector::Viewing(HDC hdc,LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(EngineConnector,Viewing);
	return S_FALSE;
}
STDMETHODIMP EngineConnector::NewTransform(IMCCTransform *) {
    METHTRACE(EngineConnector,NewTransform);
	return S_FALSE;
}
HRESULT EngineConnector::GroupOp(BOOL start) {
	METHTRACE(EngineConnector,GroupOp);
	if(start)
		m_engine->GraphLock();
	else
		m_engine->GraphUnlock();
	return S_OK;
}
STDMETHODIMP EngineConnector::SiteDeleted(IMCCSite*) {
    METHTRACE(EngineConnector,SiteDeleted);
	return S_FALSE;
}
STDMETHODIMP EngineConnector::LayerDeleted(IMCCLayer*) {
    METHTRACE(EngineConnector,LayerDeleted);
	return S_FALSE;
}
STDMETHODIMP EngineConnector::Shifting(long oldState,long newState) {
    METHTRACE(EngineConnector,Shifting);
	return S_FALSE;
}
// IPersist
STDMETHODIMP EngineConnector::GetClassID(CLSID *pClassID) {
	METHTRACE(EngineConnector,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
