/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// GraphDrawMode.cpp : Implementation of GraphDrawMode
#include "stdafx.h"
#include "GraphDrawMode.h"
#include "CvtMetrics.h"
#include "getSinks.h"
#include "ezMenu.h"
#include "getSites.h"
#include "allocDGShapes.h"

void drawTempEdge(HDC hdc,POINT ptTail,POINT ptHead) {
	MoveToEx(hdc,ptTail.x,ptTail.y,0);
	LineTo(hdc,ptHead.x,ptHead.y);
}
inline long abs(long x) {
	return (x<0)?-x:x;
}
HRGN segmentRgn(POINT ptTail,POINT ptHead) {
	// figure a little parallelogram around edge
	// if |slope|>1, stretch widthwise else heightwise
	bool tall = abs(ptHead.y-ptTail.y)>abs(ptHead.x-ptTail.x);
	POINT pts[4];
	if(tall) {
		ptTail.x-=1; ptHead.x-=1;
	}
	else {
		ptTail.y-=1; ptHead.y-=1;
	}
	pts[0] = ptTail;
	pts[1] = ptHead;
	if(tall) {
		ptTail.x+=2; ptHead.x+=2;
	}
	else {
		ptTail.y+=2; ptHead.y+=2;
	}
	pts[2] = ptHead;
	pts[3] = ptTail;
	return CreatePolygonRgn(pts,4,WINDING);
}
void drawTempNode(HDC hdc,RECT rc) {
	Rectangle(hdc,rc.left,rc.top,rc.right,rc.bottom);
}

#define SIGTRACK 7
template<class T>
inline T sqr(T x) { return x*x; }
GraphDrawMode::GraphDrawMode() : m_bTracking(false), m_drawTempEdge(false),
	m_graphMenu(0),m_orientMenu(0), m_splineMenu(0),m_layoutMenu(0),m_raspberryNewton(0),m_appleNewton(0),
	m_showBounds(false),m_showChangeRect(false) {
	CONSTRUCTORTRACE(GraphDrawMode);
	CLSIDFromProgID(L"SHAPENODE.ShapeNode.1",&m_clsidNode);
	m_cursor = LoadCursor(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDC_Pencil));
    RUN(m_layoutChoices.CoCreateInstance(CLSID_DGLayoutChoices));
}
GraphDrawMode::~GraphDrawMode() {
	DESTRUCTORTRACE(GraphDrawMode);
	for(list<IMCCLayer*>::iterator i = m_layers.begin();i!=m_layers.end();++i) {
		IMCCLayer *l = *i;
		l->Release();
	}
}
HRESULT GraphDrawMode::getMC(IMontageContainer **ret) {
    METHTRACE(GraphDrawMode,getMC);
	if(!m_layers.size())
		ERR(E_FAIL);
	CComPtr<IMontageContainer> mc;
	RUN((*m_layers.begin())->GetContainer(riidret(ret)));
	return S_OK;
}
HRESULT GraphDrawMode::invalLines() {
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));

	CComPtr<IMCCTransform> trans;
	RUN(mc->get_Transform(&trans));

	vector<vector<POINT> > wlines;
	RUN(transformLines(trans,false,wlines));

	for(size_t i=0; i<wlines.size(); ++i)
		for(size_t j=0; j<wlines[i].size(); ++j) {
			HRGN hrgn = segmentRgn(wlines[i][j],wlines[i][j+1]);
			RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
			DeleteObject(hrgn);
		}
	return S_OK;
}
HRESULT GraphDrawMode::invalRectangle(RECT r) {
	if(!m_spInPlaceSite)
		return S_FALSE;
	HRGN hrgn = segmentRgn(point(r.left,r.top),point(r.right,r.top));
	RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
	DeleteObject(hrgn);
	hrgn = segmentRgn(point(r.right,r.top),point(r.right,r.bottom));
	RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
	DeleteObject(hrgn);
	hrgn = segmentRgn(point(r.right,r.bottom),point(r.left,r.bottom));
	RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
	DeleteObject(hrgn);
	hrgn = segmentRgn(point(r.left,r.top),point(r.left,r.bottom));
	RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
	DeleteObject(hrgn);
	return S_OK;
}
HRESULT GraphDrawMode::transformLines(IMCCTransform *trans,bool view,vector<vector<POINT> > &out) {
	out.resize(m_lines.size());
	for(int i = 0; i!=m_lines.size(); ++i) {
		out[i].resize(m_lines[i].size());
		for(int j = 0; j!=m_lines[i].size(); ++j)
			if(view)
				RUN(trans->C2VP(m_lines[i][j],&out[i][j]));
			else
				RUN(trans->C2WP(m_lines[i][j],&out[i][j]));
	}
	return S_OK;
}
typedef enum_iterator<IEnumMCCSites, &IID_IEnumMCCSites, IMCCSite*, _CopyInterface<IMCCSite> > site_iterator;
HRESULT GraphDrawMode::findNode(POINT pt,IMCCSite **retSite,IDGDrawnNode **retNode) {
    METHTRACE(GraphDrawMode,findNode);
	for(list<IMCCLayer*>::iterator i = m_layers.begin();i!=m_layers.end();++i) {
		CComPtr<IEnumMCCSites> sites;
		RUN((*i)->EnumSites(&sites));
		for(site_iterator is(sites); is!=site_iterator(); ++is) {
			long intent;
			RUN((*is)->get_Intent(&intent));
			if(intent<MCCS_SHOWN)
				continue;
			RECT rc;
			RUN((*is)->get_Rect(&rc));
			DWORD hit;
			CComPtr<IViewObjectEx> vo;
			if(SUCCEEDED((*is)->GetControl(riidret(&vo))))
				RUN(vo->QueryHitPoint(DVASPECT_CONTENT,&rc,pt,50,&hit));
			else
				hit = PtInRect(&rc,pt)?HITRESULT_HIT:HITRESULT_OUTSIDE;
			CComPtr<IDGDrawnNode> n;
			if(hit!=HITRESULT_OUTSIDE && layoutFromSite(*is,&n)==S_OK) {
				if(retSite)
					(*retSite = *is)->AddRef();
				if(retNode)
					n.CopyTo(retNode);
				return S_OK;
			}
		}
	}
	return S_FALSE;
}
HRESULT GraphDrawMode::clear() {
	METHTRACE(GraphDrawMode,clear);
	RUN(m_selection->Clear());
	RUN(m_activator->Stop());
	return S_OK;
}
HRESULT GraphDrawMode::activate(IMCCSite *s, LPMSG pmsg) {
	METHTRACE(GraphDrawMode,activate);
	// make selected activated
	RUN(m_activator->Start());
	// add new to selection
	RUN(fireSiteEvent(s,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Add,m_selection)));
	if(pmsg) { // (co)UI activate on click
		RUN_NIOK(s->SayVerb(OLEIVERB_INPLACEACTIVATE,pmsg)); // hopefully the click will mean something *inside* too
		RUN(s->put_Intent(MCCS_UIACTIVE));
	}
	else // only coUI on draw
		RUN(s->put_Intent(MCCS_COUIACTIVE));
	return S_OK;
}
// for IViewObjectImpl
HRESULT GraphDrawMode::OnDraw(ATL_DRAWINFO& di) {
	METHTRACE(GraphDrawMode,OnDraw);
	HDC hdc = di.hdcDraw;
	if(m_drawTempEdge) {
		SelectObject(hdc,GetStockObject(BLACK_PEN));
		drawTempEdge(hdc,m_ptFirst,m_ptLast);
	}
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));

	CComPtr<IMCCTransform> trans;
	RUN(mc->get_Transform(&trans));

	vector<vector<POINT> > vlines;
	RUN(transformLines(trans,true,vlines));

	for(size_t i = 0; i<vlines.size(); ++i) {
		HPEN pen = CreatePen(PS_SOLID,0,RGB(rand()%240+15,rand()%240+15,rand()%240+15)),
			pold = (HPEN)SelectObject(hdc,pen);
		if(m_degrees[i]==3)
			PolyBezier(hdc,&vlines[i][0],vlines[i].size());
		else if(m_degrees[i]==1)
			Polyline(hdc,&vlines[i][0],vlines[i].size());
		SelectObject(hdc,pold);
		DeleteObject(pen);
	}

	DCObject pen(hdc,CreatePen(PS_SOLID,0,RGB(64,0,128))),
		brush(hdc,GetStockObject(NULL_BRUSH));
	if(m_showChangeRect) {
		RECT wchrect;
		RUN(trans->C2WR(&m_chrect,&wchrect));
		wchrect.right++; wchrect.bottom++;
		Rectangle(hdc,wchrect.left,wchrect.top,wchrect.right,wchrect.bottom);
	}
	if(m_showBounds) {
		pen = CreatePen(PS_SOLID,0,RGB(128,0,32));
		RECT wbounds;
		RUN(trans->C2WR(&m_bounds,&wbounds));
		wbounds.right++; wbounds.bottom++;
		Rectangle(hdc,wbounds.left,wbounds.top,wbounds.right,wbounds.bottom);
	}
	return S_OK;
}
// ICoUIActiveObject
/*
void printIconToBitmap(HICON in, HBITMAP *out) {
	_ASSERTE(in);
	HDC hdc = CreateCompatibleDC(0);
	HBITMAP ret = CreateBitmap(16,16,1,8,0),
		last = (HBITMAP)SelectObject(hdc,ret);
	RECT rc = {0,0,16,16};
	HBRUSH menuBrush = CreateSolidBrush(GetSysColor(COLOR_ACTIVECAPTION));
	CHECK(FillRect(hdc,&rc,menuBrush));
	CHECK(DrawIcon(hdc,0,0,in));
	SelectObject(hdc,last);
	*out = ret;
}
*/
HRESULT GraphDrawMode::Activate(HMENU hMenu,OLEMENUGROUPWIDTHS *pWidths) {
	METHTRACE(GraphDrawMode,Activate);
	_ASSERTE(!m_orientMenu);
	m_viewMenu = findOrAddMenu("&View",hMenu,pWidths,1,false);
	m_graphMenu = findOrAddMenu("&Graph",hMenu,pWidths,1,true);
	m_orientMenu = CreateMenu();
	/*
	HICON downIcon = LoadIcon(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDI_DirDown)),
		rightIcon = LoadIcon(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDI_DirRight)),
		upIcon = LoadIcon(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDI_DirUp)),
		leftIcon = LoadIcon(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDI_DirLeft));
	HBITMAP downBmp,rightBmp,upBmp,leftBmp;
	printIconToBitmap(downIcon,&downBmp);
	printIconToBitmap(rightIcon,&rightBmp);
	printIconToBitmap(upIcon,&upBmp);
	printIconToBitmap(leftIcon,&leftBmp);
	*/
	MenuItem mi;
	mi.insert(m_orientMenu,"&Down",CMD_ORIENTDOWN,-1,0,true);
	mi.insert(m_orientMenu,"&Right",CMD_ORIENTRIGHT,-1,0,true);
	mi.insert(m_orientMenu,"&Up",CMD_ORIENTUP,-1,0,true);
	mi.insert(m_orientMenu,"&Left",CMD_ORIENTLEFT,-1,0,true);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	enum DG_ORIENTATION orient;
	RUN(eng->get_Orientation(&orient));
	WORD id=0;
	switch(orient) {
	case DGO_DOWN: id = CMD_ORIENTDOWN; break;
	case DGO_RIGHT: id = CMD_ORIENTRIGHT; break;
	case DGO_UP: id = CMD_ORIENTUP; break;
	case DGO_LEFT: id = CMD_ORIENTLEFT; break;
	}
	_ASSERTE(id);
	CheckMenuItem(m_orientMenu,id,MF_CHECKED);
	mi.insert(m_graphMenu,"&Orientation",CMD_ORIENTMENU,-1,m_orientMenu);

    m_layoutMenu = CreateMenu();
    id = CMD_LAYOUT_FIRST;
    CComBSTR defaultLayout;
    RUN(m_layoutChoices->get_Name(&defaultLayout));
    CComPtr<IEnumString> sit;
    RUN(m_layoutChoices->EnumAttrs(&sit));
    USES_CONVERSION;
    for(olestr_iterator si(sit); si!=olestr_iterator(); ++si,++id) {
        mi.insert(m_layoutMenu,W2T(*si),id,-1,0,true);
        if(!wcscmp(*si,defaultLayout))
            CheckMenuItem(m_layoutMenu,id,MF_CHECKED);
    }
    mi.insert(m_graphMenu,"Choose &Layout",CMD_LAYOUT_MENU,-1,m_layoutMenu);

    mi.insert(m_graphMenu,"&Color by Age",CMD_COLORBYAGE,-1);
	mi.insert(m_graphMenu,"&Adjust Layout",CMD_TWIDDLE,-1);
	mi.insert(m_graphMenu,"&Rework Layout",CMD_BANG,-1);

	m_splineMenu = CreateMenu();
	mi.insert(m_splineMenu,"&V-Node",CMD_SPLINELEVEL_VNODE,-1,0,true);
	mi.insert(m_splineMenu,"&Bounds",CMD_SPLINELEVEL_BOUNDS,-1,0,true);
	mi.insert(m_splineMenu,"&Shortest",CMD_SPLINELEVEL_SHORTEST,-1,0,true);
	mi.insert(m_splineMenu,"&Curve",CMD_SPLINELEVEL_SPLINE,-1,0,true);
	enum DG_SPLINELEVEL level;
	RUN(eng->get_SplineLevel(&level));
	id = 0;
	switch(level) {
	case DGSL_VNODE: id = CMD_SPLINELEVEL_VNODE;
		break;
	case DGSL_BOUNDS: id = CMD_SPLINELEVEL_BOUNDS;
		break;
	case DGSL_SHORTEST: id = CMD_SPLINELEVEL_SHORTEST;
		break;
	case DGSL_SPLINE: id = CMD_SPLINELEVEL_SPLINE;
		break;
	}
	_ASSERTE(id);
	CheckMenuItem(m_splineMenu,id,MF_CHECKED);
	mi.insert(m_viewMenu,"&Edge Drawing",CMD_SPLINELEVEL_MENU,-1,m_splineMenu);
	mi.insert(m_viewMenu,"&Scroll to Changes",CMD_SCROLLTOCHANGES,-1);
	mi.insert(m_viewMenu,"Show &Bounds",CMD_SHOWBOUNDS,-1);
	mi.insert(m_viewMenu,"Show &Change Rect",CMD_SHOWCHANGERECT,-1);

	CComQIPtr<IMCCSite> s = m_spInPlaceSite;
	RUN(s->RegisterCommand(CMD_ORIENTDOWN));
	RUN(s->RegisterCommand(CMD_ORIENTRIGHT));
	RUN(s->RegisterCommand(CMD_ORIENTUP));
	RUN(s->RegisterCommand(CMD_ORIENTLEFT));
    for(id = CMD_LAYOUT_FIRST; id<CMD_LAYOUT_LAST; ++id)
        RUN(s->RegisterCommand(id));
	RUN(s->RegisterCommand(CMD_SPLINELEVEL_VNODE));
	RUN(s->RegisterCommand(CMD_SPLINELEVEL_BOUNDS));
	RUN(s->RegisterCommand(CMD_SPLINELEVEL_SHORTEST));
	RUN(s->RegisterCommand(CMD_SPLINELEVEL_SPLINE));
    RUN(s->RegisterCommand(CMD_COLORBYAGE));
    RUN(s->RegisterCommand(CMD_SCROLLTOCHANGES));
    RUN(s->RegisterCommand(CMD_SHOWBOUNDS));
    RUN(s->RegisterCommand(CMD_SHOWCHANGERECT));
	if(m_editMode) {
		RUN(m_editMode->AddContextItem(VARIANT_TRUE,CComBSTR("&Adjust Whole Layout"),CMD_TWIDDLE,0));
		RUN(m_editMode->AddContextItem(VARIANT_TRUE,CComBSTR("&Rework Whole Layout"),CMD_BANG,0));
		RUN(m_editMode->AddContextItem(VARIANT_TRUE,CComBSTR("&Edge Drawing"),CMD_SPLINELEVEL_MENU,m_splineMenu));
		RUN(m_editMode->AddContextItem(VARIANT_FALSE,CComBSTR("&Adjust Layout"),CMD_TWIDDLE,0));
		RUN(m_editMode->AddContextItem(VARIANT_FALSE,CComBSTR("&Rework Layout"),CMD_BANG,0));
		RUN(s->RegisterCommand(CMD_TWIDDLE));
		RUN(s->RegisterCommand(CMD_BANG));
	}

	return S_OK;
}
HRESULT GraphDrawMode::Deactivate(HMENU hMenu,OLEMENUGROUPWIDTHS *widths) {
	METHTRACE(GraphDrawMode,RemoveMenus);
	_ASSERTE(m_orientMenu && m_splineMenu);
	pair<int,int> sect = range(hMenu,widths,1);
	m_viewMenu = findMenu("&View",hMenu,sect.first,sect.second).first;
	m_graphMenu = findMenu("&Graph",hMenu,sect.first,sect.second).first;
	RemoveMenu(m_viewMenu,CMD_ORIENTMENU,MF_BYCOMMAND);
	DestroyMenu(m_orientMenu);
	m_orientMenu = 0;
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RemoveMenu(m_viewMenu,CMD_SPLINELEVEL_MENU,MF_BYCOMMAND);
	DestroyMenu(m_splineMenu);
	m_splineMenu = 0;
	removeEmptyMenu("&View",hMenu,widths,1);

	CComQIPtr<IMCCSite> s = m_spInPlaceSite;
	RUN(s->ForgetCommand(CMD_ORIENTDOWN));
	RUN(s->ForgetCommand(CMD_ORIENTRIGHT));
	RUN(s->ForgetCommand(CMD_ORIENTUP));
	RUN(s->ForgetCommand(CMD_ORIENTLEFT));
	RUN(s->ForgetCommand(CMD_SPLINELEVEL_VNODE));
	RUN(s->ForgetCommand(CMD_SPLINELEVEL_BOUNDS));
	RUN(s->ForgetCommand(CMD_SPLINELEVEL_SHORTEST));
	RUN(s->ForgetCommand(CMD_SPLINELEVEL_SPLINE));
    RUN(s->ForgetCommand(CMD_COLORBYAGE));
    RUN(s->ForgetCommand(CMD_SCROLLTOCHANGES));
    RUN(s->ForgetCommand(CMD_SHOWBOUNDS));
    RUN(s->ForgetCommand(CMD_SHOWCHANGERECT));
	if(m_editMode) {
		RUN(m_editMode->RemoveContextItem(VARIANT_TRUE,CMD_TWIDDLE));
		RUN(m_editMode->RemoveContextItem(VARIANT_TRUE,CMD_BANG));
		RUN(m_editMode->RemoveContextItem(VARIANT_TRUE,CMD_SPLINELEVEL_MENU));
		RUN(m_editMode->RemoveContextItem(VARIANT_FALSE,CMD_TWIDDLE));
		RUN(m_editMode->RemoveContextItem(VARIANT_FALSE,CMD_BANG));
		RUN(s->ForgetCommand(CMD_TWIDDLE));
		RUN(s->ForgetCommand(CMD_BANG));
	}
	return S_OK;
}
// window messages
LRESULT GraphDrawMode::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(GraphDrawMode,ButtonDown);
	if(m_bTracking) { // clicking another button during track cancels
		m_bTracking = false;
		RUN(m_spInPlaceSite->SetCapture(false));
		return 0;
	}
	POINT pt = MAKEPOINT(lParam);
	m_bTracking = true;
	m_ptDown = pt;
	RUN(m_spInPlaceSite->SetCapture(true));

	// if mouse is on node, start drawing temp edge
	// put_TempEdgeTail might reject the point (say if mouse is over edge)
	bool foundNode = put_TempEdgeTail(&pt)==S_OK;
	if(foundNode) {
		// found a node: start drawing temp edge
		RUN(put_TempEdgeHead(&pt));
		RUN(put_DrawTempEdge(VARIANT_TRUE));
	}
	else {
		// draw temp node
		HDC hdc;
		RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
		int lastROP = SetROP2(hdc,R2_XORPEN);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		HPEN p = (HPEN)GetStockObject(WHITE_PEN),
			oldP = (HPEN)SelectObject(hdc,p);
		RECT rc = {m_ptDown.x,m_ptDown.y,m_ptDown.x,m_ptDown.y};
		drawTempNode(hdc,rc);
		SelectObject(hdc,oldP);
		DeleteObject(p);
		SetROP2(hdc,lastROP);
		RUN(m_spInPlaceSite->ReleaseDC(hdc));
		m_ptLast = m_ptDown;
	}
	return 0;
}
LRESULT GraphDrawMode::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(GraphDrawMode,MouseMove);
	if(!m_bTracking)
		return 0;
	POINT pt = MAKEPOINT(lParam);
	bool bSigTrack = (sqr(pt.x-m_ptDown.x)+sqr(pt.y-m_ptDown.y))>=sqr(SIGTRACK);
	if(!bSigTrack)
		return 0;
	if(m_drawTempEdge)
		RUN(put_TempEdgeHead(&pt));
	else {
		HDC hdc;
		RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
		int lastROP = SetROP2(hdc,R2_XORPEN);
		HPEN p = (HPEN)GetStockObject(WHITE_PEN),
			oldP = (HPEN)SelectObject(hdc,p);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		RECT rc = {m_ptDown.x,m_ptDown.y,m_ptLast.x,m_ptLast.y};
		drawTempNode(hdc,rc);
		rc.right = pt.x;
		rc.bottom = pt.y;
		drawTempNode(hdc,rc);
		SelectObject(hdc,oldP);
		DeleteObject(p);
		SetROP2(hdc,lastROP);
		RUN(m_spInPlaceSite->ReleaseDC(hdc));
		m_ptLast = pt;
	}
	return 0;
}
LRESULT GraphDrawMode::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(GraphDrawMode,ButtonUp);
	if(!m_bTracking)
		return 0;
	m_bTracking = false;
	RUN(m_spInPlaceSite->SetCapture(false));

	POINT pt = MAKEPOINT(lParam);
	bool bSigTrack = (sqr(pt.x-m_ptDown.x)+sqr(pt.y-m_ptDown.y))>=sqr(SIGTRACK);

	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));

	CComPtr<IMCCTransform> trans;
	RUN(mc->get_Transform(&trans));

	// if anything is selected (or active) get rid of that before drawing
	RUN(clear());

	if(m_drawTempEdge) { // downbutton on node: if upbutton on another, draw edge
		// erase temp edge
		RUN(put_DrawTempEdge(VARIANT_FALSE));
		if(!bSigTrack) { // click on object: activate
			MSG msg;
			RUN(m_spInPlaceSite->GetWindow(&msg.hwnd));
			msg.message = uMsg;
			msg.wParam = wParam;
			msg.lParam = lParam;
			msg.time = 0; // I don't have access to the real message, so I have to BS
			::ClientToScreen(msg.hwnd,&pt);
			msg.pt = pt;
			if(!(wParam&(MK_CONTROL|MK_SHIFT)))
				RUN(clear());
			CComPtr<IMCCSite> site;
			if(findNode(m_ptDown,&site,0)==S_OK)
				RUN(activate(site,&msg));
		}
		else RUN(DrawEdge()); // draw real edge
	}
	else { // downbutton on bkgnd: deactivate/insert new node.
		bool hasPos = false;
		RECT rc;
		if(bSigTrack) { // drag out node
			// erase temp node
			HDC hdc;
			RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
			SetROP2(hdc,R2_XORPEN);
			HPEN p = (HPEN)GetStockObject(WHITE_PEN),
				oldP = (HPEN)SelectObject(hdc,p);
			SelectObject(hdc,GetStockObject(NULL_BRUSH));
			rc = rect(m_ptDown.x,m_ptDown.y,m_ptLast.x,m_ptLast.y);
			drawTempNode(hdc,rc);
			SelectObject(hdc,oldP);
			DeleteObject(p);
			RUN(m_spInPlaceSite->ReleaseDC(hdc));
			if(wid(rc)&&hei(rc)) { // else if 1D, treat as click.
				hasPos = true;
			}
		}
		// create layout node
		POINTL downC;
		RUN(trans->W2CP(m_ptDown,&downC));
		CComPtr<IDGLayout> eng;
		if(!eng)
			RUN(m_drawer->get_Engine(&eng));
		CComPtr<IDGNode> node;
		//RUN(eng->new_Node(0,&node));

		// generate event
		MCC_POS_INFO mpi;
		if(hasPos) {
			mpi.postype = MCC_POS_WINDOWRECT;
			mpi.rc = rc;
		}
		else {
			mpi.postype = MCC_POS_WINDOWPOINT;
			mpi.pt = m_ptLast;
		}
        /*
		CComPtr<IDataObject> DO;
		RUN(m_drawer->get_NodeSource(&DO));
		RUN(m_editMode->PasteData(DO,&mpi));
        */
		CComPtr<IMCCSitePersistor> sp;
		RUN(m_drawer->NewNode(CComQIPtr<IDGDrawnNode>(node),&mpi,0,0,&sp));
		CComPtr<IUnknown> us;
		RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&us));
		RUN(activate(CComQIPtr<IMCCSite>(us),0));
	}
	return 0;
}
LRESULT GraphDrawMode::SetCursor(UINT, WPARAM, LPARAM, BOOL&) {
	METHTRACE(GraphDrawMode,SetCursor);
	::SetCursor(m_cursor);
	return 0;
}
LRESULT GraphDrawMode::ChangeOrientation(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,ChangeOrientation);
	enum DG_ORIENTATION dir;
	switch(wID) {
	case CMD_ORIENTDOWN:
		dir = DGO_DOWN;
		break;
	case CMD_ORIENTRIGHT:
		dir = DGO_RIGHT;
		break;
	case CMD_ORIENTUP:
		dir = DGO_UP;
		break;
	case CMD_ORIENTLEFT:
		dir = DGO_LEFT;
		break;
	}
	for(int i = CMD_ORIENTDOWN;i<=CMD_ORIENTLEFT;++i)
		CheckMenuItem(m_orientMenu,i,MF_UNCHECKED);
	CheckMenuItem(m_orientMenu,wID,MF_CHECKED);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RUN(eng->put_Orientation(dir));
	return 0;
}
LRESULT GraphDrawMode::ChangeLayout(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,ChangeLayout);
	for(int id = CMD_LAYOUT_FIRST; id!=CMD_LAYOUT_LAST; ++id)
		CheckMenuItem(m_layoutMenu,id,MF_UNCHECKED);
    CheckMenuItem(m_layoutMenu,wID,MF_CHECKED);
    char buf[200];
    CHECKR(GetMenuString(m_layoutMenu,wID,buf,200,MF_BYCOMMAND));
    CComBSTR layout(buf);
    RUN(m_layoutChoices->put_Name(layout)); // set as new default
    CComBSTR engines;
    RUN(m_layoutChoices->GetAttr(layout,&engines));
    if(engines.Length()) {
        CComPtr<IDGLayout> eng;
        RUN(m_drawer->get_Engine(&eng));
        RUN(CComQIPtr<IDGStringAttributes>(eng)->SetAttr(CComBSTR("engines"),engines));
        RUN(eng->Initialize());
    }
    return 0;
}
LRESULT GraphDrawMode::ChangeSplining(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,ChangeSplining);
	enum DG_SPLINELEVEL level;
	switch(wID) {
	case CMD_SPLINELEVEL_VNODE:
		level = DGSL_VNODE;
		break;
	case CMD_SPLINELEVEL_BOUNDS:
		level = DGSL_BOUNDS;
		break;
	case CMD_SPLINELEVEL_SHORTEST:
		level = DGSL_SHORTEST;
		break;
	case CMD_SPLINELEVEL_SPLINE:
		level = DGSL_SPLINE;
		break;
	}
	for(int i = CMD_SPLINELEVEL_VNODE;i<=CMD_SPLINELEVEL_SPLINE;++i)
		CheckMenuItem(m_splineMenu,i,MF_UNCHECKED);
	CheckMenuItem(m_splineMenu,wID,MF_CHECKED);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RUN(eng->put_SplineLevel(level));
	return 0;
}
LRESULT GraphDrawMode::ColorByAge(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,ColorByAge);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
    CComQIPtr<IDGStringAttributes> strings(eng);
	MENUITEMINFO info;
	info.cbSize = sizeof info;
	info.fMask = MIIM_STATE;
	CHECK(GetMenuItemInfo(m_graphMenu,CMD_COLORBYAGE,false,&info));
	info.fState ^= MFS_CHECKED;
	SetMenuItemInfo(m_graphMenu,CMD_COLORBYAGE,false,&info);
    if(info.fState&MFS_CHECKED)
        RUN(strings->SetAttr(CComBSTR("agecolors"),CComBSTR("green,blue,black")));
    else
        RUN(strings->SetAttr(CComBSTR("agecolors"),0));
    return 0;
}
LRESULT GraphDrawMode::ScrollToChanges(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
	MENUITEMINFO info;
	info.cbSize = sizeof info;
	info.fMask = MIIM_STATE;
	CHECKR(GetMenuItemInfo(m_viewMenu,CMD_SCROLLTOCHANGES,false,&info));
	info.fState ^= MFS_CHECKED;
	CHECKR(SetMenuItemInfo(m_viewMenu,CMD_SCROLLTOCHANGES,false,&info));
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));
	CComVariant val(0!=(info.fState&MFS_CHECKED));
	DISPID idStC;
	CComQIPtr<IDispatchEx> mcx(mc);
	RUN(mcx->GetDispID(CComBSTR(L"ScrollToChanges"),fdexNameEnsure|fdexNameCaseInsensitive,&idStC));
	RUN(CComDispatchDriver::PutProperty(mcx,idStC,&val));
	return 0;
}
LRESULT GraphDrawMode::ShowBounds(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
	MENUITEMINFO info;
	info.cbSize = sizeof info;
	info.fMask = MIIM_STATE;
	CHECKR(GetMenuItemInfo(m_viewMenu,CMD_SHOWBOUNDS,false,&info));
	info.fState ^= MFS_CHECKED;
	CHECKR(SetMenuItemInfo(m_viewMenu,CMD_SHOWBOUNDS,false,&info));
	m_showBounds = 0!=(info.fState&MFS_CHECKED);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RECTD r;
	RUN(eng->get_Bounds(&r));
	RUN(OnBoundsChanged(&r));
	return 0;
}
LRESULT GraphDrawMode::ShowChangeRect(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
	MENUITEMINFO info;
	info.cbSize = sizeof info;
	info.fMask = MIIM_STATE;
	CHECKR(GetMenuItemInfo(m_viewMenu,CMD_SHOWCHANGERECT,false,&info));
	info.fState ^= MFS_CHECKED;
	CHECKR(SetMenuItemInfo(m_viewMenu,CMD_SHOWCHANGERECT,false,&info));
	m_showChangeRect = 0!=(info.fState&MFS_CHECKED);
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	RECTD r;
	if(SUCCEEDED(eng->get_ChangeRect(&r)))
		RUN(OnChangeRect(&r));
	return 0;
}
HRESULT GraphDrawMode::optimize(bool full) {
    METHTRACE(GraphDrawMode,optimize);
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));
	CComPtr<IGroup> group;
	RUN(mc->GetMainGroup(riidret(&group)));
	VARIANT_BOOL varfull = full?VARIANT_TRUE:VARIANT_FALSE;
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	VS *vs = getSites(group,m_selection);
	_ASSERTE(vs);
	if(vs->size()) {
		RUN(eng->GraphLock());
		for(VS::iterator si = vs->begin(); si!=vs->end(); ++si) {
			CComPtr<IUnknown> layout;
			if(SUCCEEDED(layoutFromSite(*si,&layout))) {
				if(CComQIPtr<IDGDrawnNode> dgn = layout)
					RUN(dgn->Optimize(varfull));
				else if(CComQIPtr<IDGDrawnEdge> dge = layout)
					RUN(dge->Optimize(varfull));
				else CHECK(0);
			}
		}
		RUN(eng->GraphUnlock());
	}
	else
		RUN(eng->OptimizeAll(varfull));
	return S_OK;
}
LRESULT GraphDrawMode::Twiddle(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,Twiddle);
	RUN(optimize(false));
	return 0;
}
LRESULT GraphDrawMode::Bang(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(GraphDrawMode,Bang);
	RUN(optimize(true));
	return 0;
}
// IDGMGraphDrawMode
HRESULT GraphDrawMode::Initialize(IItemSet *selection,IMCCItemSetActivator *activator,IDGMDrawer *drawer,IContainerEditMode *editMode) {
	METHTRACE(GraphDrawMode,Initialize);
	/*
	if(m_dotImport)
		ERR(E_FAIL);
		*/
	m_selection = selection;
	m_activator = activator;
	m_drawer = drawer;
	m_editMode = editMode;

	/*
	RUN(CREATE(m_dotImport,CLSID_DGMDOTImport));
	RUN(CComQIPtr<INeedOnePointer>(m_dotImport)->TakePointer(drawer,0));
	*/
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	DWORD spongebobSquarepantsColorYourOwn;
	RUN(AtlAdvise(eng,GetUnknown(),IID_IDGLayoutShadow,&spongebobSquarepantsColorYourOwn));
	return S_OK;
}
STDMETHODIMP GraphDrawMode::AddLayer(IMCCLayer *val) {
    METHTRACE(GraphDrawMode,AddLayer);
	m_layers.push_back(val);
	val->AddRef();
	return S_OK;
}
STDMETHODIMP GraphDrawMode::put_DrawTempEdge(VARIANT_BOOL val) {
    METHTRACE(GraphDrawMode,put_DrawTempEdge);
	m_drawTempEdge = val;
	HRGN hrgn = segmentRgn(m_ptFirst,m_ptLast);
	RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
	DeleteObject(hrgn);
	return S_OK;
}
STDMETHODIMP GraphDrawMode::put_TempEdgeTail(POINT *val) {
    METHTRACE(GraphDrawMode,put_TempEdgeTail);
	m_ptFirst = *val;
	CComPtr<IDGDrawnNode> node;
	if(findNode(*val,0,&node)==S_OK) {
		POINTD ptVirt;
		RUN(node->get_Pos(&ptVirt));
		CComPtr<IMontageContainer> mc;
		RUN(getMC(&mc));
		CComPtr<IMCCTransform> trans;
		RUN(mc->get_Transform(&trans));
		RUN(trans->C2WP(pointl(ptVirt),&m_ptFirst));
		return S_OK;
	}
	else return S_FALSE;
}
STDMETHODIMP GraphDrawMode::put_TempEdgeHead(POINT *val) {
    METHTRACE(GraphDrawMode,put_TempEdgeHead);
	POINT pt = *val;
	CComPtr<IDGDrawnNode> node;
	if(findNode(pt,0,&node)==S_OK) { // magnet effect
		POINTD ptVirt;
		RUN(node->get_Pos(&ptVirt));
		CComPtr<IMontageContainer> mc;
		RUN(getMC(&mc));
		CComPtr<IMCCTransform> trans;
		RUN(mc->get_Transform(&trans));
		RUN(trans->C2WP(pointl(ptVirt),&pt));
	}
	if(m_ptLast.x!=pt.x || m_ptLast.y != pt.y) { // with magnet effect, often edge doesn't move
		if(m_drawTempEdge) {
			HRGN hrgn = segmentRgn(m_ptFirst,m_ptLast);
			RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
			DeleteObject(hrgn);
			hrgn = segmentRgn(m_ptFirst,pt);
			RUN(m_spInPlaceSite->InvalidateRgn(hrgn,true));
			DeleteObject(hrgn);
			/*
			// force redraw
			HWND hwnd;
			RUN(m_spInPlaceSite->GetWindow(&hwnd));
			::UpdateWindow(hwnd);
			*/
		}
		m_ptLast = pt;
	}
	return S_OK;
}
STDMETHODIMP GraphDrawMode::DrawEdge() {
    METHTRACE(GraphDrawMode,DrawEdge);
	// create real edge based on temp edge.
	CComPtr<IDGDrawnNode> tail,head;
	if(findNode(m_ptFirst,0,&tail)!=S_OK)
		return S_FALSE;
	if(findNode(m_ptLast,0,&head)!=S_OK)
		return S_FALSE;

	// create edge & send event
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	CComPtr<IDGEdge> edge;
	HRESULT hr = eng->new_Edge(0,tail,head,&edge);
	// already there
	if(hr==CTL_E_FILEALREADYEXISTS||hr==E_NOTIMPL)
		return S_FALSE;
	RUN(hr);
	CComPtr<IMCCSitePersistor> sp;
	hr = m_drawer->NewEdge(CComQIPtr<IDGDrawnEdge>(edge),0,0,&sp);
	if(hr==S_FALSE) // didn't work
		return S_OK;
	RUN(hr);
	CComPtr<IUnknown> us;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&us));
	RUN(activate(CComQIPtr<IMCCSite>(us),0));
	return S_OK;
}
// DIDGMGraphDrawMode
STDMETHODIMP GraphDrawMode::Initialize(DIItemSet *selection,
	DIMCCItemSetActivator *activator,
	DIDGMDrawer *drawer,DIContainerEditMode *editMode) {
    METHTRACE(GraphDrawMode,Initialize);
	CComPtr<IItemSet> selection2; if(selection) RUNRET(selection->QueryInterface(riidret(&selection2)));
	CComPtr<IMCCItemSetActivator> activator2; if(activator) RUNRET(activator->QueryInterface(riidret(&activator2)));
	CComPtr<IDGMDrawer> drawer2; if(drawer) RUNRET(drawer->QueryInterface(riidret(&drawer2)));
	CComPtr<IContainerEditMode> editMode2; if(editMode) RUNRET(editMode->QueryInterface(riidret(&editMode2)));
	return Initialize(selection2,activator2,drawer2,editMode2);
}
STDMETHODIMP GraphDrawMode::AddLayer(DIMCCLayer *val) {
    METHTRACE(GraphDrawMode,AddLayer);
	CComPtr<IMCCLayer> layer2; if(val) RUNRET(val->QueryInterface(riidret(&layer2)));
	return AddLayer(layer2);
}

// IDGLayoutShadow
STDMETHODIMP GraphDrawMode::OnBoundsChanged(PRECTD pNewBounds) {
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));

	CComPtr<IMCCTransform> trans;
	RUN(mc->get_Transform(&trans));

	RECT wbounds;
	RUN(trans->C2WR(&m_bounds,&wbounds));
	wbounds.right++; wbounds.bottom++;

	RUN(invalRectangle(wbounds));
	if(!m_showBounds)
		return S_FALSE;
	m_bounds = rectl(*pNewBounds);
	RUN(trans->C2WR(&m_bounds,&wbounds));
	wbounds.right++; wbounds.bottom++;
	RUN(invalRectangle(wbounds));
	return S_OK;
}
STDMETHODIMP GraphDrawMode::OnChangeRect(PRECTD chrect) {
	CComPtr<IMontageContainer> mc;
	RUN(getMC(&mc));

	CComPtr<IMCCTransform> trans;
	RUN(mc->get_Transform(&trans));

	RECT wchrect;
	RUN(trans->C2WR(&m_chrect,&wchrect));
	wchrect.right++; wchrect.bottom++;
	RUN(invalRectangle(wchrect));
	if(!m_showChangeRect)
		return S_FALSE;
	m_chrect = rectl(*chrect);
	RUN(trans->C2WR(&m_chrect,&wchrect));
	wchrect.right++; wchrect.bottom++;
	RUN(invalRectangle(wchrect));
	return S_OK;
}
STDMETHODIMP GraphDrawMode::OnLinesChanged() {
	RUN(invalLines());
	CComPtr<IDGLayout> eng;
	RUN(m_drawer->get_Engine(&eng));
	DGSHAPESDESC *lines;
	RUN(eng->get_Lines(&lines));
	m_lines.resize(lines->n);
	for(int i=0; i<lines->n; ++i) {
		m_degrees[i] = lines->sps[i]->isSpline?3:1;
		int npts = lines->sps[i]->n;
		m_lines[i].resize(npts);
		for(int j=0; j<npts; ++j)
			m_lines[i][j] = pointl(lines->sps[i]->pts[j]);
	}
	freeDGShapes(lines);
	RUN(invalLines());
	return S_OK;
}
// INeedSomePointers
HRESULT GraphDrawMode::TakePointers(IEnumUnknown *eu,IEnumMoniker *ms) {
	METHTRACE(GraphDrawMode,TakePointers);
	CComPtr<IUnknown> unk;
	RUN(eu->Next(1,&unk,0));
	CComQIPtr<IItemSet> iset = unk;
	unk.Release();
	RUN(eu->Next(1,&unk,0));
	CComQIPtr<IMCCItemSetActivator> activator = unk;
	unk.Release();
	RUN(eu->Next(1,&unk,0));
	CComQIPtr<IDGMDrawer> draw = unk;
	unk.Release();
	RUN(eu->Next(1,&unk,0));
	CComQIPtr<IContainerEditMode> editMode = unk;
	RUN(Initialize(iset,activator,draw,editMode));
	unk.Release();
	// rest are layers
	while(eu->Next(1,&unk,0)==S_OK) {
		CComQIPtr<IMCCLayer> l = unk;
		RUN(AddLayer(l));
		unk.Release();
	}
	return S_OK;
}
// IPersist
STDMETHODIMP GraphDrawMode::GetClassID(CLSID *pClassID) {
	METHTRACE(GraphDrawMode,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
