/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// GraphDrawMode.h : Declaration of the GraphDrawMode

#ifndef __GRAPHDRAWMODE_H_
#define __GRAPHDRAWMODE_H_

#define CMD_INSERTDOT	300
#define CMD_ORIENTMENU	320
#define CMD_ORIENTDOWN	321
#define CMD_ORIENTRIGHT	322
#define CMD_ORIENTUP	323
#define CMD_ORIENTLEFT	324

#define CMD_COLORBYAGE  330
#define CMD_SCROLLTOCHANGES 331
#define CMD_SHOWBOUNDS 332
#define CMD_SHOWCHANGERECT 333

#define CMD_TWIDDLE		350
#define CMD_BANG		351

#define CMD_SPLINELEVEL_MENU 360
#define CMD_SPLINELEVEL_VNODE 361
#define CMD_SPLINELEVEL_BOUNDS 362
#define CMD_SPLINELEVEL_SHORTEST 363
#define CMD_SPLINELEVEL_SPLINE 364

#define CMD_CROSSOPT 370
#define CMD_CROSSOPT_FIRST 371
#define CMD_CROSSOPT_LAST 379

#define CMD_LAYOUT_MENU 380
#define CMD_LAYOUT_FIRST 381
#define CMD_LAYOUT_LAST CMD_LAYOUT_FIRST+9

#include "resource.h"
#include <list>

/////////////////////////////////////////////////////////////////////////////
// GraphDrawMode
class ATL_NO_VTABLE GraphDrawMode :
	public CComObjectRoot,
	public CComCoClass<GraphDrawMode,&CLSID_DGMGraphDrawMode>,
	public CComControl<GraphDrawMode>,
	public IDGMGraphDrawMode,
	public IDispatchImpl<DIDGMGraphDrawMode, &IID_DIDGMGraphDrawMode, &LIBID_DGM>,
	public IOleControlImpl<GraphDrawMode>,
	public IOleObjectImpl<GraphDrawMode>,
	public IOleInPlaceActiveObjectImpl<GraphDrawMode>,
	public ICoUIActiveObject,
	public IViewObjectExImpl<GraphDrawMode>,
	public IOleInPlaceObjectWindowlessImpl<GraphDrawMode>,
	public IDGLayoutShadow,
	public INeedSomePointers,
	public IPersist
{
	list<IMCCLayer *> m_layers; // layers to look for nodes in
	CComPtr<IDGMDrawer> m_drawer; // high-level drawer
	CComPtr<IItemSet> m_selection; // the global selection object
	CComPtr<IMCCItemSetActivator> m_activator;
	CComPtr<IContainerEditMode> m_editMode;

	bool m_showBounds,m_showChangeRect;
	RECTL m_bounds,m_chrect;
	vector<vector<POINTL> > m_lines;
	vector<int> m_degrees;

	HMENU m_viewMenu, m_graphMenu, m_orientMenu, m_splineMenu, m_layoutMenu;

	HCURSOR m_cursor; // pencil

	CLSID m_clsidNode; // GUID of control to use for nodes

	VARIANT_BOOL m_drawTempEdge; // whether we're drawing the temp edge
	POINT m_ptDown, m_ptFirst, m_ptLast; // place mouse went down, tail of temp edge, head of temp edge
	bool m_bTracking; // is button down?
	DWORD m_raspberryNewton,m_appleNewton;

	CComPtr<IDGStringAttributes> m_layoutChoices;

	typedef enum {psNothing,psInit,psComplete} persistStorageAction;

	HRESULT getMC(IMontageContainer **ret);
	HRESULT findNode(POINT pt,IMCCSite **retSite,IDGDrawnNode **retNode);
	HRESULT clear();
	HRESULT activate(IMCCSite *s, LPMSG pmsg);
	HRESULT invalLines();
	HRESULT invalRectangle(RECT r);
	HRESULT transformLines(IMCCTransform *trans,bool view,vector<vector<POINT> > &out);
public:
	GraphDrawMode();
	~GraphDrawMode();

REGGY_RID(IDR_GRAPHDRAWMODE)

BEGIN_COM_MAP(GraphDrawMode)
	COM_INTERFACE_ENTRY(IDGMGraphDrawMode)
	COM_INTERFACE_ENTRY(DIDGMGraphDrawMode)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IDGLayoutShadow)
	COM_INTERFACE_ENTRY(INeedSomePointers)
	COM_INTERFACE_ENTRY(IPersist)
END_COM_MAP()

BEGIN_PROPERTY_MAP(GraphDrawMode)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(GraphDrawMode)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE, MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_SETCURSOR,SetCursor)
	COMMAND_ID_HANDLER(CMD_TWIDDLE,Twiddle)
	COMMAND_ID_HANDLER(CMD_BANG,Bang)
    COMMAND_ID_HANDLER(CMD_COLORBYAGE,ColorByAge)
    COMMAND_ID_HANDLER(CMD_SCROLLTOCHANGES,ScrollToChanges)
    COMMAND_ID_HANDLER(CMD_SHOWBOUNDS,ShowBounds)
    COMMAND_ID_HANDLER(CMD_SHOWCHANGERECT,ShowChangeRect)
	COMMAND_RANGE_HANDLER(CMD_ORIENTDOWN,CMD_ORIENTLEFT,ChangeOrientation)
	COMMAND_RANGE_HANDLER(CMD_LAYOUT_FIRST,CMD_LAYOUT_LAST,ChangeLayout)
	COMMAND_RANGE_HANDLER(CMD_SPLINELEVEL_VNODE,CMD_SPLINELEVEL_SPLINE,ChangeSplining)
	CHAIN_MSG_MAP(CComControl<GraphDrawMode>)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT ChangeOrientation(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT ChangeLayout(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT ChangeSplining(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT ColorByAge(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT ScrollToChanges(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT ShowBounds(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT ShowChangeRect(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

	HRESULT optimize(bool full);
	LRESULT Twiddle(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT Bang(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);


// IViewObjectEx
	STDMETHODIMP GetViewStatus(DWORD* pdwStatus)
	{
		METHTRACE(GraphDrawMode,GetViewStatus);
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}
	STDMETHODIMP QueryHitPoint(DWORD,LPCRECT,POINT,LONG,DWORD *ret) {
		*ret = HITRESULT_TRANSPARENT;
		return S_OK;
	}
	HRESULT OnDraw(ATL_DRAWINFO& di);
// IOleInPlaceObjectWindowless
	STDMETHODIMP GetDropTarget(IDropTarget **ret) {
		// Let CEM handle all drops, graph drawer will sink a higher level layout event.
		ERR(E_NOTIMPL);
	}
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU hMenu,OLEMENUGROUPWIDTHS *pWidths);
	STDMETHODIMP Deactivate(HMENU hMenu,OLEMENUGROUPWIDTHS *pWidths);
// IDGMGraphDrawMode
	STDMETHODIMP Initialize(IItemSet *selection, IMCCItemSetActivator *activator,IDGMDrawer *drawer,IContainerEditMode *editMode);
	STDMETHODIMP AddLayer(IMCCLayer *val);
	STDMETHODIMP put_DrawTempEdge(VARIANT_BOOL val);
	STDMETHODIMP put_TempEdgeTail(POINT *val);
	STDMETHODIMP put_TempEdgeHead(POINT *val);
	STDMETHODIMP DrawEdge();
// DIDGMGraphDrawMode
	STDMETHODIMP Initialize(DIItemSet *selection,
		DIMCCItemSetActivator *activator,
		DIDGMDrawer *drawer,DIContainerEditMode *editMode);
	STDMETHODIMP AddLayer(DIMCCLayer *val);
// IDGLayoutShadow
	STDMETHODIMP OnBoundsChanged(PRECTD pNewBounds);
	STDMETHODIMP OnChangeRect(PRECTD chrect);
	STDMETHODIMP OnLinesChanged();
	STDMETHODIMP OnNewNode(IDGDrawnNode *node) { return S_FALSE; }
	STDMETHODIMP OnNewEdge(IDGDrawnEdge *edge) { return S_FALSE; }
    STDMETHODIMP OnStringsChanged() { return S_FALSE; }
// INeedSomePointers
	STDMETHODIMP TakePointers(IEnumUnknown *os,IEnumMoniker *ms);
// IPersist
    STDMETHODIMP GetClassID(CLSID *pClassID);
};

#endif //__GRAPHDRAWMODE_H_

