/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// implementation of the Simple Edge control

#include "stdafx.h"
#include "LabelMode.h"

#include <common/Geometry.h>

#include "labelNodeSite.h"
#include "pointd2coord.h"
#include "brushRect.h"
#include "bitBrush.h"

const int WIDTH = 3;
LabelMode::LabelMode() : m_buttonDown(false),m_onBorder(false),m_sigMove(false),
		m_labsel(0),m_truffle(0),m_strawberrySugarWafer(0) {
	CONSTRUCTORTRACE(LabelMode);
}
LabelMode::~LabelMode() {
	DESTRUCTORTRACE(LabelMode);
}
HRESULT LabelMode::fit() {
    METHTRACE(LabelMode,fit);
	RECTL rc;
	if(!m_spClientSite || FAILED(m_subject->get_VirtRect(&rc)))
		return S_FALSE;
	_ASSERTE(rc.top>=rc.bottom);
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il) {
		RECTL rc2;
		RUN(il->edit->get_VirtRect(&rc2));
		rc = rc|rc2;
		_ASSERTE(rc.top>=rc.bottom);
	}
	// extra buffer
	rc += rectl(-200,200,200,-200);
	_ASSERTE(rc.top>=rc.bottom);
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_VirtRect(&rc));
	return S_OK;
}
HRESULT LabelMode::placeLabel(label &l) {
    METHTRACE(LabelMode,placeLabel);
	POINTD p;
	RECTD b;
	if(CComQIPtr<IDGEdgeLabel> el = l.l) {
		b = l.bounds;
		RUN(el->get_Pos1(&p));
		b.left += p.x; b.top += p.y;
		b.right += p.x; b.bottom += p.y;
	}
	else if(CComQIPtr<IDGNodeLabel> nl = l.l)
		RUNRET(nl->get_Bounds(&b));
	if(m_bInPlaceActive)
		inval(l.edit);
	RECTL r = rectl(b);
	RUN(l.edit->put_VirtRect(&r));
	if(m_bInPlaceActive)
		inval(l.edit);
	// site may still be hidden if this is the first time it gets position
	long state;
	RUN(l.edit->get_Intent(&state));
	if(state==MCCS_HIDDEN)
		RUN(l.edit->put_Intent(MCCS_SHOWN));
	return S_OK;
}
HRESULT LabelMode::placeLabels() {
    METHTRACE(LabelMode,placeLabels);
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il)
		RUN(placeLabel(*il));
	RUN(fit());
	return S_OK;
}
int LabelMode::findLabel(POINT pt) {
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il) {
		RECT rc;
		RUN(il->edit->get_Rect(&rc));
		InflateRect(&rc,WIDTH,WIDTH);
		if(PtInRect(&rc,pt))
			return il-m_labels.begin();
	}
	return -1;
}
MSG LabelMode::message(UINT uMsg,WPARAM wParam,LPARAM lParam,POINT pt) {
	MSG msg;
	RUN(m_spInPlaceSite->GetWindow(&msg.hwnd));
	msg.message = uMsg;
	msg.wParam = wParam;
	msg.lParam = lParam;
	msg.time = 0;
	msg.pt = pt;
	::ClientToScreen(msg.hwnd,&msg.pt);
	return msg;
}
HRESULT LabelMode::inval(IMCCSite *s) {
	RECT r;
	if(SUCCEEDED(s->get_Rect(&r))) {
		InflateRect(&r,WIDTH,WIDTH);
		RUN(m_spInPlaceSite->InvalidateRect(&r,true));
		return S_OK;
	}
	else
		return S_FALSE;
}
HRESULT LabelMode::activate(IMCCSite *s,LPMSG msg) {
    METHTRACE(LabelMode,activate);
	if(m_active)
		deactivate();
	RUN(s->PutBefore(CComQIPtr<IMCCSite>(m_spClientSite)));
	// hmm. gets activated twice here. that's silly!
	if(msg)
		RUN(s->SayVerb(OLEIVERB_INPLACEACTIVATE,msg));
	RUN(s->put_Intent(MCCS_COUIACTIVE));
	m_active = s;
	RUN(inval(m_active));
	return S_OK;
}
HRESULT LabelMode::deactivate() {
    METHTRACE(LabelMode,deactivate);
	if(m_active) {
		RUN(inval(m_active));
		RUN(m_active->put_Intent(MCCS_SHOWN));
		RUN(m_active->PutBehind(CComQIPtr<IMCCSite>(m_spClientSite)));
		m_active.Release();
	}
	return S_OK;
}
HRESULT LabelMode::ownUp(IMCCSite *labelSite,LabelSink **ret) {
    METHTRACE(LabelMode,ownUp);
	// create owner
	CComObject<LabelSink> *owner = new CComObject<LabelSink>;
	RUN(owner->init(this,labelSite));
	((IMCCSelectable*)(*ret = owner))->AddRef();
	return S_OK;
}
STDMETHODIMP LabelMode::SetClientSite(IOleClientSite *site) {
    METHTRACE(LabelMode,SetClientSite);
	if(m_spClientSite)
		RUN(AtlUnadvise(m_spClientSite,IID_IMCCSelectable,m_fudgeMint));
	HRESULT hr = IOleObjectImpl<LabelMode>::SetClientSite(site);
	if(m_spClientSite)
		RUN(APCAdvise(m_spClientSite,GetUnknown(),IID_IMCCSelectable,&m_fudgeMint));
	return hr;
}
// IDGMLabelMode
STDMETHODIMP LabelMode::Initialize(IUnknown *layoutObject, IMCCSite *content,
								   IDataObject *labelSource,IMCCDataObjectUnpacker *reader) {
    METHTRACE(LabelMode,Initialize);
	RUN(put_LayoutObject(layoutObject));
	m_subject = content;
	DWORD animalCracker=0;
	RUN(APCAdvise(m_subject,GetUnknown(),IID_IMCCSiteEvents,&animalCracker));
	animalCracker=0;
	RUN(APCAdvise(m_subject,GetUnknown(),IID_IMCCExpandSelection,&animalCracker));
	m_labelSource = labelSource;
	m_reader = reader;
	return S_OK;
}
STDMETHODIMP LabelMode::put_LayoutObject(IUnknown *val) {
    METHTRACE(LabelMode,put_LayoutObject);
    if(m_node) {
		RUN(AtlUnadvise(m_node,IID_IDGNodeShadow,m_truffle));
        m_node.Release();
    }
    else if(m_edge) {
		RUN(AtlUnadvise(m_edge,IID_IDGEdgeShadow,m_truffle));
        m_edge.Release();
    }
	if(val) {
		if(SUCCEEDED(val->QueryInterface(riidret(&m_edge))))
			RUN(AtlAdvise(m_edge,GetUnknown(),IID_IDGEdgeShadow,&m_truffle));
		else if(SUCCEEDED(val->QueryInterface(riidret(&m_node))))
			RUN(AtlAdvise(m_node,GetUnknown(),IID_IDGNodeShadow,&m_truffle));
		else
			ERR(E_FAIL);
		for(vector<label>::iterator li = m_labels.begin(); li!=m_labels.end(); ++li) {
			CComPtr<IDGLabel> l;
			if(m_edge) {
				RUN(m_edge->new_Label((IDGEdgeLabel**)&l));
				IDGEdgeLabel *el1 = (IDGEdgeLabel*)li->l.p,
					*el2 = (IDGEdgeLabel*)l.p;
				double a;
				RUN(el1->get_Where(&a));
				RUN(el2->put_Where(a));
				RUN(el1->get_Length(&a));
				RUN(el2->put_Length(a));
			}
			else {
				RUN(m_node->new_Label((IDGNodeLabel**)&l));
				IDGNodeLabel *nl1 = (IDGNodeLabel*)li->l.p,
					*nl2 = (IDGNodeLabel*)l.p;
				RECTD b;
				RUN(nl1->get_Bounds(&b));
				RUN(nl2->put_Size(pointd(b.right-b.left,b.top-b.bottom)));
				enum DG_NODELABELALIGN a;
				RUN(nl1->get_Align(&a));
				RUN(nl2->put_Align(a));
			}
			li->l = l;
		}
	}
	return S_OK;
}
HRESULT LabelMode::NewLabel(double where,IMCCSite **retSite,IDGLabel **retLabel) {
    METHTRACE(LabelMode,NewLabel);
	// deactivate any label being edited
	RUN(deactivate());

	// fill internal tables with good things
	m_labels.push_back(label());
	label &l = m_labels.back();
	if(m_edge)
		RUN(m_edge->new_Label((IDGEdgeLabel**)&l.l));
	else
		RUN(m_node->new_Label((IDGNodeLabel**)&l.l));
	CComPtr<IMontageContainer> mc;
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
	m_labsel = &l;

	// get a label control
	CComPtr<IMCCSitePersistor> sp;
	RUN(m_reader->UnpackOne(m_labelSource,0,0,0,&sp));

	// make sure it's hidden, with no position:
	MCCSITEPERSISTORDATA spdat;
	RUN(sp->get_Data(&spdat));
	spdat.pos.postype = MCC_POS_NONE;
	spdat.state = MCCS_HIDDEN;
	RUN(sp->put_Data(&spdat));

	// HACK: fake a layout initializer so that everyone knows not to give this new layout
	// should be unnecessary: only top-level data should hit layout annotator
	CComPtr<IOrderedItemList> oil;
	RUN(sp->get_InitList(&oil));
	ItemMoniker mk(L"Embed Unpacker"); // whatever, anything that doesn't support IInitializeOther or IObjectWithSite so that nothing happens
	RUN(oil->SetMoniker(20,mk));

	// and realize
	BindCtx bctx;
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(bctx));
	CComPtr<IUnknown> us;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&us));
	RUN(us.QueryInterface(&l.edit));

	// set initial position via DG label parameters
	CComPtr<IOleObject> oo;
	RUN(l.edit->GetControl(riidret(&oo)));
	SIZEL ext;
	RUN(oo->GetExtent(DVASPECT_CONTENT,&ext));
	// dynagraph will respond with OnLabelsChanged when it has a position for these.
	// at that point
	if(m_edge)
		RUN(CComQIPtr<IDGEdgeLabel>(l.l)->put_Where(where));
	else {
		CComQIPtr<IDGNodeLabel> nl(l.l);
		RUN(nl->put_Size(pointd(pointl(ext.cx,ext.cy))));
		RUN(nl->put_Align(DGNLA_CENTER));
	}
	RUN(ownUp(l.edit,&l.owner));

	// hackick, get LabelModePersistor to hold storage refs
	RUN(fireEvent(GetUnknown(),fireVoid<IDGMLabelModeEvents>(&IDGMLabelModeEvents::LabelsChanged)));

	// force into correct z-order
	CComQIPtr<IMCCSite> s = m_spClientSite;
	CComPtr<IMCCLayer> activl;
	RUN(s->get_Layer(&activl));
	RUN(l.edit->put_Layer(activl));
	RUN(l.edit->PutBehind(s));
	if(retSite)
		l.edit.CopyTo(retSite);
	if(retLabel)
		l.l.CopyTo(retLabel);
	if(m_labels.size()==1) {
		CComPtr<IDataObject> DO;
		if(SUCCEEDED(l.edit->GetControl(riidret(&DO)))) {
			FormatEtc fetc(CF_TEXT);
			RUN(DO->DAdvise(&fetc,ADVF_NODATA,CComQIPtr<IAdviseSink>(GetUnknown()),&m_strawberrySugarWafer));
		}
	}
	return S_OK;
}
STDMETHODIMP LabelMode::AddLabel(IDGLabel *layout, IMCCSite *content) {
    METHTRACE(LabelMode,AddLabel);
	m_labels.push_back(label());
	label &l = m_labels.back();
	l.l = layout;
	l.edit = content;
	RUN(ownUp(l.edit,&l.owner));
	HRESULT hr = placeLabel(l);
    // okay if the label's not ready because it will yell when it is
    if(SUCCEEDED(hr))
    	RUN(fit());
    if(hr!=E_PENDING)
        RUN(hr);
	RUN(fireEvent(GetUnknown(),fireVoid<IDGMLabelModeEvents>(&IDGMLabelModeEvents::LabelsChanged)));
	if(m_labels.size()==1) {
		CComPtr<IDataObject> DO;
		if(SUCCEEDED(l.edit->GetControl(riidret(&DO)))) {
			FormatEtc fetc(CF_TEXT);
			RUN(DO->DAdvise(&fetc,ADVF_NODATA,CComQIPtr<IAdviseSink>(GetUnknown()),&m_strawberrySugarWafer));
		}
	}
	return S_OK;
}
STDMETHODIMP LabelMode::get_Content(IMCCSite **ret) {
    METHTRACE(LabelMode,get_Content);
	m_subject.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP LabelMode::EnumLabels(IEnumDGMLabelInfo **ret) {
    METHTRACE(LabelMode,EnumLabels);
	typedef CComObject<CComEnum<IEnumDGMLabelInfo,&IID_IEnumDGMLabelInfo,DGMLabelInfo,copyDGMLabelInfo> > labenum;
	labenum *e = new labenum;
	DGMLabelInfo *ray = new DGMLabelInfo[m_labels.size()];
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il) {
		il->l.CopyTo(&ray[il-m_labels.begin()].layout);
		il->edit.CopyTo(&ray[il-m_labels.begin()].content);
	}
	e->Init(ray,ray+m_labels.size(),0,AtlFlagTakeOwnership);
	(*ret = e)->AddRef();
	return S_OK;
}
// ICoUIActiveObject
STDMETHODIMP LabelMode::Activate(HMENU,OLEMENUGROUPWIDTHS*) {
    METHTRACE(LabelMode,Activate);
	RUN(m_spInPlaceSite->SetFocus(true));
	return S_FALSE;
}
STDMETHODIMP LabelMode::Deactivate(HMENU,OLEMENUGROUPWIDTHS*) {
    METHTRACE(LabelMode,Deactivate);
	RUN(m_spInPlaceSite->SetFocus(false));
	RUN(deactivate());
	return S_FALSE;
}

// LabelSinkClient
IUnknown *LabelMode::mainSite() {
    METHTRACE(LabelMode,mainSite);
	return m_subject;
}
HRESULT LabelMode::Move(LabelSink *from,MCC_POS_INFO *mpi) {
    METHTRACE(LabelMode,Move);
	CComPtr<IMontageContainer> mc;
	RUN(from->site->GetContainer(riidret(&mc)));
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il)
		if(il->owner==from)
			break;
	_ASSERTE(il!=m_labels.end());
	SIZEL s;
	switch(mpi->postype) {
	case MCC_POS_CANVASRECT:
		s = siz(mpi->rcl);
		break;
	case MCC_POS_CANVASSIZE:
		s = mpi->sl;
		break;
	default:
		_ASSERTE(0);
	}
	// center.
	if(CComQIPtr<IDGNodeLabel> nl = il->l)
		RUN(nl->put_Size(pointd(pointl(s.cx,s.cy))));
	else {
		RECTD bounds = {-s.cx/2000.0,s.cy/2000.0,s.cx/2000.0,-s.cy/2000.0};
		il->bounds = bounds;
	}
	HRESULT hr = placeLabel(*il);
	if(hr==E_PENDING)
		return S_FALSE;
	RUN(hr);
	RUN(fit());
	return S_OK;
}
HRESULT LabelMode::OnChanged(LabelSink *from,DISPID dispid) {
	unsigned i;
	for(i = 0; i<m_labels.size(); ++i)
		if(m_labels[i].owner==from)
			break;
	_ASSERTE(i!=m_labels.size());
	CComPtr<IDispatch> disp;
	RUN(m_labels[i].edit->GetControl(riidret(&disp)));
	CComVariant var;
	RUN(disp.GetProperty(0,&var));
	var.ChangeType(VT_BSTR);
	CComQIPtr<IDGStringAttributes> sattr = m_node;
	if(!sattr)
		sattr = m_edge;
	if(sattr) {
		char buf[20] = "label";
		if(i>0)
			sprintf(buf+5,"%d",i+1);
		RUN(sattr->SetAttr(CComBSTR(buf),var.bstrVal));
	}
	return S_OK;
}
void LabelMode::stop() {
    METHTRACE(LabelMode,stop);
}
HRESULT LabelMode::fetchLabelText(bool changed,IDGStringAttributes *sattr) {
    CComPtr<IEnumString> ena;
	if(changed)
		RUN(sattr->EnumChangedAttrs(&ena));
	else
		RUN(sattr->EnumAttrs(&ena));
    for(olestr_iterator si(ena); si!=olestr_iterator(); ++si) {
        if(!wcscmp(*si,L"label")) {
            int i=0;
            swscanf(*si+5,L"%d",&i);
            CComBSTR name(*si),value;
            RUN(sattr->GetAttr(name,&value));
            CComPtr<IMCCTextEdit> ted;
            if(SUCCEEDED(m_labels[i].edit->GetControl(riidret(&ted))))
                RUN(ted->put_Text(value));
        }
    }
    return S_OK;
}
// IDGEdgeShadow:
STDMETHODIMP LabelMode::XDGEdgeShadow::Insert() {
    METHTRACE(LabelMode,Insert);
    return S_FALSE; }
STDMETHODIMP LabelMode::XDGEdgeShadow::Delete() {
    METHTRACE(LabelMode,Delete);
    return S_FALSE; }
STDMETHODIMP LabelMode::XDGEdgeShadow::Modify(int flags) {
    if(flags&(DGMF_POS|DGMF_LABELS))
    	RUN(This()->placeLabels());
    if(flags&DGMF_STRINGS)
        RUN(This()->fetchLabelText(true,CComQIPtr<IDGStringAttributes>(This()->m_edge)));
	return S_OK;
}
// IDGNodeShadow
STDMETHODIMP LabelMode::XDGNodeShadow::Insert(void) {
    METHTRACE(LabelMode,Insert);
    return Modify(DGMF_LABELS);
}
STDMETHODIMP LabelMode::XDGNodeShadow::Delete(void) {
    METHTRACE(LabelMode,Delete);
    return S_FALSE;
}
STDMETHODIMP LabelMode::XDGNodeShadow::Modify(int flags) {
    if(flags&(DGMF_POS|DGMF_LABELS|DGMF_SHAPE))
    	RUN(This()->placeLabels());
    if(flags&DGMF_STRINGS)
        RUN(This()->fetchLabelText(true,CComQIPtr<IDGStringAttributes>(This()->m_node)));
    return S_FALSE;
}
// IViewObject
BitBrush g_faintCheckBrush(0x77eebbdd77eebbdd),
	g_tightdiagCross(0x77aaddaa77aaddaa);
HBRUSH g_crossBrush = CreateHatchBrush(HS_CROSS,RGB(0,0,0));
STDMETHODIMP LabelMode::Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
     DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdc,LPCRECTL lprcBounds,
     LPCRECTL lprcWBounds,BOOL(STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
     DWORD dwContinue) {
	METHTRACE(LabelMode,Draw);
	if(!m_spClientSite)
		ERR(E_PENDING);
	if(m_labsel && m_bInPlaceActive) {
		RECT r;
		RUN(m_labsel->edit->get_Rect(&r));
		InflateRect(&r,WIDTH,WIDTH);
		brushRect(hdc,r,WIDTH,(HBRUSH)GetStockObject(m_active?GRAY_BRUSH:LTGRAY_BRUSH));
	}
	return S_OK;
}
STDMETHODIMP LabelMode::QueryHitPoint(DWORD aspect,LPCRECT bounds,POINT pt,
		LONG closeness,DWORD *ret) {
	METHTRACE(LabelMode,QueryHitPoint);
	*ret = HITRESULT_OUTSIDE;
	// my area is that of node/edge + m_labels
	RECT rc;
	RUN(m_subject->get_Rect(&rc));
	CComPtr<IViewObjectEx> vex;
	if(SUCCEEDED(m_subject->GetControl(riidret(&vex))))
		RUN(vex->QueryHitPoint(aspect,&rc,pt,closeness,ret));
	else
		*ret = PtInRect(&rc,pt)?HITRESULT_HIT:HITRESULT_OUTSIDE;
	if(*ret!=HITRESULT_OUTSIDE)
		return S_OK;
	*ret = findLabel(pt)<0?HITRESULT_OUTSIDE:HITRESULT_HIT;
	return S_OK;
}
STDMETHODIMP LabelMode::QueryHitRect(DWORD dwAspect,LPCRECT pRectBounds,
		LPCRECT pRectLoc,LONG lCloseHint,DWORD *pHitResult) {
	METHTRACE(LabelMode,QueryHitRect);
	// don't bother "stealing" for selection - they'll handle fine.
	if(dwAspect!=DVASPECT_CONTENT)
		ERR(E_FAIL);
	*pHitResult = HITRESULT_OUTSIDE;
	return S_OK;
}
// MESSAGES
LRESULT LabelMode::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(LabelMode,ButtonDown);
	POINT pt = MAKEPOINT(lParam);
	m_buttonDown = false;
	int lab = findLabel(pt);
	RUN(deactivate());
	if(m_labsel && m_labsel->edit)
		RUN(inval(m_labsel->edit));
	if(lab>=0) {
		m_labsel = &m_labels[lab];
		RECTL rcc;
		RUN(m_labsel->edit->get_VirtRect(&rcc));
		m_startpt = ctr(rcc);
		m_buttonDown = true;
		RECT r;
		RUN(m_labsel->edit->get_Rect(&r));
		m_onBorder = !PtInRect(&r,pt);
		RUN(inval(m_labsel->edit));
	}
	else { // fall through
		m_labsel = 0;
		bHandled = false;
		return 0;
	}
	m_downpt = pt;
	m_sigMove = false;
	RUN(m_spInPlaceSite->SetCapture(true));
	return 0;
}
LRESULT LabelMode::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(LabelMode,MouseMove);
	POINT pt = MAKEPOINT(lParam);
	if(m_buttonDown) {
		SIZE s = pt-m_downpt;
		CComPtr<IMontageContainer> mc;
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
		if(!m_sigMove) {
			if(mag2(s)>9) {
				m_sigMove = true;
			}
			else
				return 0;
		}
		CComPtr<IMCCTransform> t;
		RUN(mc->get_Transform(&t));
		POINTL now;
		RUN(t->W2CP(pt,&now));
		POINTD jkl = pointd(now);
		if(m_edge) {
			DGSHAPEDESC *pts;
			RUN(m_edge->GetPoints(&pts));
			double where = close((Coord*)pts->pts,pts->n,pts->isSpline?3:1,coord(jkl),0.200).second;
			CoTaskMemFree(pts);
			RUN(CComQIPtr<IDGEdgeLabel>(m_labsel->l)->put_Where(where));
		}
		else {
			POINTD pos;
			RUN(m_node->get_Pos(&pos));
			enum DG_NODELABELALIGN align;
			if(jkl.y > pos.y+0.5)
				align = DGNLA_TOP;
			else if(jkl.x < pos.x-0.5)
				align = DGNLA_LEFT;
			else if(jkl.x > pos.x+0.5)
				align = DGNLA_RIGHT;
			else if(jkl.y < pos.y-0.5)
				align = DGNLA_BOTTOM;
			else
				align = DGNLA_CENTER;
			RUN(CComQIPtr<IDGNodeLabel>(m_labsel->l)->put_Align(align));
		}
	}
	return 0;
}
LRESULT LabelMode::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(LabelMode,ButtonUp);
	POINT pt = MAKEPOINT(lParam);
	if(!m_buttonDown)
		return 0;
	m_buttonDown = false;
	RUN(m_spInPlaceSite->SetCapture(false));
	if(!m_sigMove) { // click label
		RECT textrect;
		RUN(m_labsel->edit->get_Rect(&textrect));
		if(!m_onBorder) { // if on text, activate it
			MSG msg = message(uMsg,wParam,lParam,pt);
			RUN(activate(m_labsel->edit,&msg));
		}
	}
	return 0;
}
LRESULT LabelMode::DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(LabelMode,DoubleClick);
	POINT pt = MAKEPOINT(lParam);
	double where;
	if(m_edge) {
		CComPtr<IMontageContainer> mc;
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
		CComPtr<IMCCTransform> t;
		RUN(mc->get_Transform(&t));
		POINTL ptC;
		RUN(t->W2CP(pt,&ptC));
		POINTD jkl = pointd(ptC);
		DGSHAPEDESC *pts;
		RUN(m_edge->GetPoints(&pts));
		where = close((Coord*)pts->pts,pts->n,pts->isSpline?3:1,coord(jkl),0.200).second;
		CoTaskMemFree(pts);
	}
	RUN(NewLabel(where,0,0));
	// if dynagraph reacted immediately (not locked) and the label has already been shown
	// (in placeLabel), activate the control.  otherwise this is batch mode
	// and this isn't necessary or possible
	long state;
	RUN(m_labsel->edit->get_Intent(&state));
	if(state>MCCS_HIDDEN)
		RUN(activate(m_labsel->edit));
	return 0;
}
LRESULT LabelMode::Char(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(LabelMode,Char);
	_ASSERTE(!m_active); // if a label is active it should be getting the keys not me
	if(!m_labsel)
		RUN(NewLabel(0.5,0,0));
	MSG msg = message(uMsg,wParam,lParam,point(0,0));
	// dynagraph better not be locked or there's no pos & this will fail
	RUN(activate(m_labsel->edit,&msg));
	MTrace(KE)("LabelMode starting TextEdit with '%c'\n",(char)wParam);
	return 0;
}
// IMCCSiteEvents
STDMETHODIMP LabelMode::Moving(LPCRECT winRect) {
    METHTRACE(LabelMode,Moving);
	// only pay attention to change events from layout side
	return S_OK;
}
STDMETHODIMP LabelMode::Shifting(long oldState, long newState) {
    METHTRACE(LabelMode,Shifting);
	// magic of the LabelMode:
	// - when core node/edge control goes active, this activates on top, expanding the sphere of activation
	// - when control is inactive, LabelMode is hidden
	// - when control gets deleted, m_labels and LabelMode go too
	if(m_spClientSite) {
		if(oldState>=MCCS_ACTIVE && newState<MCCS_ACTIVE)
			RUN(deactivate());
		if(oldState<MCCS_SHOWN && newState>=MCCS_SHOWN)
			RUN(placeLabels());
	}
	long myNewState = newState;
	switch(newState) {
	case MCCS_DELETED: {
		for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il) {
			label &l = *il;
			HRESULT hr = l.edit->put_Intent(MCCS_DELETED);
			if(hr!=MONTAGE_E_DELETED)
				RUN(hr);
			hr = l.owner->stop();
			((IMCCSelectable*)l.owner)->Release();
			l.owner = 0;
		}
		if(m_truffle) {
			if(m_node) {
				RUN(AtlUnadvise(m_node,IID_IDGNodeShadow,m_truffle));
				m_node.Release();
			}
			else if(m_edge) {
				RUN(AtlUnadvise(m_edge,IID_IDGEdgeShadow,m_truffle));
				m_edge.Release();
			}
		}
		break;
	}
	case MCCS_SHOWN:
	case MCCS_HIDDEN:
		myNewState = MCCS_HIDDEN;
		break;
	case MCCS_ACTIVE:
	case MCCS_COUIACTIVE:
	case MCCS_OPEN:
		myNewState = MCCS_COUIACTIVE;
		break;
	case MCCS_UIACTIVE:
		myNewState = MCCS_HIDDEN; // nothing we can do if control is windowed.
		break;
	default:
		_ASSERTE(0); // ?
	}
	if(m_spClientSite)
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Intent(myNewState));
	return S_OK;
}
STDMETHODIMP LabelMode::Layering(IMCCLayer *newLayer) {
    METHTRACE(LabelMode,Layering);
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il)
		RUN(il->edit->put_Layer(newLayer));
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Layer(newLayer));
	return S_OK;
}
STDMETHODIMP LabelMode::ZChanged() {
    METHTRACE(LabelMode,ZChanged);
	// sandwich m_labels between node/edge control and self
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->PutBefore(m_subject));
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il)
		RUN(il->edit->PutBefore(m_subject));
	return S_OK;
}
STDMETHODIMP LabelMode::Wakeup() {
    METHTRACE(LabelMode,Wakeup);
	return S_FALSE;
}
// IMCCExpandSelection
STDMETHODIMP LabelMode::Expand(IItemSet *selection) {
	METHTRACE(LabelMode,Expand);
	for(vector<label>::iterator il = m_labels.begin(); il!=m_labels.end(); ++il) {
		CComPtr<IMoniker> mk;
		RUN(il->edit->get_RelMoniker(&mk));
		RUN(selection->Add(mk));
	}
	return S_OK;
}
// IMCCSelectable
STDMETHODIMP LabelMode::Add(IItemSet *selection) {
    METHTRACE(LabelMode,Add);
	RUN(fireSiteEvent(m_subject,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Add,selection)));
    return S_OK;
}
STDMETHODIMP LabelMode::Remove(IItemSet *selection) {
    METHTRACE(LabelMode,Remove);
	RUN(fireSiteEvent(m_subject,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Remove,selection)));
    return S_OK;
}
STDMETHODIMP LabelMode::Toggle(IItemSet *selection) {
    METHTRACE(LabelMode,Toggle);
	RUN(fireSiteEvent(m_subject,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Toggle,selection)));
    return S_OK;
}
STDMETHODIMP LabelMode::Check(IItemSet *selection) {
    METHTRACE(LabelMode,Check);
	return fireSiteEvent(m_subject,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Check,selection));
}
// IAdviseSink
void STDMETHODCALLTYPE LabelMode::OnDataChange(
        FORMATETC *pFormatetc,
		STGMEDIUM *pStgmed) {
    /*
	_ASSERTE(m_labels.size());
	// really should use IDataObject(CF_TEXT) but this is a last minute hack
	CComPtr<IMCCTextEdit> tedit;
	if(SUCCEEDED(m_labels[0].edit->GetControl(riidret(&tedit)))) {
		CComBSTR text;
		RUN(tedit->get_Text(&text));
		CComQIPtr<IDGStringAttributes> attrs;
		if(m_node)
			attrs = m_node;
		else {
			_ASSERTE(m_edge);
			attrs = m_edge;
		}
		RUN(attrs->SetAttr(CComBSTR("label"),text));
	}
    */
}

void STDMETHODCALLTYPE LabelMode::OnViewChange(
        DWORD dwAspect,
		LONG lindex) {
}
void STDMETHODCALLTYPE LabelMode::OnRename(
	IMoniker *pmk) {
}

void STDMETHODCALLTYPE LabelMode::OnSave(void) {
}

void STDMETHODCALLTYPE LabelMode::OnClose(void) {
}
