/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "LabelSink.h"
#include "atlcompose.h"
class LabelMode :
	public CComObjectRoot,
	public CComCoClass<LabelMode,&CLSID_DGMLabelMode>,
	public CComControl<LabelMode>,
	public IPersistStreamInitImpl<LabelMode>,
	public IOleObjectImpl<LabelMode>,
	public IViewObjectExImpl<LabelMode>,
	public IOleInPlaceObjectWindowlessImpl<LabelMode>,
	public IConnectionPointContainerImpl<LabelMode>,
	public IConnectionPointImpl<LabelMode,&IID_IDGMLabelModeEvents>,
	public ICoUIActiveObject,
	public IDGMLabelMode,
	public LabelSinkClient,
	public IMCCSiteEvents,
	public IMCCExpandSelection,
	public IMCCSelectable, // respond to own site, defer to node ctrl
	public IAdviseSink // listen to first label and send text to dynagraph
{
	// only one of these filled.  would be good to abstract from DG entirely, using label owners
	CComPtr<IDGDrawnNode> m_node;
	CComPtr<IDGDrawnEdge> m_edge;
	CComPtr<IMCCSite> m_subject; // site of node control
	CComPtr<IDataObject> m_labelSource;
	CComPtr<IMCCDataObjectUnpacker> m_reader;

	DWORD m_truffle, // hey that's no c----e!
		m_fudgeMint, // selecct
		m_strawberrySugarWafer; // first label's data

	bool m_buttonDown, // whether..
		m_sigMove, // whether mouse moved much while button down
		m_onBorder; // if button down was on border around text
	POINT m_downpt; // where button down
	POINTL m_startpt; // where moving label started

	struct label {
		CComPtr<IDGLabel> l;
		CComPtr<IMCCSite> edit;
		LabelSink *owner;
		RECTD bounds; // only for edge labels; dynagraph doesn't use this
	};
	vector<label> m_labels;
	label *m_labsel; // selected label
	CComPtr<IMCCSite> m_active;

	DECLARE_GET_CONTROLLING_UNKNOWN()
public:
	LabelMode();
	~LabelMode();

	HRESULT fit();
	HRESULT placeLabel(label &l);
	HRESULT placeLabels();
	HRESULT newLabel(float where); // (where ignored for nodes)
	int findLabel(POINT loc);
	HRESULT inval(IMCCSite *s);

	MSG message(UINT uMsg,WPARAM wParam,LPARAM lParam,POINT pt);
	HRESULT activate(IMCCSite *s,LPMSG msg=0);
	HRESULT deactivate();
	HRESULT ownUp(IMCCSite *s,LabelSink **ret);

    HRESULT fetchLabelText(bool changed,IDGStringAttributes *sattr);

BEGIN_COM_MAP(LabelMode)
	COM_INTERFACE_ENTRY(IDGMLabelMode)
	COM_INTERFACE_ENTRY(IMCCSiteEvents)
	COM_INTERFACE_ENTRY(IMCCExpandSelection)
	COM_INTERFACE_ENTRY(IMCCSelectable)
	COM_INTERFACE_ENTRY(IAdviseSink)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IOleWindow)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_COMPOSED(IDGEdgeShadow,XDGEdgeShadow)
	COM_INTERFACE_ENTRY_COMPOSED(IDGNodeShadow,XDGNodeShadow)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(LabelMode)
	CONNECTION_POINT_ENTRY(IID_IDGMLabelModeEvents)
END_CONNECTION_POINT_MAP()

BEGIN_PROP_MAP(LabelMode)
END_PROP_MAP()

BEGIN_MSG_MAP(LabelMode)
	MESSAGE_HANDLER(WM_LBUTTONDOWN,ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE,MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP,ButtonUp)
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK,DoubleClick);
	MESSAGE_HANDLER(WM_CHAR,Char);
	CHAIN_MSG_MAP(CComControl<LabelMode>)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT Char(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

	REGGY_RID(IDR_LabelMode)
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU,OLEMENUGROUPWIDTHS*);
	STDMETHODIMP Deactivate(HMENU,OLEMENUGROUPWIDTHS*);
// IDGMLabelMode
	STDMETHODIMP Initialize(IUnknown *layoutObject,IMCCSite *content,IDataObject *labelSource,IMCCDataObjectUnpacker *reader);
	STDMETHODIMP put_LayoutObject(IUnknown *lo);
	STDMETHODIMP NewLabel(double where,IMCCSite **retSite,IDGLabel **retLabel);
	STDMETHODIMP AddLabel(IDGLabel *layout, IMCCSite *content);
	STDMETHODIMP get_Content(IMCCSite **ret);
	STDMETHODIMP EnumLabels(IEnumDGMLabelInfo **ret);
// IDGEdgeShadow:
BEGIN_COMPOSED_CLASS(LabelMode,IDGEdgeShadow,XDGEdgeShadow)
    STDMETHODIMP Insert();
    STDMETHODIMP Delete();
    STDMETHODIMP Modify(int flags);
END_COMPOSED_CLASS(XDGEdgeShadow)
// IDGNodeShadow:
BEGIN_COMPOSED_CLASS(LabelMode,IDGNodeShadow,XDGNodeShadow)
    STDMETHODIMP Insert(void);
    STDMETHODIMP Delete(void);
    STDMETHODIMP Modify(int flags);
END_COMPOSED_CLASS(XDGNodeShadow)
// IOleObject -- oh how i hate overriding setclientsite!
	STDMETHODIMP SetClientSite(IOleClientSite *site);
// IViewObject
    STDMETHODIMP Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
         DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdcDraw,LPCRECTL lprcBounds,
         LPCRECTL lprcWBounds,BOOL(STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
         DWORD dwContinue);
    STDMETHODIMP QueryHitPoint(DWORD dwAspect,LPCRECT pRectBounds,POINT ptlLoc,
        LONG lCloseHint,DWORD *pHitResult);
	STDMETHODIMP QueryHitRect(DWORD dwAspect,LPCRECT pRectBounds,LPCRECT pRectLoc,
        LONG lCloseHint,DWORD *pHitResult);
// LabelSinkClient
	IUnknown *mainSite();
	HRESULT Move(LabelSink *from,MCC_POS_INFO *mpi);
	HRESULT OnChanged(LabelSink *from,DISPID dispid);
	void stop();
// IMCCSelectable
	STDMETHODIMP Add(IItemSet *selection);
	STDMETHODIMP Remove(IItemSet *selection);
	STDMETHODIMP Toggle(IItemSet *selection);
	STDMETHODIMP Check(IItemSet *selection);
// IMCCSiteEvents
	STDMETHODIMP Moving(LPCRECT winRect);
	STDMETHODIMP Shifting(long oldState, long newState);
	STDMETHODIMP Layering(IMCCLayer *newLayer);
	STDMETHODIMP ZChanged();
	STDMETHODIMP Wakeup();
// IMCCExpandSelection
	STDMETHODIMP Expand(IItemSet *selection);
// IAdviseSink
    void STDMETHODCALLTYPE OnDataChange(
         FORMATETC *pFormatetc,
         STGMEDIUM *pStgmed);

    void STDMETHODCALLTYPE OnViewChange(
         DWORD dwAspect,
         LONG lindex);

    void STDMETHODCALLTYPE OnRename(
         IMoniker *pmk);

    void STDMETHODCALLTYPE OnSave(void);

    void STDMETHODCALLTYPE OnClose(void);
};
