/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "LabelModePersistor.h"

LabelModePersistor::LabelModePersistor() : m_MintnCreme(0) {
	CONSTRUCTORTRACE(LabelModePersistor);
}
LabelModePersistor::~LabelModePersistor() {
	DESTRUCTORTRACE(LabelModePersistor);
	clear();
}
void LabelModePersistor::clear() {
	for(vector<IMoniker*>::iterator mi = siteMks.begin(); mi!=siteMks.end(); ++mi)
		(*mi)->Release();
	siteMks.clear();
	labelInf.clear();
}
HRESULT LabelModePersistor::infer() { // detective work
    METHTRACE(LabelModePersistor,infer);
	if(!m_subject)
		ERR(E_FAIL);

	// super-weirdness: since we're a clueless persistor (as opposed to those other ones which
	// are in control), we must AddStgRef everything we infer, and then ReleaseStgRef anything we knew before
	vector<IMoniker*> before;
	for(vector<IMoniker*>::iterator im = siteMks.begin(); im!=siteMks.end(); ++im) {
		(*im)->AddRef();
		before.push_back(*im);
	}
	m_mkContent.Release();
	// but m_mkContent is an exception because it's pointing at us (via LabelMode's site)
	clear();
	PointerMoniker mkGroup(m_group);
	BindCtx bctx;
	CComPtr<IMCCSite> s;
	RUN(m_subject->get_Content(&s));
	CComPtr<IMoniker> mkContentP, mkLayout = mkLayoutFromSite(s);
    if(m_mkLayout) {
        HRESULT hr = mkLayout->IsEqual(m_mkLayout);
        RUN(hr);
        if(hr==S_FALSE) {
            RUN(m_group->ReleaseStgRef(m_mkLayout));
            m_mkLayout.Release();
        }
    }
    if(!m_mkLayout)
	    RUN(m_group->AddStgRef(m_mkLayout = mkLayout));
	RUN(s->get_RelMoniker(&mkContentP));
	m_mkContent = ComposeMk(mkContentP,GetSubjectMk());

	CComPtr<IEnumDGMLabelInfo> eli;
	RUN(m_subject->EnumLabels(&eli));
	for(enumLI ili(eli); ili!=enumLI(); ++ili) {
		CComPtr<IMoniker> mkSP;
		RUN(ili->content->get_RelMoniker(&mkSP));
		if(mkSP) {
			IMoniker *mkS=0;
			ComposeMk(mkSP,GetSubjectMk()).CopyTo(&mkS); // keep ref
			siteMks.push_back(mkS);
			RUN(m_group->AddStgRef(mkS));
			labelInfo li;
			if(CComQIPtr<IDGEdgeLabel> el = ili->layout)
				RUN(el->get_Where(&li.whereF));
			else if(CComQIPtr<IDGNodeLabel> nl = ili->layout) {
				RUN(nl->get_Align(&li.whereA));
				RUN(nl->get_Size(&li.size));
			}
			labelInf.push_back(li);
		}
	}
	for(im = before.begin(); im!=before.end(); ++im) {
		RUN(m_group->ReleaseStgRef(*im));
		(*im)->Release();
	}
	return S_OK;
}
// IDGMLabelModePersistor
STDMETHODIMP LabelModePersistor::Initialize(IMoniker *mkLabelSource,IMoniker *mkReader) {
    METHTRACE(LabelModePersistor,Initialize);
	RUN(m_group->AddStgRef(m_mkLabelSource = mkLabelSource));
	RUN(m_group->AddStgRef(m_mkReader = mkReader));
	return S_OK;
}
STDMETHODIMP LabelModePersistor::put_LayoutMk(IMoniker *val) {
    CComPtr<IMoniker> old = m_mkLayout;
	RUN(m_group->AddStgRef(m_mkLayout = val));
    if(old)
        RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
// IDGMLabelModeEvents
STDMETHODIMP LabelModePersistor::LabelsChanged() {
	RUN(infer());
	return S_OK;
}
// DIPersistor
STDMETHODIMP LabelModePersistor::SetSubject(VARIANT_BOOL realized,IUnknown *pUnkSite) {
    METHTRACE(LabelModePersistor,SetSubject);
	if(!realized)
		ERR(E_INVALIDARG);
	if(m_subject)
		RUN(AtlUnadvise(m_subject,IID_IDGMLabelModeEvents,m_MintnCreme));
	if((m_subject = pUnkSite))
		RUN(AtlAdvise(m_subject,GetUnknown(),IID_IDGMLabelModeEvents,&m_MintnCreme));
	if(!m_subject)
		ERR(E_INVALIDARG);
	return S_OK;
}
STDMETHODIMP LabelModePersistor::get_Subject(IUnknown **ret) {
    METHTRACE(LabelModePersistor,get_Subject);
	m_subject.CopyTo((IDGMLabelMode**)ret);
	return S_OK;
}
// IGroupItem
STDMETHODIMP LabelModePersistor::Connect(IGroup *group) {
    METHTRACE(LabelModePersistor,Connect);
	m_group = group;
	return S_OK;
}
STDMETHODIMP LabelModePersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
    METHTRACE(LabelModePersistor,Copy);
	if(m_subject)
		RUN(infer());
	_ASSERTE(m_mkLayout);
	_ASSERTE(labelInf.size()==siteMks.size());
	LabelModePersistor *c;
	CComPtr<IUnknown> p; // get a ref in case QI at end fails
	if(aggr) {
		CComAggObject<LabelModePersistor> *f;
		RUN(CComAggObject<LabelModePersistor>::CreateInstance(aggr,&f));
		c = &f->m_contained;
		p = f;
	}
	else {
		CComObject<LabelModePersistor> *f;
		RUN(CComObject<LabelModePersistor>::CreateInstance(&f));
		c = f;
		p = (DIPersistor*)f;
	}
	RUN(c->Connect(dest));
	CComPtr<IMoniker> mkContent2,mkLayout2,mkLabelSource2,mkReader2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkContent,IID_NULL,0,&mkContent2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkLayout,IID_NULL,0,&mkLayout2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkLabelSource,IID_NULL,0,&mkLabelSource2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkReader,IID_NULL,0,&mkReader2));
	c->m_mkContent = mkContent2;
	RUN(dest->AddStgRef(c->m_mkLayout = mkLayout2));
	RUN(dest->AddStgRef(c->m_mkLabelSource = mkLabelSource2));
	RUN(dest->AddStgRef(c->m_mkReader = mkReader2));
	c->siteMks.resize(siteMks.size());
	for(vector<IMoniker*>::iterator mi = siteMks.begin(); mi!=siteMks.end(); ++mi) {
		int i = mi-siteMks.begin();
		RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,*mi,IID_NULL,0,&c->siteMks[i]));
		RUN(dest->AddStgRef(c->siteMks[i]));
	}
	c->labelInf.insert(c->labelInf.begin(),labelInf.begin(),labelInf.end());
	return p->QueryInterface(riid,ret);
}
STDMETHODIMP LabelModePersistor::Realize(IBindCtx *bctx) {
    METHTRACE(LabelModePersistor,Realize);
	if(m_subject)
		return S_FALSE;
	PointerMoniker mkGroup(m_group);
	CComPtr<IDGMLabelMode> subj;
	RUN(CREATE(subj,CLSID_DGMLabelMode));
	CComPtr<IUnknown> layout;
	RUN(m_mkLayout->BindToObject(bctx,mkGroup,riidret(&layout)));
	CComPtr<IMCCSite> content;
	RUN(m_mkContent->BindToObject(bctx,mkGroup,riidret(&content)));
	CComPtr<IDataObject> labelSource;
	RUN(m_mkLabelSource->BindToObject(bctx,mkGroup,riidret(&labelSource)));
	CComPtr<IMCCDataObjectUnpacker> reader;
	RUN(m_mkReader->BindToObject(bctx,mkGroup,riidret(&reader)));
	RUN(subj->Initialize(layout,content,labelSource,reader));
	CComQIPtr<IDGDrawnNode> n = layout;
	CComQIPtr<IDGDrawnEdge> e = layout;
	_ASSERTE(n||e);
	for(vector<labelInfo>::iterator ili = labelInf.begin(); ili!=labelInf.end(); ++ili) {
		int i = ili-labelInf.begin();
		CComPtr<IMCCSite> s;
		RUN(siteMks[i]->BindToObject(bctx,mkGroup,riidret(&s)));
		CComPtr<IDGLabel> l;
		if(n) {
			CComPtr<IDGNodeLabel> nl;
			RUN(n->new_Label(&nl));
			RUN(nl->put_Align(ili->whereA));
			RUN(nl->put_Size(ili->size));
			l = nl;
		}
		else {
			CComPtr<IDGEdgeLabel> el;
			RUN(e->new_Label(&el));
			RUN(el->put_Where(ili->whereF));
			l = el;
		}
		RUN(subj->AddLabel(l,s));
	}
	RUN(SetSubject(true,subj));
	return S_OK;
}
STDMETHODIMP LabelModePersistor::Remove() {
    METHTRACE(LabelModePersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkLayout)); // but not content cuz that points at me
	RUN(m_group->ReleaseStgRef(m_mkLabelSource));
	RUN(m_group->ReleaseStgRef(m_mkReader));
	for(vector<IMoniker*>::iterator im = siteMks.begin(); im!=siteMks.end(); ++im)
		RUN(m_group->ReleaseStgRef(*im));
	return S_OK;
}
// IPersistStream
STDMETHODIMP LabelModePersistor::GetClassID(CLSID *pClassID) {
    METHTRACE(LabelModePersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP LabelModePersistor::IsDirty(void) {
    METHTRACE(LabelModePersistor,IsDirty);
	return S_OK;
}
STDMETHODIMP LabelModePersistor::Load(IStream *pStm) {
    METHTRACE(LabelModePersistor,Load);
	if(m_mkLayout || labelInf.size())
		ERR(E_FAIL);
	RUN(OleLoadFromStream(pStm,riidret(&m_mkLayout)));
	RUN(OleLoadFromStream(pStm,riidret(&m_mkContent)));
	RUN(OleLoadFromStream(pStm,riidret(&m_mkLabelSource)));
	RUN(OleLoadFromStream(pStm,riidret(&m_mkReader)));
	int n;
	RUN(pStm->Read(&n,sizeof(int),0));
	labelInf.resize(n);
	if(n)
		RUN(pStm->Read(&*labelInf.begin(),sizeof(labelInfo)*n,0));
	while(n) {
		IMoniker *m;
		RUN(OleLoadFromStream(pStm,riidret(&m)));
		siteMks.push_back(m);
		--n;
	}
	return S_OK;
}
STDMETHODIMP LabelModePersistor::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(LabelModePersistor,Save);
	if(m_subject)
		RUN(infer());
	_ASSERTE(m_mkLayout);
	_ASSERTE(labelInf.size()==siteMks.size());
	RUN(OleSaveToStream(m_mkLayout,pStm));
	RUN(OleSaveToStream(m_mkContent,pStm));
	RUN(OleSaveToStream(m_mkLabelSource,pStm));
	RUN(OleSaveToStream(m_mkReader,pStm));
	int n = labelInf.size();
	RUN(pStm->Write(&n,sizeof(int),0));
	if(n)
		RUN(pStm->Write(&*labelInf.begin(),sizeof(labelInfo)*n,0));
	for(vector<IMoniker*>::iterator im = siteMks.begin(); im!=siteMks.end(); ++im)
		RUN(OleSaveToStream(*im,pStm));
	return S_OK;
}
STDMETHODIMP LabelModePersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(LabelModePersistor,GetSizeMax);
	ERR(E_NOTIMPL);
}