/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "StdAfx.h"
#include "LabelSink.h"
#include "fireSiteEvent.h"
#include "fireFunctors.h"

LabelSink::LabelSink() : client(0) {
	CONSTRUCTORTRACE(LabelSink);
}
LabelSink::~LabelSink() {
	DESTRUCTORTRACE(LabelSink);
}
HRESULT LabelSink::init(LabelSinkClient *client,IMCCSite *labelSite) {
    METHTRACE(LabelSink,init);
	this->client = client;
	this->site = labelSite;
	RUN(APCAdvise(labelSite,(IMCCSelectable*)this,IID_IMCCSelectable,&m_oatmealRaisin));
	RUN(APCAdvise(labelSite,(IMCCSiteOwner*)this,IID_IMCCSiteOwner,&m_grasshopper));
	RUN(APCAdvise(labelSite,(IMCCSiteEvents*)this,IID_IMCCSiteEvents,&m_twix));
	CComPtr<IUnknown> unkontrol;
	RUN(labelSite->GetControl(riidret(&unkontrol)));
	RUN(AtlAdvise(unkontrol,(IPropertyNotifySink*)this,IID_IPropertyNotifySink,&m_dogBiscuit));
	return S_OK;
}
HRESULT LabelSink::stop() {
    METHTRACE(LabelSink,stop);
	RUNRET(AtlUnadvise(site,IID_IMCCSelectable,m_oatmealRaisin));
	RUNRET(AtlUnadvise(site,IID_IMCCSiteOwner,m_grasshopper));
	RUNRET(AtlUnadvise(site,IID_IMCCSiteEvents,m_twix));
	CComPtr<IUnknown> unkontrol;
	RUN(site->GetControl(riidret(&unkontrol)));
	RUNRET(AtlUnadvise(unkontrol,IID_IPropertyNotifySink,m_dogBiscuit));
	return S_OK;
}

// in all cases but IMCCSiteOwner::Move, the reaction is to bounce the request on
// to the node or edge.

// IMCCSelectable
STDMETHODIMP LabelSink::Add(IItemSet *selection) {
    METHTRACE(LabelSink,Add);
	RUN(fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Add,selection)));
    return S_OK;
}
STDMETHODIMP LabelSink::Remove(IItemSet *selection) {
    METHTRACE(LabelSink,Remove);
	RUN(fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Remove,selection)));
    return S_OK;
}
STDMETHODIMP LabelSink::Toggle(IItemSet *selection) {
    METHTRACE(LabelSink,Toggle);
	RUN(fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Toggle,selection)));
    return S_OK;
}
STDMETHODIMP LabelSink::Check(IItemSet *selection) {
    METHTRACE(LabelSink,Check);
	return fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Check,selection));
}
// IMCCSiteOwner
STDMETHODIMP LabelSink::Move(MCC_POS_INFO *mpi) {
    METHTRACE(LabelSink,Move);
	// only happens on resize (e.g. typing makes bigger)
	return client->Move(this,mpi);
}
STDMETHODIMP LabelSink::PutBefore(IMCCSite *other) {
    METHTRACE(LabelSink,PutBefore);
	return fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSiteOwner,IMCCSite *>(&IMCCSiteOwner::PutBefore,other));
}
STDMETHODIMP LabelSink::PutBehind(IMCCSite *other) {
    METHTRACE(LabelSink,PutBehind);
	return fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSiteOwner,IMCCSite *>(&IMCCSiteOwner::PutBehind,other));
}
STDMETHODIMP LabelSink::Intend(long intent) {
    METHTRACE(LabelSink,Intend);
	// inconsistent.
	return fireSiteEvent(CComQIPtr<IMCCSite>(client->mainSite()),fireOneParam<IMCCSiteOwner,long>(&IMCCSiteOwner::Intend,intent));
}
// IMCCSiteEvents
STDMETHODIMP LabelSink::Moving(LPCRECT winRect) {
    METHTRACE(LabelSink,Moving);
	return S_OK;
}
STDMETHODIMP LabelSink::Shifting(long oldState, long newState) {
    METHTRACE(LabelSink,Shifting);
	if(newState==MCCS_DELETED)
		client->stop();
	return S_OK;
}
STDMETHODIMP LabelSink::ZChanged() {
    METHTRACE(LabelSink,ZChanged);
	return S_OK;
}
STDMETHODIMP LabelSink::Layering(IMCCLayer *newLayer) {
    METHTRACE(LabelSink,Layering);
	return S_OK;
}
STDMETHODIMP LabelSink::Wakeup() {
    METHTRACE(LabelSink,Wakeup);
	return S_OK;
}
// IPropertyNotifySink
STDMETHODIMP LabelSink::OnChanged(DISPID id) {
	return client->OnChanged(this,id);
}
STDMETHODIMP LabelSink::OnRequestEdit(DISPID id) {
	return S_OK;
}
