/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#pragma once

struct LabelSink;
struct LabelSinkClient {
	virtual IUnknown *mainSite() = 0;
	virtual HRESULT Move(LabelSink *from,MCC_POS_INFO *mpi) = 0;
	virtual HRESULT OnChanged(LabelSink *from,DISPID dispid) = 0;
	virtual void stop() = 0;
};

struct LabelSink :
	CComObjectRoot,
	IMCCSelectable,
	IMCCSiteOwner,
	IMCCSiteEvents,
	IPropertyNotifySink
{
	LabelSinkClient *client;
	CComPtr<IMCCSite> site;
	// remember connection IDs to undo later
	DWORD m_oatmealRaisin,m_grasshopper,m_twix,m_dogBiscuit;

	LabelSink();
	~LabelSink();

	HRESULT init(LabelSinkClient *client,IMCCSite *labelSite);
	HRESULT stop();

BEGIN_COM_MAP(LabelSink)
	COM_INTERFACE_ENTRY(IMCCSelectable)
	COM_INTERFACE_ENTRY(IMCCSiteOwner)
	COM_INTERFACE_ENTRY(IMCCSiteEvents)
	COM_INTERFACE_ENTRY(IPropertyNotifySink)
END_COM_MAP()

// IMCCSelectable
	STDMETHODIMP Add(IItemSet *selection);
	STDMETHODIMP Remove(IItemSet *selection);
	STDMETHODIMP Toggle(IItemSet *selection);
	STDMETHODIMP Check(IItemSet *selection);

// IMCCSiteOwner
    STDMETHODIMP Move(MCC_POS_INFO *mpi);
    STDMETHODIMP PutBefore(IMCCSite *other);
    STDMETHODIMP PutBehind(IMCCSite *other);
	STDMETHODIMP Intend(long intent);

// IMCCSiteEvents
	STDMETHODIMP Moving(LPCRECT winRect);
	STDMETHODIMP Shifting(long oldState, long newState);
	STDMETHODIMP ZChanged();
	STDMETHODIMP Layering(IMCCLayer *newLayer);
	STDMETHODIMP Wakeup();

// IPropertyNotifySink
	STDMETHODIMP OnChanged(DISPID id);
	STDMETHODIMP OnRequestEdit(DISPID id);
};
