/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "allocDGShapes.h"

// code to convert a metafile to a DGSHAPESDESC

// note this is tuned precisely for ATL's IDataObjectImpl, which
// returns a metafile drawn in upside-down (y increases down)
// HIMETRIC.  I think that may be a standard but I haven't found
// any documentation (OLE Prog. Reference is out of print).  It
// looks like everyone supported the win3.1 standard HMETAFILEPICT
// but then there's no incentive to move to HENHMETAFILE.

// anyway I understand very little of that (too many coordinate
// systems for 1 brain!), but I do know if you set m_bDrawGetDataInHimetric
// in the control constructor and inherit from IDataObjectImpl, your
// control will draw to 0,0,extX,extY, which just needs to be flipped
// to get canvas coordinates, which I then convert to POINTDs.

struct SWorking {
	POINTL pos;
	double xmult, ymult;
	POINTL center;
	vector<PDGSHAPEDESC> shapes;
	void setpos(POINTS pts) {
		pos.x = pts.x; pos.y = pts.y;
	}
	void setpos(POINTL ptl) {
		pos = ptl;
	}
};
// HIMETRIC to POINTD, with centering
template<class PT>
inline POINTD HM2PD(SWorking *pWork,PT pt) {
	// convert to Cartesian centimeters
	POINTD ret = {double(pt.x - pWork->center.x)*pWork->xmult,
		double(pt.y - pWork->center.y)*pWork->ymult};
	return ret;
}
inline bool ISSPLINE(int iType) {
	return ((iType==EMR_POLYBEZIER)||(iType==EMR_POLYBEZIERTO)||
	 (iType==EMR_POLYBEZIER16)||(iType==EMR_POLYBEZIERTO16));
}
inline bool UPDATESPOS(int iType) {
	return ((iType==EMR_POLYBEZIERTO)||(iType==EMR_POLYLINETO)||
	 (iType==EMR_POLYBEZIERTO16)||(iType==EMR_POLYLINETO16));
}
template<class PT>
BOOL Poly(SWorking *pWork,DWORD iType,DWORD cpt,const PT rgpt[],bool close)
{
	int N = cpt;
	if(UPDATESPOS(iType)||close)
		N++;
	DGSHAPEDESC *curr = allocDGShape(N);
	curr->isSpline = ISSPLINE(iType);
	POINTD *pi = curr->pts;
	if(UPDATESPOS(iType))
		*pi++ = HM2PD(pWork,pWork->pos);
	for(DWORD i = 0;i < cpt; i++)
		*pi++ = HM2PD(pWork,rgpt[i]);
	if(close)
		*pi = curr->pts[0];
	if(UPDATESPOS(iType))
		pWork->setpos(rgpt[cpt-1]);
	pWork->shapes.push_back(curr);
	return TRUE;
}
template<class PT>
BOOL PolyPoly(SWorking *pWork,DWORD iType,DWORD nPolys,DWORD rgPolySize[],
			  PT *pptl,bool close) {
	for(DWORD i = 0; i < nPolys; i++) {
		int N = rgPolySize[i];
		if(close)
			N++;
		DGSHAPEDESC *curr = allocDGShape(N);
		curr->isSpline = false;
		for(DWORD j = 0; j < rgPolySize[i]; j++)
			curr->pts[j] = HM2PD(pWork,*pptl++);
		if(close)
			curr->pts[j] = curr->pts[0];
		pWork->shapes.push_back(curr);
	}
	return TRUE;
}
static int CALLBACK EMFCallback(
    HDC hDC,	// handle to device context
    HANDLETABLE FAR *lpHTable,	// pointer to metafile handle table
    const ENHMETARECORD FAR *lpEMFR,	// pointer to metafile record
    int nObj,	// count of objects
    LPARAM lpData 	// pointer to optional data
   ) {
	SWorking *pWork = (SWorking *)lpData;
	// a reasonable subset of shape-related stuff, I think:
	switch(lpEMFR->iType) {
	case EMR_MOVETOEX:
	{
		PEMRMOVETOEX pMove = (PEMRMOVETOEX)lpEMFR;
		pWork->setpos(pMove->ptl);
		break;
	}
	case EMR_LINETO:
	{
		PEMRLINETO pLine = (PEMRLINETO)lpEMFR;
		PDGSHAPEDESC curr = allocDGShape(2);
		curr->isSpline = false;
		curr->n = 2;
		curr->pts[0] = HM2PD(pWork,pWork->pos);
		curr->pts[1] = HM2PD(pWork,pLine->ptl);
		pWork->setpos(pLine->ptl);
		pWork->shapes.push_back(curr);
		break;
	}
	case EMR_ARC:
	case EMR_ARCTO:
	{
		// not yet implemented.
		/*
		PEMRARC pArc = (PEMRARC)lpEMFR;
		pWork->curr = aggr_shape(pWork->curr);
		*/
		break;
	}
	case EMR_POLYBEZIER:
	case EMR_POLYGON:
	case EMR_POLYLINE:
	case EMR_POLYBEZIERTO:
	case EMR_POLYLINETO:
	{
		PEMRPOLYGON pPoly = (PEMRPOLYGON)lpEMFR;
		Poly(pWork,lpEMFR->iType,pPoly->cptl,pPoly->aptl,lpEMFR->iType==EMR_POLYGON);
		break;
	}
	case EMR_POLYBEZIER16:
	case EMR_POLYGON16:
	case EMR_POLYLINE16:
	case EMR_POLYBEZIERTO16:
	case EMR_POLYLINETO16:
	{
		PEMRPOLYGON16 pPoly16 = (PEMRPOLYGON16)lpEMFR;
		Poly(pWork,lpEMFR->iType,pPoly16->cpts,pPoly16->apts,lpEMFR->iType==EMR_POLYGON16);
		break;
	}
	case EMR_POLYPOLYLINE:
	case EMR_POLYPOLYGON:
	{
		PEMRPOLYPOLYLINE pPolPol = (PEMRPOLYPOLYLINE)lpEMFR;
		// wingdi sez aPolyCounts is a DWORD[1], and then aptl comes after;
		// I'll believe the latter but not the former.
		PolyPoly(pWork,lpEMFR->iType,pPolPol->nPolys,pPolPol->aPolyCounts,
			(PPOINTL)(pPolPol->aPolyCounts+pPolPol->nPolys),lpEMFR->iType==EMR_POLYPOLYGON);
		break;
	}
	case EMR_POLYPOLYLINE16:
	case EMR_POLYPOLYGON16:
	case EMR_POLYDRAW16:
	{
		PEMRPOLYPOLYLINE16 pPolPol16 = (PEMRPOLYPOLYLINE16)lpEMFR;
		// wingdi sez aPolyCounts is a DWORD[1], and then apts comes after;
		// I'll believe the latter but not the former.
		PolyPoly(pWork,lpEMFR->iType,pPolPol16->nPolys,pPolPol16->aPolyCounts,
			(PPOINTS)(pPolPol16->aPolyCounts+pPolPol16->nPolys),lpEMFR->iType==EMR_POLYPOLYGON16);
		break;
	}
	case EMR_ELLIPSE:
	{
		// NYI
		break;
	}
	case EMR_RECTANGLE:
	case EMR_ROUNDRECT: //close 'nuff 4 now.
	{
		PEMRRECTANGLE pRect = (PEMRRECTANGLE)lpEMFR;
		PDGSHAPEDESC curr = allocDGShape(5);
		curr->isSpline = false;
		POINT pt;
		pt.x = pRect->rclBox.left;
		pt.y = pRect->rclBox.top;
		curr->pts[0] = curr->pts[4] = HM2PD(pWork,pt);
		pt.x = pRect->rclBox.right;
		curr->pts[1] = HM2PD(pWork,pt);
		pt.y = pRect->rclBox.bottom;
		curr->pts[2] = HM2PD(pWork,pt);
		pt.x = pRect->rclBox.left;
		curr->pts[3] = HM2PD(pWork,pt);
		break;
	}
	case EMR_POLYDRAW:;
	}
	return TRUE;
}
PDGSHAPESDESC EMF2SH(HENHMETAFILE hEMF,POINTL center) {
	SWorking work;
	work.center = center;
	work.xmult = 0.001; // to cm
	work.ymult = -0.001;
	work.pos.x = work.pos.y = 0;
	EnumEnhMetaFile(0,hEMF,EMFCallback,&work,0);
	DGSHAPESDESC *ret = allocDGShapes(work.shapes.size());
	memcpy(ret->sps,&*work.shapes.begin(),work.shapes.size()*sizeof(PDGSHAPEDESC));
	return ret;
}
PDGSHAPESDESC MF2SH(LPMETAFILEPICT mfp,POINTL center) {
	UINT len;
	// convert to EMF.  EMF is preferred, but nobody supports it yet!
	// wait'll they see those Beziers
	if(!(len = GetMetaFileBitsEx(mfp->hMF,0,0)))
		return 0;
	BYTE *buf = new BYTE[len];
	if(!GetMetaFileBitsEx(mfp->hMF,len,buf))
		return 0;
	HENHMETAFILE hEnh = SetWinMetaFileBits(len,buf,0,mfp);
	delete [] buf;
	if(!hEnh)
		return 0;
	PDGSHAPESDESC ret = EMF2SH(hEnh,center);
	DeleteEnhMetaFile(hEnh);
	return ret;
}

/*
ilshape_t *MFP2Shape(LPMETAFILEPICT lpmfp) {
	UINT sz;
	// if we can convert this to EMF, lets:
	if(sz = GetMetaFileBitsEx(lpmfp->hMF,900,0)) {
		BYTE *buf = new BYTE[sz];
		if(!GetMetaFileBitsEx(lpmfp->hMF,sz,buf))
			return 0;
		HENHMETAFILE hEnh = SetWinMetaFileBits(sz,buf,0,lpmfp);
		delete [] buf;

		if(!hEnh)
			return 0;
		return EMF2Shape(hEnh);
	}
	else return 0;
}
ilshape_t *DO2Shape(IDataObject *pData) {
	HRESULT hr;
	FORMATETC fetz;
	ilshape_t *pRet = 0;
	fetz.cfFormat = CF_ENHMETAFILE;
	fetz.ptd = 0; // ?
	fetz.dwAspect = DVASPECT_CONTENT;
	fetz.lindex = -1;
	fetz.tymed = TYMED_ENHMF;
	STGMEDIUM stuj;
	stuj.tymed = TYMED_ENHMF;
	stuj.pUnkForRelease = 0;
	hr = pData->GetData(&fetz,&stuj);
	if(SUCCEEDED(hr)&&(stuj.tymed==TYMED_ENHMF)) {
		pRet = EMF2Shape(stuj.hEnhMetaFile);
		ReleaseStgMedium(&stuj);
	}
	else if(hr == DV_E_FORMATETC) { // probably didn't know CF_ENHMETAFILE
		fetz.cfFormat = CF_METAFILEPICT;
		fetz.tymed = stuj.tymed = TYMED_MFPICT;
		if(SUCCEEDED(pData->GetData(&fetz,&stuj))&&(stuj.tymed==TYMED_MFPICT)) {
			// MFPICT seems to mean hGlobal to METAFILEPICT
			LPMETAFILEPICT lpmfp = (LPMETAFILEPICT)GlobalLock(stuj.hGlobal);
			if(lpmfp)
				pRet = MFP2Shape(lpmfp);
			ReleaseStgMedium(&stuj);
		}
	}
	return pRet;
}
*/