/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// NodeShadow.cpp : implementation of NodeShadow, the
// glue between engine node and node control

#include "stdafx.h"
#include "NodeShadow.h"
#include "mf2sh.h"
#include "allocDGShapes.h"

NodeShadow::NodeShadow() : m_inserted(false), m_amSettingExtent(false),m_shape(0),m_mandm(0) {
	CONSTRUCTORTRACE(NodeShadow);
	m_extent.cx = m_extent.cy = 0;
}

NodeShadow::~NodeShadow() {
	DESTRUCTORTRACE(NodeShadow);
	if(m_shape)
		freeShape();
}
void NodeShadow::freeShape() {
	freeDGShapes(m_shape);
	m_shape = 0;
}
HRESULT NodeShadow::getExtent() {
	CComPtr<IOleObject> oo;
	CComPtr<IViewObject2> vo2;
	if(SUCCEEDED(m_msite->GetControl(riidret(&oo))))
		return oo->GetExtent(DVASPECT_CONTENT,&m_extent);
	else if(SUCCEEDED(m_msite->GetControl(riidret(&vo2))))
		return vo2->GetExtent(DVASPECT_CONTENT,-1,0,&m_extent);
	else
		ERR(E_FAIL);
}
FormatEtc g_mfpictFormat(CF_METAFILEPICT),
	g_emfFormat(CF_ENHMETAFILE);
// if someone has registered control as DGMMetafileShape, the metafile data
// is appropriate for inferring a shape.
HRESULT NodeShadow::adviseIfAppropriate() {
    METHTRACE(NodeShadow,adviseIfAppropriate);
	// m_mandm (the DAdvise cookie) is the sign that this is a shape-inferable node
	CComPtr<IDataObject> doCtrl;
	if(!m_mandm && SUCCEEDED(m_msite->GetControl(riidret(&doCtrl)))) {
		CLSID clsid;
		RUN(getObjectCLSID(doCtrl,&clsid));
		CComPtr<ICatInformation> catinf;
		RUN(CREATE(catinf,CLSID_StdComponentCategoriesMgr));
		HRESULT hr = catinf->IsClassOfCategories(clsid,1,const_cast<CATID*>(&CATID_DGMMetafileShape),0,0);
		if(hr==S_OK)
#ifdef USE_MFPICT
			RUN(doCtrl->DAdvise(&g_mfpictFormat,0,this,&m_mandm));
#else
			RUN(doCtrl->DAdvise(&g_emfFormat,0,this,&m_mandm));
#endif
	}
	return S_OK;
}
HRESULT NodeShadow::findShape() {
    METHTRACE(NodeShadow,findShape);
	if(m_mandm) { // advisory set up, get shape
		CComPtr<IDataObject> doCtrl;
		RUN(m_msite->GetControl(riidret(&doCtrl)));
		StgMedium stgm;
#ifdef USE_MFPICT
		RUN(doCtrl->GetData(&g_mfpictFormat,&stgm));
		getShape(&g_mfpictFormat,&stgm);
#else
		RUN(doCtrl->GetData(&g_emfFormat,&stgm));
		getShape(&g_emfFormat,&stgm);
#endif
		ReleaseStgMedium(&stgm);
		return S_OK;
	}
	// object isn't appropriate for shape inference: just use bounding rect
	m_shape = allocDGShapes(1);
	m_shape->n = 1;
	m_shape->sps[0] = allocDGShape(5);
	m_shape->sps[0]->isSpline = false;
	m_shape->sps[0]->n = 5;
	m_shape->sps[0]->pts[0] = pointd(m_extent.cx/2000.0,m_extent.cy/2000.0);
	m_shape->sps[0]->pts[1] = pointd(m_extent.cx/2000.0,-m_extent.cy/2000.0);
	m_shape->sps[0]->pts[2] = pointd(-m_extent.cx/2000.0,-m_extent.cy/2000.0);
	m_shape->sps[0]->pts[3] = pointd(-m_extent.cx/2000.0,m_extent.cy/2000.0);
	m_shape->sps[0]->pts[4] = m_shape->sps[0]->pts[0];
	return S_OK;
}
void NodeShadow::getShape(FORMATETC *fetc, STGMEDIUM *stgm) {
    METHTRACE(NodeShadow,getShape);
	if(m_shape)
		freeShape();
#ifdef USE_MFPICT
	_ASSERTE(fetc->cfFormat==CF_METAFILEPICT);
	_ASSERTE(fetc->dwAspect==DVASPECT_CONTENT);
	_ASSERTE(fetc->tymed==TYMED_MFPICT);
	METAFILEPICT *mfp = LPMETAFILEPICT(GlobalLock(stgm->hMetaFilePict));
	// should expose as prop
	POINTL center = {mfp->xExt/2,mfp->yExt/2};
	m_shape = MF2SH(mfp,center);
	DeleteMetaFile(mfp->hMF);
	GlobalUnlock(stgm->hMetaFilePict);
#else
	_ASSERTE(fetc->cfFormat==CF_ENHMETAFILE);
	_ASSERTE(fetc->dwAspect==DVASPECT_CONTENT);
	_ASSERTE(fetc->tymed==TYMED_ENHMF);
	ENHMETAHEADER header;
	CHECK(GetEnhMetaFileHeader(stgm->hEnhMetaFile,sizeof header,&header));
	POINTL center = ctr(header.rclBounds);
	m_shape = EMF2SH(stgm->hEnhMetaFile,center);
#endif
}
HRESULT NodeShadow::getrect(POINTL pos, PRECTL prc) {
    METHTRACE(NodeShadow,getrect);
	if(!m_msite)
		ERR(E_PENDING);
	RECTD r;
	RUN(m_node->GetRect(&r));
	*prc = rectl(r);
	/*
	prc->left = pos.x - m_extent.cx/2;
	prc->top = pos.y + m_extent.cy/2;
	prc->right = prc->left + m_extent.cx;
	prc->bottom = prc->top - m_extent.cy;
	*/
	return S_OK;
}
// IDGMConnector
STDMETHODIMP NodeShadow::put_LayoutObject(IUnknown *val) {
	METHTRACE(NodeShadow,put_LayoutObject);
	if(m_node)
		RUN(AtlUnadvise(m_node,IID_IDGNodeShadow,m_dwNodeCookie));
	if((m_node = val))
		RUN(AtlAdvise(m_node,(IDGNodeShadow*)this,IID_IDGNodeShadow,&m_dwNodeCookie));
	return S_OK;
}
STDMETHODIMP NodeShadow::get_LayoutObject(IUnknown **ret) {
	METHTRACE(NodeShadow,get_LayoutObject);
	m_node.CopyTo((IDGDrawnNode**)ret);
	return S_OK;
}
STDMETHODIMP NodeShadow::put_Site(DIMCCSite *val) {
    METHTRACE(NodeShadow,put_Site);
	return SetSite(val);
}
// DGNodeShadow
STDMETHODIMP NodeShadow::Insert(void) {
	METHTRACE(NodeShadow,Insert);
	if(!m_msite)
		ERR(E_PENDING);
	if(m_inserted)
		return S_FALSE;
	m_inserted = TRUE;
	POINTD p;
	RUN(m_node->get_Pos(&p));
	RECTL r;
	RUN(getrect(pointl(p),&r));
	RUN(m_msite->put_VirtRect(&r));
	RUN(m_msite->put_Intent(m_nextState));
	return S_OK;
}
STDMETHODIMP NodeShadow::Delete(void) {
	METHTRACE(NodeShadow,Delete);
	if(!m_msite)
		ERR(E_PENDING);
	if(!m_inserted)
		return S_OK;
	m_inserted = FALSE;
	CComPtr<IDGMNodeControl> nc;
	if(SUCCEEDED(m_msite->GetControl(riidret(&nc))))
		RUN(nc->put_Node(0));
	else if(m_mandm) { // release advisory sink if any
		CComPtr<IDataObject> dat;
		RUN(m_msite->GetControl(riidret(&dat)));
		RUN(dat->DUnadvise(m_mandm));
		m_mandm = 0;
	}
	RUN(m_msite->put_Intent(MCCS_DELETED));
	return S_OK;
}
STDMETHODIMP NodeShadow::Modify(int flags) {
    if(flags&(DGMF_POS|DGMF_SHAPE)) {
	    if(!m_msite)
		    ERR(E_PENDING);
        POINTD pt;
        RUN(m_node->get_Pos(&pt));
	    RECTL r;
	    RUN(getrect(pointl(pt),&r));
	    // it likes this extent so don't let it call back and mess us up!
	    m_amSettingExtent = true;
	    RUN(m_msite->put_VirtRect(&r));
	    m_amSettingExtent = false;
    }
    return S_OK;
}
// IMCCSiteOwner
STDMETHODIMP NodeShadow::Move(MCC_POS_INFO *pos) {
	METHTRACE(NodeShadow,Move);
	HRESULT hr;
	if(!m_msite)
		ERR(E_PENDING);
	if(!pos)
		ERR(E_POINTER);
	if(m_amSettingExtent)
		return S_OK;

	// get transform
	CComPtr<IMontageContainer> cc;
	RUN(m_msite->GetContainer(riidret(&cc)));
	CComPtr<IMCCTransform> t;
	RUN(cc->get_Transform(&t));

	// work pos into a canvas point
	MCC_POS_INFO p2 = *pos; // so we can mess w/ it
	switch(pos->postype) {
	case MCC_POS_WINDOWPOINT:
		RUN(t->W2CP(pos->pt,&p2.ptl));
		goto have_canvaspoint;
	case MCC_POS_WINDOWRECT:
		RUN(t->W2CR(&pos->rc,&p2.rcl));
	case MCC_POS_CANVASRECT:
		// this overwrite is actually safe because we write over X w/ X, Y w/ Y
		p2.pt.x = (p2.rcl.left+p2.rcl.right)/2; // (eg this is pt.rcl.left = ...)
		p2.pt.y = (p2.rcl.top+p2.rcl.bottom)/2;
	case MCC_POS_CANVASPOINT:
	have_canvaspoint:
		RUN(m_node->put_Pos(pointd(p2.pt.x/1000.0,p2.pt.y/1000.0)));
		break;
	case MCC_POS_WINDOWSIZE:
	case MCC_POS_CANVASSIZE:
		break; // no position
	default:
		ERR(E_UNEXPECTED);
	}

	// resize control if necessary
	// find new extent
	bool haveNewExtent(true);
	switch(pos->postype) {
	case MCC_POS_WINDOWRECT: {
		SIZE s = {wid(pos->rc),-hei(pos->rc)}; // negative b/c W2CS is *vector*
		RUN(t->W2CS(s,&m_extent));
		break;
	}
	case MCC_POS_CANVASRECT:
		m_extent = siz(pos->rcl);
		break;
	case MCC_POS_WINDOWSIZE:
		RUN(t->W2CS(pos->s,&m_extent));
		m_extent.cy *= -1;
		break;
	case MCC_POS_CANVASSIZE:
		m_extent = pos->sl;
		break;
	default:
		haveNewExtent = false;
	}

	// figure out whether control's shape can be inferred
	RUN(adviseIfAppropriate());

	bool newShape(false);
	if(haveNewExtent) {
		CComPtr<IDGMNodeControl> nctrl;
		if(SUCCEEDED(m_msite->GetControl(riidret(&nctrl)))) // control delegates entirely to dynagraph
			return CComQIPtr<DIDGShapeDefinition>(m_node)->SetOutside(m_extent.cx/1000.0,m_extent.cy/1000.0);
		// call object's SetExtent if m_extent differs from canvas extent or object's internal extent
		// call my findShape if object is new or extent has changed
		SIZEL oldExtent;
		bool gotOldExtent(false),setExtent(false);
		{ // look at canvas first
			RECTL rcl;
			if(SUCCEEDED(m_msite->get_VirtRect(&rcl))) {
				oldExtent = siz(rcl);
				gotOldExtent = true;
			}
			else newShape = true;
		}
		CComPtr<IOleObject> oo;
		m_msite->GetControl(riidret(&oo));
		if(!gotOldExtent && oo && SUCCEEDED(oo->GetExtent(DVASPECT_CONTENT,&oldExtent)))
			gotOldExtent = true;
		if(gotOldExtent)
			// these HIMETRIC coords can get knocked by one or two and it doesn't mean anything.
			if(tabs(oldExtent.cx-m_extent.cx)>2||tabs(oldExtent.cy-m_extent.cy)>2)
				newShape = setExtent = true;
		if(!m_mandm)
			setExtent = false; // semiHACK: shouldn't set extent unless we need to (messes up Excel offset)
		if(setExtent && oo) {
			m_amSettingExtent = true;
			hr = oo->SetExtent(DVASPECT_CONTENT,&m_extent); // method not obligatory
			m_amSettingExtent = false;
			RUN(getExtent()); // it may "interpret"
		}
	}
	else newShape = true; // mmmmffff. bad, bad, bad.
	if(newShape) {
		RUN(findShape());
		Trigger(); // go ahead and send shape to node
	}
    return S_OK;
}
STDMETHODIMP NodeShadow::PutBefore(IMCCSite *other) {
	METHTRACE(NodeShadow,PutBefore);
	return S_FALSE; // default processing
}
STDMETHODIMP NodeShadow::PutBehind(IMCCSite *other) {
	METHTRACE(NodeShadow,PutBehind);
	return S_FALSE; // default processing
}
STDMETHODIMP NodeShadow::Intend(long intent) {
	METHTRACE(NodeShadow,Intend);
	if(!m_msite)
		ERR(E_PENDING);
	if(intent==MCCS_DELETED) {
		if(!m_inserted)
			return S_OK;
		return m_node->Delete(); // will call this->Delete(), which will call site->put_Intend(MCCS_DELETED)
	}
	if(!m_inserted) {
		RUN(adviseIfAppropriate());
		if(!(m_extent.cx||m_extent.cy))
			RUN(getExtent());
		RUN(m_msite->put_Intent(MCCS_HIDDEN)); // insert
		CComPtr<IDGMNodeControl> nc;
		if(SUCCEEDED(m_msite->GetControl(riidret(&nc)))) {
			RUN(CComQIPtr<DIDGShapeDefinition>(m_node)->SetOutside(m_extent.cx/1000.0,m_extent.cy/1000.0));
			RUN(nc->put_Node(m_node));
		}
		else {
			RUN(findShape());
			Trigger(); // send shape on to node
		}
		m_nextState = intent;
		RUN(m_node->Insert());
	}
	else
		RUN(m_msite->put_Intent(intent));
    return S_OK;
}
// IMCCInThreadTrigger
void STDMETHODCALLTYPE NodeShadow::Trigger() {
	METHTRACE(NodeShadow,Trigger);
	if(m_shape)
		RUN(m_node->put_Shape(m_shape));
}
// IMCCSiteEvents
STDMETHODIMP NodeShadow::Moving(LPCRECT winRect) {
	METHTRACE(NodeShadow,Moving);
	return S_FALSE;
}
STDMETHODIMP NodeShadow::Shifting(long,long next) {
	METHTRACE(NodeShadow,Shifting);
	// eliminate refcount loop so system can shut down.
	if(next==MCCS_DELETED) {
		if(m_node)
			RUN(AtlUnadvise(m_node,IID_IDGNodeShadow,m_dwNodeCookie));
	}
	return S_OK;
}
STDMETHODIMP NodeShadow::Layering(IMCCLayer *newLayer) {
    METHTRACE(NodeShadow,Layering);
	return S_FALSE;
}
STDMETHODIMP NodeShadow::ZChanged() {
    METHTRACE(NodeShadow,ZChanged);
	return S_FALSE;
}

// INeedOnePointer
STDMETHODIMP NodeShadow::TakePointer(IUnknown *o,IMoniker *m) {
	METHTRACE(NodeShadow,TakePointer);
	return put_LayoutObject(o); // ack, redundancy.  I think a general persistor can be devised....
}
// IObjectWithSite
STDMETHODIMP NodeShadow::SetSite(IUnknown *pUnkSite) {
	METHTRACE(NodeShadow,SetSite);
	m_msite = m_site = pUnkSite;
	if(!m_msite)
		ERR(E_INVALIDARG);
	RUN(APCAdvise(m_msite,(IMCCSiteEvents*)this,IID_IMCCSiteEvents,&m_dwSiteEventsCookie));
	RUN(APCAdvise(m_msite,(IMCCSiteOwner*)this,IID_IMCCSiteOwner,&m_dwSiteOwnerCookie));
	// only this one is vital
	return S_OK;
}
STDMETHODIMP NodeShadow::GetSite(REFIID riid, void **ret) {
	METHTRACE(NodeShadow,GetSite);
	if(!m_site)
		ERR(E_PENDING);
	return m_site->QueryInterface(riid,ret);
}
// IAdviseSink
void NodeShadow::OnDataChange(FORMATETC *fetc,STGMEDIUM *stgm) {
	METHTRACE(NodeShadow,OnDataChange);
	if(m_amSettingExtent)
		return;
	getShape(fetc,stgm);
	CComPtr<IMontageContainer> mc;
	RUN(m_msite->GetContainer(riidret(&mc)));
	RUN(mc->RequestTrigger(this));
}
void NodeShadow::OnViewChange(DWORD dwAspect,LONG lindex) {
	METHTRACE(NodeShadow,OnViewChange);
}

void NodeShadow::OnRename(IMoniker *pmk) {
	METHTRACE(NodeShadow,OnRename);
}

void NodeShadow::OnSave(void) {
	METHTRACE(NodeShadow,OnSave);
}
void NodeShadow::OnClose(void) {
	METHTRACE(NodeShadow,OnClose);
}
// IPersist
STDMETHODIMP NodeShadow::GetClassID(CLSID *ret) {
	METHTRACE(NodeShadow,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
