/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ShapeNode.h : Declaration of the ShapeNode

#ifndef __SHAPENODE_H_
#define __SHAPENODE_H_

#include "resource.h"       // main symbols
using namespace std;

// clockwise, counterclockwise
const bool c = true, cc = false;

/////////////////////////////////////////////////////////////////////////////
// ShapeNode
class ATL_NO_VTABLE ShapeNode :
	public CComObjectRoot,
	public CComCoClass<ShapeNode,&CLSID_ShapeNode>,
	public CComControl<ShapeNode>,
	public CStockPropImpl<ShapeNode, DIShapeNode, &IID_DIShapeNode, &LIBID_DGM>,
	public IDGMNodeControl,
	public IDGNodeShadow,
	public IPersistStreamInit, // no need to use Impl; we do it all ourselves.
	public IPersistStorageImpl<ShapeNode>,
	public IOleControlImpl<ShapeNode>,
	public IOleObjectImpl<ShapeNode>,
	public IOleInPlaceActiveObjectImpl<ShapeNode>,
	public IViewObjectExImpl<ShapeNode>,
	public IDataObjectImpl<ShapeNode>,
	public IOleInPlaceObjectWindowlessImpl<ShapeNode>,
	public ISpecifyPropertyPagesImpl<ShapeNode>,
	public ICoUIActiveObject,
	public IProvideClassInfo2Impl<&CLSID_ShapeNode,&IID_DIShapeNode,&LIBID_DGM>
{
	typedef CStockPropImpl<ShapeNode, DIShapeNode, &IID_DIShapeNode, &LIBID_DGM> stalk;
protected:
	CComPtr<IDGDrawnNode> m_node;
	DWORD m_carobWheatGermBar;
	DGSHAPESDESC *m_toDraw;

	void doDrawActions(LPCRECTL lpcrl,HDC hdc);
public:
	BOOL m_bBorderVisible;
	long m_nBorderStyle;
	long m_nBorderWidth;
	long m_nFillStyle; // as yet unused
#ifdef _DEBUG
	bool m_inserted;
#endif

	ShapeNode();
	~ShapeNode();

	REGGY_RID(IDR_ShapeNode)

	typedef IProvideClassInfo2Impl<&CLSID_ShapeNode,&IID_DIShapeNode,&LIBID_DGM> pci2Impl_t;
BEGIN_COM_MAP(ShapeNode)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(DIShapeNode)
	COM_INTERFACE_ENTRY(IDGMNodeControl)
	COM_INTERFACE_ENTRY(IDGNodeShadow)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY2(IPersist,IPersistStreamInit)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY_IID(IID_IProvideClassInfo2,pci2Impl_t)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
END_COM_MAP()

BEGIN_PROPERTY_MAP(ShapeNode)
	PROP_PAGE(CLSID_ShapeNodeProp)
	PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(ShapeNode)
	MESSAGE_HANDLER(WM_PAINT, OnPaint)
	MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
	MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
END_MSG_MAP()

// overridden IViewObjectExImpl
	STDMETHODIMP QueryHitPoint(DWORD dwAspect, LPCRECT pRectBounds, POINT ptlLoc, LONG /* lCloseHint */, DWORD *pHitResult);
	STDMETHODIMP QueryHitRect(DWORD dwAspect,LPCRECT pRectBounds,
		LPCRECT pRectLoc,LONG lCloseHint,DWORD *pHitResult);
	STDMETHODIMP GetViewStatus(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}
	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);
// IDGMNodeControl
	STDMETHODIMP get_Node(IDGDrawnNode **ret);
	STDMETHODIMP put_Node(IDGDrawnNode *val);
// IDGNodeShadow
    STDMETHODIMP Insert(void);
    STDMETHODIMP Delete(void);
    STDMETHODIMP Modify(int flags);
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU,OLEMENUGROUPWIDTHS*);
	STDMETHODIMP Deactivate(HMENU,OLEMENUGROUPWIDTHS*);
// IPersistStreamInitImpl
	STDMETHODIMP GetClassID(CLSID *ret);
	STDMETHODIMP InitNew();
	STDMETHODIMP IsDirty();
	STDMETHODIMP Load(IStream *stream);
	STDMETHODIMP Save(IStream *stream,BOOL clean);
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *ret);
};

#endif //__SHAPENODE_H_
