/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ShapeNodeProp.cpp : Implementation of ShapeNodeProp
#include "stdafx.h"
#include "ShapeNodeProp.h"
#include "large.h"

/////////////////////////////////////////////////////////////////////////////
// ShapeNodeProp

ShapeNodeProp::ShapeNodeProp()
{
	CONSTRUCTORTRACE(ShapeNodeProp);
	m_dwTitleID = IDS_TITLEShapeNodeProp;
	m_dwHelpFileID = IDS_HELPFILEShapeNodeProp;
	m_dwDocStringID = IDS_DOCSTRINGShapeNodeProp;
}
ShapeNodeProp::~ShapeNodeProp() {
	DESTRUCTORTRACE(ShapeNodeProp);
}

// combo-related stuff
LRESULT ShapeNodeProp::OnMeasureItem(UINT,WORD wParam,LPARAM lParam,BOOL&) {
	METHTRACE(ShapeNodeProp,OnMeasureItem);
	LPMEASUREITEMSTRUCT lpmis = LPMEASUREITEMSTRUCT(lParam);
	switch(wParam) {
	case IDC_BORDERSTYLE:
		lpmis->itemHeight = 16;
		lpmis->itemWidth = 60;
		break;
	}
	return 0;
}
LRESULT ShapeNodeProp::OnDrawItem(UINT,WORD wParam,LPARAM lParam,BOOL&) {
	METHTRACE(ShapeNodeProp,OnDrawItem);
	LPDRAWITEMSTRUCT lpdis = LPDRAWITEMSTRUCT(lParam);
	switch(wParam) {
	case IDC_BORDERSTYLE:
		{
			RECT tuh;
			tuh.left = -200;
			tuh.right = 200;
			tuh.top = -200;
			tuh.bottom = 200;
			HBRUSH hbr = (HBRUSH)GetStockObject(WHITE_BRUSH);
			FillRect(lpdis->hDC,&tuh,hbr);
			HPEN hp = CreatePen(lpdis->itemID,1,RGB(0,0,0)),
				hpOld = (HPEN)SelectObject(lpdis->hDC,hp);
			int y = (lpdis->rcItem.top+lpdis->rcItem.bottom)/2;
			MoveToEx(lpdis->hDC,lpdis->rcItem.left,y,NULL);
			LineTo(lpdis->hDC,lpdis->rcItem.right,y);
			SelectObject(lpdis->hDC,hpOld);
			DeleteObject(hp);
		}
		break;
	}
	return true;
}
LRESULT ShapeNodeProp::OnInitDialog(UINT,WORD,LPARAM,BOOL&b) {
	METHTRACE(ShapeNodeProp,OnInitDialogn);
	for(int i = 0;i<6;i++) {
		DWORD dwIndex = SendDlgItemMessage(IDC_BORDERSTYLE,CB_ADDSTRING,i);
		SendDlgItemMessage(IDC_BORDERSTYLE,CB_SETITEMDATA,dwIndex,i);
	}
	SendDlgItemMessage(IDC_BORDERSTYLE,CB_SETCURSEL,0);
	RUN(GetDlgControl(IDC_SHAPENODE,riidret(&m_demo)));
	RUN(m_shapeeng.CoCreateInstance(CLSID_DynagraphEngine));
	RUN(CComQIPtr<IDGStringAttributes>(m_shapeeng)->SetAttr(CComBSTR("engines"),CComBSTR("shapegen")));
	CComPtr<IDGNode> demonode;
	RUN(m_shapeeng->new_Node(CComBSTR("demo"),&demonode));
	if(m_tempstream) {
		RUN(CComQIPtr<IPersistStream>(demonode)->Load(m_tempstream));
		m_tempstream.Release();
	}
	RUN(CComQIPtr<IDGMNodeControl>(m_demo)->put_Node(CComQIPtr<IDGDrawnNode>(demonode)));
	RUN(demonode.QueryInterface(&m_demonode));
	return true;
}
LRESULT ShapeNodeProp::OnShowWindow(UINT,WORD,LPARAM,BOOL&b) {
	m_text.changed = m_sides.changed = m_peripheries.changed = m_spacing.changed =
		m_rotation.changed = m_skew.changed = m_distortion.changed = m_style.changed =
		m_width.changed = false;
	return 0;
}


///
// property related stuff
void ShapeNodeProp::SetControls() {
	METHTRACE(ShapeNodeProp,SetControls);
	if(m_hWnd) {
		if(m_ellipse.valid) {
			SendMessage(GetDlgItem(IDC_ELLIPSE),BM_SETCHECK,m_ellipse.value,0);
			SendMessage(GetDlgItem(IDC_POLYGON),BM_SETCHECK,!m_ellipse.value,0);
			disenableSides();
		}
		if(m_text.valid)
			SetDlgItemText(IDC_TEXT,m_text.value);
		if(m_sides.valid)
			SetDlgItemInt(IDC_SIDES,m_sides.value);
		if(m_peripheries.valid) {
			SetDlgItemInt(IDC_PERIPHERIES,m_peripheries.value);
			disenableSpacing();
		}
		if(m_spacing.valid)
			SetDlgItemDouble(IDC_SPACING,m_spacing.value);
		if(m_rotation.valid)
			SetDlgItemDouble(IDC_ROTATION,m_rotation.value);
		if(m_skew.valid)
			SetDlgItemDouble(IDC_SKEW,m_skew.value);
		if(m_distortion.valid)
			SetDlgItemDouble(IDC_DISTORTION,m_distortion.value);
		if(m_style.valid)
			SendDlgItemMessage(IDC_BORDERSTYLE,CB_SETCURSEL,m_style.value);
		if(m_width.valid)
			SetDlgItemInt(IDC_BORDERWIDTH,m_width.value);
	}
	SetDirty(false);
}
// hooray for templates!
template<typename T>
inline ShapeNodeProp::Maybe<T> SetupProp(ULONG nObjects, IUnknown **ppUnk,HRESULT (__stdcall DIDGShapeDefinition::*access)(T*)) {
	CComQIPtr<IDGMNodeControl> ctrl = ppUnk[0];
	_ASSERTE(ctrl);
	CComPtr<IDGDrawnNode> n;
	SRUN(ctrl->get_Node(&n));
	_ASSERTE(n);
	CComQIPtr<DIDGShapeDefinition> defn = n;
	_ASSERTE(defn);
	ShapeNodeProp::Maybe<T> ret;
	(defn->*access)(&ret.value);
	ret.valid = true;
	for(UINT i = 1; i < nObjects; i++) {
		T v;
		defn = ppUnk[i];
		(defn->*access)(&v);
		if(v!=ret.value) {
			ret.valid = false;
			break;
		}
	}
	return ret;
}
STDMETHODIMP ShapeNodeProp::SetObjects(ULONG nObjects, IUnknown **ppUnk) {
	METHTRACE(ShapeNodeProp,SetObjects);
	if(nObjects) {
		CComQIPtr<DIShapeNode> pCtrl(ppUnk[0]);
		/*
		CComBSTR bs(2048,LPCOLESTR(NULL)), bs2(2048,LPCOLESTR(NULL));
		pCtrl->get_Caption(&bs);
		m_text.valid = true;
		for(UINT i = 1; i < nObjects; i++) {
			pCtrl = ppUnk[i];
			pCtrl->get_Caption(&bs2);
			if(wcscmp(bs.m_str,bs2.m_str)) {
				m_text.valid = false;
				break;
			}
		}
		if(m_text.valid)
			AtlW2AHelper(m_text.value,bs.m_str,bs.Length()+1);
		*/
		m_ellipse = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_IsEllipse);
		m_sides = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Sides);
		m_peripheries = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Peripheries);
		m_spacing = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Spacing);
		m_spacing.value /= 1000.0f;
		m_rotation = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Rotation);
		m_skew = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Skew);
		m_distortion = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_Distortion);
		//m_style = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_BorderStyle);
		//m_width = SetupProp(nObjects,ppUnk,DIDGShapeDefinition::get_BorderWidth);
		// initialize example from first in list
		RUN(CreateStreamOnHGlobal(0,true,&m_tempstream));
		RUN(CComQIPtr<IPersistStreamInit>(ppUnk[0])->Save(m_tempstream,false));
		RUN(m_tempstream->Seek(castLarge(0),SEEK_SET,0));
	}
	return IPropertyPageImpl<ShapeNodeProp>::SetObjects(nObjects,ppUnk);
}
STDMETHODIMP ShapeNodeProp::Activate(HWND hWndParent, LPCRECT pRect, BOOL bModal) {
	METHTRACE(ShapeNodeProp,Activate);
	HRESULT hr = IPropertyPageImpl<ShapeNodeProp>::Activate(hWndParent,pRect,bModal);
	if(SUCCEEDED(hr))
		SetControls();
	return hr;
}

STDMETHODIMP ShapeNodeProp::Apply(void)
{
	METHTRACE(ShapeNodeProp,Apply);
	CComBSTR bs(strlen(m_text.value)+1,m_text.value);
	CComQIPtr<DIDGShapeDefinition> sh;
	for (UINT i = 0; i < m_nObjects; i++)
	{
		CComQIPtr<IDGMNodeControl> pCtrl = m_ppUnk[i];
		CComPtr<IDGDrawnNode> n;
		RUN(pCtrl->get_Node(&n));
		sh = n;
		// changed implies valid
		if(m_ellipse.changed)
			RUN(sh->put_IsEllipse(m_ellipse.value));
		if(m_sides.changed)
			RUN(sh->put_Sides(m_sides.value));
		if(m_peripheries.changed)
			RUN(sh->put_Peripheries(m_peripheries.value));
		if(m_spacing.changed)
			RUN(sh->put_Spacing(m_spacing.value));
		if(m_rotation.changed)
			RUN(sh->put_Rotation(m_rotation.value));
		if(m_skew.changed)
			RUN(sh->put_Skew(m_skew.value));
		if(m_distortion.changed)
			RUN(sh->put_Distortion(m_distortion.value));
		/*
		if(m_style.changed)
			RUN(sh->put_BorderStyle(m_style.value));
		if(m_width.changed)
			RUN(sh->put_BorderWidth(m_width.value));
		*/
	}
	// now fetch back values b/c ctrl may bound them
	// this assumes that every ctrl will respond the same to the same
	// parameter.  so far, so true.
	if(m_sides.valid)
		RUN(sh->get_Sides(&m_sides.value));
	if(m_peripheries.valid)
		RUN(sh->get_Peripheries(&m_peripheries.value));
	if(m_spacing.valid)
		RUN(sh->get_Spacing(&m_spacing.value));
	if(m_rotation.valid)
		RUN(sh->get_Rotation(&m_rotation.value));
	if(m_skew.valid)
		RUN(sh->get_Skew(&m_skew.value));
	if(m_distortion.valid)
		RUN(sh->get_Distortion(&m_distortion.value));
	/*
	if(m_style.valid)
		RUN(sh->get_BorderStyle(&m_style.value));
	if(m_width.valid)
		RUN(sh->get_BorderWidth(&m_width.value));
	*/
	SetControls();
	m_bDirty = FALSE;
	return S_OK;
}
