/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ShapeNodeProp.h : Declaration of the ShapeNodeProp

#ifndef __SHAPENODEPROP_H_
#define __SHAPENODEPROP_H_

#include <stdio.h>

/////////////////////////////////////////////////////////////////////////////
// ShapeNodeProp
class ATL_NO_VTABLE ShapeNodeProp :
	public CComObjectRoot,
	public CComCoClass<ShapeNodeProp, &CLSID_ShapeNodeProp>,
	public IPropertyPageImpl<ShapeNodeProp>,
	public CAxDialogImpl<ShapeNodeProp>
{
public:
	ShapeNodeProp();
	~ShapeNodeProp();

	enum {IDD = IDD_ShapeNodeProp};

REGGY_RID(IDR_ShapeNodeProp)

BEGIN_COM_MAP(ShapeNodeProp)
	COM_INTERFACE_ENTRY(IPropertyPage)
END_COM_MAP()

BEGIN_MSG_MAP(ShapeNodeProp)
	COMMAND_HANDLER(IDC_ELLIPSE,BN_CLICKED,OnEllipseChange)
	COMMAND_HANDLER(IDC_POLYGON,BN_CLICKED,OnEllipseChange)
	COMMAND_HANDLER(IDC_TEXT,EN_CHANGE,OnTextChange)
	COMMAND_HANDLER(IDC_SIDES,EN_CHANGE,OnSidesChange)
	COMMAND_HANDLER(IDC_PERIPHERIES,EN_CHANGE,OnPeripheriesChange)
	COMMAND_HANDLER(IDC_SPACING,EN_CHANGE,OnSpacingChange)
	COMMAND_HANDLER(IDC_ROTATION,EN_CHANGE,OnRotationChange)
	COMMAND_HANDLER(IDC_SKEW,EN_CHANGE,OnSkewChange)
	COMMAND_HANDLER(IDC_DISTORTION,EN_CHANGE,OnDistortionChange)
	COMMAND_HANDLER(IDC_BORDERSTYLE,CBN_SELCHANGE,OnStyleChange)
	COMMAND_HANDLER(IDC_BORDERWIDTH,EN_CHANGE,OnWidthChange)
	MESSAGE_HANDLER(WM_MEASUREITEM,OnMeasureItem)
	MESSAGE_HANDLER(WM_DRAWITEM,OnDrawItem)
	MESSAGE_HANDLER(WM_SHOWWINDOW,OnShowWindow)
	MESSAGE_HANDLER(WM_INITDIALOG,OnInitDialog)
	CHAIN_MSG_MAP(IPropertyPageImpl<ShapeNodeProp>)
END_MSG_MAP()

	void SetControls();
	STDMETHODIMP SetObjects(ULONG nObjects, IUnknown **ppUnk);
	STDMETHODIMP Activate(HWND hWndParent, LPCRECT pRect, BOOL bModal);
	STDMETHODIMP Apply(void);

// change callbacks from edits
	double GetDlgItemDouble(int nID) {
		char buf[40];
		GetDlgItemText(nID,buf,40);
		return atof(buf);
	}
	void SetDlgItemDouble(int nID,double val) {
		char buf[40];
		sprintf(buf,"%g",val);
		SetDlgItemText(nID,buf);
	}
	void disenableSides() {
		if(m_ellipse.value)
			::EnableWindow(GetDlgItem(IDC_SIDES),false);
		else
			::EnableWindow(GetDlgItem(IDC_SIDES),true);
	}
	LRESULT OnEllipseChange(WORD,WORD,HWND,BOOL&) {
		m_ellipse.changed = m_ellipse.valid = true;
		HWND ellRad = GetDlgItem(IDC_ELLIPSE);
		m_ellipse.value = SendMessage(ellRad,BM_GETCHECK,0,0)==BST_CHECKED;
		disenableSides();
		RUN(m_demonode->put_IsEllipse(m_ellipse.value?VARIANT_TRUE:VARIANT_FALSE));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnTextChange(WORD,WORD,HWND,BOOL&) {
		m_text.changed = m_text.valid = true;
		GetDlgItemText(IDC_TEXT,m_text.value,2048);
		CComBSTR bs(strlen(m_text.value)+1,m_text.value);
		//RUN(m_demonode->put_Caption(bs));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnSidesChange(WORD,WORD,HWND,BOOL&) {
		m_sides.changed = m_sides.valid = true;
		m_sides.value = GetDlgItemInt(IDC_SIDES);
		RUN(m_demonode->put_Sides(m_sides.value));
		SetDirty(TRUE);
		return 0;
	}
	void disenableSpacing() {
		if(m_peripheries.value>0)
			::EnableWindow(GetDlgItem(IDC_SPACING),true);
		else
			::EnableWindow(GetDlgItem(IDC_SPACING),false);
	}
	LRESULT OnPeripheriesChange(WORD,WORD,HWND,BOOL&) {
		m_peripheries.changed = m_peripheries.valid = true;
		m_peripheries.value = GetDlgItemInt(IDC_PERIPHERIES);
		disenableSpacing();
		RUN(m_demonode->put_Peripheries(m_peripheries.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnSpacingChange(WORD,WORD,HWND,BOOL&) {
		m_spacing.changed = m_spacing.valid = true;
		m_spacing.value = GetDlgItemDouble(IDC_SPACING);
		RUN(m_demonode->put_Spacing(m_spacing.value*1000.f));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnRotationChange(WORD,WORD,HWND,BOOL&) {
		m_rotation.changed = m_rotation.valid = true;
		m_rotation.value = GetDlgItemDouble(IDC_ROTATION);
		RUN(m_demonode->put_Rotation(m_rotation.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnSkewChange(WORD,WORD,HWND,BOOL&) {
		m_skew.changed = m_skew.valid = true;
		m_skew.value = GetDlgItemDouble(IDC_SKEW);
		RUN(m_demonode->put_Skew(m_skew.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnDistortionChange(WORD,WORD,HWND,BOOL&) {
		m_distortion.changed = m_distortion.valid = true;
		m_distortion.value = GetDlgItemDouble(IDC_DISTORTION);
		RUN(m_demonode->put_Distortion(m_distortion.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnStyleChange(WORD,WORD,HWND,BOOL&) {
		m_style.changed = m_style.valid = true;
		m_style.value = SendDlgItemMessage(IDC_BORDERSTYLE,CB_GETCURSEL);
		//RUN(m_demonode->put_BorderStyle(m_style.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnWidthChange(WORD,WORD,HWND,BOOL&) {
		m_width.changed = m_width.valid = true;
		m_width.value = GetDlgItemInt(IDC_BORDERWIDTH);
		if(m_width.value>1) {
			SendDlgItemMessage(IDC_BORDERSTYLE,CB_SETCURSEL,0);
			m_style.value = 0;
			::EnableWindow(GetDlgItem(IDC_BORDERSTYLE),false);
		}
		else
			::EnableWindow(GetDlgItem(IDC_BORDERSTYLE),true);
		//RUN(m_demonode->put_BorderWidth(m_width.value));
		SetDirty(TRUE);
		return 0;
	}
	LRESULT OnMeasureItem(UINT,WORD wParam,LPARAM lParam,BOOL&);
	LRESULT OnDrawItem(UINT,WORD wParam,LPARAM lParam,BOOL&);
	LRESULT OnInitDialog(UINT,WORD,LPARAM,BOOL&b);
	LRESULT OnShowWindow(UINT,WORD,LPARAM,BOOL&b);
	template<class T>
	struct Maybe {
		bool valid, changed;
		T value;
		Maybe() : valid(false), changed(false) {}
	};
	Maybe<char[2048]> m_text;
	Maybe<VARIANT_BOOL> m_ellipse;
	Maybe<int> m_sides,m_peripheries;
	Maybe<double> m_spacing,m_rotation,m_skew,m_distortion;
	Maybe<long> m_style,m_width;
	// embedded shapenode
	CComPtr<DIShapeNode> m_demo;
	CComPtr<IStream> m_tempstream; // and data to init it (from first of SetObjects)
	CComPtr<IDGLayout> m_shapeeng; // my own engine just for this shape
	CComPtr<DIDGShapeDefinition> m_demonode; // and the node
};

#endif //__SHAPENODEPROP_H_
