/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <common/Geometry.h>

class SimpleEdge :
	public CComObjectRoot,
	public CComCoClass<SimpleEdge,&CLSID_DGMSimpleEdge>,
	public CComControl<SimpleEdge>,
	public IDGMEdgeControl,
	public IMCCSelectionDraw,
	public IPersistStreamInitImpl<SimpleEdge>,
	public IOleObjectImpl<SimpleEdge>,
	public IViewObjectExImpl<SimpleEdge>,
	public IOleInPlaceObjectWindowlessImpl<SimpleEdge>,
	public ICoUIActiveObject,
	public IDGEdgeShadow
{
	CComPtr<IDGDrawnEdge> m_edge; // through IDGMEdgeControl
	CComPtr<IDGEdgeLabel> m_arrowNote;
	Coord m_arrowHead[4];

	DWORD m_coconut;

	bool m_amSpline,
		m_amSelected;
	int m_pulling; // control point being dragged
	POINT m_downpt; // where mouse down
	small unsigned m_transparency; // hack for spline bounds

	typedef vector<Coord> vpd_t;
	typedef vector<POINT> vp_t;
	vpd_t m_vp;

	DECLARE_GET_CONTROLLING_UNKNOWN()
public:
	SimpleEdge();
	~SimpleEdge();

	HRESULT fit();
	bool transformToBounds(vp_t &out);

BEGIN_COM_MAP(SimpleEdge)
	COM_INTERFACE_ENTRY(IDGEdgeShadow)
	COM_INTERFACE_ENTRY(IDGMEdgeControl)
	COM_INTERFACE_ENTRY(IMCCSelectionDraw)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IOleWindow)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
END_COM_MAP()

BEGIN_PROP_MAP(SimpleEdge)
END_PROP_MAP()

BEGIN_MSG_MAP(SimpleEdge)
	MESSAGE_HANDLER(WM_LBUTTONDOWN,ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE,MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP,ButtonUp)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

	REGGY_RID(IDR_SimpleEdge)
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU,OLEMENUGROUPWIDTHS*);
	STDMETHODIMP Deactivate(HMENU,OLEMENUGROUPWIDTHS*);
// IDGEdgeControl
	STDMETHODIMP Initialize(IDGDrawnEdge *edge);
// IDGEdgeShadow:
    STDMETHODIMP Insert();
    STDMETHODIMP Delete();
    STDMETHODIMP Modify(int flags);
// IMCCSelectionDraw
	STDMETHODIMP Show();
	STDMETHODIMP Hide();
// overridden IOleObject methods
	STDMETHODIMP SetClientSite(IOleClientSite *site);
	STDMETHODIMP DoVerb(LONG iVerb,LPMSG lpmsg,IOleClientSite *activeSite,LONG lindex,HWND hwndParent,LPCRECT pos);
// IViewObject
    STDMETHODIMP Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
         DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdcDraw,LPCRECTL lprcBounds,
         LPCRECTL lprcWBounds,BOOL(STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
         DWORD dwContinue);
    STDMETHODIMP QueryHitPoint(DWORD dwAspect,LPCRECT pRectBounds,POINT ptlLoc,
        LONG lCloseHint,DWORD *pHitResult);
	STDMETHODIMP QueryHitRect(DWORD dwAspect,LPCRECT pRectBounds,LPCRECT pRectLoc,
        LONG lCloseHint,DWORD *pHitResult);
};
