/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/

#include <stdafx.h>
#include "Spine.h"

Spine::Spine() : m_wasForward(false), m_respond(true) {}
Spine::~Spine() {}

// IDGMSpine
STDMETHODIMP Spine::Initialize(IMontageContainer *mc,IDGLayout *dynagraph,IDGIncrClient *incrClient,IMCCPlayControls *playControls) {
	RUN(mc->get_RefObject(&m_mref));
	m_dynagraph = dynagraph;
	m_incrClient = incrClient;
	m_play = playControls;
	RUN(AtlAdvise(playControls,GetUnknown(),IID_IMCCPlayControlsEvents,&m_yellowOreo));
	RUN(AtlAdvise(m_incrClient,GetUnknown(),IID_IDGIncrClientEvents,&m_oldFashionedMolasses));
	return S_OK;
}
STDMETHODIMP Spine::put_AutoRespond(VARIANT_BOOL val) {
    METHTRACE(Spine,put_AutoRespond);
	m_respond = val!=0;
	return S_OK;
}
// IMCCInThreadTrigger
void STDMETHODCALLTYPE Spine::Trigger() {
    METHTRACE(Spine,Trigger);
    HRESULT hr;
	RUN(hr = m_incrClient->ProcessResponse());
	if(m_wasForward) {
		RUN(m_play->Trigger(MCCPLAY_FORWARD));
		m_wasForward = false;
        if(hr==S_FALSE)
            RUN(m_play->put_FwdEnabled(VARIANT_FALSE));
	}
}
// IDGIncrClientEvents
STDMETHODIMP Spine::ReceivedResponse() {
    METHTRACE(Spine,ReceivedResponse);
	if(!m_respond)
		return S_FALSE;
	CComPtr<IUnknown> umc;
	RUN(m_mref->get_Montage(&umc));
	CComQIPtr<IMontageContainer> mc(umc);
    if(!mc) // hack: we've been discarded
        RUN(AtlUnadvise(m_incrClient,IID_IDGIncrClientEvents,m_yellowOreo));
    else
	    RUN(mc->RequestTrigger(this));
	return S_OK;
}
// IMCCPlayControlsEvents
STDMETHODIMP Spine::Before(enum MCCPLAY_BUTTONS button, BOOL *wait) {
    METHTRACE(Spine,Before);
	char buf[200];
	int len;
	CComBSTR viewName;
	RUN(CComQIPtr<IDGStringAttributes>(m_dynagraph)->get_Name(&viewName));
	switch(button) {
    case MCCPLAY_PAUSE:
		len = sprintf(buf,"lock view %S\n",viewName.m_str);
		*wait = false;
        break;
    case MCCPLAY_PLAY:
		len = sprintf(buf,"unlock view %S\n",viewName.m_str);
		*wait = false;
        break;
    case MCCPLAY_FORWARD:
		m_wasForward = true;
        *wait = true;
		if(!m_respond) {
			Trigger();
			return S_OK;
		}
		len = sprintf(buf,"unlock view %S\n"
			"lock view %S\n",viewName.m_str,viewName.m_str);
		break;
	}
	RUN(m_incrClient->Write(buf,len));
	return S_OK;
}
STDMETHODIMP Spine::After(enum MCCPLAY_BUTTONS button) {
    METHTRACE(Spine,After);
	return S_OK;
}
// INeedSomePointers
STDMETHODIMP Spine::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	CComPtr<IUnknown> punk;
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IMontageContainer> mc = punk;
	if(!mc)
		ERR(E_UNEXPECTED);
	punk.Release();
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IDGLayout> eng = punk;
	if(!eng)
		ERR(E_UNEXPECTED);
	punk.Release();
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IDGIncrClient> incr = punk;
	if(!incr)
		ERR(E_UNEXPECTED);
	punk.Release();
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IMCCPlayControls> play = punk;
	return Initialize(mc,eng,incr,play);
}
