/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


class Drawer :
	public CComCoClass<Drawer,&CLSID_DGMDrawer>,
	public CComObjectRoot,
	public IDGMDrawer,
	public IDispatchImpl<DIDGMDrawer>,
	public IMCCSiteLayoutEvents,
	public INeedSomePointers,
	public IPersistStream
{
    CComPtr<DIMontageContainerRef> m_mref; // semi-circular reference counting
	CComPtr<IDGLayout> m_engine;
	CComPtr<IDataObject> m_nodeSource,m_edgeSource,m_labelSource;
	CComPtr<IMCCDataObjectUnpacker> m_reader;
	CComPtr<IMoniker> m_mkLabelSource, m_mkReader;
    int m_labelLength;

    CComPtr<IMontageContainer> mc()  {
        METHTRACE(Drawer,mc);
        CComPtr<IUnknown> umc;
        RUN(m_mref->get_Montage(&umc));
        return CComQIPtr<IMontageContainer>(umc);
    }
	CComPtr<IItemSet> dataSet();
	HRESULT annotateNode(IBindCtx *bctx, IMCCSitePersistor *sp, IDGDrawnNode *node, LPOLESTR baseName);
	HRESULT addLabelMode(IBindCtx *bctx,IMCCSitePersistor *sp,IUnknown *layout,LPOLESTR baseName,IDGMLabelMode **ret);
public:
	Drawer();
	~Drawer();
	REGISTER_GENERIC(L"Drawer")
BEGIN_COM_MAP(Drawer)
	COM_INTERFACE_ENTRY(IDGMDrawer)
	COM_INTERFACE_ENTRY(DIDGMDrawer)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IMCCSiteLayoutEvents)
	COM_INTERFACE_ENTRY(INeedSomePointers)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IPersistStream)
END_COM_MAP()
// IDGMDrawer
	// the supreme automator must know all!
	STDMETHODIMP Initialize(IMontageContainer *mc,IDGLayout *engine,
		IDataObject *nodeSource,IDataObject *edgeSource,IDataObject *labelSource,
		IMCCDataObjectUnpacker *reader,IMoniker *mkLabelSource, IMoniker *mkReader);
	STDMETHODIMP get_Engine(IDGLayout **ret);
	STDMETHODIMP get_Container(IMontageContainer **ret);
	STDMETHODIMP get_NodeSource(IDataObject **ret);
    STDMETHODIMP get_LabelLength(int *ret);
    STDMETHODIMP put_LabelLength(int val);
	STDMETHODIMP NewNode(IDGDrawnNode *node, MCC_POS_INFO *mpi, LPOLESTR baseName, LPOLESTR caption, IMCCSitePersistor **ret);
	STDMETHODIMP NewEdge(IDGDrawnEdge *edge, LPOLESTR baseName, LPOLESTR caption, IMCCSitePersistor **ret);
// DIDGMDrawer
	STDMETHODIMP Initialize(DIMontageContainer *mc,DIDGLayout *engine,
		IUnknown *nodeSource,IUnknown *edgeSource,IUnknown *labelSource,IUnknown *reader,
		IUnknown *mkLabelSource,IUnknown *mkReader);
	STDMETHODIMP NewNode(DIDGDrawnNode *node, BSTR baseName, LPOLESTR caption, DIMCCPosInfo *pos,DIMCCSitePersistor **ret);
	STDMETHODIMP NewEdge(DIDGDrawnEdge *edge, BSTR baseName, LPOLESTR caption, DIMCCSitePersistor **ret);
// IMCCSiteLayoutEvents
	STDMETHODIMP Process(IMCCSitePersistor *pers, LPOLESTR baseName);
// INeedSomePointers
	STDMETHODIMP TakePointers(IEnumUnknown *os,IEnumMoniker *ms);
// IPersistStream
    STDMETHODIMP GetClassID(CLSID *ret) { return *ret = GetObjectCLSID(), S_OK; }
    STDMETHODIMP IsDirty() { return S_OK; }
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
    STDMETHODIMP GetSizeMax(ULARGE_INTEGER*) { ERR(E_NOTIMPL); }
};
