/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"

// CreateGroupMoniker defaults to creating item monikers but can create debug mks too.

struct DebugItemMoniker : IMoniker {
	IMoniker *realmk;
	DWORD serverID;
	LPOLESTR name;
    DebugItemMoniker(DWORD id,IMoniker *realmk) : realmk(realmk), serverID(id) {
        METHTRACE(DebugItemMoniker,DebugItemMoniker);
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		RUN(realmk->GetDisplayName(bctx,0,&name));
		ATLTRACE2("mk(%d%S) created\n",serverID,name);
	}
	~DebugItemMoniker() {
		ATLTRACE2("mk(%d%S) destroyed\n",serverID,name);
		CoTaskMemFree(name);
	}
// IUnknown
	STDMETHODIMP_(ULONG) AddRef() {
		ULONG u = realmk->AddRef();
		ATLTRACE2("mk(%d%S)::AddRef to %d\n",serverID,name,u);
		return u;
	}
	STDMETHODIMP_(ULONG) Release() {
		ULONG u = realmk->Release();
		ATLTRACE2("mk(%d%S)::Release to %d\n",serverID,name,u);
		if(!u)
			delete this;
		return u;
	}
	STDMETHODIMP QueryInterface(REFIID riid,void **ppRet) {
		HRESULT hr = realmk->QueryInterface(riid,ppRet);
		ATLTRACE2("mk(%d%S)::QueryInterface\n",serverID,name);
		return hr;
	}
// IPersistStream
	STDMETHODIMP GetClassID(CLSID *pClassID) { return realmk->GetClassID(pClassID); }
	STDMETHODIMP IsDirty(void) { return realmk->IsDirty(); }
	STDMETHODIMP Load(IStream *pStm) { return realmk->Load(pStm); }
	STDMETHODIMP Save(IStream *pStm, BOOL fClearDirty) { return realmk->Save(pStm,fClearDirty); }
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize) { return realmk->GetSizeMax(pcbSize); }

// .. IMoniker
    STDMETHODIMP BindToObject(IBindCtx *pbc, IMoniker *pmkToLeft, REFIID riidResult, void **ppvResult) { return realmk->BindToObject(pbc,pmkToLeft,riidResult,ppvResult); }
    STDMETHODIMP BindToStorage(IBindCtx *pbc, IMoniker *pmkToLeft, REFIID riid, void **ppvObj) { return realmk->BindToStorage(pbc,pmkToLeft,riid,ppvObj); }
    STDMETHODIMP Reduce(IBindCtx *pbc, DWORD dwReduceHowFar, IMoniker **ppmkToLeft, IMoniker **ppmkReduced) { return realmk->Reduce(pbc,dwReduceHowFar,ppmkToLeft,ppmkReduced); }
    STDMETHODIMP ComposeWith(IMoniker *pmkRight, BOOL fOnlyIfNotGeneric, IMoniker **ppmkComposite) { return realmk->ComposeWith(pmkRight, fOnlyIfNotGeneric, ppmkComposite); }
	STDMETHODIMP Enum(BOOL fForward, IEnumMoniker **ppenumMoniker) { return realmk->Enum(fForward, ppenumMoniker); }
    STDMETHODIMP IsEqual(IMoniker *pmkOtherMoniker) { return realmk->IsEqual(pmkOtherMoniker); }
    STDMETHODIMP Hash(DWORD *pdwHash) { return realmk->Hash(pdwHash); }
    STDMETHODIMP IsRunning(IBindCtx *pbc, IMoniker *pmkToLeft, IMoniker *pmkNewlyRunning) { return realmk->IsRunning(pbc, pmkToLeft, pmkNewlyRunning); }
    STDMETHODIMP GetTimeOfLastChange(IBindCtx *pbc, IMoniker *pmkToLeft, FILETIME *pFileTime) { return realmk->GetTimeOfLastChange(pbc, pmkToLeft, pFileTime); }
    STDMETHODIMP Inverse(IMoniker **ppmk) { return realmk->Inverse(ppmk); }
    STDMETHODIMP CommonPrefixWith(IMoniker *pmkOther, IMoniker **ppmkPrefix) { return realmk->CommonPrefixWith(pmkOther, ppmkPrefix); }
    STDMETHODIMP RelativePathTo(IMoniker *pmkOther, IMoniker **ppmkRelPath) { return realmk->RelativePathTo(pmkOther, ppmkRelPath); }
    STDMETHODIMP GetDisplayName(IBindCtx *pbc, IMoniker *pmkToLeft, LPOLESTR *ppszDisplayName) { return realmk->GetDisplayName(pbc, pmkToLeft, ppszDisplayName); }
    STDMETHODIMP ParseDisplayName(IBindCtx *pbc, IMoniker *pmkToLeft, LPOLESTR pszDisplayName, ULONG *pchEaten, IMoniker **ppmkOut) { return realmk->ParseDisplayName(pbc, pmkToLeft, pszDisplayName, pchEaten, ppmkOut); }
    STDMETHODIMP IsSystemMoniker(DWORD *pdwMksys) { return realmk->IsSystemMoniker(pdwMksys); }
};

//#define DEBUG_GROUP_MK
HRESULT CreateGroupMoniker(DWORD id,LPCOLESTR name,IMoniker **ret) {
    FUNTRACE(CreateGroupMoniker);
	IMoniker *mk;
	SRUN(CreateItemMoniker(L"!",name,&mk));
#ifdef DEBUG_GROUP_MK
	*ret = new DebugItemMoniker(id,mk);
#else
	*ret = mk;
#endif
	return S_OK;
}