/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CreationPersistor.h"
#include "cacheStreamableObject.h"

CreationPersistor::CreationPersistor() {
	CONSTRUCTORTRACE(CreationPersistor);
}
CreationPersistor::~CreationPersistor() {
	DESTRUCTORTRACE(CreationPersistor);
}


// DIPersistor
STDMETHODIMP CreationPersistor::SetSubject(VARIANT_BOOL realized,IUnknown *val) {
    METHTRACE(CreationPersistor,SetSubject);
	if(!realized)
		ERR(E_INVALIDARG); // you don't understand!
	m_subject = val;
	return S_OK;
}
STDMETHODIMP CreationPersistor::get_Subject(IUnknown **ret) {
    METHTRACE(CreationPersistor,get_Subject);
	m_subject.CopyTo(ret);
	return S_OK;
}
// DIServerCreationPersistor
STDMETHODIMP CreationPersistor::put_ServerMoniker(IUnknown *val) {
    METHTRACE(CreationPersistor,put_ServerMoniker);
	return val->QueryInterface(riidret(&m_mkServer));
}
// IGroupItem
STDMETHODIMP CreationPersistor::Connect(IGroup *group) {
    METHTRACE(CreationPersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	return S_OK;
}
STDMETHODIMP CreationPersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
    METHTRACE(CreationPersistor,Copy);
	CComPtr<IMoniker> mkServer2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkServer,IID_NULL,0,&mkServer2));
	CComPtr<DIServerCreationPersistor> scp2;
	RUN(CREATE(scp2,GetObjectCLSID())); TRACECREATEV(GetObjectCLSID());
	RUN(CComQIPtr<IGroupItem>(scp2)->Connect(dest));
	RUN(scp2->put_ServerMoniker(mkServer2));
	if(m_subject)
		RUN(CComQIPtr<DIPersistor>(scp2)->SetSubject(true,m_subject));
	return scp2->QueryInterface(riid,ret);
}
STDMETHODIMP CreationPersistor::Realize(IBindCtx *bctx) {
    METHTRACE(CreationPersistor,Realize);
	if(m_subject)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	CComPtr<IMoniker> mkGroup;
	RUN(FoolishCreatePointerMoniker(m_group,&mkGroup));
	CComPtr<IClassFactory> casteOmatic;
	RUN(m_mkServer->BindToObject(bctx,mkGroup,riidret(&casteOmatic)));
	RUN(casteOmatic->CreateInstance(0,riidret(&m_subject)));
	if(m_cache) {
		// could support IPersistStreamInit but I beginning to think that's a weird interface.
		// (mostly needed for re-initializing? Don't re-initialize MY objects thank you!)
		RUN(CComQIPtr<IPersistStream>(m_subject)->Load(m_cache));
		m_cache.Release();
	}
	return S_OK;
}
STDMETHODIMP CreationPersistor::Remove() {
    METHTRACE(CreationPersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkServer));
	return S_OK;
}
// IPersistStream
STDMETHODIMP CreationPersistor::GetClassID(CLSID *pClassID) {
    METHTRACE(CreationPersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP CreationPersistor::IsDirty(void) {
    METHTRACE(CreationPersistor,IsDirty);
	return S_OK;
}
STDMETHODIMP CreationPersistor::Load(IStream *pStm) {
    METHTRACE(CreationPersistor,Load);
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkServer)));
	RUN(cacheStreamableObject(pStm,&m_cache));
	return S_OK;
}
STDMETHODIMP CreationPersistor::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(CreationPersistor,Save);
	RUN(FixedOleSaveToStream(m_mkServer,pStm));
	if(!m_subject && !m_cache)
		ERR(E_PENDING);
	if(m_subject)
		RUN(saveWithSize(pStm,CComQIPtr<IPersistStream>(m_subject),fClearDirty));
	else
		RUN(writeCache(pStm,m_cache));
	return S_OK;
}
STDMETHODIMP CreationPersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(CreationPersistor,GetSizeMax);
	ERR(E_NOTIMPL);
}