/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "GetSubjectMoniker.h"

GetSubjectMoniker::GetSubjectMoniker() {
	CONSTRUCTORTRACE(GetSubjectMoniker);
}
GetSubjectMoniker::~GetSubjectMoniker() {
	DESTRUCTORTRACE(GetSubjectMoniker);
}


// IPersistStream
STDMETHODIMP GetSubjectMoniker::GetClassID(CLSID *pClassID) {
	METHTRACE(GetSubjectMoniker,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::IsDirty(void) {
	METHTRACE(GetSubjectMoniker,IsDirty);
	return S_FALSE;
}
STDMETHODIMP GetSubjectMoniker::Load(IStream *pStm) {
	METHTRACE(GetSubjectMoniker,Load);
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::Save(IStream *pStm, BOOL fClearDirty) {
	METHTRACE(GetSubjectMoniker,Save);
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(GetSubjectMoniker,GetSizeMax);
	pcbSize->QuadPart = 0;
	return S_OK;
}

// IMoniker
STDMETHODIMP GetSubjectMoniker::BindToObject(IBindCtx *pbc, IMoniker *pmkToLeft, REFIID riidResult, void **ppvResult) {
	METHTRACE(GetSubjectMoniker,BindToObject);
	HRESULT hr;
	CComPtr<DIPersistor> per;
	hr = pmkToLeft->BindToObject(pbc,0,riidret(&per));
	if(hr==E_NOINTERFACE) // distinguish between this failure and the right o not supporting riidResult
		ERR(MK_E_INTERMEDIATEINTERFACENOTSUPPORTED);
	if(FAILED(hr))
		return hr;
	if(CComQIPtr<IGroupItem> gi = per) // make sure subject is fully initialized
		RUN(gi->Realize(pbc));
	CComPtr<IUnknown> u;
	RUN(per->get_Subject(&u));
	return LOOK(u->QueryInterface(riidResult,ppvResult));
}
STDMETHODIMP GetSubjectMoniker::BindToStorage(IBindCtx *pbc, IMoniker *pmkToLeft, REFIID riid, void **ppvObj) {
	METHTRACE(GetSubjectMoniker,BindToStorage);
	ERR(MK_E_NOSTORAGE); // well, maybe it does, but I don't know about it!
}
STDMETHODIMP GetSubjectMoniker::Reduce(IBindCtx *pbc, DWORD dwReduceHowFar, IMoniker **ppmkToLeft, IMoniker **ppmkReduced) {
	METHTRACE(GetSubjectMoniker,Reduce);
	(*ppmkReduced = this)->AddRef();
	return MK_S_REDUCED_TO_SELF;
}
STDMETHODIMP GetSubjectMoniker::ComposeWith(IMoniker *pmkRight, BOOL fOnlyIfNotGeneric, IMoniker **ppmkComposite) {
	METHTRACE(GetSubjectMoniker,ComposeWith);
	DWORD type;
	if(pmkRight->IsSystemMoniker(&type)==S_OK && type==MKSYS_ANTIMONIKER) {
		*ppmkComposite = 0;
		return S_OK;
	}
	if(fOnlyIfNotGeneric)
		ERR(MK_E_NEEDGENERIC);
	else return CreateGenericComposite(this,pmkRight,ppmkComposite);
}
STDMETHODIMP GetSubjectMoniker::Enum(BOOL fForward, IEnumMoniker **ppenumMoniker) {
	METHTRACE(GetSubjectMoniker,Enum);
	*ppenumMoniker = 0;
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::IsEqual(IMoniker *pmkOtherMoniker) {
	METHTRACE(GetSubjectMoniker,IsEqual);
	CLSID clsid;
	RUN(pmkOtherMoniker->GetClassID(&clsid));
	return clsid==GetObjectCLSID()?S_OK:S_FALSE;
}
STDMETHODIMP GetSubjectMoniker::Hash(DWORD *pdwHash) {
	METHTRACE(GetSubjectMoniker,Hash);
	// mm?
	*pdwHash = *(DWORD*)&GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::IsRunning(IBindCtx *pbc, IMoniker *pmkToLeft, IMoniker *pmkNewlyRunning) {
	METHTRACE(GetSubjectMoniker,IsRunning);
	return pmkToLeft->IsRunning(pbc,pmkToLeft,pmkNewlyRunning);
}
STDMETHODIMP GetSubjectMoniker::GetTimeOfLastChange(IBindCtx *pbc, IMoniker *pmkToLeft, FILETIME *pFileTime) {
	METHTRACE(GetSubjectMoniker,GetTimeOfLastChange);
	return pmkToLeft->GetTimeOfLastChange(pbc,pmkToLeft,pFileTime);
}
STDMETHODIMP GetSubjectMoniker::Inverse(IMoniker **ppmk) {
	METHTRACE(GetSubjectMoniker,Inverse);
	// this isn't really an inverse: you can't bind it to get the object from the site.
	// but I guess that is how inverse monikers are supposed to work: only compose them
	// never bind them.
	return CreateAntiMoniker(ppmk);
}
STDMETHODIMP GetSubjectMoniker::CommonPrefixWith(IMoniker *pmkOther, IMoniker **ppmkPrefix) {
	METHTRACE(GetSubjectMoniker,CommonPrefixWith);
	ERR(MK_E_NOTBINDABLE);
}
STDMETHODIMP GetSubjectMoniker::RelativePathTo(IMoniker *pmkOther, IMoniker **ppmkRelPath) {
	METHTRACE(GetSubjectMoniker,RelativePathTo);
	ERR(MK_E_NOTBINDABLE );
}
STDMETHODIMP GetSubjectMoniker::GetDisplayName(IBindCtx *pbc, IMoniker *pmkToLeft, LPOLESTR *ppszDisplayName) {
	METHTRACE(GetSubjectMoniker,GetDisplayName);
	*ppszDisplayName = (LPOLESTR)CoTaskMemAlloc(16);
	memcpy(*ppszDisplayName,L"!{site}",16);
	return S_OK;
}
STDMETHODIMP GetSubjectMoniker::ParseDisplayName(IBindCtx *pbc, IMoniker *pmkToLeft, LPOLESTR pszDisplayName, ULONG *pchEaten, IMoniker **ppmkOut) {
	METHTRACE(GetSubjectMoniker,ParseDisplayName);
	ERR(E_NOTIMPL); // I give up
}
STDMETHODIMP GetSubjectMoniker::IsSystemMoniker(DWORD *pdwMksys) {
	METHTRACE(GetSubjectMoniker,IsSystemMoniker);
	return S_FALSE;
}
