/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Group.h"
#include "IdentityServer.h"

intID intify(IDENTITY p) {
	return (((intID)p.server)<<32)|p.item;
}
IdentityServer::IdentityServer() : m_currID(17) {
	CONSTRUCTORTRACE(IdentityServer);
}
IdentityServer::~IdentityServer() {
	DESTRUCTORTRACE(IdentityServer);
}

STDMETHODIMP IdentityServer::Equals(IDENTITY p, IDENTITY q) {
	METHTRACE(IdentityServer,Equals);
	intID pint = intify(p), qint = intify(q);
	m64mSI::iterator pi = m_map.find(pint);
	m64mSI::iterator qi = m_map.find(qint);
	int cas = (pi==m_map.end()?0:2)|(qi==m_map.end()?0:1);
	switch(cas) {
	case 0: {
		mSI *pm = new mSI,
			&m = *pm;
		m[p.server] = p.item;
		m[q.server] = q.item;
		m_map[pint] = pm;
		m_map[qint] = pm;
		return S_OK;
	}
	case 1: { // q, not p
		mSI &m = *qi->second;
		mSI::iterator j = m.find(p.server);
		if(j!=m.end())
			return j->second==p.item?S_FALSE:E_FAIL;
		m_map[pint] = qi->second;
		m[p.server] = p.item;
		return S_OK;
	}
	case 2: { // p, not q
		mSI &m = *pi->second;
		mSI::iterator j = m.find(q.server);
		if(j!=m.end())
			return j->second==q.item?S_FALSE:E_FAIL;
		m_map[qint] = pi->second;
		m[q.server] = q.item;
		return S_OK;
	}
	default: // (for compiler)
	case 3: { // both
		if(pi->second==qi->second)
			return S_FALSE; // already joined
		mSI &mp = *pi->second,
			&mq = *qi->second;
		// must merge the two mSIs; first verify that this is legal.
		// they must not share servers, as that would mean two IDs
		// sharing the same object (perhaps a useful feature, but not
		// one I want to deal with right now).
		for(mSI::iterator j = mp.begin();j!=mp.end();j++)
			if(mq.find(j->first)!=mq.end())
				ERR(E_FAIL);
		// merge mq into mp, mapping their IDs to &mp
		for(j = mq.begin();j !=mq.end();j++) {
			mp[j->first] = j->second;
			IDENTITY id = {j->first,j->second};
			m_map[intify(id)] = pi->second;
		}
		return S_OK;
	}
	}
}
STDMETHODIMP IdentityServer::Find(IDENTITY p, DWORD qServer, DWORD *retQItem) {
	METHTRACE(IdentityServer,Find);
    if(p.server==qServer) {
        *retQItem = p.item;
        return S_FALSE;
    }
	m64mSI::iterator i = m_map.find(intify(p));
	if(i==m_map.end())
		ERR(E_FAIL);
	mSI::iterator j = i->second->find(qServer);
	if(j==i->second->end())
		ERR(E_FAIL);
	*retQItem = j->second;
	return S_OK;
}
STDMETHODIMP IdentityServer::Remove(IDENTITY p) {
	METHTRACE(IdentityServer,Remove);
	m64mSI::iterator i = m_map.find(intify(p));
	if(i== m_map.end())
		return S_FALSE;
	mSI &m = *i->second;
	m.erase(p.server);
	m_map.erase(i);
	if(m.size()==1) { // not useful, erase
		m_map.erase(intify(*(IDENTITY*)&*m.begin()));
		delete &m;
	}
	return S_OK;
}
