/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


/////////////////////////////////////////////////////////////////////////////
// Group-IOleItemContainer
//
#include "stdafx.h"
#include "GroupClass.h"
#include "notFound.h"

HRESULT Group::realize(IBindCtx *bctx, itemMap::iterator i) {
	LPOLESTR name = i->first;
	METHTRACE(Group,realize);
	Item &item = i->second;
	ULONGLONG istreamBak, zero = 0;
	// find CLSID (could have changed since save)
	CLSID clsid;
	CComPtr<IStream> loadstm;
	if(item.method==persistStorage) {
		if(!item.storage)
			RUN(open(m_storage,i->first,&item.storage));
		RUN(ReadClassStg(item.storage,&clsid));
	}
	else {
		if(!item.element) {
			RUN(m_itemStream->Seek(castLarge(zero),STREAM_SEEK_CUR,&castULarge(istreamBak)));
			RUN(m_itemStream->Seek(castLarge(item.pos),STREAM_SEEK_SET,0));
			loadstm = m_itemStream;
		}
		else {
			if(!item.stream)
				RUN(open(m_storage,i->first,&item.stream));
			loadstm = item.stream;
		}
		RUN(ReadClassStm(loadstm,&clsid));
	}
	_ASSERTE(clsid==item.clsid);
	// create object
	IID iidP;
	switch(item.method) {
	case persist:
		iidP = IID_IPersist;
		break;
	case persistStream:
		iidP = IID_IPersistStream;
		break;
	case persistStreamInit:
		iidP = IID_IPersistStreamInit;
		break;
	case persistStorage:
		iidP = IID_IPersistStorage;
		break;
	default:
		ERR(E_UNEXPECTED);
	}
	TRACECREATEV(clsid);
	// hack: prefer inproc handler because our container might have a half-object registered with CoRegisterClassObject
//	if(FAILED(LOOK(CoCreateInstance(clsid,0,CLSCTX_INPROC_HANDLER,iidP,(void**)&item.p)))

	HRESULT hr;
	CComPtr<IUnknown> u;
	if(FAILED(LOOK(hr = CoCreateInstance(clsid,0,CLSCTX_ALL,riidret(&u))))) {
		OLECHAR buf[50];
		StringFromGUID2(clsid,buf,50);
        ATLTRACE2("Group::realize %p err %x couldn't create class %S\n",this,hr,buf);
		return hr;
	}
	if(item.method==persistStream) { // a catch-all
		RUN(findPersistType(u,item));
		if(item.method==persistStorage)
			ERR(OLE_E_CANTCONVERT);  // you know, or something...
	}
	else
		RUN(u->QueryInterface(iidP,(void**)&item.p));
	// connect.  not clear whether this should be before or after load.  An object
	// shouldn't really do anything with the group until Realize, but sometimes it
	// must cheat.
	if(CComQIPtr<IGroupItem> pg = item.p)
		RUN(pg->Connect(this));
	// load the object (not necessary for IPersist objects)
	switch(item.method) {
	case persistStream:
	case persistStreamInit:
		RUNRET(item.pStream->Load(loadstm));
		if(!item.element) {
			ULONGLONG end;
			RUN(m_itemStream->Seek(castLarge(zero),STREAM_SEEK_CUR,&castULarge(end)));
			_ASSERTE(end-item.pos == item.size);
		}
	case persist:
		if(!item.element)
			RUN(m_itemStream->Seek(castLarge(istreamBak),STREAM_SEEK_SET,0));
		break;
	case persistStorage:
		RUNRET(item.pStorage->Load(item.storage));
		item.isMainStorage = true;
	}
	item.pending = false;
	return S_OK;
}
// IOleItemContainer
// -IParseDisplayName
STDMETHODIMP Group::ParseDisplayName(IBindCtx *, LPOLESTR name, ULONG *retEat, IMoniker **ret) {
	METHTRACE(Group,ParseDisplayName);
	int len;
	for(len=0;name[len] && name[len]!=L'!';len++);
	if(name[len]) {
		LPOLESTR buf = new OLECHAR[len];
		wcsncpy(buf,name,len);
		RUN(CreateGroupMoniker(m_serverID,buf,ret));
		delete [] buf;
	}
	else
		RUN(CreateGroupMoniker(m_serverID,name,ret));
	return S_OK;
}
// -IOleContainer
typedef CComObject<CComEnum<IEnumUnknown,&IID_IEnumUnknown,IUnknown*,_CopyInterface<IUnknown> > > eunk;
STDMETHODIMP Group::EnumObjects(DWORD flags,IEnumUnknown **ret) {
	METHTRACE(Group,EnumObjects);
	eunk *r = new eunk;
	int n = m_data.size();
	IUnknown **ray = new IUnknown *[n],**ri = ray;
	for(itemMap::iterator i = m_data.begin();i!=m_data.end();i++)
		(*ri++ = i->second.p)->AddRef();
	r->Init(ray,ray+n,0,AtlFlagTakeOwnership);
	(*ret = r)->AddRef();
	return S_OK;
}
STDMETHODIMP Group::LockContainer(BOOL bLock) {
	METHTRACE(Group,LockContainer);
	//?
	return S_OK;
}
// .
STDMETHODIMP Group::GetObject(LPOLESTR	name,DWORD speed,IBindCtx *bctx,REFIID riid,void **ret) {
	METHTRACE(Group,GetObject);
	if(!wcsncmp(name,L"..",2)) {
		if(m_parent)
			return m_parent->QueryInterface(riid,ret);
		else {
			*ret = 0;
			return S_OK;
		}
	}
	itemMap::iterator i = m_data.find(name);
	if(i==m_data.end()) {
		GNOTFOUND(IOleItemContainer,GetObject,name);
		ERR(MK_E_NOOBJECT);
	}
	Item &item = i->second;
	if(!item.p) {
		if(item.method==persistPasting)
			ERR(E_PENDING);
		if(speed!=BINDSPEED_INDEFINITE)
			ERR(MK_E_EXCEEDEDDEADLINE);
		RUNRET(realize(bctx,i));
	}
	HRESULT hr = item.p->QueryInterface(riid,ret);
	if(FAILED(hr))
		MTrace(ER)("Group::GetObject on %S: QI failed\n",name);
	return hr;
}
STDMETHODIMP Group::GetObjectStorage(LPOLESTR name, IBindCtx *bctx,REFIID riid,void **ret) {
	METHTRACE(Group,GetObjectStorage);
	if(!wcsncmp(name,L"..",2)) // can't bind to parent's storage!
		ERR(E_FAIL);
	itemMap::iterator i = m_data.find(name);
	if(i==m_data.end())
		ERR(MK_E_NOOBJECT);
	// the documentation is ambiguous as to whether this method is allowed to return
	// streams.  IOleContainer::GetObjectStorage seems to imply no, but
	// IMoniker::BindToStorage (which relies on this) sez yes.  Also, no documentation
	// on whether the stream pointer should point to the data or to the CLSID which
	// precedes it.  I choose the latter because that's awfully useful info.
	Item &item = i->second;
	switch(item.method) {
	case persistStorage: {
		if(!item.storage)
			RUN(open(m_storage,name,&item.storage));
		return item.storage->QueryInterface(riid,ret);
	}
	case persist: // IPersist objects are treated like 0-length IPersistStream objects in this implementation.
			// Again, little or no documentation on IPersist itself.
	case persistStreamInit:
	case persistStream: {
		CComPtr<IStream> s;
		if(!item.element) {
			if(item.pos==-1)
				ERR(E_FAIL);
			RUN(m_itemStream->Clone(&s));
			// weird xp doesn't like STREAM_SEEK_SET??
			if(STG_E_INVALIDFUNCTION==s->Seek(castLarge(item.pos),STREAM_SEEK_SET,0)) {
				_ASSERTE(-1 > 0); // i think it's actually the -1 problem?
				ULONGLONG p;
				RUN(s->Seek(castLarge(0),STREAM_SEEK_CUR,&castULarge(p)));
				RUN(s->Seek(castLarge(-LONGLONG(p)),STREAM_SEEK_CUR,0));
			}
		}
		else {
			if(!item.stream)
				RUN(open(m_storage,i->first,&item.stream));
			s = item.stream;
		}
		return s->QueryInterface(riid,ret);
	}
	case persistPasting:
		ERR(E_PENDING);
	default:
		ERR(E_UNEXPECTED);
	}
}
STDMETHODIMP Group::IsRunning(LPOLESTR name) {
	METHTRACE(Group,IsRunning);
	itemMap::iterator i = m_data.find(name);
	if(i==m_data.end())
		ERR(MK_E_NOOBJECT);
	if(!i->second.p)
		return S_FALSE;
	if(CComQIPtr<IRunnableObject> r = i->second.p)
		if(r->IsRunning())
			return S_OK;
		else
			return S_FALSE;
	return S_OK;
}
