/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "OrderedItemList.h"
#include "iterEnum.h"
#include "mapValueIterator.h"
#include "Collection.h"

OrderedItemList::OrderedItemList() : m_realized(false) {
	CONSTRUCTORTRACE(OrderedItemList);
}
OrderedItemList::~OrderedItemList() {
	DESTRUCTORTRACE(OrderedItemList);
	for(ordmon::iterator i = m_ordmon.begin(); i!=m_ordmon.end(); ++i)
		i->second->Release();
}

HRESULT OrderedItemList::setMoniker(long order, IMoniker *val, bool addStgRef) {
	METHTRACE(OrderedItemList,setMoniker);
	if(!m_group)
		ERR(E_PENDING);
	// it frustrates me that I can never use those []
	ordmon::iterator i = m_ordmon.find(order);
	if(i==m_ordmon.end())
		 i = m_ordmon.insert(make_pair_first_const(order,(IMoniker*)0)).first;
	ordmon::value_type &m = *i;
	if(m.second) {
		if(addStgRef)
			RUN(m_group->ReleaseStgRef(m.second));
		m.second->Release();
	}
	if(m.second = val) {
		m.second->AddRef();
		if(addStgRef)
			RUN(m_group->AddStgRef(val));
	}
	else
		m_ordmon.erase(i);
	return S_OK;
}
// IOrderedItemList
STDMETHODIMP OrderedItemList::GetMoniker(long order, IMoniker **ret) {
	METHTRACE(OrderedItemList,GetMoniker);
	ordmon::iterator i = m_ordmon.find(order);
	if(i==m_ordmon.end())
		return S_FALSE;
	(*ret = i->second)->AddRef();
	return S_OK;
}
STDMETHODIMP OrderedItemList::SetMoniker(long order, IMoniker *val) {
	METHTRACE(OrderedItemList,SetMoniker);
	return setMoniker(order,val,true);
}
// wow, type hell.  three-level iterator.
typedef itSecond<ordmon::iterator,IMoniker*> ItValuesFwd;
typedef itSecond<ordmon::reverse_iterator,IMoniker*> ItValuesBwd;

STDMETHODIMP OrderedItemList::Enum(long start, VARIANT_BOOL ascending, IEnumMoniker **ret) {
	METHTRACE(OrderedItemList,Enum);
	if(ascending) {
		ItValuesFwd begin(m_ordmon.lower_bound(start)), end(m_ordmon.end());
		(*ret = new IterEnum<IEnumMoniker,&IID_IEnumMoniker,IMoniker*,ItValuesFwd,_CopyInterface<IMoniker> >
			(begin,end,begin))->AddRef();
	}
	else {
		ItValuesBwd begin(ordmon::reverse_iterator(m_ordmon.lower_bound(start))),
			end(ordmon::reverse_iterator(m_ordmon.end()));
		(*ret = new IterEnum<IEnumMoniker,&IID_IEnumMoniker,IMoniker*,ItValuesBwd,_CopyInterface<IMoniker> >
			(begin,end,begin))->AddRef();
	}
	return S_OK;
}
// DIOrderedItemList
STDMETHODIMP OrderedItemList::get_Item(long order, IUnknown **ret) {
    METHTRACE(OrderedItemList,get_Item);
	return GetMoniker(order,(IMoniker**)ret);
}
STDMETHODIMP OrderedItemList::put_Item(long order, IUnknown *val) {
    METHTRACE(OrderedItemList,put_Item);
	CComPtr<IMoniker> val2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&val2)));
	return SetMoniker(order,val2);
}
STDMETHODIMP OrderedItemList::get_AllItems(long start, VARIANT_BOOL ascending, IDispatch **ret) {
    METHTRACE(OrderedItemList,get_AllItems);
	if(ascending) {
		ItValuesFwd begin(m_ordmon.lower_bound(start)), end(m_ordmon.end());
		(*ret = new collection(make_colldata(begin,end)))->AddRef();
	}
	else {
		ItValuesBwd begin(ordmon::reverse_iterator(m_ordmon.lower_bound(start))),
			end(ordmon::reverse_iterator(m_ordmon.end()));
		(*ret = new collection(make_colldata(begin,end)))->AddRef();
	}
	return S_OK;
}

// IOrderedItemList
STDMETHODIMP OrderedItemList::Connect(IGroup *group) {
	METHTRACE(OrderedItemList,Connect);
	m_group = group;
	return S_OK;
}
STDMETHODIMP OrderedItemList::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(OrderedItemList,Copy);
	RUN(CoCreateInstance(CLSID_OrderedItemList,aggr,CLSCTX_ALL,riid,ret));
	CComQIPtr<IOrderedItemList> oii = (IUnknown*)*ret;
	RUN(CComQIPtr<IGroupItem>(oii)->Connect(dest));
	for(ordmon::iterator i = m_ordmon.begin();i!=m_ordmon.end();i++) {
		CComPtr<IMoniker> mk2;
		RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,i->second,IID_NULL,0,&mk2));
		RUN(oii->SetMoniker(i->first,mk2));
	}
	return S_OK;
}
STDMETHODIMP OrderedItemList::Realize(IBindCtx *bctx) {
	METHTRACE(OrderedItemList,Realize);
	if(m_realized)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	m_realized = true;
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	for(ordmon::iterator i = m_ordmon.begin();i!=m_ordmon.end();i++) {
		DWORD debug = i->first;
		if(i->second) {
			CComPtr<IBindCtx> bctx2;
			RUN(CreateBindCtx(0,&bctx2));
			CComPtr<IUnknown> u;
			RUN(i->second->BindToObject(bctx2,mkG,riidret(&u)));
			if(CComQIPtr<IGroupItem> gi = u)
				RUN(gi->Realize(bctx2));
		}
	}
	return S_OK;
}
STDMETHODIMP OrderedItemList::Remove() {
	METHTRACE(OrderedItemList,Remove);
	for(ordmon::iterator i = m_ordmon.begin();i!=m_ordmon.end();i++)
		if(i->second)
				RUN(m_group->ReleaseStgRef(i->second));
	return S_OK;
}
// IPersistStream
STDMETHODIMP OrderedItemList::GetClassID(CLSID *pClassID) {
	METHTRACE(OrderedItemList,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP OrderedItemList::IsDirty(void) {
	METHTRACE(OrderedItemList,IsDirty);
	return S_FALSE;
}
STDMETHODIMP OrderedItemList::Load(IStream *pStm) {
	METHTRACE(OrderedItemList,Load);
	unsigned n;
	RUN(pStm->Read(&n,sizeof(unsigned),0));
	while(n--) {
		int order;
		RUN(pStm->Read(&order,sizeof(int),0));
		CComPtr<IMoniker> mk;
		RUN(FixedOleLoadFromStream(pStm,riidret(&mk)));
		RUN(setMoniker(order,mk,false));
	}
	return S_OK;
}
STDMETHODIMP OrderedItemList::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(OrderedItemList,Save);
	unsigned n = m_ordmon.size();
	RUN(pStm->Write(&n,sizeof(unsigned),0));
	for(ordmon::iterator i = m_ordmon.begin();i!=m_ordmon.end();i++) {
		RUN(pStm->Write(&i->first,sizeof(int),0));
		RUN(FixedOleSaveToStream(i->second,pStm));
	}
	return S_OK;
}
STDMETHODIMP OrderedItemList::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(OrderedItemList,GetSizeMax);
	CComObjectStack<countStream> s;
	RUN(Save(&s,false));
	*pcbSize = s.m_pos;
	return S_OK;
}
