/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "SomePointersPersistor.h"
#include "Collection.h"

SomePointersPersistor::SomePointersPersistor() : m_realized(false) {
	CONSTRUCTORTRACE(SomePointersPersistor);
}
SomePointersPersistor::~SomePointersPersistor() {
	DESTRUCTORTRACE(SomePointersPersistor);
	RUN(clear(false));
}
HRESULT SomePointersPersistor::clear(bool releaseStgRef) {
    METHTRACE(SomePointersPersistor,clear);
	for(vm::iterator i = m_pointers.begin();i!=m_pointers.end();++i)
		if(*i) {
			if(releaseStgRef)
				RUN(m_group->ReleaseStgRef(*i));
			(*i)->Release();
		}
	m_pointers.clear();
	return S_OK;
}
// IPersistor
STDMETHODIMP SomePointersPersistor::SetSubject(VARIANT_BOOL realized,IUnknown *site) {
	METHTRACE(SomePointersPersistor,SetSubject);
	m_subject = site;
	m_realized = realized!=0; // supports "realize this" mode
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::get_Subject(IUnknown **ret) {
	METHTRACE(SomePointersPersistor,GetSubject);
	m_subject.CopyTo(ret);
	return S_OK;
}
// IHoldSomePointers
STDMETHODIMP SomePointersPersistor::Push(IMoniker *mk) {
	METHTRACE(SomePointersPersistor,Push);
	if(!m_group)
		ERR(E_PENDING);
	m_pointers.push_back(mk);
	if(mk) {
		mk->AddRef();
		RUN(m_group->AddStgRef(mk));
	}
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::Replace(int index, IMoniker *mk) {
    METHTRACE(SomePointersPersistor,Replace);
	if(!m_group)
		ERR(E_PENDING);
	if(index<0 || index>=int(m_pointers.size()))
		ERR(E_INVALIDARG);
	CComPtr<IMoniker> last = m_pointers[index];
	if(m_pointers[index] = mk)
		RUN(m_group->AddStgRef(mk));
	if(last)
		RUN(m_group->ReleaseStgRef(last));
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::Clear() {
	METHTRACE(SomePointersPersistor,Clear);
	return clear(true);
}
typedef CComObject<CComEnum<IEnumMoniker,&IID_IEnumMoniker,IMoniker*,_CopyInterface<IMoniker> > > enumk;
STDMETHODIMP SomePointersPersistor::Enum(IEnumMoniker **ret) {
	METHTRACE(SomePointersPersistor,Enum);
	enumk *e = new enumk;
	e->Init(&*m_pointers.begin(),&*m_pointers.end(),0,AtlFlagCopy);
	(*ret = e)->AddRef();
	return S_OK;
}
// DIHoldSomePointers
STDMETHODIMP SomePointersPersistor::PushOne(IUnknown *val) {
    METHTRACE(SomePointersPersistor,PushOne);
	CComPtr<IMoniker> mk2;
	RUNRET(val->QueryInterface(riidret(&mk2)));
	return Push(mk2);
}
STDMETHODIMP SomePointersPersistor::Push(SAFEARRAY *args) {
    METHTRACE(SomePointersPersistor,Push);
	if(!(args->fFeatures&FADF_VARIANT))
		ERR(E_INVALIDARG);
	if(args->cDims!=1)
		ERR(E_INVALIDARG);
	VARIANT *args2 = (VARIANT*)args->pvData;
	int N = args->rgsabound[0].cElements;
	for(int i = 0;i<N;++i) {
		if(args2[i].vt!=VT_UNKNOWN)
			ERR(E_INVALIDARG);
		CComPtr<IMoniker> mk2;
		if(args2[i].punkVal)
			RUNRET(args2[i].punkVal->QueryInterface(riidret(&mk2)));
		RUNRET(Push(mk2));
	}
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::get_Params(IDispatch **ret) {
    METHTRACE(SomePointersPersistor,get_Params);
	(*ret = new collection(make_colldata(m_pointers)))->AddRef();
	return S_OK;
}
// IGroupItem
STDMETHODIMP SomePointersPersistor::Connect(IGroup *group) {
	METHTRACE(SomePointersPersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(SomePointersPersistor,Copy);
	CComPtr<IHoldSomePointers> hsp2;
	RUN(CREATE(hsp2,GetObjectCLSID()));
	RUN(CComQIPtr<IGroupItem>(hsp2)->Connect(dest));
	for(vm::iterator i = m_pointers.begin();i!=m_pointers.end();++i) {
		CComPtr<IMoniker> mk2;
		if(*i) // mks can be null
			RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,*i,IID_NULL,0,&mk2));
		RUN(hsp2->Push(mk2));
	}
	RUN(CComQIPtr<DIPersistor>(hsp2)->SetSubject(m_realized,m_subject));
	return hsp2->QueryInterface(riid,ret);
}
typedef CComObject<CComEnum<IEnumUnknown,&IID_IEnumUnknown,IUnknown*,_CopyInterface<IUnknown> > > enumu;
STDMETHODIMP SomePointersPersistor::Realize(IBindCtx *bctx) {
	METHTRACE(SomePointersPersistor,Realize);
	if(m_realized)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	m_realized = true;
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	IUnknown **ray = new IUnknown*[m_pointers.size()],
		**ui = ray;
	for(vm::iterator i = m_pointers.begin();i!=m_pointers.end();i++) {
		if(*i) {
 			RUN((*i)->BindToObject(bctx,mkG,IID_IUnknown,(void**)ui));
			if(CComQIPtr<IGroupItem> gi = *ui)
				RUN(gi->Realize(bctx));
		}
		else
			*ui = 0;
		ui++;
	}
	enumu *e = new enumu;
	e->Init(ray,ui,0,AtlFlagTakeOwnership);
	CComPtr<IEnumUnknown> eu = e; // for vital refcount
	CComQIPtr<INeedSomePointers> nsp = m_subject;
	if(!nsp)
		ERR(E_UNEXPECTED);
	CComPtr<IEnumMoniker> em;
	RUN(Enum(&em));
	return nsp->TakePointers(eu,em);
}
STDMETHODIMP SomePointersPersistor::Remove() {
	METHTRACE(SomePointersPersistor,Remove);
	for(vm::iterator i = m_pointers.begin();i!=m_pointers.end();i++)
		RUN(m_group->ReleaseStgRef(*i));
	return S_OK;
}
// IPersistStream
STDMETHODIMP SomePointersPersistor::GetClassID(CLSID *pClassID) {
	METHTRACE(SomePointersPersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::IsDirty(void) {
	METHTRACE(SomePointersPersistor,IsDirty);
	return S_FALSE;
}
STDMETHODIMP SomePointersPersistor::Load(IStream *pStm) {
	METHTRACE(SomePointersPersistor,Load);
	int n;
	RUN(pStm->Read(&n,sizeof(int),0));
	while(n--) {
		IMoniker *mk; // not CComPtr to keep ref
		RUN(FixedOleLoadFromStream(pStm,riidret(&mk)));
		m_pointers.push_back(mk);
	}
	// here I use the "fixed" because ^$*@%$#* std ole doesn't support IPersistStreamInit (how dumb is that?)
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_subject)));
	if(!m_subject) // must make this explicit because FixedOle.. above will allow null
		ERR(E_UNEXPECTED);
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(SomePointersPersistor,Save);
	if(!m_subject)
		ERR(E_PENDING);
	int n = m_pointers.size();
	RUN(pStm->Write(&n,sizeof(int),0));
	for(vm::iterator i = m_pointers.begin();i!=m_pointers.end();++i)
		RUN(FixedOleSaveToStream(*i,pStm));
	RUN(FixedOleSaveToStream(m_subject,pStm));
	return S_OK;
}
STDMETHODIMP SomePointersPersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(SomePointersPersistor,GetSizeMax);
	ERR(E_NOTIMPL); // oy.  I give up.
}
