/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "AllPointsContainer.h"

//-=-=-=-=-=-=-=-=-=-=-=
// ConnectionPoint
//
static int IDOFS = 42;
ConnectionPoint::ConnectionPoint() : m_currID(IDOFS) {
	CONSTRUCTORTRACE(ConnectionPoint);
}
ConnectionPoint::~ConnectionPoint() {
	DESTRUCTORTRACE(ConnectionPoint);
	_ASSERTE(m_currID!=0);
	m_currID = 0;
	for(cxnmap::iterator i = m_connections.begin();i!=m_connections.end();++i)
		i->second->Release();
}
STDMETHODIMP ConnectionPoint::Advise(IUnknown * pUnkSink,DWORD * pdwCookie) {
	METHTRACE(ConnectionPoint,Advise);
	// ensure it really supports this
	CComPtr<IUnknown> u;
	if(FAILED(pUnkSink->QueryInterface(m_iid,(void**)&u)))
		ERR(CONNECT_E_CANNOTCONNECT);
	// & keep it.
	(m_connections[*pdwCookie = m_currID++] = pUnkSink)->AddRef();
	return S_OK;
}
STDMETHODIMP ConnectionPoint::Unadvise(DWORD dwCookie) {
	METHTRACE(ConnectionPoint,Unadvise);
	cxnmap::iterator i = m_connections.find(dwCookie);
	if(i==m_connections.end())
		ERR(CONNECT_E_NOCONNECTION);
	i->second->Release();
	m_connections.erase(i);
	return S_OK;
}
typedef CComObject<CComEnum<IEnumConnections, &IID_IEnumConnections, CONNECTDATA,
	_Copy<CONNECTDATA> > > enumCONNECTDATA;
STDMETHODIMP ConnectionPoint::EnumConnections(IEnumConnections ** ppEnum) {
	METHTRACE(ConnectionPoint,EnumConnections);
	CONNECTDATA *ray = new CONNECTDATA[m_connections.size()],
		*icd = ray;
	for(cxnmap::iterator i = m_connections.begin();i!=m_connections.end();i++) {
		(icd->pUnk = i->second)->AddRef();
		icd->dwCookie = i->first;
		*icd++;
	}
	enumCONNECTDATA *ec = new enumCONNECTDATA;
	ec->Init(ray,icd,0,AtlFlagTakeOwnership);
	(*ppEnum = ec)->AddRef();
	return S_OK;
}
//-=-=-=-=-=-=-=-=-=-=-=
// AllPointsContainer
//
AllPointsContainer::AllPointsContainer() {
	CONSTRUCTORTRACE(AllPointsContainer);
}
AllPointsContainer::~AllPointsContainer() {
	DESTRUCTORTRACE(AllPointsContainer);
	for(cpmap::iterator i = m_points.begin();i!=m_points.end();++i)
		i->second->Release();
}
// IConnectionPointContainer
typedef CComObject<CComEnum<IEnumConnectionPoints, &IID_IEnumConnectionPoints, IConnectionPoint*,
	_CopyInterface<IConnectionPoint> > > enumCP;
STDMETHODIMP AllPointsContainer::EnumConnectionPoints(IEnumConnectionPoints ** ppEnum) {
	METHTRACE(AllPointsContainer,EnumConnectionPoints);
	IConnectionPoint **ray = new IConnectionPoint*[m_points.size()],
		**icp =ray;
	for(cpmap::iterator i = m_points.begin();i!=m_points.end();i++)
		(*icp++ = i->second)->AddRef();
	enumCP *ecp = new enumCP;
	ecp->Init(ray,icp,0,AtlFlagTakeOwnership);
	(*ppEnum = ecp)->AddRef();
	return S_OK;
}
STDMETHODIMP AllPointsContainer::FindConnectionPoint(REFIID riid,IConnectionPoint ** ppCP) {
	METHTRACE(AllPointsContainer,FindConnectionPoint);
	cpmap::iterator i = m_points.find(riid);
	if(i==m_points.end())
		ERR(E_FAIL);
	if(ppCP)
		(*ppCP = i->second)->AddRef();
	return S_OK;
}

// IMCCAllPointsContainer
STDMETHODIMP AllPointsContainer::EnsurePoint(REFIID riid, IConnectionPoint **ret) {
	METHTRACE(AllPointsContainer,EnsurePoint);
	if(FAILED(FindConnectionPoint(riid,ret))) {
		IConnectionPoint *newCP = new CP(riid,this);
		(m_points[riid] = newCP)->AddRef();
		if(ret)
			(*ret = newCP)->AddRef();
	}
	return S_OK;
}
