/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "AmbientMapper.h"

typedef struct {
	DISPID dispid;
	LPOLESTR name;
} dispMap;
dispMap g_stdAmbients[] =
{
	{DISPID_AMBIENT_APPEARANCE,L"APPEARANCE"},
	{DISPID_AMBIENT_AUTOCLIP,L"AUTOCLIP"},
	{DISPID_AMBIENT_BACKCOLOR,L"BACKCOLOR"},
	{DISPID_AMBIENT_DISPLAYASDEFAULT,L"DISPLAYASDEFAULT"},
	{DISPID_AMBIENT_DISPLAYNAME,L"DISPLAYNAME"},
	{DISPID_AMBIENT_FONT,L"FONT"},
	{DISPID_AMBIENT_FORECOLOR,L"FORECOLOR"},
	{DISPID_AMBIENT_LOCALEID,L"LOCALEID"},
	{DISPID_AMBIENT_MESSAGEREFLECT,L"MESSAGEREFLECT"},
	{DISPID_AMBIENT_PALETTE,L"PALETTE"},
	{DISPID_AMBIENT_SCALEUNITS,L"SCALEUNITS"},
	{DISPID_AMBIENT_SHOWGRABHANDLES,L"SHOWGRABHANDLES"},
	{DISPID_AMBIENT_SHOWHATCHING,L"SHOWHATCHING"},
	{DISPID_AMBIENT_SUPPORTSMNEMONICS,L"SUPPORTSMNEMONICS"},
	{DISPID_AMBIENT_TEXTALIGN,L"TEXTALIGN"},
	{DISPID_AMBIENT_TRANSFERPRIORITY,L"TRANSFERPRIORITY"},
	{DISPID_AMBIENT_UIDEAD,L"UIDEAD"},
	{DISPID_AMBIENT_USERMODE,L"USERMODE"}
};
const g_nAmbients = 18;
AmbientMapper::AmbientMapper() {
	CONSTRUCTORTRACE(AmbientMapper);
	for(int i = 0;i<g_nAmbients;i++) {
		m_map[g_stdAmbients[i].name] = g_stdAmbients[i].dispid;
		m_backmap[g_stdAmbients[i].dispid] = g_stdAmbients[i].name;
	}
}
AmbientMapper::~AmbientMapper() {
	DESTRUCTORTRACE(AmbientMapper);
}
STDMETHODIMP AmbientMapper::GetTypeInfoCount(UINT * pctinfo) {
	METHTRACE(AmbientMapper,GetTypeInfoCount);
	*pctinfo = 0;
	return S_OK;
}
STDMETHODIMP AmbientMapper::GetTypeInfo(UINT iTInfo,LCID lcid,ITypeInfo ** ppTInfo) {
	METHTRACE(AmbientMapper,GetTypeInfo);
	ERR(E_FAIL);
}
STDMETHODIMP AmbientMapper::GetIDsOfNames(REFIID riid,LPOLESTR * rgszNames,UINT cNames,
	LCID lcid,DISPID * rgDispId) {
	METHTRACE(AmbientMapper,GetIDsOfNames);
	if(cNames!=1)
		ERR(DISP_E_NONAMEDARGS);
	if(!rgszNames)
		ERR(E_POINTER); // SO unlikely!
	return GetDispID(*rgszNames,fdexNameCaseInsensitive,rgDispId);
}
STDMETHODIMP AmbientMapper::Invoke(DISPID dispIdMember,REFIID riid,LCID lcid,WORD wFlags,
	DISPPARAMS * pDispParams,VARIANT * pVarResult,EXCEPINFO * pExcepInfo,UINT * puArgErr) {
	METHTRACE(AmbientMapper,Invoke);
	ERR( DISP_E_MEMBERNOTFOUND);
}
STDMETHODIMP AmbientMapper::GetDispID(BSTR bstrName,DWORD grfdex,DISPID *pid) {
	METHTRACE(AmbientMapper,GetDispID);
	dispmap::iterator i = m_map.find(bstrName);
	if(i==m_map.end()) {
		*pid = DISPID_UNKNOWN;
		ERR(DISP_E_UNKNOWNNAME);
	}
	*pid = i->second;
	return S_OK;
}
STDMETHODIMP AmbientMapper::InvokeEx(DISPID id,LCID lcid,WORD wFlags,DISPPARAMS *pdp,VARIANT *pvarRes,
	EXCEPINFO *pei,IServiceProvider *pspCaller) {
	METHTRACE(AmbientMapper,InvokeEx);
	ERR( DISP_E_MEMBERNOTFOUND);
}
STDMETHODIMP AmbientMapper::DeleteMemberByName(BSTR bstr, DWORD grfdex) {
	METHTRACE(AmbientMapper,DeleteMemberByName);
	return S_FALSE;
}
STDMETHODIMP AmbientMapper::DeleteMemberByDispID(DISPID id) {
	METHTRACE(AmbientMapper,DeleteMemberByDispID);
	return S_FALSE;
}
STDMETHODIMP AmbientMapper::GetMemberProperties(DISPID id,DWORD grfdexFetch,DWORD *pgrfdex) {
	METHTRACE(AmbientMapper,GetMemberProperties);
	*pgrfdex = grfdexPropCannotAll & grfdexFetch;
	return S_OK;
}
STDMETHODIMP AmbientMapper::GetMemberName(DISPID id,BSTR *pbstrName) {
	METHTRACE(AmbientMapper,GetMemberName);
	backmap::iterator i = m_backmap.find(id);
	if(i==m_backmap.end())
		ERR(DISP_E_MEMBERNOTFOUND);
	*pbstrName = SysAllocString(i->second);
	return S_OK;
}
STDMETHODIMP AmbientMapper::GetNextDispID(DWORD grfdex,DISPID id,DISPID *pid) {
	METHTRACE(AmbientMapper,GetNextDispID);
	if(id==DISPID_STARTENUM)
		id = m_backmap.begin()->first;
	else {
		backmap::iterator i = m_backmap.find(id);
		if(i==m_backmap.end())
			ERR(DISP_E_MEMBERNOTFOUND);
		++i;
		if(i==m_backmap.end())
			return S_FALSE;
		*pid = i->first;
	}
	return S_OK;
}
STDMETHODIMP AmbientMapper::GetNameSpaceParent(IUnknown **ppunk) {
	METHTRACE(AmbientMapper,GetNameSpaceParent);
	*ppunk = 0;
	return S_OK;
}
// IPersist
STDMETHODIMP AmbientMapper::GetClassID(CLSID *ret) {
	METHTRACE(AmbientMapper,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
