/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef bounds_h
#define bounds_h
#include "rect.h"

// all rects are Cartesian, ie top>bottom
class bounder {
	// front is least, back is greatest
	struct ordered : multiset<long> {
		long front() { _ASSERTE(size()); return *begin(); }
		long back() { _ASSERTE(size()); return *(--end()); }
		bool remove(long l) {
			// erase(l) would remove all l
			iterator i = find(l);
			if(i==end())
				return false;
			erase(i);
			return true;
		}
	};
	ordered lr,tb;
public:
	bool valid() {
		return !lr.empty();
	}
	RECTL get() {
		RECTL rv = {lr.front(),tb.back(),lr.back(),tb.front()};
		return rv;
	}
	bool add(const RECTL &rc) {
		RECTL rcOld;
		if(valid())
			rcOld = get();
		lr.insert(rc.left);
		tb.insert(rc.top);
		lr.insert(rc.right);
		tb.insert(rc.bottom);
		return (lr.size()==2) || (get()!=rcOld);
	}
	bool remove(const RECTL &rc) {
		_ASSERTE(valid());
		RECTL rcOld = get();
		CHECK(lr.remove(rc.left));
		CHECK(tb.remove(rc.top));
		CHECK(lr.remove(rc.right));
		CHECK(tb.remove(rc.bottom));
		return !valid() || get()!=rcOld;
	}
};
#endif bounds_h