/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "BrutePriceEngine.h"
#include "rect.h"

BoxFace::BoxFace() {
	CONSTRUCTORTRACE(BoxFace);
}
BoxFace::~BoxFace() {
	DESTRUCTORTRACE(BoxFace);
}
STDMETHODIMP BoxFace::Move(LPCRECT pos) {
	if(!eng)
		ERR(E_FAIL);
	BPObject &o = (*eng->m_board)[n];
	if(pos) {
		if(o.valid&&o.pos==*pos)
			return S_FALSE;
		o.pos = *pos;
		o.valid = true;
	}
	else {
		if(!o.valid)
			return S_FALSE;
		o.valid = false;
	}
	eng->run(n);
	/*
		// remove
		// must decrement n member of later boxes
		for(int i = n+1;i<eng->m_board->size();++i)
			--eng->m_boxFaces[i]->n;
		// remove from tables
		eng->m_board->erase(eng->m_board->begin()+n);
		eng->m_boxFaces.erase(eng->m_boxFaces.begin()+n);
		// tell sinks
		for(IUnknown **iu = m_vec.begin();iu!=m_vec.end();iu++)
			if(CComQIPtr<IMoveRect> mr = *iu)
				RUN(mr->Move(0));
		// mark self as deleted
		eng = 0;
		GetUnknown()->Release();
	}
	*/
	return S_OK;
}
void BoxFace::tell() {
    METHTRACE(BoxFace,tell);
	for(IUnknown **iu = m_vec.begin();iu!=m_vec.end();iu++)
		if(CComQIPtr<IMoveRect> mr = *iu)
			RUN(mr->Move(&(*eng->m_board)[n].pos));
}

BrutePriceEngine::BrutePriceEngine() : m_flexibleBoundary(false) {
	CONSTRUCTORTRACE(BrutePriceEngine);
	m_board = new BPBoard;

	RECT rcInvalid = {-1,-1,-1,-1};
	m_board->push_back(make_bounds(rcInvalid,false)); // position 0
	m_board->push_back(make_border(rect(500,500,500,500),false)); // 1
	face *out = new face(this,0),*in = new face(this,1);
	m_boxFaces.push_back(out); out->AddRef(); // 0
	m_boxFaces.push_back(in); in->AddRef(); // 1
	_ASSERTE(m_board->size()==m_boxFaces.size()); // size 2
}
BrutePriceEngine::~BrutePriceEngine() {
	DESTRUCTORTRACE(BrutePriceEngine);
	for(vface::iterator i = m_boxFaces.begin();i!=m_boxFaces.end();++i)
		if(*i) {
			(*i)->eng = 0;
			(*i)->Release();
		}
}
void BrutePriceEngine::run(int changed) {
    METHTRACE(BrutePriceEngine,run);
	BPGame game;
	game.flexibleBoundary = m_flexibleBoundary;
	BPBoard *old = m_board;
	m_board = game.run(old);
#ifdef BP_INDICATE
	HBRUSH br;
#endif
	if(m_board) {
		for(unsigned i = 0; i<m_board->size();++i)
			if((*m_board)[i].valid && ((*m_board)[i].pos!=(*old)[i].pos || i == changed)) // notify object that was changed before run() was called
				m_boxFaces[i]->tell();
		delete old;
#ifdef BP_INDICATE
		br = CreateSolidBrush(RGB(0,200,0));
#endif
	}
	else {
		m_board = old; // engine failed to find a solution (it's not all that smart)
#ifdef BP_INDICATE
		br = CreateSolidBrush(RGB(200,0,0));
#endif
	}
#ifdef BP_INDICATE
	HDC hdc = GetDC(0);
	RECT r = {1000,0,1024,24};
	FillRect(hdc,&r,br);
	ReleaseDC(0,hdc);
#endif
}
// IRectEngine
STDMETHODIMP BrutePriceEngine::new_Delegate(IMoveRect **ret) {
    METHTRACE(BrutePriceEngine,new_Delegate);
	RECT rcInvalid = {-1,-1,-1,-1};
	m_board->push_back(make_box(BPP_Normal,rcInvalid,false));
	face *d = new face(this,m_boxFaces.size());
	m_boxFaces.push_back(d);
	d->AddRef(); // engine holds ref
	(*ret = d)->AddRef();
	_ASSERTE(m_board->size()==m_boxFaces.size());
	return S_OK;
}
STDMETHODIMP BrutePriceEngine::get_Outer(IMoveRect **ret) {
    METHTRACE(BrutePriceEngine,get_Outer);
	(*ret = m_boxFaces[0])->AddRef();
	return S_OK;
}
STDMETHODIMP BrutePriceEngine::get_Inner(IMoveRect **ret) {
    METHTRACE(BrutePriceEngine,get_Inner);
	(*ret = m_boxFaces[1])->AddRef();
	return S_OK;
}
STDMETHODIMP BrutePriceEngine::put_OuterFlexible(VARIANT_BOOL val) {
    METHTRACE(BrutePriceEngine,put_OuterFlexible);
	m_flexibleBoundary = val!=VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP BrutePriceEngine::get_OuterFlexible(VARIANT_BOOL *ret) {
    METHTRACE(BrutePriceEngine,get_OuterFlexible);
	*ret = m_flexibleBoundary?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
// IMontageInclusion
STDMETHODIMP BrutePriceEngine::ContainsRect(LPCRECT lpcr) {
    METHTRACE(BrutePriceEngine,ContainsRect);
	// the rect doesn't touch our bounds if it's within the inner or
	// doesn't intersect with the outer
	if((*m_board)[1].valid)
		if(within(*lpcr,(*m_board)[1].pos))
			return S_FALSE;
	if((*m_board)[0].valid)
		if(intersectType(*lpcr,(*m_board)[0].pos)==int_none)
			return S_FALSE;
	return S_OK;
}
// IPersist
STDMETHODIMP BrutePriceEngine::GetClassID(CLSID *ret) {
    METHTRACE(BrutePriceEngine,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
