/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ButtonSet.h"

ButtonSet::ButtonSet() :
	m_clrFillColor(0x80000000|COLOR_BTNTEXT),
	m_clrForeColor(0x80000000|COLOR_3DDKSHADOW),
	m_clrBackColor(0x80000000|COLOR_3DFACE),
	m_clrBorderColor(0x80000000|COLOR_3DSHADOW),
	m_clrPushedColor(RGB(0,255,32)),
	m_clrDisabledColor(0x80000000|COLOR_GRAYTEXT),
	m_nFillStyle(8076332),
	m_nDrawStyle(PS_INSIDEFRAME),
	m_nDrawWidth(0),
	m_nBorderStyle(999999),
	m_nBorderWidth(0),
	m_bBorderVisible(false),
	m_hit(m_buttons.end())
{
}
ButtonSet::~ButtonSet() {
}
STDMETHODIMP ButtonSet::get_PushedColor(OLE_COLOR *ret) {
	*ret = m_clrPushedColor;
	return S_OK;
}
STDMETHODIMP ButtonSet::put_PushedColor(OLE_COLOR val) {
	m_clrPushedColor = val;
	return S_OK;
}
STDMETHODIMP ButtonSet::get_DisabledColor(OLE_COLOR *ret) {
	*ret = m_clrDisabledColor;
	return S_OK;
}
STDMETHODIMP ButtonSet::put_DisabledColor(OLE_COLOR val) {
	m_clrDisabledColor = val;
	return S_OK;
}

HRESULT readPoints(vector<POINT> &pdest,vector<int> &ndest,POINT *points, int npoints) {
	if(!npoints)
		return S_FALSE;
	if(npoints<0)
		SERR(E_INVALIDARG);
	int starti = 0;
	pdest.push_back(points[0]);
	for(int i = 1; i<npoints; ++i) {
		if(points[i]==points[starti]) {
			ndest.push_back(i-starti);
			if((starti = ++i)==npoints)
				break;
		}
		if(i==npoints-1) // must close!
			SERR(E_INVALIDARG);
		pdest.push_back(points[i]);
	}
	return S_OK;
}
void offsetPoints(vector<POINT> &points,SIZE offset) {
	for(vector<POINT>::iterator pi = points.begin(); pi!=points.end(); ++pi)
		*pi += offset;
}
STDMETHODIMP ButtonSet::Add(int ID,DWORD flags,RECT *pos,POINT *points,int npoints) {
	if(!(flags&&pos))
		ERR(E_INVALIDARG);
	m_buttons[ID];
	return Modify(ID,flags,pos,points,npoints);
}
STDMETHODIMP ButtonSet::Modify(int ID,DWORD flags,RECT *pos,POINT *points,int npoints) {
    METHTRACE(ButtonSet,Modify);
	map<int,Button>::iterator i = m_buttons.find(ID);
	if(i==m_buttons.end())
		return E_INVALIDARG;
	Button &b = i->second;
	if(flags)
		b.flags = flags;
	if(pos) {
		if(!points) // move
			offsetPoints(b.points,size(pos->left-b.pos.left,pos->top-b.pos.top));
		b.pos = *pos;
	}
	if(points) {
		b.points.clear();
		RUN(readPoints(b.points,b.sizes,points,npoints));
		offsetPoints(b.points,size(b.pos.left,b.pos.top));
	}
	if(m_hWnd)
		SetTimer(17,20); // wait to display the change
	return S_OK;
}
STDMETHODIMP ButtonSet::get_Flags(int ID,DWORD *ret) {
    METHTRACE(ButtonSet,get_Flags);
	map<int,Button>::iterator i = m_buttons.find(ID);
	if(i==m_buttons.end())
		return E_INVALIDARG;
	*ret = i->second.flags;
	return S_OK;
}
STDMETHODIMP ButtonSet::Delete(int ID) {
    METHTRACE(ButtonSet,Delete);
	map<int,Button>::iterator i = m_buttons.find(ID);
	if(i==m_buttons.end())
		return E_INVALIDARG;
	i->second.deleted = true;
	return S_OK;
}
RECT ButtonSet::buttonArea(b_iter i) {
	RECT r = i->second.pos;
	if(m_bWndLess)
		r += size(m_rcPos.left,m_rcPos.top);
	return r;
}
void ButtonSet::invalidateButton(b_iter i) {
    METHTRACE(ButtonSet,invalidateButton);
	RECT r = buttonArea(i);
	if(m_bWndLess)
		RUN(m_spInPlaceSite->InvalidateRect(&r,false));
	else
		if(m_hWnd)
			InvalidateRect(&r,false);
}
HRESULT ButtonSet::OnDrawAdvanced(ATL_DRAWINFO& di) {
    METHTRACE(ButtonSet,OnDrawAdvanced);
	HDC hdc = di.hdcDraw;
	const RECTL &r = *di.prcBounds;
	COLORREF clrBack,clrLine,clrFill,clrPushed,clrDisabled;
	RUN(OleTranslateColor(m_clrBackColor,0,&clrBack));
	HBRUSH backBrush = CreateSolidBrush(clrBack);
	FillRect(hdc,(RECT*)&r,backBrush);
	DeleteObject(backBrush);
	RUN(OleTranslateColor(m_clrForeColor,0,&clrLine));
	DCObject pen(hdc,CreatePen(m_nDrawStyle,m_nDrawWidth,clrLine));
	RUN(OleTranslateColor(m_clrPushedColor,0,&clrPushed));
	DCObject brushPush(hdc,CreateSolidBrush(clrPushed));
	RUN(OleTranslateColor(m_clrFillColor,0,&clrFill));
	DCObject brushNormal(hdc,CreateSolidBrush(clrFill));
	RUN(OleTranslateColor(m_clrDisabledColor,0,&clrDisabled));
	DCObject brushDisabled(hdc,CreateSolidBrush(clrDisabled));
	for(map<int,Button>::iterator i = m_buttons.begin(); i!=m_buttons.end(); ++i) {
		Button &b = i->second;
		vector<POINT> points2(b.points);
		offsetPoints(points2,size(r.left,r.top));
		if(b.flags&MCCB_DISABLED)
			brushDisabled.use();
		else if(b.flags&MCCB_DOWN)
			brushPush.use();
		else
			brushNormal.use();
		switch(b.sizes.size()) {
			case 0:
				break;
			case 1:
				Polygon(hdc,&points2.front(),points2.size());
				break;
			default:
				PolyPolygon(hdc,&points2.front(),&b.sizes.front(),b.sizes.size());
				break;
		}
	}
	return S_OK;
}
LRESULT ButtonSet::Timer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(ButtonSet,Timer);
	if(m_bWndLess)
		RUN(m_spInPlaceSite->InvalidateRect(0,false));
	else
		InvalidateRect(0,false);
	KillTimer(17);
	return 0;
}
LRESULT ButtonSet::LButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(ButtonSet,LButtonDown);
	POINT pt = MAKEPOINT(lParam);
	for(m_hit = m_buttons.begin(); m_hit!=m_buttons.end(); ++m_hit) {
		RECT r2 = buttonArea(m_hit);
		if(PtInRect(&r2,pt))
			break;
	}
	if(m_hit!=m_buttons.end()) {
		if(m_hit->second.flags&MCCB_DISABLED)
			m_hit = m_buttons.end();
		else {
			m_wasDown = m_hit->second.flags&MCCB_DOWN;
			if(m_bWndLess)
				RUN(m_spInPlaceSite->SetCapture(true));
			else
				SetCapture();
		}
	}
	MouseMove(uMsg,wParam,lParam,bHandled);
	return 0;
}

inline void setbit(DWORD &lhs,DWORD mask,bool val) {
	if(val)
		lhs |= mask;
	else
		lhs &= ~mask;
}

LRESULT ButtonSet::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(ButtonSet,MouseMove);
	POINT pt = MAKEPOINT(lParam);
	if(m_hit==m_buttons.end())
		return 0;
	bool newdown;
	RECT r2 = buttonArea(m_hit);
	if(PtInRect(&r2,pt))
		switch(m_hit->second.flags&0xf0) {
		case MCCB_TOGGLE:
			newdown = !(m_hit->second.flags&MCCB_DOWN);
			break;
		default:
			newdown = true;
			break;
		}
	else
		newdown = m_wasDown;
	setbit(m_hit->second.flags,MCCB_DOWN,newdown);
	invalidateButton(m_hit);
	return 0;
}
LRESULT ButtonSet::LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(ButtonSet,LButtonUp);
	if(m_hit==m_buttons.end())
		return 0;
	if(bool(m_hit->second.flags&MCCB_DOWN) != m_wasDown) {
		RUN(fireEvent(GetUnknown(),fireTwoParams<IMCCButtonSetEvents,int,DWORD>
			(&IMCCButtonSetEvents::Changed,m_hit->first,m_hit->second.flags)));
		if((m_hit->second.flags&0xf0)==MCCB_CLICK) {
			setbit(m_hit->second.flags,MCCB_DOWN,false);
			SetTimer(17,20); // stay illuminated for an instant
		}
	}
	if(m_bWndLess)
		RUN(m_spInPlaceSite->SetCapture(false));
	else
		ReleaseCapture();
	m_hit = m_buttons.end();
	return 0;
}