/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct ButtonSet :
	CComCoClass<ButtonSet,&CLSID_MCCButtonSet>,
	CComObjectRoot,
	CComControl<ButtonSet>,
	CStockPropImpl<ButtonSet,DIMCCButtonSet,&IID_DIMCCButtonSet,&LIBID_Montage>,
	IViewObjectExImpl<ButtonSet>,
	IOleInPlaceObjectWindowlessImpl<ButtonSet>,
	IOleObjectImpl<ButtonSet>,
	IPersistStreamInitImpl<ButtonSet>,
	IConnectionPointContainerImpl<ButtonSet>,
	IConnectionPointImpl<ButtonSet,&IID_IMCCButtonSetEvents>,
	IMCCButtonSet
{
	OLE_COLOR m_clrFillColor,
		m_clrForeColor,
		m_clrBackColor,
		m_clrBorderColor,
		m_clrPushedColor,
		m_clrDisabledColor;
	long m_nFillStyle,
		m_nDrawStyle,
		m_nDrawWidth,
		m_nBorderStyle,
		m_nBorderWidth;
	bool m_bBorderVisible;

	struct Button {
		bool deleted;
		DWORD flags;
		RECT pos;
		vector<POINT> points;
		vector<int> sizes;
		Button() : deleted(false),flags(0) {}
	};
	map<int,Button> m_buttons;
	typedef map<int,Button>::iterator b_iter;
	b_iter m_hit; // what mouse hit on lbuttondown, or m_buttons.end()
	bool m_wasDown; // before lbuttondown
	HWND m_lastCapture;



	ButtonSet();
	~ButtonSet();

	REGISTER_GENERIC(L"ButtonSet")
BEGIN_COM_MAP(ButtonSet)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(DIMCCButtonSet)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IMCCButtonSet)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(ButtonSet)
	CONNECTION_POINT_ENTRY(IID_IMCCButtonSetEvents)
END_CONNECTION_POINT_MAP()

BEGIN_PROP_MAP(ButtonSet)
	PROP_DATA_ENTRY("Width", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("Height", m_sizeExtent.cy, VT_UI4)
END_PROP_MAP()


// DIMCCButtonSet (mostly handled by CStockPropImpl
	STDMETHODIMP get_PushedColor(OLE_COLOR *ret);
	STDMETHODIMP put_PushedColor(OLE_COLOR val);
	STDMETHODIMP get_DisabledColor(OLE_COLOR *ret);
	STDMETHODIMP put_DisabledColor(OLE_COLOR val);

// IMCCButtonSet
	STDMETHODIMP Add(int ID,DWORD flags,RECT *pos,POINT *points,int npoints);
	STDMETHODIMP Modify(int ID,DWORD flags,RECT *pos,POINT *points,int npoints);
	STDMETHODIMP get_Flags(int ID,DWORD *ret);
	STDMETHODIMP Delete(int ID);


	RECT buttonArea(b_iter i);
	void invalidateButton(b_iter i);
	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);
BEGIN_MSG_MAP(ButtonSet)
	MESSAGE_HANDLER(WM_TIMER,Timer)
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK,LButtonDown)
	MESSAGE_HANDLER(WM_LBUTTONDOWN,LButtonDown)
	MESSAGE_HANDLER(WM_LBUTTONUP,LButtonUp)
	MESSAGE_HANDLER(WM_MOUSEMOVE,MouseMove)
	CHAIN_MSG_MAP(CComControl<ButtonSet>)
END_MSG_MAP()
	LRESULT Timer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT LButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

};