/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CheapButton.h"

CheapButton::CheapButton() :
	m_button(0),
    m_wstyle(0)
{
	CONSTRUCTORTRACE(CheapButton);
	m_bWindowOnly = TRUE;
}
CheapButton::~CheapButton() {
	DESTRUCTORTRACE(CheapButton);
}


HWND CheapButton::CreateControlWindow(HWND parent, RECT &pos) {
	USES_CONVERSION;
	HWND res = CComControl<CheapButton>::CreateControlWindow(parent,pos);
    m_wstyle |= WS_CHILD | WS_VISIBLE;
    m_button = CreateWindow("BUTTON",W2T(m_text),m_wstyle,
		0,0,wid(pos),hei(pos),m_hWnd,0,_Module.GetModuleInstance(),0);
	return res;
}
STDMETHODIMP CheapButton::InPlaceDeactivate(void) {
	if(m_button) {
		TCHAR buf[200];
		::GetWindowText(m_button,buf,200);
		m_text = buf;
	}
	HRESULT res = CComControl<CheapButton>::IOleInPlaceObject_InPlaceDeactivate();
	m_button = 0; // destroyed because it was a child
	return res;
}
STDMETHODIMP CheapButton::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(CheapButton,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<CheapButton>::SetObjectRects(prcPos,prcClip));
	if(m_button)
		::MoveWindow(m_button,0,0,wid(*prcPos),hei(*prcPos),true);
	return S_OK;
}
LRESULT CheapButton::OnClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(CheapButton,OnClick);
	fireEvent(GetUnknown(),fireVoid<IMCCButtonEvents>(&IMCCButtonEvents::Click));
	return 0;
}
STDMETHODIMP CheapButton::put_WStyle(DWORD val) {
    METHTRACE(CheapButton,put_WStyle);
    m_wstyle = val | WS_CHILD | WS_VISIBLE;
    if(m_button)
		::SetWindowLongPtr(m_button,GWL_STYLE,m_wstyle);
    return S_OK;
}
STDMETHODIMP CheapButton::get_Text(BSTR *ret) {
    METHTRACE(CheapButton,get_Text);
	if(m_button) {
		TCHAR buf[200];
		::GetWindowText(m_button,buf,200);
		*ret = CComBSTR(buf).Detach();
	}
	else
		*ret = m_text.Copy();
	return S_OK;
}
STDMETHODIMP CheapButton::put_Text(BSTR val) {
    METHTRACE(CheapButton,put_Text);
	if(m_button) {
		USES_CONVERSION;
		::SetWindowText(m_button,W2T(val));
	}
	else m_text = val;
	return S_OK;
}
STDMETHODIMP CheapButton::get_State(int *ret) {
    *ret = ::SendMessage(m_button,BM_GETSTATE,0,0);
	return S_OK;
}
