/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CheapCheckBox.h"

CheapCheckBox::CheapCheckBox() :
	m_check(0),
    m_wstyle(0)
{
	CONSTRUCTORTRACE(CheapCheckBox);
	m_bWindowOnly = TRUE;
}
CheapCheckBox::~CheapCheckBox() {
	DESTRUCTORTRACE(CheapCheckBox);
}


HWND CheapCheckBox::CreateControlWindow(HWND parent, RECT &pos) {
	USES_CONVERSION;
	HWND res = CComControl<CheapCheckBox>::CreateControlWindow(parent,pos);
    m_wstyle |= WS_CHILD | WS_VISIBLE;
    m_check = CreateWindow("checkbox",W2T(m_text),m_wstyle,
		0,0,wid(pos),hei(pos),m_hWnd,0,_Module.GetModuleInstance(),0);
	return res;
}
STDMETHODIMP CheapCheckBox::InPlaceDeactivate(void) {
	if(m_check) {
		TCHAR buf[200];
		::GetWindowText(m_check,buf,200);
		m_text = buf;
	}
	HRESULT res = CComControl<CheapCheckBox>::IOleInPlaceObject_InPlaceDeactivate();
	m_check = 0; // destroyed because it was a child
	return res;
}
STDMETHODIMP CheapCheckBox::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(CheapCheckBox,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<CheapCheckBox>::SetObjectRects(prcPos,prcClip));
	if(m_check)
		::MoveWindow(m_check,0,0,wid(*prcPos),hei(*prcPos),true);
	return S_OK;
}
LRESULT CheapCheckBox::OnClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(CheapCheckBox,OnClick);
	fireEvent(GetUnknown(),fireVoid<IMCCCheckBoxEvents>(&IMCCCheckBoxEvents::Click));
	return 0;
}
STDMETHODIMP CheapCheckBox::put_WStyle(DWORD val) {
    METHTRACE(CheapCheckBox,put_WStyle);
    m_wstyle = val | WS_CHILD | WS_VISIBLE;
    if(m_check)
		::SetWindowLongPtr(m_check,GWL_STYLE,m_wstyle);
    return S_OK;
}
STDMETHODIMP CheapCheckBox::get_Text(BSTR *ret) {
    METHTRACE(CheapCheckBox,get_Text);
	if(m_check) {
		TCHAR buf[200];
		::GetWindowText(m_check,buf,200);
		*ret = CComBSTR(buf).Detach();
	}
	else
		*ret = m_text.Copy();
	return S_OK;
}
STDMETHODIMP CheapCheckBox::put_Text(BSTR val) {
    METHTRACE(CheapCheckBox,put_Text);
	if(m_check) {
		USES_CONVERSION;
		::SetWindowText(m_check,W2T(val));
	}
	else m_text = val;
	return S_OK;
}
