/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CheapCombo.h"

CheapCombo::CheapCombo() :
	m_combo(0),
	m_wstyle(CBS_SORT|CBS_DROPDOWN)
{
	CONSTRUCTORTRACE(CheapCombo);
	m_bWindowOnly = TRUE;
}
CheapCombo::~CheapCombo() {
	DESTRUCTORTRACE(CheapCombo);
}
const int HEIGHT = 150;
HWND CheapCombo::CreateControlWindow(HWND parent, RECT &pos) {
    METHTRACE(CheapCombo,CreateControlWindow);
	HWND res = CComControl<CheapCombo>::CreateControlWindow(parent,pos);
    m_wstyle |= WS_CHILD | WS_VISIBLE;
	m_combo = CreateWindow("COMBOBOX","",m_wstyle,
		0,0,wid(pos),hei(pos)+HEIGHT,m_hWnd,0,_Module.GetModuleInstance(),0);
	_ASSERTE(m_combo);
	for(vector<wstring>::iterator si = m_offline.begin(); si!=m_offline.end(); ++si)
		RUN(Add(CComBSTR(si->c_str())));
	if(!m_offline.empty())
		RUN(Choose(CComBSTR(m_offsel.c_str())));
	return res;
}
STDMETHODIMP CheapCombo::InPlaceDeactivate(void) {
    METHTRACE(CheapCombo,InPlaceDeactivate);
	if(m_combo) {
	}
	HRESULT res = CComControl<CheapCombo>::IOleInPlaceObject_InPlaceDeactivate();
	m_combo = 0; // destroyed because it was a child
	return res;
}
STDMETHODIMP CheapCombo::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(CheapCombo,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<CheapCombo>::SetObjectRects(prcPos,prcClip));
	if(m_combo)
		::MoveWindow(m_combo,0,0,wid(*prcPos),hei(*prcPos)+HEIGHT,true);
	return S_OK;
}
LRESULT CheapCombo::OnSelect(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(CheapCombo,OnSelect);
    if(!m_bInPlaceActive) // will receive this message on close
        return 0;
	int n = SendMessage(m_combo,CB_GETCURSEL,0,0);
	if(n!=CB_ERR) {
		TCHAR buf[200];
		SendMessage(m_combo,CB_GETLBTEXT,n,(LPARAM)buf);
		CComBSTR bstr(buf);
		RUN(fireEvent(GetUnknown(),fireOneParam<IMCCComboBoxEvents,BSTR>(&IMCCComboBoxEvents::Chose,bstr)));
	}
	return 0;
}
LRESULT CheapCombo::OnKillFocus(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(CheapCombo,OnKillFocus);
    if(!m_bInPlaceActive) // will receive this message on close
        return 0;
	TCHAR buf[200];
	SendMessage(m_combo,WM_GETTEXT,200,(LPARAM)buf);
	CComBSTR bstr(buf);
	RUN(fireEvent(GetUnknown(),fireOneParam<IMCCComboBoxEvents,BSTR>(&IMCCComboBoxEvents::TextChanged,bstr)));
	return 0;
}
// IMCCComboBox
STDMETHODIMP CheapCombo::put_WStyle(DWORD val) {
    METHTRACE(CheapCombo,put_WStyle);
	m_wstyle = val | WS_CHILD | WS_VISIBLE;
	if(m_combo)
		::SetWindowLongPtr(m_combo,GWL_STYLE,m_wstyle);
	return S_OK;
}
STDMETHODIMP CheapCombo::Get(BSTR *ret) {
    METHTRACE(CheapCombo,Get);
	if(!m_combo) {
		if(m_offline.empty())
			ERR(E_FAIL);
		*ret = CComBSTR(m_offsel.c_str());
	}
	else {
		char buf[200];
		::GetWindowText(m_combo,buf,200);
		*ret = CComBSTR(buf).Detach();
	}
	return S_OK;
}
STDMETHODIMP CheapCombo::Clear() {
    METHTRACE(CheapCombo,Clear);
	if(!m_combo)
		m_offline.clear();
	else
		SendMessage(m_combo,CB_RESETCONTENT,0,0);
	return S_OK;
}
STDMETHODIMP CheapCombo::Add(BSTR val) {
    METHTRACE(CheapCombo,Add);
	if(!m_combo) {
		if(m_offline.empty())
			m_offsel = val;
		m_offline.push_back(val);
	}
	else {
		int N = SendMessage(m_combo,CB_GETCOUNT,0,0);
		USES_CONVERSION;
		char *cvt = W2T(val);
		SendMessage(m_combo,CB_ADDSTRING,0,(LPARAM)cvt);
	}
	return S_OK;
}
STDMETHODIMP CheapCombo::Choose(BSTR val) {
    METHTRACE(CheapCombo,Choose);
	if(!m_combo)
		m_offsel = val;
	else {
		USES_CONVERSION;
		SendMessage(m_combo,CB_SELECTSTRING,-1,(LPARAM)W2T(val));
	}
	return S_OK;
}
STDMETHODIMP CheapCombo::Remove(BSTR val) {
    METHTRACE(CheapCombo,Remove);
	if(!m_combo)
		for(vector<wstring>::iterator si = m_offline.begin(); si!=m_offline.end();)
			if(!wcscmp(si->c_str(),val))
				si = m_offline.erase(si);
			else
				++si;
	else {
		USES_CONVERSION;
		int n = SendMessage(m_combo,CB_FINDSTRINGEXACT,-1,(LPARAM)W2T(val));
		if(n!=CB_ERR)
			SendMessage(m_combo,CB_DELETESTRING,n,0);
	}
	return S_OK;
}

