/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CheapEdit.h"

CheapEdit::CheapEdit() :
	m_edit(0),
	m_wstyle(ES_MULTILINE | ES_WANTRETURN | ES_AUTOVSCROLL)
{
	CONSTRUCTORTRACE(CheapEdit);
	m_bWindowOnly = TRUE;
}
CheapEdit::~CheapEdit() {
	DESTRUCTORTRACE(CheapEdit);
}

LRESULT CheapEdit::OnKillFocus(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    RUN(fireEvent(GetUnknown(),fireVoid<IMCCEditEvents>(&IMCCEditEvents::TextChanged)));
    return 0;
}


HWND CheapEdit::CreateControlWindow(HWND parent, RECT &pos) {
	HWND res = CComControl<CheapEdit>::CreateControlWindow(parent,pos);
    m_wstyle |= WS_CHILD | WS_VISIBLE;
	m_edit = CreateWindow("EDIT","",m_wstyle,
        0,0,wid(pos),hei(pos),m_hWnd,0,_Module.GetModuleInstance(),0);
    if(m_text)
        RUN(put_Text(m_text));
	return res;
}
STDMETHODIMP CheapEdit::InPlaceDeactivate(void) {
	if(m_edit)
        RUN(get_Text(&m_text));
	HRESULT res = CComControl<CheapEdit>::IOleInPlaceObject_InPlaceDeactivate();
	m_edit = 0; // destroyed because it was a child
	return res;
}
STDMETHODIMP CheapEdit::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(CheapEdit,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<CheapEdit>::SetObjectRects(prcPos,prcClip));
	if(m_edit)
		::MoveWindow(m_edit,0,0,wid(*prcPos),hei(*prcPos),true);
	return S_OK;
}
void breaksIn(const char *src,char *dest) {
    if(src) {
        if(*src==10)
            ++src;
        while(*src) {
            switch(*src) {
            case 10:
                *dest++ = 13;
                *dest++ = 13;
                *dest++ = 10;
                break;
            default:
                *dest++ = *src;
            }
            ++src;
        }
    }
    *dest = 0;
}
void breaksOut(const char *src,char *dest) {
    while(*src) {
        switch(*src) {
        case 13:
            while(*src==13||*src==10)
                ++src;
            *dest++ = 10;
        default:
            *dest++ = *src;
        }
        ++src;
    }
    *dest = 0;
}
STDMETHODIMP CheapEdit::get_Text(BSTR *ret) {
    METHTRACE(CheapEdit,get_Text);
    if(m_edit) {
        char buf[10000],buf2[10000];
        ::SendMessage(m_edit,WM_GETTEXT,10000,(LPARAM)buf);
        breaksOut(buf,buf2);
        *ret = CComBSTR(buf2).Detach();
        /*
	    LRESULT lhand = SendMessage(m_edit,EM_GETHANDLE,0,0);
	    char *text = (char*)LocalLock((HLOCAL)lhand);
	    *ret = CComBSTR(text).Detach();
        */
    }
    else
        m_text.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP CheapEdit::put_Text(BSTR val) {
    METHTRACE(CheapEdit,put_Text);
    if(m_edit) {
        USES_CONVERSION;
        char buf[10000];
        breaksIn(OLE2A(val),buf);
        RUN(::SendMessage(m_edit,WM_SETTEXT,0,(LPARAM)buf));
        /*
	    LRESULT lhand = SendMessage(m_edit,EM_GETHANDLE,0,0);
	    LocalFree((HLOCAL)lhand);
	    HLOCAL htext = 0;
        if(val && SysStringLen(val)) {
            htext = LocalAlloc(LPTR,SysStringLen(val)+1);
	        char *text = (char*)LocalLock(htext); // guess these do nothing but why not?
	        int n = SysStringLen(val);
	        AtlW2AHelper(text,val,n);
	        LocalUnlock(htext);
        }
        else htext = LocalAlloc(LPTR,1);
	    SendMessage(m_edit,EM_SETHANDLE,(WPARAM)htext,0);
        */
    }
    else
        m_text = val;
	return S_OK;
}
STDMETHODIMP CheapEdit::put_WStyle(DWORD val) {
    METHTRACE(CheapEdit,put_WStyle);
    m_wstyle = val | WS_CHILD | WS_VISIBLE;
	if(m_edit)
		::SetWindowLongPtr(m_edit,GWL_STYLE,m_wstyle);
	return S_OK;
}
