/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "CheapScroll.h"

CheapScroll::CheapScroll() :
	m_vert(true),
	m_scrollbar(0),
	m_cache(0)
{
	CONSTRUCTORTRACE(CheapScroll);
	m_bWindowOnly = TRUE;
}
CheapScroll::~CheapScroll() {
	DESTRUCTORTRACE(CheapScroll);
}

HWND CheapScroll::CreateControlWindow(HWND parent, RECT &pos) {
	HWND res = CComControl<CheapScroll>::CreateControlWindow(parent,pos);
	m_scrollbar = CreateWindow( "SCROLLBAR","",WS_CHILD | WS_VISIBLE | (m_vert?SBS_VERT:SBS_HORZ),0,0,wid(pos),hei(pos),m_hWnd,HMENU(m_vert+5),_Module.GetModuleInstance(),0);
	if(m_cache.fMask) {
		m_cache.fMask |= SIF_DISABLENOSCROLL;
		::SetScrollInfo(m_scrollbar,SB_CTL,&m_cache,true);
		m_cache.fMask = 0;
	}
	return res;
}
STDMETHODIMP CheapScroll::InPlaceDeactivate(void) {
    METHTRACE(CheapScroll,InPlaceDeactivate);
	if(m_scrollbar) { // remember for later
		m_cache.fMask = SIF_ALL;
		RUN(::GetScrollInfo(m_scrollbar,SB_CTL,&m_cache));
	}
	HRESULT res = CComControl<CheapScroll>::IOleInPlaceObject_InPlaceDeactivate();
	m_scrollbar = 0; // destroyed because it was a child
	return res;
}
STDMETHODIMP CheapScroll::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(CheapScroll,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<CheapScroll>::SetObjectRects(prcPos,prcClip));
	if(m_scrollbar)
		::MoveWindow(m_scrollbar,0,0,wid(*prcPos),hei(*prcPos),true);
	return S_OK;
}
LRESULT CheapScroll::OnScroll(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(CheapScroll,OnScroll);
	int d = 0,x;
	bool xSet(false);
	switch(LOWORD(wParam)) {
	case SB_LEFT:
	case SB_LINELEFT:
		d = -100; // 0.1 cm;
		break;
	case SB_RIGHT:
	case SB_LINERIGHT:
		d = 100; // 0.1 cm;
		break;
	case SB_PAGELEFT:
	case SB_PAGERIGHT: {
		int page;
		RUN(get_Thumb(&page));
		d = (LOWORD(wParam)==SB_PAGELEFT)?-page:page;
		break;
		}
	case SB_THUMBTRACK:
	case SB_THUMBPOSITION: {
		Sinf sinf(SIF_TRACKPOS);
		CHECK(::GetScrollInfo(m_scrollbar,SB_CTL,&sinf));
		x = sinf.nTrackPos;
		xSet = true;
		break;
		}
	}
	if(d) {
		Sinf sinf(SIF_POS|SIF_RANGE|SIF_PAGE);
		CHECK(::GetScrollInfo(m_scrollbar,SB_CTL,&sinf));
		x = min(max(sinf.nPos+d,sinf.nMin),sinf.nMax-int(sinf.nPage+1));
		xSet = true;
	}
	if(xSet) {
		RUN(put_Pos(x));
		RUN(fireEvent(GetUnknown(),fireVoid<IMCCScrollbarEvents>(&IMCCScrollbarEvents::PosChanged)));
	}
	return 0;
}
STDMETHODIMP CheapScroll::get_Vert(VARIANT_BOOL *ret) {
    METHTRACE(CheapScroll,get_Vert);
	*ret = m_vert?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP CheapScroll::put_Vert(VARIANT_BOOL val) {
    METHTRACE(CheapScroll,put_Vert);
	if(m_scrollbar)
		ERR(E_FAIL);
	m_vert = val != VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP CheapScroll::get_Pos(int *ret) {
    METHTRACE(CheapScroll,get_Pos);
	if(!m_scrollbar)
		ERR(E_PENDING);
	Sinf sinf(SIF_POS);
	CHECK(::GetScrollInfo(m_scrollbar,SB_CTL,&sinf));
	*ret = sinf.nPos;
	return S_OK;
}
STDMETHODIMP CheapScroll::put_Pos(int val) {
    METHTRACE(CheapScroll,put_Pos);
	if(!m_scrollbar) {
		m_cache.fMask |= SIF_POS;
		m_cache.nPos = val;
		return S_OK;
	}
	Sinf sinf(SIF_POS);
	sinf.nPos = val;
	::SetScrollInfo(m_scrollbar,SB_CTL,&sinf,true);
	return S_OK;
}
STDMETHODIMP CheapScroll::GetRange(int *retMin, int *retMax) {
    METHTRACE(CheapScroll,GetRange);
	if(!m_scrollbar)
		ERR(E_PENDING);
	Sinf sinf(SIF_RANGE);
	CHECK(::GetScrollInfo(m_scrollbar,SB_CTL,&sinf));
	*retMin = sinf.nMin;
	*retMax = sinf.nMax;
	return S_OK;
}
STDMETHODIMP CheapScroll::SetRange(int min, int max) {
    METHTRACE(CheapScroll,SetRange);
	if(!m_scrollbar) {
		m_cache.fMask |= SIF_RANGE;
		m_cache.nMin = min;
		m_cache.nMax = max;
		return S_OK;
	}
	Sinf sinf(SIF_RANGE);
	sinf.nMin = min;
	sinf.nMax = max;
	::SetScrollInfo(m_scrollbar,SB_CTL,&sinf,true);
	return S_OK;
}
STDMETHODIMP CheapScroll::get_Thumb(int *ret) {
    METHTRACE(CheapScroll,get_Thumb);
	if(!m_scrollbar)
		ERR(E_PENDING);
	Sinf sinf(SIF_PAGE);
	CHECK(::GetScrollInfo(m_scrollbar,SB_CTL,&sinf));
	*ret = sinf.nPage;
	return S_OK;
}
STDMETHODIMP CheapScroll::put_Thumb(int val) {
    METHTRACE(CheapScroll,put_Thumb);
	if(!m_scrollbar) {
		m_cache.fMask |= SIF_PAGE;
		m_cache.nPage = val;
		return S_OK;
	}
	Sinf sinf(SIF_PAGE);
	sinf.nPage = val;
	::SetScrollInfo(m_scrollbar,SB_CTL,&sinf,true);
	return S_OK;
}

