/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ColorBox.h"
ColorBox::ColorBox() {
	CONSTRUCTORTRACE(ColorBox);
	m_clrFillColor = RGB(64,0,196);
}
ColorBox::~ColorBox() {
	DESTRUCTORTRACE(ColorBox);
}
HRESULT ColorBox::OnDrawAdvanced(ATL_DRAWINFO& di) {
    METHTRACE(ColorBox,OnDrawAdvanced);
	COLORREF color;
	RUN(OleTranslateColor(m_clrFillColor,0,&color));
	HBRUSH brush = CreateSolidBrush(color),
		old = (HBRUSH)SelectObject(di.hdcDraw,brush);
	FillRect(di.hdcDraw,(RECT*)di.prcBounds,brush);
	SelectObject(di.hdcDraw,old);
	DeleteObject(brush);
	return S_OK;
}
// overridden IOleObjectImpl
STDMETHODIMP ColorBox::SetClientSite(IOleClientSite *val) {
	RUNRET(IOleObjectImpl<ColorBox>::SetClientSite(val));
	if(val && m_buddy) {
		RECT rc;
		if(SUCCEEDED(m_buddy->get_Rect(&rc)))
			Moving(&rc);
	}
	return S_OK;
}
// INeedOnePointer
STDMETHODIMP ColorBox::TakePointer(IUnknown *u,IMoniker *m) {
    METHTRACE(ColorBox,TakePointer);
	RUN(u->QueryInterface(riidret(&m_buddy)));
	RUN(AtlAdvise(m_buddy,GetUnknown(),IID_IMCCSiteEvents,&m_mnm));
	RUN(APCAdvise(m_buddy,GetUnknown(),IID_IMCCSelectionDraw,&m_mnmWithWalnuts));
	if(m_spClientSite) {
		RECT rc;
		if(SUCCEEDED(m_buddy->get_Rect(&rc)))
			Moving(&rc);
	}
	return S_OK;
}

// IMCCSiteEvents
STDMETHODIMP ColorBox::Moving(LPCRECT winRect) {
    METHTRACE(ColorBox,Moving);
	if(m_spClientSite) {
		RECT rc = {winRect->left-m_nBorderWidth,winRect->top-m_nBorderWidth,
			winRect->right+m_nBorderWidth,winRect->bottom+m_nBorderWidth};
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Rect(&rc));
	}
	return S_OK;
}
STDMETHODIMP ColorBox::Shifting(long oldState, long newState) {
    METHTRACE(ColorBox,Shifting);
	if(newState==MCCS_DELETED)
		RUN(Hide());
	return S_OK;
}
STDMETHODIMP ColorBox::Layering(IMCCLayer *newLayer) {
    METHTRACE(ColorBox,Layering);
	return S_FALSE;
}
STDMETHODIMP ColorBox::ZChanged() {
    METHTRACE(ColorBox,ZChanged);
	return S_FALSE;
}
STDMETHODIMP ColorBox::Wakeup() {
    METHTRACE(ColorBox,Wakeup);
	return S_FALSE;
}
// IMCCSelectionDraw
STDMETHODIMP ColorBox::Show() {
    METHTRACE(ColorBox,Show);
	return S_FALSE; // shouldn't get hit.
}
STDMETHODIMP ColorBox::Hide() {
    METHTRACE(ColorBox,Hide);
	RUN(AtlUnadvise(m_buddy,IID_IMCCSiteEvents,m_mnm));
	RUN(AtlUnadvise(m_buddy,IID_IMCCSelectionDraw,m_mnmWithWalnuts));
	if(m_spClientSite)
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Intent(MCCS_DELETED));
	return S_OK;
}
/*
// DIMCCColorBox
STDMETHODIMP ColorBox::put_BorderWidth(long width) {
	m_nBorderWidth = width;
	return S_OK;
}
STDMETHODIMP ColorBox::get_BorderWidth(long* width) {
	*width = m_nBorderWidth;
	return S_OK;
}
STDMETHODIMP ColorBox::put_FillColor(OLE_COLOR clr) {
	m_color = clr;
	return S_OK;
}
STDMETHODIMP ColorBox::get_FillColor(OLE_COLOR* pclr) {
	*pclr = m_color;
	return S_OK;
}
*/