/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// CtlContainer.cpp : internal cctlcontainer stuff

#include "stdafx.h"
#include "Container.h"
#include "ContainerRef.h"
#include "Hatchling.h"

#include "RegCategory.h"
#include "CvtMetrics.h"
#include "enumIter.h"
#include "blank.h"

#include <stdio.h>

Container::Container() :
	m_hWnd(NULL),
    m_intoWnd(0),
	m_ownwindow(true),
	m_extent(size(7000,7000)), // will get overridden by MontagePersistor, but just in case...
	m_minExtent(m_extent), // note MontagePersistor value is larger
	m_zoom(10000),
	m_state(MCCS_HIDDEN),
	m_pHatchling(NULL),
	m_hatchMode(MHM_ON), // always show hatching as doc
	m_pCaptureObject(0),
	m_pFocusObject(0),
	m_menu(0),
	m_holeMenu(0),
	m_amRoot(false),
	m_haveUI(false),
	m_untolddata(false),
	m_dirty(false),
	m_autoSize(true),
	m_currentDC(0)
{
	CONSTRUCTORTRACE(Container);
	blank(m_extraArea);
	blank(m_reservedArea);
	blank(m_rect);
	blank(m_cliprect);
    RUN(fakeRectFromExtent());
	RUN(createLayer(this,&m_nonlayer));
	m_nonlayer->AddRef();
	(m_refObject = new CComObject<ContainerRef>)->AddRef();
	m_refObject->m_mc = GetUnknown();
	readPrefs();
	m_clsid = CLSID_MontageControlContainer;

	SIZEL smidgen = {6,6},smet;
	AtlPixelToHiMetric(&smidgen,&smet);
	m_margin = rectl(-smet.cx,smet.cy,smet.cx,-smet.cy);
	m_alwaysMargin = false;
}
void Container::readPrefs() {
	USES_CONVERSION;
	CRegKey montage;
	if(!montage.Open(HKEY_CURRENT_USER,"Software\\AT&T\\Montage")) {
		TCHAR exts[100];
		DWORD count(100);
		if(!montage.QueryValue("Native Formats",0,exts,&count)) {
			USES_CONVERSION;
			m_exts = T2OLE(exts);
		}
	}
}
HRESULT freeSites(Layer *l) {
    FUNTRACE(freeSites);
	sitelist &sl = l->m_sites;
    // now remove and release, probably delete
	while(!sl.empty())
		SRUN(sl.front()->approachstate(MCCS_DELETED));
	return S_OK;
}

Container::~Container() {
	DESTRUCTORTRACE(Container);
	m_refObject->m_mc = 0;
	m_refObject->Release();
	m_refObject = 0;

	CComPtr<IMontagePersistor> persistor;
	if(m_amRoot && m_mkPersistor) {
		// tell persistor to forget me so it doesn't try to release me
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		CComPtr<IMoniker> mkG;
		RUN(FoolishCreatePointerMoniker(m_group,&mkG));
		RUN(m_mkPersistor->BindToObject(bctx,mkG,riidret(&persistor)));

		RUN(persistor->ForgetSubject());
	}

	// shut down renderer
	if(m_renderer)
		RUN(CComQIPtr<IMCCItemSetRenderer>(m_renderer)->Stop());

	// detach persistence
	if(m_group) {
		// skip Save step of Unload - it doesn't make sense to scribble on delete
		// (and it's causing bugs of course)
		RUN(CComQIPtr<IPersistStorage>(m_group)->HandsOffStorage());
		RUN(m_group->Unload(0));
	}

	// delete sites & layers
	m_state = MCCS_DELETED;
	while(m_layers.size()) {
		Layer *l = *m_layers.begin();
		RUN(freeSites(l));
		RUN(l->Delete());
	}
	while(m_unlayers.size()) {
		Layer *l = *m_unlayers.begin();
		RUN(freeSites(l));
		RUN(l->Delete());
	}
	RUN(freeSites(m_nonlayer));
	RUN(m_nonlayer->Delete());
	// (explicit for debug.  refs should be vital.)
	m_group.Release();
	m_toolpanel.Release();
	m_dragdata.Release();
}
// OleInPlaceBorderWidth is mentioned in a footnote on page 1022 of Brockschmidt
// and yet
CComBSTR g_OleInPlaceBorderWidth("OleInPlaceBorderWidth"),
	g_DragScrollInset("DragScrollInset"),
	g_DragScrollDelay("DragScrollDelay"),
	g_DragScrollInterval("DragScrollInterval"),
	g_DragDelay("DragDelay"),
	g_DragMinDist("DragMinDist");

// read from WIN.INI and put into dispatch
HRESULT retrieveProfileInt(IDispatchEx *dest,BSTR name,int defVal) {
    FUNTRACE(retrieveProfileInt);
	USES_CONVERSION;
	int val = GetProfileInt(_T("windows"), OLE2T(name), defVal);
	DISPID propid;
	SRUN(dest->GetDispID(name,fdexNameEnsure,&propid));
	CComVariant v(val);
	SRUN(CComDispatchDriver::PutProperty(dest,propid,&v));
	return S_OK;
}
HRESULT Container::FinalConstruct() {
	METHTRACE(Container,FinalConstruct);
	RUN(CREATE(m_defaultSiteSinks,CLSID_MCCAllPointsContainer));
	// HACK: why is this here?
	// A: no good reason at all.
	CComPtr<IMCCSiteOwner> defaultOwnerSink;
	RUN(CREATE(defaultOwnerSink,CLSID_MCCDefaultSiteOwner));
	CComPtr<IConnectionPoint> cp;
	RUN(m_defaultSiteSinks->EnsurePoint(IID_IMCCSiteOwner,&cp));
	DWORD figbar;
	RUN(cp->Advise(defaultOwnerSink,&figbar));
	// aggregation (really should be here)
	// allpoints
	RUN(CREATEA(m_points,CLSID_MCCAllPointsContainer,GetUnknown()));
	RUN(CComQIPtr<IMCCAllPointsContainer>(m_points)->EnsurePoint(IID_IMontageContainerEvents,0));
	// style
	RUN(CREATE(m_stuff,CLSID_MCCStyle));
	// OLE parameters
	USES_CONVERSION;
	// since this one doesn't exist anywhere but in "Inside OLE 2" it's not likely to be defined.
	RUN(retrieveProfileInt(m_stuff,g_OleInPlaceBorderWidth,4));
	RUN(retrieveProfileInt(m_stuff,g_DragScrollInset,DD_DEFSCROLLINSET));
	RUN(retrieveProfileInt(m_stuff,g_DragScrollDelay,DD_DEFSCROLLDELAY));
	RUN(retrieveProfileInt(m_stuff,g_DragScrollInterval,DD_DEFSCROLLINTERVAL));
	RUN(retrieveProfileInt(m_stuff,g_DragDelay,DD_DEFDRAGDELAY));
	RUN(retrieveProfileInt(m_stuff,g_DragMinDist,DD_DEFDRAGMINDIST));
	return S_OK;
}
void Container::FinalRelease() {
	METHTRACE(Container,FinalRelease);
	// fire a GroupOp event so that any layout engines won't take the string of deletions
	// too seriously
	groupEvent(true);
	RUN(fireEvent((IConnectionPointContainer*)this,fireVoid<IMontageContainerEvents>(IMontageContainerEvents::Shutdown)));
	m_points.Release();
}
LPCWSTR g_szMiscKey = L"OleMisc";
HRESULT WINAPI Container::UpdateRegistry(BOOL bRegister) {
	FUNTRACE(ContainerUpdateRegistry);
	_ATL_REGMAP_ENTRY rgReplace[4];
	rgReplace[0].szKey = g_szMiscKey;
	// OLEMISC_RECOMPOSEONRESIZE //|OLEMISC_INSIDEOUT|OLEMISC_ACTIVATEWHENVISIBLE
	rgReplace[0].szData = L"1"; // was buf.c_str() but that bugged out on me
    reggy_things stf(&rgReplace[1],GetObjectCLSID());
	// null termination of sorts:
	rgReplace[3].szKey = rgReplace[3].szData = NULL;
	SRUN(_Module.UpdateRegistryFromResource(IDR_Montage, bRegister, rgReplace));
	// HACK to remove the key "Control" from under the CLSID, so that Corel products will show
	// Montage in their Insert Object dlgs
	//RUN(_Module.UpdateRegistryFromResource(IDR_removeControlTag, bRegister, rgReplace));
	return S_OK;
}
STDMETHODIMP Container::InterfaceSupportsErrorInfo(REFIID riid)
{
	METHTRACE(Container,InterfaceSupportsErrorInfo);
	static const IID* arr[] =
	{
		&IID_IMontageContainer,
	};

	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
HRESULT Container::fakeRectFromExtent() {
	SIZE sz = g_cvt.HI2Pix(m_extent);
	m_rect.left = m_rect.top = 0;
	m_rect.right = sz.cx;
	m_rect.bottom = sz.cy;
    return S_OK;
}
HRESULT Container::getInPlaceWindowContext() {
    METHTRACE(Container,getInPlaceWindowContext);
	RECT rcPos, rcClip;
	m_frameInfo.cb = sizeof(OLEINPLACEFRAMEINFO);
	m_pFrame.Release();
	m_pDoc.Release();
	RUN(m_pInPlaceSite->GetWindowContext(&m_pFrame,&m_pDoc,&rcPos,&rcClip,&m_frameInfo));
	bool sizing = wid(rcPos)!=wid(m_rect) || hei(rcPos)!=hei(m_rect);
	m_rect = rcPos;
	m_cliprect = rcClip;
	if(sizing)
		sizeEvent();
	return S_OK;
}
RECT Container::windowRect() {
	RECT big = boundsPlusExtra();
	if(m_pHatchling)
        return ::rect(m_pHatchling->m_size,m_pHatchling->m_size,m_pHatchling->m_size + wid(big),m_pHatchling->m_size + hei(big));
	else
        return big;
}
HRGN Container::clipRgn() {
    METHTRACE(Container,clipRgn);
	RECT big = boundsPlusExtra(),viewable;
    if(IntersectRect(&viewable,&big,&m_cliprect)) { //&&!EqualRect(&viewable,&big)) {
		OffsetRect(&viewable,-big.left,-big.top);
		return CreateRectRgnIndirect(&viewable);
	}
	else return 0;
}
HRESULT Container::setExtentFromRect(const RECT &rcNew,bool useCurrent) {
    METHTRACE(Container,setExtentFromRect);
	SIZEL newSize;
	if(useCurrent && wid(m_rect) && hei(m_rect)) // scale according to what the container has said is right.
		newSize = size(wid(rcNew)*m_extent.cx/wid(m_rect),
			hei(rcNew)*m_extent.cy/hei(m_rect));
	else {
		// this method doesn't allow container scaling.  Necessary for DocObject
		SIZE in = {wid(rcNew),hei(rcNew)}; // not inverted
		AtlPixelToHiMetric(&in,&newSize); // kind of arbitrary
	}
	/*
	// gives too much extent when the contents are crunched
	CComPtr<IMCCTransform> t;
	RUN(get_Transform(&t));
	SIZE in = {wid(rcNew),-hei(rcNew)}, // inverted for vector
		out;
	RUN(t->W2CS(in,&out));	// which is why I did this first.
	*/
	RUN(SetExtent(DVASPECT_CONTENT,&newSize));
	return S_OK;
}
BOOL g_bRegistered = FALSE;
LPCSTR g_lpszClassName = "MontageContainer";
HRESULT Container::makewindow(bool show) {
	METHTRACE(Container,makewindow);
	HRESULT hr;
	HWND hParentWnd;
    if(m_pInPlaceSite)
	    RUN(m_pInPlaceSite->GetWindow(&hParentWnd));
    else if(!(hParentWnd = m_intoWnd))
        ERR(E_UNEXPECTED);
	if(m_hWnd) { // prob show was not set last time
		if(!show)
			ERR(E_INVALIDARG);
		if(m_pHatchling)
			ShowWindow(m_pHatchling->Wnd(),SW_SHOWNORMAL);
		ShowWindow(m_hWnd,SW_SHOWNORMAL);
		return S_OK;
	}
	if(!g_bRegistered) {
		WNDCLASS wc;

		// main class
		wc.style = CS_DBLCLKS;
		wc.lpfnWndProc = WndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = sizeof (Container *);
		wc.hInstance = _Module.GetModuleInstance();
		wc.hIcon = NULL;
		wc.hCursor = LoadCursor(NULL,IDC_ARROW);
		wc.hbrBackground = NULL;
		wc.lpszMenuName = NULL;
		wc.lpszClassName = g_lpszClassName;
		RegisterClass(&wc);

		g_bRegistered = true;
	}
	BOOL bHatch(true);
	bool bGrabbers(true);
	if(m_hatchMode==MHM_AUTO) {
		CComQIPtr<IDispatch> pAmbient(m_pSite);
		if(pAmbient) {
			CComVariant var;
			DISPPARAMS noargs = {NULL, NULL, 0, 0};

			hr = pAmbient->Invoke(DISPID_AMBIENT_SHOWHATCHING,IID_NULL,LOCALE_USER_DEFAULT,
				DISPATCH_PROPERTYGET,&noargs,&var,NULL,NULL);
			bHatch = (FAILED(hr)||(var.vt!=VT_BOOL))?true:var.boolVal;

			hr = pAmbient->Invoke(DISPID_AMBIENT_SHOWGRABHANDLES,IID_NULL,LOCALE_USER_DEFAULT,
				DISPATCH_PROPERTYGET,&noargs,&var,NULL,NULL);
			bGrabbers = (FAILED(hr)||var.vt!=VT_BOOL||var.boolVal)?true:false;
		}
	}
	else if(m_hatchMode==MHM_OFF)
		bHatch = false;
	if(bHatch) {
		DISPID widthID;
		RUN(m_stuff->GetDispID(g_OleInPlaceBorderWidth,0,&widthID));
		VARIANT width;
		RUN(CComDispatchDriver::GetProperty(m_stuff,widthID,&width));
		_ASSERTE(width.vt==VT_I4);
		m_pHatchling = new Hatchling(this,hParentWnd,bGrabbers,width.lVal,show);
	}
	RECT rc = windowRect();
	HRGN hrgn = clipRgn();
	m_hWnd = CreateWindow(g_lpszClassName,"Container",
		WS_CHILD|WS_CLIPCHILDREN|WS_CLIPSIBLINGS|(show?WS_VISIBLE:0),
		rc.left,rc.top,wid(rc),hei(rc),
		m_pHatchling?m_pHatchling->Wnd():hParentWnd,NULL,_Module.GetModuleInstance(),NULL);
	SetWindowRgn(m_hWnd,hrgn,true);
	SetWindowLong(m_hWnd,0,(LONG)this);
	RUN(RegisterDragDrop(m_hWnd,this));
	HDC temp = GetDC(m_hWnd);
	m_bufferDC = CreateCompatibleDC(temp);
	HBITMAP bitmap = CreateCompatibleBitmap(temp,wid(rc),hei(rc));
	SelectObject(m_bufferDC,bitmap);
	ReleaseDC(m_hWnd,temp);
	if(m_untolddata)
		dirtyMsg();
	return S_OK;
}
void Container::_InvalidateRect(LPCRECT lprc,BOOL erase) {
    METHTRACE(Container,_InvalidateRect);
	if(!m_hWnd)
		return;
	InvalidateRect(m_hWnd,lprc,erase);
	RECT lnr = m_rect;
	lnr.left = lnr.right-30;
	lnr.bottom = lnr.top+140;
	InvalidateRect(m_hWnd,&lnr,true);
}
void Container::InvalidateSite(site *pSite) {
    METHTRACE(Container,InvalidateSite);
	RECT rc = pSite->rect();
	// hard to tell whether ole view/pos rects are RB inclusive (Windows' aren't)
	rc.right++;
	rc.bottom++;
	_InvalidateRect(&rc);
}

void Container::sizeEvent() {
	METHTRACE(Container,sizeEvent);
	groupEvent(true);
	RUN(fireEvent(GetUnknown(),fireOneParam<IMontageContainerEvents,SIZE>(&IMontageContainerEvents::Resize,siz(boundsPlusExtra()))));
	groupEvent(false);
}
void Container::extentEvent() {
	METHTRACE(Container,extentEvent);
	groupEvent(true);
	RUN(fireEvent(GetUnknown(),fireOneParam<IMontageContainerEvents,SIZEL>(&IMontageContainerEvents::NewExtent,m_extent)));
	groupEvent(false);
}

void Container::groupEvent(bool start) {
	METHTRACE(Container,groupEvent);
	RUN(fireEvent(GetUnknown(),fireOneParam<IMontageContainerEvents,BOOL>(&IMontageContainerEvents::GroupOp,start)));
}
HRESULT Container::remove(site *s) {
	METHTRACE(Container,remove);
	sitelist &sl = s->m_pLayer->m_sites;
	sitelist::iterator i = find(sl.begin(),sl.end(),s);
	if(i==sl.end())
		ERR(E_FAIL);
	sl.erase(i);
	s->Release();
	return S_OK;
}
HRESULT Container::show(site *s) {
	METHTRACE(Container,show);
	if(m_transform && s->m_pLayer->m_amTransformed)
		RUN(m_transform->AddRect(&s->m_virtRect));
	InvalidateSite(s);
	return S_OK;
}
HRESULT Container::hide(site *s) {
	METHTRACE(Container,hide);
	if(!m_state) // caused by ~Container: don't bother
		return S_FALSE;
	if(m_transform && s->m_pLayer->m_amTransformed)
		RUN(m_transform->RemoveRect(&s->m_virtRect));
	InvalidateSite(s);
	return S_OK;
}
/* in the following two functions, we only call SetMenu
	when we currently have control over the menus (m_haveUI).
	(note also that if m_haveUI is false, m_holeMenu is NULL)
	This because it can confuse the container to have a contained
	object asking for menu control when it's not supposed to
	(for example, during UIDeactivate!)
*/
HRESULT Container::couiactivate(site *s) {
	METHTRACE(Container,couiactivate);
	CComQIPtr<ICoUIActiveObject> coUI = s->m_pControl;
	if(!coUI)
		ERR(E_FAIL);
	m_menusites.push_back(s);
	HRESULT hr;
	RUN(hr = coUI->Activate(m_menu,&m_widths));
	if(hr==S_OK && m_haveUI) {
		HOLEMENU holeMenu = OleCreateMenuDescriptor(m_menu,&m_widths);
		RUN(m_pFrame->SetMenu(m_menu,holeMenu,m_hWnd));
		OleDestroyMenuDescriptor(m_holeMenu);
		m_holeMenu = holeMenu;
	}
	return S_OK;
}
HRESULT Container::couideactivate(site *s) {
	METHTRACE(Container,couideactivate);
	sitelist::iterator si = find(m_menusites.begin(),m_menusites.end(),s);
	if(si!=m_menusites.end()) {
		m_menusites.erase(si);
		CComQIPtr<ICoUIActiveObject> coUI = s->m_pControl;
		HRESULT hr;
		RUN(hr = coUI->Deactivate(m_menu,&m_widths));
		if(hr==S_OK && m_haveUI) {
			HOLEMENU holeMenu = OleCreateMenuDescriptor(m_menu,&m_widths);
			RUN(m_pFrame->SetMenu(m_menu,holeMenu,m_hWnd));
			OleDestroyMenuDescriptor(m_holeMenu);
			m_holeMenu = holeMenu;
		}
	}
	return S_OK;
}
template<class List>
HRESULT placeRelative(List &l,typename List::value_type v, typename List::value_type other, bool before) {
	typedef typename List::value_type vt;
	typename List::iterator i = find(l.begin(),l.end(),v);
	if(i!=l.end())
		l.erase(i);
	if(other) {
		i = find(l.begin(),l.end(),other);
		if(i==l.end())
			SERR(E_INVALIDARG);
		if(!before)
			i++;
	}
	else
		i = before?l.end():l.begin(); // before 0 is in back; behind 0 is in front
	l.insert(i,v);
	return S_OK;
}
HRESULT Container::before(site *s,site *other) {
	METHTRACE(Container,before);
	HRESULT hr;
	if(!s->m_pLayer)
		ERR(E_PENDING);
	if(other && s->m_pLayer!=other->m_pLayer)
		ERR(E_INVALIDARG);
	if(FAILED(hr = placeRelative(s->m_pLayer->m_sites,s,other,true))) // allowable error
		return hr;
	if(s->m_bRectValid)
		InvalidateSite(s);
	return S_OK;
}
HRESULT Container::after(site *s,site *other) {
	METHTRACE(Container,after);
	HRESULT hr;
	if(!s->m_pLayer)
		ERR(E_PENDING);
	if(other && s->m_pLayer!=other->m_pLayer)
		ERR(E_INVALIDARG);
	if(FAILED(hr = placeRelative(s->m_pLayer->m_sites,s,other,false)))
		return hr;
	if(s->m_bRectValid)
		InvalidateSite(s);
	return S_OK;
}
HRESULT Container::beforeLayer(Layer *l, Layer *other) {
	METHTRACE(Container,beforeLayer);
	HRESULT hr;
	RUN(hr = placeRelative(m_layers,l,other,true));
	if(hr==S_OK) { // add, not change
		m_unlayers.remove(l);
		//l->Release(); // placeRelative grabbed new ref
	}
	_InvalidateRect(0);
	return S_OK;
}
HRESULT Container::afterLayer(Layer *l, Layer *other) {
	METHTRACE(Container,afterLayer);
	HRESULT hr;
	RUN(hr = placeRelative(m_layers,l,other,false));
	if(hr==S_OK) { // add, not change
		m_unlayers.remove(l);
		//l->Release(); // placeRelative grabbed new ref
	}
	_InvalidateRect(0);
	return S_OK;
}
HDC Container::startBuffer(HDC hdc,RECT *r,bool copy) {
	m_currentDC = hdc;
	m_currentRC = *r;
	if(copy)
		BitBlt(m_bufferDC,r->left,r->top,wid(*r),hei(*r),m_currentDC,r->left,r->top,SRCCOPY);
	return m_bufferDC;
}
HDC Container::blitBuffer() {
	_ASSERTE(m_currentDC);
	BitBlt(m_currentDC,m_currentRC.left,m_currentRC.top,wid(m_currentRC),hei(m_currentRC),m_bufferDC,m_currentRC.left,m_currentRC.top,SRCCOPY);
	HDC t = m_currentDC;
	m_currentDC = 0;
	return t;
}
HRESULT Container::sayChange() {
    METHTRACE(Container,sayChange);
    if(!m_untolddata)
        return S_FALSE;
	m_untolddata = false;
	// if we're relying on container to resize based on extent,
	// we'd better know our own extent.
	// this is not the case as an Active Document or in ActivateInto, where extent is
	// based entirely on screen area.
	// this HAPPENS TO BE a good place to check this (ie this is another hack)
	if(m_autoSize && m_renderer &&
            !CComQIPtr<IOleDocumentSite>(m_pSite) && !m_intoWnd) {
		SIZEL s;
		RUN(getExtentFromRenderer(&s));
		if(s!=m_extent) {
			// either this or the OnDataChange event below should cause my
			// container to resize me
            if(CComQIPtr<IOleClientSite> ocs = m_pSite)
			    RUN(ocs->RequestNewObjectLayout());
			setExtent(s);
		}
	}
	if(m_pViewSink)
		m_pViewSink->OnViewChange(DVASPECT_CONTENT,0);
	if(m_dadvise) {
		CComPtr<IEnumSTATDATA> esd;
		RUN(m_dadvise->EnumAdvise(&esd));
		STATDATA sd;
		while(esd->Next(1,&sd,0) == S_OK) {
			StgMedium stg;
			STGMEDIUM *stgm = 0;
			if(!(sd.advf&ADVF_NODATA)&&m_renderer)
				RUN(GetData(&sd.formatetc,stgm = &stg));
			sd.pAdvSink->OnDataChange(&sd.formatetc,stgm);
			ReleaseStgMedium(&stg);
			ULONG q = sd.pAdvSink->Release(); //?
		}
		//RUN(m_dadvise->SendOnDataChange(this,0,0));
	}
	return S_OK;
}
HRESULT Container::dirtyMsg() {
    if(m_hWnd&&m_untolddata)
        CHECKR(PostMessage(m_hWnd,MONTAGE_NOTICE_CHANGES,0,0)); // response calls sayChange above
    return S_OK;
}
HRESULT Container::dirty() {
    METHTRACE(Container,dirty);
    if(!m_untolddata) {
	    m_dirty = m_untolddata = true;
        dirtyMsg();
    }
	return S_OK;
}
void Container::changeFocus() {
    RUN(fireEvent(GetUnknown(),fireVoid<IMontageContainerEvents>(&IMontageContainerEvents::FocusChanged)));
}
