/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Container.h : Main Montage Container class

#ifndef CONTAINER_H
#define CONTAINER_H


#include "Site.h"
#include "Layer.h"

// hatch window class
class Hatchling;

// montage posts this message to itself on RequestTrigger
#define MONTAGE_TRIGGER_MESSAGE (WM_USER + 200)
// posts this to self when the m_untolddata bit get set to calculate and send it (view data)
#define MONTAGE_NOTICE_CHANGES (WM_USER + 201)
// this one to respond to OnDataChange
#define MONTAGE_CHECK_SITE_SIZE (WM_USER + 202)
inline bool mousemess(UINT uMsg) {
	return (uMsg>=WM_MOUSEFIRST&&uMsg<=WM_MOUSELAST) || uMsg == WM_SETCURSOR;
}
inline bool keymess(UINT uMsg) {
	return uMsg>=WM_KEYFIRST&&uMsg<=WM_KEYLAST
		||uMsg==WM_IME_SETCONTEXT
		||uMsg==WM_IME_NOTIFY
		||uMsg==WM_IME_CONTROL
		||uMsg==WM_IME_COMPOSITIONFULL
		||uMsg==WM_IME_SELECT
		||uMsg==WM_IME_CHAR
		||uMsg==WM_IME_KEYDOWN
		||uMsg==WM_IME_KEYUP
		||uMsg==WM_HELP
		||uMsg==WM_CANCELMODE;
}
struct ContainerRef;
typedef map<site*,CONTROLINFO> mnemon_info;
/////////////////////////////////////////////////////////////////////////////
// Container
extern const LPOLESTR g_szType;
struct ATL_NO_VTABLE Container :
	CComObjectRoot,
	CComCoClass<Container,&CLSID_MontageControlContainer>,
	IMontageContainer,
	IDispatchImpl<DIMontageContainer>,
	//IProvideClassInfo,

	ISupportErrorInfo,

	IViewObjectEx,
	IPersistStorage,
	IOleObject,
    IObjectWithSite,
	IDataObject,
	IOleCache, // dummy implementation to fool apps expecting default handler
	IOleInPlaceObject,
	IOleInPlaceActiveObject,
	IPointerInactive,
	IDropTarget,
	IDispatchEx,
	IOleDocument,
	IOleDocumentView,
	IPersistFile,
	IRunnableObject, // dummy
	IOleItemContainer, // mostly deferred to group
	IHlinkTarget

{
	CComPtr<IUnknown> m_points; // AllPointsContainer aggregated
	ContainerRef *m_refObject;

	CComPtr<IInitializeOther> m_bootstrap;
	CComPtr<IMoniker> m_mkPersistor; // the one object this object names itself
	CComPtr<IMCCAllPointsContainer> m_defaultSiteSinks; // wotta hack.

	// see IMontagePersistor::ForgetSubject
	bool m_amRoot;

	// the contained objects are in layers:
	layerlist m_layers; // front is front
	layerlist m_unlayers; // layers that don't yet have Z
	Layer *m_nonlayer; // initial Layer for sites (through which we keep a ref on all non-deleted sites so we can delete them on ~Container)

	long m_state; // OLE state
	HWND m_hWnd, // when in-place active
        m_intoWnd; // if ActivateInto called
	bool m_ownwindow; // if ActivateAs called, this is false
	CComPtr<IUnknown> m_pSite; // where I am runnin
	CComPtr<IOleInPlaceSite> m_pInPlaceSite; // where active
	CComPtr<IOleInPlaceFrame> m_pFrame; // of my container
	CComPtr<IOleInPlaceUIWindow> m_pDoc; //     "
	OLEINPLACEFRAMEINFO m_frameInfo;
	CComPtr<IMCCTransform> m_transform;
	CComPtr<IMCCToolPanel> m_toolpanel;

	CLSID m_clsid; // what I should say in IPersistStorage
	CComPtr<IMCCFileFormat> m_fileport; // file im/export for PersistFile and Insert Contents of File
	wstring m_exts, m_filename;

	CComPtr<IHlinkBrowseContext> m_browseContext;
	CComPtr<IDispatchEx> m_stuff;

	CComPtr<IGroup> m_group; // repository
	CComPtr<IAdviseSink> m_pViewSink; // for view changes
	CComPtr<IDataAdviseHolder> m_dadvise; // holder of advisory connections
	CComPtr<IOleAdviseHolder> m_oledvise; // as well

    // the data to expose via IDataObject
    CComPtr<IMCCDataObject> m_cache;
    // how to create that data
	CComPtr<IMCCItemSetRenderer> m_renderer;
	HRESULT makeRenderer(); // hack in IPersistStorage

	// ay, more hacks
	bool setExtent(SIZEL s); // in IOleObject
	HRESULT getExtentFromRenderer(SIZEL *ret); // get it from how much canvas actually filled (expensive)

	// menus
	sitelist m_menusites; // sites of controls that have menus up
	HMENU m_menu; // composite menu
	OLEMENUGROUPWIDTHS m_widths; // ...
	HOLEMENU m_holeMenu; // and its ole descriptor


    // map WM_COMMANDs to the sites of ctrls that want the msgs
    typedef map<int,site *> menumap;
	menumap m_menumap;
    // or to arbitrary objects..
	typedef map<int,IWindowMessageHandler *> messmap;
    messmap m_commandMap, m_notifyMap;

    // IMCCSite::SetTimer support
    map<UINT,site*> m_timers;

	// drop state
	CComPtr<IDataObject> m_dragdata; // data to be dropped
	CComPtr<IDropTarget> m_dragtarget; // contained object we're negociating drag for

	// mnemonics
	mnemon_info m_mnemon;

	site *m_pCaptureObject,
	    *m_pFocusObject;
    bool m_haveFocus;
	CComQIPtr<IOleInPlaceObject> m_pUIActiveObject;
	bool m_haveUI; // whether Montage UI is currently installed

	// hatching (activation border)
	Hatchling *m_pHatchling;
	long m_hatchMode;

	// coordinates
	RECT m_rect, m_cliprect; // span of the view rect in the container; maximum area we can use
	RECT m_extraArea,m_reservedArea; // how much this differs from in-place rect; how much smaller content is than full
	SIZEL m_extent,m_minExtent; // size in container, & minimal
	bool m_autoSize; // whether m_extent is set from canvas boundary
	int m_zoom; // ratio between internal canvas size and extent we tell container
		// expressed in parts per myriad (10000=100%)
	// the transform uses these; not used directly by Montage
	RECTL m_margin;
	bool m_alwaysMargin;

	// in-thread trigger
	vector<IMCCInThreadTrigger*> m_triggers;

	// dirtiness
    // whether there's data to OnDataChange, whether data to save.
    // both are set by dirty(); m_untolddata gets reset when container sends data (sayChange)
    // m_dirty gets reset in IPersistStorage::Save
	bool m_untolddata,m_dirty;
    HRESULT dirtyMsg(); // post a MONTAGE_NOTICE_CHANGES message to self that it's time to OnDataChange
	HRESULT dirty(); // set both flags and if that's news, call dirtyMsg
	HRESULT sayChange(); // the actual event firer, triggered by MONTAGE_NOTICE_CHANGES, not to be called directly

	// drawing
	HDC m_currentDC,m_bufferDC;
	RECT m_currentRC;
	HDC startBuffer(HDC hdc,RECT *r,bool copy);
	HDC blitBuffer();

	// one-time (!)
	bool m_skipPaint; // flicker-free activation.

	// receiver of windows messages.
	static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

// window coords rect
	RECT rect() {
		// (equiv. to GetClientRect(m_hWnd) when active)
		RECT bpe = boundsPlusExtra();
		RECT ret = {0,0,wid(bpe),hei(bpe)};
		return ret;
	}
// rectangle of window in container
	RECT boundsPlusExtra() {
		if(m_rect==m_cliprect) // special case: if no space given, don't take any!  Unclear how this is supposed to work in general.
			return m_rect;
		else
			return m_rect+m_extraArea;
	}
// area for transformed things
	RECT contentRect() {
		return rect() - m_reservedArea;
	}
	void _InvalidateRect(LPCRECT lprc,BOOL erase = TRUE);
	void InvalidateSite(site *pSite);
// before active, m_rect must be estimated from extent
    HRESULT fakeRectFromExtent();
// call IOleInPlaceSite::GetWindowContext and do appropriate things.
	HRESULT getInPlaceWindowContext();
// helpers to resize and reclip window
	RECT windowRect();
    HRGN clipRgn();
// internal version
    STDMETHODIMP setObjectRects(LPCRECT lprcPosRect,LPCRECT lprcClipRect,bool setExtent,bool moveWindow,bool setClip);
// when resizing, either grapples or IOleDocumentView::SetRect, must update extent too
	HRESULT setExtentFromRect(const RECT &rcNew, bool useCurrent); // flag: whether to use current m_rect:m_extent ratio
// ah good ol' Windows.
	HRESULT makewindow(bool show);
// clear background w/ ambient bk color
	void erase(HDC hdc,RECT *r);
// the real draw method
	enum howRefType {irrel,upTo,frontOf};
	// couldn't get template version of this to work
	// This is actually nicer - well-defined.
	struct drawTransformBase {
		const bool drawUntransformed;
		drawTransformBase(bool du) : drawUntransformed(du) {}
		virtual HRESULT getPos(site *s,RECTL *ret) const = 0;
	};
	HRESULT draw(const drawTransformBase &t,HDC hdc,LPCRECTL bounds,LPCRECTL wbounds,
		LPCRECTL areaHint = 0, // what part isn't just going to be clipped
		site *ref = 0, enum howRefType how = irrel); // to draw everything behind or in front of a site
    void changeFocus();
// events
	void sizeEvent();
	void extentEvent();
	void groupEvent(bool start);
// calls approachstate(m_intent) on all sites.
	HRESULT pullsites();
///////////////////
// internal interface to change OLE state
	HRESULT hide();
	HRESULT show();
	HRESULT inplaceactivate();
	HRESULT inplacedeactivate();
	HRESULT uiactivate();
	HRESULT uideactivate();
	// and a fn to shift between 'em
	HRESULT chstate(long intent);
// worker for the dragdrop functions
	HRESULT drag(DWORD grfKeyState,POINTL pt,DWORD *retEffect);
// internal UI fns
	HRESULT createmenus();
	HRESULT placemenus();
	HRESULT killmenus();
	HRESULT installUI(); // menus+toolbar
	HRESULT removeUI();

	Container();
	void readPrefs();
	~Container();
	// finalconstruct qis on m_points, which causes refcount fluctuation
	DECLARE_PROTECT_FINAL_CONSTRUCT()
	HRESULT FinalConstruct();
	void FinalRelease();

///////////////////////////////
// internal "interface" for sites to call in.
	//HRESULT insert(site *s); // yep.
	HRESULT remove(site *s); // uh huh.
	HRESULT show(site *s);
	HRESULT hide(site *s);
	HRESULT couiactivate(site *s);
	HRESULT couideactivate(site *s);
	HRESULT before(site *s,site *other = 0);
	HRESULT after(site *s,site *other = 0);
	HRESULT beforeLayer(Layer *l, Layer *other = 0); // same for layers
	HRESULT afterLayer(Layer *l, Layer *other = 0);

// internal notification:
	bool bounds_changed();

	DECLARE_GET_CONTROLLING_UNKNOWN();

//DEBUG_REFCOUNT_IMPL(Container);

BEGIN_CATEGORY_MAP(Container)
	IMPLEMENTED_CATEGORY(CATID_Insertable)
	IMPLEMENTED_CATEGORY(CATID_Control)
	IMPLEMENTED_CATEGORY(CATID_Programmable)
	IMPLEMENTED_CATEGORY(CATID_SafeForScripting)
	IMPLEMENTED_CATEGORY(CATID_SafeForInitializing)
	IMPLEMENTED_CATEGORY(CATID_DocObject)
END_CATEGORY_MAP();
////////////////////////
// INTERFACES
//
BEGIN_COM_MAP(Container)
	COM_INTERFACE_ENTRY(IMontageContainer)
	COM_INTERFACE_ENTRY(DIMontageContainer)
	COM_INTERFACE_ENTRY2(IDispatch,IDispatchEx)
	COM_INTERFACE_ENTRY(IDispatchEx)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	//COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IOleObject)
    COM_INTERFACE_ENTRY(IObjectWithSite)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2) // in IViewObjectEx
	COM_INTERFACE_ENTRY(IViewObject) // in IViewObject2
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY2(IPersist,IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IOleCache)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow,IOleInPlaceObject)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IPointerInactive)
	COM_INTERFACE_ENTRY(IOleDocument)
	COM_INTERFACE_ENTRY(IOleDocumentView)
	COM_INTERFACE_ENTRY(IPersistFile)
	COM_INTERFACE_ENTRY(IOleContainer)
	COM_INTERFACE_ENTRY(IOleItemContainer)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IConnectionPointContainer,m_points.p)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMCCAllPointsContainer,m_points.p)
	//COM_INTERFACE_ENTRY(IRunnableObject)
	//COM_INTERFACE_ENTRY(IHlinkTarget)
END_COM_MAP()

	static HRESULT WINAPI UpdateRegistry(BOOL bRegister);

// ISupportsErrorInfo
	STDMETHODIMP InterfaceSupportsErrorInfo(REFIID riid);

// IMontageContainer
	STDMETHODIMP get_ClassID(CLSID *ret);
	STDMETHODIMP put_ClassID(REFCLSID val);
	STDMETHODIMP get_RefObject(DIMontageContainerRef **ret);
	STDMETHODIMP put_Group(IGroup *val);
	STDMETHODIMP GetMainGroup(REFIID riid, void **ret);
	STDMETHODIMP get_Bootstrapper(IInitializeOther **ret);
	STDMETHODIMP put_Bootstrapper(IInitializeOther *val);
	STDMETHODIMP get_RelMoniker(IMoniker **ret);
	STDMETHODIMP put_RelMoniker(IMoniker *val);
	STDMETHODIMP get_Transform(IMCCTransform **ret);
	STDMETHODIMP put_Transform(IMCCTransform *val);
	STDMETHODIMP get_ToolPanel(IMCCToolPanel **ret);
	STDMETHODIMP put_ToolPanel(IMCCToolPanel *val);
	STDMETHODIMP get_FilePort(IMCCFileFormat **ret);
	STDMETHODIMP put_FilePort(IMCCFileFormat *val);
	STDMETHODIMP get_Stuff(IDispatchEx **ret);
    STDMETHODIMP SetAreas(RECT extra,RECT reserved);
	STDMETHODIMP get_ExtraArea(RECT *ret);
	STDMETHODIMP get_ReservedArea(RECT *ret);
	STDMETHODIMP get_Size(PSIZE ps);
	STDMETHODIMP get_State(long *ret);
    STDMETHODIMP GetFocus(BOOL *any,IMCCSite **ret);
	STDMETHODIMP put_MinExtent(SIZEL val);
	STDMETHODIMP get_MinExtent(SIZEL *ret);
	STDMETHODIMP put_AutoSize(VARIANT_BOOL val);
	STDMETHODIMP get_AutoSize(VARIANT_BOOL *ret);
	STDMETHODIMP put_Zoom(long val);
	STDMETHODIMP get_Zoom(long *ret);
	STDMETHODIMP GetZoomedExtent(SIZEL *ret);
	STDMETHODIMP put_Margin(RECTL val);
	STDMETHODIMP get_Margin(RECTL *ret);
	STDMETHODIMP put_AlwaysMargin(VARIANT_BOOL val);
	STDMETHODIMP get_AlwaysMargin(VARIANT_BOOL *ret);
	STDMETHODIMP get_UseMargin(VARIANT_BOOL *ret);
	STDMETHODIMP new_Layer(IMCCLayer **ret);
	STDMETHODIMP new_Site(IMCCSite **ret);
	STDMETHODIMP EnumLayers(IEnumMCCLayers **ret);
	STDMETHODIMP get_DefaultSiteSinks(IMCCAllPointsContainer **ret);
	STDMETHODIMP ActivateAs(HWND hWnd);
	STDMETHODIMP ActivateInto(HWND hWnd,VARIANT_BOOL show);
	STDMETHODIMP get_HatchMode(long *ret);
	STDMETHODIMP put_HatchMode(long val);
	STDMETHODIMP RequestTrigger(IMCCInThreadTrigger *callback);
    STDMETHODIMP RegisterMessageHandler(BOOL isCommand, UINT msg,IWindowMessageHandler *handler);
    STDMETHODIMP ForgetHandler(BOOL isCommand, UINT msg);
    STDMETHODIMP TransformChanged(VARIANT_BOOL offsetChanged,VARIANT_BOOL scaleChanged);
// DIMontageContainer
	STDMETHODIMP get_Group(DIGroup **ret);
	STDMETHODIMP put_Group(DIGroup *val);
	STDMETHODIMP get_Bootstrapper(IUnknown **ret);
	STDMETHODIMP put_Bootstrapper(IUnknown *val);
	STDMETHODIMP get_RelMoniker(IUnknown **ret);
	STDMETHODIMP put_RelMoniker(IUnknown *val);
	STDMETHODIMP get_Transform(IUnknown **ret);
	STDMETHODIMP put_Transform(IUnknown *val);
	STDMETHODIMP get_ToolPanel(IUnknown **ret);
	STDMETHODIMP put_ToolPanel(IUnknown *val);
	STDMETHODIMP get_Stuff(IDispatch **ret);
	STDMETHODIMP SetAreas(long eleft, long etop, long eright, long ebottom,long rleft, long rtop, long rright, long rbottom);
	STDMETHODIMP GetExtraArea(long *left, long *top, long *right, long *bottom);
	STDMETHODIMP GetReservedArea(long *left, long *top, long *right, long *bottom);
	//STDMETHODIMP get_Size(PSIZE ps);
	// use custom version
	STDMETHODIMP get_State(int *ret);
	STDMETHODIMP NewLayer(DIMCCLayer **ret);
	STDMETHODIMP NewSite(DIMCCSite **ret);
	STDMETHODIMP get_Layers(IDispatch **ret);
	//STDMETHODIMP get_DefaultSiteSinks(IMCCAllPointsContainer **ret);
	//STDMETHODIMP ActivateAs(HWND hWnd);
// IDispatch
	STDMETHODIMP GetTypeInfoCount(UINT * pctinfo);
	STDMETHODIMP GetTypeInfo(UINT iTInfo,LCID lcid,ITypeInfo ** ppTInfo);
	STDMETHODIMP GetIDsOfNames(REFIID riid,LPOLESTR * rgszNames,UINT cNames,
		LCID lcid,DISPID * rgDispId);
	STDMETHODIMP Invoke(DISPID dispIdMember,REFIID riid,LCID lcid,WORD wFlags,
		DISPPARAMS * pDispParams,VARIANT * pVarResult,EXCEPINFO * pExcepInfo,UINT * puArgErr);
// -Ex
	STDMETHODIMP GetDispID(BSTR bstrName,DWORD grfdex,DISPID *pid);
	STDMETHODIMP InvokeEx(DISPID id,LCID lcid,WORD wFlags,DISPPARAMS *pdp,VARIANT *pvarRes,
		EXCEPINFO *pei,IServiceProvider *pspCaller);
	STDMETHODIMP DeleteMemberByName(BSTR bstr, DWORD grfdex);
	STDMETHODIMP DeleteMemberByDispID(DISPID id);
	STDMETHODIMP GetMemberProperties(DISPID id,DWORD grfdexFetch,DWORD *pgrfdex);
	STDMETHODIMP GetMemberName(DISPID id,BSTR *pbstrName);
	STDMETHODIMP GetNextDispID(DWORD grfdex,DISPID id,DISPID *pid);
	STDMETHODIMP GetNameSpaceParent(IUnknown **ppunk);
// IOleObject
    STDMETHODIMP SetClientSite(IOleClientSite *pClientSite);
    STDMETHODIMP GetClientSite(IOleClientSite **ppClientSite);
    STDMETHODIMP SetHostNames(LPCOLESTR szContainerApp,LPCOLESTR szContainerObj);
    STDMETHODIMP Close(DWORD dwSaveOption);
    STDMETHODIMP SetMoniker(DWORD dwWhichMoniker,IMoniker *pmk);
    STDMETHODIMP GetMoniker(DWORD dwAssign,DWORD dwWhichMoniker,IMoniker **ppmk);
    STDMETHODIMP InitFromData(IDataObject *pDataObject,BOOL fCreation,DWORD dwReserved);
    STDMETHODIMP GetClipboardData(DWORD dwReserved,IDataObject **ppDataObject);
    STDMETHODIMP DoVerb(LONG iVerb,LPMSG lpmsg,IOleClientSite *pActiveSite,LONG lindex,HWND hwndParent,LPCRECT lprcPosRect);
    STDMETHODIMP EnumVerbs(IEnumOLEVERB **ppEnumOleVerb);
    STDMETHODIMP Update(void);
    STDMETHODIMP IsUpToDate(void);
    STDMETHODIMP GetUserClassID(CLSID *pClsid);
    STDMETHODIMP GetUserType(DWORD dwFormOfType,LPOLESTR *pszUserType);
    STDMETHODIMP SetExtent(DWORD dwDrawAspect,SIZEL *psizel);
    STDMETHODIMP GetExtent(DWORD dwDrawAspect,SIZEL *psizel);
    STDMETHODIMP Advise(IAdviseSink *pAdvSink,DWORD *pdwConnection);
    STDMETHODIMP Unadvise(DWORD dwConnection);
    STDMETHODIMP EnumAdvise(IEnumSTATDATA **ppenumAdvise);
    STDMETHODIMP GetMiscStatus(DWORD dwAspect,DWORD *pdwStatus);
    STDMETHODIMP SetColorScheme(LOGPALETTE *pLogpal);
// IObjectWithSite
	STDMETHODIMP SetSite(IUnknown *site);
	STDMETHODIMP GetSite(REFIID riid, void **ret);
//IOleCache
    STDMETHODIMP Cache(FORMATETC *pformatetc,DWORD advf,DWORD *pdwConnection);
    STDMETHODIMP Uncache(DWORD dwConnection);
    STDMETHODIMP EnumCache(IEnumSTATDATA **ppenumSTATDATA);
    STDMETHODIMP InitCache(IDataObject *pDataObject);
	// also in IDataObject.  This shouldn't get called.
    //STDMETHODIMP SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease);
// IViewObjectEx
	//(IViewObject)
    STDMETHODIMP Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
         DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdcDraw,LPCRECTL lprcBounds,
         LPCRECTL lprcWBounds,BOOL ( STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
         DWORD dwContinue);
    STDMETHODIMP GetColorSet(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
         DVTARGETDEVICE *ptd,HDC hicTargetDev,LOGPALETTE **ppColorSet);
    STDMETHODIMP Freeze(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
         DWORD *pdwFreeze);
    STDMETHODIMP Unfreeze(DWORD dwFreeze);
    STDMETHODIMP SetAdvise(DWORD aspects,DWORD advf,IAdviseSink *pAdvSink);
    STDMETHODIMP GetAdvise(DWORD *pAspects,DWORD *pAdvf,IAdviseSink **ppAdvSink);
    //(IViewObject2)
    STDMETHODIMP GetExtent(DWORD dwDrawAspect,LONG lindex,DVTARGETDEVICE *ptd,
         LPSIZEL lpsizel);
	//(IViewObjectEx)
    STDMETHODIMP GetRect(DWORD dwAspect,LPRECTL pRect);
    STDMETHODIMP GetViewStatus(DWORD *pdwStatus);
    STDMETHODIMP QueryHitPoint(DWORD dwAspect,LPCRECT pRectBounds,POINT ptlLoc,
         LONG lCloseHint,DWORD *pHitResult);
    STDMETHODIMP QueryHitRect(DWORD dwAspect,LPCRECT pRectBounds,LPCRECT pRectLoc,
         LONG lCloseHint,DWORD *pHitResult);
    STDMETHODIMP GetNaturalExtent(DWORD dwAspect,LONG lindex,DVTARGETDEVICE *ptd,
         HDC hicTargetDev,DVEXTENTINFO *pExtentInfo,LPSIZEL pSizel);
// IDataObject
    STDMETHODIMP GetData(FORMATETC *pformatetcIn,STGMEDIUM *pmedium);
    STDMETHODIMP GetDataHere(FORMATETC *pformatetc,STGMEDIUM *pmedium);
    STDMETHODIMP QueryGetData(FORMATETC *pformatetc);
    STDMETHODIMP GetCanonicalFormatEtc(FORMATETC *pformatectIn,FORMATETC *pformatetcOut);
    STDMETHODIMP SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease);
    STDMETHODIMP EnumFormatEtc(DWORD dwDirection,IEnumFORMATETC **ppenumFormatEtc);
    STDMETHODIMP DAdvise(FORMATETC *pformatetc,DWORD advf,IAdviseSink *pAdvSink,
         DWORD *pdwConnection);
    STDMETHODIMP DUnadvise(DWORD dwConnection);
    STDMETHODIMP EnumDAdvise(IEnumSTATDATA **ppenumAdvise);
//IOleInPlaceObject
	//(IOleWindow)
    STDMETHODIMP GetWindow(HWND *phwnd);
    STDMETHODIMP ContextSensitiveHelp(BOOL fEnterMode);
	//(IOleInPlaceObject)
    STDMETHODIMP InPlaceDeactivate(void);
    STDMETHODIMP UIDeactivate(void);
    STDMETHODIMP SetObjectRects(LPCRECT lprcPosRect,LPCRECT lprcClipRect);
    STDMETHODIMP ReactivateAndUndo(void);
//IOleInPlaceActiveObject
	STDMETHODIMP TranslateAccelerator(LPMSG lpmsg);
	STDMETHODIMP OnFrameWindowActivate(BOOL fActivate);
	STDMETHODIMP OnDocWindowActivate(BOOL fActivate);
	STDMETHODIMP ResizeBorder(LPCRECT prcBorder,IOleInPlaceUIWindow *pUIWindow,
		BOOL fFrameWindow);
	STDMETHODIMP EnableModeless(BOOL fEnable);
// IPointerInactive
    STDMETHODIMP GetActivationPolicy(DWORD *pdwPolicy);
    STDMETHODIMP OnInactiveMouseMove(LPCRECT pRectBounds,LONG x,LONG y,
		DWORD grfKeyState);
    STDMETHODIMP OnInactiveSetCursor(LPCRECT pRectBounds,LONG x,LONG y,
        DWORD dwMouseMsg,BOOL fSetAlways);

// IPersistStorage
	//(IPersist)
    STDMETHODIMP GetClassID(CLSID *pClassID);
	//(IPersistStorage)
    STDMETHODIMP IsDirty(void);
    STDMETHODIMP InitNew(IStorage *pStg);
    STDMETHODIMP Load(IStorage *pStg);
    STDMETHODIMP Save(IStorage *pStgSave,BOOL fSameAsLoad);
    STDMETHODIMP SaveCompleted(IStorage *pStgNew);
    STDMETHODIMP HandsOffStorage(void);

// Active Document (DocObject)
// IOleDocument
    STDMETHODIMP CreateView(IOleInPlaceSite *pIPSite,IStream *pstm,DWORD dwReserved,IOleDocumentView **ppView);
    STDMETHODIMP GetDocMiscStatus(DWORD *pdwStatus);
    STDMETHODIMP EnumViews(IEnumOleDocumentViews **ppEnum,IOleDocumentView **ppView);
// IOleDocumentView
    STDMETHODIMP SetInPlaceSite(IOleInPlaceSite *pIPSite);
    STDMETHODIMP GetInPlaceSite(IOleInPlaceSite **ppIPSite);
    STDMETHODIMP GetDocument(IUnknown **ppunk);
    STDMETHODIMP SetRect(LPRECT prcView);
    STDMETHODIMP GetRect(LPRECT prcView);
    STDMETHODIMP SetRectComplex(LPRECT prcView,LPRECT prcHScroll,LPRECT prcVScroll,LPRECT prcSizeBox);
    STDMETHODIMP Show(BOOL fShow);
    STDMETHODIMP UIActivate(BOOL fUIActivate);
    STDMETHODIMP Open(void);
    STDMETHODIMP CloseView(DWORD dwReserved);
    STDMETHODIMP SaveViewState(LPSTREAM pstm);
    STDMETHODIMP ApplyViewState(LPSTREAM pstm);
    STDMETHODIMP Clone(IOleInPlaceSite *pIPSiteNew,IOleDocumentView **ppViewNew);
// IPersistFile
	// IsDirty in IPersistStorage
	STDMETHODIMP Load(LPCOLESTR pszFileName,DWORD dwMode);
	STDMETHODIMP Save(LPCOLESTR pszFileName,BOOL fRemember);
	STDMETHODIMP SaveCompleted(LPCOLESTR pszFileName);
	STDMETHODIMP GetCurFile(LPOLESTR *ppszFileName);
// IOleItemContainer
	//(IParseDisplayName)
    STDMETHODIMP ParseDisplayName(IBindCtx *pbc,LPOLESTR pszDisplayName,
        ULONG *pchEaten,IMoniker **ppmkOut);
	//(IOleContainer)
    STDMETHODIMP EnumObjects(DWORD grfFlags,IEnumUnknown **ppenum);
    STDMETHODIMP LockContainer(BOOL fLock);
	//(IOleItemContainer)
	STDMETHODIMP GetObject(LPOLESTR	name,DWORD speed,IBindCtx *bctx,REFIID riid,void **ret);
	STDMETHODIMP GetObjectStorage(LPOLESTR name, IBindCtx *bctx,REFIID riid,void **ret);
	STDMETHODIMP IsRunning(LPOLESTR name);
// IDropTarget
    STDMETHODIMP DragEnter(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
        DWORD *pdwEffect);
    STDMETHODIMP DragOver(DWORD grfKeyState,POINTL pt,DWORD *pdwEffect);
    STDMETHODIMP DragLeave();
    STDMETHODIMP Drop(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
        DWORD *pdwEffect);
// IRunnableObject
	STDMETHODIMP GetRunningClass(LPCLSID lpClsid);
	STDMETHODIMP Run(LPBINDCTX pbc);
	STDMETHODIMP_(BOOL) IsRunning();
	STDMETHODIMP LockRunning(BOOL fLock,BOOL fLastUnlockCloses);
	STDMETHODIMP SetContainedObject(BOOL fContained);
// IHlinkTarget
	STDMETHODIMP SetBrowseContext(IHlinkBrowseContext * pihlbc);
    STDMETHODIMP GetBrowseContext(IHlinkBrowseContext ** ppihlbc);
    STDMETHODIMP Navigate(DWORD grfHLNF,LPCWSTR pwzJumpLocation);
    STDMETHODIMP GetMoniker(LPCWSTR pwzLocation,DWORD dwAssign,IMoniker ** ppimkLocation);
    STDMETHODIMP GetFriendlyName(LPCWSTR pwzLocation,LPWSTR * ppwzFriendlyName);
};
struct posBounds : Container::drawTransformBase {
	posBounds() : drawTransformBase(true) {}
	HRESULT getPos(site *s,RECTL *ret) const {
		*ret = rectl(s->m_windRect);
		return S_OK;
	}
};
#endif CONTAINER_H
