/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ContainerEditMode.cpp : Implementation of ContainerEditMode
#include "stdafx.h"
#include "ContainerEditMode.h"
#include "getSites.h"
#include "combinePos.h"
#include "scrollWay.h"
#include "moreW2A.h"

const int FRAMESIZE = 8;

ContainerEditMode::ContainerEditMode() :
	m_bTracking(false), // no button is down
	m_amsource(false), // am not currently dragdrop source
	m_allowDrop(true),
	m_defaultDropEffect(DROPEFFECT_MOVE),
	m_allowedDragEffects(DROPEFFECT_COPY|DROPEFFECT_MOVE), // allow all we can do at first
	m_allowedDropEffects(DROPEFFECT_COPY|DROPEFFECT_MOVE),
	m_accel(0), // must build accelerators
	m_editMenu(0), // and menus
	m_viewMenu(0),
	m_insertMenu(0),
	m_zoomMenu(0),
	m_scrollTimeout(0),
	m_bRectSelect(false)
{
	CONSTRUCTORTRACE(ContainerEditMode);
	USES_CONVERSION;
	std::fill(m_recentClasses,m_recentClasses+MAX_RECENT,CLSID_NULL);
	CRegKey options;
	if(!options.Open(HKEY_CURRENT_USER,"Software\\AT&T\\Montage\\Edit Mode")) {
		CRegKey recent;
		if(!recent.Open(options,"Recent Classes")) {
			TCHAR valueName[10],classString[50];
			int i;
			for(i=0;i<MAX_RECENT;++i) {
				wsprintf(valueName,"Class%d",i+1);
				DWORD n = 50;
				if(recent.QueryValue(valueName,0,classString,&n))
					continue; // entries can be missing, e.g. if someone like me's been messing with reg.
				RUN(CLSIDFromString(T2OLE(classString),&m_recentClasses[i]));
				m_classOrder.push_back(i);
			}
		}

	}
}
ContainerEditMode::~ContainerEditMode() {
	DESTRUCTORTRACE(ContainerEditMode);
	if(m_accel)
		GlobalFree(m_accel);
	USES_CONVERSION;
	CRegKey options;
	CHECK(!options.Create(HKEY_CURRENT_USER,"Software\\AT&T\\Montage\\Edit Mode"));
	CRegKey recent;
	CHECK(!recent.Create(options,"Recent Classes"));
	// write them out in order so we don't have to save order.
	int j(1);
	for(vector<int>::iterator i = m_classOrder.begin();i!=m_classOrder.end();++i) {
		OLECHAR classString[50];
		RUN(StringFromGUID2(m_recentClasses[*i],classString,50));
		TCHAR valueName[10];
		wsprintf(valueName,"Class%d",j++);
		CHECK(!recent.SetValue(valueName,REG_SZ,OLE2T(classString),wcslen(classString)+1));
	}
}
// IContainerEditMode
STDMETHODIMP ContainerEditMode::Initialize(IItemSet *selection, IMCCItemSetActivator *activator,
		IMCCLayer *mainlayer, IMCCDataObjectUnpacker *unpacker) {
    METHTRACE(ContainerEditMode,Initialize);
	if(m_selection)
		return S_FALSE;
	m_selection = selection;
	m_activator = activator;
	m_mainl = mainlayer;
	m_unpacker = unpacker;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::PasteData(IDataObject *src,MCC_POS_INFO *mpi) {
    METHTRACE(ContainerEditMode,PasteData);
	RUN(insertEvent(src,mpi));
	return S_OK;
}
STDMETHODIMP ContainerEditMode::get_AllowDrop(VARIANT_BOOL *ret) {
    METHTRACE(ContainerEditMode,get_AllowDrop);
	*ret = m_allowDrop?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::put_AllowDrop(VARIANT_BOOL val) {
    METHTRACE(ContainerEditMode,put_AllowDrop);
	m_allowDrop = val;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::put_DefaultDropEffect(DWORD val) {
    METHTRACE(ContainerEditMode,put_DefaultDropEffect);
	if(val!=DROPEFFECT_COPY && val!=DROPEFFECT_MOVE)
		ERR(E_INVALIDARG);
	m_defaultDropEffect = val;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::get_DefaultDropEffect(DWORD *ret) {
    METHTRACE(ContainerEditMode,get_DefaultDropEffect);
	*ret = m_defaultDropEffect;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::put_AllowedDragEffects(DWORD val) {
    METHTRACE(ContainerEditMode,put_AllowedDragEffects);
	m_allowedDragEffects = val;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::get_AllowedDragEffects(DWORD *ret) {
    METHTRACE(ContainerEditMode,get_AllowedDragEffects);
	if(!ret)
		ERR(E_POINTER);
	*ret = m_allowedDragEffects;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::put_AllowedDropEffects(DWORD val) {
    METHTRACE(ContainerEditMode,put_AllowedDropEffects);
	m_allowedDropEffects = val;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::get_AllowedDropEffects(DWORD *ret) {
    METHTRACE(ContainerEditMode,get_AllowedDropEffects);
	if(!ret)
		ERR(E_POINTER);
	*ret = m_allowedDropEffects;
	return S_OK;
}
STDMETHODIMP ContainerEditMode::AddContextItem(VARIANT_BOOL empty, // if for empty selection
		BSTR text, int cmd, HMENU subMenu) {
    METHTRACE(ContainerEditMode,AddContextItem);
	USES_CONVERSION;
	ciList &itemList = empty==VARIANT_TRUE?emptyItems:fullItems;
	itemList.push_back(contextItem(string(W2A(text)),cmd,subMenu));
	return S_OK;
}
STDMETHODIMP ContainerEditMode::RemoveContextItem(VARIANT_BOOL empty, int cmd) {
    METHTRACE(ContainerEditMode,RemoveContextItem);
	USES_CONVERSION;
	ciList &itemList = empty==VARIANT_TRUE?emptyItems:fullItems;
	for(ciList::iterator ili = itemList.begin(); ili!=itemList.end(); ++ili)
		if(cmd==(*ili).id) {
			itemList.erase(ili);
			return S_OK;
		}
	ERR(E_FAIL);
}

// DIContainerEditMode
STDMETHODIMP ContainerEditMode::Initialize(DIItemSet *selection, IUnknown *activator,
		DIMCCLayer *mainlayer, IUnknown *unpacker) {
    METHTRACE(ContainerEditMode,Initialize);
	CComPtr<IItemSet> is2;
	if(selection)
		RUNRET(selection->QueryInterface(riidret(&is2)));
	CComPtr<IMCCItemSetActivator> a2;
	if(activator)
		RUNRET(activator->QueryInterface(riidret(&a2)));
	CComPtr<IMCCLayer> l2;
	if(mainlayer)
		RUNRET(mainlayer->QueryInterface(riidret(&l2)));
	CComPtr<IMCCDataObjectUnpacker> u2;
	if(unpacker)
		RUNRET(unpacker->QueryInterface(riidret(&u2)));
	return Initialize(is2,a2,l2,u2);
}
STDMETHODIMP ContainerEditMode::put_AllowedDragEffects(long val) {
    METHTRACE(ContainerEditMode,put_AllowedDragEffects);
	return put_AllowedDragEffects(DWORD(val));
}
STDMETHODIMP ContainerEditMode::get_AllowedDragEffects(long *ret) {
    METHTRACE(ContainerEditMode,get_AllowedDragEffects);
	return get_AllowedDragEffects((DWORD*)ret);
}
STDMETHODIMP ContainerEditMode::put_AllowedDropEffects(long val) {
    METHTRACE(ContainerEditMode,put_AllowedDropEffects);
	return put_AllowedDropEffects(DWORD(val));
}
STDMETHODIMP ContainerEditMode::get_AllowedDropEffects(long *ret) {
    METHTRACE(ContainerEditMode,get_AllowedDropEffects);
	return get_AllowedDropEffects((DWORD*)ret);
}
extern CComBSTR g_DragScrollInset,g_DragScrollDelay,g_DragScrollInterval;
// IOleObjectImpl override
STDMETHODIMP ContainerEditMode::SetClientSite(IOleClientSite *val) {
	METHTRACE(ContainerEditMode,SetClientSite);
	if(val) {
		CComPtr<IOleContainer> oc;
		RUN(val->GetContainer(&oc));
        CComPtr<IMontageContainer> mc;
        RUN(oc.QueryInterface(&mc));
        RUN(mc->get_RefObject(&m_mref));
		CComDispatchDriver amb(mc);
		CComVariant v;
		RUN(amb.GetPropertyByName(g_DragScrollInset,&v));
		v.ChangeType(VT_I4);
		m_scrollInset = v.lVal;
		RUN(amb.GetPropertyByName(g_DragScrollDelay,&v));
		v.ChangeType(VT_I4);
		m_scrollDelay = v.lVal;
		RUN(amb.GetPropertyByName(g_DragScrollInterval,&v));
		v.ChangeType(VT_I4);
		m_scrollInterval = v.lVal;
	}
	else
		m_mref.Release();
	return IOleObjectImpl<ContainerEditMode>::SetClientSite(val);
}

CComPtr<IGroup> ContainerEditMode::mainGroup() {
	METHTRACE(ContainerEditMode,mainGroup);
	CComPtr<IGroup> ret;
	if(m_mref)
		mc()->GetMainGroup(riidret(&ret));
	return ret;
}
CComPtr<IItemSet> ContainerEditMode::dataSet() {
    METHTRACE(ContainerEditMode,dataSet);
	CComPtr<IItemSet> dataset;
	RUN(ItemMoniker(L"Data")->BindToObject(BindCtx(),PointerMoniker(mainGroup()),riidret(&dataset)));
	return dataset;
}

HRESULT ContainerEditMode::add(bool toggle, IMCCSite *s) {
	METHTRACE(ContainerEditMode,add);
    HRESULT (__stdcall IMCCSelectable::*f)(IItemSet*) = toggle?&IMCCSelectable::Toggle:&IMCCSelectable::Add;
	RUN(fireSiteEvent(s,fireOneParam<IMCCSelectable,IItemSet*>(f,m_selection)));
	return S_OK;
}
HRESULT ContainerEditMode::check(IMCCSite *s) {
	METHTRACE(ContainerEditMode,check);
	// this isn't quite right because of the overlap between S_FALSE not-selected
	// and S_FALSE not-handled
	return fireSiteEvent(s,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Check,m_selection));
}
HRESULT ContainerEditMode::removeall() {
	METHTRACE(ContainerEditMode,removeall);
	RUN(m_selection->Clear());
	if(m_activator)
		RUN(m_activator->Stop());
	// grab focus again
	RUN(m_spInPlaceSite->SetFocus(true));
	return S_OK;
}
// quite an annoyance: objects don't always expose IPersist itself
const IID *persistXXX[] = {&IID_IPersist,&IID_IPersistStream,&IID_IPersistStreamInit,&IID_IPersistStorage};
const nPersist = 4;
HRESULT getCLSID(IUnknown *o,CLSID *ret) {
	CComPtr<IPersist> p;
	for(int i=0;i<nPersist;++i)
		if(SUCCEEDED(o->QueryInterface(*persistXXX[i],(void**)&p)))
			return p->GetClassID(ret);
	SERR(E_FAIL);
}
HRESULT ContainerEditMode::packageOne(IDataObject *DO,IMCCSite *s,POINT pt) {
	METHTRACE(ContainerEditMode,packageOne);

	// get embedded o's IUnknown
	CComPtr<IUnknown> item;
	RUN(s->GetControl(riidret(&item)));

	// get canvas item pos
	RECTL pos;
	RUN(s->get_VirtRect(&pos));
	// canvas mouse pos
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	POINTL ptl;
	RUN(t->W2CP(pt,&ptl));

	// replace renderer's OBJECTDESCRIPTOR
	RUN(CComQIPtr<IMCCDataObject>(DO)->SetDelayedData(g_clipformats.objectdescriptor,0));

	HGLOBAL hDesc = GlobalAlloc(GHND,sizeof(OBJECTDESCRIPTOR));
	OBJECTDESCRIPTOR *pod = (OBJECTDESCRIPTOR*)GlobalLock(hDesc);
	pod->cbSize = sizeof(OBJECTDESCRIPTOR);
	RUN(getCLSID(item,&pod->clsid));
	pod->dwDrawAspect = DVASPECT_CONTENT;
	pod->sizel = size(wid(pos),hei(pos));
	pod->pointl = pointl(ptl.x-pos.left,pos.top-ptl.y);
	if(CComQIPtr<IOleObject> oo = item)
		RUN(oo->GetMiscStatus(DVASPECT_CONTENT,&pod->dwStatus));
	GlobalUnlock(hDesc); pod = 0;
	FormatEtc fetcOD(g_clipformats.objectdescriptor);
	StgMedium stgm;
	stgm.tymed = TYMED_HGLOBAL;
	stgm.hGlobal = hDesc;
	RUN(DO->SetData(&fetcOD,&stgm,true));

	// save item to data object
	FormatEtc fetcEmb(g_clipformats.embedded);
	stgm.pstm = 0;
	CComPtr<IPersistStream> pm;
	CComPtr<IPersistStorage> pg;
	if(SUCCEEDED(item->QueryInterface(riidret(&pm)))
		|| SUCCEEDED(item->QueryInterface(IID_IPersistStreamInit,(void**)&pm))) {
		// (I don't think anyone accepts streamed embedded objects)
		fetcEmb.tymed = stgm.tymed = TYMED_ISTREAM;
		RUN(CreateStreamOnHGlobal(0,true,&stgm.pstm));
		CLSID clsid;
		RUN(pm->GetClassID(&clsid));
		RUN(WriteClassStm(stgm.pstm,clsid));
		RUN(pm->Save(stgm.pstm,false));
	}
	else if(SUCCEEDED(item->QueryInterface(riidret(&pg)))) {
		fetcEmb.tymed = stgm.tymed = TYMED_ISTORAGE;
		CComPtr<ILockBytes> bytes;
		RUN(CreateILockBytesOnHGlobal(0,true,&bytes));
		RUN(StgCreateDocfileOnILockBytes(bytes,STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_CREATE,0,&stgm.pstg));
		RUN(OleSave(pg,stgm.pstg,false));
	}
	if(stgm.pstm) {
		RUN(DO->SetData(&fetcEmb,&stgm,true));

		// put Embedded Object just ahead of Embed Source
		RUN(CComQIPtr<IMCCDataObject>(DO)->Prioritize(g_clipformats.embedded,101));
	}
	return S_OK;
}
HRESULT ContainerEditMode::package(bool continual,POINT pt,IDataObject **retDO,IMCCItemSetRenderer **retRend) {
	METHTRACE(ContainerEditMode,package);

	// so here's how you package some Montage stuff up in a data object
	// MCCItemSetRenderer creates most of the data
	// MCCDataObject serves as a buffer and holds extra data
	CLSID clsid;
	RUN(mc()->get_ClassID(&clsid));

	// make a data object (I bet the standard OLE cache could work too)
	CComPtr<IDataObject> DO;
	SRUN(CREATE(DO,CLSID_MCCDataObject));
	CComQIPtr<IMCCDataObject> mccDO = DO;

	// create an ItemSetRenderer to render metafile, storage & objectdescriptor on demand
	CComPtr<IMCCItemSetRenderer> renderer;
	SRUN(CREATE(renderer,CLSID_MCCItemSetRenderer));
	SRUN(renderer->Initialize(mainGroup(),m_selection,clsid,mc(),mccDO));
	SRUN(renderer->put_MousePoint(pt));

	// renderer created data to instantiate Montage containing a copy of the selection
	// if there's just one in the selection we also want to offer that alone
	long count;
	RUN(m_selection->get_Count(&count));
	if(count==1) {
		VS *vs = getSites(mainGroup(),m_selection);
		_ASSERTE(vs);
		if(!vs||vs->size()!=1)
			ERR(E_UNEXPECTED);
		CComPtr<IMCCSite> s = (*vs)[0];
		delete vs;
		RUN(packageOne(DO,s,pt));
	}

	if(!continual)
		SRUN(mccDO->Fix(0));

	DO.CopyTo(retDO);
	renderer.CopyTo(retRend);
	return S_OK;
}
HRESULT ContainerEditMode::insertEvent(IDataObject *data, MCC_POS_INFO *mpi) {
	METHTRACE(ContainerEditMode,insertEvent);

    CComPtr<IMontageContainer> MC = mc();
	RUN(groupOp(MC,true));
	// unpack the data into local data set
	CComPtr<IEnumUnknown> eu;
	if(FAILED(m_unpacker->UnpackMany(data,0,dataSet(),&eu)))
		return S_FALSE;
	// layout annotators
	CComPtr<IEnumConnections> ecLayout = getSinks(MC,IID_IMCCSiteLayoutEvents);
	// bind context
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	// transform
	CComPtr<IMCCTransform>t;
	RUN(MC->get_Transform(&t));
	// relocate
	unk_iterator i;
	for(i = unk_iterator(eu);i!=unk_iterator();++i) {
		if(CComQIPtr<IMCCSitePersistor> sp = *i) {
			// position object
			MCCSITEPERSISTORDATA data;
			if(FAILED(sp->get_Data(&data))) {
				data.z = 0; // on top
				data.pos.postype = MCC_POS_NONE; // caught below
				data.state = MCCS_SHOWN;
			}
			if(!mpi)
				data.pos.postype = MCC_POS_NONE;
			else if(data.pos.postype!=MCC_POS_NONE)
				RUN(combine_pos(t,*mpi,data.pos));
			else
				data.pos = *mpi;
			RUN(sp->put_Data(&data));
			// tell layout annotators
			if(ecLayout) {
				RUN(fireEventOnEC(ecLayout,fireTwoParams<IMCCSiteLayoutEvents,IMCCSitePersistor*,LPOLESTR>
					(&IMCCSiteLayoutEvents::Process,sp,0)));
				RUN(ecLayout->Reset());
			}
		}
	}

	RUN(eu->Reset());
	// realize
	for(i = unk_iterator(eu);i!=unk_iterator();++i)
		if(CComQIPtr<IGroupItem> gi = *i)
			RUN(gi->Realize(bctx));
	RUN(groupOp(MC,false));
	return S_OK;
}
HRESULT ContainerEditMode::queryEvent(IDataObject *data) {
	METHTRACE(ContainerEditMode,queryEvent);
	return m_unpacker->Query(data);
}
// ICoUIActiveObject
HRESULT getTransformManager(IMontageContainer *mc, DIMCCTransformManager **ret) {
    FUNTRACE(getTransformManager);
	// hack: shouldn't use name here
	CComPtr<IBindCtx> bctx;
	SRUN(CreateBindCtx(0,&bctx));
	CComPtr<IMoniker> mkM;
	ItemMoniker mkTManagerP(L"Transform Manager");
	SRUN(FoolishCreatePointerMoniker(mc,&mkM));
	CComPtr<DIPersistor> tmP;
	SRUN(mkTManagerP->BindToObject(bctx,mkM,riidret(&tmP)));
	CComPtr<IUnknown> uTM;
	SRUN(tmP->get_Subject(&uTM));
	SRUN(uTM.QueryInterface(ret));
	return S_OK;
}
MenuItem g_menuItem;
LPTSTR g_propertiesName = _T("Properties"),
	g_cutName = _T("Cu&t\tCtrl-X"),
	g_copyName = _T("&Copy\tCtrl-C"),
	g_pasteName = _T("&Paste\tCtrl-V"),
	g_deleteName = _T("&Delete\tDel"),
	g_selectAllName = _T("Select &All\tCtrl-A");
void dozoom(int z,HMENU menu,IMCCSite *s) {
	char buf[10];
	wsprintf(buf,"%d%%",z);
	g_menuItem.insert(menu,buf,CMD_ZOOM(z),-1,0,true);
	SRUN(s->RegisterCommand(CMD_ZOOM(z)));
}
int g_zooms[] = {10,25,33,50,75,100,125,150,200,300,400},
	n_zooms = sizeof g_zooms/sizeof g_zooms[0];
STDMETHODIMP ContainerEditMode::Activate(HMENU hMenu,OLEMENUGROUPWIDTHS*pWidths) {
	METHTRACE(ContainerEditMode,InsertMenus);
	USES_CONVERSION;
	_ASSERTE(!m_insertMenu && !m_editMenu && !m_viewMenu && !m_zoomMenu);
	// Edit, View, Insert at left of menu section 1
	m_insertMenu = findOrAddMenu(_T("&Insert"),hMenu,pWidths,1,false);
	g_menuItem.insert(m_insertMenu,"Contents of File...",CMD_INSERTFILE);
	g_menuItem.insertSep(m_insertMenu,CMD_INSERTSEP,-1);
	g_menuItem.insert(m_insertMenu,_T("&Object..."),CMD_INSERTOBJECT,-1);
	CRegKey key;
	CHECK(!key.Open(HKEY_CLASSES_ROOT,"CLSID"));
	for(vector<int>::iterator i(m_classOrder.begin());i!=m_classOrder.end();) {
		OLECHAR classString[50];
		RUN(StringFromGUID2(m_recentClasses[*i],classString,50));
		CRegKey classKey;
		if(classKey.Open(key,OLE2T(classString))) { // class missing so forget it.
			m_recentClasses[*i] = CLSID_NULL;
			m_classOrder.erase(i); // i now points to next in vector
			continue;
		}
		TCHAR classDesc[200];
		DWORD n(200);
		CHECK(!classKey.QueryValue(0,0,classDesc,&n));
		g_menuItem.insert(m_insertMenu,classDesc,CMD_FIRSTOBJECTTYPE+*i,-1);
		++i;
	}
	m_viewMenu = findOrAddMenu(_T("&View"),hMenu,pWidths,1,false);
	g_menuItem.insert(m_viewMenu,g_propertiesName,CMD_PROPERTIES);
	m_zoomMenu = CreateMenu();
	g_menuItem.insert(m_zoomMenu,"Size to &Fit",CMD_SIZETOFIT,-1,0);
	g_menuItem.insertSep(m_zoomMenu,CMD_ZOOMSEP,-1);
	CComQIPtr<IMCCSite> s = m_spClientSite;
	int j;
	for(j = 0; j<n_zooms; ++j)
		dozoom(g_zooms[j],m_zoomMenu,s);
	// hack: stupid way to check for size-to-fit
    CComPtr<IMontageContainer> MC = mc();
	CComPtr<IMCCTransform> t;
	RUN(MC->get_Transform(&t));
	if(!CComQIPtr<IMCCScrollingTransform>(t))
		CheckMenuItem(m_zoomMenu,CMD_SIZETOFIT,MF_CHECKED);
	WORD current = 0;
	long zoom;
	RUN(MC->get_Zoom(&zoom));
	if(current)
		CheckMenuItem(m_zoomMenu,CMD_ZOOM(zoom/100),MF_CHECKED); // may miss
	g_menuItem.insert(m_viewMenu,"&Zoom",0,1,m_zoomMenu);
	m_editMenu = findOrAddMenu(_T("&Edit"),hMenu,pWidths,1,false);
	g_menuItem.insert(m_editMenu,g_selectAllName,CMD_SELECTALL);
	g_menuItem.insert(m_editMenu,g_deleteName,CMD_DELETE);
	g_menuItem.insert(m_editMenu,g_pasteName,CMD_PASTE);
	g_menuItem.insert(m_editMenu,g_copyName,CMD_COPY);
	g_menuItem.insert(m_editMenu,g_cutName,CMD_CUT);

	RUN(s->RegisterCommand(CMD_CUT));
	RUN(s->RegisterCommand(CMD_COPY));
	RUN(s->RegisterCommand(CMD_PASTE));
	RUN(s->RegisterCommand(CMD_DELETE));
	RUN(s->RegisterCommand(CMD_SELECTALL));
	RUN(s->RegisterCommand(CMD_PROPERTIES));
	RUN(s->RegisterCommand(CMD_INSERTFILE));
	RUN(s->RegisterCommand(CMD_INSERTOBJECT));
	for(int j=CMD_FIRSTOBJECTTYPE;j!=CMD_LASTOBJECTTYPE;++j)
		RUN(s->RegisterCommand(j));
	//RUN(s->RegisterCommand(CMD_BASEPROPERTIES));
	RUN(s->RegisterCommand(CMD_SIZETOFIT));

	RUN(m_spInPlaceSite->SetFocus(true));
	return S_OK;
}
STDMETHODIMP ContainerEditMode::Deactivate(HMENU menu,OLEMENUGROUPWIDTHS *widths) {
    METHTRACE(ContainerEditMode,Deactivate);
	_ASSERTE(m_insertMenu && m_editMenu && m_viewMenu && m_zoomMenu);
	DeleteMenu(m_insertMenu,CMD_INSERTFILE,MF_BYCOMMAND);
	DeleteMenu(m_insertMenu,CMD_INSERTSEP,MF_BYCOMMAND);
	DeleteMenu(m_insertMenu,CMD_INSERTOBJECT,MF_BYCOMMAND);
	for(int i = CMD_FIRSTOBJECTTYPE; i<= CMD_LASTOBJECTTYPE; ++i)
		DeleteMenu(m_insertMenu,i,MF_BYCOMMAND);
	removeEmptyMenu("&Insert",menu,widths,1);
	m_insertMenu = 0;
	DeleteMenu(m_viewMenu,CMD_PROPERTIES,MF_BYCOMMAND);
	DeleteMenu(m_viewMenu,CMD_ZOOMMENU,MF_BYCOMMAND);
	removeEmptyMenu("&View",menu,widths,1);
	m_viewMenu = 0;
	DestroyMenu(m_zoomMenu);
	m_zoomMenu = 0;
	DeleteMenu(m_editMenu,CMD_DELETE,MF_BYCOMMAND);
	DeleteMenu(m_editMenu,CMD_PASTE,MF_BYCOMMAND);
	DeleteMenu(m_editMenu,CMD_COPY,MF_BYCOMMAND);
	DeleteMenu(m_editMenu,CMD_CUT,MF_BYCOMMAND);
	removeEmptyMenu("&Edit",menu,widths,1);
	m_editMenu = 0;

	CComQIPtr<IMCCSite> s = m_spClientSite;
	RUN(s->ForgetCommand(CMD_CUT));
	RUN(s->ForgetCommand(CMD_COPY));
	RUN(s->ForgetCommand(CMD_PASTE));
	RUN(s->ForgetCommand(CMD_DELETE));
	RUN(s->ForgetCommand(CMD_SELECTALL));
	RUN(s->ForgetCommand(CMD_PROPERTIES));
	RUN(s->ForgetCommand(CMD_INSERTFILE));
	RUN(s->ForgetCommand(CMD_INSERTOBJECT));
	for(i=CMD_FIRSTOBJECTTYPE;i!=CMD_LASTOBJECTTYPE;++i)
		RUN(s->ForgetCommand(i));
	//RUN(s->ForgetCommand(CMD_BASEPROPERTIES));
	RUN(s->ForgetCommand(CMD_SIZETOFIT));
	for(i=0;i<n_zooms;++i)
		RUN(s->ForgetCommand(CMD_ZOOM(g_zooms[i])));

	RUN(m_spInPlaceSite->SetFocus(false));
	return S_OK;
}

// IViewObjectImpl
HRESULT ContainerEditMode::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	METHTRACE(ContainerEditMode,OnDrawAdvanced);
	if(m_bRectSelect) {
		HDC hdc = di.hdcDraw;
		int oldROP = SetROP2(hdc,R2_XORPEN);
		SelectObject(hdc,GetStockObject(WHITE_PEN));
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		Rectangle(hdc,m_ptTrack.x,m_ptTrack.y,m_ptLast.x,m_ptLast.y);
		SetROP2(hdc,oldROP);
	}
	return S_OK;
}
RECT ContainerEditMode::curSelRect() {
    METHTRACE(ContainerEditMode,curSelRect);
	return rect(min(m_ptTrack.x,m_ptLast.x),min(m_ptTrack.y,m_ptLast.y),max(m_ptTrack.x,m_ptLast.x),max(m_ptTrack.y,m_ptLast.y));
}
void ContainerEditMode::inval() {
    METHTRACE(ContainerEditMode,inval);
	RECT rc = curSelRect();
	RUN(m_spInPlaceSite->InvalidateRect(&rc,true));
}
void ContainerEditMode::resel() {
    METHTRACE(ContainerEditMode,resel);
	// make a new selection from the new area, then assign that to the old selection
	CComPtr<IEnumMCCSites> currEnum;
	RECT rcCurr = curSelRect();
	RUN(m_mainl->FindControls(&rcCurr,&currEnum));
	CComPtr<IItemSet> newSelection;
	CREATE(newSelection,CLSID_ItemSet);
	RUN(CComQIPtr<IGroupItem>(newSelection)->Connect(mainGroup()));
	typedef enum_iterator<IEnumMCCSites,&IID_IEnumMCCSites,IMCCSite*,_CopyInterface<IMCCSite> > enumSite;
	for(enumSite i(currEnum);i!=enumSite();++i)
		RUN(fireSiteEvent(*i,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Add,newSelection)));
	RUN(m_selection->Assign(newSelection)); // fires lots of change events so grapples show+hide
	RUN(CComQIPtr<IGroupItem>(newSelection)->Remove());
	RUN(m_spInPlaceSite->InvalidateRect(&rcCurr,true));
}
// safe enough, since only one can be selecting at a time
ContainerEditMode *g_this=0;
UINT g_timerID=0;
bool g_firstTimeout;
SIZE g_whatScroll;
VOID CALLBACK ContainerEditMode::selectTimer(HWND hwnd,UINT uMsg,UINT idEvent,DWORD dwTime) {
    FUNTRACE(selectTimer);
	if(g_firstTimeout) {
		g_firstTimeout = false;
		::KillTimer(0,g_timerID);
		g_timerID = ::SetTimer(0,0,g_this->m_scrollInterval,selectTimer);
	}
	g_this->inval();
	SRUN(g_this->m_spInPlaceSite->Scroll(g_whatScroll));
	CComPtr<IMCCTransform> t;
	SRUN(g_this->mc()->get_Transform(&t));
	SRUN(t->C2WP(g_this->m_ptTrackC,&g_this->m_ptTrack));
	g_this->resel();
}
// window messages
LRESULT ContainerEditMode::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,ButtonDown);
	POINT pt = MAKEPOINT(lParam);
	m_bTracking = true;
	m_ptTrack = pt;
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	RUN(t->W2CP(m_ptTrack,&m_ptTrackC));
	m_szDist.cx = m_szDist.cy = 0;
	RUN(m_spInPlaceSite->SetCapture(true));

	CComPtr<IMCCSite> s;
	RUN(m_mainl->FindControl(pt,&s));
	if(!s) {// hit background
		m_bRectSelect = true;
		m_ptLast = pt;
		if(!(wParam&(MK_SHIFT|MK_CONTROL)))
			RUN(removeall());
	}
	else {
		m_bRectSelect = false;
		if(wParam&MK_CONTROL) { // toggle
			RUN(add(true,s));
		}
		else if(wParam&MK_SHIFT) { // add
			RUN(add(false,s));
		}
		else { // new selection
			if(check(s)==S_FALSE) {
				RUN(removeall());
				RUN(add(false,s));
			}
		}

	}
	return 0;
}

//#define DEBUG_HITTEST
#define SIGTRACK 2
template<class T>
inline T sqr(T x) { return x*x; }
LRESULT ContainerEditMode::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,MouseMove);
	HRESULT hr;
	POINT pt = MAKEPOINT(lParam);
#ifdef DEBUG_HITTEST
	CComPtr<IMCCSite> debug;
	RUN(m_mainl->FindControl(pt,&debug));
#endif
	if(!m_bTracking) {
	#ifdef DEBUG_HITTEST
		CComPtr<IMCCSite> s;
		RUN(m_mainl->FindControl(pt,&s));
		if(s) {
			HDC hdc;
			CComQIPtr<IOleInPlaceSiteWindowless> wls(m_spClientSite);
			RUN(wls->GetDC(0,0,&hdc));
			SetPixel(hdc,pt.x,pt.y,RGB(255,0,0));
			RUN(wls->ReleaseDC(hdc));
		}
	#endif
		return 0;
	}
    CComPtr<IMontageContainer> MC = mc();
	if(m_bRectSelect) {
		CComPtr<IMCCTransform> t;
		RUN(MC->get_Transform(&t));
		RUN(t->C2WP(m_ptTrackC,&m_ptTrack));

		inval();
		m_ptLast = pt;
		resel();

		// start/stop auto-scroll
		g_whatScroll = determineScrollWay(m_rcPos,m_scrollInset,pt)*20;
		if(g_whatScroll.cx||g_whatScroll.cy) {
			if(!g_this) {
				g_this = this;
				g_firstTimeout = true;
				g_timerID = ::SetTimer(0,0,m_scrollDelay,selectTimer);
			}
		}
		else if(g_this) {
			::KillTimer(0,g_timerID);
			g_this = 0;
			g_timerID = 0;
		}
	}
	else {
		if((sqr(pt.x-m_ptTrack.x)+sqr(pt.y-m_ptTrack.y))<sqr(SIGTRACK))
			return 0; // not (yet?) a drag

		// OLE will take mouse
		m_bTracking = false;
		RUN(m_spInPlaceSite->SetCapture(false));

		// make data object
		CComPtr<IDataObject> DO;
		CComPtr<IMCCItemSetRenderer> rend;
		RUN(hr = package(true,m_ptTrack,&DO,&rend)); // use point where mouse went down, not current!
		if(hr==S_FALSE) // nothing selected
			return 0;

		// .. drop source (silly)
		CComPtr<IMCCDropSource> ds;
		RUN(CREATE(ds,CLSID_MCCDropSource));

		// drag
		RUN(ds->StartDrag(wParam&(VK_LBUTTON|VK_MBUTTON|VK_RBUTTON)));
		DWORD effect;
		m_amsource = true;
		RUN(groupOp(MC,true));
		hr = DoDragDrop(DO,CComQIPtr<IDropSource>(ds),m_allowedDragEffects,&effect);
		if(rend)
			RUN(rend->Stop());
		// if drop was within same container, Drop() will signal by resetting m_amsource
		if(hr == DRAGDROP_S_DROP && m_amsource) {
			if(effect&DROPEFFECT_MOVE) {
				BOOL boo;
				OnDelete(0,0,0,boo);
			}
		}
		m_amsource = false;
		RUN(groupOp(MC,false));
	}
	return 0;
}
// this can only get hit on a click or a rectangular selection, because
// drags go to DoDragDrop and no mouse messages get out
LRESULT ContainerEditMode::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,ButtonUp);
	if(!m_bTracking)
		return 0;
	m_bTracking = false;
	RUN(m_spInPlaceSite->SetCapture(false));
    CComPtr<IMontageContainer> MC = mc();
	POINT pt = MAKEPOINT(lParam);
	if(uMsg==WM_LBUTTONUP) {
		if(m_bRectSelect) {
			inval();
			m_bRectSelect = false;
			if(g_this) {
				::KillTimer(0,g_timerID);
				g_this = 0;
				g_timerID = 0;
			}
		}
		else {
			RUN(groupOp(MC,true));
			// bring to front
			VS *vs = getSites(mainGroup(),m_selection);
			_ASSERTE(vs);
			for(VS::iterator i = vs->begin();i!=vs->end();++i)
				RUN(fireSiteEvent(*i,firePutBehind(0)));
			delete vs;
			RUN(groupOp(MC,false));
		}
	}
	if(uMsg==WM_RBUTTONUP) {
		long count;
		RUN(m_selection->get_Count(&count));
		HMENU hMenu = CreatePopupMenu();
		if(count) {
			g_menuItem.insert(hMenu,g_cutName,CMD_CUT,-1);
			g_menuItem.insert(hMenu,g_copyName,CMD_COPY,-1);
			g_menuItem.insert(hMenu,g_deleteName,CMD_DELETE,-1);
		}
		else
			g_menuItem.insert(hMenu,g_pasteName,CMD_PASTE,-1);
		ciList &itemList = count?fullItems:emptyItems;
		if(itemList.size()) {
			g_menuItem.insertSep(hMenu,0,-1);
			for(ciList::iterator ili = itemList.begin(); ili!=itemList.end(); ++ili)
				g_menuItem.insert(hMenu,(*ili).text.c_str(),(*ili).id,-1,(*ili).menu);
		}
		g_menuItem.insertSep(hMenu,0,-1);
		g_menuItem.insert(hMenu,g_propertiesName,CMD_PROPERTIES,-1);//count?CMD_PROPERTIES:CMD_BASEPROPERTIES);

		HWND hWnd;
		RUN(m_spInPlaceSite->GetWindow(&hWnd));
		RECT rc;
		::GetWindowRect(hWnd,&rc);
		TrackPopupMenu(hMenu,TPM_LEFTALIGN|TPM_RIGHTBUTTON,rc.left+pt.x,rc.top+pt.y,0,hWnd,NULL);
		// remove any pop-ups so they don't get destroyed
		for(ciList::iterator ili = itemList.begin(); ili!=itemList.end(); ++ili)
			if((*ili).menu)
				RemoveMenu(hMenu,(*ili).id,MF_BYCOMMAND);
		DestroyMenu(hMenu);
	}
	return 0;
}
LRESULT ContainerEditMode::DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,DoubleClick);
	// if m_activator == 0, we must be configured not to allow activation
	if(!m_activator)
		return 0;
	// make sure there's something to activate
	long count;
	RUN(m_selection->get_Count(&count));
	if(count) {
		POINT pt = MAKEPOINT(lParam);
		CComPtr<IMCCSite> s;
		RUN(m_mainl->FindControl(pt,&s));
		// activate the selection
		RUN(m_activator->Start());
		// then UI activate the clicked
		if(s)
			RUN(fireSiteEvent(s,fireIntend(MCCS_UIACTIVE)));
	}
	return 0;
}
LRESULT ContainerEditMode::KeyPress(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,KeyPress);
	int wmCommand = 0;
	if(GetKeyState(VK_CONTROL)&0x8000)
		switch(wParam) {
		case 'C':
			wmCommand = CMD_COPY;
			break;
		case 'V':
			wmCommand = CMD_PASTE;
			break;
		case 'X':
			wmCommand = CMD_CUT;
			break;
		case 'A':
			wmCommand = CMD_SELECTALL;
			break;
	}
	else switch(wParam) {
		case VK_DELETE:
			wmCommand = CMD_DELETE;
			break;
	}
	LRESULT lr;
	if(wmCommand)
		RUN(OnWindowMessage(WM_COMMAND,MAKELONG(wmCommand,1),0,&lr));
	else RUN(m_spInPlaceSite->OnDefWindowMessage(uMsg,wParam,lParam,&lr));
	return 0;
}
LRESULT ContainerEditMode::SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ContainerEditMode,SetCursor);
	::SetCursor(LoadCursor(0,IDC_ARROW));
	return 0;
}
LRESULT ContainerEditMode::OnCut(WORD, WORD, HWND, BOOL&b) {
	METHTRACE(ContainerEditMode,OnCut);
	OnCopy(0,0,0,b);
	OnDelete(0,0,0,b);
	return 0;
}
LRESULT ContainerEditMode::OnCopy(WORD, WORD, HWND, BOOL&) {
	METHTRACE(ContainerEditMode,OnCopy);
	CComPtr<IDataObject> data;
	POINT pt = {0,0};
	CComPtr<IMCCItemSetRenderer> rend;
	RUN(package(false,pt,&data,&rend));
	RUN(OleSetClipboard(data));
	RUN(rend->Stop());
	return 0;
}
LRESULT ContainerEditMode::OnPaste(WORD, WORD, HWND, BOOL&) {
	METHTRACE(ContainerEditMode,OnPaste);
	CComPtr<IDataObject> data;
	RUN(OleGetClipboard(&data));
	RUN(removeall());
	RUN(insertEvent(data,0));
	return 0;
}
LRESULT ContainerEditMode::OnDelete(WORD, WORD, HWND, BOOL&) {
	METHTRACE(ContainerEditMode,OnDelete);
	CComPtr<IItemSet> data = dataSet();
	// remove from Montage
	VS *vs = getSites(mainGroup(),m_selection);
	_ASSERTE(vs);
    CComPtr<IMontageContainer> MC = mc();
	RUN(groupOp(MC,true));
	for(VS::iterator i = vs->begin();i!=vs->end();++i) {
		CComPtr<IMoniker> mk;
		RUN((*i)->get_RelMoniker(&mk));
		RUN(data->Remove(mk));
		RUN(fireSiteEvent(*i,fireIntend(MCCS_DELETED)));
	}
	delete vs;
	RUN(groupOp(MC,false));
	// the selection now holds the last stgrefs on the data.  This clear causes real deletion.
	RUN(m_selection->Clear());
	return 0;
}
LRESULT ContainerEditMode::OnSelectAll(WORD, WORD, HWND, BOOL&) {
    METHTRACE(ContainerEditMode,OnSelectAll);
	/*
	CComPtr<IEnumMoniker> em;
	RUN(dataSet()->Enum(&em));
	BindCtx bctx;
	for(mk_iterator im(em); im!=mk_iterator(); ++im) {
		LPOLESTR name;
		RUN((*im)->GetDisplayName(bctx,0,&name));
		ATLTRACE2("selecting %S\n",name);
		CoTaskMemFree(name);
	}
	*/
	VS *vs = getSites(mainGroup(),dataSet());
	_ASSERTE(vs);
	for(VS::iterator is = vs->begin(); is!=vs->end(); ++is)
		RUN(fireSiteEvent(*is,fireOneParam<IMCCSelectable,IItemSet*>(&IMCCSelectable::Add,m_selection)));
	delete vs;
	return S_OK;
}
LRESULT ContainerEditMode::OnProperties(WORD, WORD, HWND, BOOL&) {
	METHTRACE(ContainerEditMode,OnProperties);
	HRESULT hr;
	int nclasses;
	CLSID *classes;
	ULONG nobjects;
	IUnknown **objects; // HACK: not refcounted
	long count;
	RUN(m_selection->get_Count(&count));
	VS *vs = count?getSites(mainGroup(),m_selection):0; // (else==CMD_BASEPROPERTIES)
	if(vs&&vs->size()) {
		// get all proppage classes for first object, then go through rest and remove
		// any classes that the others don't share.
		bool first = true;
		int nclasses0;
		CLSID *classes0 = 0;
		nobjects = vs->size();
		objects = new IUnknown*[nobjects];
		IUnknown **io = objects;
		for(VS::iterator is = vs->begin();is!=vs->end();is++) {
			CComPtr<ISpecifyPropertyPages> spp;
			if(FAILED(hr = (*is)->GetControl(riidret(&spp))))
				break; // probably should remove from objects
			CAUUID cauuid;
			if(FAILED(hr = spp->GetPages(&cauuid)))
				break;
			if(first) {
				classes0 = new CLSID[nclasses = nclasses0 = cauuid.cElems];
				copy(cauuid.pElems,cauuid.pElems+cauuid.cElems,classes0);
				first = false;
			}
			else {
				for(int i = 0;i<nclasses0;i++) {
					CLSID *end = cauuid.pElems+cauuid.cElems;
					CLSID *f = find(cauuid.pElems,end,classes0[i]);
					if(f==end) {
						classes0[i] = CLSID_NULL;
						nclasses--;
					}
				}
			}
			CoTaskMemFree(cauuid.pElems);
			*io++ = spp;
		}
		delete vs;
		if(FAILED(hr)) {
			if(classes0)
				delete [] classes0;
			return hr;
		}
		// to match use of cauuid.pElems below
		classes = (CLSID*)CoTaskMemAlloc(sizeof(CLSID)*nclasses);
		int i,j;
		for(i=j=0;i<nclasses0;i++)
			if(classes0[i]!=CLSID_NULL)
				classes[j++] = classes0[i];
		delete [] classes0;
	}
	else {
		// nothing is selected: get proppages for the main layer's ambiance
		// (which is shared by active layer)
		CComPtr<IDispatch> d;
		RUN(m_mainl->get_DefaultAmbiance(&d));
		CComQIPtr<ISpecifyPropertyPages> ipp = d;
		if(!ipp)
			return 0;
		objects = new IUnknown*;
		objects[0] = ipp;
		nobjects = 1;
		CAUUID cauuid;
		RUN(ipp->GetPages(&cauuid));
		nclasses = cauuid.cElems;
		classes = cauuid.pElems;
	}
	HWND hWnd;
	RUN(m_spInPlaceSite->GetWindow(&hWnd));
	RUN(OleCreatePropertyFrame(hWnd,0,0,L"Properties",nobjects,objects,nclasses,classes,LOCALE_USER_DEFAULT,0,0));
	CoTaskMemFree(classes);
	delete [] objects;
	return 0;
}
LRESULT ContainerEditMode::OnZoom(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled) {
    METHTRACE(ContainerEditMode,OnZoom);
	MENUITEMINFO info;
	info.cbSize = sizeof info;
	info.fMask = MIIM_STATE;
    CComPtr<IMontageContainer> MC = mc();
	if(wID==CMD_SIZETOFIT) {
		CComPtr<IMCCTransform> newTransform;
		GetMenuItemInfo(m_zoomMenu,CMD_SIZETOFIT,false,&info);
		info.fState ^= MFS_CHECKED;
		SetMenuItemInfo(m_zoomMenu,CMD_SIZETOFIT,false,&info);
		if(info.fState&MFS_CHECKED) {
			RUN(MC->put_AutoSize(VARIANT_FALSE));
			RUN(CREATE(newTransform,CLSID_MCCSizeToFitTransform));
		}
		else
			RUN(CREATE(newTransform,CLSID_MCCScrollingTransform));
		RUN(MC->put_Transform(newTransform));
		return 0;
	}
	GetMenuItemInfo(m_zoomMenu,wID,false,&info);
	if(info.fState&MFS_CHECKED)
		return 0;
	RUN(MC->put_Zoom(GETZOOM(wID)*100));
	for(int i = 0; i<n_zooms; ++i)
		CheckMenuItem(m_zoomMenu,CMD_ZOOM(g_zooms[i]),MF_UNCHECKED);
	CheckMenuItem(m_zoomMenu,wID,MF_CHECKED);
	return 0;
}
#include <oledlg.h>
#include "siteControl.h"
LRESULT ContainerEditMode::siteIt(IUnknown *embed,IMoniker *mkEmbed, IMoniker **ret) {
    METHTRACE(ContainerEditMode,siteIt);
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	RUN(mainGroup()->Paste(bctx,GP_SrcIsObj|GP_Replace,embed,mkEmbed,IID_NULL,0,0));
	if(!m_mkCacheInit)
		m_mkCacheInit = ItemMoniker(L"Cache Init");
	if(!m_mkRunInit)
		m_mkRunInit = ItemMoniker(L"Run Init");
	IMoniker *inits[2] = {m_mkCacheInit,m_mkRunInit};
	int initP[2] = {0,1};
	CComPtr<IMCCSitePersistor> sp;
	CComPtr<IMoniker> mkSP;
	RUN(siteControl(bctx,mkEmbed,0,m_mainl,0,MCCS_SHOWN,2,inits,initP,&mkSP,&sp));
	RUN(dataSet()->Add(mkSP));
	RUN(fireEvent(mc(),fireTwoParams<IMCCSiteLayoutEvents,IMCCSitePersistor*,LPOLESTR>
		(&IMCCSiteLayoutEvents::Process,sp,0)));
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(bctx));
	if(ret)
		RUN(sp->get_RelMoniker(ret));
	return S_OK;
}
LRESULT ContainerEditMode::OnInsertObject(WORD, WORD, HWND, BOOL&) {
    METHTRACE(ContainerEditMode,OnInsertObject);
	CComPtr<IStorage> stg;
	CComPtr<IMoniker> mkEmbed;
	RUN(mainGroup()->CreatePassive(true,0,0,&mkEmbed,riidret(&stg)));
	CComPtr<IOleObject> embed;

	OLEUIINSERTOBJECTA spec;
	spec.cbStruct = sizeof(OLEUIINSERTOBJECT);
	spec.dwFlags = IOF_SHOWHELP|IOF_SELECTCREATENEW|IOF_DISABLELINK|IOF_DISABLEDISPLAYASICON|
		IOF_CREATENEWOBJECT|IOF_CREATEFILEOBJECT|IOF_CREATELINKOBJECT;
	spec.hWndOwner = m_hWnd;
	spec.lpszCaption = 0;
	spec.lpfnHook = 0;
	spec.lCustData = 0;
	spec.hInstance = 0;
	spec.lpszTemplate = 0;
	spec.hResource = 0;
	spec.clsid = CLSID_NULL;
	TCHAR fileName[MAX_PATH];
	spec.lpszFile = fileName;
	spec.cchFile = MAX_PATH;
	spec.cClsidExclude = 0;
	spec.lpClsidExclude = 0;
	spec.iid = IID_IOleObject;
	spec.oleRender = OLERENDER_DRAW;
	spec.lpFormatEtc = 0;
	spec.lpIOleClientSite = 0;
	spec.lpIStorage = stg;
	spec.ppvObj = (void**)&embed;
	spec.sc = S_OK;
	spec.hMetaPict = 0;
	RUN(OleUIInsertObject(&spec));
	if(FAILED(spec.sc))
		::MessageBox(0,"Error Inserting Object","Montage Edit Mode",MB_OK);
	if(!embed) // cancel/error
		return 0;
	CComPtr<IMoniker> mkSite;
	RUN(siteIt(embed,mkEmbed,&mkSite));
	RUN(m_selection->Clear());
	RUN(m_selection->Add(mkSite));

	// remember CLSID
	int i;
	// find empty slot
	for(i=0;i<MAX_RECENT;++i)
		if(m_recentClasses[i]==CLSID_NULL)
			break;
	if(i==MAX_RECENT) { // full, so replace oldest
		_ASSERTE(m_classOrder.size());
		i = *m_classOrder.begin();
		m_classOrder.erase(m_classOrder.begin());
		RemoveMenu(m_insertMenu,CMD_FIRSTOBJECTTYPE+i,MF_BYCOMMAND);
	}
	m_recentClasses[i] = spec.clsid;
	m_classOrder.push_back(i);

	USES_CONVERSION;
	CRegKey key;
	CHECK(!key.Open(HKEY_CLASSES_ROOT,"CLSID"));
	OLECHAR classString[50];
	RUN(StringFromGUID2(spec.clsid,classString,50));
	CHECK(!key.Open(key,OLE2T(classString)));
	TCHAR classDesc[200];
	DWORD n(200);
	CHECK(!key.QueryValue(0,0,classDesc,&n));
	g_menuItem.insert(m_insertMenu,classDesc,CMD_FIRSTOBJECTTYPE+i,1);
	return 0;
}
LRESULT ContainerEditMode::OnInsertAgain(WORD, WORD wID, HWND, BOOL&) {
    METHTRACE(ContainerEditMode,OnInsertAgain);
	int n = wID-CMD_FIRSTOBJECTTYPE;
	_ASSERTE(n<MAX_RECENT);
	CComPtr<IStorage> stg;
	CComPtr<IMoniker> mkEmbed;
	RUN(mainGroup()->CreatePassive(true,0,0,&mkEmbed,riidret(&stg)));
	CComPtr<IOleObject> embed;
	RUN(OleCreate(m_recentClasses[n],IID_IOleObject,OLERENDER_DRAW,0,0,stg,(void**)&embed));
	RUN(siteIt(embed,mkEmbed));
	return 0;
}
#include "getPortFormats.h"
LRESULT ContainerEditMode::OnInsertFile(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(ContainerEditMode,OnInsertFile);
    CComPtr<IMontageContainer> MC = mc();
	CComPtr<IMCCFileFormat> portmat;
	RUN(MC->get_FilePort(&portmat));
	if(!portmat)
		return 0;
	OLECHAR filter[1000];
	int len = getPortFormats(MC,false,filter);
	USES_CONVERSION;
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = 0;
	ofn.hInstance = 0;
	char f2[1000];
	W2AN(f2,1000,filter,len+1);
	ofn.lpstrFilter = f2;
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	char filename[MAX_PATH] = "";
	ofn.lpstrFile = filename;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFileTitle = 0;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = 0;
	ofn.lpstrTitle = "Insert Contents of File";
	ofn.Flags = OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = "dot";
	ofn.lCustData = 0;
	ofn.lpfnHook = 0;
	ofn.lpTemplateName = 0;
	if(GetOpenFileName(&ofn)&&filename[0])
		if(FAILED(portmat->Import(T2OLE(filename))))
			MessageBox("Insert File failed","Problems");
	return 0;
}
// IOleControl
STDMETHODIMP ContainerEditMode::GetControlInfo(LPCONTROLINFO ret) {
	METHTRACE(ContainerEditMode,GetControlInfo);
	ERR(E_NOTIMPL);
	/*
	if(!m_accel) {
		m_accel = (HACCEL)GlobalAlloc(GHND,sizeof(ACCEL)*4);
		ACCEL *acc = (ACCEL*)GlobalLock(m_accel);
		acc[0].fVirt = FCONTROL|FVIRTKEY;
		acc[0].key = L'C';
		acc[0].cmd = CMD_COPY;

		acc[1].fVirt = FCONTROL|FVIRTKEY;
		acc[1].key = L'V';
		acc[1].cmd = CMD_PASTE;

		acc[2].fVirt = FCONTROL|FVIRTKEY;
		acc[2].key = L'X';
		acc[2].cmd = CMD_CUT;

		acc[3].fVirt = FVIRTKEY;
		acc[3].key = VK_DELETE;
		acc[3].cmd = CMD_DELETE;
		GlobalUnlock(m_accel);
	}
	ret->cb = sizeof(CONTROLINFO);
	ret->hAccel = m_accel;
	ret->cAccel = 4;
	ret->dwFlags = 0;
	return S_OK;
	*/
}
#include "NotherIsAccelerator.h"

STDMETHODIMP ContainerEditMode::OnMnemonic(LPMSG msg) {
	METHTRACE(ContainerEditMode,OnMnemonic);
	ERR(E_NOTIMPL);
	/*
	LRESULT lr;
	WORD cmd;
	if(!m_accel || !NotherIsAccelerator(m_accel,4,msg,&cmd))
		ERR(E_UNEXPECTED);
	return OnWindowMessage(WM_COMMAND,MAKELONG(cmd,1),0,&lr);
	*/
}
// all dragdrop points are in screen coords
void ContainerEditMode::unScreen(POINTL *ppt) {
    METHTRACE(ContainerEditMode,unScreen);
	HWND hWnd;
	RUN(m_spInPlaceSite->GetWindow(&hWnd));
	::ScreenToClient(hWnd,(POINT *)ppt);
}
HRESULT ContainerEditMode::getdroprect(POINTL ptScreen) {
	METHTRACE(ContainerEditMode,getdroprect);

	// combine that with OBJECTDESCRIPTOR drop info
	m_rcDrop.left = ptScreen.x - m_droppoint.cx;
	m_rcDrop.top = ptScreen.y - m_droppoint.cy;
	m_rcDrop.right = m_rcDrop.left + m_dropsize.cx;
	m_rcDrop.bottom = m_rcDrop.top + m_dropsize.cy;
	return S_OK;
}

// uses keys and allowed effects to determine single effect
// the allowed effects are
// those passed to DoDragDrop & the allowed drop effects (property)
// | DROPEFFECT_MOVE if source==dest (ie move is always allowed within a container)

// code below draws the drop hint as long as the data is good, even if there are no allowed
// effects
DWORD ContainerEditMode::determineEffect(DWORD okEffects, DWORD keyState) {
    METHTRACE(ContainerEditMode,determineEffect);
	okEffects &= m_allowedDropEffects;
	if(!okEffects)
		return DROPEFFECT_NONE;
	DWORD primary = m_amsource?DROPEFFECT_MOVE:m_defaultDropEffect,
		secondary = (primary==DROPEFFECT_COPY)?DROPEFFECT_MOVE:DROPEFFECT_COPY;
	if(keyState&MK_CONTROL) {
		if(secondary&okEffects)
			return secondary;
		else
			return primary&okEffects;
	}
	else {
		if(primary&okEffects)
			return primary;
		else
			return secondary&okEffects;
	}
}
// IDropTarget
STDMETHODIMP ContainerEditMode::DragEnter(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
										   DWORD *pdwEffect) {
	METHTRACE(ContainerEditMode,DragEnter);
	if(m_amsource)
		m_dropable = true;
	else
		m_dropable = queryEvent(pDataObj)==S_OK;
	if(!m_dropable)
		*pdwEffect = DROPEFFECT_NONE;
	else {
		unScreen(&pt);

		// get drop positioning info
		SIZEL s;
		POINTL p;
		RUN(getsizepoint(pDataObj,&s,&p));
		CComPtr<IMCCTransform> t;
		RUN(mc()->get_Transform(&t));
		RUN(t->C2WS(s,&m_dropsize));
		m_dropsize.cy = tabs(m_dropsize.cy); // HIMETRIC only in units, not direction
		// for our purposes this POINTL is a SIZEL
		RUN(t->C2WS(*(SIZEL*)&p,&m_droppoint));
		m_droppoint.cy = tabs(m_droppoint.cy); // this too will be negated.

		// draw drophint
		RUN(getdroprect(pt));
		HDC hdc;
		RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
		int lastROP = SetROP2(hdc,R2_XORPEN);
		HPEN hpen = CreatePen(PS_SOLID,0,RGB(128,128,128)),
			holdpen = (HPEN)SelectObject(hdc,hpen);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		Rectangle(hdc,m_rcDrop.left,m_rcDrop.top,m_rcDrop.right,m_rcDrop.bottom);
		SelectObject(hdc,holdpen);
		DeleteObject(hpen);
		SetROP2(hdc,lastROP);
		RUN(m_spInPlaceSite->ReleaseDC(hdc));

		*pdwEffect = determineEffect(*pdwEffect,grfKeyState);

		SIZE scrollWay = determineScrollWay(m_rcPos,m_scrollInset,point(pt));
		if(scrollWay.cx||scrollWay.cy) {
			*pdwEffect |= DROPEFFECT_SCROLL;
			m_scrollTimeout = GetTickCount()+m_scrollDelay;
		}
	}
	return S_OK;
}
STDMETHODIMP ContainerEditMode::DragOver(DWORD grfKeyState,POINTL pt,DWORD *pdwEffect) {
	METHTRACE(ContainerEditMode,DragOver);
	if(!m_dropable)
		*pdwEffect = DROPEFFECT_NONE;
	else {
		unScreen(&pt);

		// move drophint
		HDC hdc;
		RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
		SetROP2(hdc,R2_XORPEN);
		HPEN hpen = CreatePen(PS_SOLID,0,RGB(128,128,128)),
			holdpen = (HPEN)SelectObject(hdc,hpen);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		Rectangle(hdc,m_rcDrop.left,m_rcDrop.top,m_rcDrop.right,m_rcDrop.bottom);
		RUN(getdroprect(pt));
		Rectangle(hdc,m_rcDrop.left,m_rcDrop.top,m_rcDrop.right,m_rcDrop.bottom);
		SelectObject(hdc,holdpen);
		DeleteObject(hpen);
		RUN(m_spInPlaceSite->ReleaseDC(hdc));

		*pdwEffect = determineEffect(*pdwEffect,grfKeyState);

		SIZE scrollWay = determineScrollWay(m_rcPos,m_scrollInset,point(pt));
		if(scrollWay.cx||scrollWay.cy) {
			*pdwEffect |= DROPEFFECT_SCROLL;
			if(m_scrollTimeout) {
				if(GetTickCount()>m_scrollTimeout) {
					RUN(m_spInPlaceSite->Scroll(scrollWay*20));
					m_scrollTimeout += m_scrollInterval;
				}
			}
			else m_scrollTimeout = GetTickCount()+m_scrollDelay;
		}
		else
			m_scrollTimeout = 0;
	}
	return S_OK;
}
STDMETHODIMP ContainerEditMode::DragLeave() {
	METHTRACE(ContainerEditMode,DragLeave);
	m_scrollTimeout = 0;
	if(m_dropable) {
		// erase drophint
		HDC hdc;
		RUN(m_spInPlaceSite->GetDC(0,0,&hdc));
		SetROP2(hdc,R2_XORPEN);
		HPEN hpen = CreatePen(PS_SOLID,0,RGB(128,128,128)),
			holdpen = (HPEN)SelectObject(hdc,hpen);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		Rectangle(hdc,m_rcDrop.left,m_rcDrop.top,m_rcDrop.right,m_rcDrop.bottom);
		SelectObject(hdc,holdpen);
		DeleteObject(hpen);
		RUN(m_spInPlaceSite->ReleaseDC(hdc));
	}
	return S_OK;
}
STDMETHODIMP ContainerEditMode::Drop(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,DWORD *pdwEffect) {
	METHTRACE(ContainerEditMode,Drop);
	if(!m_dropable) {
		*pdwEffect = DROPEFFECT_NONE;
		return S_OK;
	}

	DragLeave(); // erase drophint

	HWND hWnd;
	RUN(m_spInPlaceSite->GetWindow(&hWnd));
	::ScreenToClient(hWnd,(POINT *)&pt);

	*pdwEffect = determineEffect(*pdwEffect,grfKeyState);

	// and move is a special case
	if(m_amsource && (*pdwEffect & DROPEFFECT_MOVE)) {
        CComPtr<IMontageContainer> MC = mc();
		RUN(groupOp(MC,true));
		// calculate in canvas coords so we can account for autoscrolling that might have happened
		CComPtr<IMCCTransform> t;
		RUN(MC->get_Transform(&t));
		POINTL ptC;
		RUN(t->W2CP(point(pt),&ptC));
		SIZEL delta = ptC - m_ptTrackC;
		//SIZE delta = {pt.x - m_ptTrack.x,pt.y - m_ptTrack.y};
		VS *vs = getSites(mainGroup(),m_selection);
		_ASSERTE(vs);
		if(!vs)
			ERR(E_UNEXPECTED);
		for(VS::iterator i = vs->begin();i!=vs->end();i++) {
			MCC_POS_INFO mpi;
			mpi.postype = MCC_POS_CANVASRECT;
			RUN((*i)->get_VirtRect(&mpi.rcl));
			mpi.rcl += delta;
			RUN(fireSiteEvent(*i,fireMove(&mpi)));
		}
		delete vs;
		RUN(groupOp(MC,false));
		m_amsource = false; // signal that source (I) should not erase on this move.
		*pdwEffect = DROPEFFECT_MOVE;
	}
	else {
		MCC_POS_INFO mpi;
		mpi.postype = MCC_POS_WINDOWPOINT;
		mpi.ptl = pt; // note these are actually the same type in win32: erroneous POINTL in method avoided.
		::SetCursor(LoadCursor(0,IDC_WAIT));
		if(FAILED(insertEvent(pDataObj,&mpi)))
			*pdwEffect = DROPEFFECT_NONE;
		// else *pdwEffect was already set
	}
	return S_OK;
}
STDMETHODIMP ContainerEditMode::Load(IStream *s) {
    METHTRACE(ContainerEditMode,Load);
	RUN(IPersistStreamInitImpl<ContainerEditMode>::Load(s));
	RUN(s->Read(&m_allowDrop,sizeof(bool),0));
	RUN(s->Read(&m_defaultDropEffect,sizeof(DWORD),0));
	RUN(s->Read(&m_allowedDragEffects,sizeof(DWORD),0));
	RUN(s->Read(&m_allowedDropEffects,sizeof(DWORD),0));
	return S_OK;
}
STDMETHODIMP ContainerEditMode::Save(IStream *s,BOOL clearDirty) {
    METHTRACE(ContainerEditMode,Save);
	RUN(IPersistStreamInitImpl<ContainerEditMode>::Save(s,clearDirty));
	RUN(s->Write(&m_allowDrop,sizeof(bool),0));
	RUN(s->Write(&m_defaultDropEffect,sizeof(DWORD),0));
	RUN(s->Write(&m_allowedDragEffects,sizeof(DWORD),0));
	RUN(s->Write(&m_allowedDropEffects,sizeof(DWORD),0));
	return S_OK;
}
STDMETHODIMP ContainerEditMode::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(ContainerEditMode,TakePointers);
	CComPtr<IUnknown> unk;
	RUN(os->Next(1,&unk,0));
	CComQIPtr<IItemSet> selection(unk);
	unk.Release();
	RUN(os->Next(1,&unk,0));
	CComQIPtr<IMCCItemSetActivator> activator(unk);
	unk.Release();
	RUN(os->Next(1,&unk,0));
	CComQIPtr<IMCCLayer> mainl(unk);
	unk.Release();
	RUN(os->Next(1,&unk,0));
	CComQIPtr<IMCCDataObjectUnpacker> unpacker(unk);
	return Initialize(selection,activator,mainl,unpacker);
}
