/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ContainerEditMode.h : Declaration of the ContainerEditMode

#ifndef __CONTAINERMODE_H_
#define __CONTAINERMODE_H_

#include "resource.h"       // main symbols
#include "CEMPublic.h" // stuff shared with viewmode

#define MAX_RECENT 15 // items below Object... in Insert menu

#define CMD_CUT 210
#define CMD_COPY 211
#define CMD_DELETE 212
#define CMD_PASTE 213
#define CMD_SELECTALL 214
#define CMD_INSERTFILE 230
#define CMD_INSERTSEP 231
#define CMD_INSERTOBJECT 232
#define CMD_FIRSTOBJECTTYPE 233
#define CMD_LASTOBJECTTYPE (CMD_FIRSTOBJECTTYPE+MAX_RECENT-1)
#define CMD_ZOOMMENU 250
#define CMD_SIZETOFIT 251
#define CMD_ZOOMSEP 252
// use 0x1000-0x1fff for zooms
#define CMD_ZOOMMASK 0x1000
#define CMD_ZOOM(p) (WORD)(CMD_ZOOMMASK|p)
#define GETZOOM(c) (((c&0xf000)!=CMD_ZOOMMASK)?-1:c^CMD_ZOOMMASK)

/////////////////////////////////////////////////////////////////////////////
// ContainerEditMode
class ATL_NO_VTABLE ContainerEditMode :
	public CComObjectRoot,
	public CComCoClass<ContainerEditMode,&CLSID_MCCContainerEditMode>,
	public CComControl<ContainerEditMode>,
	//public IConnectionPointImpl<ContainerEditMode,&IID_ICEMTransferEvents>,
	public IConnectionPointContainerImpl<ContainerEditMode>,
	public IPersistStreamInitImpl<ContainerEditMode>,
	public IOleControlImpl<ContainerEditMode>,
	public IOleObjectImpl<ContainerEditMode>,
	public IOleInPlaceActiveObjectImpl<ContainerEditMode>,
	public IViewObjectExImpl<ContainerEditMode>,
	public IOleInPlaceObjectWindowlessImpl<ContainerEditMode>,
	public ICoUIActiveObject,
	public IContainerEditMode,
	public IDispatchImpl<DIContainerEditMode>,
	public IDropTarget,
	public INeedSomePointers
{
	friend struct _CEMSiteSink;
    CComPtr<DIMontageContainerRef> m_mref; // semi-circular reference counting
	CComPtr<IMCCLayer> m_mainl;
	CComPtr<IMCCItemSetActivator> m_activator;
	CComPtr<IMCCDataObjectUnpacker> m_unpacker;
	bool m_allowDrop;
	HACCEL m_accel; // accelerator table;
	HMENU m_editMenu, m_viewMenu, m_zoomMenu, m_insertMenu; // put up under coUI activation
	// for Insert Object
	CComPtr<IMoniker> m_mkCacheInit, m_mkRunInit;
	CLSID m_recentClasses[MAX_RECENT]; // maps menu IDs (minus CMD_FIRSTOBJECTTYPE) to CLSID
	vector<int> m_classOrder; // maps order to menu IDs (minus).  oldest to newest.

	// mouse state
	POINT m_ptTrack, // place mouse button went down
		m_ptLast; // last position processed (for rect. selection)
	POINTL m_ptTrackC; // in canvas coords, to ignore scrolling
	SIZE m_szDist; // last distance from down
	bool m_bTracking; // are we tracking the mouse?
	bool m_bRectSelect; // whether rectangular selection is reason for tracking

	// selection
	CComPtr<IItemSet> m_selection;
	RECT curSelRect();
	void inval();
	void resel();

	// selection auto-scroll (also uses globals - bleagh!)
	static VOID CALLBACK selectTimer(HWND hwnd,UINT uMsg,UINT idEvent,DWORD dwTime);

	// drag'n'drop state
	DWORD m_defaultDropEffect, m_allowedDragEffects,m_allowedDropEffects;
	bool m_dropable,
		m_amsource;
	RECT m_rcDrop;
	SIZE m_droppoint,m_dropsize;
	int m_scrollInset,m_scrollDelay,m_scrollInterval;
	DWORD m_scrollTimeout;

	// extra context menu items
	struct contextItem {
		string text;
		int id;
		HMENU menu;
		contextItem(string text,int id,HMENU menu) : text(text), id(id), menu(menu) {}
	};
	typedef list<contextItem> ciList;
	ciList emptyItems,fullItems;

    CComPtr<IMontageContainer> mc()  {
        METHTRACE(ContainerEditMode,mc);
        CComPtr<IUnknown> umc;
        RUN(m_mref->get_Montage(&umc));
        return CComQIPtr<IMontageContainer>(umc);
    }
	CComPtr<IGroup> mainGroup();
	CComPtr<IItemSet> dataSet();

	// selection
	HRESULT add(bool toggle,IMCCSite *s);
	HRESULT check(IMCCSite *s);
	HRESULT removeall();

	HRESULT fillObjectDescriptor(POINTL ptMouse, OBJECTDESCRIPTOR*retDesc,POINTL *retRefPoint);
	HRESULT insertEvent(IDataObject *data, MCC_POS_INFO *mpi);
	HRESULT queryEvent(IDataObject *data);
	HRESULT packageOne(IDataObject *DO,IMCCSite *s,POINT pt);
	HRESULT package(bool continual,POINT pt,IDataObject **retDO,IMCCItemSetRenderer **retRend);

	void unScreen(POINTL *ppt);
	HRESULT getdroprect(POINTL ptScreen);
	DWORD determineEffect(DWORD okEffects, DWORD keyState);

	HRESULT siteIt(IUnknown *embed, IMoniker *mkEmbed, IMoniker **retMkSite = 0);
public:
	ContainerEditMode();
	~ContainerEditMode();

REGGY_RID(IDR_ContainerEditMode)

BEGIN_COM_MAP(ContainerEditMode)
	COM_INTERFACE_ENTRY(IContainerEditMode)
	COM_INTERFACE_ENTRY(DIContainerEditMode)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
	COM_INTERFACE_ENTRY(IDropTarget)
	COM_INTERFACE_ENTRY(INeedSomePointers)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(ContainerEditMode)
   //CONNECTION_POINT_ENTRY(IID_ICEMTransferEvents)
END_CONNECTION_POINT_MAP()

BEGIN_PROPERTY_MAP(ContainerEditMode)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(ContainerEditMode)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_MBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_RBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE, MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_MBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_RBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK, DoubleClick)
	MESSAGE_HANDLER(WM_MBUTTONDBLCLK, DoubleClick)
	MESSAGE_HANDLER(WM_RBUTTONDBLCLK, DoubleClick)
	MESSAGE_HANDLER(WM_KEYDOWN,KeyPress)
	MESSAGE_HANDLER(WM_SETCURSOR,SetCursor)
	COMMAND_ID_HANDLER(CMD_CUT,OnCut)
	COMMAND_ID_HANDLER(CMD_COPY,OnCopy)
	COMMAND_ID_HANDLER(CMD_PASTE,OnPaste)
	COMMAND_ID_HANDLER(CMD_DELETE,OnDelete)
	COMMAND_ID_HANDLER(CMD_SELECTALL,OnSelectAll)
	COMMAND_ID_HANDLER(CMD_INSERTOBJECT,OnInsertObject)
	COMMAND_ID_HANDLER(CMD_INSERTFILE,OnInsertFile)
	COMMAND_RANGE_HANDLER(CMD_FIRSTOBJECTTYPE,CMD_LASTOBJECTTYPE,OnInsertAgain)
	COMMAND_ID_HANDLER(CMD_PROPERTIES,OnProperties)
	COMMAND_ID_HANDLER(CMD_SIZETOFIT,OnZoom)
	COMMAND_RANGE_HANDLER(CMD_ZOOMMASK,CMD_ZOOMMASK+0xfff,OnZoom)
	//COMMAND_ID_HANDLER(CMD_BASEPROPERTIES,OnProperties)
	CHAIN_MSG_MAP(CComControl<ContainerEditMode>)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT KeyPress(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnCut(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnCopy(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnPaste(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnDelete(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnSelectAll(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnInsertObject(WORD, WORD, HWND, BOOL&);
	LRESULT OnInsertAgain(WORD, WORD, HWND, BOOL&);
	LRESULT OnInsertFile(WORD, WORD, HWND, BOOL&);
	LRESULT OnProperties(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);
	LRESULT OnZoom(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL &bHandled);

// IOleControl
	STDMETHODIMP GetControlInfo(LPCONTROLINFO ret);
	STDMETHODIMP OnMnemonic(LPMSG msg);

// IViewObjectEx
	STDMETHODIMP GetViewStatus(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = 0;
		return S_OK;
	}
	STDMETHODIMP QueryHitPoint(DWORD,LPCRECT,POINT,LONG,DWORD *ret) {
		*ret = HITRESULT_TRANSPARENT;
		return S_OK;
	}


// IOleInPlaceObjectWindowless
	STDMETHODIMP GetDropTarget(IDropTarget **ret) {
		if(!m_allowDrop)
			return E_NOTIMPL;
		(*ret = this)->AddRef();
		return S_OK;
	}

// overridden IOleObjectImpl
	STDMETHODIMP SetClientSite(IOleClientSite *val);
// IContainerEditMode
	STDMETHODIMP Initialize(IItemSet *selection, IMCCItemSetActivator *activator,
			IMCCLayer *mainlayer, IMCCDataObjectUnpacker *unpacker);
	STDMETHODIMP PasteData(IDataObject *src,MCC_POS_INFO *mpi);
	STDMETHODIMP get_AllowDrop(VARIANT_BOOL *ret);
	STDMETHODIMP put_AllowDrop(VARIANT_BOOL val);
	STDMETHODIMP put_DefaultDropEffect(DWORD val);
	STDMETHODIMP get_DefaultDropEffect(DWORD *ret);
	STDMETHODIMP put_AllowedDragEffects(DWORD val);
	STDMETHODIMP get_AllowedDragEffects(DWORD *ret);
	STDMETHODIMP put_AllowedDropEffects(DWORD val);
	STDMETHODIMP get_AllowedDropEffects(DWORD *ret);
	STDMETHODIMP AddContextItem(VARIANT_BOOL empty, // if for empty selection
		BSTR text, int cmd, HMENU subMenu);
	STDMETHODIMP RemoveContextItem(VARIANT_BOOL empty, int cmd);
// DIContainerEditMode
	STDMETHODIMP Initialize(DIItemSet *selection, IUnknown *activator,
		DIMCCLayer *mainlayer, IUnknown *unpacker);
	STDMETHODIMP put_AllowedDragEffects(long val);
	STDMETHODIMP get_AllowedDragEffects(long *ret);
	STDMETHODIMP put_AllowedDropEffects(long val);
	STDMETHODIMP get_AllowedDropEffects(long *ret);
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU,OLEMENUGROUPWIDTHS*);
	STDMETHODIMP Deactivate(HMENU menu,OLEMENUGROUPWIDTHS *widths);
// IDropTarget
    STDMETHODIMP DragEnter(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
        DWORD *pdwEffect);
    STDMETHODIMP DragOver(DWORD grfKeyState,POINTL pt,DWORD *pdwEffect);
    STDMETHODIMP DragLeave();
    STDMETHODIMP Drop(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
        DWORD *pdwEffect);
// for IViewObjectImpl
	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);
// overridden IPersistStreamInitImpl
	STDMETHODIMP Load(IStream *s);
	STDMETHODIMP Save(IStream *s,BOOL clearDirty);
// INeedSomePointers
	STDMETHODIMP TakePointers(IEnumUnknown *os,IEnumMoniker *ms);
};

#endif //__CONTAINERMODE_H_
