/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ControlMode.h"

ControlMode::ControlMode() : m_sinking(false),m_state(MODE_HIDDEN) {
	CONSTRUCTORTRACE(ControlMode);
}
ControlMode::~ControlMode() {
	DESTRUCTORTRACE(ControlMode);
}
// DIMCCControlMode
STDMETHODIMP ControlMode::Initialize(DIMCCModeDelegate *del) {
	METHTRACE(ControlMode,Initialize);
	if(del) { // may use a controlmode just for placement.  don't really need to call init in that case!
		DWORD spice; // when a mode delegate gets deleted it releases sinks itself.
		RUN(AtlAdvise(del,GetUnknown(),IID_IMCCMode,&spice));
		m_sinking = true;
		RUN(del->get_State(&m_state));
	}
	return S_OK;
}
STDMETHODIMP ControlMode::put_Site(DIMCCSite *val) {
	return SetSite(val);
}
// DIMCCMode
STDMETHODIMP ControlMode::Activate(VARIANT_BOOL once) {
	METHTRACE(ControlMode,Activate);
	CComQIPtr<IMCCSite> s2 = m_site;
	if(!s2)
		ERR(E_UNEXPECTED);
	return s2->PutBehind(0);
}
STDMETHODIMP ControlMode::Deactivate() {
	METHTRACE(ControlMode,Deactivate);
	return S_OK;
}
// INeedSomePointers
STDMETHODIMP ControlMode::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(ControlMode,TakePointers);
	if(m_sinking)
		return S_FALSE;
	CComPtr<DIMCCModeDelegate> del;
	CComPtr<IUnknown> uncle;
	RUN(os->Next(1,&uncle,0));
	if(uncle)
		RUN(uncle->QueryInterface(riidret(&del)));
	return Initialize(del);
}
// IObjectWithSite
STDMETHODIMP ControlMode::SetSite(IUnknown *pUnkSite) {
	METHTRACE(ControlMode,SetSite);
	// SetSite is not supposed to fail.  But I need to do all the work here.
	// so f-f-forget the rules!
	QISET(m_site = pUnkSite);

	// get Montage, sink size events, get size, and set
	CComPtr<IMCCSite> s2;
	RUN(m_site->QueryInterface(riidret(&s2)));
	CComPtr<IMontageContainer> montage;
	RUN(s2->GetContainer(riidret(&montage)));
	DWORD dwSink;
	RUN(AtlAdvise(montage,(IObjectWithSite*)this,IID_IMontageContainerEvents,&dwSink));
	SIZE si;
	RUN(montage->get_Size(&si));
	RECT rc = rect(si);
	RUN(s2->put_Rect(&rc));
	// follow delegate's state
	if(m_state>=MODE_ACTIVE)
		RUN(Activate(m_state==MODE_ACTIVEONCE));
	return S_OK;
}
STDMETHODIMP ControlMode::GetSite(REFIID riid, void **ppvSite) {
	METHTRACE(ControlMode,GetSite);
	return m_site->QueryInterface(riid,ppvSite);
}
// IMontageContainerEvents
STDMETHODIMP ControlMode::Shutdown() {
	return S_OK;
}
HRESULT ControlMode::Resize(SIZE s) {
	METHTRACE(ControlMode,Resize);
	if(CComQIPtr<IMCCSite> s2 = m_site){
		CComPtr<IMontageContainer> mc;
		RUN(s2->GetContainer(riidret(&mc)));
		RECT extra;
		RUN(mc->get_ExtraArea(&extra));
		RECT rc = rect(s)-extra;
		RUN(s2->put_Rect(&rc));
	}
	return S_OK;
}
HRESULT ControlMode::NewExtent(SIZEL s) {
	METHTRACE(ControlMode,NewExtent);
	return S_FALSE;
}
STDMETHODIMP ControlMode::Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
	return S_FALSE;
}
HRESULT ControlMode::GroupOp(BOOL) {
	METHTRACE(ControlMode,GroupOp);
	return S_FALSE;
}
STDMETHODIMP ControlMode::NewTransform(IMCCTransform *) {
	METHTRACE(ControlMode,NewTransform);
	return S_FALSE;
}
STDMETHODIMP ControlMode::SiteDeleted(IMCCSite*) {
	return S_FALSE;
}
STDMETHODIMP ControlMode::LayerDeleted(IMCCLayer*) {
	return S_FALSE;
}
STDMETHODIMP ControlMode::Shifting(long oldState,long newState) {
	return S_FALSE;
}
// IPersistStreamInit
STDMETHODIMP ControlMode::GetClassID(CLSID *pClassID) {
	METHTRACE(ControlMode,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP ControlMode::IsDirty(void) {
	METHTRACE(ControlMode,IsDirty);
	return S_OK;
}
STDMETHODIMP ControlMode::Load(IStream *pStm) {
	METHTRACE(ControlMode,Load);
	return S_OK;
}
STDMETHODIMP ControlMode::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(ControlMode,Save);
	return S_OK;
}
STDMETHODIMP ControlMode::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(ControlMode,GetSizeMax);
	ERR(E_NOTIMPL);
}
STDMETHODIMP ControlMode::InitNew(void) {
	METHTRACE(ControlMode,InitNew);
	return S_OK;
}
