/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"
STDMETHODIMP Site::get_Container(DIMontageContainer **ret) {
    METHTRACE(Site,get_Container);
	return GetContainer(riidret(ret));
}
STDMETHODIMP Site::get_RelMoniker(IUnknown **ret) {
    METHTRACE(Site,get_RelMoniker);
	return get_RelMoniker((IMoniker**)ret);
}
STDMETHODIMP Site::put_RelMoniker(IUnknown *val) {
    METHTRACE(Site,put_RelMoniker);
	CComPtr<IMoniker> mk2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&mk2)));
	return put_RelMoniker(mk2);
}
STDMETHODIMP Site::get_Control(IUnknown **ret) {
    METHTRACE(Site,get_Control);
	return GetControl(riidret(ret));
}
STDMETHODIMP Site::get_Layer(DIMCCLayer **ret) {
    METHTRACE(Site,get_Layer);
	CComPtr<IMCCLayer> l;
	RUNRET(get_Layer(&l));
	if(!l)
		return *ret = 0,S_OK;
	return l.QueryInterface(ret);
}
STDMETHODIMP Site::put_Layer(DIMCCLayer *parm) {
    METHTRACE(Site,put_Layer);
	CComPtr<IMCCLayer> l2;
	if(parm)
		RUNRET(parm->QueryInterface(riidret(&l2)));
	return put_Layer(l2);
}
STDMETHODIMP Site::get_Left(long *ret) {
    METHTRACE(Site,get_Left);
	if(!m_bRectValid)
		ERR(E_PENDING);
	*ret = m_windRect.left;
	return S_OK;
}
STDMETHODIMP Site::get_Top(long *ret) {
    METHTRACE(Site,get_Top);
	if(!m_bRectValid)
		ERR(E_PENDING);
	*ret = m_windRect.top;
	return S_OK;
}
STDMETHODIMP Site::get_Right(long *ret) {
    METHTRACE(Site,get_Right);
	if(!m_bRectValid)
		ERR(E_PENDING);
	*ret = m_windRect.right;
	return S_OK;
}
STDMETHODIMP Site::get_Bottom(long *ret) {
    METHTRACE(Site,get_Bottom);
	if(!m_bRectValid)
		ERR(E_PENDING);
	*ret = m_windRect.bottom;
	return S_OK;
}
STDMETHODIMP Site::SetPos(long left, long top, long right, long bottom) {
    METHTRACE(Site,SetPos);
	RECT rc = {left,top,right,bottom};
	return put_Rect(&rc);
}
STDMETHODIMP Site::get_VirtLeft(long *ret) {
    METHTRACE(Site,get_VirtLeft);
	if(!m_bRectValid || !m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	*ret = m_virtRect.left;
	return S_OK;
}
STDMETHODIMP Site::get_VirtTop(long *ret) {
    METHTRACE(Site,get_VirtTop);
	if(!m_bRectValid || !m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	*ret = m_virtRect.top;
	return S_OK;
}
STDMETHODIMP Site::get_VirtRight(long *ret) {
    METHTRACE(Site,get_VirtRight);
	if(!m_bRectValid || !m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	*ret = m_virtRect.right;
	return S_OK;
}
STDMETHODIMP Site::get_VirtBottom(long *ret) {
    METHTRACE(Site,get_VirtBottom);
	if(!m_bRectValid || !m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	*ret = m_virtRect.bottom;
	return S_OK;
}
STDMETHODIMP Site::SetVirtPos(long left, long top, long right, long bottom) {
    METHTRACE(Site,SetVirtPos);
	RECTL rcl = {left,top,right,bottom};
	return put_VirtRect(&rcl);
}
STDMETHODIMP Site::PutBefore(DIMCCSite *other) {
    METHTRACE(Site,PutBefore);
	CComPtr<IMCCSite> s2;
	if(other)
		RUNRET(other->QueryInterface(riidret(&s2)));
	return PutBefore(s2);
}
STDMETHODIMP Site::PutBehind(DIMCCSite *other) {
    METHTRACE(Site,PutBehind);
	CComPtr<IMCCSite> s2;
	if(other)
		RUNRET(other->QueryInterface(riidret(&s2)));
	return PutBehind(s2);
}
