/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <stdafx.h>
#include "Container.h"
#include "Collection.h"

// dispatch interface implemented almost as a wrapper on custom interface

#define VAL2(I)\
	CComPtr<I> val2;\
	if(val)\
		RUN(val->QueryInterface(riidret(&val2)))

STDMETHODIMP Container::get_Group(DIGroup **ret) {
    METHTRACE(Container,get_Group);
	return GetMainGroup(riidret(ret));
}
STDMETHODIMP Container::put_Group(DIGroup *val) {
    METHTRACE(Container,put_Group);
	VAL2(IGroup);
	return put_Group(val2);
}
STDMETHODIMP Container::get_Bootstrapper(IUnknown **ret) {
    METHTRACE(Container,get_Bootstrapper);
	return get_Bootstrapper((IInitializeOther**)ret);
}
STDMETHODIMP Container::put_Bootstrapper(IUnknown *val) {
    METHTRACE(Container,put_Bootstrapper);
	VAL2(IInitializeOther);
	return put_Bootstrapper(val2);
}
STDMETHODIMP Container::get_RelMoniker(IUnknown **ret) {
    METHTRACE(Container,get_RelMoniker);
	return get_RelMoniker((IMoniker**)ret);
}
STDMETHODIMP Container::put_RelMoniker(IUnknown *val) {
    METHTRACE(Container,put_RelMoniker);
	VAL2(IMoniker);
	return put_RelMoniker(val2);
}
STDMETHODIMP Container::get_Transform(IUnknown **ret) {
    METHTRACE(Container,get_Transform);
	return get_Transform((IMCCTransform**)ret);
}
STDMETHODIMP Container::put_Transform(IUnknown *val) {
    METHTRACE(Container,put_Transform);
	VAL2(IMCCTransform);
	return put_Transform(val2);
}
STDMETHODIMP Container::get_ToolPanel(IUnknown **ret) {
    METHTRACE(Container,get_ToolPanel);
	return get_ToolPanel((IMCCToolPanel**)ret);
}
STDMETHODIMP Container::put_ToolPanel(IUnknown *val) {
    METHTRACE(Container,put_ToolPanel);
	VAL2(IMCCToolPanel);
	return put_ToolPanel(val2);
}
STDMETHODIMP Container::get_Stuff(IDispatch **ret) {
    METHTRACE(Container,get_Stuff);
	(*ret = m_stuff)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::SetAreas(long eleft, long etop, long eright, long ebottom,long rleft, long rtop, long rright, long rbottom) {
    METHTRACE(Container,SetAreas);
	RECT extra= {eleft,etop,eright,ebottom},
        reserved = {rleft,rtop,rright,rbottom};
	return SetAreas(extra,reserved);
}
STDMETHODIMP Container::GetExtraArea(long *left, long *top, long *right, long *bottom) {
    METHTRACE(Container,GetExtraArea);
	*left = m_extraArea.left;
	*top = m_extraArea.top;
	*right = m_extraArea.right;
	*bottom = m_extraArea.bottom;
	return S_OK;
}
STDMETHODIMP Container::GetReservedArea(long *left, long *top, long *right, long *bottom) {
    METHTRACE(Container,GetReservedArea);
	*left = m_reservedArea.left;
	*top = m_reservedArea.top;
	*right = m_reservedArea.right;
	*bottom = m_reservedArea.bottom;
	return S_OK;
}
STDMETHODIMP Container::get_State(int *ret) {
    METHTRACE(Container,get_State);
	*ret = m_state;
	return S_OK;
}
STDMETHODIMP Container::NewLayer(DIMCCLayer **ret) {
    METHTRACE(Container,NewLayer);
	CComPtr<IMCCLayer> l;
	RUNRET(new_Layer(&l));
	return l.QueryInterface(ret);
}
STDMETHODIMP Container::NewSite(DIMCCSite **ret) {
    METHTRACE(Container,NewSite);
	CComPtr<IMCCSite> s;
	RUNRET(new_Site(&s));
	return s.QueryInterface(ret);
}
STDMETHODIMP Container::get_Layers(IDispatch **ret) {
    METHTRACE(Container,get_Layers);
	(*ret = new collection(make_colldata(m_layers)))->AddRef();
	return S_OK;
}
