/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DataObject.h"

// seems to change this pointer with optimizations!
#pragma optimize("",off)
void mediumEntry::releaseMedium() {
	if(stgm.tymed!=TYMED_NULL) {
		ReleaseStgMedium(&stgm);
		stgm.tymed = TYMED_NULL;
	}
}
HRESULT mediumEntry::detach() {
    METHTRACE(mediumEntry,detach);
	if(renderer) {
		if(mintOreo) {
			RUN(renderer->DUnadvise(mintOreo));
			mintOreo = 0;
		}
		renderer.Release();
	}
	return S_OK;
}
HRESULT mediumEntry::clear() {
    METHTRACE(mediumEntry,clear);
	RUN(detach());
	releaseMedium();
	return S_OK;
}
//#pragma optimize("",on)
// mediumEntry IAdviseSink implementation
// IUnknown
STDMETHODIMP_(ULONG) mediumEntry::AddRef() {
	return 1; // client doesn't control deletion
}
STDMETHODIMP_(ULONG) mediumEntry::Release() {
	return 1;
}
STDMETHODIMP mediumEntry::QueryInterface(REFIID riid,void **ppRet) {
	if(riid==IID_IAdviseSink)
		*ppRet = (IAdviseSink*)this;
	else if(riid==IID_IUnknown)
		*ppRet = (IUnknown*)this;
	else ERR(E_NOINTERFACE);
	return S_OK;
}
// IAdviseSink
void STDMETHODCALLTYPE mediumEntry::OnDataChange(FORMATETC *pFormatetc,STGMEDIUM *pStgmed) {
	releaseMedium();
}
void STDMETHODCALLTYPE mediumEntry::OnViewChange(DWORD dwAspect,LONG lindex) {
	_ASSERTE(false);
}
void STDMETHODCALLTYPE mediumEntry::OnRename(IMoniker *pmk) {
	_ASSERTE(false);
}
void STDMETHODCALLTYPE mediumEntry::OnSave(void) {
	_ASSERTE(false);
}
void STDMETHODCALLTYPE mediumEntry::OnClose(void) {
	_ASSERTE(false);
}


const WCHAR *g_szIndex = L"Index";
const ULARGE_INTEGER MOST = castULarge(0xfffffff); // disputable

HGLOBAL GlobalCopy(HGLOBAL hGlobal) {
	DWORD size = GlobalSize(hGlobal);
	HGLOBAL ret = GlobalAlloc(GHND,size);
	void *src = GlobalLock(hGlobal), *dst = GlobalLock(ret);
	memcpy(dst,src,size);
	GlobalUnlock(hGlobal);
	GlobalUnlock(ret);
	return ret;
}

// I don't trust the standard CopyStgMedium because there's no docs
HRESULT MyCopyStgMedium(const STGMEDIUM *src, STGMEDIUM *dest) {
	FUNTRACE(MyCopyStgMedium);
	memset(dest,0,sizeof(STGMEDIUM));
	dest->tymed = src->tymed;
	dest->pUnkForRelease = 0; // always fully owned by dest
	switch(src->tymed) {
    case TYMED_GDI: // too hard?
        CopyStgMedium(src,dest);
        break;
    case TYMED_MFPICT:
		{
			METAFILEPICT *pMFP = (METAFILEPICT*)GlobalLock(src->hMetaFilePict);
			HMETAFILE hMFCopy = CopyMetaFile(pMFP->hMF,0);
			GlobalUnlock(src->hMetaFilePict);
			dest->hMetaFilePict = GlobalCopy(src->hMetaFilePict);
			pMFP = (METAFILEPICT*)GlobalLock(dest->hMetaFilePict);
			pMFP->hMF = hMFCopy;
			GlobalUnlock(dest->hMetaFilePict);
		}
		break;
	case TYMED_ENHMF:
		dest->hEnhMetaFile = CopyEnhMetaFile(src->hEnhMetaFile,0);
		break;
	case TYMED_HGLOBAL:
		dest->hGlobal = GlobalCopy(src->hGlobal);
		break;
	case TYMED_FILE:
		dest->lpszFileName = (LPOLESTR)CoTaskMemAlloc(wcslen(src->lpszFileName));
		wcscpy(dest->lpszFileName,src->lpszFileName);
		break;
	case TYMED_ISTREAM:
		SRUN(src->pstm->Clone(&dest->pstm));
		SRUN(dest->pstm->Seek(castLarge(0),STREAM_SEEK_SET,0));
		break;
	case TYMED_ISTORAGE: {
		(dest->pstg = src->pstg)->AddRef();
		break;
	}
	default: //TYMED_GDI (bitmaps)
		SERR(E_NOTIMPL);
	}
	return S_OK;
}
// when releasing storage, need to copy out streams & storages
HRESULT CopyStream(IStream *src,IStream **ret) {
    FUNTRACE(CopyStream);
	SRUN(src->Seek(castLarge(0),STREAM_SEEK_CUR,0));
	SRUN(CreateStreamOnHGlobal(0,true,ret));
	SRUN(src->CopyTo(*ret,MOST,0,0));
	return S_OK;
}
HRESULT CopyStorage(IStorage *src,IStorage **ret) {
    FUNTRACE(CopyStorage);
	CComPtr<ILockBytes> bytes;
	SRUN(CreateILockBytesOnHGlobal(0,true,&bytes));
	SRUN(StgCreateDocfileOnILockBytes(bytes,STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_CREATE,0,ret));
	SRUN(src->CopyTo(0,0,0,*ret));
	return S_OK;
}
DataObject::DataObject() {
	CONSTRUCTORTRACE(DataObject);
}
DataObject::~DataObject() {
	DESTRUCTORTRACE(DataObject);
	for(mediumMap::iterator i = m_data.begin();i!=m_data.end();i++)
		RUN(i->second.clear());
}
DataObjectPersistent::DataObjectPersistent() : m_dirty(false) {
	CONSTRUCTORTRACE(DataObjectPersistent);
}
DataObjectPersistent::~DataObjectPersistent() {
	DESTRUCTORTRACE(DataObjectPersistent);
}
// bug: these should be wrapped in DataObjectPersistent to set dirty bit
// IMCCDataObject
STDMETHODIMP DataObject::Prioritize(CLIPFORMAT cf, long order) {
	METHTRACE(DataObject,Prioritize);
	for(cfOrder::iterator i = m_order.begin();i!=m_order.end();++i)
		if(i->second==cf) {
			m_order.erase(i);
			m_order.insert(make_pair_first_const(order,cf));
			return S_OK;
		}
	// must already be in the order (order entries can only be created by SetData)
	ERR(E_FAIL);
}
STDMETHODIMP DataObject::SetDelayedData(CLIPFORMAT cf, IDataObject *renderer) {
	METHTRACE(DataObject,SetDelayedData);
	HRESULT hr;
	// broken m_data[cf]... {sigh}. but w/ [] you can't find out whether it was an insert!
	mediumMap::iterator i = m_data.find(cf);
	if(i==m_data.end()) {
		m_order.insert(make_pair_first_const(long(0),cf)); // default order
		i = m_data.insert(i,make_pair_first_const(cf,mediumEntry()));
	}
	mediumEntry &entry = i->second;
	if(entry.renderer && entry.mintOreo) {
		hr = entry.renderer->DUnadvise(entry.mintOreo);
		entry.mintOreo = 0;
	}
	if(entry.renderer = renderer) {
		FORMATETC fetc = { cf,0,DVASPECT_CONTENT,-1,-1};
		hr = entry.renderer->DAdvise(&fetc,ADVF_NODATA,&entry,&entry.mintOreo); // may fail
	}
	return S_OK;
}
HRESULT DataObject::fix(mediumMap::iterator i) {
	METHTRACE(DataObject,fix);
	mediumEntry &entry = i->second;
	if(entry.renderer) {
		if(!entry.stgm.tymed) {
			FORMATETC fetc = {i->first,0,DVASPECT_CONTENT,-1,-1};
			RUN(entry.renderer->GetData(&fetc,&entry.stgm));
		}
		RUN(entry.detach());
	}
	return S_OK;
}
STDMETHODIMP DataObject::Fix(CLIPFORMAT cf) {
	METHTRACE(DataObject,Fix);
	if(cf) {
		mediumMap::iterator i = m_data.find(cf);
		if(i==m_data.end())
			ERR(E_FAIL);
		else return fix(i);
	}
	else {
		for(mediumMap::iterator i = m_data.begin();i!=m_data.end();++i)
			RUN(fix(i));
		return S_OK;
	}
}
// DIMCCDataObject
STDMETHODIMP DataObject::CreateEmbeddedObject(IUnknown *src) {
    METHTRACE(DataObject,CreateEmbeddedObject);
	StgMedium stgm;
	FormatEtc fetc(g_clipformats.embedded);
	// precedence: IPersistStream[Init], IPersistStorage, IPersist
	bool streamIt = false;
	CComPtr<IUnknown> test;
	if(SUCCEEDED(src->QueryInterface(IID_IPersistStream,(void**)&test)) ||
			SUCCEEDED(src->QueryInterface(IID_IPersistStreamInit,(void**)&test)))
		streamIt = true;
	else if(CComQIPtr<IPersistStorage> stor = src) {
		stgm.tymed = fetc.tymed = TYMED_ISTORAGE;
		CComPtr<ILockBytes> lb;
		RUN(CreateILockBytesOnHGlobal(0,true,&lb));
		RUN(StgCreateDocfileOnILockBytes(lb,STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_CREATE,0,&stgm.pstg));
		CLSID clsid;
		RUN(stor->GetClassID(&clsid));
		RUN(WriteClassStg(stgm.pstg,clsid));
		RUN(stor->Save(stgm.pstg,false));
	}
	else  if(SUCCEEDED(src->QueryInterface(IID_IPersist,(void**)&test)))
		streamIt = true;
	else ERR(E_FAIL);
	if(streamIt) {
		stgm.tymed = fetc.tymed = TYMED_ISTREAM;
		RUN(CreateStreamOnHGlobal(0,true,&stgm.pstm));
		RUN(FixedOleSaveToStream(src,stgm.pstm));
	}
	return SetData(&fetc,&stgm,true);
}
STDMETHODIMP DataObject::CreateEmbeddedByClass(BSTR strCLSID) {
    METHTRACE(DataObject,CreateEmbeddedByClass);
	CComPtr<IUnknown> u;
	CLSID clsid;
	RUNRET(CLSIDFromString(strCLSID,&clsid));
	RUNRET(CREATE(u,clsid)); TRACECREATEV(clsid);
	return CreateEmbeddedObject(u);
}
// IDataObject
STDMETHODIMP DataObject::GetData(FORMATETC *pFetc,STGMEDIUM *pmedium) {
	METHTRACE(DataObject,GetData);
	/*
	TCHAR buf[200];
	int len;
	if(len = GetClipboardFormatName(pFetc->cfFormat,buf,200))
		ATLTRACE2(_T("GetData for \"%s\"\n"),buf);
	*/
	if(pFetc->lindex!=-1)
		ERR(DV_E_LINDEX);
	if(pFetc->dwAspect!=DVASPECT_CONTENT)
		ERR(DV_E_DVASPECT);
	mediumMap::iterator i = m_data.find(pFetc->cfFormat);
	if(i==m_data.end())
		ERR(DV_E_FORMATETC);
	mediumEntry &entry = i->second;
	if(entry.stgm.tymed==TYMED_NULL && entry.renderer)  // has not been rendered yet
		RUN(entry.renderer->GetData(pFetc,&entry.stgm));
	STGMEDIUM &stgm = i->second.stgm;
	if((stgm.tymed&pFetc->tymed) == 0)
		ERR(DV_E_TYMED);
	RUN(MyCopyStgMedium(&stgm,pmedium));
	return S_OK;
}
STDMETHODIMP DataObject::GetDataHere(FORMATETC *pFetc,STGMEDIUM *pmedium) {
	METHTRACE(DataObject,GetDataHere);
    /*
	TCHAR buf[200];
	int len;
	if(len = GetClipboardFormatName(pFetc->cfFormat,buf,200))
		ATLTRACE2(_T("GetDataHere for \"%s\"\n"),buf);
    */
	HRESULT hr;
	if(pFetc->lindex!=-1)
		ERR(DV_E_LINDEX);
	if(pFetc->dwAspect!=DVASPECT_CONTENT)
		ERR(DV_E_DVASPECT);
	mediumMap::iterator i = m_data.find(pFetc->cfFormat);
	if(i==m_data.end())
		ERR(DV_E_FORMATETC);
	// hack?  if you do GetData on a delayed-rendering format, the rendering gets cached
	// however if you do GetDataHere, it gets rendered for the client each time
	mediumEntry &entry = i->second;
	if(entry.stgm.tymed==TYMED_NULL && entry.renderer)
		return entry.renderer->GetDataHere(pFetc,pmedium);
	if(entry.stgm.tymed != pmedium->tymed)
		ERR(DV_E_TYMED);
	switch(entry.stgm.tymed) {
	case TYMED_HGLOBAL:
		if(GlobalSize(pmedium->hGlobal)<GlobalSize(entry.stgm.hGlobal))
			ERR(STG_E_MEDIUMFULL);
		{
			DWORD count = GlobalSize(entry.stgm.hGlobal);
			void *src = GlobalLock(entry.stgm.hGlobal),
				*dest = GlobalLock(pmedium->hGlobal);
			memcpy(dest,src,count);
			GlobalUnlock(entry.stgm.hGlobal);
			GlobalUnlock(pmedium->hGlobal);
		}
		return S_OK;
	case TYMED_ISTORAGE:
		hr = entry.stgm.pstg->CopyTo(0,0,0,pmedium->pstg);
		return hr;
	case TYMED_ISTREAM:
		RUN(entry.stgm.pstm->CopyTo(pmedium->pstm,MOST,0,0));
		RUN(entry.stgm.pstm->Seek(castLarge(0),STREAM_SEEK_SET,0));
		return S_OK;
	default:
		ERR(DV_E_TYMED);
	}
}
STDMETHODIMP DataObject::QueryGetData(FORMATETC *pFetc) {
	METHTRACE(DataObject,QueryGetData);
	/*
	TCHAR buf[200];
	int len;
	if(len = GetClipboardFormatName(pFetc->cfFormat,buf,200))
		ATLTRACE2(_T("QueryGetData for \"%s\"\n"),buf);
	*/
	if(pFetc->lindex!=-1)
		ERR(DV_E_LINDEX);
	if(pFetc->dwAspect!=DVASPECT_CONTENT)
		ERR(DV_E_DVASPECT);
	mediumMap::iterator i = m_data.find(pFetc->cfFormat);
	if(i==m_data.end())
		ERR(DV_E_FORMATETC);
	mediumEntry &entry = i->second;
	if(entry.stgm.tymed==TYMED_NULL && entry.renderer)
		return entry.renderer->QueryGetData(pFetc);
	// this is lenient: QGD is supposed to check if a particular fetc would work not a class of fetzes
	if((i->second.stgm.tymed&pFetc->tymed) == 0)
		ERR(DV_E_TYMED);
	return S_OK;
}
STDMETHODIMP DataObject::GetCanonicalFormatEtc(FORMATETC *pFetcIn,FORMATETC *pFetcOut) {
	METHTRACE(DataObject,GetCanonicalFormatEtc);
	RUN(QueryGetData(pFetcIn)); // make sure format exists!
	mediumEntry &entry = m_data[pFetcIn->cfFormat];
	if(entry.renderer)
		return entry.renderer->GetCanonicalFormatEtc(pFetcIn,pFetcOut);
	*pFetcOut = *pFetcIn;
	pFetcOut->ptd = 0;
	return DATA_S_SAMEFORMATETC;
}
STDMETHODIMP DataObject::SetData(FORMATETC *pFetc,STGMEDIUM *pmedium,BOOL fRelease) {
	METHTRACE(DataObject,SetData);
	mediumMap::iterator i = m_data.find(pFetc->cfFormat);
	if(i==m_data.end()) {
		m_order.insert(make_pair_first_const(long(0),pFetc->cfFormat)); // default order
		i = m_data.insert(i,make_pair_first_const(pFetc->cfFormat,mediumEntry()));
	}
	mediumEntry &entry = i->second;
	entry.clear();
	entry.local = false;
	if(fRelease) {
		entry.stgm = *pmedium;
		// weird special case: only streams have to be reset when "grabbed"
		// by a data object.
		if(pmedium->tymed==TYMED_ISTREAM)
			RUN(pmedium->pstm->Seek(castLarge(0),STREAM_SEEK_SET,0));
	}
	else
		RUN(MyCopyStgMedium(pmedium,&entry.stgm));
	return S_OK;
}
// there's no direct way to ask a data object what tymed is appropriate for a clipformat
HRESULT getTymed(IDataObject *DO,CLIPFORMAT cf,DWORD *ret) {
    FUNTRACE(getTymed);
	*ret = TYMED_NULL;
	CComPtr<IEnumFORMATETC> enumfetc;
	SRUN(DO->EnumFormatEtc(DATADIR_GET,&enumfetc));
	for(formatetc_iterator ifetc(enumfetc);ifetc!=formatetc_iterator();++ifetc)
		if(ifetc->cfFormat==cf) {
			*ret = ifetc->tymed;
			break;
		}
	if(!*ret)
		SERR(E_UNEXPECTED);
	return S_OK;
}
STDMETHODIMP DataObject::EnumFormatEtc(DWORD dwDirection,IEnumFORMATETC **ppenum) {
	METHTRACE(DataObject,EnumFormatEtc);
	if(dwDirection==DATADIR_SET)
		ERR(E_NOTIMPL);
	FORMATETC *ray = new FORMATETC[m_data.size()],
		*fi = ray;
	FORMATETC fetcTemplate;
	fetcTemplate.ptd = 0;
	fetcTemplate.dwAspect = DVASPECT_CONTENT;
	fetcTemplate.lindex = -1;
	for(cfOrder::reverse_iterator i = m_order.rbegin();i!=m_order.rend();++i) {
		// grr. SGI STL (which I'm using cause it has hashtables) doesn't support ->
		// and I've hacked it in some places but seems pointless to add it everywhere
		CLIPFORMAT cf = (*i).second;
		fetcTemplate.cfFormat = cf;
		mediumEntry &entry = m_data[cf];
		fetcTemplate.tymed = entry.stgm.tymed;
		if(!fetcTemplate.tymed) {
			if(!entry.renderer)
				ERR(E_UNEXPECTED);
			RUN(getTymed(entry.renderer,fetcTemplate.cfFormat,&fetcTemplate.tymed));
		}
		*fi++ = fetcTemplate;
	}
	fetcenum *fe = new fetcenum;
	fe->Init(ray,fi,0,AtlFlagTakeOwnership);
	(*ppenum = fe)->AddRef();
	return S_OK;
}
STDMETHODIMP DataObject::DAdvise(FORMATETC *pFetc,DWORD advf,IAdviseSink *pAdvSink,
     DWORD *pdwConnection) {
	METHTRACE(DataObject,DAdvise);
	ERR(OLE_E_ADVISENOTSUPPORTED);
}
STDMETHODIMP DataObject::DUnadvise(DWORD dwConnection) {
	METHTRACE(DataObject,DUnadvise);
	ERR(OLE_E_ADVISENOTSUPPORTED);
}
STDMETHODIMP DataObject::EnumDAdvise(IEnumSTATDATA **ppenumAdvise) {
	METHTRACE(DataObject,EnumDAdvise);
	ERR(OLE_E_ADVISENOTSUPPORTED);
}
// IPersistStorage
STDMETHODIMP DataObjectPersistent::GetClassID(CLSID *pClassID) {
	METHTRACE(DataObjectPersistent,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP DataObjectPersistent::IsDirty(void) {
	METHTRACE(DataObjectPersistent,IsDirty);
	return m_dirty?S_OK:S_FALSE;
}
STDMETHODIMP DataObjectPersistent::InitNew(IStorage *pStg) {
	METHTRACE(DataObjectPersistent,InitNew);
	m_storage = pStg;
	return S_OK;
}
STDMETHODIMP DataObjectPersistent::Load(IStorage *pStg) {
	METHTRACE(DataObjectPersistent,Load);
	USES_CONVERSION;
	m_storage = pStg;
	CComPtr<IStream> index;
	RUN(pStg->OpenStream(g_szIndex,0,STGM_READ|STGM_SHARE_EXCLUSIVE,0,&index));
	cfOrder::size_type n;
	RUN(index->Read(&n,sizeof(cfOrder::size_type),0));
	while(n--) {
		CComBSTR bstr;
		RUN(bstr.ReadFromStream(index));
		CLIPFORMAT cf;
		if(!wcsncmp(bstr,L"format",6))
			cf = _wtoi(bstr+6);
		else
			cf = RegisterClipboardFormat(OLE2T(bstr));
		if(!cf)
			ERR(E_UNEXPECTED);
		long order;
		RUN(index->Read(&order,sizeof(long),0));
		m_order.insert(make_pair_first_const(order,cf));
		mediumEntry &entry = m_data[cf];
		entry.local = true;
		RUN(index->Read(&entry.stgm.tymed,sizeof(TYMED),0));
		switch(entry.stgm.tymed) {
		case TYMED_ISTREAM:
			RUN(pStg->OpenStream(bstr,0,STGM_READWRITE|STGM_SHARE_EXCLUSIVE,0,&entry.stgm.pstm));
			RUN(entry.stgm.pstm->Seek(castLarge(0),STREAM_SEEK_CUR,0));
			break;
		case TYMED_ISTORAGE:
			RUN(pStg->OpenStorage(bstr,0,STGM_READWRITE|STGM_SHARE_EXCLUSIVE,0,0,&entry.stgm.pstg));
			break;
		default:
			ERR(E_UNEXPECTED);
		}
		entry.stgm.pUnkForRelease = 0;
	}
	return S_OK;
}
STDMETHODIMP DataObjectPersistent::Save(IStorage *pStgSave,BOOL fSameAsLoad) {
	METHTRACE(DataObjectPersistent,Save);
	CComPtr<IStream> index;
	RUN(pStgSave->CreateStream(L"Index",STGM_CREATE|STGM_WRITE|STGM_SHARE_EXCLUSIVE,0,0,&index));
	cfOrder::size_type n = 0;
	RUN(index->Write(&n,sizeof(cfOrder::size_type),0));
	for(cfOrder::iterator i = m_order.begin();i!=m_order.end();++i) {
		mediumEntry &entry = m_data[i->second];
		// renderers must reconnect themselves
		if(entry.renderer)
			continue;
		n++;

		// get a name for the format and write that as a bstr to the index
		TCHAR buf[200];
		if(!GetClipboardFormatName(i->second,buf,200))
			wsprintf(buf,_T("format%d"),i->second);
		CComBSTR bstr(buf);
		RUN(bstr.WriteToStream(index));
		// write order
		RUN(index->Write(&i->first,sizeof(DWORD),0));

		// get the data and write that in a subobject with the name
		/*
		// this caused an accessdenied; instead I'll insist that renderers reconnect
		if(!entry.stgm.tymed && entry.renderer) { // force a render (I think pointless to save the renderer)
			FORMATETC fetc;
			fetc.cfFormat = i->second;
			fetc.ptd = 0;
			fetc.dwAspect =	DVASPECT_CONTENT;
			fetc.lindex = -1;
			RUN(getTymed(entry.renderer,i->second,&fetc.tymed));
			RUN(entry.renderer->GetData(&fetc,&entry.stgm));
		}
		*/
		RUN(index->Write(&entry.stgm.tymed,sizeof(TYMED),0));
		// only copy stuff if this is not our storage or the entry was not loaded from here
		if(!fSameAsLoad||!entry.local) {
			switch(entry.stgm.tymed) {
			case TYMED_ISTREAM: {
				CComPtr<IStream> dest;
				RUN(pStgSave->CreateStream(bstr,STGM_CREATE|STGM_WRITE|STGM_SHARE_EXCLUSIVE,0,0,&dest));
				ULONGLONG tuh;
				RUN(entry.stgm.pstm->Seek(castLarge(0),STREAM_SEEK_SET,0)); // just in case
				RUN(entry.stgm.pstm->CopyTo(dest,MOST,&castULarge(tuh),0));
				RUN(entry.stgm.pstm->Seek(castLarge(0),STREAM_SEEK_SET,0));
				break;
			}
			case TYMED_ISTORAGE: {
				CComPtr<IStorage> dest;
				RUN(pStgSave->CreateStorage(bstr,STGM_CREATE|STGM_WRITE|STGM_SHARE_EXCLUSIVE,0,0,&dest));
				RUN(entry.stgm.pstg->CopyTo(0,0,0,dest));
				break;
			}
			default: // sorry, won't implement many things I don't need!
				ERR(E_NOTIMPL);
			}
		}
	}
	RUN(index->Seek(castLarge(0),STREAM_SEEK_SET,0));
	RUN(index->Write(&n,sizeof(cfOrder::size_type),0));
	RUN(pStgSave->Commit(STGC_ONLYIFCURRENT));
	return S_OK;
}
STDMETHODIMP DataObjectPersistent::SaveCompleted(IStorage *pStgNew) {
	METHTRACE(DataObjectPersistent,SaveCompleted);
	if(pStgNew)
		m_storage = pStgNew;
	return S_OK;
}
STDMETHODIMP DataObjectPersistent::HandsOffStorage(void) {
	METHTRACE(DataObjectPersistent,HandsOffStorage);
	for(cfOrder::iterator i = m_order.begin();i!=m_order.end();++i) {
		mediumEntry &entry = m_data[i->second];
		if(entry.local) {
			switch(entry.stgm.tymed) {
			case TYMED_ISTREAM: {
				IStream *src = entry.stgm.pstm;
				RUN(CopyStream(src,&entry.stgm.pstm));
				src->Release();
				break;
			}
			case TYMED_ISTORAGE: {
				IStorage *src = entry.stgm.pstg;
				RUN(CopyStorage(src,&entry.stgm.pstg));
				src->Release();
				break;
			}
			}
			entry.local = false;
		}

	}
	m_storage.Release();
	return S_OK;
}
// IDataObject
STDMETHODIMP DataObjectPersistent::SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease) {
    METHTRACE(DataObjectPersistent,SetData);
	HRESULT hr;
	if(SUCCEEDED(hr = DataObject::SetData(pformatetc,pmedium,fRelease)))
		m_dirty = true;
	return hr;
}
