/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct mediumEntry : public IAdviseSink {
	STGMEDIUM stgm;
	bool local; // whether stream/storage is in my storage
	CComPtr<IDataObject> renderer; // if rendering is delayed
	DWORD mintOreo;
	mediumEntry() : mintOreo(0),local(false) {
		//CONSTRUCTORTRACE(mediumEntry);
		stgm.tymed = TYMED_NULL;
		stgm.pUnkForRelease = 0;
	}
	~mediumEntry() {
		// it appears far more get deleted than created??
		//DESTRUCTORTRACE(mediumEntry);
	}
	void releaseMedium();
	HRESULT detach();
	HRESULT clear();
// IUnknown
	STDMETHODIMP_(ULONG) AddRef();
	STDMETHODIMP_(ULONG) Release();
	STDMETHODIMP QueryInterface(REFIID riid,void **ppRet);
// IAdviseSink
    void STDMETHODCALLTYPE OnDataChange(FORMATETC *pFormatetc,STGMEDIUM *pStgmed);
	void STDMETHODCALLTYPE OnViewChange(DWORD dwAspect,LONG lindex);
    void STDMETHODCALLTYPE OnRename(IMoniker *pmk);
    void STDMETHODCALLTYPE OnSave(void);
    void STDMETHODCALLTYPE OnClose(void);
};
class ATL_NO_VTABLE DataObject :
	public CComObjectRoot,
	public IMCCDataObject,
	public IDispatchImpl<DIMCCDataObject>,
	public IDataObject
{
protected:
	typedef map<CLIPFORMAT,mediumEntry> mediumMap;
	mediumMap m_data;
	// everything has an order (default 0, lowest); there are no orders w/o data.
	typedef multimap<long, CLIPFORMAT> cfOrder;
	cfOrder m_order;
	HRESULT fix(mediumMap::iterator i);
public:
	DataObject();
	~DataObject();
BEGIN_COM_MAP(DataObject)
	COM_INTERFACE_ENTRY(IMCCDataObject)
	COM_INTERFACE_ENTRY(DIMCCDataObject)
	COM_INTERFACE_ENTRY(IDataObject)
END_COM_MAP()
// IMCCDataObject
	STDMETHODIMP Prioritize(CLIPFORMAT cf, long order);
	STDMETHODIMP SetDelayedData(CLIPFORMAT cf, IDataObject *renderer);
	STDMETHODIMP Fix(CLIPFORMAT cf);
// DIMCCDataObjecct
	STDMETHODIMP CreateEmbeddedObject(IUnknown *src);
	STDMETHODIMP CreateEmbeddedByClass(BSTR strCLSID);
// IDataObject
    STDMETHODIMP GetData(FORMATETC *pformatetcIn,STGMEDIUM *pmedium);
    STDMETHODIMP GetDataHere(FORMATETC *pformatetc,STGMEDIUM *pmedium);
    STDMETHODIMP QueryGetData(FORMATETC *pformatetc);
    STDMETHODIMP GetCanonicalFormatEtc(FORMATETC *pformatectIn,FORMATETC *pformatetcOut);
    STDMETHODIMP SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease);
    STDMETHODIMP EnumFormatEtc(DWORD dwDirection,IEnumFORMATETC **ppenumFormatEtc);
    STDMETHODIMP DAdvise(FORMATETC *pformatetc,DWORD advf,IAdviseSink *pAdvSink,
         DWORD *pdwConnection);
    STDMETHODIMP DUnadvise(DWORD dwConnection);
    STDMETHODIMP EnumDAdvise(IEnumSTATDATA **ppenumAdvise);
};
class ATL_NO_VTABLE DataObjectClass :
	public DataObject,
	public CComCoClass<DataObject,&CLSID_MCCDataObject>
{
public:
	REGISTER_GENERIC(L"DataObject")
};
class DataObjectPersistent : public DataObject,
	public CComCoClass<DataObjectPersistent,&CLSID_MCCDataObjectPersistent>,
	public IPersistStorage
{
	bool m_dirty;
	CComPtr<IStorage> m_storage;
public:
	DataObjectPersistent();
	~DataObjectPersistent();
	/*
	typedef CComCoClass<DataObjectPersistent,&CLSID_MCCDataObjectPersistent>::
		_ClassFactoryCreatorClass _ClassFactoryCreatorClass;
	typedef CComCoClass<DataObjectPersistent,&CLSID_MCCDataObjectPersistent>::
		_CreatorClass _CreatorClass;
	static LPCTSTR WINAPI GetObjectDescription() { return CComCoClass<DataObjectPersistent,&CLSID_MCCDataObjectPersistent>::GetObjectDescription(); }
	*/
	REGISTER_GENERIC(L"DataObjectPersistent")
BEGIN_COM_MAP(DataObjectPersistent)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY_CHAIN(DataObject)
END_COM_MAP()
// IPersistStorage
    STDMETHODIMP GetClassID(CLSID *pClassID);
    STDMETHODIMP IsDirty(void);
    STDMETHODIMP InitNew(IStorage *pStg);
    STDMETHODIMP Load(IStorage *pStg);
    STDMETHODIMP Save(IStorage *pStgSave,BOOL fSameAsLoad);
    STDMETHODIMP SaveCompleted(IStorage *pStgNew);
    STDMETHODIMP HandsOffStorage(void);
// IDataObject (wrapper)
    STDMETHODIMP SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease);
};