/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IDataObject for Container
#include "stdafx.h"
#include "Container.h"

#include "Metafile.h"

// enhanced metafiles are greatly preferred, because they support
// beziers, but only Excel seems to ask for them!
STDMETHODIMP Container::GetData(
     FORMATETC *pformatetcIn,
     STGMEDIUM *pmedium) {
	IMETHTRACE(Container,IDataObject,GetData);
	if(!pmedium)
		ERR(E_POINTER);
	if(!(pformatetcIn->tymed & (TYMED_MFPICT|TYMED_ENHMF)))
		ERR(DATA_E_FORMATETC);
	if(!m_renderer)
		ERR(E_PENDING);

	// always use GetExtent size
	SIZEL ext;
	RUN(GetExtent(DVASPECT_CONTENT,&ext));
	// if scrolling, use scroll offset.  otherwise use upper left
	CComQIPtr<IMCCScrollingTransform> st;
	POINTL ofs;
	if(st = m_transform)
		RUN(st->get_Offset(&ofs));
	else {
		RECTL bounds;
		RUN(m_renderer->get_Bounds(&bounds));
		ofs = pointl(bounds.left,bounds.top);
	}

	RECTL r = {ofs.x,ofs.y,ofs.x+ext.cx,ofs.y-ext.cy};
	RUN(m_renderer->SetBoundary(&r));

	RUN(CComQIPtr<IDataObject>(m_renderer)->GetData(pformatetcIn,pmedium));

	RUN(m_renderer->SetBoundary(0));
	return S_OK;
}

STDMETHODIMP Container::GetDataHere(
     FORMATETC *pformatetc,
     STGMEDIUM *pmedium) {
	IMETHTRACE(Container,IDataObject,GetDataHere);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Container::QueryGetData(FORMATETC *pformatetc) {
	IMETHTRACE(Container,IDataObject,QueryGetData);
	switch(pformatetc->cfFormat) {
	case 0: // render data
		return (pformatetc->tymed|TYMED_ENHMF||pformatetc->tymed|TYMED_MFPICT)?S_OK:DV_E_TYMED;
	case CF_METAFILEPICT:
		return (pformatetc->tymed|TYMED_MFPICT)?S_OK:DV_E_TYMED;
	case CF_ENHMETAFILE:
		return (pformatetc->tymed|TYMED_ENHMF)?S_OK:DV_E_TYMED;
	default:
		return DV_E_FORMATETC;
	}
}

STDMETHODIMP Container::GetCanonicalFormatEtc(
     FORMATETC *pformatectIn,
     FORMATETC *pformatetcOut) {
	IMETHTRACE(Container,IDataObject,GetCanonicalFormatEtc);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Container::SetData(
     FORMATETC *pformatetc,
     STGMEDIUM *pmedium,
     BOOL fRelease) {
	IMETHTRACE(Container,IDataObject,SetData);
	ERR(E_NOTIMPL);
}

/*
// I would love to write to ENHMETAFILE but noone seems to ask for it!
FORMATETC g_formats[2] = {//{CF_ENHMETAFILE,0,DVASPECT_CONTENT,-1,TYMED_ENHMF},
	{CF_METAFILEPICT,0,DVASPECT_CONTENT,-1,TYMED_MFPICT}};
const int g_nformats = sizeof(g_formats)/sizeof(FORMATETC);
typedef CComObject<CComEnum<IEnumFORMATETC,&IID_IEnumFORMATETC,FORMATETC, _Copy<FORMATETC> > > enumfetz;
*/
STDMETHODIMP Container::EnumFormatEtc(
     DWORD dwDirection,
     IEnumFORMATETC **ppenumFormatEtc) {
	IMETHTRACE(Container,IDataObject,EnumFormatEtc);
	RUN(OleRegEnumFormatEtc(GetObjectCLSID(),dwDirection,ppenumFormatEtc));
#ifdef _DEBUG
	FORMATETC fetcray[9];
	ULONG count;
	RUN((*ppenumFormatEtc)->Next(9,fetcray,&count));
#endif
/*
	enumfetz *ef = new enumfetz;
	ef->Init(g_formats,g_formats+g_nformats,(IDataObject*)this,AtlFlagNoCopy);
	(*ppenumFormatEtc = ef)->AddRef();
*/
	return S_OK;
}

STDMETHODIMP Container::DAdvise(
     FORMATETC *pformatetc,
     DWORD advf,
     IAdviseSink *pAdvSink,
     DWORD *pdwConnection) {
	IMETHTRACE(Container,IDataObject,DAdvise);
	if(!m_dadvise)
		RUN(CreateDataAdviseHolder(&m_dadvise));
	return m_dadvise->Advise(this,pformatetc,advf,pAdvSink,pdwConnection);
}

STDMETHODIMP Container::DUnadvise(DWORD dwConnection) {
	IMETHTRACE(Container,IDataObject,DUnadvise);
	if(!m_dadvise)
		ERR(OLE_E_NOCONNECTION);
	return m_dadvise->Unadvise(dwConnection);
}

STDMETHODIMP Container::EnumDAdvise(IEnumSTATDATA **ppenumAdvise) {
	IMETHTRACE(Container,IDataObject,EnumDAdvise);
	if(!m_dadvise)
		ERR(OLE_E_NOCONNECTION);
	return m_dadvise->EnumAdvise(ppenumAdvise);
}
