/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DataObjectReader.h"

DataObjectReader::DataObjectReader() {
	CONSTRUCTORTRACE(DataObjectReader);
}
DataObjectReader::~DataObjectReader() {
	DESTRUCTORTRACE(DataObjectReader);
	for(clipmap::iterator i = m_oneFormats.begin();i!=m_oneFormats.end();++i)
		i->second->Release();
	for(i = m_manyFormats.begin();i!=m_manyFormats.end();++i)
		i->second->Release();
}
// IMCCDataObjectReader
STDMETHODIMP DataObjectReader::RegisterUnpacker(CLIPFORMAT cf, VARIANT_BOOL many, IMCCDataObjectUnpacker *unpacker) {
	METHTRACE(DataObjectReader,RegisterUnpacker);
	if(!unpacker)
		ERR(E_POINTER);
	clipmap &formats = many?m_manyFormats:m_oneFormats;
	IMCCDataObjectUnpacker *&entry = formats[cf];
	if(entry)
		entry->Release();
	(entry = unpacker)->AddRef();
	return S_OK;
}
STDMETHODIMP DataObjectReader::UnregisterUnpacker(VARIANT_BOOL many, CLIPFORMAT cf) {
	METHTRACE(DataObjectReader,UnregisterUnpacker);
	clipmap &formats = many?m_manyFormats:m_oneFormats;
	clipmap::iterator i = formats.find(cf);
	if(i==formats.end())
		ERR(E_FAIL);
	i->second->Release();
	formats.erase(i);
	return S_OK;
}
// IMCCDataObjectUnpacker
HRESULT DataObjectReader::tryOneFormat(IDataObject *data, FORMATETC *fetc, LPOLESTR baseName, IItemSet *isetPaste,IMCCSitePersistor **ret) {
	METHTRACE(DataObjectReader,tryOneFormat);
	if(!fetc)
		ERR(E_POINTER);
	CLIPFORMAT cf = fetc->cfFormat;
	clipmap::iterator j = m_oneFormats.find(cf);
	if(j==m_oneFormats.end())
		ERR(E_FAIL);
	return j->second->UnpackOne(data,fetc,baseName,isetPaste,ret);
}
STDMETHODIMP DataObjectReader::UnpackOne(IDataObject *data, FORMATETC *fetc, LPOLESTR baseName, IItemSet *isetPaste,IMCCSitePersistor **ret) {
	METHTRACE(DataObjectReader,UnpackOne);
	HRESULT hr;
	if(!data)
		ERR(E_INVALIDARG);
	if(fetc)
		return tryOneFormat(data,fetc,baseName,isetPaste,ret);
	CComPtr<IEnumFORMATETC> ef;
	if(SUCCEEDED(data->EnumFormatEtc(DATADIR_GET,&ef)))
		for(formatetc_iterator i(ef);i!=formatetc_iterator();++i) {
			FORMATETC fetc = *i;
			if(SUCCEEDED(hr = tryOneFormat(data,&fetc,baseName,isetPaste,ret)))
				return hr;
		}
	else // no enumerator; DO probably just supports one format so try all (!)
		for(clipmap::iterator i(m_oneFormats.begin());i!=m_oneFormats.end();++i)
			if(SUCCEEDED(hr = i->second->UnpackOne(data,0,baseName,isetPaste,ret)))
				return hr;
	ERR(E_FAIL);
}
void wrapU(IUnknown *sp,IEnumUnknown **ret) {
	typedef CComObject< CComEnum<IEnumUnknown,&IID_IEnumUnknown,IUnknown*,_CopyInterface<IUnknown> > > enum_unk;
	enum_unk *eu = new enum_unk;
	eu->Init(&sp,&sp + 1,0,AtlFlagCopy);
	(*ret = eu)->AddRef();
}
HRESULT DataObjectReader::tryManyFormat(IDataObject *data, FORMATETC *fetc, IItemSet *isetPaste,IEnumUnknown **ret) {
	METHTRACE(DataObjectReader,tryManyFormat);
	if(!fetc)
		ERR(E_POINTER);
	CLIPFORMAT cf = fetc->cfFormat;
	clipmap::iterator j = m_manyFormats.find(cf);
	if(j!=m_manyFormats.end())
		return j->second->UnpackMany(data,fetc,isetPaste,ret);
	CComPtr<IMCCSitePersistor> sp;
	RUNRET(tryOneFormat(data,fetc,0,isetPaste,&sp));
	wrapU(sp,ret);
	return S_OK;
}
STDMETHODIMP DataObjectReader::UnpackMany(IDataObject *data, FORMATETC *fetc, IItemSet *isetPaste,IEnumUnknown **ret) {
	METHTRACE(DataObjectReader,UnpackMany);
	HRESULT hr;
	if(fetc)
		return tryManyFormat(data,fetc,isetPaste,ret);
	CComPtr<IEnumFORMATETC> ef;
	if(SUCCEEDED(data->EnumFormatEtc(DATADIR_GET,&ef)))
		for(formatetc_iterator i(ef);i!=formatetc_iterator();++i) {
			FORMATETC fetc = *i;
			if(SUCCEEDED(hr = tryManyFormat(data,&fetc,isetPaste,ret)))
				return hr;
		}
	else {// no enumerator; DO probably just supports one format so try all (!)
		for(clipmap::iterator i(m_manyFormats.begin());i!=m_manyFormats.end();++i)
			if(SUCCEEDED(hr = i->second->UnpackMany(data,0,isetPaste,ret)))
				return hr;
		CComPtr<IMCCSitePersistor> sp;
		RUN(UnpackOne(data,fetc,0,isetPaste,&sp));
		wrapU(sp,ret);
	}
	ERR(E_FAIL);
}
STDMETHODIMP DataObjectReader::Query(IDataObject *data) {
	METHTRACE(DataObjectReader,Query);
	CComPtr<IEnumFORMATETC> ef;
	RUN(data->EnumFormatEtc(DATADIR_GET,&ef));
	for(formatetc_iterator i(ef);i!=formatetc_iterator();++i) {
		FORMATETC &debug = *i;
		CLIPFORMAT cf = i->cfFormat;

        /*
		ATLTRACE2("format %d: ",cf);
		TCHAR buf[200];
		int len;
		if(len = GetClipboardFormatName(cf,buf,200))
			ATLTRACE2(_T("\"%s\"\n"),buf);
		else
			ATLTRACE2(_T("(unknown)\n"));
        */

		clipmap::iterator j = m_oneFormats.find(cf);
		if(j!=m_oneFormats.end())
			return j->second->Query(data);
		j = m_manyFormats.find(cf);
		if(j!=m_manyFormats.end())
			return j->second->Query(data);
	}
	ERR(E_FAIL);
}
