/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DefaultMCCSelectable.h"

DefaultMCCSelectable::DefaultMCCSelectable() : m_realized(false), m_singleton(false) {
	CONSTRUCTORTRACE(DefaultMCCSelectable);
}
DefaultMCCSelectable::~DefaultMCCSelectable() {
	DESTRUCTORTRACE(DefaultMCCSelectable);
}
// DIDefaultMCCSelectable
STDMETHODIMP DefaultMCCSelectable::Initialize(DIMontageContainer *mc) {
	return TakePointer(mc,0);
}
STDMETHODIMP DefaultMCCSelectable::get_Singleton(VARIANT_BOOL *ret) {
	*ret = m_singleton?-1:0;
	return S_OK;
}
STDMETHODIMP DefaultMCCSelectable::put_Singleton(VARIANT_BOOL val) {
	m_singleton = val?true:false;//c4800
	return S_OK;
}
// IDefaultSink
STDMETHODIMP DefaultMCCSelectable::StartSink(IUnknown *firedOn) {
	METHTRACE(DefaultMCCSelectable,StartSink);
	m_subject = firedOn;
	return S_OK;
}
// IMCCSelectable
STDMETHODIMP DefaultMCCSelectable::Add(IItemSet *selection) {
	METHTRACE(DefaultMCCSelectable,Add);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	CComPtr<IMoniker> mk;
	RUN(m_subject->get_RelMoniker(&mk));
	if(!mk)
		return E_FAIL; // can't select without name
	m_subject.Release();
	if(m_singleton) {
		if(selection->Check(mk)==S_OK)
			return S_OK;
		RUN(selection->Clear());
	}
	RUN(selection->Add(mk));
    return S_OK;
}
STDMETHODIMP DefaultMCCSelectable::Remove(IItemSet *selection) {
	METHTRACE(DefaultMCCSelectable,Remove);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	CComPtr<IMoniker> mk;
	RUN(m_subject->get_RelMoniker(&mk));
	if(!mk)
		return E_FAIL; // can't select without name
	m_subject.Release();
	RUN(selection->Remove(mk));
    return S_OK;
}
STDMETHODIMP DefaultMCCSelectable::Toggle(IItemSet *selection) {
	METHTRACE(DefaultMCCSelectable,Toggle);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	CComPtr<IMoniker> mk;
	RUN(m_subject->get_RelMoniker(&mk));
	if(!mk)
		return E_FAIL; // can't select without name
	if(selection->Check(mk)==S_OK) {
		m_subject.Release();
		RUN(selection->Remove(mk));
	}
	else RUN(Add(selection)); // defer to self for proper singleton reaction
    return S_OK;
}

STDMETHODIMP DefaultMCCSelectable::Check(IItemSet *selection) {
	METHTRACE(DefaultMCCSelectable,Check);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	CComPtr<IMoniker> mk;
	RUN(m_subject->get_RelMoniker(&mk));
	m_subject.Release();
	if(!mk)
		return E_FAIL; // nothing we can do!
    return selection->Check(mk);
}
// INeedOnePointer
STDMETHODIMP DefaultMCCSelectable::TakePointer(IUnknown *u,IMoniker *m) {
	METHTRACE(DefaultMCCSelectable,TakePointer);
	if(m_realized)
		return S_FALSE;
	m_realized = true;
	// connect as default IMCCSelectable sink
	CComPtr<IMontageContainer> mc;
	RUN(u->QueryInterface(riidret(&mc)));
	CComPtr<IMCCAllPointsContainer> apc;
	RUN(mc->get_DefaultSiteSinks(&apc));
	DWORD mysticMint;
	RUN(APCAdvise(apc,(IMCCSelectable*)this,IID_IMCCSelectable,&mysticMint));
	return S_OK;
}
// IPersist
STDMETHODIMP DefaultMCCSelectable::GetClassID(CLSID *pClassID) {
	METHTRACE(DefaultMCCSelectable,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
