/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DefaultMCCSelectionDraw.h"

DefaultMCCSelectionDraw::DefaultMCCSelectionDraw() : m_grapClass(CLSID_MCCGrapple) {
	CONSTRUCTORTRACE(DefaultMCCSelectionDraw);
}
DefaultMCCSelectionDraw::~DefaultMCCSelectionDraw() {
	DESTRUCTORTRACE(DefaultMCCSelectionDraw);
}
// DIDefaultMCCSelectionDraw
STDMETHODIMP DefaultMCCSelectionDraw::Initialize(DIMCCLayer *grappleLayer) {
    METHTRACE(DefaultMCCSelectionDraw,Initialize);
	return TakePointer(grappleLayer,0);
}
STDMETHODIMP DefaultMCCSelectionDraw::put_GrappleClass(BSTR val) {
    METHTRACE(DefaultMCCSelectionDraw,put_GrappleClass);
	RUN(CLSIDFromString(val,&m_grapClass));
	return S_OK;
}
extern CComBSTR g_OleInPlaceBorderWidth;
// IMCCSelectionDraw
HRESULT DefaultMCCSelectionDraw::Show() {
	METHTRACE(DefaultMCCSelectionDraw,Show);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	CComPtr<IMontageContainer> mc;
	RUN(m_subject->GetContainer(riidret(&mc)));
	CComPtr<IMCCSite> sgrap;
	RUN(mc->new_Site(&sgrap));
	RUN(sgrap->put_Layer(m_grapplel));
	CComPtr<IDispatch> dgrap;
	RUN(CREATE(dgrap,m_grapClass)); TRACECREATEV(m_grapClass);
	CComPtr<IDispatchEx> contStuff;
	RUN(m_subject->GetContainer(riidret(&contStuff)));
	DISPID widthID;
	RUN(contStuff->GetDispID(g_OleInPlaceBorderWidth,0,&widthID));
	VARIANT width;
	RUN(CComDispatchDriver::GetProperty(contStuff,widthID,&width));
	_ASSERTE(width.vt==VT_I4);
	RUN(CComDispatchDriver::PutProperty(dgrap,DISPID_BORDERWIDTH,&width));
	RUN(sgrap->put_Control(dgrap));
	RUN(CComQIPtr<INeedOnePointer>(dgrap)->TakePointer(m_subject,0));
	RUN(sgrap->put_Intent(MCCS_ACTIVE));
	m_subject.Release();
	return S_OK;
}
HRESULT DefaultMCCSelectionDraw::Hide() {
	METHTRACE(DefaultMCCSelectionDraw,Hide);
	m_subject.Release();
	return S_FALSE; // shouldn't get hit: grapple has taken over.
	/*
	if(!m_subject)
		ERR(E_UNEXPECTED);
	// fire fake deletion event on subject so that grapple will leave
	RUN(fireEvent(m_subject,fireTwoParams<IMCCSiteEvents,long,long>(&IMCCSiteEvents::Shifting,MCCS_ACTIVE,MCCS_DELETED)));
	m_subject.Release();
	return S_OK;
	*/
}
STDMETHODIMP DefaultMCCSelectionDraw::TakePointer(IUnknown *p,IMoniker *m) {
	METHTRACE(DefaultMCCSelectionDraw,TakePointer);
	if(m_grapplel)
		return S_FALSE;
	RUN(p->QueryInterface(riidret(&m_grapplel)));
	// connect as default IMCCSelectionDraw sink
	CComPtr<IMontageContainer> mc;
	RUN(m_grapplel->GetContainer(riidret(&mc)));
	CComPtr<IMCCAllPointsContainer> apc;
	RUN(mc->get_DefaultSiteSinks(&apc));
	DWORD mysticMint;
	RUN(APCAdvise(apc,(IMCCSelectionDraw*)this,IID_IMCCSelectionDraw,&mysticMint));
	return S_OK;
}
// IPersistStream
STDMETHODIMP DefaultMCCSelectionDraw::GetClassID(CLSID *ret) {
    METHTRACE(DefaultMCCSelectionDraw,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP DefaultMCCSelectionDraw::IsDirty(void) {
    METHTRACE(DefaultMCCSelectionDraw,IsDirty);
	return S_FALSE;
}
STDMETHODIMP DefaultMCCSelectionDraw::Load(IStream *pStm) {
    METHTRACE(DefaultMCCSelectionDraw,Load);
	RUN(pStm->Read(&m_grapClass,sizeof(CLSID),0));
	return S_OK;
}
STDMETHODIMP DefaultMCCSelectionDraw::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(DefaultMCCSelectionDraw,Save);
	RUN(pStm->Write(&m_grapClass,sizeof(CLSID),0));
	return S_OK;
}
STDMETHODIMP DefaultMCCSelectionDraw::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(DefaultMCCSelectionDraw,GetSizeMax);
	pcbSize->QuadPart = sizeof(CLSID);
	return S_OK;
}
