/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "DefaultSiteOwner.h"

DefaultSiteOwner::DefaultSiteOwner() {
	CONSTRUCTORTRACE(DefaultSiteOwner);
}
DefaultSiteOwner::~DefaultSiteOwner() {
	DESTRUCTORTRACE(DefaultSiteOwner);
}
// IDefaultSink
STDMETHODIMP DefaultSiteOwner::StartSink(IUnknown *firedOn) {
	METHTRACE(DefaultSiteOwner,StartSink);
	m_subject = firedOn;
	return S_OK;
}
// IMCCSiteOwner
STDMETHODIMP DefaultSiteOwner::Move(MCC_POS_INFO *pos) {
	METHTRACE(DefaultSiteOwner,Move);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	if(!pos)
		ERR(E_POINTER);
	// try to interpret the position manually (some of the more convoluted
	// code here will only be needed when layout components can disappear from the
	// system).
	CComPtr<IMontageContainer> mc;
	RUN(m_subject->GetContainer(riidret(&mc)));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
	CComPtr<IMCCLayer> layer;
	RUN(m_subject->get_Layer(&layer));
	VARIANT_BOOL isTransformed;
	RUN(layer->get_IsTransformed(&isTransformed));
	CComPtr<IOleObject> oo;
	MCC_POS_INFO mpi2;
	switch(pos->postype) {
	case MCC_POS_CANVASSIZE:
		if(isTransformed)
			RUNRET(m_subject->get_VirtRect(&mpi2.rcl));
		else {
			RECT rr;
			RUNRET(m_subject->get_Rect(&rr));
			RUN(t->W2CP(*(POINT*)&rr,(PPOINTL)&mpi2.rcl));
		}
		mpi2.rcl.right = mpi2.rcl.left + pos->sl.cx;
		mpi2.rcl.bottom = mpi2.rcl.top - pos->sl.cy;
		pos = &mpi2;
	case MCC_POS_CANVASRECT:
		if(isTransformed)
			RUN(m_subject->put_VirtRect(&pos->rcl));
		else {
			RECT rc;
			RUN(t->C2WR(&pos->rcl,&rc));
			RUN(m_subject->put_Rect(&rc));
		}
		break;
	case MCC_POS_WINDOWSIZE:
		RUNRET(m_subject->get_Rect(&mpi2.rc));
		mpi2.rc.right = mpi2.rc.left + pos->s.cx;
		mpi2.rc.bottom = mpi2.rc.top + pos->s.cy;
		pos = &mpi2;
	case MCC_POS_WINDOWRECT: {
		if(isTransformed){
			if(!t)
				ERR(E_UNEXPECTED);
			RECTL rcl;
			RUN(t->W2CR(&pos->rc,&rcl));
			RUN(m_subject->put_VirtRect(&rcl));
		}
		else
			RUN(m_subject->put_Rect(&pos->rc));
		break;
	}
	case MCC_POS_WINDOWPOINT:
	case MCC_POS_CANVASPOINT: {
		SIZEL size;
		{
		CComPtr<IOleObject> oo;
		CComPtr<IViewObject2> vo2;
		if(SUCCEEDED(m_subject->GetControl(riidret(&oo))))
			RUN(oo->GetExtent(DVASPECT_CONTENT,&size));
		else if(SUCCEEDED(m_subject->GetControl(riidret(&vo2))))
			RUN(vo2->GetExtent(DVASPECT_CONTENT,-1,0,&size));
		else ERR(E_FAIL);
		}
		if(pos->postype == MCC_POS_CANVASPOINT) {
			RECTL rcl = {pos->ptl.x,pos->ptl.y,pos->ptl.x+size.cx,pos->ptl.y - size.cy};
			if(isTransformed)
				RUN(m_subject->put_VirtRect(&rcl));
			else {
				if(!t)
					ERR(E_UNEXPECTED);
				RECT rc;
				RUN(t->C2WR(&rcl,&rc));
				RUN(m_subject->put_Rect(&rc));
			}
		}
		else {
			if(isTransformed) {
				if(!t)
					ERR(E_UNEXPECTED);
				POINTL ptl;
				RUN(t->W2CP(pos->pt,&ptl));
				RECTL rcl = {ptl.x,ptl.y,ptl.x+size.cx,ptl.y-size.cy};
				RUN(m_subject->put_VirtRect(&rcl));
			}
			else {
				SIZE pixsize;
				if(t) {
					RUN(t->C2WS(size,&pixsize));
					pixsize.cy *= -1; // C2WS is actually for vectors, not extents.
				}
				else
					AtlHiMetricToPixel(&size,&pixsize);
				RECT rc = {pos->pt.x,pos->pt.y,pos->pt.x + pixsize.cx,pos->pt.y + pixsize.cy};
				RUN(m_subject->put_Rect(&rc));
			}
		}
		break;
	}
	default:
		ERR(E_UNEXPECTED);
	}
	m_subject.Release();
	return S_OK;
}
STDMETHODIMP DefaultSiteOwner::PutBefore(IMCCSite *other) {
	METHTRACE(DefaultSiteOwner,PutBefore);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	RUN(m_subject->PutBefore(other));
	m_subject.Release();
	return S_OK;
}
STDMETHODIMP DefaultSiteOwner::PutBehind(IMCCSite *other) {
	METHTRACE(DefaultSiteOwner,PutBehind);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	RUN(m_subject->PutBehind(other));
	m_subject.Release();
	return S_OK;
}
STDMETHODIMP DefaultSiteOwner::Intend(long val) {
	METHTRACE(DefaultSiteOwner,Intend);
	if(!m_subject)
		ERR(E_UNEXPECTED);
	HRESULT hr = m_subject->put_Intent(val);
	return hr;
	/*
	// this is no good: wreaks havoc with scrolling transform
	// and besides you just shouldn't show something w/o position!!
	if(hr==E_PENDING) { // prob. doesn't have position
		MCC_POS_INFO mpi;
		mpi.postype = MCC_POS_WINDOWPOINT;
		mpi.pt.x = mpi.pt.y = 0;
		CComPtr<IMCCSite> back = m_subject;
		RUN(Move(&mpi));
		hr = back->put_Intent(val);
	}
	RUN(hr);
	m_subject.Release();
	return S_OK;
	*/
}
