/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// DefaultTransform.cpp : Implementation of DefaultTransform
#include "stdafx.h"
#include "Montage.h"
#include "DefaultTransform.h"
#include "CvtMetrics.h"

/////////////////////////////////////////////////////////////////////////////
// DefaultTransform

DefaultTransform::DefaultTransform() {
	CONSTRUCTORTRACE(DefaultTransform);
}
DefaultTransform::~DefaultTransform() {
	DESTRUCTORTRACE(DefaultTransform);
}

// IMCCTransform
STDMETHODIMP DefaultTransform::put_TransformedArea(RECT *val) {
	ERR(E_NOTIMPL);
}
STDMETHODIMP DefaultTransform::get_TransformedArea(RECT *val) {
	ERR(E_NOTIMPL);
}
STDMETHODIMP DefaultTransform::put_Extent(SIZEL *val) {
	ERR(E_NOTIMPL);
}
STDMETHODIMP DefaultTransform::get_Extent(SIZEL *val) {
	ERR(E_NOTIMPL);
}
STDMETHODIMP DefaultTransform::C2WS(SIZEL siz, PSIZE psiz) {
	METHTRACE(DefaultTransform,C2WS);
	*psiz = g_cvt.HI2Pix(siz);
	psiz->cy = -psiz->cy;
	return S_OK;
}
STDMETHODIMP DefaultTransform::C2VS(SIZEL siz, PSIZE psiz) {
	METHTRACE(DefaultTransform,C2VS);
	return C2WS(siz,psiz);
}
STDMETHODIMP DefaultTransform::W2CS(SIZE siz, PSIZEL psiz) {
	METHTRACE(DefaultTransform,W2CS);
	*psiz = g_cvt.Pix2HI(siz);
	psiz->cy = -psiz->cy;
	return S_OK;
}
STDMETHODIMP DefaultTransform::C2WP(POINTL pt, PPOINT ppt) {
	METHTRACE(DefaultTransform,C2WP);
	SIZEL ulPoint = {pt.x,- pt.y};
	*(SIZE*)ppt = g_cvt.HI2Pix(ulPoint);
	return S_OK;
}
STDMETHODIMP DefaultTransform::C2VP(POINTL pt, PPOINT ppt) {
	METHTRACE(DefaultTransform,C2VP);
	return C2WP(pt,ppt);
}
STDMETHODIMP DefaultTransform::W2CP(POINT pt, PPOINTL ppt) {
	METHTRACE(DefaultTransform,W2CP);
	SIZEL ptHI = g_cvt.Pix2HI(*(SIZE*)&pt);
	ppt->x = ptHI.cx;
	ppt->y = -ptHI.cy;
	return S_OK;
}
STDMETHODIMP DefaultTransform::C2WR(LPCRECTL pcr, PRECT prc) {
	METHTRACE(DefaultTransform,C2WR);
	// a HACK you're probably familiar with by now....
	POINTL *sourcePoints = (POINTL*)pcr;
	POINT *destPoints = (POINT*)prc;
	RUN(C2WP(*sourcePoints,destPoints));
	RUN(C2WP(sourcePoints[1],destPoints+1));
	return S_OK;
}
STDMETHODIMP DefaultTransform::C2VR(LPCRECTL pcr, PRECT prc) {
	METHTRACE(DefaultTransform,C2VR);
	return C2WR(pcr,prc);
}
STDMETHODIMP DefaultTransform::W2CR(LPCRECT pcr, PRECTL prc) {
	METHTRACE(DefaultTransform,W2CR);
	POINT *sourcePoints = (POINT*)pcr;
	POINTL *destPoints = (POINTL*)prc;
	RUN(W2CP(*sourcePoints,destPoints));
	RUN(W2CP(sourcePoints[1],destPoints+1));
	return S_OK;
}
STDMETHODIMP DefaultTransform::Viewing(LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
	METHTRACE(DefaultTransform,Viewing);
	ERR(E_NOTIMPL);
}
STDMETHODIMP DefaultTransform::get_IsViewing(VARIANT_BOOL *ret) {
	*ret = VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP DefaultTransform::AddRect(LPCRECTL r) {
	METHTRACE(DefaultTransform,AddRect);
	return S_OK;
}
STDMETHODIMP DefaultTransform::RemoveRect(LPCRECTL r) {
	METHTRACE(DefaultTransform,RemoveRect);
	return S_OK;
}
// IPersist
STDMETHODIMP DefaultTransform::GetClassID(CLSID *pcls) {
	*pcls = GetObjectCLSID();
	return S_OK;
}
