/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"

HRESULT Container::drag(DWORD grfKeyState,POINTL pt,DWORD *retEffect) {
	METHTRACE(Container,drag);
	// a drop should try all opaque objects under the point first, and then
	// the transparent ones based on their coUI drop priority.
	typedef multimap<DWORD,site *> prioridrop;
	prioridrop transparents;
	// dox say pt is in window coords, but in fact (and according to Inside OLE) it's in screen coords
	// certainly windowless objects can't deal with that!
	POINT wpoint = *(POINT*)&pt;
	ScreenToClient(m_hWnd,&wpoint);
	for(layerlist::iterator i=m_layers.begin();i!=m_layers.end();i++)
		for(sitelist::iterator j=(*i)->m_sites.begin();j!=(*i)->m_sites.end();j++)
			if((*j)->m_state>MCCS_OPEN) { // needs IPointerInactive support
				site *dest = *j;
				DWORD hit = dest->hitpoint(*(POINT*)&wpoint);
				switch(hit) {
				case HITRESULT_HIT: {// very simple if you've got a hit: just defer if it's a target
					// if active contained object's window doesn't exactly cover the inplacerect
					// you can get msgs that were intended for it: throw them out
					if(!dest->m_pWindowless) {
						*retEffect = DROPEFFECT_NONE;
						return S_OK;
					}
					CComPtr<IDropTarget> dt;
					if(SUCCEEDED(dest->m_pWindowless->GetDropTarget(&dt)))
						if(dt==m_dragtarget)  // already started negociation
							return dt->DragOver(grfKeyState,pt,retEffect);
						else {
							if(m_dragtarget)
								RUN(m_dragtarget->DragLeave()); // cancel negociation with old target
							m_dragtarget = dt;
							return dt->DragEnter(m_dragdata,grfKeyState,pt,retEffect);
						}
					continue;
				}
				case HITRESULT_CLOSE:
				case HITRESULT_TRANSPARENT: {
					// OLE dox say nothing about this, but when it's truly ambiguous z-order doesn't
					// matter, only *something else*, which I define thru coUI
					DWORD priority = 0; // non coUI objects get priority
					/*
					// naw,it just sucks.
					if(CComQIPtr<ICoUIActiveObject> coUI = dest->m_pControl)
						RUN(coUI->GetDropPriority(&priority));
					*/
					transparents.insert(make_pair_first_const(priority,dest));
					continue;
				}
				default: // HITRESULT_OUTSIDE
					continue;
				}
			}
	for(prioridrop::iterator it = transparents.begin();it!=transparents.end();++it) {
		CComPtr<IDropTarget> dt;
		if(SUCCEEDED(it->second->m_pWindowless->GetDropTarget(&dt))) {
			DWORD effect = *retEffect;
			if(dt==m_dragtarget)
				RUN(dt->DragOver(grfKeyState,pt,&effect));
			else {
				if(m_dragtarget)
					RUN(m_dragtarget->DragLeave());
				m_dragtarget = dt;
				RUN(dt->DragEnter(m_dragdata,grfKeyState,pt,&effect));
			}
			if(effect!=DROPEFFECT_NONE) {
				*retEffect = effect;
				return S_OK;
			}
		}
	}
	*retEffect = DROPEFFECT_NONE;
	return S_OK;
}
STDMETHODIMP Container::DragEnter(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
									  DWORD *pdwEffect) {
	IMETHTRACE(Container,IDropTarget,DragEnter);
	m_dragdata = pDataObj;
	m_dragtarget = 0;
	return drag(grfKeyState,pt,pdwEffect);
}
STDMETHODIMP Container::DragOver(DWORD grfKeyState,POINTL pt,DWORD *pdwEffect) {
	IMETHTRACE(Container,IDropTarget,DragOver);
	return drag(grfKeyState,pt,pdwEffect);
}
STDMETHODIMP Container::DragLeave() {
	IMETHTRACE(Container,IDropTarget,DragLeave);
	if(m_dragtarget) {
		RUN(m_dragtarget->DragLeave());
		m_dragtarget.Release();
	}
	m_dragdata.Release();
	return S_OK;
}
STDMETHODIMP Container::Drop(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
				DWORD *pdwEffect) {
	IMETHTRACE(Container,IDropTarget,Drop);
	if(m_dragtarget) {
		RUN(m_dragtarget->Drop(pDataObj,grfKeyState,pt,pdwEffect));
		m_dragtarget.Release();
	}
	else
		*pdwEffect = DROPEFFECT_NONE;
	m_dragdata.Release();
	return S_OK;
}
