/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// this unpacker will try the clip formats "Embedded Object" and "Embed Source"
// w/ TYMED_ISTORAGE (GetDataHere) and TYMED_ISTREAM (GetData and GetDataHere-to-temp)

#include "stdafx.h"
#include "EmbedUnpacker.h"
#include "createAffixMoniker.h"

EmbedUnpacker::EmbedUnpacker() {
	CONSTRUCTORTRACE(EmbedUnpacker);
}
EmbedUnpacker::~EmbedUnpacker() {
	DESTRUCTORTRACE(EmbedUnpacker);
}
// IMCCEmbedUnpacker
STDMETHODIMP EmbedUnpacker::Initialize(IMCCDataObjectReader *reader, IMCCLayer *layer) {
	METHTRACE(EmbedUnpacker,Initialize);
	RUN(reader->RegisterUnpacker(g_clipformats.embedded,false,this));
	RUN(reader->RegisterUnpacker(g_clipformats.embedsource,false,this));
	m_layer = layer;
	return S_OK;
}
HRESULT getToStorage(CLIPFORMAT cf, IDataObject *data, IStorage *stg) {
	FORMATETC fetz;
	fetz.ptd = 0;
	fetz.dwAspect = DVASPECT_CONTENT;
	fetz.lindex = -1;
	fetz.tymed = TYMED_ISTORAGE;

	STGMEDIUM stgm;
	stgm.tymed = TYMED_ISTORAGE;
	stgm.pstg = stg;
	stgm.pUnkForRelease = 0;

	if(cf) {
		fetz.cfFormat = cf;
		return data->GetDataHere(&fetz,&stgm);
	}
	else {
		fetz.cfFormat = g_clipformats.embedded;
		if(SUCCEEDED(data->GetDataHere(&fetz,&stgm)))
			return S_OK;
		fetz.cfFormat = g_clipformats.embedsource;
		if(SUCCEEDED(data->GetDataHere(&fetz,&stgm)))
			return S_OK;
		SERR(E_FAIL);
	}
}
// just a helper.
HRESULT getToNewStream(FORMATETC *fetc,IDataObject *data, STGMEDIUM *ret) {
    FUNTRACE(getToNewStream);
	HRESULT hr;
	if(SUCCEEDED(hr = data->GetData(fetc,ret)))
		return hr;
	// unfortunately there's no easy way to do a CreateStream in IGroup,
	// so get to a buf and then load
	ret->tymed = TYMED_ISTREAM;
	SRUN(CreateStreamOnHGlobal(0,true,&ret->pstm));
	ret->pUnkForRelease = 0;
	if(FAILED(hr = data->GetDataHere(fetc,ret)))
		ReleaseStgMedium(ret);
	return hr;
}
HRESULT getToStreamAndLoad(CLIPFORMAT cf, IDataObject *data, REFIID riid, void **ret) {
    FUNTRACE(getToStreamAndLoad);
	HRESULT hr;
	FORMATETC fetz;
	fetz.ptd = 0;
	fetz.dwAspect = DVASPECT_CONTENT;
	fetz.lindex = -1;
	fetz.tymed = TYMED_ISTREAM;

	StgMedium stgm;
	if(cf) {
		fetz.cfFormat = cf;
		SRUN(getToNewStream(&fetz,data,&stgm));
	}
	else {
		fetz.cfFormat = g_clipformats.embedded;
		if(FAILED(getToNewStream(&fetz,data,&stgm))) {
			fetz.cfFormat = g_clipformats.embedsource;
			SRUN(getToNewStream(&fetz,data,&stgm));
		}
	}
	if(stgm.tymed!=TYMED_ISTREAM || !stgm.pstm)
		SERR(E_UNEXPECTED);
	hr = FixedOleLoadFromStream(stgm.pstm,riid,ret);
	ReleaseStgMedium(&stgm);
	return hr;
}
// IMCCDataObjectUnpacker
#include "siteControl.h"
STDMETHODIMP EmbedUnpacker::UnpackOne(IDataObject *data,
                                      FORMATETC *fetc, LPOLESTR baseName, IItemSet *iset, IMCCSitePersistor **ret) {
	METHTRACE(EmbedUnpacker,UnpackOne);

	// context
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<IMontageContainer> mc;
	RUN(m_layer->GetContainer(riidret(&mc)));
	CComPtr<IGroup> group;
	RUN(mc->GetMainGroup(riidret(&group)));

	// see if this name's already been used & return existent if so
	CComPtr<IMoniker> spName;
	if(baseName) {
		RUN(createAffixMoniker(baseName,L"Site",&spName));
		CComPtr<IMoniker> mkG;
		RUN(FoolishCreatePointerMoniker(group,&mkG));
		CComPtr<IMCCSitePersistor> spExists;
		if(SUCCEEDED(spName->BindToObject(bctx,mkG,riidret(&spExists)))) {
			if(ret)
				(*ret = spExists)->AddRef();
			return S_FALSE;
		}
	}
	// the idea is to validate mkEmbed (some solutions leave the embedding
	// passive, some loaded, it does not matter.)
	CComPtr<IMoniker> mkEmbed0,mkEmbed;
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Embedding",&mkEmbed0));

	CComPtr<IUnknown> u;
	HRESULT hr;
	// try fetching to a stream first
	if(!fetc || fetc->tymed&TYMED_ISTREAM)
		hr = getToStreamAndLoad(fetc?fetc->cfFormat:0,data,riidret(&u));

	if(!u && (!fetc || fetc->tymed&TYMED_ISTORAGE)) {
		// create the embedded object's storage
		CComPtr<IStorage> stg;
		RUN(group->CreatePassive(true,bctx,mkEmbed0,&mkEmbed,riidret(&stg)));

		/* the following is equivalent to
			RUN(OleCreateFromData(data,IID_IUnknown,OLERENDER_DRAW,0,0,stg,(void**)&u));
			RUN(OleLoad(stg,IID_IUnknown,0,(void**)&u));
		  but this is debuggable!
			*/
		// get the storage data
		HRESULT hr;
		if(FAILED(hr = getToStorage(fetc?fetc->cfFormat:0,data,stg))) {
			RUN(group->ReleaseStgRef(mkEmbed));
			return hr;
		}
		STATSTG stat;
		RUN(stg->Stat(&stat,STATFLAG_NONAME));
		RUN(u.CoCreateInstance(stat.clsid));
		RUN(CComQIPtr<IPersistStorage>(u)->Load(stg));
		// and tell group that this object considers its storage home
		RUN(group->SetMainStorage(bctx,mkEmbed,true));
	}
	if(!u)
		ERR(E_FAIL);
	RUN(group->Paste(bctx,GP_SrcIsObj|GP_Replace,u,mkEmbed?mkEmbed:mkEmbed0,IID_NULL,0,mkEmbed?0:&mkEmbed));

	MCC_POS_INFO mpi;
	// set up rect as at origin
	SIZEL size;
	if(SUCCEEDED(getsizepoint(data,&size,0))&&size.cx&&size.cy) { // can return 0,0
		mpi.postype = MCC_POS_CANVASRECT;
		mpi.rcl.left = mpi.rcl.bottom = 0;
		mpi.rcl.right = size.cx;
		mpi.rcl.top = size.cy;
	}
	else {
		// get size from METAFILEPICT (wasteful but sometimes necessary)
		FormatEtc mfF(CF_METAFILEPICT);
		StgMedium stgm;
		bool got(false);
		if(SUCCEEDED(data->GetData(&mfF,&stgm))) {
			METAFILEPICT *mfp = (METAFILEPICT*)GlobalLock(stgm.hMetaFilePict);
			if(mfp->xExt&&mfp->yExt) {
				mpi.postype = MCC_POS_CANVASRECT;
				mpi.rcl.left = mpi.rcl.bottom = 0;
				mpi.rcl.right = mfp->xExt;
				mpi.rcl.top = mfp->yExt;
				got = true;
			}
			GlobalUnlock(stgm.hMetaFilePict);
			ReleaseStgMedium(&stgm);
		}
		if(!got)
			mpi.postype = MCC_POS_NONE;
	}
	if(!m_mkCacheInit)
		m_mkCacheInit = ItemMoniker(L"Cache Init");
	if(!m_mkRunInit)
		m_mkRunInit = ItemMoniker(L"Run Init");
	IMoniker *inits[2] = {m_mkCacheInit,m_mkRunInit};
	int initP[2] = {0,1};
	CComPtr<IMoniker> mkSP;
	RUN(siteControl(bctx,mkEmbed,spName,m_layer,&mpi,MCCS_SHOWN,2,inits,initP,&mkSP,ret));
	if(iset)
		RUN(iset->Add(mkSP));
	return S_OK;
}
STDMETHODIMP EmbedUnpacker::UnpackMany(IDataObject *data, FORMATETC *fetc, IItemSet *iset, IEnumUnknown **ret) {
	ERR(E_NOTIMPL);
}
STDMETHODIMP EmbedUnpacker::Query(IDataObject *data) {
	METHTRACE(EmbedUnpacker,Query);
	HRESULT hr;
	FormatEtc fetz(g_clipformats.embedded);
	fetz.tymed = TYMED_ISTREAM;
	if(SUCCEEDED(hr = data->QueryGetData(&fetz)))
		return S_OK;
	fetz.tymed = TYMED_ISTORAGE;
	if(SUCCEEDED(hr = data->QueryGetData(&fetz)))
		return S_OK;
	fetz.cfFormat = g_clipformats.embedsource;
	fetz.tymed = TYMED_ISTREAM;
	if(SUCCEEDED(hr = data->QueryGetData(&fetz)))
		return S_OK;
	fetz.tymed = TYMED_ISTORAGE;
	if(SUCCEEDED(hr = data->QueryGetData(&fetz)))
		return S_OK;
	ERR(E_FAIL);
}
// DIMCCEmbedUnpacker
STDMETHODIMP EmbedUnpacker::Initialize(IUnknown *reader, IUnknown *layer) {
    METHTRACE(EmbedUnpacker,Initialize);
	CComPtr<IMCCDataObjectReader> r2;
	if(reader)
		RUNRET(reader->QueryInterface(riidret(&r2)));
	CComPtr<IMCCLayer> l2;
	if(layer)
		RUNRET(layer->QueryInterface(riidret(&l2)));
	return Initialize(r2,l2);
}
// INeedSomePointers
STDMETHODIMP EmbedUnpacker::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(EmbedUnpacker,TakePointers);
	if(m_layer)
		return S_FALSE;
	CComPtr<IUnknown> punk;
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IMCCDataObjectReader> reader = punk;
	QISET(reader);
	punk.Release();
	RUN(os->Next(1,&punk,0));
	CComQIPtr<IMCCLayer> layer = punk;
	QISET(layer);
	return Initialize(reader,layer);
}
