/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "FilePort.h"

IMCCFileFormat *getFormat(FilePort::extMap &m,LPCOLESTR filename) {
	LPOLESTR dot = wcsrchr(filename,'.');
	FilePort::extMap::iterator fi =  m.find(dot?dot+1:L"");
	if(fi==m.end())
		return 0;
	else return fi->second;
}
struct reset_DWORD {
	const DWORD val;
	DWORD temp;
	reset_DWORD(int val) : val(val) {}
	DWORD *operator &() {
		temp = val;
		return &temp;
	}
};
FilePort::FilePort() {
	CONSTRUCTORTRACE(FilePort);
}
FilePort::~FilePort() {
	DESTRUCTORTRACE(FilePort);
}
STDMETHODIMP FilePort::RegisterFileFormat(enum MFP_PORTTYPE type, LPCOLESTR desc, IMCCFileFormat *handler) {
	extMap &m = getMap(type);
	const OLECHAR *isemi, *icolon, *last = desc;
	if(!(icolon = wcschr(desc,':')))
		ERR(E_INVALIDARG);
	if(icolon==desc)  // default handler
		m[L""] = handler;
	else {
		while(isemi = wcschr(last,'|')) {
			m[wstring(last,isemi-last)] = handler;
			last = isemi + 1;
		}
		if(icolon>last) // ignore blank last eg ext|:
			m[wstring(last,icolon-last)] = handler;
	}
	if(type==MFP_IMPORT)
		m_impDescs.push_back(desc);
	else
		m_expDescs.push_back(desc);
	return S_OK;
}
STDMETHODIMP FilePort::RegisterNativeFileFormat(enum MFP_PORTTYPE type,LPOLESTR desc) {
	((type==MFP_EXPORT)?m_nativeOut:m_nativeIn) = desc;
	return S_OK;
}
STDMETHODIMP FilePort::UnregisterFileFormat(enum MFP_PORTTYPE type, LPCOLESTR desc) {
	ERR(E_NOTIMPL); // heh heh
}
STDMETHODIMP FilePort::EnumDescriptions(enum MFP_PORTTYPE type, IEnumString **ret) {
	descVec &descs = type==MFP_IMPORT?m_impDescs:m_expDescs;
	wstring &native =type==MFP_IMPORT?m_nativeIn:m_nativeOut;
	unsigned N = descs.size(),i=0;
	if(!native.empty())
		++N;
	LPCOLESTR *ray = new LPCOLESTR[N];
	for(unsigned j=0;j<descs.size();++j)
		ray[i++] = descs[j].c_str();
	// put native format last because it's less useful or less familiar for dynagraph, dinograph
	if(!native.empty())
		ray[i++] = native.c_str();
	typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > en;
	en *e = new en;
	e->Init(const_cast<LPOLESTR*>(ray),const_cast<LPOLESTR*>(ray+N),0,AtlFlagCopy);
	(*ret = e)->AddRef();
	delete [] ray;
	return S_OK;
}
STDMETHODIMP FilePort::Import(LPCOLESTR filename) {
	IMCCFileFormat *ff = getFormat(m_import,filename);
	if(!ff)
		ERR(E_FAIL);
	return LOOK(ff->Import(filename));
}
STDMETHODIMP FilePort::Export(LPCOLESTR filename) {
	IMCCFileFormat *ff = getFormat(m_export,filename);
	if(!ff)
		ERR(E_FAIL);
	return LOOK(ff->Export(filename));
}

STDMETHODIMP FilePort::GetClassID(CLSID *ret) {
	*ret = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP FilePort::IsDirty(void) {
    METHTRACE(FilePort,IsDirty);
	return S_OK;
}
STDMETHODIMP FilePort::Load(IStream *pStm) {
    METHTRACE(FilePort,Load);
	unsigned len;
	RUN(pStm->Read(&len,sizeof len,0));
	m_nativeIn.resize(len);
	RUN(pStm->Read(&m_nativeIn[0],len*2,0));
	RUN(pStm->Read(&len,sizeof len,0));
	m_nativeOut.resize(len);
	RUN(pStm->Read(&m_nativeOut[0],len*2,0));
	return S_OK;
}
STDMETHODIMP FilePort::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(FilePort,Save);
	unsigned len = m_nativeIn.length();
	RUN(pStm->Write(&len,sizeof len,0));
	RUN(pStm->Write(m_nativeIn.c_str(),len*2,0));
	len = m_nativeOut.length();
	RUN(pStm->Write(&len,sizeof len,0));
	RUN(pStm->Write(m_nativeOut.c_str(),len*2,0));
	return S_OK;
}
STDMETHODIMP FilePort::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(FilePort,GetSizeMax);
	ERR(E_NOTIMPL);
}
