/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Firer.h"

// IDispatch
STDMETHODIMP FireHelper::Invoke(
			DISPID  dispIdMember,
			REFIID  riid,
			LCID  lcid,
			WORD  wFlags,
			DISPPARAMS FAR*  pDispParams,
			VARIANT FAR*  pVarResult,
			EXCEPINFO FAR*  pExcepInfo,
			unsigned int FAR*  puArgErr) {
    METHTRACE(FireHelper,Invoke);
	if(!m_typeInfo||!m_point)
		ERR(E_UNEXPECTED);
	bool someOkay = false;
	CComPtr<IEnumConnections> ec;
	RUNRET(m_point->EnumConnections(&ec));
	for(connection_iterator i(ec);i!=connection_iterator();++i)
		if(i->pUnk) {
			IUnknown *u0 = i->pUnk;
			CComPtr<IUnknown> u;
			RUN(u0->QueryInterface(m_iid,(void**)&u));
			if(m_typeInfo->Invoke(u,dispIdMember,wFlags,pDispParams,pVarResult,pExcepInfo,puArgErr)==S_OK)
				someOkay = true;
		}
	if(pVarResult) {
		pVarResult->vt = VT_BOOL;
		pVarResult->boolVal = someOkay?VARIANT_TRUE:VARIANT_FALSE;
	}
	return S_OK;
}
// DIMCCFirer
STDMETHODIMP Firer::Initialize(BSTR strLibid,short major, short minor, BSTR strIid) {
    METHTRACE(Firer,Initialize);
	m_typeInfo.Release();
	GUID libid;
	RUNRET(CLSIDFromString(strLibid,&libid));
	RUNRET(CLSIDFromString(strIid,&m_iid));
	// hack: locale?  has to be option, not just dependent on callers language
	CComPtr<ITypeLib> lib;
	RUNRET(LoadRegTypeLib(libid, major, minor, LOCALE_USER_DEFAULT, &lib));
	RUNRET(lib->GetTypeInfoOfGuid(m_iid, &m_typeInfo));
	return S_OK;
}
STDMETHODIMP Firer::put_Subject(IUnknown *u) {
    METHTRACE(Firer,put_Subject);
	m_point.Release();
	CComPtr<IConnectionPointContainer> cpc;
	RUNRET(u->QueryInterface(riidret(&cpc)));
	RUNRET(cpc->FindConnectionPoint(m_iid,&m_point));
	return S_OK;
}
