/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Montage.h"
#include "Grapple.h"
#include "cursors.h"
#include "brushRect.h"

bool Grapple::GrabHandle::next() {
	METHTRACE(Hatchling,GrabHandle);
	// counterclockwise from upper left
	switch(n++) {
	case 0:
		rc.top = (rcCntr.top+rcCntr.bottom-size)/2; rc.bottom = rc.top + size;
		return true;
	case 1:
		rc.top = rcCntr.bottom - size; rc.bottom = rcCntr.bottom;
		return true;
	case 2:
		rc.left = (rcCntr.left+rcCntr.right-size)/2; rc.right = rc.left + size;
		return true;
	case 3:
		rc.left = rcCntr.right-size; rc.right = rcCntr.right;
		return true;
	case 4:
		rc.top = (rcCntr.top+rcCntr.bottom-size)/2; rc.bottom = rc.top + size;
		return true;
	case 5:
		rc.top = rcCntr.top; rc.bottom = rcCntr.top+size;
		return true;
	case 6:
		rc.left = (rcCntr.left+rcCntr.right-size)/2; rc.right = rc.left+size;
		return true;
	default:
		return false;
	}
}
/////////////////////////////////////////////////////////////////////////////
// Grapple
Grapple::Grapple() : m_buddyState(MCCS_SHOWN), m_bSigMove(false)
{
	CONSTRUCTORTRACE(Grapple);
}
Grapple::~Grapple() {
	DESTRUCTORTRACE(Grapple);
}
bool Grapple::tracking() {
	if(m_bWndLess)
		return m_spInPlaceSite->GetCapture()==S_OK;
	else
		return GetCapture()==m_hWnd;
}
		// from Brockschmidt
WORD g_wHatchBmp[]={0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x77},
	g_wLightHatchBmp[] = {0xfe,0xfd,0xfb,0xf7,0xef,0xdf,0xbf,0x7f};

HRESULT Grapple::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	METHTRACE(Grapple,OnDrawAdvanced);
	// check if we're in Viewing mode, where I'd be an eyesore
	CComPtr<IMontageContainer> mc;
	if(FAILED(m_buddy->GetContainer(riidret(&mc))))
		return S_OK;
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
	VARIANT_BOOL isViewing;
	RUN(t->get_IsViewing(&isViewing));
	if(isViewing)
		return S_OK;

	RECT& rc = *(RECT*)di.prcBounds;
	HDC hdc = di.hdcDraw;

	HBRUSH hbr;
	if(m_buddyState>MCCS_SHOWN) {
		HBITMAP hBM=CreateBitmap(8, 8, 1, 1, (m_buddyState==MCCS_ACTIVE)?g_wLightHatchBmp:g_wHatchBmp);
		hbr = CreatePatternBrush(hBM);

		if(m_buddyState == MCCS_OPEN)
			FillRect(hdc,&rc,hbr);
		else
			brushRect(hdc,rc,m_nBorderWidth,hbr);

		DeleteObject(hbr);
		DeleteObject(hBM);
	}
	hbr = (HBRUSH)GetStockObject(BLACK_BRUSH);
	GrabHandle gh(rc,m_nBorderWidth);
	do
		FillRect(hdc,&gh.rc,hbr);
	while(gh.next());
	return S_OK;
}

STDMETHODIMP Grapple::QueryHitPoint(DWORD dwAspect,LPCRECT pRectBounds,POINT ptlLoc,
						   LONG lCloseHint,DWORD *pHitResult) {
	METHTRACE(Grapple,QueryHitPoint);
	if(dwAspect!=DVASPECT_CONTENT)
		ERR(E_FAIL);
	if(m_buddyState>MCCS_SHOWN) {
		if(m_buddyState==MCCS_OPEN)
			*pHitResult = PtInRect(pRectBounds,ptlLoc)?HITRESULT_HIT:HITRESULT_OUTSIDE;
		else {
			RECT rc = *pRectBounds;
			InflateRect(&rc,-m_nBorderWidth,-m_nBorderWidth);
			*pHitResult = (PtInRect(pRectBounds,ptlLoc)&&!PtInRect(&rc,ptlLoc))?
				HITRESULT_HIT:HITRESULT_OUTSIDE;
		}
	}
	else {
		int which = which_un(*pRectBounds,ptlLoc);
		if(which==8)
			*pHitResult = HITRESULT_OUTSIDE;
		else
			*pHitResult = HITRESULT_HIT;
	}
	return S_OK;
}
LRESULT Grapple::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(Grapple,OnMouseMove);
	POINT pt = MAKEPOINT(lParam);
	if(!tracking()) {
		int which = which_un(m_rcPos,pt);
		if(which==8)
			SetCursor(g_cursors.fourway);
		else
			SetCursor(g_cursors.track[which]);
	}
	else {
		int dx = pt.x - m_ptDown.x,
			dy = pt.y - m_ptDown.y;
		if(!m_bSigMove) {
			if(abs(dx)+abs(dy)<8)
				return 0;
			m_bSigMove = true;
		}
		RECT rcNew = m_rcFirst;
		switch(m_track) {
		case 0:
		case 1:
		case 2:
			rcNew.left += dx;
			break;
		case 8:
			rcNew.left += dx;
		case 4:
		case 5:
		case 6:
			rcNew.right += dx;
		}
		switch(m_track) {
		case 0:
		case 7:
		case 6:
			rcNew.top += dy;
			break;
		case 8:
			rcNew.top += dy;
		case 2:
		case 3:
		case 4:
			rcNew.bottom += dy;
		}
		if(rcNew.left>rcNew.right)
			swap(rcNew.left,rcNew.right);
		if(rcNew.top>rcNew.bottom)
			swap(rcNew.top,rcNew.bottom);
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Rect(&rcNew));
	}
	return 0;
}
LRESULT Grapple::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(Grapple,OnLButtonDown);
	POINT pt = MAKEPOINT(lParam);
	m_track = which_un(m_rcPos,pt);
	if(m_bWndLess)
		RUN(m_spInPlaceSite->SetCapture(true));
	else
		SetCapture();
	m_rcFirst = m_rcPos;
	m_bSigMove = false;
	m_ptDown = pt;
	return 0;
}
LRESULT Grapple::OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(Grapple,OnLButtonUp);
	if(m_bWndLess)
		RUN(m_spInPlaceSite->SetCapture(false));
	else
		ReleaseCapture();
	if(!m_bSigMove && m_buddyState<MCCS_UIACTIVE)
		RUN(m_buddy->put_Intent(MCCS_UIACTIVE));
	MCC_POS_INFO mpi;
	mpi.postype = MCC_POS_WINDOWRECT;
	mpi.rc.left = m_rcPos.left + m_nBorderWidth;
	mpi.rc.top = m_rcPos.top + m_nBorderWidth;
	mpi.rc.right = m_rcPos.right - m_nBorderWidth;
	mpi.rc.bottom = m_rcPos.bottom - m_nBorderWidth;
	return fireSiteEvent(m_buddy,fireMove(&mpi));
}

// IMCCSiteEvents
STDMETHODIMP Grapple::Moving(LPCRECT winRect) {
	METHTRACE(Grapple,Moving);
	if(m_spClientSite) {
		RECT rc = {winRect->left-m_nBorderWidth,winRect->top-m_nBorderWidth,
			winRect->right+m_nBorderWidth,winRect->bottom+m_nBorderWidth};
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Rect(&rc));
	}
	return S_OK;
}
STDMETHODIMP Grapple::Shifting(long oldState, long newState) {
	METHTRACE(Grapple,Shifting);
	if(newState<MCCS_SHOWN)  // buddy has disappeared
		RUN(Hide());
	else {
		if(m_buddyState != newState) {
			if(m_bWndLess)
				RUN(m_spInPlaceSite->InvalidateRect(0,true));
			m_buddyState = newState;
		}
	}
	return S_OK;
}
STDMETHODIMP Grapple::Layering(IMCCLayer *newLayer) {
    METHTRACE(Grapple,Layering);
	return S_FALSE;
}
STDMETHODIMP Grapple::ZChanged() {
    METHTRACE(Grapple,ZChanged);
	return S_FALSE;
}
STDMETHODIMP Grapple::Wakeup() {
	METHTRACE(Grapple,Wakeup);
	return S_FALSE;
}
// IMCCSelectionDraw
STDMETHODIMP Grapple::Show() {
    METHTRACE(Grapple,Show);
	return S_FALSE; // shouldn't get hit.
}
STDMETHODIMP Grapple::Hide() {
    METHTRACE(Grapple,Hide);
	RUN(AtlUnadvise(m_buddy,IID_IMCCSiteEvents,m_mnm));
	RUN(AtlUnadvise(m_buddy,IID_IMCCSelectionDraw,m_mnmWithWalnuts));
	if(m_spClientSite)
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->put_Intent(MCCS_DELETED));
	return S_OK;
}

// INeedOnePointer
STDMETHODIMP Grapple::TakePointer(IUnknown *u,IMoniker *m) {
	METHTRACE(Grapple,TakePointer);
	RUN(u->QueryInterface(riidret(&m_buddy)));
	RUN(AtlAdvise(m_buddy,GetUnknown(),IID_IMCCSiteEvents,&m_mnm));
	RUN(APCAdvise(m_buddy,GetUnknown(),IID_IMCCSelectionDraw,&m_mnmWithWalnuts));
	RECT rc;
	RUN(m_buddy->get_Rect(&rc));
	Moving(&rc);
	return S_OK;
}