/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef __GRAPPLE_H_
#define __GRAPPLE_H_

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////// Grapple
class ATL_NO_VTABLE Grapple :
	public CComObjectRoot,
	public CComCoClass<Grapple,&CLSID_MCCGrapple>,
	public CComControl<Grapple>,
	public CStockPropImpl<Grapple, DIGrapple, &IID_DIGrapple, &LIBID_Montage>,
	public IPersistStreamInitImpl<Grapple>,
	public IOleControlImpl<Grapple>,
	public IOleObjectImpl<Grapple>,
	public IOleInPlaceActiveObjectImpl<Grapple>,
	public IViewObjectExImpl<Grapple>,
	public IOleInPlaceObjectWindowlessImpl<Grapple>,
	public IMCCSiteEvents,
	public IMCCSelectionDraw,
	public INeedOnePointer // to site
{
	CComPtr<IMCCSite> m_buddy;
	DWORD m_mnm,m_mnmWithWalnuts;
	RECT m_rcFirst; // before tracking
	int m_track;  // which grapple we're tracking, or 8 for hatching
	POINT m_ptDown;
	bool m_bSigMove;
	DWORD m_buddyState;

	class GrabHandle {
		int size;
		RECT rcCntr;
	public:
		int n;
		RECT rc;
		GrabHandle(const RECT &rcCntr, int size) : rcCntr(rcCntr), size(size) {
			rc.left = rcCntr.left; rc.top = rcCntr.top;
			rc.right = rcCntr.left+size; rc.bottom = rcCntr.top + size;
			n = 0;
		}
		bool next();
	};
	int which_un(const RECT &rc,POINT pt) {
		GrabHandle gh(rc,m_nBorderWidth);
		do
			if(PtInRect(&gh.rc,pt))
				break;
		while(gh.next());
		return gh.n;
	}
	bool tracking();
public:
	Grapple();
	~Grapple();

REGGY_RID(IDR_GRAPPLE)

BEGIN_COM_MAP(Grapple)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(DIGrapple)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(IMCCSiteEvents)
	COM_INTERFACE_ENTRY(IMCCSelectionDraw)
	COM_INTERFACE_ENTRY(INeedOnePointer)
END_COM_MAP()

BEGIN_PROPERTY_MAP(Grapple)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(Grapple)
	MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
	MESSAGE_HANDLER(WM_LBUTTONUP, OnLButtonUp)
	CHAIN_MSG_MAP(CComControl<Grapple>)
END_MSG_MAP()

	LRESULT OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

// IViewObjectEx
	STDMETHOD(GetViewStatus)(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = 0;
		return S_OK;
	}
    STDMETHODIMP QueryHitPoint(DWORD dwAspect,LPCRECT pRectBounds,POINT ptlLoc,
        LONG lCloseHint,DWORD *pHitResult);

public:
// IMCCSiteEvents
	STDMETHODIMP Moving(LPCRECT winRect);
	STDMETHODIMP Shifting(long oldState, long newState);
	STDMETHODIMP Layering(IMCCLayer *newLayer);
	STDMETHODIMP ZChanged();
	STDMETHODIMP Wakeup();
// IMCCSelectionDraw
	STDMETHODIMP Show();
	STDMETHODIMP Hide();

// INeedOnePointer
	STDMETHODIMP TakePointer(IUnknown *u,IMoniker *m);

	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);

	long m_nBorderWidth;
};

#endif //__GRAPPLE_H_
