/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"
#include "Hatchling.h"
#include "cursors.h"
#include "bitBrush.h"

LPCSTR Hatchling::lpszClassName = "CCHatchWnd";
bool Hatchling::bRegged = false;

BitBrush g_hatchBrush(0xeeddbb77eeddbb77);

Hatchling::Hatchling(Container *montage, HWND hParentWnd,bool bGrabbers,int size, bool show) :
	m_montage(montage),m_bGrabbers(bGrabbers),
	m_size(size),m_currgrab(8),m_bCapture(false),m_hWnd(0) {
	CONSTRUCTORTRACE(Hatchling);
	if(!bRegged) {
		WNDCLASS wc;
		wc.style = CS_VREDRAW|CS_HREDRAW|CS_DBLCLKS;
		wc.lpfnWndProc = WndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = sizeof(Hatchling*);
		wc.hInstance = _Module.GetModuleInstance();
		wc.hIcon = NULL;
		wc.hCursor = LoadCursor(NULL,IDC_ARROW);;
		wc.hbrBackground = 0;
		wc.lpszMenuName = NULL;
		wc.lpszClassName = lpszClassName;
		RegisterClass(&wc);
		bRegged = true;
	}
	m_hWnd = CreateWindow(lpszClassName,"Container hatching",
		WS_CHILD|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
		0,0,0,0,
		hParentWnd,NULL,_Module.GetModuleInstance(),NULL);
	fit(m_montage->boundsPlusExtra(),m_montage->m_cliprect);
	if(show)
		ShowWindow(m_hWnd,SW_SHOWNORMAL);
	SetWindowLong(m_hWnd,0,(LONG)this);
}
Hatchling::~Hatchling() {
	DESTRUCTORTRACE(Hatchling);
	DestroyWindow(m_hWnd);
}
void Hatchling::fit(const RECT &pos,const RECT &clip) {
	RECT rc = pos;
	InflateRect(&rc,m_size,m_size);
	MoveWindow(m_hWnd,rc.left,rc.top,wid(rc),hei(rc),true);
	RECT visible;
	HRGN clipRgn(0);
	if(IntersectRect(&visible,&rc,&clip)&&!EqualRect(&visible,&rc)) {
		OffsetRect(&visible,-rc.left,-rc.top);
		clipRgn = CreateRectRgnIndirect(&visible);
	}
	SetWindowRgn(m_hWnd,clipRgn,true);
}
bool Hatchling::GrabHandle::next() {
	METHTRACE(Hatchling,GrabHandle);
	// counterclockwise from upper left
	switch(n++) {
	case 0:
		rc.top = (rcCntr.top+rcCntr.bottom-size)/2; rc.bottom = rc.top + size;
		return true;
	case 1:
		rc.top = rcCntr.bottom - size; rc.bottom = rcCntr.bottom;
		return true;
	case 2:
		rc.left = (rcCntr.left+rcCntr.right-size)/2; rc.right = rc.left + size;
		return true;
	case 3:
		rc.left = rcCntr.right-size; rc.right = rcCntr.right;
		return true;
	case 4:
		rc.top = (rcCntr.top+rcCntr.bottom-size)/2; rc.bottom = rc.top + size;
		return true;
	case 5:
		rc.top = rcCntr.top; rc.bottom = rcCntr.top+size;
		return true;
	case 6:
		rc.left = (rcCntr.left+rcCntr.right-size)/2; rc.right = rc.left+size;
		return true;
	default:
		return false;
	}
}
LRESULT CALLBACK Hatchling::WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {
	FUNTRACE(HatchlingWndProc);
	Hatchling *pThis = (Hatchling *)GetWindowLong(hWnd,0);
	switch(uMsg) {
	case WM_LBUTTONDOWN: {
		POINT pt = MAKEPOINT(lParam);
		int grab = pThis->which_un(pt);
		if(grab<8) {
			SetCapture(hWnd);
			pThis->m_bCapture = true;
			pThis->m_currgrab = grab;
			pThis->m_ptDown = pt;
			// show greything
		}
		return 0;
	}
	case WM_LBUTTONUP:
		if(pThis->m_bCapture) {
			ReleaseCapture();
			pThis->m_bCapture = false;
			POINT ptUp = MAKEPOINT(lParam);
			int dx = ptUp.x-pThis->m_ptDown.x,
				dy = ptUp.y-pThis->m_ptDown.y;
			HWND hParWnd = GetParent(hWnd);
			RECT rcNew = pThis->m_montage->m_rect;
			// hide greything
			switch(pThis->m_currgrab) {
			case 0:
			case 1:
			case 2:
				rcNew.left += dx;
				break;
			case 4:
			case 5:
			case 6:
				rcNew.right += dx;
			}
			switch(pThis->m_currgrab) {
			case 0:
			case 7:
			case 6:
				rcNew.top += dy;
				break;
			case 2:
			case 3:
			case 4:
				rcNew.bottom += dy;
			}
			// tell montage to go to new pos and adjust extent as appropriate
			SRUN(pThis->m_montage->setExtentFromRect(rcNew,true));
			SRUN(pThis->m_montage->put_AutoSize(VARIANT_FALSE));
			SRUN(pThis->m_montage->m_pInPlaceSite->OnPosRectChange(&rcNew));
			return 0;
		}
	case WM_RBUTTONDOWN:
	case WM_RBUTTONUP:
	case WM_MBUTTONDOWN:
	case WM_MBUTTONUP:
		return 0;
	case WM_LBUTTONDBLCLK:
		VARIANT_BOOL v;
		SRUN(pThis->m_montage->get_AutoSize(&v));
		SRUN(pThis->m_montage->put_AutoSize(v^VARIANT_TRUE));
		InvalidateRect(hWnd,0,false);
		return 0;
	case WM_MOUSEMOVE:
		if(pThis->m_bCapture) {
			// move greything
			return 0;
		}
		// else...
	case WM_SETCURSOR:
		if(pThis->m_bGrabbers) {
			POINT pt;
			if(uMsg==WM_SETCURSOR) {
				GetCursorPos(&pt);
				ScreenToClient(hWnd,&pt);
			}
			else
				pt = MAKEPOINT(lParam);
			int grab = pThis->which_un(pt);
			// always set if SETCURSOR
			if((grab!=pThis->m_currgrab)||(uMsg==WM_SETCURSOR)) {
				if(grab<8)
					SetCursor(g_cursors.track[grab]);
				else
					SetCursor(g_cursors.fourway);
				pThis->m_currgrab = grab;
			}
		}
		return 0;
	case WM_SETFOCUS:
		SetFocus(pThis->m_montage->m_hWnd);
		return 0;
	case WM_CHAR:
	case WM_KEYDOWN:
		goto def;
	case WM_ERASEBKGND:
		RECT r;
		SRUN(GetClientRect(hWnd,&r));
		SCHECK(FillRect((HDC)wParam,&r,g_hatchBrush.brush));
		return true;
	case WM_PAINT:
		if(pThis->m_bGrabbers) { // draw 'em
			PAINTSTRUCT ps;
			HDC hDC = BeginPaint(hWnd,&ps);
			HBRUSH hbr;
			if(pThis&&pThis->m_montage->m_autoSize)
				hbr = CreateSolidBrush(RGB(0x0ff,0x6c,0));
			else
				hbr = (HBRUSH)GetStockObject(BLACK_BRUSH);
			GrabHandle gh(pThis->rect(),pThis->m_size);
			DCObject brush(hDC,hbr),
				pen(hDC,GetStockObject(BLACK_PEN));
			do
				Rectangle(hDC,gh.rc.left,gh.rc.top,gh.rc.right,gh.rc.bottom);
			while(gh.next());
			EndPaint(hWnd,&ps);
			return 0;
		}
		// else drop through: validate update region
	default:
	def:
		return DefWindowProc(hWnd,uMsg,wParam,lParam);
	}
}
