/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "InitList.h"

InitList::InitList() {
	CONSTRUCTORTRACE(InitList);
}
InitList::~InitList() {
	DESTRUCTORTRACE(InitList);
}
STDMETHODIMP InitList::InitializeOther(IUnknown *other) {
	METHTRACE(InitList,InitializeOther);
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<IEnumMoniker> em;
	RUN(CComQIPtr<IOrderedItemList>(m_list)->Enum(INT_MIN,true,&em));
	CComPtr<IMoniker> mk,mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	while(em->Next(1,&mk,0)==S_OK)
		if(mk) {
			CComPtr<IUnknown> u;
			RUN(mk->BindToObject(bctx,mkG,riidret(&u)));
			if(CComQIPtr<IInitializeOther> io = u)
				RUN(io->InitializeOther(other));
			else if(CComQIPtr<IObjectWithSite> ows = u)
				RUN(ows->SetSite(other));
			mk.Release();
		}
	return S_OK;
}
// IGroupItem
STDMETHODIMP InitList::Connect(IGroup *group) {
	METHTRACE(InitList,Connect);
	m_group = group;
	return CComQIPtr<IGroupItem>(m_list)->Connect(group);
}
STDMETHODIMP InitList::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(InitList,Copy);
	HRESULT hr = E_FAIL;
	*ret = 0;
	// aggregation is 0 fun.  It would be easier to simply create a new InitList and feed it each
	// moniker, but I had to prove (to myself) that the Copy-aggregation scheme was actually
	// possible.  It is, but barely.  The basic problem is that the standard creation method
	// (CCom[Agg]Object::CreateInstance) calls FinalConstruct, which uses CoCreateInstance
	// to create the aggregated OrderedItemList, when we really want to create it with
	// the current aggregate's IGroupItem::Copy.
	if(aggr) {
		if(!InlineIsEqualUnknown(riid))
			ERR(E_INVALIDARG);
		typedef CComAggObject<InitList> full;
		full *create = new full(aggr);
		CComPtr<IUnknown> aggr2 = create;
		if(SUCCEEDED(CComQIPtr<IGroupItem>(m_list)->Copy(bctx,dest,aggr2,riidret(&create->m_contained.m_list)))) {
			*ret = aggr2;
			aggr2.p = 0; // keep refcount by disabling destructor
			hr = S_OK;
		}
	}
	else {
		typedef CComObject<InitList> full;
		full *il2 = new full;
		CComPtr<IUnknown> aggr2 = (IGroupItem*)il2; // vital refcount: if QI below fails or isn't hit, il2 will be destroyed
		if(SUCCEEDED(CComQIPtr<IGroupItem>(m_list)->Copy(bctx,dest,aggr2,riidret(&il2->m_list))))
			hr = aggr2->QueryInterface(riid,ret);
	}
	if(SUCCEEDED(hr))
		RUN(CComQIPtr<IGroupItem>((IUnknown*)*ret)->Connect(dest));
	return hr;
}
STDMETHODIMP InitList::Realize(IBindCtx *bctx) {
	METHTRACE(InitList,Realize);
	return CComQIPtr<IGroupItem>(m_list)->Realize(bctx);
}
STDMETHODIMP InitList::Remove() {
	METHTRACE(InitList,Remove);
	return CComQIPtr<IGroupItem>(m_list)->Remove();
}
