/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ItemSetActivator.h"

ItemSetActivator::ItemSetActivator() : m_amStarted(false) {
	CONSTRUCTORTRACE(ItemSetActivator);
}
ItemSetActivator::~ItemSetActivator() {
	DESTRUCTORTRACE(ItemSetActivator);
}
HRESULT ItemSetActivator::bind(IMoniker *mk, IMCCSite **ret) {
	METHTRACE(ItemSetActivator,bind);
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<DIPersistor> per;
	RUN(mk->BindToObject(bctx,m_mkGroup,riidret(&per)));
	CComPtr<IUnknown> uS;
	RUN(per->get_Subject(&uS));
	if(!uS) // not realized yet: don't force
		return S_FALSE;
	return uS.QueryInterface(ret);
}
HRESULT ItemSetActivator::activate(IMoniker *mk) {
	METHTRACE(ItemSetActivator,activate);
	CComPtr<IMCCSite> s;
	RUN(bind(mk,&s));
	if(!s)
		return S_FALSE;
	return Activate(s);
}
HRESULT ItemSetActivator::deactivate(IMoniker *mk) {
	METHTRACE(ItemSetActivator,deactivate);
	CComPtr<IMCCSite> s;
	RUN(bind(mk,&s));
	if(!s)
		return S_FALSE;
	return Deactivate(s);
}
// ICEMItemSetActivator
STDMETHODIMP ItemSetActivator::Initialize(IItemSet *selection,IMCCLayer *mainl, IMCCLayer *activel) {
	METHTRACE(ItemSetActivator,Initialize);
	if(m_selection	)
		return S_FALSE;
	m_selection = selection;
	m_mainl = mainl;
	m_activel = activel;
	CComPtr<IMontageContainer> mc;
	RUN(mainl->GetContainer(riidret(&mc)));
	CComPtr<IGroup> g;
	RUN(mc->GetMainGroup(riidret(&g)));
	return FoolishCreatePointerMoniker(g,&m_mkGroup);
}
STDMETHODIMP ItemSetActivator::Activate(IMCCSite *s) {
    METHTRACE(ItemSetActivator,Activate);
	long state;
	RUN(s->get_Intent(&state));
	if(state==MCCS_DELETED)
		return S_FALSE;
	RUN(s->put_Layer(m_activel));
	RUN(fireSiteEvent(s,fireIntend(MCCS_COUIACTIVE)));
	RUN(s->get_Intent(&state));
	if(state<MCCS_ACTIVE) // activation failed or opened in other window instead
		RUN(s->put_Layer(m_mainl));
	return S_OK;
}
STDMETHODIMP ItemSetActivator::Deactivate(IMCCSite *s) {
    METHTRACE(ItemSetActivator,Deactivate);
	long intent;
	RUN(s->get_Intent(&intent));
	if(intent==MCCS_DELETED)
		return S_FALSE;
	RUN(s->put_Layer(m_mainl));
	return fireSiteEvent(s,fireIntend(MCCS_SHOWN));
}

STDMETHODIMP ItemSetActivator::Start() {
	METHTRACE(ItemSetActivator,Start);
	if(m_amStarted)
		return S_FALSE;
	m_amStarted = true;
	CComPtr<IEnumMoniker> em;
	RUN(m_selection->Enum(&em));
	for(mk_iterator i(em);i!=mk_iterator();++i)
		RUN(activate(*i));
	return AtlAdvise(m_selection,(IItemSetEvents*)this,IID_IItemSetEvents,&m_hydrox);
}
STDMETHODIMP ItemSetActivator::Stop() {
	METHTRACE(ItemSetActivator,Stop);
	if(!m_amStarted)
		return S_FALSE;
	m_amStarted = false;
	RUN(AtlUnadvise(m_selection,IID_IItemSetEvents,m_hydrox));
	CComPtr<IEnumMoniker> em;
	RUN(m_selection->Enum(&em));
	for(mk_iterator i(em);i!=mk_iterator();++i)
		RUN(deactivate(*i));
	return S_OK;
}
STDMETHODIMP ItemSetActivator::NoUI() {
	METHTRACE(ItemSetActivator,NoUI);
	HRESULT hr;
	CComPtr<IEnumMoniker> em;
	RUN(m_selection->Enum(&em));
	for(mk_iterator i(em);i!=mk_iterator();++i) {
		CComPtr<IMCCSite> s;
		RUN(bind(*i,&s));
		if(!s)
			continue;
		CComPtr<IOleInPlaceObject> oipo;
		if(SUCCEEDED(s->GetControl(riidret(&oipo))))
			hr = oipo->UIDeactivate(); // can fail if the thing's not UI activated?
	}

	return S_OK;
}
// DICEMItemSetActivator
STDMETHODIMP ItemSetActivator::Initialize(DIItemSet *selection,DIMCCLayer *mainl, DIMCCLayer *activel) {
    METHTRACE(ItemSetActivator,Initialize);
	CComPtr<IItemSet> is2;
	if(selection)
		RUNRET(selection->QueryInterface(riidret(&is2)));
	CComPtr<IMCCLayer> ml2,al2;
	if(mainl)
		RUNRET(mainl->QueryInterface(riidret(&ml2)));
	if(activel)
		RUNRET(activel->QueryInterface(riidret(&al2)));
	return Initialize(is2,ml2,al2);
}
// IItemSetEvents
STDMETHODIMP ItemSetActivator::Added(IMoniker *mk) {
	METHTRACE(ItemSetActivator,Added);
	activate(mk);
	return S_OK;
}
STDMETHODIMP ItemSetActivator::Removed(IMoniker *mk) {
	METHTRACE(ItemSetActivator,Removed);
	deactivate(mk);
	return S_OK;
}
// INeedSomePointers
STDMETHODIMP ItemSetActivator::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(ItemSetActivator,TakePointers);
	CComPtr<IUnknown> u;
	RUN(os->Next(1,&u,0));
	CComQIPtr<IItemSet> iset = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMCCLayer> mainl = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMCCLayer> activel = u;
	return Initialize(iset,mainl,activel);
}
