/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ItemSetDrawer.h"

ItemSetDrawer::ItemSetDrawer() : m_amDrawing(false) {
	CONSTRUCTORTRACE(ItemSetDrawer);
}
ItemSetDrawer::~ItemSetDrawer() {
	DESTRUCTORTRACE(ItemSetDrawer);
}
// IItemSetDrawer
STDMETHODIMP ItemSetDrawer::Initialize(IMontageContainer *mc, IItemSet *iset) {
	METHTRACE(ItemSetDrawer,Initialize);
	if(m_mkGroup)
		return S_FALSE;
	CComPtr<IGroup> g;
	RUN(mc->GetMainGroup(riidret(&g)));
	RUN(FoolishCreatePointerMoniker(g,&m_mkGroup));
	m_set = iset;
	DWORD molasses;
	RUN(AtlAdvise(m_set,(IItemSetEvents*)this,IID_IItemSetEvents,&molasses));
	if(m_amDrawing) { // re-loaded: fake a start
		m_amDrawing = false;
		RUN(Start());
	}
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::Start() {
    METHTRACE(ItemSetDrawer,Start);
	if(m_amDrawing)
		return S_FALSE;
	m_amDrawing = true;
	// fire ItemSetEvents on self
	CComPtr<IEnumMoniker> em;
	RUN(m_set->Enum(&em));
	for(mk_iterator i(em);i!=mk_iterator();++i)
		RUN(Added(*i));
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::Stop() {
    METHTRACE(ItemSetDrawer,Stop);
	if(!m_amDrawing)
		return S_FALSE;
	m_amDrawing = false;
	// fire ItemSetEvents on self
	CComPtr<IEnumMoniker> em;
	RUN(m_set->Enum(&em));
	for(mk_iterator i(em);i!=mk_iterator();++i)
		RUN(Removed(*i));
	return S_OK;
}
// DICEMItemSetDrawer
STDMETHODIMP ItemSetDrawer::Initialize(DIMontageContainer *mc, DIItemSet *iset) {
    METHTRACE(ItemSetDrawer,Initialize);
	CComPtr<IMontageContainer> mc2;
	if(mc)
		RUNRET(mc->QueryInterface(riidret(&mc2)));
	CComPtr<IItemSet> is2;
	if(iset)
		RUNRET(iset->QueryInterface(riidret(&is2)));
	return Initialize(mc2,is2);
}
// INeedSomePointers
STDMETHODIMP ItemSetDrawer::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(ItemSetDrawer,TakePointers);
	CComPtr<IUnknown> u;
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMontageContainer> mc = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IItemSet> s = u;
	return Initialize(mc,s);
}
HRESULT ItemSetDrawer::bind(IMoniker *mk, IMCCSite **ret) {
	METHTRACE(ItemSetDrawer,bind);
	HRESULT hr;
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<DIPersistor> per;
	if(FAILED(hr = mk->BindToObject(bctx,m_mkGroup,riidret(&per))))
		return hr; // "legal error"
	CComPtr<IUnknown> uS;
	RUN(per->get_Subject(&uS));
	if(!uS)
		return S_FALSE;
	return uS.QueryInterface(ret);
}
typedef CComObject<SelectedOwner> SO;
// IItemSetEvents
STDMETHODIMP ItemSetDrawer::Added(IMoniker *mk) {
	METHTRACE(ItemSetDrawer,Added);
	if(!m_amDrawing)
		return S_FALSE;
	CComPtr<IMCCSite> s;
	RUN(bind(mk,&s));
	if(!s)
		return S_FALSE;
	long intent;
	RUN(s->get_Intent(&intent));
	if(intent==MCCS_DELETED)
		return S_FALSE;
	// fire selection-show event
	RUN(fireSiteEvent(s,fireVoid<IMCCSelectionDraw,&IID_IMCCSelectionDraw>(&IMCCSelectionDraw::Show)));
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::Removed(IMoniker *mk) {
	METHTRACE(ItemSetDrawer,Removed);
	if(!m_amDrawing)
		return S_FALSE;
	CComPtr<IMCCSite> s;
	LOOK(bind(mk,&s));// item may have been deleted
	if(!s)
		return S_FALSE;
	long intent;
	RUN(s->get_Intent(&intent));
	if(intent==MCCS_DELETED)
		return S_FALSE;
	// hide selection stuff
	RUN(fireSiteEvent(s,fireVoid<IMCCSelectionDraw,&IID_IMCCSelectionDraw>(&IMCCSelectionDraw::Hide)));
	return S_OK;
}
// IPersistStream
STDMETHODIMP ItemSetDrawer::GetClassID(CLSID *ret) {
    METHTRACE(ItemSetDrawer,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::IsDirty(void) {
    METHTRACE(ItemSetDrawer,IsDirty);
	return S_FALSE;
}
STDMETHODIMP ItemSetDrawer::Load(IStream *pStm) {
    METHTRACE(ItemSetDrawer,Load);
	RUN(pStm->Read(&m_amDrawing,sizeof m_amDrawing,0));
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(ItemSetDrawer,Save);
	RUN(pStm->Write(&m_amDrawing,sizeof m_amDrawing,0));
	return S_OK;
}
STDMETHODIMP ItemSetDrawer::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(ItemSetDrawer,GetSizeMax);
	ERR(E_NOTIMPL);
}
