/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


class ItemSetRenderer :
	public CComObjectRoot,
	public CComCoClass<ItemSetRenderer,&CLSID_MCCItemSetRenderer>,
	public IMCCItemSetRenderer,
	public IDispatchImpl<DIMCCItemSetRenderer>,
	public IDataObject,
	public IItemSetEvents,
	public INeedSomePointers,
	public IPersistStream,
	public IMontageContainerEvents
{
	CComPtr<IGroup> m_srcGroup;
	CComPtr<IItemSet> m_isetData;
	CComPtr<IMCCDataObject> m_buffer;
	DWORD m_gingerbreadMan; // itemsetevents
	CLSID m_clsid;
    CComPtr<DIMontageContainerRef> m_mref; // semi-circular reference counting
    CComPtr<IMontageContainer> mc()  {
        METHTRACE(ItemSetRenderer,mc);
        CComPtr<IUnknown> umc;
        RUN(m_mref->get_Montage(&umc));
        return CComQIPtr<IMontageContainer>(umc);
    }
	bool m_haveMousePt,m_overrideBounds;
	RECTL m_override;
	POINTL m_mousePt;
	HGLOBAL m_descriptor;
	void freeDesc() {
		if(m_descriptor) {
			GlobalFree(m_descriptor);
			m_descriptor = 0;
		}
	}
	CComPtr<IDataAdviseHolder> m_advice;

	CComPtr<IItemSet> expandDataSet();
	HRESULT calcBoundary(IItemSet *data,RECTL *ret);
	HRESULT calcDescriptor(RECTL bounds);
	HRESULT getMetafile(RECTL bounds,IItemSet *data,DWORD tymed,STGMEDIUM *stgm);
	HRESULT getStorage(RECTL bounds,IStorage *stg);
	HRESULT getOD(HGLOBAL glob);

public:
	ItemSetRenderer();
	~ItemSetRenderer();

	void changed();

	REGISTER_GENERIC(L"ItemSetRenderer")
BEGIN_COM_MAP(ItemSetRenderer)
	COM_INTERFACE_ENTRY(IMCCItemSetRenderer)
	COM_INTERFACE_ENTRY(DIMCCItemSetRenderer)
	COM_INTERFACE_ENTRY(IMontageContainerEvents)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IItemSetEvents)
	COM_INTERFACE_ENTRY(INeedSomePointers)
	COM_INTERFACE_ENTRY(IPersistStream)
	COM_INTERFACE_ENTRY(IPersist)
END_COM_MAP()
// IMCCItemSetRenderer
	STDMETHODIMP Initialize(IGroup *srcGroup, IItemSet *isetData,REFCLSID rcls,IMontageContainer *mc, IMCCDataObject *mdo);
	STDMETHODIMP Stop();
	STDMETHODIMP put_MousePoint(POINT val);
	STDMETHODIMP put_UpperLeft(POINTL val);
	STDMETHODIMP get_Bounds(RECTL *ret);
	STDMETHODIMP SetBoundary(RECTL *val);
// DIMCCItemSetRenderer
	STDMETHODIMP Initialize(DIGroup *srcGroup,DIItemSet *isetData,BSTR rcls,DIMontageContainer *mc,
		IUnknown *buffer);
	STDMETHODIMP SetMousePoint(long x,long y);
// IMontageContainerEvents
	STDMETHODIMP Shutdown();
	STDMETHODIMP Resize(SIZE s);
	STDMETHODIMP NewExtent(SIZEL);
	STDMETHODIMP Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv);
	STDMETHODIMP NewTransform(IMCCTransform*);
	STDMETHODIMP GroupOp(BOOL);
	STDMETHODIMP SiteDeleted(IMCCSite*);
	STDMETHODIMP LayerDeleted(IMCCLayer*);
	STDMETHODIMP Shifting(long oldState,long newState);
	STDMETHODIMP ShowArea(LPCRECTL r) { return S_FALSE; }
    STDMETHODIMP FocusChanged() { return S_FALSE; }
// IItemSetEvents
	STDMETHODIMP Added(IMoniker *mk);
	STDMETHODIMP Removed(IMoniker *mk);
// IDataObject
    STDMETHODIMP GetData(FORMATETC *pformatetcIn,STGMEDIUM *pmedium);
    STDMETHODIMP GetDataHere(FORMATETC *pformatetc,STGMEDIUM *pmedium);
    STDMETHODIMP QueryGetData(FORMATETC *pformatetc);
    STDMETHODIMP GetCanonicalFormatEtc(FORMATETC *pformatectIn,FORMATETC *pformatetcOut);
    STDMETHODIMP SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease);
    STDMETHODIMP EnumFormatEtc(DWORD dwDirection,IEnumFORMATETC **ppenumFormatEtc);
    STDMETHODIMP DAdvise(FORMATETC *pformatetc,DWORD advf,IAdviseSink *pAdvSink,
         DWORD *pdwConnection);
    STDMETHODIMP DUnadvise(DWORD dwConnection);
    STDMETHODIMP EnumDAdvise(IEnumSTATDATA **ppenumAdvise);
// INeedSomePointers
	STDMETHODIMP TakePointers(IEnumUnknown *os,IEnumMoniker *ms);
// IPersistStream
	STDMETHODIMP GetClassID(CLSID *pClassID);
	STDMETHODIMP IsDirty(void);
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
};
