/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "LayerPersistor.h"

LayerPersistor::LayerPersistor() {
	CONSTRUCTORTRACE(LayerPersistor);
}
LayerPersistor::~LayerPersistor() {
	DESTRUCTORTRACE(LayerPersistor);
}
// IPersistor
STDMETHODIMP LayerPersistor::SetSubject(VARIANT_BOOL realized,IUnknown *pUnkSite) {
	METHTRACE(LayerPersistor,SetSubject);
	// like the other custom persistors which need to bind a moniker to create their subject,
	// this persistor doesn't support "realize this" mode - if it has a subject, the persistor
	// considers itself realized and will return S_FALSE (won't connect a live object)
	m_subject = pUnkSite;
	return S_OK;
}
STDMETHODIMP LayerPersistor::get_Subject(IUnknown **ret) {
	METHTRACE(LayerPersistor,get_Subject);
	m_subject.CopyTo((IMCCLayer**)ret);
	return S_OK;
}
// IMCCLayerPersistor
STDMETHODIMP LayerPersistor::Initialize(IMoniker *mkMontage,
		IMoniker *mkLayerOrdering,IMoniker *mkSiteOrdering, IMoniker *mkDefaultAmbience) {
	METHTRACE(LayerPersistor,Initialize);
	if(!m_group)
		ERR(E_PENDING);
	if(m_mkMontage)
		ERR(E_UNEXPECTED);
	RUN(m_group->AddStgRef(m_mkMontage = mkMontage));
	RUN(m_group->AddStgRef(m_mkLayerOrdering = mkLayerOrdering));
	RUN(m_group->AddStgRef(m_mkSiteOrdering = mkSiteOrdering));
	RUN(m_group->AddStgRef(m_mkAmbience = mkDefaultAmbience));
	return S_OK;
}
STDMETHODIMP LayerPersistor::put_SelfMk(IMoniker *val) {
	METHTRACE(LayerPersistor,put_SelfMk);
	m_mkSelf = val;
	return S_OK;
}
// DIMCCLayerPersistor
STDMETHODIMP LayerPersistor::Initialize(IUnknown *mkMontage,
	IUnknown *mkLayerOrdering, IUnknown *mkSiteOrdering,
	IUnknown *mkDefaultAmbience) {
    METHTRACE(LayerPersistor,Initialize);
	CComPtr<IMoniker> m2,lo2,so2,ds2;
	if(mkMontage)
		RUNRET(mkMontage->QueryInterface(riidret(&m2)));
	if(mkLayerOrdering)
		RUNRET(mkLayerOrdering->QueryInterface(riidret(&lo2)));
	if(mkSiteOrdering)
		RUNRET(mkSiteOrdering->QueryInterface(riidret(&so2)));
	if(mkDefaultAmbience)
		RUNRET(mkDefaultAmbience->QueryInterface(riidret(&ds2)));
	return Initialize(m2,lo2,so2,ds2);
}
STDMETHODIMP LayerPersistor::put_SelfMk(IUnknown *val) {
    METHTRACE(LayerPersistor,put_SelfMk);
	CComPtr<IMoniker> m2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&m2)));
	return put_SelfMk(m2);
}
// IGroupItem
STDMETHODIMP LayerPersistor::Connect(IGroup *group) {
	METHTRACE(LayerPersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	return S_OK;
}
STDMETHODIMP LayerPersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(LayerPersistor,Copy);
	CComPtr<IMCCLayerPersistor> lp2;
	RUN(CREATE(lp2,CLSID_MCCLayerPersistor));
	CComPtr<IMoniker> mon2, layord2, sitord2, sty2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkMontage,IID_NULL,0,&mon2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkLayerOrdering,IID_NULL,0,&layord2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkSiteOrdering,IID_NULL,0,&sitord2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkAmbience,IID_NULL,0,&sty2));
	RUN(CComQIPtr<IGroupItem>(lp2)->Connect(dest));
	RUN(lp2->Initialize(mon2,layord2,sitord2,sty2));
	if(m_subject)
		RUN(CComQIPtr<DIPersistor>(lp2)->SetSubject(m_subject!=0,m_subject));
	else {
		CComPtr<IMoniker> self2;
		RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkSelf,IID_NULL,0,&self2));
		RUN(lp2->put_SelfMk(self2));
	}
	return lp2->QueryInterface(riid,ret);
}
STDMETHODIMP LayerPersistor::Realize(IBindCtx *bctx) {
	METHTRACE(LayerPersistor,Realize);
	if(m_subject)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}

	// create layer
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	CComPtr<DIPersistor> p;
	RUN(m_mkMontage->BindToObject(bctx,mkG,riidret(&p)));
	CComPtr<IUnknown> umc;
	RUN(p->get_Subject(&umc));
	CComPtr<IMontageContainer> mc;
	RUN(umc.QueryInterface(&mc));
	RUN(mc->new_Layer(&m_subject));
	m_subject = m_subject;

	// put mkSelf and mkLayerOrder (we're already in this group)
	RUN(m_subject->put_RelMoniker(m_mkSelf));
	RUN(m_subject->put_OrderMk(m_mkSiteOrdering));

	// init transformedness
	RUN(m_subject->put_IsTransformed(m_isTransformed));

	// re-insert
	if(m_inserted) {
		CComPtr<IMCCLayerOrdering> order;
		RUN(m_mkLayerOrdering->BindToObject(bctx,mkG,riidret(&order)));
		RUN(order->Place(m_Z,m_subject));
	}

	// init ambiance
	if(m_mkAmbience) {
		CComPtr<IDispatch> defAmb;
		RUN(m_mkAmbience->BindToObject(bctx,mkG,riidret(&defAmb)));
		RUN(m_subject->put_DefaultAmbiance(defAmb));
	}
	return S_OK;
}
STDMETHODIMP LayerPersistor::Remove() {
	METHTRACE(LayerPersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkMontage));
	RUN(m_group->ReleaseStgRef(m_mkLayerOrdering));
	RUN(m_group->ReleaseStgRef(m_mkSiteOrdering));
	RUN(m_group->ReleaseStgRef(m_mkAmbience));
	return S_OK;
}
// IPersistStream
STDMETHODIMP LayerPersistor::GetClassID(CLSID *ret) {
	METHTRACE(LayerPersistor,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP LayerPersistor::IsDirty(void) {
	METHTRACE(LayerPersistor,IsDirty);
	return S_FALSE;
}
STDMETHODIMP LayerPersistor::Load(IStream *pStm) {
	METHTRACE(LayerPersistor,Load);
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkSelf)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkMontage)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkLayerOrdering)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkSiteOrdering)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkAmbience)));
	RUN(pStm->Read(&m_isTransformed,sizeof(BOOL),0));
	RUN(pStm->Read(&m_inserted,sizeof(bool),0));
	if(m_inserted)
		RUN(pStm->Read(&m_Z,sizeof(DWORD),0));
	return S_OK;
}
STDMETHODIMP LayerPersistor::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(LayerPersistor,Save);
	if(m_subject) { // get info from layer
		// the complicated part is the relative moniker, which must be translated to
		// our group. See the SitePersistor for a little more info on what I'm doing here.
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		CComPtr<IMontageContainer> mc;
		RUN(m_subject->GetContainer(riidret(&mc)));
		CComPtr<IGroup> mainGroup;
		RUN(mc->GetMainGroup(riidret(&mainGroup)));
		CComPtr<IMoniker> relMk;
		RUN(m_subject->get_RelMoniker(&relMk));
		m_mkSelf.Release();
		RUN(m_group->Paste(bctx,GP_SrcIsCont,mainGroup,relMk,IID_NULL,0,&m_mkSelf));
		// get transformedness
		RUN(m_subject->get_IsTransformed(&m_isTransformed));
		// & Z
		m_inserted = SUCCEEDED(m_subject->get_Z(&m_Z));
	}
	else if(!m_mkMontage) // cannot save if I haven't been loaded
		ERR(E_PENDING);
	RUN(FixedOleSaveToStream(m_mkSelf,pStm));
	RUN(FixedOleSaveToStream(m_mkMontage,pStm));
	RUN(FixedOleSaveToStream(m_mkLayerOrdering,pStm));
	RUN(FixedOleSaveToStream(m_mkSiteOrdering,pStm));
	RUN(FixedOleSaveToStream(m_mkAmbience,pStm));
	RUN(pStm->Write(&m_isTransformed,sizeof(BOOL),0));
	RUN(pStm->Write(&m_inserted,sizeof(bool),0));
	if(m_inserted)
		RUN(pStm->Write(&m_Z,sizeof(DWORD),0));
	return S_OK;
}
STDMETHODIMP LayerPersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(LayerPersistor,GetSizeMax);
	ULARGE_INTEGER ul;
	RUN(m_mkMontage->GetSizeMax(&ul));
	*pcbSize = ul;
	RUN(m_mkAmbience->GetSizeMax(&ul));
	pcbSize->QuadPart += ul.QuadPart;
	return S_OK;
}

