/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"

HRESULT Site::GetContainer(REFIID riid, void **ret) {
	IMETHTRACE(Site,IMCCSite,GetContainer);
	CHECK_DEL;
	return ((IMontageContainer*)m_cont)->QueryInterface(riid,ret);
}
STDMETHODIMP Site::get_RelMoniker(IMoniker **ret) {
	IMETHTRACE(Site,IMCCSite,get_RelMoniker);
	CHECK_DEL;
	m_mkPersistor.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Site::put_RelMoniker(IMoniker *val) {
	IMETHTRACE(Site,IMCCSite,put_RelMoniker);
	CHECK_DEL;
	m_mkPersistor = val;
	return S_OK;
}

HRESULT Site::GetControl(REFIID riid, void **ppCtrl) {
	IMETHTRACE(Site,IMCCSite,GetControl);
	CHECK_DEL;
	if(!m_pControl) {
		*ppCtrl = 0;
		return S_OK;
	}
	else return m_pControl->QueryInterface(riid,ppCtrl);
}
HRESULT Site::put_Control(IUnknown *pCtrl) {
	IMETHTRACE(Site,IMCCSite,PutControl);
	HRESULT hr;
	// shut off the old control if any
	if(m_state>MCCS_HIDDEN)
		RUN(approachstate(MCCS_HIDDEN));
	if(CComQIPtr<IOleObject> oo = m_pControl)
		RUN(oo->SetClientSite(0));
	else if(CComQIPtr<IObjectWithSite> ows = m_pControl)
		RUN(ows->SetSite(0));
	m_pControl.Release();
	if(!pCtrl)
		return S_OK;
	CHECK_DEL;
	if(FAILED(pCtrl->QueryInterface(riidret(&m_pControl)))) {
		// if you get here it's probably because you've got a control instance that
		// doesn't have its required OLE handler aggregate: this happens when embedded
		// in-process into the same object type as embedding (e.g. word-in-montage-in-word)
		void *v;
		hr = pCtrl->QueryInterface(IID_IPersistStorage,&v);
		hr = pCtrl->QueryInterface(IID_IDataObject,&v);
		hr = pCtrl->QueryInterface(IID_IOleCache,&v);
		hr = pCtrl->QueryInterface(IID_IOleObject,&v);
		RUN(E_FAIL);
	}
	// Montage does not support OLEMISC_SETCLIENTSITEFIRST at this point.
	// I think a special persistor could fix this - I tried a bind context hack
	// (getting the group to call SetClientSite before Load) but that exploded
	// into lots of buggy code.)
	if(CComQIPtr<IOleObject> oo = pCtrl)
		RUN(oo->SetClientSite((IOleClientSite*)this));
	else if(CComQIPtr<IObjectWithSite> ows = pCtrl)
		RUN(ows->SetSite((IOleClientSite*)this));
	// restore state
	RUN(approachstate(m_intent));
	return S_OK;
}
STDMETHODIMP Site::get_Layer(IMCCLayer **ppRet) {
	IMETHTRACE(Site,IMCCSite,get_Layer);
	CHECK_DEL;
	Layer *l = (m_pLayer==m_cont->m_nonlayer)?0:m_pLayer;
	if(*ppRet = l)
		l->AddRef();
	return S_OK;
}
STDMETHODIMP Site::put_Layer(IMCCLayer *pParm) {
	IMETHTRACE(Site,IMCCSite,put_Layer);
	CHECK_DEL;
	RUN(fireEvent(GetUnknown(),fireOneParam<IMCCSiteEvents,IMCCLayer*>(&IMCCSiteEvents::Layering,pParm)));
	RUN(m_cont->remove((site*)this));
	m_pLayer = (Layer*)(pParm?pParm:m_cont->m_nonlayer);
	AddRef();
	RUN(m_cont->after((site*)this));
	if(m_state > MCCS_HIDDEN) {
		m_cont->InvalidateSite((site*)this);
		RUN(m_cont->dirty());
	}
	RUN(fireEvent(GetUnknown(),fireVoid<IMCCSiteEvents>(&IMCCSiteEvents::ZChanged)));
	return S_OK;
}
STDMETHODIMP Site::get_Ambiance(IDispatch **ret) {
	METHTRACE(Site,get_Ambiance);
	CHECK_DEL;
	m_ambiance.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Site::put_Ambiance(IDispatch *val) {
	METHTRACE(Site,put_Ambiance);
	CHECK_DEL;
	m_ambiance = val;
	return S_OK;
}

STDMETHODIMP Site::get_Z(long *ret) {
	METHTRACE(Site,get_Z);
	CHECK_DEL;
	long depth(0);
	for(sitelist::iterator i = m_pLayer->m_sites.begin(); i != m_pLayer->m_sites.end();i++)
		if(*i == this)
			break;
		else
			depth++;
	if(i==m_pLayer->m_sites.end())
		ERR(E_UNEXPECTED);
	*ret = depth;
	return S_OK;
}
STDMETHODIMP Site::put_ClipMode(enum MCC_SITE_CLIP_MODE val) {
    METHTRACE(Site,put_ClipMode);
	m_clipMode = val;
	return S_OK;
}
STDMETHODIMP Site::get_ClipMode(enum MCC_SITE_CLIP_MODE *ret) {
    METHTRACE(Site,get_ClipMode);
	*ret = m_clipMode;
	return S_OK;
}
STDMETHODIMP Site::put_AutoSetExtent(VARIANT_BOOL val) {
    if(m_pLayer && m_pLayer->m_amTransformed && val)
        ERR(E_INVALIDARG);
    m_autoSetExtent = val!=0;
	return S_OK;
}
STDMETHODIMP Site::get_AutoSetExtent(VARIANT_BOOL *ret) {
    *ret = m_autoSetExtent*VARIANT_TRUE;
	return S_OK;
}

/*
// I don't really think this is necessary.  use a SiteOrdering for relative placement!
STDMETHODIMP Site::put_Z(DWORD val) {
	METHTRACE(Site,put_Z);
	if(m_state<MCCS_HIDDEN)
		ERR(E_PENDING);
	int tasksCompleted = 0;
	sitelist::iterator next = m_pLayer->m_sites.end(), prev = m_pLayer->m_sites.end();
	for(sitelist::iterator i = m_pLayer->m_sites.begin();i != m_pLayer->m_sites.end(); i++) {
		if(!val--) {
			next = i;
			if(tasksCompleted++)
				break;
		}
		if(*i==this) {
			prev = i;
			if(tasksCompleted++)
				break;
		}
	}
	m_pLayer->m_sites.insert(next,(site*)this);
	if(prev==m_pLayer->m_sites.end())
		ERR(E_UNEXPECTED);
	m_pLayer->m_sites.erase(prev);
	return S_OK;
}
*/
STDMETHODIMP Site::get_Rect(LPRECT lprc) {
	IMETHTRACE(Site,IMCCSite,get_Rect);
	CHECK_DEL;
	if(!m_bRectValid)
		ERR(E_PENDING);
	if(lprc)
		*lprc = m_windRect;
	return S_OK;
}
STDMETHODIMP Site::put_Rect(LPCRECT lpcr) {
	IMETHTRACE(Site,IMCCSite,put_Rect);
	CHECK_DEL;
	if(*lpcr==m_windRect && m_bRectValid) // the latter for the weird case where new site is allocated over old
		return S_FALSE;
	if(!lpcr)
		ERR(E_POINTER);
	if(lpcr->left>lpcr->right||lpcr->top>lpcr->bottom)
		ERR(E_INVALIDARG);
	moveEvent(lpcr);
	bool visibleButNotWindowed = m_state==MCCS_SHOWN||m_state>MCCS_OPEN&&m_pWindowless;
	if(visibleButNotWindowed)
		m_cont->InvalidateSite((site*)this);

	bool moved = m_windRect != *lpcr,
		sized = wid(m_windRect)!=wid(*lpcr) || hei(m_windRect)!=hei(*lpcr) || !m_bRectValid;
	m_windRect = *lpcr;
    if(!m_bRectValid)
        m_virtRect = rectl(0,0,0,0);
	m_bRectValid  = TRUE;
	// if control is in-place active, tell it
	if(moved && m_state>MCCS_OPEN) {
		CComQIPtr<IOleInPlaceObject> ipo = m_pControl;
		if(ipo) {
			RECT rcClip = clipRect();
			// servers (e.g. Word) don't seem to like being told something different
			// from what they asked in OnPosRectChange but then they move anyway?
			LOOK(ipo->SetObjectRects(&m_windRect,&rcClip));
		}
	}
    if(m_autoSetExtent&&m_state>MCCS_HIDDEN)
        RUN(UpdateExtent(VARIANT_TRUE));
	if(visibleButNotWindowed)
		m_cont->InvalidateSite((site*)this);
	if(moved && m_state>MCCS_HIDDEN)
		RUN(m_cont->dirty());
	return S_OK;
}
STDMETHODIMP Site::get_VirtRect(PRECTL ret) {
	IMETHTRACE(Site,IMCCSite,get_VirtRect);
	CHECK_DEL;
	if(!m_bRectValid || !m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	*ret = m_virtRect;
	return S_OK;
}
STDMETHODIMP Site::put_VirtRect(LPCRECTL val) {
	IMETHTRACE(Site,IMCCSite,put_VirtRect);
	CHECK_DEL;
	if(!val)
		ERR(E_POINTER);
	if(!m_pLayer)
		ERR(E_PENDING);
	if(!m_pLayer->m_amTransformed)
		ERR(E_FAIL);
	if(!m_cont->m_transform)
		ERR(E_UNEXPECTED);
	if(val->left>val->right||val->bottom>val->top)
		ERR(E_INVALIDARG);
    if(m_bRectValid && *val==m_virtRect)
        return S_FALSE;
	if(m_state > MCCS_HIDDEN && m_cont->m_state > MCCS_DELETED) {
		// less erratic to add new, then remove old
		RUN(m_cont->m_transform->AddRect(val));
		if(m_bRectValid)
			RUN(m_cont->m_transform->RemoveRect(&m_virtRect));
	}
	bool sized = m_bRectValid ? wid(m_virtRect)!=wid(*val) || hei(m_virtRect)!= hei(*val)
                    : true;
	RECT rc;
	RUN(m_cont->m_transform->C2WR(val,&rc));
	m_virtRect = *val;
	m_bRectValid = true;
	RUN(put_Rect(&rc));
	// set new extent if any
	if(sized)
		RUN(UpdateExtent(false));
	return S_OK;
}
STDMETHODIMP Site::RegisterCommand(int cmd) {
	IMETHTRACE(Site,IMCCSite,RegisterCommand);
	CHECK_DEL;
	Container::menumap::iterator im = m_cont->m_menumap.find(cmd);
	if(im!=m_cont->m_menumap.end())
		return (im->second==(site*)this) ? S_OK : E_FAIL;
	m_cont->m_menumap[cmd] = (site*)this;
	return S_OK;
}
STDMETHODIMP Site::ForgetCommand(int cmd) {
	IMETHTRACE(Site,IMCCSite,ForgetCommand);
	CHECK_DEL;
	Container::menumap::iterator im = m_cont->m_menumap.find(cmd);
	if(im==m_cont->m_menumap.end())
		ERR(E_FAIL);
	if(im->second!=(site*)this)
		ERR(E_FAIL);
	m_cont->m_menumap.erase(im);
	return S_OK;
}
STDMETHODIMP Site::PutBefore(IMCCSite *other) {
	IMETHTRACE(Site,IMCCSite,PutBefore);
	CHECK_DEL;
	HRESULT hr;
	DWORD ct = m_dwRef;
	if(FAILED(hr = m_cont->before((site*)this,(site *)other)))
		return hr;
	RUN(activeZ());
	RUN(fireEvent(GetUnknown(),fireVoid<IMCCSiteEvents>(&IMCCSiteEvents::ZChanged)));
	if(m_state>MCCS_HIDDEN)
		RUN(m_cont->dirty());
	_ASSERTE(ct==m_dwRef);
	return S_OK;
}
STDMETHODIMP Site::PutBehind(IMCCSite *other) {
	IMETHTRACE(Site,IMCCSite,PutBehind);
	CHECK_DEL;
	HRESULT hr;
	DWORD ct = m_dwRef;
	if(FAILED(hr = m_cont->after((site*)this,(site *)other)))
		return hr;
	RUN(activeZ());
	RUN(fireEvent(GetUnknown(),fireVoid<IMCCSiteEvents>(&IMCCSiteEvents::ZChanged)));
	if(m_state>MCCS_HIDDEN)
		RUN(m_cont->dirty());
	_ASSERTE(ct==m_dwRef);
	return S_OK;
}
// returns the window that's the immediate child of Montage (may be hatching
// rather than IOleWindow window)
HWND Site::window() {
	METHTRACE(Site,window);
	// used to have m_state<MCCS_ACTIVE check but that's redundant and
	// this is in fact a good test whether it's inplace active or open elsewhere
	// (changestate calls activeZ which calls this)
	if(m_pWindowless)
		return 0;
	CComQIPtr<IOleInPlaceObject> ipo = m_pControl;
	if(!ipo)
		return 0;
	HWND ret;
	if(FAILED(ipo->GetWindow(&ret))||!ret)
		return 0;
	// go up tree to find the child of Montage.  (Hatch windows sit between
	// container and child)
	HWND parent;
	while((parent = GetParent(ret))!=m_cont->m_hWnd) {
		if(!parent)
			return 0;
		ret = parent;
	}
	return ret;
}
// called after a control has gone active to make sure it's at the right Z
HRESULT Site::activeZ() {
	METHTRACE(Site,activeZ);
	CHECK_DEL;
	HWND mywnd = window();
	if(!mywnd)
		return S_FALSE;
	sitelist::iterator is = find(m_pLayer->m_sites.begin(),m_pLayer->m_sites.end(),(site*)this);
	if(is==m_pLayer->m_sites.end())
		ERR(E_FAIL);
	// now, it's a very bad idea for any windowless or inactive objects to
	// be in front of an active windowed object, but that's not our business:
	// we ignore them and try to find *some* windowed object in front.
	HWND behind = 0;
	layerlist::iterator il = find(m_cont->m_layers.begin(),m_cont->m_layers.end(),m_pLayer);
	if(il==m_cont->m_layers.end())
		ERR(E_FAIL);
	do {
		while(is!=(*il)->m_sites.begin() && !behind) {
			site *s = *--is;
			// we shouldn't consider controls that are "activated into" particular
			// windows (eg floaters), because that's not the same z order.
			if(!s->m_usewnd && s->m_state>MCCS_SHOWN)
				behind = s->window();
		}
		if(il==m_cont->m_layers.begin())
			break;
		il--;
		is = (*il)->m_sites.end();
	} while(!behind);
	if(!behind)
		behind = HWND_TOP; // still zero, but a meaningful zero
	SetWindowPos(mywnd,behind,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
	return S_OK;
}

HRESULT Site::releaseSinks() {
	METHTRACE(Site,releaseSinks);
	CComPtr<IEnumConnectionPoints> ecp;
	RUN(CComQIPtr<IConnectionPointContainer>(m_connectionpoints)->EnumConnectionPoints(&ecp));
	for(connectionpoint_iterator i(ecp);i!=connectionpoint_iterator();++i) {
		CComPtr<IEnumConnections> ec;
		RUN((*i)->EnumConnections(&ec));
		for(connection_iterator j(ec);j!=connection_iterator();++j)
			RUN((*i)->Unadvise(j->dwCookie));
	}
	return S_OK;
}
// main state-changing method.  It "does the best it can,"
// won't go any further than container's state.  Then when cc
// changes state, it will notify (through ccstatechanged) and
// this will get called to reflect.
HRESULT Site::changestate(long dest) {
	METHTRACE(Site,changestate);
	HRESULT hr;
	bool uiFromShown = false;
	// IOleObject is optional: we notify the control if it has the iface.
	CComQIPtr<IOleObject> pOle = m_pControl;
	if(m_state==dest)
		return S_OK;
	if(m_state<dest)
		switch(m_state) {
		case MCCS_DELETED:
			// you can't undelete (until there are container-site interfaces?)
			ERR(MONTAGE_E_DELETED);
		case MCCS_HIDDEN:
			// show
			if(!m_bRectValid) {
				HRERR(E_PENDING);
				break;
			}
			if(!m_pControl) {
				HRERR(E_PENDING);
				break;
			}
			if(m_pLayer==m_cont->m_nonlayer) {
				HRERR(E_PENDING);
				break;
			}
			RUN(m_cont->show((site*)this));
            if(m_autoSetExtent)
                UpdateExtent(VARIANT_TRUE); // this will fail if rect=={0,0,0,0} and so wait until next TransformChanged
			RUN(m_pControl->SetAdvise(DVASPECT_CONTENT,0, this));
			// do this just to hear of extent change
			if(CComQIPtr<IDataObject> data = m_pControl) {
				FormatEtc fetc(CF_METAFILEPICT);
				RUN(data->DAdvise(&fetc,ADVF_NODATA,CComQIPtr<IAdviseSink>(GetUnknown()),&m_peanutButter));
			}
			RUN(processMnemonics(true));
			m_state = MCCS_SHOWN;
			if(dest==MCCS_SHOWN)
				return S_OK;
		case MCCS_SHOWN:
			// branch in the activation level: open in separate window or activate in place?
			if(dest==MCCS_OPEN)
				return LOOK(SayVerb(OLEIVERB_OPEN,0));
			LOOK(hr = SayVerb(OLEIVERB_INPLACEACTIVATE,0));
			if(SUCCEEDED(hr)) {
				if(!m_pWindowless) {
					// It's very hard to tell what actually happens when you call DoVerb, DoVerb is
					// kind of a request/recommendation...
					// We need to know if the object opened in a separate window instead of activating in place.
					// (if we have m_pWindowless it's definitely active inplace.)
					// OnShowWindow does state transition between shown and open, so we just want to quit here if it's not inplace.
					// the object may fail on GetWindow OR return 0 OR even return the separate app window.
					// activeZ checks for all these things.as it sets the z-order of the control's window
					RUN(hr = activeZ());
					if(hr==S_FALSE) { // window was not a child of Montage: must be Open somewhere else.
						hr = S_OK;
						break;
					}
				}
			}
			else {
				switch(dest) {
				case MCCS_ACTIVE:
				case MCCS_COUIACTIVE:
					// OLEIVERB_INPLACEACTIVATE must work for active and coUI active objects
					break;
				case MCCS_UIACTIVE:
					uiFromShown = true;
					goto uwey; // go ahead and try other verbs; make sure you check Z once there!
				}
			}
			m_state = MCCS_ACTIVE;
			if(dest==MCCS_ACTIVE)
				return S_OK;
		case MCCS_ACTIVE:
			// (co)ui activate
			// always try coUI first
			LOOK(hr = m_cont->couiactivate((site*)this));
			if(SUCCEEDED(hr)) {
				m_intent = m_state = MCCS_COUIACTIVE;
				return hr;
			}
			// if coUI was explicitly requested, succeed with a warning
			if(dest == MCCS_COUIACTIVE) {
				hr = MONTAGE_S_NOCOUI;
				break;
			}
			// otherwise try standard UI activation
		uwey: // OLE UI activate only (if OLEIVERB_INPLACEACTIVATE failed)
			if(FAILED(hr = SayVerb(OLEIVERB_UIACTIVATE,0)))
				if(FAILED(LOOK(hr = SayVerb(OLEIVERB_PRIMARY,0)))) // "okay, if you won't UI, just do anything!"
					break;
			if(uiFromShown)
				RUN(activeZ());
			m_state = MCCS_UIACTIVE;
			return S_OK;
		case MCCS_COUIACTIVE: // coUI-active is not lower than UI-active
			m_intent = MCCS_COUIACTIVE;
			return S_OK;
		case MCCS_OPEN: // open is active enough.
			return S_FALSE;
		default:
			HRERR(E_UNEXPECTED); // can't go higher than uwey!
		}
	else { // curr >=dest
		CComQIPtr<IOleInPlaceObject> ipo = m_pControl;
		switch(m_state) { // downgrade
		case MCCS_OPEN:
			// objects open in a separate window must be closed manually
			return S_FALSE;
		case MCCS_UIACTIVE:
			// ui deactivate
			if(ipo)
 RUN(ipo->UIDeactivate());
			m_state = MCCS_ACTIVE;
			if(dest==MCCS_ACTIVE)
				return S_OK;
			goto a;
		case MCCS_COUIACTIVE:
			RUN(m_cont->couideactivate((site*)this));
			m_state = MCCS_ACTIVE;
			if(dest==MCCS_ACTIVE)
				return S_OK;
		a:
		case MCCS_ACTIVE:
			// deactivate
            if(m_cont->m_pFocusObject==(site*)this) {
				m_cont->m_pFocusObject = 0;
                m_cont->changeFocus();
			}
			if(ipo)
				RUN(ipo->InPlaceDeactivate());
			m_state = MCCS_SHOWN;
			if(dest==MCCS_SHOWN)
				return S_OK;
		case MCCS_SHOWN:
			// hide
			RUN(m_cont->hide((site*)this));
			RUN(processMnemonics(false));
			RUN(m_pControl->SetAdvise(DVASPECT_CONTENT,0,0));
			if(m_peanutButter) {
				RUN(CComQIPtr<IDataObject>(m_pControl)->DUnadvise(m_peanutButter));
				m_peanutButter = 0;
			}
			m_state = MCCS_HIDDEN;
			if(dest==MCCS_HIDDEN)
				return S_OK;
		case MCCS_HIDDEN:
			// delete
			// save object "yourself," explicitly, if that's what's wanted.
			if(pOle)
				RUN(pOle->Close(OLECLOSE_NOSAVE));
			// release control & have control release me.
			RUN(put_Control(0));
			// remove from layer (derefs; may destroy)
			RUN(m_cont->remove((site*)this));
			// detach from container
			m_cont = 0;
			m_state = MCCS_DELETED;
			return S_OK;
		default:
			HRERR(E_UNEXPECTED); // can't go lower than MCCS_DELETED
		}
	}
	// some sort of error occurred, so set the intent
	// to the current state so we don't try it again.
	m_intent = m_state;
	return hr;
}
// does not affect intent
HRESULT Site::approachstate(long val) {
	METHTRACE(Site,approachstate);
	CHECK_DEL;
	HRESULT hr;

	// figure out how far we can actually follow this request
	long next = min(m_cont->m_state,val);

	// then go.
	if(m_state!=next) {
		GetUnknown()->AddRef(); // protect self for sinks
		DWORD last = m_state;
		Container *c = m_cont;
		hr = changestate(next);
		if(last!=m_state) {
			stateEvent(last,m_state);  // doesn't necessarily get to next
			if(m_state==MCCS_DELETED)
				releaseSinks();
		}
        // if hiding or showing, signal view/data change
		if((last > MCCS_HIDDEN) ^ (m_state>MCCS_HIDDEN))
			RUN(c->dirty());
		GetUnknown()->Release(); // matches AddRef above; perhaps kills.
		return hr;
	}
	else return S_FALSE;
}
STDMETHODIMP Site::get_Intent(long *ret) {
	IMETHTRACE(Site,IMCCSite,get_Intent);
	*ret = m_intent;
	return S_OK;
}
STDMETHODIMP Site::put_Intent(long val) {
	IMETHTRACE(Site,IMCCSite,put_Intent);
	CHECK_DEL;

	// "shown" and "ui active" are not options if we don't own the window
	// "open" implies shown as well.
	if(!m_cont->m_ownwindow || m_usewnd)
		if(val==MCCS_SHOWN || val==MCCS_OPEN || val==MCCS_UIACTIVE) // is the latter necessary?
			val = MCCS_ACTIVE;

	// set the intent
	m_intent = val;

	// fire SiteDeleted event (causing this to be removed from all SiteLists)
	if(val==MCCS_DELETED && m_mkPersistor && m_cont->m_group) {
		RUN(fireEvent(m_cont->GetUnknown(),
			fireOneParam<IMontageContainerEvents,IMCCSite*>(&IMontageContainerEvents::SiteDeleted,this)));
		// hack: release mk so this can't get hit again
		m_mkPersistor.Release();
	}
	// and actually change state
	return approachstate(val);
}
HRESULT Site::SayVerb(LONG iVerb, LPMSG lpmsg) {
	METHTRACE(Site,SayVerb);
    /*
	if(lpmsg)
		ATLTRACE2(_T("{0x%x, 0x%x, %d, %d}\n"), lpmsg->hwnd, lpmsg->message, lpmsg->wParam, lpmsg->lParam);
    */
	CHECK_DEL;
	if(!m_pControl)
		ERR(E_PENDING);
	HWND hWnd;
	GetWindow(&hWnd);
	CComQIPtr<IOleObject> pOle = m_pControl;
	if(pOle)
		return pOle->DoVerb(iVerb,lpmsg,this,0,hWnd,&m_windRect);
	else
		return S_OK;
}

STDMETHODIMP Site::UseWindow(HWND hWnd) {
	IMETHTRACE(Site,IMCCSite,UseWindow);
	CHECK_DEL;
	if(!hWnd) {
		m_usewnd = 0;
		return put_Intent(MCCS_HIDDEN);
	}
	if(m_intent>MCCS_HIDDEN)
		RUN(approachstate(MCCS_HIDDEN));
	m_usewnd = hWnd;
	return approachstate(m_intent);
}
STDMETHODIMP Site::UpdateExtent(VARIANT_BOOL fromRect) {
    METHTRACE(Site,UpdateExtent);
    if(fromRect) {
        if(!m_cont->m_transform)
            ERR(E_PENDING);
        // not compatible with automatic transform
        if(m_pLayer&&m_pLayer->m_amTransformed)
            ERR(E_FAIL);
        RUNRET(m_cont->m_transform->W2CR(&m_windRect,&m_virtRect));
    }

    SIZEL s = siz(m_virtRect);
	HRESULT hr;
	if(CComQIPtr<IOleObject> oo = m_pControl) {
		m_amSettingExtent = true;
		// WordPerfect & Visio don't like this (sometimes?)
		hr = oo->SetExtent(DVASPECT_CONTENT,&s);
		m_amSettingExtent = false;
	}
	return S_OK;
}
STDMETHODIMP Site::SetTimer(UINT id,UINT elapse) {
    m_cont->m_timers[id] = static_cast<site*>(this);
    ::SetTimer(m_cont->m_hWnd,id,elapse,0);
	return S_OK;
}
STDMETHODIMP Site::KillTimer(UINT id) {
    if(m_cont->m_timers[id] != this)
        ERR(E_INVALIDARG);
    if(!::KillTimer(m_cont->m_hWnd,id))
        ERR(E_FAIL);
    m_cont->m_timers.erase(id);
	return S_OK;
}
