/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ModeDelegate.h" // includes "ModeSwitcher.h"
#include "releaseDUA.h"

inline HRESULT fireActivate(VARIANT_BOOL once, IUnknown *del) {
	return fireEvent(del,fireOneParam<IMCCMode,VARIANT_BOOL>(&IMCCMode::Activate,once));
}
inline HRESULT fireDeactivate(IUnknown *del) {
	return fireEvent(del,fireVoid<IMCCMode>(&IMCCMode::Deactivate));
}
ModeDelegate::ModeDelegate() : m_order(0),m_shown(false) {
	CONSTRUCTORTRACE(ModeDelegate);
}
ModeDelegate::~ModeDelegate() {
	DESTRUCTORTRACE(ModeDelegate);
	if(m_switcher)
		RUN(Delete());
	releaseDUA(m_vec);
}
HRESULT ModeDelegate::releaseSinks() {
	return S_OK;
}
HRESULT ModeDelegate::insert() {
	METHTRACE(ModeDelegate,insert);
	pair<int const,ModeDelegate*> inf(m_order,this);
	m_switcher->m_delegates.insert(inf);
	return S_OK;
}
HRESULT ModeDelegate::remove() {
	METHTRACE(ModeDelegate,remove);
	delOrder::iterator i = m_switcher->m_delegates.find(m_order);
	if(i==m_switcher->m_delegates.end())
		ERR(E_FAIL);
	delOrder::iterator fin = m_switcher->m_delegates.upper_bound(m_order);
	for(;i!=fin;i++)
		if((*i).second==this) {
			m_switcher->m_delegates.erase(i);
			return S_OK;
		}
	ERR(E_UNEXPECTED);
}
// DIMCCModeDelegate
STDMETHODIMP ModeDelegate::get_Name(BSTR *ret) {
	METHTRACE(ModeDelegate,get_Name);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	*ret = m_name.Copy();
	return S_OK;
}
STDMETHODIMP ModeDelegate::put_Name(BSTR val) {
	METHTRACE(ModeDelegate,put_Name);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	m_name = val;
	return S_OK;
}
STDMETHODIMP ModeDelegate::get_Order(long *ret) {
	METHTRACE(ModeDelegate,get_Order);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	*ret = m_order;
	return S_OK;
}
STDMETHODIMP ModeDelegate::put_Order(long val) {
	METHTRACE(ModeDelegate,put_Order);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	RUN(remove());
	m_order = val;
	RUN(insert());
	if(m_shown)
		m_switcher->reorderEvent();
	return S_OK;
}
STDMETHODIMP ModeDelegate::get_State(long *ret) {
	METHTRACE(ModeDelegate,get_State);
	if(!m_switcher)
		*ret = MODE_DELETED;
	else if(!m_shown)
		*ret = MODE_HIDDEN;
	else {
		if(this==m_switcher->m_active)
			*ret = m_switcher->m_once?MODE_ACTIVEONCE:MODE_ACTIVE;
		else
			*ret = MODE_SHOWN;
	}
	return S_OK;
}
STDMETHODIMP ModeDelegate::Show() {
	METHTRACE(ModeDelegate,Insert);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	m_shown = true;
	m_switcher->reorderEvent();
	// this is here instead of in insert() because it must occur after the reorder event.
	// it's okay to put it here because put_Order cannot cause a change in activation.
	if(!m_switcher->m_active) // make sure there is an active mode
		RUN(Activate(false));
	return S_OK;
}
STDMETHODIMP ModeDelegate::Hide() {
	METHTRACE(ModeDelegate,Remove);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	if(this==m_switcher->m_once)
		RUN(Done());
	// try to keep something active
	if(this==m_switcher->m_active) {
		// look for something else shown
		for(ModeSwitcher::delOrder::iterator i = m_switcher->m_delegates.begin();
			i!=m_switcher->m_delegates.end();++i)
				if(i->second->m_shown) {
					RUN(i->second->Activate(false));
					break;
				}
		if(i==m_switcher->m_delegates.end()) {
			m_switcher->m_active = 0;
			RUN(fireDeactivate(GetUnknown()));
			m_switcher->activateEvent(0,false);
		}
	}
	m_shown = false;
	m_switcher->reorderEvent();
	return S_OK;
}
STDMETHODIMP ModeDelegate::Delete() {
    METHTRACE(ModeDelegate,Delete);
	if(!m_switcher) ERR(S_FALSE);
	RUN(remove());
	RUN(releaseSinks());
	m_switcher = 0;
	((DIMCCModeDelegate*)this)->Release();
	return S_OK;
}
STDMETHODIMP ModeDelegate::Activate(VARIANT_BOOL once) {
	METHTRACE(ModeDelegate,Activate);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	if(!m_shown)
		RUN(Show());
	// tell whatever is active that it isn't
	if(m_switcher->m_once) {
		RUN(fireDeactivate(m_switcher->m_once));
		m_switcher->m_once = 0;
	}
	else if(m_switcher->m_active)
		RUN(fireDeactivate(m_switcher->m_active));
	if(once)
		m_switcher->m_once = this;
	else
		m_switcher->m_active = this;
	RUN(fireActivate(once,GetUnknown()));
	m_switcher->activateEvent(this,once);
	return S_OK;
}
STDMETHODIMP ModeDelegate::Done() {
	METHTRACE(ModeDelegate,Done);
	if(!m_switcher) ERR(MONTAGE_E_DELETED);
	if(m_switcher->m_once!=this)
		ERR(E_UNEXPECTED);
	m_switcher->m_once = 0;
	RUN(fireDeactivate(GetUnknown()));
	RUN(fireActivate(false,m_switcher->m_active));
	m_switcher->activateEvent(m_switcher->m_active,false);
	return S_OK;
}
// IPersistStream
STDMETHODIMP ModeDelegate::GetClassID(CLSID *pClassID) {
	METHTRACE(ModeDelegate,GetClassID);
	*pClassID = CLSID_NULL; // not possible to create w/ CoCreate
	return S_OK;
}
STDMETHODIMP ModeDelegate::IsDirty(void) {
	METHTRACE(ModeDelegate,IsDirty);
	return S_FALSE;
}
STDMETHODIMP ModeDelegate::Load(IStream *pStm) {
	METHTRACE(ModeDelegate,Load);
	RUN(m_name.ReadFromStream(pStm));
	int order;
	RUN(pStm->Read(&order,sizeof(int),0));
	RUN(put_Order(order));
	long state;
	RUN(pStm->Read(&state,sizeof(long),0));
	switch(state) {
	case MODE_SHOWN:
		RUN(Show());
	case MODE_HIDDEN:
		return S_OK;
	case MODE_ACTIVE:
		RUN(Activate(false));
		return S_OK;
	default: // deleted? activeonce?
		ERR(E_UNEXPECTED);
	}
}
STDMETHODIMP ModeDelegate::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(ModeDelegate,Save);
	RUN(m_name.WriteToStream(pStm));
	RUN(pStm->Write(&m_order,sizeof(int),0));
	long state;
	RUN(get_State(&state));
	_ASSERTE(state!=MODE_DELETED);
	if(state==MODE_ACTIVEONCE)
		state = MODE_SHOWN; // activeonce not preserved
	RUN(pStm->Write(&state,sizeof(DWORD),0));
	return S_OK;
}
STDMETHODIMP ModeDelegate::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(ModeDelegate,GetSizeMax);
	pcbSize->QuadPart = m_name.Length()*sizeof(OLECHAR) + 2*sizeof(int);
	return S_OK;
}

