/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ModeSwitchCtl.cpp : Implementation of ModeSwitcherCtl
#include "stdafx.h"
#include "ModeSwitchCtl.h"

/////////////////////////////////////////////////////////////////////////////
// ModeSwitcherCtl
ModeSwitcherCtl::ModeSwitcherCtl() : m_active(-1), m_oatmeal(0),m_bigFont(0),m_smallFont(0)
{
	CONSTRUCTORTRACE(ModeSwitcherCtl);
	m_selFill = CreateSolidBrush(RGB(0,150,255));
	m_bWindowOnly = true;
}
ModeSwitcherCtl::~ModeSwitcherCtl() {
	DESTRUCTORTRACE(ModeSwitcherCtl);
	setNamesFree();
	if(m_selFill)
		DeleteObject(m_selFill);
	if(m_bigFont)
		DeleteObject(m_bigFont);
	if(m_smallFont)
		DeleteObject(m_smallFont);
}
void ModeSwitcherCtl::setNamesFree() {
	METHTRACE(ModeSwitcherCtl,setNamesFree);
	for(vbstr::iterator i = m_modenames.begin();i!=m_modenames.end();i++)
		SysFreeString(*i);
	m_modenames.clear();
}
STDMETHODIMP ModeSwitcherCtl::SetClientSite(IOleClientSite *val) {
	METHTRACE(ModeSwitcherCtl,SetClientSite);
	RUN(IOleObjectImpl<ModeSwitcherCtl>::SetClientSite(val));
	if(!val) {
		if(m_oatmeal)
			RUN(AtlUnadvise(m_switcher,IID_IMCCModeSwitcherEvents,m_oatmeal));
		m_oatmeal = 0;
		return S_OK;
	}
	return S_OK;
}

HRESULT ModeSwitcherCtl::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	METHTRACE(ModeSwitcherCtl,OnDrawAdvanced);
	USES_CONVERSION;
	RECT& rc = *(RECT*)di.prcBounds;
	HDC hdc = di.hdcDraw;

	HFONT hOldFont = (HFONT)SelectObject(hdc,m_smallFont);
	int nmodes = m_modenames.size();
	SetBkMode(hdc,TRANSPARENT);
	for(int i=0;i<nmodes;i++) {
		RECT rc2 = {((nmodes-i)*rc.left + i*rc.right)/nmodes,rc.top,(((nmodes-1)-i)*rc.left + (i+1)*rc.right)/nmodes,rc.bottom};
		if(i==m_active) {
			FillRect(hdc,&rc2,m_selFill);
			SelectObject(hdc,m_bigFont);
		}
		DrawText(hdc, OLE2T(m_modenames[i]), -1, &rc2, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
		if(i==m_active)
			SelectObject(hdc,m_smallFont);
	}
	SelectObject(hdc,hOldFont);
/*
	SetBkMode(hdc,TRANSPARENT);
	HBRUSH hbrNormal = CreateSolidBrush(RGB(0,0,192));
	HBRUSH hbrSelected = CreateSolidBrush(RGB(0,255,0));
	int nmodes = m_modenames.size();
	for(int i = 0; i<nmodes;i++) {
		RECT rc2 = {i*wid(rc) / nmodes,rc.top,
				(i+1)*wid(rc) / nmodes,rc.bottom};
		FillRect(hdc,&rc2,(i==m_active)?hbrSelected:hbrNormal);
		UINT len = SysStringLen(m_modenames[i]);
		LPTSTR s = OLE2T(m_modenames[i]);
		SIZE size;
		GetTextExtentPoint32(hdc,s,len,&size);
		POINT pt = ctr(rc2);
		pt.x -= size.cx/2;
		pt.y -= size.cy/2;
		ExtTextOut(hdc,pt.x,pt.y,ETO_CLIPPED,&rc2,s,len,0);
	}
	DeleteObject(hbrNormal);
	DeleteObject(hbrSelected);
	*/
	return S_OK;
}
STDMETHODIMP ModeSwitcherCtl::SetObjectRects(LPCRECT lpcr,LPCRECT lpcrClip) {
	METHTRACE(ModeSwitcherCtl,SetObjectRects);
	USES_CONVERSION;
	HRESULT hr;
	hr = IOleInPlaceObjectWindowlessImpl<ModeSwitcherCtl>::SetObjectRects(lpcr,lpcrClip);
	if(m_bigFont)
		DeleteObject(m_bigFont);
	if(m_smallFont)
		DeleteObject(m_smallFont);
	int wid = m_rcPos.right-m_rcPos.left,
		hei = m_rcPos.bottom - m_rcPos.top;
	HDC hScreenDC = ::GetWindowDC(0);
	m_bigFont = CreateFont(hei-2,//-MulDiv(50, GetDeviceCaps(hScreenDC, LOGPIXELSY), 72),
		0,
		0,0,
		FW_ULTRABOLD,
		false,
		false,
		false,
		ANSI_CHARSET,
		OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		PROOF_QUALITY,
		DEFAULT_PITCH,
		"Arial");
	HFONT hfOld = (HFONT)SelectObject(hScreenDC,m_bigFont);
	int maxX = 0;
	for(unsigned i = 0;i<m_modenames.size();++i) {
		SIZE size;
		if(GetTextExtentPoint32(hScreenDC,OLE2T(m_modenames[i]),SysStringLen(m_modenames[i]),&size))
			if(size.cx+4>maxX)
				maxX = size.cx+4;
	}
	if(maxX && maxX > wid/int(m_modenames.size())) { // adjust
		DeleteObject(m_bigFont);
		hei = hei*wid/(m_modenames.size()*maxX);
		m_bigFont = CreateFont(hei-2,
			0,
			0,0,
			FW_ULTRABOLD,
			false,
			false,
			false,
			ANSI_CHARSET,
			OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS,
			PROOF_QUALITY,
			DEFAULT_PITCH,
			"Arial");
	}
	m_smallFont = CreateFont(hei*2/3,//-MulDiv(30, GetDeviceCaps(hScreenDC, LOGPIXELSY), 72),
		0,
		0,0,
		FW_THIN,
		false,
		false,
		false,
		ANSI_CHARSET,
		OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH,
		"Arial");
	::ReleaseDC(0,hScreenDC);
	return hr;
}
void ModeSwitcherCtl::draw(HDC hdc, const RECT &rc) {
	METHTRACE(ModeSwitcherCtl,draw);
	USES_CONVERSION;
	int nmodes = m_modenames.size();
	for(int i=0;i<nmodes;i++) {
		RECT rc2 = {((nmodes-i)*rc.left + i*rc.right)/nmodes,rc.top,(((nmodes-1)-i)*rc.left + (i+1)*rc.right)/nmodes,rc.bottom};
		DrawText(hdc, OLE2T(m_modenames[i]), -1, &rc2, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
	}
}
LRESULT ModeSwitcherCtl::LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ModeSwitcherCtl,LButtonUp);
	int nmodes = m_modenames.size();
	if(!nmodes)
		return 0;
	int sel = (MAKEPOINTS(lParam).x*nmodes)/(m_rcPos.right-m_rcPos.left);
	if(sel!=m_active) {
		m_dcAnim = GetDC();
		m_dcNoSel = CreateCompatibleDC(m_dcAnim);
		m_dcAllSel = CreateCompatibleDC(m_dcAnim);

		int wid = m_rcPos.right-m_rcPos.left,
			hei = m_rcPos.bottom - m_rcPos.top;
		m_bitNoSel = CreateCompatibleBitmap(m_dcAnim,wid,hei);
		m_bitAllSel = CreateCompatibleBitmap(m_dcAnim,wid,hei);
		SelectObject(m_dcNoSel,m_bitNoSel);
		SelectObject(m_dcAllSel,m_bitAllSel);
		RECT rc = {0,0,wid,hei};
		// draw allsel
		FillRect(m_dcAllSel,&rc,m_selFill);
		SetBkMode(m_dcAllSel,TRANSPARENT);
		HFONT hfOld = (HFONT)SelectObject(m_dcAllSel,m_bigFont);
		draw(m_dcAllSel,rc);
		SelectObject(m_dcAllSel,hfOld);
		// draw nosel
		HBRUSH hBackBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
		FillRect(m_dcNoSel,&rc,hBackBrush);
		DeleteObject(hBackBrush);
		SetBkMode(m_dcNoSel,TRANSPARENT);
		hfOld = (HFONT)SelectObject(m_dcNoSel,m_smallFont);
		draw(m_dcNoSel,rc);
		SelectObject(m_dcNoSel,hfOld);
		SetTimer(1,0); // as fast as possible, which seems to be about 10 frames/sec

		m_leftAnim = (m_active*wid)/nmodes;
		m_active = sel;
	}
	BSTR thaName = m_modenames[sel];
	UINT n = SysStringLen(thaName);

	CComPtr<IEnumMCCModeDelegates> emd;
	RUN(m_switcher->EnumDelegates(&emd));
	CComPtr<DIMCCModeDelegate> md;
	while(emd->Next(1,&md,0)==S_OK) {
		CComBSTR bstr;
		RUN(md->get_Name(&bstr));
		if(!wcsncmp(thaName,bstr,n)) {
			RUN(md->Activate(wParam&MK_CONTROL));
			return 0;
		}
		md.Release();
	}
	return 0;
}
LRESULT ModeSwitcherCtl::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(ModeSwitcherCtl,OnTimer);
	int nmodes = m_modenames.size();
	int wid = m_rcPos.right-m_rcPos.left,
		hei = m_rcPos.bottom-m_rcPos.top;
	int dest = (m_active*wid)/nmodes,
		incr = wid/9;
	bool done(false);
	if(dest<m_leftAnim) {
		m_leftAnim -= incr;
		if(m_leftAnim<dest) {
			m_leftAnim = dest;
			done = true;
		}
		BitBlt(m_dcAnim,m_leftAnim,0,incr,hei,
			m_dcAllSel,m_leftAnim,0,SRCCOPY);
		BitBlt(m_dcAnim,m_leftAnim+wid/nmodes,0,incr+1,hei,
			m_dcNoSel,m_leftAnim+wid/nmodes,0,SRCCOPY);
	}
	else {
		m_leftAnim += incr;
		if(m_leftAnim>dest) {
			m_leftAnim = dest;
			done = true;
		}
		BitBlt(m_dcAnim,m_leftAnim+wid/nmodes - incr,0,incr,hei,
			m_dcAllSel,m_leftAnim+wid/nmodes - incr,0,SRCCOPY);
		BitBlt(m_dcAnim,m_leftAnim-incr,0,incr,hei,
			m_dcNoSel,m_leftAnim-incr,0,SRCCOPY);
	}
	if(done) {
		ReleaseDC(m_dcAnim);
		DeleteDC(m_dcAllSel);
		DeleteDC(m_dcNoSel);
		DeleteObject(m_bitAllSel);
		DeleteObject(m_bitNoSel);
		KillTimer(1);
	}
	return 0;
}
// INeedOnePointer
STDMETHODIMP ModeSwitcherCtl::TakePointer(IUnknown *modeSwitcher,IMoniker *m) {
	METHTRACE(ModeSwitcherCtl,Initialize);
	if(m_switcher)
		return S_FALSE;
	RUN(modeSwitcher->QueryInterface(riidret(&m_switcher)));
	if(m_oatmeal)
		return S_OK;
	Reorder();
	CComPtr<DIMCCModeDelegate> md;
	if(SUCCEEDED(m_switcher->get_ActiveMode(false,&md))&&md) {
		ModeActivated(md,false);
		md.Release();
		if(SUCCEEDED(m_switcher->get_ActiveMode(-1,&md))&&md)
			ModeActivated(md,true);
	}
	return AtlAdvise(m_switcher,(IMCCModeSwitcherEvents*)this,IID_IMCCModeSwitcherEvents,&m_oatmeal);
}
// IMCCModeSwitcherEvents
STDMETHODIMP ModeSwitcherCtl::Reorder() {
	METHTRACE(ModeSwitcherCtl,Reorder);
	if(m_modenames.size())
		setNamesFree();
	CComPtr<IEnumMCCModeDelegates> emd;
	RUN(m_switcher->EnumDelegates(&emd));
	CComPtr<DIMCCModeDelegate> md;
	while(emd->Next(1,&md,0)==S_OK) {
		BSTR bstr;
		RUN(md->get_Name(&bstr));
		m_modenames.push_back(bstr);
		md.Release();
	}
	// reorder causes position of active to change; re-find
	if(SUCCEEDED(m_switcher->get_ActiveMode(false,&md))&&md) {
		ModeActivated(md,false);
		md.Release();
		if(SUCCEEDED(m_switcher->get_ActiveMode(true,&md))&&md)
			ModeActivated(md,true);
	}
	return S_OK;
}
STDMETHODIMP ModeSwitcherCtl::ModeActivated(DIMCCModeDelegate *mode,BOOL onetime) {
	METHTRACE(ModeSwitcherCtl,ModeActivated);
	if(!mode) {
		m_active = -1;
		return S_OK;
	}
	CComBSTR bstr;
	RUN(mode->get_Name(&bstr));
	UINT n = SysStringLen(bstr);
	// why won't stl please save me
	// find_if(m_modenames.begin(),m_modenames.end(),pointer_to_binary_function(equal<BSTR,BSTR ... ????
	for(vbstr::iterator i = m_modenames.begin();i!=m_modenames.end();i++)
		if(!wcsncmp(*i,bstr,n))
			break;
	_ASSERTE(i!=m_modenames.end());
	int found =  i-m_modenames.begin();
	if(found!=m_active) {
		m_active = found;
		if(m_hWnd)
			InvalidateRect(0,0);
	}
	return S_OK;
}
