/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef __MODESWITCHCTL_H_
#define __MODESWITCHCTL_H_

#include "resource.h"       // main symbols


/////////////////////////////////////////////////////////////////////////////
// ModeSwitcherCtl
class ATL_NO_VTABLE ModeSwitcherCtl :
	public CComObjectRoot,
	public CComCoClass<ModeSwitcherCtl,&CLSID_MCCModeSwitchCtl>,
	public CComControl<ModeSwitcherCtl>,
	public IPersistStreamInitImpl<ModeSwitcherCtl>,
	public IOleControlImpl<ModeSwitcherCtl>,
	public IOleObjectImpl<ModeSwitcherCtl>,
	public IOleInPlaceActiveObjectImpl<ModeSwitcherCtl>,
	public IViewObjectExImpl<ModeSwitcherCtl>,
	public IOleInPlaceObjectWindowlessImpl<ModeSwitcherCtl>,
	public INeedOnePointer,
	public IMCCModeSwitcherEvents,
	public IDispatchImpl<DIMCCModeSwitchCtl>
{
	CComPtr<IMCCModeSwitcher> m_switcher;
	DWORD m_oatmeal; // sink id
	int m_active;
	typedef vector<BSTR> vbstr;
	vbstr m_modenames;
	HFONT m_smallFont,m_bigFont;
	HBRUSH m_selFill;
	HDC m_dcNoSel,m_dcAllSel, m_dcAnim;
	HBITMAP m_bitNoSel,m_bitAllSel;
	int m_leftAnim; // left side of animation

	void setNamesFree();
	void draw(HDC hdc, const RECT &rc);
public:
	ModeSwitcherCtl();
	~ModeSwitcherCtl();

DECLARE_WND_CLASS("Montage ModeSwitcher Control")

REGGY_RID(IDR_MODESWITCHCTL)

BEGIN_COM_MAP(ModeSwitcherCtl)
	COM_INTERFACE_ENTRY(DIMCCModeSwitchCtl)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(INeedOnePointer)
	COM_INTERFACE_ENTRY(IMCCModeSwitcherEvents)
END_COM_MAP()

BEGIN_PROPERTY_MAP(ModeSwitcherCtl)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(ModeSwitcherCtl)
	MESSAGE_HANDLER(WM_LBUTTONUP, LButtonUp)
	MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
	MESSAGE_HANDLER(WM_KILLFOCUS, OnKillFocus)
	MESSAGE_HANDLER(WM_TIMER,OnTimer);
	CHAIN_MSG_MAP(CComControl<ModeSwitcherCtl>)
END_MSG_MAP()

	LRESULT LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

// overriden from CComControl, which does not specify WS_CLIPSIBLINGS
	virtual HWND CreateControlWindow(HWND hWndParent, RECT& rcPos) {
		return Create(hWndParent, rcPos, 0, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS);
	}
// this IOleObjectImpl overload is just a convenient place to put ModeSwitcher sinking/unsinking
	STDMETHODIMP SetClientSite(IOleClientSite *val);
// IViewObjectEx
	STDMETHODIMP GetViewStatus(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}
// IViewObjectImpl
	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);
// IOleInPlaceObject
	STDMETHODIMP SetObjectRects(LPCRECT lpcr,LPCRECT);
// INeedOnePointer
	STDMETHODIMP TakePointer(IUnknown *modeSwitcher,IMoniker *m);
// IMCCModeSwitcherEvents
	STDMETHODIMP Reorder();
	STDMETHODIMP ModeActivated(DIMCCModeDelegate *mode,BOOL onetime);
};

#endif //__MODESWITCHCTL_H_
