/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ModeDelegate.h" // includes ModeSwitcher.h
#include "mapValueIterator.h"
#include "iterEnum.h"
#include "Collection.h"

ModeSwitcher::ModeSwitcher() : m_state(switcher_none), m_active(0),m_once(0) {
	CONSTRUCTORTRACE(ModeSwitcher);
}
ModeSwitcher::~ModeSwitcher() {
	DESTRUCTORTRACE(ModeSwitcher);
	while(m_delegates.size())
		RUN(m_delegates.begin()->second->Delete());
}
void ModeSwitcher::reorderEvent() {
	METHTRACE(ModeSwitcher,reorderEvent);
	for(IUnknown **iu = m_vec.begin();iu!=m_vec.end();iu++)
		if(*iu)
			((IMCCModeSwitcherEvents*)*iu)->Reorder();
}
void ModeSwitcher::activateEvent(DIMCCModeDelegate *mode, BOOL once) {
	METHTRACE(ModeSwitcher,activateEvent);
	for(IUnknown **iu = m_vec.begin();iu!=m_vec.end();iu++)
		if(*iu)
			((IMCCModeSwitcherEvents*)*iu)->ModeActivated(mode,once);
}
// IMCCModeSwitcher
STDMETHODIMP ModeSwitcher::new_Delegate(DIMCCModeDelegate **ret) {
	METHTRACE(ModeSwitcher,new_Delegate);
	typedef CComObject<ModeDelegate> md;
	md *d = new md;
	// this code matched by ModeDelegate::Delete()
	d->m_switcher = this;
	d->GetUnknown()->AddRef(); // like sites, container holds a ref
	(*ret = d)->AddRef();
	return d->insert();
}
STDMETHODIMP ModeSwitcher::get_ActiveMode(VARIANT_BOOL once, DIMCCModeDelegate **ret) {
	METHTRACE(ModeSwitcher,get_ActiveMode);
	if(once) {
		if(*ret = m_once)
			((DIMCCModeDelegate*)m_once)->AddRef();
	}
	else
		if(*ret = m_active)
			((DIMCCModeDelegate*)m_active)->AddRef();
	return S_OK;
}

STDMETHODIMP ModeSwitcher::EnumDelegates(IEnumMCCModeDelegates **ret) {
	METHTRACE(ModeSwitcher,EnumDelegates);
	/*
	best-laid plans...
	// heh heh heh.  now you see why I'm willing to use a completely unsafe struct.
	// I believe this is the longest line I've ever written in C++ (SML, on the other hand...)
	(*ret = new IterEnum<IEnumMCCModeDelegates,&IID_IEnumMCCModeDelegates,IMCCModeDelegate*,
		itSecond<delOrder::iterator,IMCCModeDelegate*>,_CopyInterface<IMCCModeDelegate> >
		(m_delegates.begin(),m_delegates.end(),m_delegates.begin()))->AddRef();
	*/
	vector<DIMCCModeDelegate*> ray;
	for(delOrder::iterator i = m_delegates.begin();i != m_delegates.end(); ++i)
		if(i->second->m_shown)
			ray.push_back(i->second);
	typedef CComObject<CComEnum<IEnumMCCModeDelegates,&IID_IEnumMCCModeDelegates,DIMCCModeDelegate*,_CopyInterface<DIMCCModeDelegate> > > enumDel;
	enumDel *rv = new enumDel;
	rv->Init(&*ray.begin(),&*ray.end(),0,AtlFlagCopy);
	(*ret = rv)->AddRef();
	return S_OK;
}
// DIMCCModeSwitcher
STDMETHODIMP ModeSwitcher::NewDelegate(DIMCCModeDelegate **ret) {
    METHTRACE(ModeSwitcher,NewDelegate);
	return new_Delegate(ret);
}
STDMETHODIMP ModeSwitcher::get_Delegates(IDispatch **ret) {
    METHTRACE(ModeSwitcher,get_Delegates);
	itSecond<delOrder::iterator,ModeDelegate*> begin(m_delegates.begin()),end(m_delegates.end());
	(*ret = new collection(make_colldata(begin,end)))->AddRef();
	return S_OK;
}
STDMETHODIMP ModeSwitcher::get_DelegateClassMoniker(IUnknown **ret) {
    METHTRACE(ModeSwitcher,get_DelegateClassMoniker);
	return CreateClassMoniker(CLSID_MCCModeDelegate,(IMoniker**)ret);
}
HRESULT ModeSwitcher::CI4Factory(const CLSID&clsid,IUnknown *aggr,REFIID riid,void **ret) {
    METHTRACE(ModeSwitcher,CI4Factory);
	_ASSERTE(clsid==CLSID_MCCModeDelegate);
	// not likely unless they invent an aggregating moniker.  heh heh, keep outer unknown in the bind context.  heh heh.
	if(aggr)
		return CLASS_E_NOAGGREGATION;
	CComPtr<DIMCCModeDelegate> d;
	RUN(new_Delegate(&d));
	return d->QueryInterface(riid,ret);
}
// IPersist
STDMETHODIMP ModeSwitcher::GetClassID(CLSID *pClassID) {
	METHTRACE(ModeSwitcher,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
