/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Container.cpp : class factory & dll exports

#include "stdafx.h"
#include "resource.h"
#include "initguid.h"
#include "cursors.h"

#define IID_DEFINED
#include "Montage_i.c"

#include "Manual.h"

#include "Container.h"
#include "DefaultSiteOwner.h"

#include "MontagePersistor.h"
#include "SitePersistor.h"
#include "LayerPersistor.h"
#include "Ordering.h"
#include "SiteSet.h"
#include "SizeToFitTransform.h"
#include "ScrollingTransform.h"
#include "DefaultTransform.h"
#include "TransformManager.h"

#include "ContainerEditMode.h"
#include "DefaultMCCSelectable.h"
#include "ItemSetDrawer.h"
#include "ItemSetActivator.h"
#include "DefaultMCCSelectionDraw.h"
#include "ModeSwitcher.h"
#include "ControlMode.h"
#include "ModeSwitchCtl.h"
#include "DropSource.h"
#include "DataObjectReader.h"
#include "PasteSetUnpacker.h"
#include "EmbedUnpacker.h"

#include "ToolPanel.h"
#include "ToolBack.h"
#include "Toolbar.h"
#include "ToolbarOwner.h"
#include "RectEngine.h"
#include "BrutePriceEngine.h"
#include "ToolLayout.h"

#include "PosInfo.h"

#include "style.h"
#include "AmbientMapper.h"

#include "DataObject.h"
#include "ItemSetRenderer.h"
#include "InitList.h"
#include "AllPointsContainer.h"

#include "ole_init.h"

#include "boot.h"
#include "BootBlank.h"

#include "Grapple.h"
#include "ColorBox.h"
#include "ViewMode.h"

#include "Firer.h"

#include "TextEdit.h"
#include "ButtonSet.h"
#include "PlayControls.h"
#include "StickyButton.h"
#include "CheapScroll.h"
#include "CheapEdit.h"
#include "CheapCombo.h"
#include "CheapButton.h"
#include "Indicator.h"

#include "FilePort.h"
#include "Pane.h"
#include "PaneTitle.h"
#include "PropertiesPane.h"


CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
	OBJECT_ENTRY(CLSID_MontageManualLayout,MontageManual)

	OBJECT_ENTRY(CLSID_MontageControlContainer, Container)
	OBJECT_ENTRY(CLSID_MCCDefaultSiteOwner,DefaultSiteOwner)
	OBJECT_ENTRY(CLSID_MontagePersistor,MontagePersistor)
	OBJECT_ENTRY(CLSID_MCCSitePersistor,SitePersistor)
	OBJECT_ENTRY(CLSID_MCCLayerPersistor,LayerPersistor)
	OBJECT_ENTRY(CLSID_MCCSiteOrdering,SiteOrdering)
	OBJECT_ENTRY(CLSID_MCCLayerOrdering,LayerOrdering)
	OBJECT_ENTRY(CLSID_MCCSiteSet,SiteSet)
	OBJECT_ENTRY(CLSID_MCCSizeToFitTransform,SizeToFitTransform)
	OBJECT_ENTRY(CLSID_MCCScrollingTransform,ScrollingTransform)
	OBJECT_ENTRY(CLSID_MCCDefaultTransform, DefaultTransform)
	OBJECT_ENTRY(CLSID_MCCTransformManager, TransformManager)

	OBJECT_ENTRY(CLSID_MCCContainerEditMode, ContainerEditMode)
	OBJECT_ENTRY(CLSID_DefaultMCCSelectable,DefaultMCCSelectable)
	OBJECT_ENTRY(CLSID_MCCItemSetDrawer,ItemSetDrawer)
	OBJECT_ENTRY(CLSID_MCCItemSetActivator,ItemSetActivator)
	OBJECT_ENTRY(CLSID_DefaultMCCSelectionDraw,DefaultMCCSelectionDraw)
	OBJECT_ENTRY(CLSID_MCCModeSwitcher,ModeSwitcher)
	OBJECT_ENTRY(CLSID_MCCControlMode,ControlMode)
	OBJECT_ENTRY(CLSID_MCCModeSwitchCtl, ModeSwitcherCtl)
	OBJECT_ENTRY(CLSID_MCCDropSource,DropSource)
	OBJECT_ENTRY(CLSID_MCCDataObjectReader,DataObjectReader)
	OBJECT_ENTRY(CLSID_MCCPasteSetUnpacker,PasteSetUnpacker)
	OBJECT_ENTRY(CLSID_MCCEmbedUnpacker,EmbedUnpacker)

	OBJECT_ENTRY(CLSID_MCCToolPanel, ToolPanel)
	OBJECT_ENTRY(CLSID_MCCToolBack, ToolBack)
	OBJECT_ENTRY(CLSID_MCCToolbar,Toolbar)
	OBJECT_ENTRY(CLSID_MCCToolbarOwner, ToolbarOwner)
	OBJECT_ENTRY(CLSID_MCCRectEngine, RectEngine)
	OBJECT_ENTRY(CLSID_MCCBrutePriceEngine, BrutePriceEngine)
	OBJECT_ENTRY(CLSID_MCCToolLayout, ToolLayout)

	OBJECT_ENTRY(CLSID_MCCPosInfo, PosInfo)

	OBJECT_ENTRY(CLSID_MCCStyle,Style)
	OBJECT_ENTRY(CLSID_MCCAmbientMapper,AmbientMapper)

	OBJECT_ENTRY(CLSID_MCCDataObject,DataObjectClass)
	OBJECT_ENTRY(CLSID_MCCDataObjectPersistent,DataObjectPersistent)
	OBJECT_ENTRY(CLSID_MCCItemSetRenderer,ItemSetRenderer)
	OBJECT_ENTRY(CLSID_MCCInitList,InitList)
	OBJECT_ENTRY(CLSID_MCCAllPointsContainer,AllPointsContainer)

	OBJECT_ENTRY(CLSID_MCCRunInit,RunInit)
	OBJECT_ENTRY(CLSID_MCCCacheInit,CacheInit)

	OBJECT_ENTRY(CLSID_MCCBootPalette,bootPalette)
	OBJECT_ENTRY(CLSID_MCCBootPanel,bootPanel)
	OBJECT_ENTRY(CLSID_MCCBootRegularPanel,bootRegularPanel)
	OBJECT_ENTRY(CLSID_MCCBootStandard,bootStandard)
	OBJECT_ENTRY(CLSID_MCCBootBlank,bootBlank)
	OBJECT_ENTRY(CLSID_MCCBootDocument,bootDocument)
	OBJECT_ENTRY(CLSID_MCCBootDocToolbars,bootDocToolbars)
	OBJECT_ENTRY(CLSID_MCCBootModeSupport,bootModeSupport)
	OBJECT_ENTRY(CLSID_MCCBootViewMode,bootViewMode)
	OBJECT_ENTRY(CLSID_MCCBootEditMode,bootEditMode)
	OBJECT_ENTRY(CLSID_MCCGrapple, Grapple)
	OBJECT_ENTRY(CLSID_MCCColorBox, ColorBox)
	OBJECT_ENTRY(CLSID_MCCViewMode, ViewMode)

	OBJECT_ENTRY(CLSID_MCCEventFirer,Firer)

	OBJECT_ENTRY(CLSID_MCCTextEdit,TextEdit)
	OBJECT_ENTRY(CLSID_MCCButtonSet,ButtonSet)
	OBJECT_ENTRY(CLSID_MCCPlayControls,PlayControls)
    OBJECT_ENTRY(CLSID_MCCStickyButton,StickyButton)
	OBJECT_ENTRY(CLSID_MCCScrollbar, CheapScroll)
	OBJECT_ENTRY(CLSID_MCCEdit, CheapEdit)
	OBJECT_ENTRY(CLSID_MCCComboBox, CheapCombo)
	OBJECT_ENTRY(CLSID_MCCButton, CheapButton)
	OBJECT_ENTRY(CLSID_Indicator, Indicator)

	OBJECT_ENTRY(CLSID_MCCFilePort,FilePort)
	OBJECT_ENTRY(CLSID_MCCPane,Pane)
	OBJECT_ENTRY(CLSID_MCCPaneTitle,PaneTitle)
	OBJECT_ENTRY(CLSID_MCCPropertiesPane,PropertiesPane)
END_OBJECT_MAP()

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		_Module.Init(ObjectMap, hInstance,&LIBID_Montage);
		DisableThreadLibraryCalls(hInstance);
		g_cursors.init();
	}
	else if (dwReason == DLL_PROCESS_DETACH)
		_Module.Term();
	return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
	return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
	return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry
STDAPI DllRegisterServer(void)
{
	return _Module.RegisterServer(true);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
	return _Module.UnregisterServer(true);
}
