/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// MontageContainerCC.cpp : Implementation of IMontageContainer

#include "stdafx.h"
#include "Container.h"
#include "ContainerRef.h"

#include "enums.h"


STDMETHODIMP Container::get_ClassID(CLSID *ret) {
	*ret = m_clsid;
	return S_OK;
}
STDMETHODIMP Container::put_ClassID(REFCLSID val) {
	m_clsid = val;
	return S_OK;
}
STDMETHODIMP Container::get_RefObject(DIMontageContainerRef **ret) {
	(*ret = m_refObject)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::put_Group(IGroup *val) {
	METHTRACE(Container,put_Group);
	if(m_state>MCCS_HIDDEN)
		ERR(E_FAIL);
	m_group = val;
	// equiv. to InitNew, but this assumes the persistor will be attached manually
	// and the bootstrapper will be called directly.
	m_state = MCCS_SHOWN;
	// so it has to repeat this IPersistStorage HACK:
	RUN(makeRenderer());
	return S_OK;
}
STDMETHODIMP Container::GetMainGroup(REFIID riid, void **ret) {
	IMETHTRACE(Site,IMontageContainer,GetMainGroup);
	if(!m_group)
		ERR(E_PENDING);
	return m_group->QueryInterface(riid,ret);
}
STDMETHODIMP Container::get_Bootstrapper(IInitializeOther **ret) {
	IMETHTRACE(Site,IMontageContainer,put_Bootstrapper);
	m_bootstrap.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Container::put_Bootstrapper(IInitializeOther *bootz) {
	IMETHTRACE(Site,IMontageContainer,put_Bootstrapper);
	m_bootstrap = bootz;
	return S_OK;
}
STDMETHODIMP Container::get_RelMoniker(IMoniker **ret) {
	IMETHTRACE(Site,IMontageContainer,get_RelMoniker);
	m_mkPersistor.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Container::put_RelMoniker(IMoniker *val) {
	IMETHTRACE(Site,IMontageContainer,put_RelMoniker);
	m_mkPersistor = val;
	return S_OK;
}
STDMETHODIMP Container::get_Transform(IMCCTransform **ret) {
	IMETHTRACE(Site,IMontageContainer,get_Transform);
	m_transform.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Container::put_Transform(IMCCTransform *val) {
	IMETHTRACE(Site,IMontageContainer,put_Transform);

	if(CComQIPtr<IObjectWithSite> ows = m_transform)
		RUN(ows->SetSite(0));
	if(CComQIPtr<IObjectWithSite> ows = val)
		RUN(ows->SetSite((IMontageContainer*)this));
	CComPtr<IMCCTransform> old = m_transform;
	m_transform = val;
	if(m_transform)
		for(layerlist::iterator il = m_layers.begin();il!=m_layers.end();il++)
			if((*il)->m_amTransformed)
				for(sitelist::iterator is = (*il)->m_sites.begin();is!=(*il)->m_sites.end();is++) {
					RUN(m_transform->AddRect(&(*is)->m_virtRect));
					RECT rc;
					RUN(m_transform->C2WR(&(*is)->m_virtRect,&rc));
					RUN((*is)->put_Rect(&rc));
				}
	RUN(fireEvent(GetUnknown(),fireOneParam<IMontageContainerEvents,IMCCTransform*>(&IMontageContainerEvents::NewTransform,old)));
	return S_OK;
}
STDMETHODIMP Container::get_ToolPanel(IMCCToolPanel **ret) {
	IMETHTRACE(Site,IMontageContainer,get_ToolbarMgr);
	m_toolpanel.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Container::put_ToolPanel(IMCCToolPanel *val) {
	IMETHTRACE(Site,IMontageContainer,put_ToolbarMgr);
	if(m_state==MCCS_UIACTIVE)
		ERR(E_FAIL);
	m_toolpanel = val;
	return S_OK;
}
STDMETHODIMP Container::get_FilePort(IMCCFileFormat **ret) {
	m_fileport.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Container::put_FilePort(IMCCFileFormat *val) {
	m_fileport = val;
	return S_OK;
}
STDMETHODIMP Container::get_Stuff(IDispatchEx **ret) {
	(*ret = m_stuff)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::SetAreas(RECT extra,RECT reserved) {
	if(m_extraArea != extra || m_reservedArea != reserved) {
        SIZE s0 = siz(boundsPlusExtra());
		m_extraArea = extra;
        m_reservedArea = reserved;
        if(siz(boundsPlusExtra())!=s0)
		    sizeEvent();
	}
	return S_OK;
}
STDMETHODIMP Container::get_ExtraArea(RECT *ret) {
	if(m_rect==m_cliprect)
		*ret = ::rect(0,0,0,0);
	else
		*ret = m_extraArea;
	return S_OK;
}
STDMETHODIMP Container::get_ReservedArea(RECT *ret) {
	*ret = m_reservedArea;
	return S_OK;
}
HRESULT Container::get_Size(SIZE *ret) {
	IMETHTRACE(Site,IMontageContainer,get_Size);
	*ret = siz(boundsPlusExtra());
	return S_OK;
}
HRESULT Container::get_State(long *ret) {
	IMETHTRACE(Site,IMontageContainer,get_State);
	*ret = m_state;
	return S_OK;
}
STDMETHODIMP Container::GetFocus(BOOL *any,IMCCSite **ret) {
    *any = m_haveFocus||m_pFocusObject;
    if(m_pFocusObject)
        (*ret = m_pFocusObject)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::put_MinExtent(SIZEL val) {
    METHTRACE(ItemSetRenderer,put_MinExtent);
	m_minExtent = val;
	return S_OK;
}
STDMETHODIMP Container::get_MinExtent(SIZEL *ret) {
    METHTRACE(ItemSetRenderer,get_MinExtent);
	*ret = m_minExtent;
	return S_OK;
}
STDMETHODIMP Container::put_AutoSize(VARIANT_BOOL val) {
	if(m_autoSize = val!=VARIANT_FALSE)
		RUN(dirty());
	return S_OK;
}
STDMETHODIMP Container::get_AutoSize(VARIANT_BOOL *ret) {
	*ret = m_autoSize*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP Container::put_Zoom(long val) {
    METHTRACE(Container,put_Zoom);
	m_zoom = val;
	extentEvent();
	return S_OK;
}
STDMETHODIMP Container::get_Zoom(long *ret) {
    METHTRACE(Container,get_Zoom);
	*ret = m_zoom;
	return S_OK;
}
STDMETHODIMP Container::GetZoomedExtent(SIZEL *ret) {
    METHTRACE(Container,GetZoomedExtent);
	*ret = size(long(__int64(m_extent.cx)*10000/__int64(m_zoom)),
				long(__int64(m_extent.cy)*10000/__int64(m_zoom)));
	return S_OK;
}
STDMETHODIMP Container::put_Margin(RECTL val) {
	m_margin = val;
	return S_OK;
}
STDMETHODIMP Container::get_Margin(RECTL *ret) {
	*ret = m_margin;
	return S_OK;
}
STDMETHODIMP Container::put_AlwaysMargin(VARIANT_BOOL val) {
	m_alwaysMargin = val!=0;
	return S_OK;
}
STDMETHODIMP Container::get_AlwaysMargin(VARIANT_BOOL *ret) {
	*ret = VARIANT_TRUE*m_alwaysMargin;
	return S_OK;
}
STDMETHODIMP Container::get_UseMargin(VARIANT_BOOL *ret) {
	// always use margin if a document; otherwise rely on field
	*ret = VARIANT_TRUE*(m_alwaysMargin||CComQIPtr<IOleDocumentSite>(m_pSite));
	return S_OK;
}
STDMETHODIMP Container::new_Layer(IMCCLayer **ret) {
	IMETHTRACE(Site,IMontageContainer,new_Layer);
	Layer *l;
	RUN(createLayer(this,&l));
	m_unlayers.push_back(l); l->AddRef(); // balanced by IMCCLayer::Delete
	(*ret = l)->AddRef();
	return S_OK;
}
HRESULT Container::new_Site(IMCCSite **ret) {
	IMETHTRACE(Site,IMontageContainer,new_Site);
	site *s;
	RUN(createSite(this,&s));
	s->m_pLayer = m_nonlayer;
	s->AddRef();
	RUN(after(s)); // put in front
	(*ret = s)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::EnumLayers(IEnumMCCLayers **ret) {
	METHTRACE(Container,EnumLayers);
	layerenum *le = new layerenum;
	IMCCLayer **ray = new IMCCLayer*[m_layers.size()],
		**i = ray;
	for(layerlist::iterator il = m_layers.begin();il!=m_layers.end();il++)
		(*i++ = *il)->AddRef();
	le->Init(ray,i,0,AtlFlagTakeOwnership);
	(*ret = le)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::get_DefaultSiteSinks(IMCCAllPointsContainer **ret) {
	METHTRACE(Container,get_DefaultSiteSinks);
	(*ret = m_defaultSiteSinks)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::ActivateAs(HWND hWnd) {
	IMETHTRACE(Site,IMontageContainer,ActivateAs);
	if(!hWnd) { // call with NULL to drop out of parasitic active
		if(m_ownwindow)
			ERR(E_INVALIDARG);
		m_hWnd = 0;
		m_ownwindow = true;
        DWORD old = m_state;
		m_state = MCCS_SHOWN;
		RUN(fireEvent(GetUnknown(),fireTwoParams<IMontageContainerEvents,long,long>(&IMontageContainerEvents::Shifting,old,m_state)));
		return pullsites();
	}
	else {
		if(m_state>MCCS_HIDDEN) {
			if(m_hWnd)
				return (hWnd==m_hWnd)?S_FALSE:E_FAIL;
			RUN(chstate(MCCS_SHOWN));
		}
		m_hWnd = hWnd;
		m_ownwindow = false;
        DWORD old = m_state;
		m_state = MCCS_ACTIVE;
		RUN(fireEvent(GetUnknown(),fireTwoParams<IMontageContainerEvents,long,long>(&IMontageContainerEvents::Shifting,old,m_state)));
		return pullsites();
	}
}
STDMETHODIMP Container::ActivateInto(HWND hWnd,VARIANT_BOOL show) {
    if(!hWnd) {
        CComQIPtr<IOleWindow> ow = m_pSite;
        if(ow)
            ow->GetWindow(&hWnd);
        if(!hWnd)
            ERR(E_UNEXPECTED);
    }
    m_intoWnd = hWnd;
    GetClientRect(hWnd,&m_rect);
    m_cliprect = m_rect;
    DWORD old = m_state;
    m_state = MCCS_ACTIVE;
    RUN(makewindow(show!=VARIANT_FALSE));
	RUN(fireEvent(GetUnknown(),fireTwoParams<IMontageContainerEvents,long,long>(&IMontageContainerEvents::Shifting,old,m_state)));
    return pullsites();
}
STDMETHODIMP Container::get_HatchMode(long *ret) {
    METHTRACE(Container,get_HatchMode);
	*ret = m_hatchMode;
	return S_OK;
}
STDMETHODIMP Container::put_HatchMode(long val) {
    METHTRACE(Container,put_HatchMode);
	m_hatchMode = val;
	return S_OK;
}
STDMETHODIMP Container::RequestTrigger(IMCCInThreadTrigger *callback) {
    METHTRACE(Container,RequestTrigger);
	if(!m_ownwindow)
		ERR(E_FAIL);
	callback->AddRef();
	m_triggers.push_back(callback);
	PostMessage(m_hWnd,MONTAGE_TRIGGER_MESSAGE,0,0);
	return S_OK;
}
STDMETHODIMP Container::RegisterMessageHandler(BOOL isCommand, UINT msg,IWindowMessageHandler *handler) {
    if(!handler)
        ERR(E_INVALIDARG);
    IWindowMessageHandler *&slot = (isCommand?m_commandMap:m_notifyMap)[msg];
    if(slot)
        slot->Release();
    (slot = handler)->AddRef();
	return S_OK;
}
STDMETHODIMP Container::ForgetHandler(BOOL isCommand, UINT msg) {
    IWindowMessageHandler *&slot = (isCommand?m_commandMap:m_notifyMap)[msg];
    if(slot)
        slot->Release();
    slot = 0;
	return S_OK;
}
STDMETHODIMP Container::TransformChanged(VARIANT_BOOL offsetChanged,VARIANT_BOOL scaleChanged) {
    for(layerlist::iterator li = m_layers.begin(); li!=m_layers.end(); ++li)
        for(sitelist::iterator si = (*li)->m_sites.begin(); si != (*li)->m_sites.end(); ++si)
            if((*li)->m_amTransformed) {
                if((*si)->m_bRectValid) {
					RECT rc;
					RUN(m_transform->C2WR(&(*si)->m_virtRect,&rc));
					RUN((*si)->put_Rect(&rc));
				}
			}
            else if(scaleChanged && (*si)->m_autoSetExtent)
                RUN((*si)->UpdateExtent(VARIANT_TRUE));
	return S_OK;
}
