/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "MontagePersistor.h"
#include "cacheStreamableObject.h"
MontagePersistor::MontagePersistor() :
	m_realized(false),
	m_classID(CLSID_MontageControlContainer),
	m_useSubjectExtent(-1),
	m_extent(size(2500,2500)), // default 5x5in
	m_minExtent(m_extent)
{
	CONSTRUCTORTRACE(MontagePersistor);
}
MontagePersistor::~MontagePersistor() {
	DESTRUCTORTRACE(MontagePersistor);
}

// IMontagePersistor
STDMETHODIMP MontagePersistor::put_SelfMoniker(IMoniker *val) {
	METHTRACE(MontagePersistor,put_SelfMoniker);
	m_mkSelf = val;
	return S_OK;
}
STDMETHODIMP MontagePersistor::get_SelfMoniker(IMoniker **ret) {
	m_mkSelf.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP MontagePersistor::get_TransformMk(IMoniker **ret) {
	METHTRACE(MontagePersistor,get_TransformMk);
	m_mkTransform.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_Class(REFCLSID rcls) {
	METHTRACE(MontagePersistor,put_Class);
	m_classID = rcls;
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_UseSubjectExtent(VARIANT_BOOL val) {
	m_useSubjectExtent = val;
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_InitialExtent(SIZEL val) {
	METHTRACE(MontagePersistor,put_InitialExtent);
	RUN(put_UseSubjectExtent(0));
	m_extent = val;
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_TransformMk(IMoniker *val) {
	METHTRACE(MontagePersistor,put_TransformMk);
	if(!m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> old = m_mkTransform;
	RUN(m_group->AddStgRef(m_mkTransform = val));
	if(old)
		RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
STDMETHODIMP MontagePersistor::get_ToolPanelMk(IMoniker **ret) {
	METHTRACE(MontagePersistor,get_ToolPanelMk);
	m_mkPanel.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_ToolPanelMk(IMoniker *val) {
	METHTRACE(MontagePersistor,put_ToolPanelMk);
	if(!m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> old = m_mkPanel;
	RUN(m_group->AddStgRef(m_mkPanel = val));
	if(old)
		RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
STDMETHODIMP MontagePersistor::get_FilePortMk(IMoniker **ret) {
	m_mkFilePort.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_FilePortMk(IMoniker *val) {
	m_mkFilePort = val;
	return S_OK;
}

STDMETHODIMP MontagePersistor::get_KeepSet(IItemSet **ret) {
	METHTRACE(MontagePersistor,get_KeepSet);
	m_keepSet.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP MontagePersistor::put_KeepSet(IItemSet *val) {
	METHTRACE(MontagePersistor,put_KeepSet);
	m_keepSet = val;
	return S_OK;
}
STDMETHODIMP MontagePersistor::ForgetSubject() {
	// sux
	m_subject.p = 0;
	return S_OK;
}

// DIMontagePersistor
STDMETHODIMP MontagePersistor::put_SelfMoniker(IUnknown *val) {
    METHTRACE(MontagePersistor,put_SelfMoniker);
	CComPtr<IMoniker> mk2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&mk2)));
	return put_SelfMoniker(mk2);
}
STDMETHODIMP MontagePersistor::put_Class(BSTR rcls) {
    METHTRACE(MontagePersistor,put_Class);
	CLSID clsid;
	RUNRET(CLSIDFromString(rcls,&clsid));
	return put_Class(clsid);
}
STDMETHODIMP MontagePersistor::put_InitialExtent(long cx, long cy) {
    METHTRACE(MontagePersistor,put_InitialExtent);
	SIZEL s = {cx,cy};
	return put_InitialExtent(s);
}
STDMETHODIMP MontagePersistor::get_TransformMk(IUnknown **ret) {
    METHTRACE(MontagePersistor,get_TransformMk);
	return get_TransformMk((IMoniker**)ret);
}
STDMETHODIMP MontagePersistor::put_TransformMk(IUnknown *val) {
    METHTRACE(MontagePersistor,put_TransformMk);
	CComPtr<IMoniker> mk2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&mk2)));
	return put_TransformMk(mk2);
}
STDMETHODIMP MontagePersistor::get_ToolPanelMk(IUnknown **ret) {
    METHTRACE(MontagePersistor,get_ToolPanelMk);
	return get_ToolPanelMk((IMoniker**)ret);
}
STDMETHODIMP MontagePersistor::put_ToolPanelMk(IUnknown *val) {
    METHTRACE(MontagePersistor,put_ToolPanelMk);
	CComPtr<IMoniker> mk2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&mk2)));
	return put_ToolPanelMk(mk2);
}
STDMETHODIMP MontagePersistor::get_KeepSet(DIItemSet **ret) {
    METHTRACE(MontagePersistor,get_KeepSet);
	return m_keepSet->QueryInterface(riidret(ret));
}
STDMETHODIMP MontagePersistor::put_KeepSet(DIItemSet *val) {
    METHTRACE(MontagePersistor,put_KeepSet);
	CComPtr<IItemSet> ii2;
	if(val)
		RUNRET(val->QueryInterface(riidret(&ii2)));
	return put_KeepSet(ii2);
}

// IPersistor
STDMETHODIMP MontagePersistor::SetSubject(VARIANT_BOOL realized,IUnknown *pUnkSite) {
	METHTRACE(MontagePersistor,SetSubject);
	m_subject = pUnkSite;
	m_realized = realized; // supports "realize this" mode (it must, since Montage is created by the container!)
	return S_OK;
}
STDMETHODIMP MontagePersistor::get_Subject(IUnknown **ret) {
	METHTRACE(MontagePersistor,GetSubject);
	if(!m_subject)
		ERR(E_FAIL);
	(*ret = m_subject)->AddRef();
	return S_OK;
}
// IGroupItem
STDMETHODIMP MontagePersistor::Connect(IGroup *group) {
	METHTRACE(MontagePersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	if(m_keepSet)
		RUN(CComQIPtr<IGroupItem>(m_keepSet)->Connect(group));
	return S_OK;
}
STDMETHODIMP MontagePersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(MontagePersistor,Copy);
	CComPtr<IMoniker> mkSelf2,mkTrans2,mkTool2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkSelf,IID_NULL,0,&mkSelf2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkTransform,IID_NULL,0,&mkTrans2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkPanel,IID_NULL,0,&mkTool2));
	{
		// paste Data, if there... ItemSetRenderer overrides this by pasting a blank data set first.
		ItemMoniker mkData(L"Data");
		CComPtr<IUnknown> uData;
		if(SUCCEEDED(mkData->BindToObject(bctx,PointerMoniker(m_group),riidret(&uData))))
			RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,mkData,IID_NULL,0,0));
	}
	CComPtr<IItemSet> ks2;
	if(m_keepSet)
		RUN(CComQIPtr<IGroupItem>(m_keepSet)->Copy(bctx,dest,0,riidret(&ks2)));
	CComPtr<IMontagePersistor> mp2;
	RUN(CREATE(mp2,CLSID_MontagePersistor));
	RUN(CComQIPtr<IGroupItem>(mp2)->Connect(dest));
	if(m_subject)
		RUN(CComQIPtr<DIPersistor>(mp2)->SetSubject(m_realized,m_subject));
	// don't need to call InitNew because InitNew just creates the keepset
	// hack: doesn't call SetSubject.  Would probably cause refcounting problems.
	RUN(mp2->put_SelfMoniker(mkSelf2));
	RUN(mp2->put_Class(m_classID));
	RUN(mp2->put_InitialExtent(m_extent));
	RUN(mp2->put_TransformMk(mkTrans2));
	RUN(mp2->put_ToolPanelMk(mkTool2));
	RUN(mp2->put_KeepSet(ks2));
	return mp2->QueryInterface(riid,ret);
}
STDMETHODIMP MontagePersistor::Realize(IBindCtx *bctx) {
	METHTRACE(MontagePersistor,Realize);
	if(m_realized)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	m_realized = true;
	if(!m_group)
		ERR(E_PENDING);
	if(!m_subject) { // I'm to create Montage: Montage is in a group and I'm a real persistor
		RUN(CREATE(m_subject,m_classID));
		RUN(m_subject->put_Group(m_group));
	}
	// else Montage is loading a storage itself and then just siccing me on it.

	// tell it mk
	RUN(m_subject->put_RelMoniker(m_mkSelf));

	// set extent
	RUN(CComQIPtr<IOleObject>(m_subject)->SetExtent(DVASPECT_CONTENT,&m_extent));
	RUN(m_subject->put_MinExtent(m_minExtent));

	// place Montage's other parameters (transform, toolpanel, file im/export)
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	if(m_mkTransform) {
		CComPtr<IMCCTransform> tron;
		RUN(m_mkTransform->BindToObject(bctx,mkG,riidret(&tron)));
		RUN(m_subject->put_Transform(tron));
	}
	if(m_mkPanel) {
		CComPtr<IMCCToolPanel> tanel;
		RUN(m_mkPanel->BindToObject(bctx,mkG,riidret(&tanel)));
		RUN(m_subject->put_ToolPanel(tanel));
	}
	// hack: transform manager must be realized before any sites
	ItemMoniker mkTM(L"Transform Manager");
	CComPtr<IGroupItem> tmP;
	if(SUCCEEDED(mkTM->BindToObject(bctx,mkG,riidret(&tmP))))
		RUN(tmP->Realize(bctx));
	if(m_mkFilePort) {
		CComPtr<IMCCFileFormat> tamrof;
		RUN(m_mkFilePort->BindToObject(bctx,mkG,riidret(&tamrof)));
		RUN(m_subject->put_FilePort(tamrof));
	}
	// load everything that's essential to the configuration
	RUN(CComQIPtr<IGroupItem>(m_keepSet)->Realize(bctx));
	if(m_cache.stuff) {
		// m_cache.stuff also indicates whether we loaded other stuff from storage
		RUN(m_subject->put_ClassID(m_cache.clsid));
		RUN(m_subject->SetAreas(m_cache.extra,m_cache.reserved));
		RUN(m_subject->put_AutoSize(m_cache.autoSize));
		RUN(m_subject->put_Zoom(m_cache.zoom));
		RUN(m_subject->put_AlwaysMargin(m_cache.alwaysMargin));
		RUN(m_subject->put_Margin(m_cache.margin));

		// restore Stuff
		CComPtr<IDispatchEx> stuff;
		RUN(m_subject->get_Stuff(&stuff));
		RUN(CComQIPtr<IPersistStream>(stuff)->Load(m_cache.stuff));
		m_cache.stuff.Release();
	}
	// and then the data
	CComPtr<IGroupItem> dataset;
	if(SUCCEEDED(CComQIPtr<IOleItemContainer>(m_group)->GetObject(L"Data",
	  BINDSPEED_INDEFINITE,bctx,riidret(&dataset))))
		RUN(dataset->Realize(bctx));
	return S_OK;
}
STDMETHODIMP MontagePersistor::Remove() {
	METHTRACE(MontagePersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkTransform));
	RUN(m_group->ReleaseStgRef(m_mkPanel));
	return CComQIPtr<IGroupItem>(m_keepSet)->Remove();
}
// IPersistStream
STDMETHODIMP MontagePersistor::GetClassID(CLSID *pClassID) {
	METHTRACE(MontagePersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP MontagePersistor::IsDirty(void) {
	METHTRACE(MontagePersistor,IsDirty);
	return S_OK;
}
static const int magic = 19;
STDMETHODIMP MontagePersistor::Load(IStream *pStm) {
	METHTRACE(MontagePersistor,Load);
	if(!m_group)
		ERR(E_PENDING);
	int test;
	RUN(pStm->Read(&test,sizeof test,0));
	if(test!=magic) // file format has changed
		ERR(E_FAIL);
	RUN(ReadClassStm(pStm,&m_classID));
	RUN(pStm->Read(&m_extent,sizeof m_extent,0));
	RUN(pStm->Read(&m_minExtent,sizeof m_minExtent,0));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkSelf)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkTransform)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkPanel)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkFilePort)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_keepSet)));
	if(m_keepSet)
		RUN(CComQIPtr<IGroupItem>(m_keepSet)->Connect(m_group));
	bool hadSubj;
	RUN(pStm->Read(&hadSubj,1,0));
	if(hadSubj) {
		RUN(pStm->Read(&m_cache.clsid,sizeof(CLSID),0));
		RUN(pStm->Read(&m_cache.extra,sizeof(RECT),0));
		RUN(pStm->Read(&m_cache.reserved,sizeof(RECT),0));
		RUN(pStm->Read(&m_cache.autoSize,sizeof m_cache.autoSize,0));
		RUN(pStm->Read(&m_cache.zoom,sizeof m_cache.zoom,0));
		RUN(pStm->Read(&m_cache.alwaysMargin,sizeof m_cache.alwaysMargin,0));
		RUN(pStm->Read(&m_cache.margin,sizeof m_cache.margin,0));
		RUN(cacheStreamableObject(pStm,&m_cache.stuff));
	}
	return S_OK;
}
STDMETHODIMP MontagePersistor::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(MontagePersistor,Save);
	bool haveSubj = m_subject;
	RUN(pStm->Write(&magic,sizeof magic,0));
	RUN(WriteClassStm(pStm,m_classID));
	if(m_useSubjectExtent&&haveSubj) {
		RUN(CComQIPtr<IOleObject>(m_subject)->GetExtent(DVASPECT_CONTENT,&m_extent));
		RUN(m_subject->get_MinExtent(&m_minExtent));
	}
	RUN(pStm->Write(&m_extent,sizeof m_extent,0));
	RUN(pStm->Write(&m_minExtent,sizeof m_minExtent,0));
	RUN(FixedOleSaveToStream(m_mkSelf,pStm));
	RUN(FixedOleSaveToStream(m_mkTransform,pStm));
	RUN(FixedOleSaveToStream(m_mkPanel,pStm));
	RUN(FixedOleSaveToStream(m_mkFilePort,pStm));
	RUN(FixedOleSaveToStream(m_keepSet,pStm));
	RUN(pStm->Write(&haveSubj,1,0));
	if(haveSubj) {
		CLSID itsClsid;
		RUN(m_subject->get_ClassID(&itsClsid)); // not sure if this need be different from m_classID but for now I'm leaving em separate
		RUN(pStm->Write(&itsClsid,sizeof(CLSID),0));
		RECT rc;
		RUN(m_subject->get_ExtraArea(&rc));
		RUN(pStm->Write(&rc,sizeof rc,0));
		RUN(m_subject->get_ReservedArea(&rc));
		RUN(pStm->Write(&rc,sizeof rc,0));
		VARIANT_BOOL autos;
		RUN(m_subject->get_AutoSize(&autos));
		RUN(pStm->Write(&autos,sizeof autos,0));
		long zoom;
		RUN(m_subject->get_Zoom(&zoom));
		RUN(pStm->Write(&zoom,sizeof zoom,0));
		VARIANT_BOOL alwaysMargin;
		RUN(m_subject->get_AlwaysMargin(&alwaysMargin));
		RUN(pStm->Write(&alwaysMargin,sizeof alwaysMargin,0));
		RECTL margin;
		RUN(m_subject->get_Margin(&margin));
		RUN(pStm->Write(&margin,sizeof margin,0));
		CComPtr<IDispatchEx> stuff;
		RUN(m_subject->get_Stuff(&stuff));
		RUN(saveWithSize(pStm,CComQIPtr<IPersistStream>(stuff),fClearDirty));
	}
	return S_OK;
}
STDMETHODIMP MontagePersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(MontagePersistor,GetSizeMax);
	CComObjectStack<countStream> s;
	RUN(Save(&s,false));
	*pcbSize = s.m_pos;
	return S_OK;
}
STDMETHODIMP MontagePersistor::InitNew() {
	METHTRACE(MontagePersistor,InitNew);
	if(!m_group)
		ERR(E_PENDING);
	// might make sense to use a SiteSet here but that needs a moniker to Montage, which
	// we don't have yet.
	if(!m_keepSet)
		RUN(CREATE(m_keepSet,CLSID_ItemSet));
	RUN(CComQIPtr<IGroupItem>(m_keepSet)->Connect(m_group));
	return S_OK;
}

