/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// OleControlSiteCS.cpp : Implementation of IOleControlSite for
// the simple control site I'm doin' up.

#include "stdafx.h"
#include "Container.h"

HRESULT Site::processMnemonics(bool add) {
	METHTRACE(Site,processMnemonics);
	if(CComQIPtr<IOleControl> oc = m_pControl) {
		if(add) {
			CONTROLINFO ci;
			ci.cb = sizeof(CONTROLINFO);
			if(SUCCEEDED(oc->GetControlInfo(&ci)))
				m_cont->m_mnemon[(site*)this] = ci;
		}
		else
			m_cont->m_mnemon.erase((site*)this);
	}
	return S_OK;
}

STDMETHODIMP Site::OnControlInfoChanged(void) {
	IMETHTRACE(Site,IOleControlSite,OnControlInfoChanged);
	return processMnemonics(true);
}
STDMETHODIMP Site::LockInPlaceActive(BOOL fLock) {
	IMETHTRACE(Site,IOleControlSite,LockInPlaceActive);
	return S_OK;
}
STDMETHODIMP Site::GetExtendedControl(IDispatch **ppDisp) {
	IMETHTRACE(Site,IOleControlSite,GetExtendedControl);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Site::TransformCoords(
     POINTL *pPtlHimetric,
     POINTF *pPtfContainer,
     DWORD dwFlags) {
	IMETHTRACE(Site,IOleControlSite,TransformCoords);
	// WHAT IS THIS FOR?
	// coords are the same.  This is a really odd function
	// anyway
	if(dwFlags&XFORMCOORDS_HIMETRICTOCONTAINER) {
		pPtfContainer->x = (float)pPtlHimetric->x;
		pPtfContainer->y = (float)pPtlHimetric->y;
	}
	else {
		pPtlHimetric->x = (long)pPtfContainer->x;
		pPtlHimetric->y = (long)pPtfContainer->y;
	}
	return S_OK;
}
STDMETHODIMP Site::TranslateAccelerator(
     MSG *pMsg,
     DWORD grfModifiers) {
	IMETHTRACE(Site,IOleControlSite,TranslateAccelerator);
	// make sure it doesn't go back to same item?
	return m_cont->TranslateAccelerator(pMsg);
}
STDMETHODIMP Site::OnFocus(BOOL fGotFocus) {
	IMETHTRACE(Site,IOleControlSite,OnFocus);
    if(fGotFocus)
        m_cont->m_pFocusObject = static_cast<site*>(this);
    else if(m_cont->m_pFocusObject==this)
        m_cont->m_pFocusObject = 0;
    m_cont->changeFocus();
	return S_OK;
}
STDMETHODIMP Site::ShowPropertyFrame(void) {
	IMETHTRACE(Site,IOleControlSite,ShowPropertyFrame);
	ERR(E_NOTIMPL);
}
