/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "StdAfx.h"
#include "Container.h"

// IOleDocumentView implementation
STDMETHODIMP Container::SetInPlaceSite(IOleInPlaceSite *pIPSite) {
    IMETHTRACE(Container,IOleDocumentView,SetInPlaceSite);
	m_pInPlaceSite = pIPSite;
	return S_OK;
}
STDMETHODIMP Container::GetInPlaceSite(IOleInPlaceSite **ppIPSite) {
    IMETHTRACE(Container,IOleDocumentView,GetInPlaceSite);
	m_pInPlaceSite.CopyTo(ppIPSite);
	return S_OK;
}
STDMETHODIMP Container::GetDocument(IUnknown **ppunk) {
    IMETHTRACE(Container,IOleDocumentView,GetDocument);
	(*ppunk = GetUnknown())->AddRef();
	return S_OK;
}
STDMETHODIMP Container::SetRect(LPRECT prcView) {
    IMETHTRACE(Container,IOleDocumentView,SetRect);
	return SetObjectRects(prcView,prcView);
}
STDMETHODIMP Container::GetRect(LPRECT prcView) {
    IMETHTRACE(Container,IOleDocumentView,GetRect);
	*prcView = m_rect;
	return S_OK;
}
STDMETHODIMP Container::SetRectComplex(LPRECT prcView,LPRECT prcHScroll,LPRECT prcVScroll,LPRECT prcSizeBox) {
    IMETHTRACE(Container,IOleDocumentView,SetRectComplex);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Container::Show(BOOL fShow) {
    IMETHTRACE(Container,IOleDocumentView,Show);
	if(fShow) {
		RUN(chstate(MCCS_UIACTIVE));
		RUN(setExtentFromRect(m_rect,false));
		return S_OK;
	}
	else
		return chstate(MCCS_SHOWN);
}
STDMETHODIMP Container::UIActivate(BOOL fUIActivate) {
    IMETHTRACE(Container,IOleDocumentView,UIActivate);
	if(fUIActivate) {
		// In old OLE activation, you show the window first then put up toolbars & menus.
		// In DocObject, it's the other way around.  So make the window hidden.
		RUN(getInPlaceWindowContext());
		RUN(makewindow(false));
		return installUI();
	}
	else
		return removeUI();
}
STDMETHODIMP Container::Open(void) {
    IMETHTRACE(Container,IOleDocumentView,Open);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Container::CloseView(DWORD dwReserved) {
    IMETHTRACE(Container,IOleDocumentView,CloseView);
	RUN(Show(false));
	RUN(SetInPlaceSite(0));
	return S_OK;
}
STDMETHODIMP Container::SaveViewState(LPSTREAM pstm) {
    IMETHTRACE(Container,IOleDocumentView,SaveViewState);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Container::ApplyViewState(LPSTREAM pstm) {
    IMETHTRACE(Container,IOleDocumentView,ApplyViewState);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Container::Clone(IOleInPlaceSite *pIPSiteNew,IOleDocumentView **ppViewNew) {
    IMETHTRACE(Container,IOleDocumentView,Clone);
	ERR(E_NOTIMPL);
}
