/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IOleInPlaceActiveObject for Container
// OLE wants this interface to be on a separate object.
#include "stdafx.h"
#include "Container.h"
#include "NotherIsAccelerator.h"
#include "keymessname.h"

STDMETHODIMP Container::TranslateAccelerator(LPMSG lpmsg) {
	IMETHTRACE(Site,IOleInPlaceActiveObject,TranslateAccelerator);
	if(!keymess(lpmsg->message))
		return S_FALSE;
	for(mnemon_info::iterator i = m_mnemon.begin();i!=m_mnemon.end();++i) {
		CONTROLINFO &ci = i->second;
		if(NotherIsAccelerator(i->second.hAccel,i->second.cAccel,lpmsg,0)) {
			RUN(CComQIPtr<IOleControl>(i->first->m_pControl)->OnMnemonic(lpmsg));
			return S_OK;
		}
	}
#define TRANSLATE_ALL_KEYS
#ifdef TRANSLATE_ALL_KEYS
	if(CComQIPtr<IOleControlSite> ctrlsite = m_pSite)
		return ctrlsite->TranslateAccelerator(lpmsg,0);
	TranslateMessage(lpmsg);
	DispatchMessage(lpmsg);
	return S_OK;
#else
	return S_FALSE;
#endif
}
// greedy?
STDMETHODIMP Container::OnFrameWindowActivate(BOOL fActivate) {
	IMETHTRACE(Site,IOleInPlaceActiveObject,OnFrameWindowActivate);
	if(fActivate)
		SetFocus(m_hWnd);
	return S_OK;
}

STDMETHODIMP Container::OnDocWindowActivate(BOOL fActivate) {
	IMETHTRACE(Site,IOleInPlaceActiveObject,OnDocWindowActivate);
	if(fActivate)
		SetFocus(m_hWnd);
	return S_OK;
}

STDMETHODIMP Container::ResizeBorder(
     LPCRECT prcBorder,
     IOleInPlaceUIWindow *pUIWindow,
     BOOL fFrameWindow) {
	IMETHTRACE(Site,IOleInPlaceActiveObject,ResizeBorder);
	if(fFrameWindow && m_toolpanel)
		RUN(m_toolpanel->Resize(prcBorder));
	// p/h allow both frame and doc panels?
	return S_OK;
}

STDMETHODIMP Container::EnableModeless(BOOL fEnable) {
	IMETHTRACE(Site,IOleInPlaceActiveObject,EnableModeless);
	return S_OK;
}
