/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IOleInPlaceObject for Container
#include "stdafx.h"
#include "Container.h"
#include "Hatchling.h"

//(IOleWindow)
STDMETHODIMP Container::GetWindow(HWND *phwnd) {
	IMETHTRACE(Site,IOleInPlaceObject,GetWindow);
	if(m_pHatchling)
		*phwnd = m_pHatchling->Wnd();
	else
		*phwnd = m_hWnd;
	return S_OK;
}
STDMETHODIMP Container::ContextSensitiveHelp(BOOL fEnterMode) {
	IMETHTRACE(Site,IOleInPlaceObject,ContextSensitiveHelp);
	ERR(E_NOTIMPL);
}

//(IOleInPlaceObject)
STDMETHODIMP Container::InPlaceDeactivate(void) {
	IMETHTRACE(Site,IOleInPlaceObject,InPlaceDeactivate);
	if(m_state>MCCS_OPEN)
		RUN(chstate(MCCS_SHOWN));
	return S_OK;
}

STDMETHODIMP Container::UIDeactivate(void) {
	IMETHTRACE(Site,IOleInPlaceObject,UIDeactivate);
	if(m_state==MCCS_UIACTIVE)
		RUN(uideactivate());
	return S_OK;
}

STDMETHODIMP Container::setObjectRects(LPCRECT lprcPosRect,LPCRECT lprcClipRect,bool setExtent,bool moveWindow,bool setClip) {
	IMETHTRACE(Site,IOleInPlaceObject,setObjectRects);
    RUN(groupOp(this,true));
    if(setExtent)
		RUN(setExtentFromRect(*lprcPosRect,false));
	bool sizing = wid(*lprcPosRect)!=wid(m_rect) || hei(*lprcPosRect) != hei(m_rect);
	m_rect = *lprcPosRect;
	m_cliprect = *lprcClipRect;
	if(m_hWnd&&m_ownwindow) {
		if(m_pHatchling)
			m_pHatchling->fit(boundsPlusExtra(),m_cliprect);
        if(moveWindow) {
		    RECT pos = windowRect();
		    SetWindowPos(m_hWnd,0,pos.left,pos.top,wid(pos),hei(pos),SWP_NOZORDER);
        }
        if(setClip) {
		    HRGN clip = clipRgn();
		    SetWindowRgn(m_hWnd,clip,true);
        }
	}
	if(sizing)
		sizeEvent();
    RUN(groupOp(this,false));
	return S_OK;
}
STDMETHODIMP Container::SetObjectRects(LPCRECT lprcPosRect,LPCRECT lprcClipRect) {
    return setObjectRects(lprcPosRect,lprcClipRect,CComQIPtr<IOleDocumentSite>(m_pSite),true,true);
}
STDMETHODIMP Container::ReactivateAndUndo(void) {
	IMETHTRACE(Site,IOleInPlaceObject,ReactivateAndUndo);
	return S_OK;
}
