/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// OleInPlaceSiteWindowlessCS.cpp : Implementation of IOleInPlaceSiteWindowlessCS for
// the simple control site I'm doin' up.

#include "stdafx.h"
#include "Container.h"

STDMETHODIMP Site::GetWindow(HWND *phwnd) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,GetWindow);
	if(m_usewnd)
		*phwnd = m_usewnd;
	else
		*phwnd = m_cont->m_hWnd;
	return S_OK;
}
STDMETHODIMP Site::ContextSensitiveHelp(BOOL fEnterMode) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,ContextSensitiveHelp);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Site::CanInPlaceActivate() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,CanInPlaceActivate);
	return S_OK;
}
STDMETHODIMP Site::OnInPlaceActivate() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnInPlaceActivate);
	return OnInPlaceActivateEx(0,0);
}
STDMETHODIMP Site::OnUIActivate() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnUIActivate);
	if(!m_cont->m_pInPlaceSite || m_cont->m_state<MCCS_UIACTIVE) // contained can't go UI active if we can't!
		ERR(E_UNEXPECTED);
	// see OnInPlaceActivate
	if(m_intent<MCCS_UIACTIVE) {
		DWORD oldState = m_state, oldIntent = m_intent;
		m_state = m_intent = MCCS_UIACTIVE;
		HRESULT hr;
		if(FAILED(hr = fireSiteEvent(this,fireIntend(MCCS_UIACTIVE)))) {
			m_intent = oldIntent;
			return hr;
		}
		stateEvent(oldState,m_state);
	}
    m_cont->m_pFocusObject = static_cast<site*>(this);
    m_cont->changeFocus();
	// replace m_pUIActiveObject before calling UIDeactivate so that OnUIDeactivate knows whether
	// it's a replacement or simply a drop of the UI
	CComPtr<IOleInPlaceObject> last = m_cont->m_pUIActiveObject;
	m_cont->m_pUIActiveObject = m_pControl;
	if(last==m_cont->m_pUIActiveObject) // just in case
		return S_OK;
	if(last)
		return last->UIDeactivate();
	else
		return m_cont->removeUI();
}
STDMETHODIMP Site::GetWindowContext(
	 IOleInPlaceFrame **ppFrame,
	 IOleInPlaceUIWindow **ppDoc,
	 LPRECT lprcPosRect,
	 LPRECT lprcClipRect,
	 LPOLEINPLACEFRAMEINFO lpFrameInfo) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,GetWindowContext);
	if(!m_cont->m_pInPlaceSite) { // eg. if ActivateAs called
		*ppFrame = 0;
		*ppDoc = 0;
		// don't set lpFrameInfo; object can't go UI active
	}
	else {
		m_cont->m_pFrame.CopyTo(ppFrame);
		m_cont->m_pDoc.CopyTo(ppDoc);
	}
	*lprcPosRect = m_windRect;
	*lprcClipRect = clipRect();
	*lpFrameInfo = m_cont->m_frameInfo;
	return S_OK;
}
STDMETHODIMP Site::Scroll(SIZE scrollExtent) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,Scroll);
	if(CComQIPtr<IMCCScrollingTransform> st = m_cont->m_transform) {
		SIZEL s2;
		RUN(m_cont->m_transform->W2CS(scrollExtent,&s2));
		POINTL ofs;
		RUN(st->get_Offset(&ofs));
		ofs += s2;
		RUN(st->put_Offset(&ofs));
	}
	return S_FALSE;
}
STDMETHODIMP Site::OnUIDeactivate(BOOL fUndoable) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnUIDeactivate);
	if(m_state==MCCS_UIACTIVE) {
		DWORD oldState = m_state;
		m_state = MCCS_ACTIVE;
		stateEvent(oldState,MCCS_ACTIVE);
		// if change was initiated by something besides Montage shutting down, change intent
		if(m_cont->m_state==MCCS_UIACTIVE && m_intent==MCCS_UIACTIVE)
			m_intent = MCCS_ACTIVE;
	}
	if(CComQIPtr<IOleInPlaceObject>(m_pControl)==m_cont->m_pUIActiveObject) {
		m_cont->m_pUIActiveObject.Release();
		if(m_cont->m_state==MCCS_UIACTIVE)
			RUN(m_cont->installUI()); // was a drop of UI from contained to Montage.
		// (else Montage is deactivating with something active within it.)
	}
	return S_OK; // was a switch from one ui-active contained to another
}
STDMETHODIMP Site::OnInPlaceDeactivate() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnInPlaceDeactivate);
	return OnInPlaceDeactivateEx(false);
}
STDMETHODIMP Site::DiscardUndoState() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,DiscardUndoState);
	return S_OK;
}
STDMETHODIMP Site::DeactivateAndUndo() {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,DeactivateAndUndo);
	CComQIPtr<IOleInPlaceObject> pIPObject = m_pControl;
	if(!pIPObject)
		ERR(E_UNEXPECTED);
	pIPObject->UIDeactivate();
	return S_OK;
}
STDMETHODIMP Site::OnPosRectChange(LPCRECT lprcPosRect) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnPosRectChange);
	if(m_amSettingExtent) // HACK: to avoid control SetExtent-OnPosRectChange loop & crash
		return put_Rect(lprcPosRect);
	MCC_POS_INFO mpi;
	mpi.postype = MCC_POS_WINDOWRECT;
	mpi.rc = *lprcPosRect;
	RUN(fireSiteEvent(this,fireMove(&mpi)));  // ONLY fire in core Montage
	return S_OK;
}

//(IOleInPlaceSiteEx)
STDMETHODIMP Site::OnInPlaceActivateEx(
     BOOL *pfNoRedraw,
     DWORD dwFlags) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnInPlaceActivateEx);
	if(dwFlags&ACTIVATE_WINDOWLESS) {
		m_pWindowless = m_pControl;
		if(!m_pWindowless)
			ERR(E_FAIL);
	}
	if(pfNoRedraw)
		*pfNoRedraw = TRUE;
	// if contained object is initiating a state change (probably because someone
	// called its IOleObject::DoVerb), make sure we reflect that.
	if(m_intent<MCCS_ACTIVE) {
		DWORD oldState = m_state, oldIntent = m_intent;
		m_state = m_intent = MCCS_ACTIVE;
		// send out the request, even though the deed is done
		// (if they call put_Intent back it will have no effect)
		HRESULT hr;
		if(FAILED(hr = fireSiteEvent(this,fireIntend(MCCS_ACTIVE)))) {
			m_intent = oldIntent;
			return hr;
		}
		stateEvent(oldState,m_state);
	}
	return S_OK;
}
STDMETHODIMP Site::OnInPlaceDeactivateEx(BOOL fNoRedraw) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnInPlaceDeactivateEx);
	if(m_pWindowless)
		m_pWindowless.Release();
	if(m_state>MCCS_OPEN) {
		DWORD lastState = m_state;
		m_state = MCCS_SHOWN;
		stateEvent(lastState,MCCS_SHOWN);
		// if change was initiated by something besides Montage shutting down, change intent
		if(m_cont->m_state>MCCS_SHOWN && m_intent>MCCS_SHOWN)
			m_intent = MCCS_SHOWN;
        if(m_cont->m_pFocusObject==this) {
            m_cont->m_pFocusObject = 0;
            m_cont->changeFocus();
        }
	}
	return S_OK;
}
STDMETHODIMP Site::RequestUIActivate(void) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,RequestUIActivate);
	return S_OK;
}

//(IOleInPlaceSiteWindowless)
STDMETHODIMP Site::CanWindowlessActivate(void) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,CanWindowlessActivate);
	// if container doesn't own a window it can't act on behalf of objects who don't
	if(!m_cont->m_ownwindow)
		return S_FALSE;
	return S_OK;
}
STDMETHODIMP Site::GetCapture(void) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,GetCapture);
	return (m_cont->m_pCaptureObject==this)?S_OK:S_FALSE;
}
STDMETHODIMP Site::SetCapture(BOOL fCapture) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,SetCapture);
	if(!m_pWindowless) // just so we don't have to check this later.
		ERR(E_FAIL);
	if(fCapture) {
		// if we haven't already captured for someone, grab capture
		if(!m_cont->m_pCaptureObject)
			::SetCapture(m_cont->m_hWnd);
		m_cont->m_pCaptureObject = (site*)this;
	}
	else if(m_cont->m_pCaptureObject==(site*)this) {
		m_cont->m_pCaptureObject = 0;
		::ReleaseCapture();
	}
	return S_OK;
}
STDMETHODIMP Site::GetFocus(void) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,GetFocus);
	return (m_cont->m_pFocusObject==this)?S_OK:S_FALSE;
}
STDMETHODIMP Site::SetFocus(BOOL fFocus) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,SetFocus);
	if(!m_pWindowless) // just so we don't have to check this later.
		ERR(E_FAIL);
	if(fFocus)
		m_cont->m_pFocusObject = (site*)this;
	else if(m_cont->m_pFocusObject==(site*)this)
		m_cont->m_pFocusObject = 0;
    m_cont->changeFocus();
	return S_OK;
}
STDMETHODIMP Site::GetDC(
     LPCRECT pRect,
     DWORD grfFlags,
     HDC *phDC) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,GetDC);
	HDC hdc = ::GetDC(m_cont->m_hWnd);
	if(grfFlags&OLEDC_NODRAW) {
		*phDC = hdc;
		return S_OK;
	}
	// clip DC to intersection of pRect and site bounds
	RECT rcClipped;
	if(pRect)
		IntersectRect(&rcClipped,pRect,&m_windRect);
	else
		rcClipped = m_windRect;
	/*
	HRGN clip = CreateRectRgnIndirect(&rcClipped);
	SelectClipRgn(*phDC,clip);
	DeleteObject(clip);
	*/
	if(grfFlags&OLEDC_OFFSCREEN) { // buffer
		if(m_cont->m_currentDC) // we only have one buffer
			return E_FAIL; // OLE_E_NESTEDPAINT not defined!!
		hdc = m_cont->startBuffer(hdc,&rcClipped,!(grfFlags&OLEDC_PAINTBKGND)); // only bother to copy into buffer if not painting bkgnd
	}
	if(grfFlags&OLEDC_PAINTBKGND) { // draw everything behind
		m_cont->erase(hdc,&rcClipped);
		RECT rcM = m_cont->rect();
		RUN(m_cont->draw(posBounds(),hdc,(RECTL*)&rcM,(RECTL*)&rcM,(RECTL*)&rcClipped,(site*)this,Container::upTo));
	}
	*phDC = hdc;
	CHECK(SaveDC(hdc));
	return S_OK;
}
STDMETHODIMP Site::ReleaseDC(HDC hDC) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,ReleaseDC);
	RestoreDC(hDC,-1);
	int rop2 = GetROP2(hDC);
	RECT rcM = m_cont->rect();
	RECT clipBox;
	GetClipBox(hDC,&clipBox);
	// draw everything in front
	RUN(m_cont->draw(posBounds(),hDC,(RECTL*)&rcM,(RECTL*)&rcM,(RECTL*)&clipBox,(site*)this,Container::frontOf));
	if(hDC==m_cont->m_bufferDC) // OLEDC_OFFSCREEN was specified
		hDC = m_cont->blitBuffer();
	::ReleaseDC(m_cont->m_hWnd,hDC);
	return S_OK;
}
STDMETHODIMP Site::InvalidateRect(LPCRECT pRect,BOOL fErase) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,InvalidateRect);
	RECT rc;
	if(pRect) {
		if(!IntersectRect(&rc,pRect,&m_windRect))
			return S_OK;
	}
	else rc = m_windRect;
	m_cont->_InvalidateRect(&rc,fErase);
	return S_OK;
}
STDMETHODIMP Site::InvalidateRgn(
     HRGN hRGN,
     BOOL fErase) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,InvalidateRgn);
	HRGN region = CreateRectRgnIndirect(&m_windRect);
	switch(CombineRgn(region,region,hRGN,RGN_AND)) {
	default:
		::InvalidateRgn(m_cont->m_hWnd,region,fErase);
	case NULLREGION:
		break;
	case ERROR:
		ERR(E_UNEXPECTED);
	}
	DeleteObject(region);
	return S_OK;;
}
STDMETHODIMP Site::ScrollRect(
     INT dx,
     INT dy,
     LPCRECT pRectScroll,
     LPCRECT pRectClip) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,ScrollRect);
	ERR(E_NOTIMPL); // sadly enough.
}
STDMETHODIMP Site::AdjustRect(
     LPRECT prc) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,AdjustRect);
	IntersectRect(prc,&m_windRect,prc);
	return S_OK;
}
STDMETHODIMP Site::OnDefWindowMessage(
     UINT msg,
     WPARAM wParam,
     LPARAM lParam,
     LRESULT *plResult) {
	IMETHTRACE(Site,IOleInPlaceSiteWindowless,OnDefWindowMessage);
	if(keymess(msg)||mousemess(msg)) {
		*plResult = DefWindowProc(m_cont->m_hWnd,msg,wParam,lParam);
		return S_OK;
	}
	return S_FALSE;
}
