/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IOleObject for Container
#include "stdafx.h"
#include "Container.h"

STDMETHODIMP Container::SetClientSite(
     IOleClientSite __RPC_FAR *pClientSite) {
	IMETHTRACE(Container,IOleObject,SetClientSite);
	m_pSite = pClientSite;
	RUN(chstate(MCCS_SHOWN)); // just a guess?
	return S_OK;
}

STDMETHODIMP Container::GetClientSite(
     IOleClientSite __RPC_FAR *__RPC_FAR *ppClientSite) {
	IMETHTRACE(Container,IOleObject,GetClientSite);
    if(!m_pSite)
        return *ppClientSite = 0, S_OK;
    else
	    return m_pSite.QueryInterface(ppClientSite);
}

STDMETHODIMP Container::SetHostNames(
     LPCOLESTR szContainerApp,
     LPCOLESTR szContainerObj) {
	IMETHTRACE(Container,IOleObject,SetHostNames);
	return S_OK;
}

STDMETHODIMP Container::Close(DWORD dwSaveOption) {
	IMETHTRACE(Container,IOleObject,Close);
    if(!m_pSite)  // sometimes container calls Close() twice
        return S_FALSE;
    CComQIPtr<IOleClientSite> ocs = m_pSite;
    if(!ocs)
        ERR(E_UNEXPECTED);
	RUN(chstate(MCCS_SHOWN)); // running and loaded states are the same for inproc
	if(dwSaveOption!=OLECLOSE_NOSAVE)
		RUN(ocs->SaveObject());
	if(m_oledvise)
		RUN(m_oledvise->SendOnClose());
	// Std documentation is for shit!  I had no idea I had to release these pointers until
	// I read the MFC code!
	m_pViewSink.Release();
	m_pSite.Release();
	RUN(HandsOffStorage()); // excel doesn't call this explicitly
	return S_OK;
}

STDMETHODIMP Container::SetMoniker(
     DWORD dwWhichMoniker,
     IMoniker __RPC_FAR *pmk) {
	IMETHTRACE(Container,IOleObject,SetMoniker);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Container::GetMoniker(
     DWORD dwAssign,
     DWORD dwWhichMoniker,
     IMoniker __RPC_FAR *__RPC_FAR *ppmk) {
	IMETHTRACE(Container,IOleObject,GetMoniker);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Container::InitFromData(
     IDataObject __RPC_FAR *pDataObject,
     BOOL fCreation,
     DWORD dwReserved) {
	IMETHTRACE(Container,IOleObject,InitFromData);
	ERR(E_NOTIMPL);
}

STDMETHODIMP Container::GetClipboardData(
     DWORD dwReserved,
     IDataObject __RPC_FAR *__RPC_FAR *ppDataObject) {
	IMETHTRACE(Container,IOleObject,GetClipboardData);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Container::DoVerb(
     LONG iVerb,
     LPMSG lpmsg,
     IOleClientSite __RPC_FAR *pActiveSite,
     LONG lindex,
     HWND hwndParent,
     LPCRECT lprcPosRect) {
	IMETHTRACE(Container,IOleObject,DoVerb);
	if(!pActiveSite)
		ERR(E_FAIL);
	DWORD intent;
	switch(iVerb) {
	// DoVerb itself is a hack!
	// Then atl defines OLEIVERB_SHOW the same as OLEIVERB_INPLACEACTIVATE,
	// and probably for some good reason.  it then devises the QI IDispatch hack
	// below to work around Word.  but this hack doesn't work with Visio!
	// what am I breaking now?
	case OLEIVERB_SHOW:
	case OLEIVERB_PRIMARY:
	case OLEIVERB_UIACTIVATE:
		intent = MCCS_UIACTIVE;
		break;
	case OLEIVERB_INPLACEACTIVATE:
		intent = MCCS_ACTIVE;
		break;
	case OLEIVERB_HIDE:
		intent = MCCS_SHOWN;
		break;
	case OLEIVERB_OPEN:
	default:
		ERR(E_NOTIMPL);
	}
	/*
	// (ATL hack that goes with moving OLEIVERB_SHOW to MCCS_ACTIVE)
	if(intent==MCCS_ACTIVE && !CComQIPtr<IDispatch>(m_pSite)) {
		// according to ATL, older containers (WORD) want SHOW to mean INPLACEACTIVATE
		// and you can detect old containers by looking for ambiance.
		intent = MCCS_UIACTIVE;
	}
	*/
	// special case for docobject: tell container to switch to that track.  v. weird.
	CComQIPtr<IOleDocumentSite> dsite(m_pSite);
	if(dsite && intent>MCCS_SHOWN) {
		RUN(dsite->ActivateMe(0));
		return S_OK;
	}
	// anomaly: the MCCS_SHOWN->MCCS_ACTIVE transition requires the inplacesite passed
	// in here.  all other transitions require no extra info!
	if(intent>MCCS_OPEN && m_state<MCCS_ACTIVE) {
		m_pInPlaceSite.Release(); // not always?
		RUN(pActiveSite->QueryInterface(riidret(&m_pInPlaceSite)));
			// could activate in a separate window if this fails, but no.
	}
	RUNRET(chstate(intent));
    return S_OK;
}


STDMETHODIMP Container::EnumVerbs(
     IEnumOLEVERB __RPC_FAR *__RPC_FAR *ppEnumOleVerb) {
	IMETHTRACE(Container,IOleObject,EnumVerbs);
	return LOOK(OleRegEnumVerbs(GetObjectCLSID(),ppEnumOleVerb));
}

STDMETHODIMP Container::Update( void) {
	IMETHTRACE(Container,IOleObject,Update);
	return S_OK;
}

STDMETHODIMP Container::IsUpToDate( void) {
	IMETHTRACE(Container,IOleObject,IsUpToDate);
	return S_OK;
}


STDMETHODIMP Container::GetUserClassID(
     CLSID __RPC_FAR *pClsid) {
	*pClsid = m_clsid;
	IMETHTRACE(Container,IOleObject,GetUserClassID);
	return S_OK;
}

const LPOLESTR g_szType = L"Montage Control Container";
STDMETHODIMP Container::GetUserType(
     DWORD dwFormOfType,
     LPOLESTR __RPC_FAR *pszUserType) {
	METHTRACE(Container,GetUserType);
	return LOOK(OleRegGetUserType(GetObjectCLSID(),dwFormOfType,pszUserType));
}
bool Container::setExtent(SIZEL s) {
    METHTRACE(Container,setExtent);
	if(m_extent!=s) { // yep another silly stack fault
		m_extent = s;
        // m_rect is simulated when not active
        if(m_state<MCCS_ACTIVE) {
            RUN(fakeRectFromExtent());
            sizeEvent();
        }
		extentEvent();
		return true;
	}
	else
		return false;
}
STDMETHODIMP Container::SetExtent(
     DWORD dwDrawAspect,
     SIZEL __RPC_FAR *psizel) {
	IMETHTRACE(Container,IOleObject,SetExtent);
	if(setExtent(*psizel))
		RUN(dirty()); // setextent causes data change so that word resizes Montage
	return S_OK;
}
HRESULT Container::getExtentFromRenderer(SIZEL *ret) {
    METHTRACE(Container,getExtentFromRenderer);
	RECTL bounds;
	RUNRET(m_renderer->get_Bounds(&bounds));
	*ret = siz(bounds)|m_minExtent;
	m_extent = *ret; // remember, in case m_autoSize gets turned off
	return S_OK;
}
STDMETHODIMP Container::GetExtent(
     DWORD dwDrawAspect,
     SIZEL __RPC_FAR *psizel) {
	IMETHTRACE(Container,IOleObject,GetExtent);
	if(m_autoSize && m_renderer
		&& !CComQIPtr<IOleDocumentSite>(m_pSite) && !m_intoWnd) // never use natural size if size is based on screen area
		if(SUCCEEDED(getExtentFromRenderer(psizel)))
			return S_OK;
	*psizel = m_extent;
	return S_OK;
}

STDMETHODIMP Container::Advise(
     IAdviseSink __RPC_FAR *pAdvSink,
     DWORD __RPC_FAR *pdwConnection) {
	METHTRACE(Container,Advise);
	if(!m_oledvise)
		CreateOleAdviseHolder(&m_oledvise);
	return m_oledvise->Advise(pAdvSink,pdwConnection);
}

STDMETHODIMP Container::Unadvise(
     DWORD dwConnection) {
	IMETHTRACE(Container,IOleObject,Unadvise);
	return m_oledvise->Unadvise(dwConnection);
}

STDMETHODIMP Container::EnumAdvise(
     IEnumSTATDATA __RPC_FAR *__RPC_FAR *ppenumAdvise) {
	IMETHTRACE(Container,IOleObject,EnumAdvise);
	return m_oledvise->EnumAdvise(ppenumAdvise);
}

STDMETHODIMP Container::GetMiscStatus(
     DWORD dwAspect,
     DWORD __RPC_FAR *pdwStatus) {
	IMETHTRACE(Container,IOleObject,GetMiscStatus);
	return LOOK(OleRegGetMiscStatus(GetObjectCLSID(),dwAspect,pdwStatus));
}

STDMETHODIMP Container::SetColorScheme(
     LOGPALETTE __RPC_FAR *pLogpal) {
	IMETHTRACE(Container,IOleObject,SetColorScheme);
	ERR(E_NOTIMPL);
}
