/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


template<class ISubject, class IImpl,class DISubject,class DIImpl>
class Ordering : public IImpl, public IDispatchImpl<DIImpl,&__uuidof(DIImpl)> {
	typedef map<long,ISubject *> order;
	order m_order;
public:
	Ordering();
	~Ordering();
	STDMETHODIMP Place(long z, ISubject *s);
	STDMETHODIMP Place(long z, DISubject *s);
};
template<class ISubject, class IImpl,class DISubject,class DIImpl>
Ordering<ISubject,IImpl,DISubject,DIImpl>::Ordering() {
	CONSTRUCTORTRACE(Ordering);
}
template<class ISubject, class IImpl,class DISubject,class DIImpl>
Ordering<ISubject,IImpl,DISubject,DIImpl>::~Ordering() {
	DESTRUCTORTRACE(Ordering);
	// WRONG WRONG WRONG but i can't figure out why you end up with extra refcounts this way!!!
	/*
	for(order::iterator i = m_order.begin();i!=m_order.end();i++)
		i->second->Release();
		*/
}
template<class ISubject, class IImpl,class DISubject,class DIImpl>
STDMETHODIMP Ordering<ISubject,IImpl,DISubject,DIImpl>::Place(long z,ISubject *s) {
	METHTRACE(Ordering,Place);
	order::iterator i;
	for(i = m_order.begin(); i!=m_order.end(); ++i)
		_ASSERTE(i->second!=s);
	// put it just behind the last item that has a lower rank, or if none does, put it in front
	order::iterator lb = m_order.lower_bound(z);
	// tolerant to removed sites
	bool inserted = false;
	i = lb;
	while(i!=m_order.begin()) {
		--i;
		if(SUCCEEDED(s->PutBehind(i->second))) {
			inserted = true;
			break;
		}
	}
	if(!inserted)
		RUN(s->PutBehind(0));
	m_order.insert(i,make_pair_first_const(z,s));
	//s->AddRef();
	return S_OK;
}
template<class ISubject, class IImpl,class DISubject,class DIImpl>
STDMETHODIMP Ordering<ISubject,IImpl,DISubject,DIImpl>::Place(long z,DISubject *s) {
    METHTRACE(Ordering,Place);
	CComPtr<ISubject> s2;
	if(s)
		RUNRET(s->QueryInterface(riidret(&s2)));
	return Place(z,s2);
}
class ATL_NO_VTABLE SiteOrdering :
	public CComCoClass<SiteOrdering,&CLSID_MCCSiteOrdering>,
	public CComObjectRoot,
	public Ordering<IMCCSite,IMCCSiteOrdering,DIMCCSite,DIMCCSiteOrdering>,
	public IPersist
{
public:
	SiteOrdering();
	~SiteOrdering();
	REGISTER_GENERIC(L"SiteOrdering")
BEGIN_COM_MAP(SiteOrdering)
	COM_INTERFACE_ENTRY(IMCCSiteOrdering)
	COM_INTERFACE_ENTRY(DIMCCSiteOrdering)
	COM_INTERFACE_ENTRY(IPersist)
END_COM_MAP()
// IPersist//Stream
	STDMETHODIMP GetClassID(CLSID *pClassID);
};
class ATL_NO_VTABLE LayerOrdering :
	public CComCoClass<LayerOrdering,&CLSID_MCCLayerOrdering>,
	public CComObjectRoot,
	public Ordering<IMCCLayer,IMCCLayerOrdering,DIMCCLayer,DIMCCLayerOrdering>,
	public IPersist
{
public:
	LayerOrdering();
	~LayerOrdering();
	REGISTER_GENERIC(L"LayerOrdering")
BEGIN_COM_MAP(LayerOrdering)
	COM_INTERFACE_ENTRY(IMCCLayerOrdering)
	COM_INTERFACE_ENTRY(DIMCCLayerOrdering)
	COM_INTERFACE_ENTRY(IPersist)
END_COM_MAP()
// IPersist
	STDMETHODIMP GetClassID(CLSID *pClassID);
};
