/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <stdafx.h>
#include "Pane.h"

Pane::Pane() : m_isVert(false),m_rect(rect(0,0,0,0)),m_border(rect(0,0,0,0)),m_spacing(0),m_eikooc(0),m_expander(-1)
{}

HRESULT Pane::recalc() {
	RECT r2 = {m_rect.left+m_border.left,m_rect.top+m_border.top,m_rect.right-m_border.right,m_rect.bottom-m_border.bottom};
	int q = m_isVert?r2.top:r2.left,
        total = 0,
        denom = 0;
	for(unsigned i = 0; i<m_panes.size(); ++i)
        total += m_panes[i].size,
        denom += m_panes[i].xtra;
    int extra = (m_isVert?hei(r2):wid(r2)) - total;
    if(extra<0)
        extra = 0;
    if(denom==0)
        denom = 1;  // fine, no one gets the extra space!
    int bonus = extra/denom,
        smidgen = extra%denom;
    for(unsigned i = 0; i<m_panes.size(); ++i) {
        int k = m_panes[i].size;
        if(m_panes[i].xtra) {
            k += bonus*m_panes[i].xtra;
            k += smidgen; // add the smidgen to the first cuz that's easiest
            smidgen = 0;
        }
        RECT r = m_isVert?rect(r2.left,q,r2.right,q+k):
            rect(q,r2.top,q+k,r2.bottom);
		RUN(m_panes[i].pane->put_Rect(&r));
		q += k;
	}
	return S_OK;
}
STDMETHODIMP Pane::put_Rect(RECT *val) {
	m_rect = *val;
    if(m_content)
		RUN(m_content->put_Rect(&m_rect));
	RUN(recalc());
	return S_OK;
}
STDMETHODIMP Pane::get_Rect(RECT *ret) {
	*ret = m_rect;
	return S_OK;
}
STDMETHODIMP Pane::put_IsVert(VARIANT_BOOL val) {
	m_isVert = val;
	return S_OK;
}
STDMETHODIMP Pane::get_IsVert(VARIANT_BOOL *ret) {
	*ret = m_isVert;
	return S_OK;
}
STDMETHODIMP Pane::put_PaneSpacing(int val) {
	m_spacing = val;
	return S_OK;
}
STDMETHODIMP Pane::get_PaneSpacing(int *ret) {
	*ret = m_spacing;
	return S_OK;
}
STDMETHODIMP Pane::AddPane(int pos,IMCCPane *val) {
	if(pos < 0 || pos >= (int)m_panes.size())
		pos = m_panes.size();
	val->AddRef();
    PaneInfo inf;
    inf.pane = val;
    inf.size = inf.xtra = 0;
	m_panes.insert(m_panes.begin()+pos,inf);
	RUN(recalc());
	return S_OK;
}
STDMETHODIMP Pane::RemovePane(int pos) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
	IMCCPane *p = m_panes[pos].pane;
	m_panes.erase(m_panes.begin()+pos);
	p->Release();
	RUN(recalc());
	return S_OK;
}
STDMETHODIMP Pane::get_PaneCount(int *ret) {
	*ret = m_panes.size();
	return S_OK;
}
STDMETHODIMP Pane::get_Pane(int pos,IMCCPane **ret) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
	(*ret = m_panes[pos].pane)->AddRef();
	return S_OK;
}
STDMETHODIMP Pane::put_PaneSize(int pos,int val) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
	m_panes[pos].size = val;
	RUN(recalc());
	return S_OK;
}
STDMETHODIMP Pane::get_PaneSize(int pos,int *ret) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
	*ret = m_panes[pos].size;
	return S_OK;
}
STDMETHODIMP Pane::put_PaneXtra(int pos,int val) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
    m_panes[pos].xtra = val;
    RUN(recalc());
	return S_OK;
}
STDMETHODIMP Pane::get_PaneXtra(int pos,int *ret) {
	if(pos<0 || pos>=(int)m_panes.size())
		ERR(E_FAIL);
    *ret = m_panes[pos].xtra;
	return S_OK;
}
STDMETHODIMP Pane::put_Border(RECT val) {
	m_border = val;
	return S_OK;
}
STDMETHODIMP Pane::get_Border(RECT *ret) {
	*ret = m_border;
	return S_OK;
}

STDMETHODIMP Pane::put_Content(IMCCSite *val) {
    if(m_content = val)
        RUN(m_content->put_Rect(&m_rect));
	return S_OK;
}
STDMETHODIMP Pane::get_Content(IMCCSite **ret) {
	if(*ret = m_content)
		(*ret)->AddRef();
	return S_OK;
}
STDMETHODIMP Pane::get_Montage(IMontageContainer **ret) {
    if(!m_refm)
        *ret = 0;
    else {
        CComPtr<IUnknown> um;
        RUN(m_refm->get_Montage(&um));
        RUN(um.QueryInterface(ret));
    }
    return S_OK;
}
STDMETHODIMP Pane::put_Montage(IMontageContainer *val) {
    if(!val) {
        if(m_refm) {
            CComPtr<IUnknown> um;
            RUN(m_refm->get_Montage(&um));
            RUN(AtlUnadvise(um,IID_IMontageContainerEvents,m_eikooc));
            m_refm.Release();
        }
    }
    else {
        RUN(val->get_RefObject(&m_refm));
        RUN(AtlAdvise(val,GetUnknown(),IID_IMontageContainerEvents,&m_eikooc));
    }
	return S_OK;
}
STDMETHODIMP Pane::DoZ(IMCCSite *back) {
    if(m_content) {
        RUN(m_content->PutBefore(back));
        back = m_content;
    }
    for(vector<PaneInfo>::iterator pi = m_panes.begin(); pi!=m_panes.end(); ++pi)
        RUN(pi->pane->DoZ(back));
	return S_OK;
}
// IMontageContainerEvents
STDMETHODIMP Pane::Shutdown() {
	return S_FALSE;
}
STDMETHODIMP Pane::Resize(SIZE s) {
    _ASSERTE(m_refm);
    CComPtr<IUnknown> um;
    RUN(m_refm->get_Montage(&um));
	CComPtr<IMontageContainer> mc;
	RUN(um.QueryInterface(&mc));
	RECT extra;
	RUN(mc->get_ExtraArea(&extra));
	RECT rc = rect(s)-extra;
	RUN(put_Rect(&rc));
	return S_OK;
}
STDMETHODIMP Pane::NewExtent(SIZEL) {
	return S_FALSE;
}
STDMETHODIMP Pane::Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
	return S_FALSE;
}
STDMETHODIMP Pane::NewTransform(IMCCTransform *) {
	return S_FALSE;
}
STDMETHODIMP Pane::GroupOp(BOOL) {
	return S_FALSE;
}
STDMETHODIMP Pane::SiteDeleted(IMCCSite*) {
	return S_FALSE;
}
STDMETHODIMP Pane::LayerDeleted(IMCCLayer*) {
	return S_FALSE;
}
STDMETHODIMP Pane::Shifting(long oldState,long newState) {
	return S_FALSE;
}
STDMETHODIMP Pane::ShowArea(LPCRECTL r) {
	return S_FALSE;
}
