/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "PaneTitle.h"
#include "RenderPolicy.h"

PaneTitle::PaneTitle() : m_titleHeight(0),m_borderSize(0),m_active(false) {
	m_activeBorderColor = RGB(16,16,255);
	m_inactiveBorderColor = GetSysColor(COLOR_3DDKSHADOW);
	m_activeTitleColor = RGB(64,110,189);
	m_inactiveTitleColor = RGB(100,128,120);
	m_activeTextColor = RGB(255,255,255);
    m_inactiveTextColor = RGB(187,199,193);
	m_activeBackgroundColor = m_inactiveBackgroundColor = GetSysColor(COLOR_3DFACE);
}

HRESULT PaneTitle::OnDrawAdvanced(ATL_DRAWINFO& di) {
    METHTRACE(ColorBox,OnDrawAdvanced);
	COLORREF borderColor,textColor,titleColor,backgroundColor;
	if(m_active) {
		RUN(OleTranslateColor(m_activeBorderColor,0,&borderColor));
		RUN(OleTranslateColor(m_activeTitleColor,0,&titleColor));
		RUN(OleTranslateColor(m_activeTextColor,0,&textColor));
		RUN(OleTranslateColor(m_activeBackgroundColor,0,&backgroundColor));
	}
	else {
		RUN(OleTranslateColor(m_inactiveBorderColor,0,&borderColor));
		RUN(OleTranslateColor(m_inactiveTitleColor,0,&titleColor));
		RUN(OleTranslateColor(m_inactiveTextColor,0,&textColor));
		RUN(OleTranslateColor(m_inactiveBackgroundColor,0,&backgroundColor));
	}
	HDC hdc = di.hdcDraw;
	HBRUSH borderBrush = CreateSolidBrush(borderColor);
	HRGN borderrgn = CreateRectRgnIndirect((RECT*)di.prcBounds);
	RECT rcCont = *(RECT*)di.prcBounds;
	CHECK(InflateRect(&rcCont,-m_borderSize,-m_borderSize));
	HRGN contrgn = CreateRectRgnIndirect(&rcCont);
	CHECK(CombineRgn(borderrgn,borderrgn,contrgn,RGN_DIFF));
	CHECK(FillRgn(hdc,borderrgn,borderBrush));
	CHECK(DeleteObject(borderrgn));
	CHECK(DeleteObject(borderBrush));
	HBRUSH backgroundBrush = CreateSolidBrush(backgroundColor);
	RECT rcTitle = rcCont;
    rcTitle.bottom = rcTitle.top + m_titleHeight;
	HRGN titlergn = CreateRectRgnIndirect(&rcTitle);
	CHECK(CombineRgn(contrgn,contrgn,titlergn,RGN_DIFF));
	CHECK(FillRgn(hdc,contrgn,backgroundBrush));
	CHECK(DeleteObject(contrgn));
	CHECK(DeleteObject(backgroundBrush));
	HBRUSH titleBrush = CreateSolidBrush(titleColor);
	CHECK(FillRgn(hdc,titlergn,titleBrush));
	CHECK(DeleteObject(titlergn));
	CHECK(DeleteObject(titleBrush));
	if(m_font && m_text.Length()) {
    	SetTextColor(hdc,textColor);
		// Montage fonts are kept in LOMETRIC
	    CComPtr<IMontageContainer> mc;
	    RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
	    CComPtr<IMCCTransform> t;
	    RUN(mc->get_Transform(&t));
	    SIZEL fivecm = {5000,5000},
		    r5cm = RenderPolicy::inSize(fivecm),
		    p5cm;
	    RUNRET(t->C2VS(fivecm,&p5cm));
		CHECK(SaveDC(hdc));
	    CHECK(SetMapMode(hdc,MM_ANISOTROPIC));
	    CHECK(SetViewportOrgEx(hdc,rcTitle.left+1,rcTitle.top,0));
	    CHECK(SetWindowExtEx(hdc,r5cm.cx,r5cm.cy,0));
	    CHECK(SetViewportExtEx(hdc,p5cm.cx,p5cm.cy,0));
		HFONT hf;
		RUN(m_font->get_hFont(&hf));
		HFONT oldF = (HFONT)SelectObject(hdc,hf);
		USES_CONVERSION;
		RECT rc2 = {0,200,1000,0};
        CHECK(SetBkColor(hdc,titleColor));
		CHECK(ExtTextOut(hdc,0,0,0,&rc2,W2A(m_text),m_text.Length(),0));
		//CHECK(DrawText(hdc,W2A(m_text),m_text.Length(),&rc2,DT_LEFT|DT_VCENTER));
		CHECK(RestoreDC(hdc,-1));
	}
	return S_OK;
}
LRESULT PaneTitle::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    if(m_content)
        RUN(m_content->put_Intent(MCCS_UIACTIVE));
	return 0;
}

// IMCCPaneTitle
STDMETHODIMP PaneTitle::get_Text(BSTR *ret) {
	m_text.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP PaneTitle::put_Text(BSTR val) {
	m_text = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_TitleHeight(int *ret) {
    *ret = m_titleHeight;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_TitleHeight(int val) {
    m_titleHeight = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_BorderSize(int *ret) {
	*ret = m_borderSize;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_BorderSize(int val) {
	m_borderSize = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_InactiveBorderColor(OLE_COLOR *ret) {
	*ret = m_inactiveBorderColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_InactiveBorderColor(OLE_COLOR val) {
	m_inactiveBorderColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_ActiveBorderColor(OLE_COLOR *ret) {
	*ret = m_activeBorderColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_ActiveBorderColor(OLE_COLOR val) {
	m_activeBorderColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_InactiveTitleColor(OLE_COLOR *ret) {
	*ret = m_inactiveTitleColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_InactiveTitleColor(OLE_COLOR val) {
	m_inactiveTitleColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_ActiveTitleColor(OLE_COLOR *ret) {
	*ret = m_activeTitleColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_ActiveTitleColor(OLE_COLOR val) {
	m_activeTitleColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_InactiveTextColor(OLE_COLOR *ret) {
	*ret = m_inactiveTextColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_InactiveTextColor(OLE_COLOR val) {
	m_inactiveTextColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_ActiveTextColor(OLE_COLOR *ret) {
	*ret = m_activeTextColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_ActiveTextColor(OLE_COLOR val) {
	m_activeTextColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_InactiveBackgroundColor(OLE_COLOR *ret) {
	*ret = m_inactiveBackgroundColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_InactiveBackgroundColor(OLE_COLOR val) {
	m_inactiveBackgroundColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_ActiveBackgroundColor(OLE_COLOR *ret) {
	*ret = m_activeBackgroundColor;
	return S_OK;
}
STDMETHODIMP PaneTitle::put_ActiveBackgroundColor(OLE_COLOR val) {
	m_activeBackgroundColor = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_Font(IFont **ret) {
	m_font.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP PaneTitle::put_Font(IFont *val) {
	m_font = val;
	return S_OK;
}
STDMETHODIMP PaneTitle::get_Content(IMCCSite **ret) {
	m_content.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP PaneTitle::put_Content(IMCCSite *val) {
    if(m_content)
        RUN(AtlUnadvise(m_content,IID_IMCCSiteEvents,m_itsIt));
	if(m_content = val)
        RUN(AtlAdvise(m_content,GetUnknown(),IID_IMCCSiteEvents,&m_itsIt));
	return S_OK;
}
// IMCCSiteEvents
STDMETHODIMP PaneTitle::Moving(LPCRECT winRect) {
	return S_OK;
}
STDMETHODIMP PaneTitle::Shifting(long oldState, long newState) {
	m_active = newState==MCCS_UIACTIVE;
    if(m_hWnd)
	    InvalidateRect(0);
	return S_OK;
}
STDMETHODIMP PaneTitle::ZChanged() {
	return S_OK;
}
STDMETHODIMP PaneTitle::Layering(IMCCLayer *newLayer) {
	return S_OK;
}
