/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "PasteSetUnpacker.h"
#include "createAffixMoniker.h"

PasteSetUnpacker::PasteSetUnpacker() {
	CONSTRUCTORTRACE(PasteSetUnpacker);
}
PasteSetUnpacker::~PasteSetUnpacker() {
	DESTRUCTORTRACE(PasteSetUnpacker);
}

STDMETHODIMP PasteSetUnpacker::Initialize(IMontageContainer *mc, IMCCDataObjectReader *reader) {
	METHTRACE(PasteSetUnpacker,Initialize);
	RUN(mc->get_RefObject(&m_mref));
	RUN(reader->RegisterUnpacker(g_clipformats.cc,true,this));
	RUN(reader->RegisterUnpacker(g_clipformats.cc,false,this));
	return S_OK;
}
// DIMCCPasteSetUnpacker
STDMETHODIMP PasteSetUnpacker::Initialize(IUnknown *mc, IUnknown *reader) {
    METHTRACE(PasteSetUnpacker,Initialize);
	CComPtr<IMontageContainer> mc2;
	if(mc)
		RUNRET(mc->QueryInterface(riidret(&mc2)));
	CComPtr<IMCCDataObjectReader> r2;
	if(reader)
		RUNRET(reader->QueryInterface(riidret(&r2)));
	return Initialize(mc2,r2);
}

// IMCCDataObjectUnpacker
STDMETHODIMP PasteSetUnpacker::UnpackOne(IDataObject *data, FORMATETC *fetc, LPOLESTR baseName, IItemSet *isetPaste,IMCCSitePersistor **ret) {
    METHTRACE(PasteSetUnpacker,UnpackOne);
	if(baseName) // renaming doesn't work
		ERR(E_NOTIMPL);
	CComPtr<IGroup> group;
	RUN(mc()->GetMainGroup(riidret(&group)));
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));

	// see if this name's already been used & return existent if so
	CComPtr<IMoniker> spName;
	if(baseName) {
		RUN(createAffixMoniker(baseName,L"Site",&spName));
		CComPtr<IMoniker> mkG;
		RUN(FoolishCreatePointerMoniker(group,&mkG));
		CComPtr<IMCCSitePersistor> spExists;
		if(SUCCEEDED(spName->BindToObject(bctx,mkG,riidret(&spExists)))) {
			if(ret)
				(*ret = spExists)->AddRef();
			return S_FALSE;
		}
	}
	// paste "Data" into a temporary data set, then make sure there was just one thing in the data set.  rename that thing.
	CComPtr<IItemSet> iset;
	RUN(iset.CoCreateInstance(CLSID_ItemSet));
	RUN(CComQIPtr<IGroupItem>(iset)->Connect(group));
	CComPtr<IEnumUnknown> eu;
	RUN(UnpackMany(data,fetc,iset,&eu));
	long count;
	RUN(iset->get_Count(&count));
	HRESULT hr;
	if(count==0) {
		RUN(CComQIPtr<IGroupItem>(iset)->Remove());
		ERR(MONTAGE_E_NODATA);
	}
	if(count!=1) {
		RUN(CComQIPtr<IGroupItem>(iset)->Remove());
		ERR(MONTAGE_E_TOOMUCHDATA);
	}
	unk_iterator top(eu);
	_ASSERTE(top!=unk_iterator());
	hr = (*top)->QueryInterface(riidret(ret));
	RUN(isetPaste->Union(iset));
	RUN(CComQIPtr<IGroupItem>(iset)->Remove());
	return hr;
	/*
	// get old name
	CComQIPtr<IMCCSitePersistor> sp = *top, sp2;
	CComPtr<IMoniker> oldName;
	RUN(sp->get_RelMoniker(&oldName));
	// copy the persistor for new name
	RUN(CComQIPtr<IGroupItem>(sp)->Copy(bctx,group,0,riidret(&sp2)));
	// paste under new name
	CComPtr<IMoniker> mkNewName;
	RUN(group->Paste(bctx,GP_SrcIsObj,sp2,spName,IID_NULL,0,&mkNewName));
	if(isetPaste)
		RUN(isetPaste->Add(mkNewName));
	// update internal name
	RUN(sp2->put_RelMoniker(spName));
	sp2.CopyTo(ret);
	hr = S_OK;
	// remove temporary set, triggering death of old-named site persistor
	RUN(CComQIPtr<IGroupItem>(iset)->Remove());
	return S_OK;
	*/
}
STDMETHODIMP PasteSetUnpacker::UnpackMany(IDataObject *data, FORMATETC *fetc, IItemSet *iset, IEnumUnknown **ret) {
	METHTRACE(PasteSetUnpacker,UnpackMany);

	// context
	CComPtr<IGroup> group;
	RUN(mc()->GetMainGroup(riidret(&group)));

	// get data
	FormatEtc fetz(g_clipformats.cc);
	StgMedium stgm;
	RUNRET(data->GetData(&fetz,&stgm));
	if(stgm.tymed!=TYMED_ISTORAGE) {
		ReleaseStgMedium(&stgm);
		ERR(E_UNEXPECTED);
	}
	// find out CLSID of group implentation:
	CLSID clsidGroupImpl;
	{
		CComPtr<IStream> index;
		RUN(stgm.pstg->OpenStream(L"GroupIndex",0,STGM_READ|STGM_SHARE_EXCLUSIVE,0,&index));
		RUN(index->Read(&clsidGroupImpl,sizeof(CLSID),0));
	}
	CComPtr<IPersistStorage> srcGroup;
	RUN(CREATE(srcGroup,clsidGroupImpl)); TRACECREATEV(clsidGroupImpl);
	RUN(srcGroup->Load(stgm.pstg));

	// get the data set and transfer it in
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	ItemMoniker mkDataSet(L"Data");
	CComPtr<IGroupItem> pasteSet0;
	{
		CComPtr<IMoniker> mkSrcGroup;
		RUN(FoolishCreatePointerMoniker(srcGroup,&mkSrcGroup));
		RUN(mkDataSet->BindToObject(bctx,mkSrcGroup,riidret(&pasteSet0)));
	}
	CComPtr<IItemSet> pasteset;
	RUN(pasteSet0->Copy(bctx,group,0,riidret(&pasteset)));

	// destroy the group
	RUN(CComQIPtr<IGroup>(srcGroup)->Unload(0));
	srcGroup.Release();

	// read mks from the translated set, add them to local data set, and bind them for the enumRet
	CComPtr<IEnumMoniker> em;
	RUN(pasteset->Enum(&em));
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(group,&mkG));
	IUnknown **uray = 0;
	int i = 0;
	for(mk_iterator j(em);j!=mk_iterator();++j) {
		RUN(iset->Add(*j));
		if(i%20==0)
			uray = (IUnknown**)realloc(uray,sizeof(IUnknown*)*(i+20));
		RUN((*j)->BindToObject(bctx,mkG,IID_IUnknown,(void**)(uray+i)));  // uray[i] holds ref
		i++;
	}
	// call Remove on pasted set because it's about to get deleted and the main data set
	// has taken over the stg refs.
	RUN(CComQIPtr<IGroupItem>(pasteset)->Remove());
	// return enumeration
	typedef CComObject< CComEnum<IEnumUnknown,&IID_IEnumUnknown,IUnknown*,_CopyInterface<IUnknown> > > enum_unk;
	enum_unk *eu = new enum_unk;
	eu->Init(uray,uray+i,0,AtlFlagTakeOwnership);
	(*ret = eu)->AddRef();
	return S_OK;
}
STDMETHODIMP PasteSetUnpacker::Query(IDataObject *data) {
	METHTRACE(PasteSetUnpacker,Query);
	FormatEtc fetz(g_clipformats.cc);
	return data->QueryGetData(&fetz);
}
// INeedSomePointers
STDMETHODIMP PasteSetUnpacker::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(PasteSetUnpacker,TakePointers);
	if(m_mref)
		return S_FALSE;
	CComPtr<IUnknown> o;
	RUN(os->Next(1,&o,0));
	CComQIPtr<IMontageContainer> mc = o;
	o.Release();
	RUN(os->Next(1,&o,0));
	CComQIPtr<IMCCDataObjectReader> dor = o;
	return Initialize(mc,dor);
}
