/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "StdAfx.h"
#include "Container.h"
#include "extList.h"
bool isNative(LPCOLESTR exts, LPCOLESTR pszFileName) {
	const OLECHAR *dot = wcsrchr(pszFileName,'.');
	if(!dot)
		return false;
	return isInExtList(exts,dot+1);
}
STDMETHODIMP Container::Load(LPCOLESTR pszFileName,DWORD dwMode) {
    METHTRACE(Container,Load);
	// make sure that all relevant bits are set (for 95?)
	// uh.. can't find any examples or guidelines for this.. just debugging as I go..
	if(!(dwMode&3))
		dwMode |= STGM_READWRITE;
	if(!(dwMode&0x70))
		dwMode |= STGM_SHARE_EXCLUSIVE;
	if(isNative(m_exts.c_str(),pszFileName)) {
		CComPtr<IStorage> stg;
		RUN(StgOpenStorage(pszFileName,0,dwMode,0,0,&stg));
		RUN(Load(stg));
		if(dwMode & STGM_WRITE) // must get write privs
			return S_OK;
		// move to temp file since we can't write original
		CComPtr<IStorage> stg2;
		RUN(StgCreateDocfile(0,STGM_DIRECT|STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_DELETEONRELEASE,0,&stg2));

		RUN(stg->CopyTo(0,0,0,stg2));
		RUN(SaveCompleted(stg2));
		m_filename = pszFileName;
		return S_OK;
	}
	// Not native: initialize a tempfile (including bootstrapping, which installs fileport) and import file into new doc
	CComPtr<IStorage> stg2;
	RUN(StgCreateDocfile(0,STGM_DIRECT|STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_DELETEONRELEASE,0,&stg2));
	RUNRET(InitNew(stg2)); // may fail if for instance user cancels graph alg selection
	if(m_fileport) // fileport gets installed by bootstrapper in InitNew.
		if(FAILED(m_fileport->Import(pszFileName)))
			MessageBox(0,"File reading failed","Problems",MB_OK);
	m_filename = pszFileName;
	return S_OK;
}
STDMETHODIMP Container::Save(LPCOLESTR pszFileName,BOOL fRemember) {
    METHTRACE(Container,Save);
	if(!pszFileName)
		pszFileName = m_filename.c_str();
	else if(fRemember)
		m_filename = pszFileName;
	if(isNative(m_exts.c_str(),pszFileName)) {
		CComPtr<IStorage> stg;
		RUN(StgCreateDocfile(pszFileName,STGM_DIRECT|STGM_READWRITE|STGM_SHARE_EXCLUSIVE|STGM_CREATE,0,&stg));
		RUN(WriteClassStg(stg,m_clsid));
		RUN(Save(stg,false));
	}
	else if(m_fileport)
		if(FAILED(m_fileport->Export(pszFileName?pszFileName:m_filename.c_str())))
			ERR(E_INVALIDARG); // for now, must have good extension
	return S_OK;
}
STDMETHODIMP Container::SaveCompleted(LPCOLESTR pszFileName) {
    METHTRACE(Container,SaveCompleted);
	return S_OK;
}
STDMETHODIMP Container::GetCurFile(LPOLESTR *ppszFileName) {
    METHTRACE(Container,GetCurFile);
	if(m_filename.empty()) {
		*ppszFileName = 0;
		ERR(E_FAIL);
	}
	*ppszFileName = (LPOLESTR)CoTaskMemAlloc(m_filename.size()+1);
	wcscpy(*ppszFileName,m_filename.c_str());
	return S_OK;
}
