/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IPersistStorage for Container
// This interface only gets called when Montage is not itself in a group
#include "stdafx.h"
#include "Container.h"

#include "ChooseFromCatDlg.h"

STDMETHODIMP Container::GetClassID(CLSID *pClassID) {
	IMETHTRACE(Container,IPersistStorage,GetClassID);
	*pClassID = m_clsid;
	return S_OK;
}
STDMETHODIMP Container::IsDirty(void) {
	IMETHTRACE(Container,IPersistStorage,IsDirty);
	return m_dirty?S_OK:S_FALSE;
}
// hackakak: why here?
HRESULT Container::makeRenderer() {
    METHTRACE(Container,makeRenderer);
	CComPtr<IItemSet> dataSet;
	HRESULT hr = CComQIPtr<IOleItemContainer>(m_group)->GetObject(L"Data",BINDSPEED_INDEFINITE,BindCtx(),riidret(&dataSet));
	if(hr==MK_E_NOOBJECT)
		return S_FALSE;
	RUN(hr);
    RUN(CREATE(m_cache,CLSID_MCCDataObject));

	RUN(CREATE(m_renderer,CLSID_MCCItemSetRenderer));
	RUN(m_renderer->Initialize(m_group,dataSet,m_clsid,this,m_cache)); // hmm perhaps this needs an expanded set for render param
	return S_OK;
}
LPOLESTR g_persistorName = L"Montage Persistor";
STDMETHODIMP Container::InitNew(IStorage *pStg) {
	IMETHTRACE(Container,IPersistStorage,InitNew);

	// if montage is getting IPersistStorage events, its group has no parent.
	m_amRoot = true;

	if(!m_group) {
		RUN(CREATE(m_group,CLSID_Group));
		CComQIPtr<IPersistStorage> psGroup(m_group);
		QISET(psGroup);
		RUN(psGroup->InitNew(pStg));
		m_state = MCCS_SHOWN; // equiv to put_Group
	}
	else { // group implementation can be replaced, but its storage must be rooted in that of IPersistStorage.
		CComQIPtr<IPersistStorage> psGroup(m_group);
		QISET(psGroup);
		RUN(psGroup->HandsOffStorage());
		RUN(psGroup->SaveCompleted(pStg));
	}

	// create, name & init persistor
	CComPtr<IMontagePersistor> persistor;
	RUN(CREATE(persistor,CLSID_MontagePersistor));
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<IMoniker> perName;
	RUN(CreateItemMoniker(L"!",g_persistorName,&perName));
	RUN(m_group->Paste(bctx,GP_SrcIsObj,persistor,perName,IID_NULL,0,&m_mkPersistor));
	RUN(CComQIPtr<IPersistStreamInit>(persistor)->InitNew());
	RUN(CComQIPtr<DIPersistor>(persistor)->SetSubject(true,(IMontageContainer*)this));
	GetUnknown()->Release(); // persistor's refcount doesn't count.
	RUN(persistor->put_SelfMoniker(m_mkPersistor));

	// bootstrap
	if(!m_bootstrap) {
		ChooseFromCatDlg dlg(CATID_MCCStarter);
		int chosen = dlg.DoModal();
		if(chosen<0)
			ERR(E_FAIL);
		RUN(CREATE(m_bootstrap,dlg.m_engines[chosen].clsid));
	}
	RUNRET(m_bootstrap->InitializeOther(persistor));
	m_bootstrap.Release(); // should now be fully operational.
	RUN(makeRenderer());
	return S_OK;
}
STDMETHODIMP Container::Load(IStorage *pStg) {
	IMETHTRACE(Container,IPersistStorage,Load);

	// only get IPersistStorage cmds when root
	m_amRoot = true;

	// find out CLSID of group implentation:
	CLSID clsidGroupImpl;
	{
		CComPtr<IStream> index;
		RUN(pStg->OpenStream(L"GroupIndex",0,STGM_READ|STGM_SHARE_EXCLUSIVE,0,&index));
		RUN(index->Read(&clsidGroupImpl,sizeof(CLSID),0));
	}
	// this group will have no parent.
	RUN(CREATE(m_group,clsidGroupImpl));
	RUN(CComQIPtr<IPersistStorage>(m_group)->Load(pStg));

	// now that we have a storage, sites can be inserted
	m_state = MCCS_HIDDEN;

	// get persistor
	CComPtr<IOleItemContainer> oic;
	RUN(m_group->QueryInterface(riidret(&oic)));
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	// if file format changes, this may fail
	CComPtr<IMontagePersistor> persistor;
	if(FAILED(oic->GetObject(g_persistorName,BINDSPEED_INDEFINITE,bctx,riidret(&persistor)))) {
		ERR(E_FAIL);
	}

	// when the persistor realizes, it initializes me
	// this is one of a very few uses of "realize this"
	RUN(CComQIPtr<DIPersistor>(persistor)->SetSubject(false,(IMontageContainer*)this));
	GetUnknown()->Release(); // persistor's refcount doesn't count.  (I'll call IMP::ForgetSubject on ~ to tell it that)
	RUN(CComQIPtr<IGroupItem>(persistor)->Realize(bctx));
	RUN(makeRenderer());
	return S_OK;
}
STDMETHODIMP Container::Save(IStorage *pStgSave,BOOL fSameAsLoad) {
	IMETHTRACE(Container,IPersistStorage,Save);
	RUNRET(CComQIPtr<IPersistStorage>(m_group)->Save(pStgSave,fSameAsLoad));
	m_dirty = false;
	return S_OK;
}
STDMETHODIMP Container::SaveCompleted(IStorage *pStgNew) {
	IMETHTRACE(Container,IPersistStorage,SaveCompleted);
	return CComQIPtr<IPersistStorage>(m_group)->SaveCompleted(pStgNew);
}
STDMETHODIMP Container::HandsOffStorage(void) {
	IMETHTRACE(Container,IPersistStorage,HandsOffStorage);
	return CComQIPtr<IPersistStorage>(m_group)->HandsOffStorage();
}
