/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "PlayControls.h"

POINT g_playpolys[] = {{10,4},{10,28},{22,16},{10,4}},
	g_fwdpolys[] = {{7,4},{7,28},{19,16},{7,4},{21,4},{24,4},{24,28},{21,28},{21,4}},
	g_pausepolys[] = {{9,6},{9,26},{14,26},{14,6},{9,6},{18,6},{18,26},{23,26},{23,6},{18,6}};
#define NPTS(x) (sizeof(x)/sizeof(POINT))

// IMCCPlayControls
STDMETHODIMP PlayControls::Initialize(IMCCButtonSet *bset) {
    METHTRACE(PlayControls,Initialize);
	if(!bset)
		ERR(E_INVALIDARG);
	m_bset = bset;
	RUN(AtlAdvise(m_bset,GetUnknown(),IID_IMCCButtonSetEvents,&m_chocolateChipCremeWich));
	RECT r = {0,0,32,32};
	RUN(m_bset->Add(MCCPLAY_PAUSE,MCCB_DOWNONLY,&r,g_pausepolys,NPTS(g_pausepolys)));
	r += size(32,0);
	RUN(m_bset->Add(MCCPLAY_PLAY,MCCB_DOWNONLY|MCCB_DOWN,&r,g_playpolys,NPTS(g_playpolys)));
	r += size(32,0);
	RUN(m_bset->Add(MCCPLAY_FORWARD,MCCB_DOWNONLY|MCCB_DISABLED,&r,g_fwdpolys,NPTS(g_fwdpolys)));
	return S_OK;
}
STDMETHODIMP PlayControls::Trigger(MCCPLAY_BUTTONS button) {
    METHTRACE(PlayControls,Trigger);
    switch(button) {
    case MCCPLAY_PAUSE:
		RUN(m_bset->Modify(MCCPLAY_PLAY,MCCB_DOWNONLY,0,0,0));
		RUN(m_bset->Modify(MCCPLAY_FORWARD,MCCB_DOWNONLY,0,0,0));
        break;
    case MCCPLAY_PLAY:
		RUN(m_bset->Modify(MCCPLAY_PAUSE,MCCB_DOWNONLY,0,0,0));
		RUN(m_bset->Modify(MCCPLAY_FORWARD,MCCB_DOWNONLY|MCCB_DISABLED,0,0,0));
        break;
    case MCCPLAY_FORWARD: // pop back up
		RUN(m_bset->Modify(MCCPLAY_FORWARD,MCCB_DOWNONLY,0,0,0));
        break;
    }
    RUN(fireEvent(GetUnknown(),fireOneParam<IMCCPlayControlsEvents,MCCPLAY_BUTTONS>
        (&IMCCPlayControlsEvents::After,button)));
    return S_OK;
}
STDMETHODIMP PlayControls::put_PlayMode(VARIANT_BOOL whether) {
    METHTRACE(PlayControls,put_PlayMode);
    RUN(Trigger(whether?MCCPLAY_PLAY:MCCPLAY_PAUSE));
    return S_OK;
}
STDMETHODIMP PlayControls::put_FwdEnabled(VARIANT_BOOL whether) {
    METHTRACE(PlayControls,put_FwdEnabled);
    RUN(m_bset->Modify(MCCPLAY_FORWARD,MCCB_DOWNONLY|(whether?0:MCCB_DISABLED),0,0,0));
    return S_OK;
}

// IMCCButtonSetEvents
STDMETHODIMP PlayControls::Changed(int id, DWORD flags) {
    METHTRACE(PlayControls,Changed);
    BOOL wait = VARIANT_FALSE;
    MCCPLAY_BUTTONS bid = (MCCPLAY_BUTTONS)id;
    RUN(fireEvent(GetUnknown(),fireTwoParams<IMCCPlayControlsEvents,MCCPLAY_BUTTONS,BOOL*>
        (&IMCCPlayControlsEvents::Before,bid,&wait)));
    if(!wait)
        RUN(Trigger(bid));
	return S_OK;
}
// INeedSomePointers
STDMETHODIMP PlayControls::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
    METHTRACE(PlayControls,TakePointers);
	CComPtr<IUnknown> u;
	RUN(os->Next(1,&u,0));
	return Initialize(CComQIPtr<IMCCButtonSet>(u));
}
// IPersist
STDMETHODIMP PlayControls::GetClassID(CLSID *ret) {
    METHTRACE(PlayControls,GetClassID);
	*ret = GetObjectCLSID();
	return S_OK;
}
