/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "PosInfo.h"

PosInfo::PosInfo() {
	CONSTRUCTORTRACE(PosInfo);
	m_data.postype = MCC_POS_NONE;
}
PosInfo::~PosInfo() {
	DESTRUCTORTRACE(PosInfo);
}
// IMCCPosInfo
STDMETHODIMP PosInfo::GetStuff(MCC_POS_INFO *ret) {
	*ret = m_data;
	return S_OK;
}
// DIMCCPosInfo
STDMETHODIMP PosInfo::get_Type(long *ret) {
	*ret = m_data.postype;
	return S_OK;
}
STDMETHODIMP PosInfo::SetWindowPoint(long x, long y) {
	m_data.postype = MCC_POS_WINDOWPOINT;
	m_data.pt.x = x;
	m_data.pt.y = y;
	return S_OK;
}
STDMETHODIMP PosInfo::SetCanvasPoint(long x, long y) {
	m_data.postype = MCC_POS_CANVASPOINT;
	m_data.ptl.x = x;
	m_data.ptl.y = y;
	return S_OK;
}
STDMETHODIMP PosInfo::GetPoint(long *x, long *y) {
	if(m_data.postype!=MCC_POS_WINDOWPOINT && m_data.postype!=MCC_POS_CANVASPOINT)
		ERR(E_FAIL);
	*x = m_data.pt.x;
	*y = m_data.pt.y;
	return S_OK;
}
STDMETHODIMP PosInfo::SetWindowRect(long left, long top, long right, long bottom) {
	m_data.postype = MCC_POS_WINDOWRECT;
	m_data.rc.left = left;
	m_data.rc.top = top;
	m_data.rc.right = right;
	m_data.rc.bottom = bottom;
	return S_OK;
}
STDMETHODIMP PosInfo::SetCanvasRect(long left, long top, long right, long bottom) {
	m_data.postype = MCC_POS_CANVASRECT;
	m_data.rc.left = left;
	m_data.rc.top = top;
	m_data.rc.right = right;
	m_data.rc.bottom = bottom;
	return S_OK;
}
STDMETHODIMP PosInfo::GetRect(long *left, long *top, long *right, long *bottom) {
	if(m_data.postype!=MCC_POS_WINDOWRECT && m_data.postype!=MCC_POS_CANVASRECT)
		ERR(E_FAIL);
	*left = m_data.rc.left;
	*top = m_data.rc.top;
	*right = m_data.rc.right;
	*bottom = m_data.rc.bottom;
	return S_OK;
}
struct mccPosInfo : CComObject<PosInfo> {
	mccPosInfo(const MCC_POS_INFO &source) {
		m_data = source;
	}
};