/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "PropertiesPane.h"

HRESULT PropertiesPane::redo() {
    int N = m_data.size() + 1,  // one blank spot
        oldN;
    RUN(m_pane->get_PaneCount(&oldN));
    CComPtr<IMontageContainer> mc;
    RUN(m_layer->GetContainer(riidret(&mc)));
    if(N<oldN) {
		for(int i = oldN-1; i>=N; --i) {
            CComPtr<IMCCPane> h,p,v;
            RUN(m_pane->get_Pane(i,&h));
            RUN(h->get_Pane(0,&p));
            RUN(h->get_Pane(1,&v));
            CComPtr<IMCCSite>ps,vs;
            RUN(p->get_Content(&ps));
            RUN(v->get_Content(&vs));
            RUN(ps->put_Intent(MCCS_DELETED));
            RUN(vs->put_Intent(MCCS_DELETED));
			m_edListens[i]->stop();
            m_coListens[i]->stop();
            RUN(m_pane->RemovePane(i));
        }
        m_edListens.resize(N);
        m_coListens.resize(N);
    }
    else {
        m_edListens.resize(N);
        m_coListens.resize(N);
        for(int i = oldN; i<N; ++i) {
            CComPtr<IMCCPane> h,p,v;
            RUN(h.CoCreateInstance(CLSID_MCCPane));
            RUN(p.CoCreateInstance(CLSID_MCCPane));
            RUN(p->put_Border(rect(1,1,0,0)));
            RUN(v.CoCreateInstance(CLSID_MCCPane));
            RUN(v->put_Border(rect(1,1,0,0)));
            RUN(m_pane->AddPane(i,h));
            RUN(m_pane->put_PaneSize(i,m_cellHeight));
            RUN(h->AddPane(0,p));
            RUN(h->AddPane(1,v));
            RUN(h->put_PaneXtra(0,1));
            RUN(h->put_PaneXtra(1,2));
            CComPtr<IMCCSite> propsite,valsite;
            RUN(mc->new_Site(&propsite));
            RUN(mc->new_Site(&valsite));
            RUN(propsite->put_Layer(m_layer));
            RUN(valsite->put_Layer(m_layer));
            CComPtr<IMCCEdit> propctl;
            RUN(propctl.CoCreateInstance(CLSID_MCCEdit));
            RUN(propctl->put_WStyle(0)); // defaults to multiline
            CComPtr<IMCCComboBox> valctl;
            RUN(valctl.CoCreateInstance(CLSID_MCCComboBox));
            RUN(propsite->put_Control(propctl));
            RUN(valsite->put_Control(valctl));
            RUN(p->put_Content(propsite));
            RUN(v->put_Content(valsite));
            RUN(propsite->put_Intent(MCCS_ACTIVE));
            RUN(valsite->put_Intent(MCCS_ACTIVE));
            m_edListens[i] = new EdListen();
            RUN(m_edListens[i]->init(this,propctl));
            m_coListens[i] = new CoListen();
            RUN(m_coListens[i]->init(this,valctl));
        }
    }
    data::iterator mi = m_data.begin();
    for(unsigned i=0; i<=m_data.size(); i++) {
        CComPtr<IMCCPane> h,p,v;
        RUN(m_pane->get_Pane(i,&h));
        RUN(h->get_Pane(0,&p));
        RUN(h->get_Pane(1,&v));
        CComPtr<IMCCSite>ps,vs;
        RUN(p->get_Content(&ps));
        RUN(v->get_Content(&vs));
        CComPtr<IMCCEdit> pe;
        RUN(ps->GetControl(riidret(&pe)));
        CComPtr<IMCCComboBox> ve;
        RUN(vs->GetControl(riidret(&ve)));
        RUN(ve->Clear());
        if(mi != m_data.end()) {
            RUN(pe->put_Text(CComBSTR(mi->first.c_str())));
            RUN(ve->Add(CComBSTR(m_nonText.c_str())));
            for(setws::iterator si = mi->second.vals.begin(); si != mi->second.vals.end(); ++si)
                RUN(ve->Add(CComBSTR(si->c_str())));
            if(!mi->second.sel.empty())
                RUN(ve->Choose(CComBSTR(mi->second.sel.c_str())));
            m_edListens[i]->prop = m_coListens[i]->prop = CComBSTR(mi->first.c_str());
            ++mi;
        }
        else
            RUN(pe->put_Text(0));
    }
    return S_OK;
}
// IMCCPropertiesPane
STDMETHODIMP PropertiesPane::AddProperty(BSTR prop) {
    m_data[prop];
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::AddValue(BSTR prop, BSTR val) {
    m_data[prop].vals.insert(val);
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::CountValues(BSTR prop, int *ret) {
    *ret = m_data[prop].vals.size();
	return S_OK;
}
STDMETHODIMP PropertiesPane::RenameProperty(BSTR prop, BSTR newName) {
    // replaces
    m_data[newName] = m_data[prop];
    m_data.erase(prop);
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::RemoveValue(BSTR prop, BSTR val) {
    m_data[prop].vals.erase(val);
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::RemoveValues(BSTR prop) {
    m_data[prop].clear();
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::RemoveProperty(BSTR prop) {
    m_data.erase(prop);
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::RemoveProperties() {
    m_data.clear();
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::GetSelected(BSTR prop, BSTR *ret) {
    wstring &v = m_data[prop].sel;
    if(v.empty() || v.compare(m_nonText)==0)
        *ret = 0;
    else
        *ret = CComBSTR(v.c_str()).Detach();
	return S_OK;
}
STDMETHODIMP PropertiesPane::SetSelected(BSTR prop, BSTR val) {
    if(val) {
        m_data[prop].vals.insert(val); // assure it's there
        m_data[prop].sel = val;
    }
    else
        m_data[prop].sel = m_nonText;
    RUN(redo());
	return S_OK;
}
typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > Enumster;

STDMETHODIMP PropertiesPane::EnumProperties(IEnumString **ret) {
    LPOLESTR *ray = new LPOLESTR[m_data.size()];
    int i=0;
    for(data::iterator pi = m_data.begin(); pi!=m_data.end(); ++pi) {
        LPOLESTR src = const_cast<LPOLESTR>(pi->first.c_str());
        _Copy<LPOLESTR>::copy(ray+i,&src);
        ++i;
    }
    Enumster *numb = new Enumster;
    numb->Init(ray,ray+i,0,AtlFlagTakeOwnership);
    (*ret = numb)->AddRef();
	return S_OK;
}
STDMETHODIMP PropertiesPane::EnumValues(BSTR prop, IEnumString **ret) {
    setws &s = m_data[prop].vals;
    LPOLESTR *ray = new LPOLESTR[s.size()];
    int i=0;
    for(setws::iterator si = s.begin(); si!=s.end(); ++si) {
        LPOLESTR src = const_cast<LPOLESTR>(si->c_str());
        _Copy<LPOLESTR>::copy(ray+i,&src);
        ++i;
    }
    Enumster *numb = new Enumster;
    numb->Init(ray,ray+i,0,AtlFlagTakeOwnership);
    (*ret = numb)->AddRef();
	return S_OK;
}
STDMETHODIMP PropertiesPane::get_NothingText(BSTR *ret) {
    *ret = CComBSTR(m_nonText.c_str()).Detach();
	return S_OK;
}
STDMETHODIMP PropertiesPane::put_NothingText(BSTR val) {
    m_nonText = val;
    RUN(redo());
	return S_OK;
}
STDMETHODIMP PropertiesPane::get_Pane(IMCCPane **ret) {
    m_pane.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP PropertiesPane::put_Pane(IMCCPane *val) {
    m_pane = val;
	return S_OK;
}
STDMETHODIMP PropertiesPane::get_Layer(IMCCLayer **ret) {
    m_layer.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP PropertiesPane::put_Layer(IMCCLayer *val) {
    m_layer = val;
	return S_OK;
}
STDMETHODIMP PropertiesPane::get_CellHeight(int *ret) {
    *ret = m_cellHeight;
	return S_OK;
}
STDMETHODIMP PropertiesPane::put_CellHeight(int val) {
    m_cellHeight = val;
	return S_OK;
}
