/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct PPEditListen :
    CComObjectRoot,
    IMCCEditEvents
{
    CComPtr<IMCCPropertiesPane> pp;
    CComBSTR prop;
    CComPtr<IMCCEdit> ed;
	DWORD mintItsIt;

	PPEditListen() : mintItsIt(0) {}

BEGIN_COM_MAP(PPEditListen)
    COM_INTERFACE_ENTRY(IMCCEditEvents)
END_COM_MAP();

	HRESULT init(IMCCPropertiesPane *pp,IMCCEdit *ed) {
		this->pp = pp;
		this->ed = ed;
		RUN(AtlAdvise(ed,GetUnknown(),IID_IMCCEditEvents,&mintItsIt));
		return S_OK;
	}
	HRESULT stop() {
		if(mintItsIt)
			RUN(AtlUnadvise(ed,IID_IMCCEditEvents,mintItsIt));
        return S_OK;
	}

// IMCCEditEvents
	STDMETHODIMP TextChanged() {
        CComBSTR v;
        RUN(ed->get_Text(&v));
        if(prop)
            RUN(pp->RenameProperty(prop,v));
        else
            RUN(pp->AddProperty(v));
        return S_OK;
    }
};
typedef CComObject<PPEditListen> EdListen;
struct PPComboBoxListen :
    CComObjectRoot,
    IMCCComboBoxEvents
{
    CComPtr<IMCCPropertiesPane> pp;
    CComBSTR prop;
    CComPtr<IMCCComboBox> bx;
	DWORD smore;

	PPComboBoxListen() : smore(0) {}

BEGIN_COM_MAP(PPComboBoxListen)
    COM_INTERFACE_ENTRY(IMCCComboBoxEvents)
END_COM_MAP();

	HRESULT init(IMCCPropertiesPane *pp,IMCCComboBox *bx) {
		this->pp = pp;
		this->bx = bx;
		RUN(AtlAdvise(bx,GetUnknown(),IID_IMCCComboBoxEvents,&smore));
		return S_OK;
	}
	HRESULT stop() {
		if(smore)
			RUN(AtlUnadvise(bx,IID_IMCCComboBoxEvents,smore));
        return S_OK;
	}
    STDMETHODIMP Chose(BSTR val) {
        if(prop)
            RUN(pp->SetSelected(prop,val));
        return S_OK;
    }
	STDMETHODIMP TextChanged(BSTR val) {
        if(prop) {
            RUN(pp->AddValue(prop,val));
            RUN(pp->SetSelected(prop,val));
        }
        return S_OK;
    }
};
typedef CComObject<PPComboBoxListen> CoListen;


struct PropertiesPane :
	public CComCoClass<PropertiesPane, &CLSID_MCCPropertiesPane>,
	public CComObjectRoot,
	public IMCCPropertiesPane
{
    typedef set<wstring,lessWstring> setws;
    struct values {
        wstring sel;
        setws vals;
        void clear() {
            sel.clear();
            vals.clear();
        }
    };
    typedef map<wstring,values,lessWstring> data;
    data m_data;
	vector<EdListen*> m_edListens;
	vector<CoListen*> m_coListens;
    CComPtr<IMCCPane> m_pane;
    CComPtr<IMCCLayer> m_layer;
    wstring m_nonText;
    int m_cellHeight;

	REGISTER_GENERIC(L"PropertiesPane")
    PropertiesPane() : m_cellHeight(0) {}

BEGIN_COM_MAP(PropertiesPane)
	COM_INTERFACE_ENTRY(IMCCPropertiesPane)
END_COM_MAP()

    HRESULT redo();

// IMCCPropertiesPane
	STDMETHODIMP AddProperty(BSTR prop);
	STDMETHODIMP AddValue(BSTR prop, BSTR val);
    STDMETHODIMP CountValues(BSTR prop, int *ret);
    STDMETHODIMP RenameProperty(BSTR prop, BSTR newName);
	STDMETHODIMP RemoveValue(BSTR prop, BSTR val);
	STDMETHODIMP RemoveValues(BSTR prop);
	STDMETHODIMP RemoveProperty(BSTR prop);
	STDMETHODIMP RemoveProperties();
	STDMETHODIMP GetSelected(BSTR prop, BSTR *ret);
	STDMETHODIMP SetSelected(BSTR prop, BSTR val);
    STDMETHODIMP EnumProperties(IEnumString **ret);
    STDMETHODIMP EnumValues(BSTR prop, IEnumString **ret);
	STDMETHODIMP get_NothingText(BSTR *ret);
	STDMETHODIMP put_NothingText(BSTR val);
    STDMETHODIMP get_Pane(IMCCPane **ret);
    STDMETHODIMP put_Pane(IMCCPane *val);
    STDMETHODIMP get_Layer(IMCCLayer **ret);
    STDMETHODIMP put_Layer(IMCCLayer *val);
    STDMETHODIMP get_CellHeight(int *ret);
    STDMETHODIMP put_CellHeight(int val);

};