/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct rd2;
typedef list<rd2*> rdlist;

class ATL_NO_VTABLE RectEngine :
	public CComCoClass<RectEngine,&CLSID_MCCRectEngine>,
	public CComObjectRoot,
	public IRectEngine,
	public IMontageInclusion,
	public IPersist
{
	friend class RectDelegate;
	rdlist m_data;
	rd2 *m_inner,*m_outer;

	// called by delegates instead of changing themselves
	HRESULT change(rd2 *o,LPCRECT r);
	HRESULT changeInner(LPCRECT r);
	HRESULT changeOuter(LPCRECT r);
	// makes border as small as possible (called at end of all other ops)
	HRESULT spring();

public:
	RectEngine();
	~RectEngine();

	REGISTER_GENERIC(L"RectEngine")

BEGIN_COM_MAP(RectEngine)
	COM_INTERFACE_ENTRY(IRectEngine)
	COM_INTERFACE_ENTRY(IMontageInclusion)
	COM_INTERFACE_ENTRY(IPersist)
END_COM_MAP()
// IRectEngine
	STDMETHODIMP new_Delegate(IMoveRect **ret);
	STDMETHODIMP get_Outer(IMoveRect **ret);
	STDMETHODIMP get_Inner(IMoveRect **ret);
	STDMETHODIMP put_OuterFlexible(VARIANT_BOOL val);
	STDMETHODIMP get_OuterFlexible(VARIANT_BOOL *ret);
// IMontageInclusion
	STDMETHODIMP ContainsRect(LPCRECT lpcr);
// IPersist
	STDMETHODIMP GetClassID(CLSID *ret);
};
class ATL_NO_VTABLE RectDelegate :
	public CComObjectRoot,
	public IMoveRect,
	public IConnectionPointContainerImpl<RectDelegate>,
	public IConnectionPointImpl<RectDelegate,&IID_IMoveRect>
{
	friend class RectEngine;
protected:
	RectEngine *m_eng;
	bool m_constrains;
	RECT m_rect;
	enum {normal,special_inner,special_outer} m_type;
public:
	RectDelegate();
	~RectDelegate(); // remove from engine

BEGIN_COM_MAP(RectDelegate)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IMoveRect)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(RectDelegate)
	CONNECTION_POINT_ENTRY(IID_IMoveRect)
END_CONNECTION_POINT_MAP()

	HRESULT set(LPCRECT val);
// IMoveRect
	STDMETHODIMP Move(LPCRECT lpcr);
};
// full COM
struct rd2 : public CComObject<RectDelegate> {
	rd2(RectEngine *eng) {
		m_eng = eng;
	}
};
