/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ScrollingTransform.h"

ScrollingTransform::ScrollingTransform() :
	m_locks(0),
	m_changed(0),
	m_pumpkinBar(0)
{
	CONSTRUCTORTRACE(ScrollingTransform);
	m_offset = pointl(0,0);
	m_extent = size(0,0);
	m_area = rect(0,0,0,0);
	m_viewArea = m_canvView = rectl(0,0,0,0);
	m_viewing = m_canving = false;
	m_doEvents = true;
	//hack
	memset(&m_offset,0,2*sizeof(POINT)+2*sizeof(RECT)+1);
}
ScrollingTransform::~ScrollingTransform() {
	DESTRUCTORTRACE(ScrollingTransform);
}
HRESULT ScrollingTransform::changed(bool offset,bool scale) {
    RUN(mc()->TransformChanged(offset*VARIANT_TRUE,scale*VARIANT_TRUE));
    return S_OK;
}
HRESULT ScrollingTransform::checkBounds() {
    METHTRACE(ScrollingTransform,checkBounds);
	if(!m_bounds.valid())
		return S_FALSE;
	if(m_locks) {
		m_changed |= CHECK_BOUNDS;
		return S_FALSE;
	}
	RECTL brc;
	RUN(get_Bounds(&brc));
	bool change = false;
	// if there's stuff to be shown and we're not showing it, shift toward it
	if(brc.left<m_offset.x && brc.right<m_offset.x+m_extent.cx) {
		m_offset.x = max(brc.left,brc.right-m_extent.cx);
		change = true;
	}
	if(brc.top>m_offset.y && brc.bottom>m_offset.y-m_extent.cy) {
		m_offset.y = min(brc.top,brc.bottom+m_extent.cy);
		change = true;
	}
	if(brc.right>m_offset.x+m_extent.cx && brc.left>m_offset.x) {
		m_offset.x = min(brc.right-m_extent.cx,brc.left);
		change = true;
	}
	if(brc.bottom<m_offset.y-m_extent.cy && brc.top<m_offset.y) {
		m_offset.y = max(brc.bottom+m_extent.cy,brc.top);
		change = true;
	}
	if(change) {
		/*
		if(m_locks)
			m_changed |= CHANGED_OFFSET;
		else
		*/
			RUN(fireEvent(GetUnknown(),
				fireVoid<IMCCScrollingTransformEvents>(&IMCCScrollingTransformEvents::OffsetChanged)));
		return S_OK;
	}
	return S_FALSE;
}
// IMCCTransform
STDMETHODIMP ScrollingTransform::put_Extent(SIZEL *val) {
    METHTRACE(ScrollingTransform,put_Extent);
    if(assign(m_extent,*val)) {
	    RUN(checkBounds());
	    RUN(changed(false,true));
    }
	return S_OK;
}
STDMETHODIMP ScrollingTransform::get_Extent(SIZEL *val) {
    METHTRACE(ScrollingTransform,get_Extent);
	*val = m_extent;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::put_TransformedArea(RECT *val) {
    METHTRACE(ScrollingTransform,put_TransformedArea);
	if(!wid(*val)||!hei(*val))
		ERR(E_INVALIDARG);
    bool chofs = val->left!=m_area.left || val->top!=m_area.top;
	// windows rects are rightbottom exclusive, so exclude it now!
	RECT adjust = {val->left,val->top,val->right-1,val->bottom-1};
    bool chsize = siz(adjust)!=siz(m_area);
	m_area = adjust;
	RUN(changed(chofs,chsize));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::get_TransformedArea(RECT *val) {
    METHTRACE(ScrollingTransform,get_TransformedArea);
	 *val = m_area;
	 return S_OK;
}
STDMETHODIMP ScrollingTransform::Viewing(LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(ScrollingTransform,Viewing);
	if(lpcrOut) {
		m_viewArea = *lpcrOut;
		m_viewArea.right--;
		if(m_viewArea.top>m_viewArea.bottom)
			m_viewArea.bottom++;
		else
			m_viewArea.bottom--;
		m_viewing = true;
		if(lpcrCanv) {
			m_canving = true;
			m_canvView = *lpcrCanv;
		}
	}
	else m_viewing = false;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::get_IsViewing(VARIANT_BOOL *ret) {
    METHTRACE(ScrollingTransform,get_IsViewing);
	*ret = m_viewing?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
HRESULT ScrollingTransform::c2ps(SIZEL s, PSIZE psiz,RECT &area) {
    METHTRACE(ScrollingTransform,c2ps);
	SIZEL e = (m_viewing&&m_canving)?siz(m_canvView):m_extent;
	if(!e.cx||!e.cy)
		ERR(E_PENDING);
	psiz->cx = e.cx?s.cx*wid(area)/e.cx:0;
	psiz->cy = e.cy?-s.cy*hei(area)/e.cy:0; // y increases downward
	return S_OK;
}
STDMETHODIMP ScrollingTransform::C2WS(SIZEL siz, PSIZE psiz) {
    METHTRACE(ScrollingTransform,C2WS);
	RUNRET(c2ps(siz,psiz,m_area));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::C2VS(SIZEL siz, PSIZE psiz) {
    METHTRACE(ScrollingTransform,C2VS);
	if(!m_viewing)
		return C2WS(siz,psiz);
	RUNRET(c2ps(siz,psiz,rect(m_viewArea)));
	if(m_viewArea.bottom<m_viewArea.top) // y increases upward
		psiz->cy = -psiz->cy;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::W2CS(SIZE siz, PSIZEL psiz) {
    METHTRACE(ScrollingTransform,W2CS);
	if(!wid(m_area)||!hei(m_area))
		ERR(E_PENDING);
	psiz->cx = siz.cx*m_extent.cx/wid(m_area);
	psiz->cy = -siz.cy*m_extent.cy/hei(m_area);
	return S_OK;
}
STDMETHODIMP ScrollingTransform::C2WP(POINTL pt, PPOINT ppt) {
    METHTRACE(ScrollingTransform,C2WP);
	SIZEL sC = {pt.x-m_offset.x,pt.y-m_offset.y};
	SIZE sW;
	RUNRET(C2WS(sC,&sW));
	*ppt = point(m_area.left,m_area.top)+sW;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::C2VP(POINTL pt, PPOINT ppt) {
    METHTRACE(ScrollingTransform,C2VP);
	if(!m_viewing)
		return C2WP(pt,ppt);
	bool cart = m_viewArea.top>m_viewArea.bottom;
	POINTL ll = m_canving?pointl(m_canvView.left,m_canvView.top):m_offset;
	SIZEL sC = {pt.x-ll.x,pt.y-ll.y};
	SIZE sW;
	RUNRET(C2VS(sC,&sW));
	*ppt = point(m_viewArea.left,m_viewArea.top)+sW;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::W2CP(POINT pt, PPOINTL ppt) {
    METHTRACE(ScrollingTransform,W2CP);
	SIZE sW = {pt.x-m_area.left,pt.y-m_area.top};
	SIZEL sC;
	RUNRET(W2CS(sW,&sC));
	point(*ppt) = point(m_offset)+sC;
	return S_OK;
}
// rect is binary equivalent to POINT[2]
STDMETHODIMP ScrollingTransform::C2WR(LPCRECTL pcr, PRECT prc) {
    METHTRACE(ScrollingTransform,C2WR);
	const POINTL *src = (const POINTL*)pcr;
	POINT *dest = (POINT*)prc;
	RUNRET(C2WP(*src,dest));
	RUNRET(C2WP(src[1],dest+1));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::C2VR(LPCRECTL pcr, PRECT prc) {
    METHTRACE(ScrollingTransform,C2VR);
	if(!m_viewing)
		return C2WR(pcr,prc);
	const POINTL *src = (const POINTL*)pcr;
	POINT *dest = (POINT*)prc;
	RUNRET(C2VP(*src,dest));
	RUNRET(C2VP(src[1],dest+1));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::W2CR(LPCRECT pcr, PRECTL prc) {
    METHTRACE(ScrollingTransform,W2CR);
	const POINT *src = (const POINT*)pcr;
	POINTL *dest = (POINTL*)prc;
	RUNRET(W2CP(*src,dest));
	RUNRET(W2CP(src[1],dest+1));
	return S_OK;
}
HRESULT ScrollingTransform::fireBoundsChanged() {
	if(m_doEvents) {
		if(m_locks)
			m_changed |= CHANGED_BOUNDS;
		else
			RUN(fireEvent(GetUnknown(),fireVoid<IMCCScrollingTransformEvents>(&IMCCScrollingTransformEvents::BoundsChanged)));
		m_boundsChanged = false;
	}
	else
		m_boundsChanged = true;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::AddRect(LPCRECTL r) {
    METHTRACE(ScrollingTransform,AddRect);
	if(m_bounds.add(*r)) {
		fireBoundsChanged();
		if(checkBounds()==S_OK)
			RUN(changed(true,false));
	}
	return S_OK;
}
STDMETHODIMP ScrollingTransform::RemoveRect(LPCRECTL r) {
    METHTRACE(ScrollingTransform,RemoveRect);
	if(m_bounds.remove(*r)) {
		fireBoundsChanged();
		if(checkBounds()==S_OK)
			RUN(changed(true,false));
	}
	return S_OK;
}
// IMCCScrollingTransform
STDMETHODIMP ScrollingTransform::put_Offset(POINTL *val) {
    METHTRACE(ScrollingTransform,put_Offset);
	m_offset = *val;
	if(m_locks)
		m_changed |= CHANGED_OFFSET;
	else
		RUN(fireEvent(GetUnknown(),fireVoid<IMCCScrollingTransformEvents>(&IMCCScrollingTransformEvents::OffsetChanged)));
	RUN(checkBounds());
	RUN(changed(true,false));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::get_Offset(POINTL *ret) {
    METHTRACE(ScrollingTransform,get_Offset);
	*ret = m_offset;
	return S_OK;
}
STDMETHODIMP ScrollingTransform::get_Bounds(RECTL *ret) {
    METHTRACE(ScrollingTransform,get_Bounds);
	if(!m_bounds.valid())
		ERR(E_FAIL);
	*ret = m_bounds.get();
	VARIANT_BOOL useM;
	RUN(mc()->get_UseMargin(&useM));
	if(useM) {
		RECTL margin;
		RUN(mc()->get_Margin(&margin));
		*ret += margin;
	}
	return S_OK;
}
STDMETHODIMP ScrollingTransform::put_DoEvents(VARIANT_BOOL val) {
	bool old = m_doEvents;
	m_doEvents = val!=VARIANT_FALSE;
	if(m_boundsChanged&&!old&&m_doEvents)
		fireBoundsChanged();
	return S_OK;
}

// IObjectWithSite
STDMETHODIMP ScrollingTransform::SetSite(IUnknown *pUnkSite) {
    METHTRACE(ScrollingTransform,SetSite);
	if(m_mcRef) {
		RUN(AtlUnadvise(mc(),IID_IMontageContainerEvents,m_pumpkinBar));
		m_mcRef.Release();
	}
	if(!pUnkSite)
		return S_OK;
	CComQIPtr<IMontageContainer> mc(pUnkSite);
	RUN(mc->get_RefObject(&m_mcRef));
	RUN(AtlAdvise(mc,GetUnknown(),IID_IMontageContainerEvents,&m_pumpkinBar));
	return S_OK;
}
STDMETHODIMP ScrollingTransform::GetSite(REFIID riid, void **ppvSite) {
    METHTRACE(ScrollingTransform,GetSite);
	*ppvSite = 0;
	if(!m_mcRef)
		ERR(E_FAIL);
	CComPtr<IUnknown> mc;
	m_mcRef->get_Montage(&mc);
	if(!mc)
		ERR(E_FAIL);
	return mc->QueryInterface(riid,ppvSite);
}
// so as not to cause lots of extra work, don't report changes during GroupOps
// IMontageContainerEvents
STDMETHODIMP ScrollingTransform::Shutdown() {
    METHTRACE(ScrollingTransform,Shutdown);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::Resize(SIZE size) {
	return S_FALSE;
}
// um, IMCCTransform has redundant event? yeex
STDMETHODIMP ScrollingTransform::NewExtent(SIZEL ext) {
    METHTRACE(ScrollingTransform,NewExtent);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::Viewing(HDC hdc,LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(ScrollingTransform,Viewing);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::NewTransform(IMCCTransform *) {
    METHTRACE(ScrollingTransform,NewTransform);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::ScaleChanged() {
    METHTRACE(ScrollingTransform,ScaleChanged);
    return S_FALSE;
}
STDMETHODIMP ScrollingTransform::GroupOp(BOOL v) {
    METHTRACE(ScrollingTransform,GroupOp);
	if(v)
		m_locks++;
	else if(!--m_locks&&m_changed) {
		HRESULT hr = S_FALSE;
		if(m_changed&CHECK_BOUNDS)
			RUN(hr=checkBounds());
		if(hr==S_FALSE && m_changed&CHANGED_OFFSET)
			RUN(fireEvent(GetUnknown(),
				fireVoid<IMCCScrollingTransformEvents>(&IMCCScrollingTransformEvents::OffsetChanged)));
		if(m_changed&CHANGED_BOUNDS)
			RUN(fireEvent(GetUnknown(),fireVoid<IMCCScrollingTransformEvents>(&IMCCScrollingTransformEvents::BoundsChanged)));
		m_changed = 0;
	}
	return S_OK;
}
STDMETHODIMP ScrollingTransform::SiteDeleted(IMCCSite*) {
    METHTRACE(ScrollingTransform,SiteDeleted);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::LayerDeleted(IMCCLayer*) {
    METHTRACE(ScrollingTransform,LayerDeleted);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::Shifting(long oldState,long newState) {
    METHTRACE(ScrollingTransform,Shifting);
	return S_FALSE;
}
// IPersistStream
STDMETHODIMP ScrollingTransform::GetClassID(CLSID *pClassID) {
    METHTRACE(ScrollingTransform,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP ScrollingTransform::IsDirty(void) {
    METHTRACE(ScrollingTransform,IsDirty);
	return S_FALSE;
}
STDMETHODIMP ScrollingTransform::Load(IStream *pStm) {
    METHTRACE(ScrollingTransform,Load);
	return pStm->Read(&m_offset,sizeof(m_offset),0);
}
STDMETHODIMP ScrollingTransform::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(ScrollingTransform,Save);
	return pStm->Write(&m_offset,sizeof(m_offset),0);
}
STDMETHODIMP ScrollingTransform::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(ScrollingTransform,GetSizeMax);
	pcbSize->QuadPart = sizeof(m_offset);
	return S_OK;
}
