/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// defines class bounder, which keeps track of canvas bounds:
#include "bounds.h"

struct ScrollingTransform : CComObjectRoot,
	CComCoClass<ScrollingTransform,&CLSID_MCCScrollingTransform>,
	IMCCTransform,
	IMCCScrollingTransform,
	IMontageContainerEvents,
	IObjectWithSite,
	IPersistStream,
	IConnectionPointContainerImpl<ScrollingTransform>,
	IConnectionPointImpl<ScrollingTransform,&IID_IMCCScrollingTransformEvents>
{
	typedef IConnectionPointImpl<ScrollingTransform,&IID_IMCCScrollingTransformEvents> eventPoint;
	CComPtr<DIMontageContainerRef> m_mcRef;
	bounder m_bounds; // the canvas bounds tracker
	// properties
	POINTL m_offset;
	SIZEL m_extent;
	RECT m_area;
	// try not to fire too many stupid events
	enum {CHANGED_OFFSET=1,CHANGED_SCALE=2,CHANGED_BOUNDS=4,CHECK_BOUNDS=8};
	int m_locks,m_changed;
	DWORD m_pumpkinBar;
	// special coords during MC's IViewObject::Draw method (replaces m_area)
	bool m_viewing,m_canving;
	RECTL m_viewArea,m_canvView;
	// to not fire immediately
	bool m_doEvents, m_boundsChanged;
	ScrollingTransform();
	~ScrollingTransform();

	CComPtr<IMontageContainer> mc() {
		CComPtr<IUnknown> umc;
		m_mcRef->get_Montage(&umc);
		return CComQIPtr<IMontageContainer>(umc);
	}

REGISTER_GENERIC(L"ScrollingTransform")
BEGIN_COM_MAP(ScrollingTransform)
	COM_INTERFACE_ENTRY(IMCCTransform)
	COM_INTERFACE_ENTRY(IMCCScrollingTransform)
	COM_INTERFACE_ENTRY(IMontageContainerEvents)
	COM_INTERFACE_ENTRY(IObjectWithSite)
	COM_INTERFACE_ENTRY(IPersistStream)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(ScrollingTransform)
	CONNECTION_POINT_ENTRY(IID_IMCCScrollingTransformEvents)
END_CONNECTION_POINT_MAP()

    HRESULT changed(bool scale,bool offset);
	HRESULT placeall(); // sync all window rects with new param
	HRESULT checkBounds(); // see if we're showing stuff that's not in the canvas bounds
	HRESULT fireBoundsChanged();
	HRESULT c2ps(SIZEL,PSIZE,RECT&);
// IMCCTransform
	STDMETHODIMP put_TransformedArea(RECT *val);
	STDMETHODIMP get_TransformedArea(RECT *val);
	STDMETHODIMP put_Extent(SIZEL *val);
	STDMETHODIMP get_Extent(SIZEL *val);
	STDMETHODIMP Viewing(LPCRECTL lpcrOut, LPCRECTL lpcrCanv);
	STDMETHODIMP get_IsViewing(VARIANT_BOOL *ret);
	STDMETHODIMP C2WS(SIZEL siz, PSIZE psiz);
	STDMETHODIMP C2VS(SIZEL siz, PSIZE psiz);
	STDMETHODIMP W2CS(SIZE siz, PSIZEL psiz);
	STDMETHODIMP C2WP(POINTL pt, PPOINT ppt);
	STDMETHODIMP C2VP(POINTL pt, PPOINT ppt);
	STDMETHODIMP W2CP(POINT pt, PPOINTL ppt);
	STDMETHODIMP C2WR(LPCRECTL pcr, PRECT prc);
	STDMETHODIMP C2VR(LPCRECTL pcr, PRECT prc);
	STDMETHODIMP W2CR(LPCRECT pcr, PRECTL prc);
	STDMETHODIMP AddRect(LPCRECTL r);
	STDMETHODIMP RemoveRect(LPCRECTL r);
// IMCCScrollingTransform
	STDMETHODIMP put_Offset(POINTL *val);
	STDMETHODIMP get_Offset(POINTL *ret);
	STDMETHODIMP get_Bounds(RECTL *ret);
	STDMETHODIMP put_DoEvents(VARIANT_BOOL val);
// IObjectWithSite
	STDMETHODIMP SetSite(IUnknown *pUnkSite);
	STDMETHODIMP GetSite(REFIID riid, void **ppvSite);
// IMontageContainerEvents
	STDMETHODIMP Shutdown();
	STDMETHODIMP Resize(SIZE s);
	STDMETHODIMP NewExtent(SIZEL);
	STDMETHODIMP Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv);
	STDMETHODIMP NewTransform(IMCCTransform *);
	STDMETHODIMP ScaleChanged();
	STDMETHODIMP GroupOp(BOOL);
	STDMETHODIMP SiteDeleted(IMCCSite*);
	STDMETHODIMP LayerDeleted(IMCCLayer*);
	STDMETHODIMP Shifting(long oldState,long newState);
	STDMETHODIMP ShowArea(LPCRECTL r) { return S_FALSE; }
    STDMETHODIMP FocusChanged() { return S_FALSE; }
// IPersistStream
	STDMETHODIMP GetClassID(CLSID *pClassID);
	STDMETHODIMP IsDirty(void);
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
};

