/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// CtlSite.cpp: internal site stuff

#include "stdafx.h"
#include "Container.h"

Site::Site() :
	m_state(MCCS_HIDDEN),
	m_intent(MCCS_HIDDEN),
	m_clipMode(MCM_CONTENT),
	m_bRectValid(false),
	m_cont(0),
	m_pLayer(0),
	m_usewnd(0),
	m_peanutButter(0),
	m_amSettingExtent(false),
	m_autoSetExtent(false)
{
	CONSTRUCTORTRACE(Site);
}
Site::~Site() {
	DESTRUCTORTRACE(Site);
	_ASSERTE(!m_pControl);
	_ASSERTE(!m_pWindowless);
}

HRESULT Site::FinalConstruct() {
	METHTRACE(Site,FinalConstruct);
	RUN(CREATEA(m_connectionpoints,CLSID_MCCAllPointsContainer,(IMCCSite*)this));
	RUN(CComQIPtr<IMCCAllPointsContainer>(m_connectionpoints)->EnsurePoint(IID_IMCCSiteEvents,0));
	return S_OK;
}
void Site::FinalRelease() {
	METHTRACE(Site,FinalRelease);
	m_connectionpoints.Release(); // probably would happen automatically?
}
RECT Site::rect() {
	METHTRACE(Site,rect);
	return m_windRect;
}
RECT Site::clipRect() {
	if(m_clipMode==MCM_SITE_RECT)
		return m_windRect;
	if(m_cont->m_ownwindow) {
		switch(m_clipMode) {
		case MCM_FULL:
			return m_cont->rect();
		default: // no reason to assert on garbage here.
		case MCM_CONTENT:
			return m_cont->contentRect();
		}
	}
	else {
		// if we don't own the window m_rect is probably garbage, and there is no
		// distinction between content and full
		RECT rcClip;
		GetClientRect(m_cont->m_hWnd,&rcClip);
		return rcClip;
	}
}

DWORD Site::hitpoint(POINT pt) {
	METHTRACE(Site,hitpoint);
	RECT rc = rect();
	if(!PtInRect(&rc,pt))
		return HITRESULT_OUTSIDE;
	CComQIPtr<IViewObjectEx> pEx = m_pControl;
	if(!pEx)
		return HITRESULT_HIT;
	DWORD dwResult;
	if(SUCCEEDED(pEx->QueryHitPoint(DVASPECT_CONTENT,&rc,pt,50,&dwResult))) // allow half a millimeter slack
		return dwResult;
	return HITRESULT_HIT;
}
DWORD Site::hitrect(LPCRECT lpcr) {
	METHTRACE(Site,hitrect);
	RECT rc = rect(),throwaway;
	if(!IntersectRect(&throwaway,&rc,lpcr))
		return HITRESULT_OUTSIDE;
	CComQIPtr<IViewObjectEx> pEx = m_pControl;
	if(!pEx)
		return HITRESULT_HIT;
	DWORD dwResult;
	if(SUCCEEDED(pEx->QueryHitRect(DVASPECT_CONTENT,&rc,lpcr,50,&dwResult)))  // allow half a millimeter slack
		return dwResult;
	return HITRESULT_HIT;
}

void Site::moveEvent(LPCRECT winRect) {
	METHTRACE(Site,moveEvent);
	CComQIPtr<IConnectionPointContainer> cpc = m_connectionpoints;
	CComPtr<IConnectionPoint> cp;
	if(SUCCEEDED(cpc->FindConnectionPoint(IID_IMCCSiteEvents,&cp))) {
		CComPtr<IEnumConnections> ec;
		RUN(cp->EnumConnections(&ec));
		CONNECTDATA cd;
		while(ec->Next(1,&cd,0)==S_OK) {
			CComQIPtr<IMCCSiteEvents> se = cd.pUnk;
			se->Moving(winRect);
			cd.pUnk->Release();
		}
	}
}
void Site::stateEvent(long last, long next) {
	METHTRACE(Site,stateEvent);
	RUN(fireEvent(GetUnknown(),fireTwoParams<IMCCSiteEvents,long,long>(&IMCCSiteEvents::Shifting,last,next)));
}
/////////////////////////////////////////////////////////////////////////////
//

STDMETHODIMP Site::InterfaceSupportsErrorInfo(REFIID riid)
{
	METHTRACE(Site,InterfaceSupportsErrorInfo);
	/*
	static const IID* arr[] =
	{
	} ;

	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	*/
	return S_FALSE;
}
