/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef SITE_H
#define SITE_H

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// Site
struct Container;
struct Layer;

// so that compiler can distinguish between IDispatch and DIMCCSite
struct dispnub : IDispatch {
	CComPtr<IDispatch> m_ambiance;
// IDispatch
    STDMETHODIMP GetTypeInfoCount(UINT *pctinfo);
    STDMETHODIMP GetTypeInfo(UINT iTInfo,LCID lcid,ITypeInfo **ppTInfo);
    STDMETHODIMP GetIDsOfNames(REFIID riid,LPOLESTR *rgszNames,UINT cNames,LCID lcid,
         DISPID *rgDispId);
    STDMETHODIMP Invoke(DISPID dispIdMember,REFIID riid,LCID lcid,WORD wFlags,DISPPARAMS *pDispParams,
         VARIANT *pVarResult,EXCEPINFO *pExcepInfo,UINT *puArgErr);
};
struct ATL_NO_VTABLE Site :
	//public ISupportErrorInfo,
	public CComObjectRoot,
	public dispnub,

	public IMCCSite,
	public IDispatchImpl<DIMCCSite>,

	public IOleInPlaceSiteWindowless,
	public IOleClientSite,
	public IOleControlSite,

	public IAdviseSink
{
	Container *m_cont; // where I am a site
	CComPtr<IMoniker> m_mkPersistor;
	// the only interface we should strictly require is IViewObject
	CComQIPtr<IViewObject> m_pControl; // the contained control
	CComQIPtr<IOleInPlaceObjectWindowless> m_pWindowless; // if a windowless control
	DWORD m_peanutButter;
	CComPtr<IUnknown> m_connectionpoints;

	Layer *m_pLayer; // what drawing level this appears in
	long m_state, m_intent; // OLE state, and intended state if something is blocking (eg CC state is lower)
	enum MCC_SITE_CLIP_MODE m_clipMode; // way to calculate clip mode for contained
	RECT m_windRect; // bounds & clipping of this control
	RECTL m_virtRect; // bounds rect in canvas coords
	bool m_bRectValid, // has it actually been set?
        m_autoSetExtent;
	HWND m_usewnd; // if activated into a particular window, that window.
	bool m_amSettingExtent; // TEMPORARY hack - ActiveX controls will call OnPosRectChange

	// gear-shifting methods.

	// changestate is the workhorse; it pushes site state from m_state to dest.
	// don't call this directly! it doesn't fire IMCCSiteEvents, and doesn't release the sinks on deletion.
	HRESULT changestate(long dest);
	// this does the work of put_Intent
	// the current state is the minimum of the site state and the CC state
	// the destination state cannot be further than the CC state
	HRESULT approachstate(long intent);

	// the last step of deletion is to unadvise all sinks, and it must
	// happen after the stateEvent
	HRESULT releaseSinks();

	// see whether extent has changed and if so fire move requests
	void checkSizeChange();
	// post this message to do the same but not now (asynchronous)
	void postCheckSizeMessage();

	RECT clipRect();
	RECT rect(); // in window coords
	HWND window(); // of active control
	DWORD hitpoint(POINT pt); // wraps iviewobjectex::, if any
	DWORD hitrect(LPCRECT lpcr);
	void moveEvent(LPCRECT winRect); // yell at sinks
	void stateEvent(long last,long next); //
	HRESULT activeZ(); // run after windowed activation to make sure window's in right Z

	HRESULT processMnemonics(bool add);

	Site();
	~Site();
	DECLARE_PROTECT_FINAL_CONSTRUCT()
	HRESULT FinalConstruct();
	void FinalRelease();

BEGIN_COM_MAP(Site)
	COM_INTERFACE_ENTRY2(IDispatch,dispnub)
	//COM_INTERFACE_ENTRY(ISupportErrorInfo)
	// custom automation interface:
	COM_INTERFACE_ENTRY(IMCCSite)
	COM_INTERFACE_ENTRY(DIMCCSite)
	// control site interfaces:
	COM_INTERFACE_ENTRY(IOleClientSite)
	COM_INTERFACE_ENTRY(IOleInPlaceSiteWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceSiteEx) // in IOleInPlaceSiteWindowless
	COM_INTERFACE_ENTRY(IOleInPlaceSite) // in IOleInPlaceSiteEx
    COM_INTERFACE_ENTRY(IOleWindow) // in IOleInPlaceSite
	COM_INTERFACE_ENTRY(IOleControlSite)
	COM_INTERFACE_ENTRY(IAdviseSink)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IConnectionPointContainer,m_connectionpoints.p)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMCCAllPointsContainer,m_connectionpoints.p)
END_COM_MAP()

// IMCCSite
	STDMETHODIMP GetContainer(REFIID riid, void **ret);
	STDMETHODIMP get_RelMoniker(IMoniker **ret);
	STDMETHODIMP put_RelMoniker(IMoniker *val);
	STDMETHODIMP GetControl(REFIID riid, void **ret);
	STDMETHODIMP put_Control(IUnknown *parm);
	STDMETHODIMP get_Layer(IMCCLayer **ret);
	STDMETHODIMP put_Layer(IMCCLayer *parm);
	STDMETHODIMP get_Ambiance(IDispatch **ret);
	STDMETHODIMP put_Ambiance(IDispatch *val);
	STDMETHODIMP put_ClipMode(enum MCC_SITE_CLIP_MODE val);
	STDMETHODIMP get_ClipMode(enum MCC_SITE_CLIP_MODE *ret);
    STDMETHODIMP put_AutoSetExtent(VARIANT_BOOL val);
    STDMETHODIMP get_AutoSetExtent(VARIANT_BOOL *ret);
	STDMETHODIMP get_Z(long *ret);
	STDMETHODIMP get_Rect(LPRECT lprc);
	STDMETHODIMP put_Rect(LPCRECT lprc);
	STDMETHODIMP get_VirtRect(LPRECTL pr);
	STDMETHODIMP put_VirtRect(LPCRECTL r);
	STDMETHODIMP RegisterCommand(int cmd);
	STDMETHODIMP ForgetCommand(int cmd);
	STDMETHODIMP PutBefore(IMCCSite *other);
	STDMETHODIMP PutBehind(IMCCSite *other);
	STDMETHODIMP get_Intent(long *ret);
	STDMETHODIMP put_Intent(long val);
	STDMETHODIMP SayVerb(LONG iVerb, LPMSG pmsg);
	STDMETHODIMP UseWindow(HWND hWnd);
	STDMETHODIMP UpdateExtent(VARIANT_BOOL fromRect);
    STDMETHODIMP SetTimer(UINT id,UINT elapse);
    STDMETHODIMP KillTimer(UINT id);
// DIMCCSite
	STDMETHODIMP get_Container(DIMontageContainer **ret);
	STDMETHODIMP get_RelMoniker(IUnknown **ret);
	STDMETHODIMP put_RelMoniker(IUnknown *val);
	STDMETHODIMP get_Control(IUnknown **ret);
	STDMETHODIMP get_Layer(DIMCCLayer **ret);
	STDMETHODIMP put_Layer(DIMCCLayer *parm);
	STDMETHODIMP get_Left(long *ret);
	STDMETHODIMP get_Top(long *ret);
	STDMETHODIMP get_Right(long *ret);
	STDMETHODIMP get_Bottom(long *ret);
	STDMETHODIMP SetPos(long left, long top, long right, long bottom);
	STDMETHODIMP get_VirtLeft(long *ret);
	STDMETHODIMP get_VirtTop(long *ret);
	STDMETHODIMP get_VirtRight(long *ret);
	STDMETHODIMP get_VirtBottom(long *ret);
	STDMETHODIMP SetVirtPos(long left, long top, long right, long bottom);
	STDMETHODIMP PutBefore(DIMCCSite *other);
	STDMETHODIMP PutBehind(DIMCCSite *other);
// ISupportsErrorInfo
	STDMETHODIMP InterfaceSupportsErrorInfo(REFIID riid);
// IOleClientSite
    STDMETHODIMP SaveObject(void);
    STDMETHODIMP GetMoniker(
         DWORD dwAssign,
         DWORD dwWhichMoniker,
         IMoniker **ppmk);
    STDMETHODIMP GetContainer(
         IOleContainer **ppContainer);
    STDMETHODIMP ShowObject(void);
    STDMETHODIMP OnShowWindow(
         BOOL fShow);
    STDMETHODIMP RequestNewObjectLayout(void);

// IOleInPlaceSiteWindowless
	//(IOleWindow)
    STDMETHODIMP GetWindow(HWND *phwnd);
    STDMETHODIMP ContextSensitiveHelp(BOOL fEnterMode);

	//(IOleInPlaceSite)
    STDMETHODIMP CanInPlaceActivate(void);
    STDMETHODIMP OnInPlaceActivate(void);
    STDMETHODIMP OnUIActivate(void);
    STDMETHODIMP GetWindowContext(
         IOleInPlaceFrame **ppFrame,
         IOleInPlaceUIWindow **ppDoc,
         LPRECT lprcPosRect,
         LPRECT lprcClipRect,
         LPOLEINPLACEFRAMEINFO lpFrameInfo);
    STDMETHODIMP Scroll(
         SIZE scrollExtant);
    STDMETHODIMP OnUIDeactivate(
         BOOL fUndoable);
    STDMETHODIMP OnInPlaceDeactivate(void);
    STDMETHODIMP DiscardUndoState(void);
    STDMETHODIMP DeactivateAndUndo(void);
    STDMETHODIMP OnPosRectChange(
         LPCRECT lprcPosRect);

	//(IOleInPlaceSiteEx)
    STDMETHODIMP OnInPlaceActivateEx(
         BOOL *pfNoRedraw,
         DWORD dwFlags);
    STDMETHODIMP OnInPlaceDeactivateEx(
         BOOL fNoRedraw);
    STDMETHODIMP RequestUIActivate(void);

	//(IOleInPlaceSiteWindowless)
    STDMETHODIMP CanWindowlessActivate(void);
    STDMETHODIMP GetCapture(void);
    STDMETHODIMP SetCapture(
         BOOL fCapture);
    STDMETHODIMP GetFocus(void);
    STDMETHODIMP SetFocus(
         BOOL fFocus);
    STDMETHODIMP GetDC(
         LPCRECT pRect,
         DWORD grfFlags,
         HDC *phDC);
    STDMETHODIMP ReleaseDC(
         HDC hDC);
    STDMETHODIMP InvalidateRect(
         LPCRECT pRect,
         BOOL fErase);
    STDMETHODIMP InvalidateRgn(
         HRGN hRGN,
         BOOL fErase);
    STDMETHODIMP ScrollRect(
         INT dx,
         INT dy,
         LPCRECT pRectScroll,
         LPCRECT pRectClip);
    STDMETHODIMP AdjustRect(
         LPRECT prc);
    STDMETHODIMP OnDefWindowMessage(
         UINT msg,
         WPARAM wParam,
         LPARAM lParam,
         LRESULT *plResult);

// IOleControlSite
    STDMETHODIMP OnControlInfoChanged(void);
    STDMETHODIMP LockInPlaceActive(
         BOOL fLock);
    STDMETHODIMP GetExtendedControl(
         IDispatch **ppDisp);
    STDMETHODIMP TransformCoords(
         POINTL *pPtlHimetric,
         POINTF *pPtfContainer,
         DWORD dwFlags);
    STDMETHODIMP TranslateAccelerator(
         MSG *pMsg,
         DWORD grfModifiers);
    STDMETHODIMP OnFocus(
         BOOL fGotFocus);
    STDMETHODIMP ShowPropertyFrame(void);

// IAdviseSink
    void STDMETHODCALLTYPE OnDataChange(
         FORMATETC *pFormatetc,
         STGMEDIUM *pStgmed);

    void STDMETHODCALLTYPE OnViewChange(
         DWORD dwAspect,
         LONG lindex);

    void STDMETHODCALLTYPE OnRename(
         IMoniker *pmk);

    void STDMETHODCALLTYPE OnSave(void);

    void STDMETHODCALLTYPE OnClose(void);
};

// the truly useful Site has its IUnknown installed.  Since templates
// block constructors, reinstall constructor.
typedef CComObject<Site> site;

inline HRESULT createSite(Container *mc, site **ret) {
    FUNTRACE(createSite);
	SRUN(site::CreateInstance(ret));
	(*ret)->m_cont = mc;
	return S_OK;
};

typedef list<site *> sitelist;

class hitpoint {
	POINT m_pt;
	long m_lowstate;
public:
	hitpoint(POINT pt,long lowstate) : m_pt(pt), m_lowstate(lowstate) {}
	BOOL operator()(site *s) {
		return s->m_state>=m_lowstate && s->hitpoint(m_pt);
	}
};

class hitrect {
	RECT m_rc;
	long m_lowstate;
public:
	hitrect(RECT rc,long lowstate) : m_rc(rc), m_lowstate(lowstate) {}
	BOOL operator()(site *s) {
		return s->m_state>=m_lowstate && s->hitrect(&m_rc);
	}
};

#define CHECK_DEL if(m_state==MCCS_DELETED) ERR(MONTAGE_E_DELETED)

#endif SITE_H
