/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "SitePersistor.h"

SitePersistor::SitePersistor() : m_cache(0) {
	CONSTRUCTORTRACE(SitePersistor);
}
SitePersistor::~SitePersistor() {
	DESTRUCTORTRACE(SitePersistor);
}

HRESULT SitePersistor::fillCache(MCCSITEPERSISTORDATA &spd) {
	METHTRACE(SitePersistor,fillCache);
	if(!m_subject)
		ERR(E_PENDING);
	spd.z = 0;
	m_subject->get_Z(&spd.z); // will fail if not inserted.
	if(SUCCEEDED(m_subject->get_Rect(&spd.pos.rc)))
		spd.pos.postype = SUCCEEDED(m_subject->get_VirtRect(&spd.pos.rcl))?
			MCC_POS_CANVASRECT:MCC_POS_WINDOWRECT;
	RUN(m_subject->get_Intent(&spd.state));
	return S_OK;
}
HRESULT SitePersistor::getSelfMk(IGroup *dest,IMoniker **ret) {
	METHTRACE(SitePersistor,getSelfMk);
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	if(m_subject) {
		CComPtr<IMontageContainer> mc;
		RUNRET(m_subject->GetContainer(riidret(&mc)));
		CComPtr<IGroup> g;
		RUN(mc->GetMainGroup(riidret(&g)));
		CComPtr<IMoniker> mk;
		RUN(m_subject->get_RelMoniker(&mk));
		_ASSERTE(mk); // any site with a persistor must have a name
		RUN(dest->Paste(bctx,GP_SrcIsCont,g,mk,IID_NULL,0,ret));
		_ASSERTE(*ret);
		return S_OK;
	}
	else {
		RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkSelf,IID_NULL,0,ret));
		return S_OK;
	}
}

//IMCCSitePersistor
STDMETHODIMP SitePersistor::Initialize(IMoniker *mkMontage,IMoniker *mkLayer,
		IMoniker *mkSiteOrdering,IMoniker *mkControl,IMoniker *mkAmbiance,IOrderedItemList *ilist) {
	METHTRACE(SitePersistor,Initialize);
	if(!m_group)
		ERR(E_PENDING);
	if(m_mkMontage)
		ERR(E_UNEXPECTED);
	RUN(m_group->AddStgRef(m_mkMontage = mkMontage));
	RUN(m_group->AddStgRef(m_mkLayer = mkLayer));
	RUN(m_group->AddStgRef(m_mkSiteOrdering = mkSiteOrdering));
	RUN(m_group->AddStgRef(m_mkControl = mkControl));
	RUN(m_group->AddStgRef(m_mkAmbiance = mkAmbiance));
	if(ilist)
		m_initlist = ilist;
	else {
		RUN(CREATE(m_initlist,CLSID_MCCInitList));
	}
	RUN(CComQIPtr<IGroupItem>(m_initlist)->Connect(m_group));
	return S_OK;
}
STDMETHODIMP SitePersistor::get_InitList(IOrderedItemList **ret) {
	METHTRACE(SitePersistor,get_InitList);
	m_initlist.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP SitePersistor::put_RelMoniker(IMoniker *mk) {
	METHTRACE(SitePersistor,put_RelMoniker);
	if(m_subject)
		ERR(E_FAIL);
	m_mkSelf = mk;
	return S_OK;
}
STDMETHODIMP SitePersistor::get_RelMoniker(IMoniker **ret) {
	METHTRACE(SitePersistor,get_RelMoniker);
	if(m_subject)
		return m_subject->get_RelMoniker(ret);
	if(*ret = m_mkSelf)
		(*ret)->AddRef();
	return S_OK;
}
STDMETHODIMP SitePersistor::put_Ordering(IMCCSiteOrdering *val) {
	METHTRACE(SitePersistor,put_Ordering);
	m_ordering = val;
	return S_OK;
}

STDMETHODIMP SitePersistor::get_Data(MCCSITEPERSISTORDATA *ret) {
	METHTRACE(SitePersistor,get_Data);
	if(!ret)
		ERR(E_POINTER);
	if(!m_cache)
		if(m_subject) {
			HRESULT hr = fillCache(*ret);
			if(ret->state==MCCS_HIDDEN) {
				CComPtr<IPersist> ps;
				RUN(m_subject->GetControl(riidret(&ps)));
				CLSID classid;
				RUN(ps->GetClassID(&classid));
			}
			return hr;
		}
		else
			ERR(E_FAIL);
	*ret = *m_cache;
	return S_OK;
}
STDMETHODIMP SitePersistor::put_Data(MCCSITEPERSISTORDATA *val) {
	METHTRACE(SitePersistor,put_Data);
	if(!val) {
		// revert to the subject's data.
		if(!m_subject)
			ERR(E_POINTER);
		if(m_cache)
			delete m_cache;
		m_cache = 0;
		return S_OK;
	}
	if(!m_cache)
		m_cache = new MCCSITEPERSISTORDATA;
	*m_cache = *val;
	return S_OK;
}
// DIMCCSitePersistor
STDMETHODIMP SitePersistor::Initialize(IUnknown *mkMontage,
	IUnknown *mkLayer,
	IUnknown *mkSiteOrdering,
	IUnknown *mkControl,
	IUnknown *mkAmbiance,
	DIOrderedItemList *initlist) {
    METHTRACE(SitePersistor,Initialize);
	CComPtr<IMoniker> m2,l2,so2,c2,a2;
	if(mkMontage)
		RUNRET(mkMontage->QueryInterface(riidret(&m2)));
	if(mkLayer)
		RUNRET(mkLayer->QueryInterface(riidret(&l2)));
	if(mkSiteOrdering)
		RUNRET(mkSiteOrdering->QueryInterface(riidret(&so2)));
	if(mkControl)
		RUNRET(mkControl->QueryInterface(riidret(&c2)));
	if(mkAmbiance)
		RUNRET(mkAmbiance->QueryInterface(riidret(&a2)));
	CComPtr<IOrderedItemList> oil2;
	if(initlist)
		RUNRET(initlist->QueryInterface(riidret(&oil2)));
	return Initialize(m2,l2,so2,c2,a2,oil2);
}
STDMETHODIMP SitePersistor::get_InitList(DIOrderedItemList **ret) {
    METHTRACE(SitePersistor,get_InitList);
	CComPtr<IOrderedItemList> oil;
	RUNRET(get_InitList(&oil));
	return oil.QueryInterface(ret);
}
STDMETHODIMP SitePersistor::put_RelMoniker(IUnknown *mk) {
    METHTRACE(SitePersistor,put_RelMoniker);
	CComPtr<IMoniker> m2;
	if(mk)
		RUNRET(mk->QueryInterface(riidret(&m2)));
	return put_RelMoniker(m2);
}
STDMETHODIMP SitePersistor::get_RelMoniker(IUnknown **ret) {
    METHTRACE(SitePersistor,get_RelMoniker);
	return get_RelMoniker((IMoniker**)ret);
}
STDMETHODIMP SitePersistor::get_ControlMk(IMoniker **ret) {
	m_mkControl.CopyTo(ret);
	return S_OK;
}

//IPersistor
STDMETHODIMP SitePersistor::SetSubject(VARIANT_BOOL realized,IUnknown *pUnkSite) {
	METHTRACE(SitePersistor,SetSubject);
	// like the other custom persistors which need to bind a moniker to create their subject,
	// this persistor doesn't support "realize this" mode - if it has a subject, the persistor
	// considers itself realized and will return S_FALSE (won't connect a live object)
	m_subject = pUnkSite;
	return S_OK;
}
STDMETHODIMP SitePersistor::get_Subject(IUnknown **ret) {
	METHTRACE(SitePersistor,GetSubject);
	m_subject.CopyTo((IMCCSite**)ret);
	return S_OK;
}
// IGroupItem
STDMETHODIMP SitePersistor::Connect(IGroup *group) {
	METHTRACE(SitePersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	if(m_initlist)
		RUN(CComQIPtr<IGroupItem>(m_initlist)->Connect(group));
	return S_OK;
}
STDMETHODIMP SitePersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(SitePersistor,Copy);

	// translate
	CComPtr<IMoniker> mkMon2,mkLay2,mkOrd2,mkCon2,mkInit2,mkAmb2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkMontage,IID_NULL,0,&mkMon2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkLayer,IID_NULL,0,&mkLay2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkSiteOrdering,IID_NULL,0,&mkOrd2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkControl,IID_NULL,0,&mkCon2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkAmbiance,IID_NULL,0,&mkAmb2));

	// init
	CComPtr<IMCCSitePersistor> sp2;
	RUN(CREATE(sp2,CLSID_MCCSitePersistor));
	CComQIPtr<IGroupItem> gi = m_initlist;
	if(!gi)
		ERR(E_UNEXPECTED);
	CComPtr<IOrderedItemList> oil2;
	RUN(gi->Copy(bctx,dest,0,riidret(&oil2)));
	RUN(CComQIPtr<IGroupItem>(sp2)->Connect(dest));
	RUN(sp2->Initialize(mkMon2,mkLay2,mkOrd2,mkCon2,mkAmb2,oil2));
	RUN(sp2->put_Ordering(m_ordering));
	// no need to copy the self moniker if you've got the subject,
	// because site persistor will always defer to the site->relmk
	if(m_subject)
		RUN(CComQIPtr<DIPersistor>(sp2)->SetSubject(m_subject!=0,m_subject));
	else {
		CComPtr<IMoniker> mkSelf2;
		RUN(getSelfMk(dest,&mkSelf2));
		RUN(sp2->put_RelMoniker(mkSelf2));
	}
	// if site not realized OR there is overriding data, copy it.
	if(m_cache)
		RUN(sp2->put_Data(m_cache));
	return sp2->QueryInterface(riid,ret);
}
STDMETHODIMP SitePersistor::Realize(IBindCtx *bctx) {
	METHTRACE(SitePersistor,Realize);
	if(m_subject)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	if(!m_cache || !m_group)
		ERR(E_PENDING);
	// some monikers are mandatory
	if(!m_mkMontage || !m_mkControl || !m_mkLayer)
		ERR(E_UNEXPECTED);
#ifdef TRACE_SITEREALIZE
	{
		LPOLESTR s=0;
		if(m_mkSelf)
			RUN(m_mkSelf->GetDisplayName(bctx,0,&s))
		MTrace(rZ)("***realizing site %S\n",s);
		CoTaskMemFree(s);
		RUN(m_mkControl->GetDisplayName(bctx,0,&s));
		MTrace(rZ)("with control %S\n",s);
		CoTaskMemFree(s);
		RUN(m_mkLayer->GetDisplayName(bctx,0,&s));
		MTrace(rZ)("to layer %S\n",s);
		CoTaskMemFree(s);
	}
#endif
	// get Montage
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	CComPtr<DIPersistor> mp;
	RUN(m_mkMontage->BindToObject(bctx,mkG,riidret(&mp)));
	CComPtr<IUnknown> umc;
	RUN(mp->get_Subject(&umc));
	CComPtr<IMontageContainer> mc;
	RUN(umc.QueryInterface(&mc));

	// don't create excessive events with the cascading realizes
	// that are about to happen
	RUN(groupOp(mc,true));

	// create site
	RUN(mc->new_Site(&m_subject));

	// we're in main group if we realize, so this is right mk
	_ASSERTE(m_mkSelf);
	RUN(m_subject->put_RelMoniker(m_mkSelf));

	// put site in layer
	CComPtr<IMCCLayer> l;
	RUN(m_mkLayer->BindToObject(bctx,mkG,riidret(&l)));
	RUN(m_subject->put_Layer(l));

	// clip mode
	RUN(m_subject->put_ClipMode(m_holdClipMode));

	// ambiance
	// should probably be saving the IID as well
	if(m_mkAmbiance) {
		CComPtr<IDispatch> amb;
		RUN(m_mkAmbiance->BindToObject(bctx,mkG,riidret(&amb)));
		RUN(m_subject->put_Ambiance(amb));
	}

	// place control.
	CComPtr<IUnknown> ctl;
 	RUN(m_mkControl->BindToObject(bctx,mkG,riidret(&ctl)));
	// hack: make sure control is in storage of site's group
	CComPtr<IStorage> stg;
	if(SUCCEEDED(m_mkControl->BindToStorage(bctx,mkG,riidret(&stg))))
		if(CComQIPtr<IPersistStorage> ps = ctl)
			RUN(ps->SaveCompleted(stg));
	RUN(m_subject->put_Control(ctl));

	// install sinks, owners, & perform "other" initialization such as OleRun
	CComQIPtr<IInitializeOther> io = m_initlist;
	QISET(io);
	RUN(io->InitializeOther(m_subject));

	// z-order
	if(!m_ordering)
		RUN(m_mkSiteOrdering->BindToObject(bctx,mkG,riidret(&m_ordering)));
	RUN(m_ordering->Place(m_cache->z,m_subject));
#ifdef _DEBUG
	CLSID classid;
	{
	// so you know what type of control this site has in the labyrinth below
	CComPtr<IUnknown> u;
	RUN(m_subject->GetControl(riidret(&u)));
	RUN(getObjectCLSID(u,&classid));
	}
#endif

	// send rect, state out to sinks (default sink will catch if nothing else has been installed)
	if(m_cache->pos.postype!=MCC_POS_NONE)
		RUN(fireSiteEvent(m_subject,fireMove(&m_cache->pos)));
	RUN(fireSiteEvent(m_subject,fireIntend(m_cache->state)));
	delete m_cache; m_cache = 0; // revert to depending on the subject for info
	RUN(groupOp(mc,false));
	return S_OK;
}
STDMETHODIMP SitePersistor::Remove() {
	METHTRACE(SitePersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkMontage));
	RUN(m_group->ReleaseStgRef(m_mkLayer));
	RUN(m_group->ReleaseStgRef(m_mkSiteOrdering));
	RUN(m_group->ReleaseStgRef(m_mkControl));
	RUN(m_group->ReleaseStgRef(m_mkAmbiance));
	return CComQIPtr<IGroupItem>(m_initlist)->Remove();
}
// IPersistStream
STDMETHODIMP SitePersistor::GetClassID(CLSID *pClassID) {
	METHTRACE(SitePersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP SitePersistor::IsDirty(void) {
	METHTRACE(SitePersistor,IsDirty);
	return S_FALSE;
}
STDMETHODIMP SitePersistor::Load(IStream *pStm) {
	METHTRACE(SitePersistor,Load);
	if(m_cache||m_subject)
		ERR(CO_E_ALREADYINITIALIZED);
	if(!m_group)
		ERR(E_PENDING);
	m_cache = new MCCSITEPERSISTORDATA;
	RUN(pStm->Read(m_cache,sizeof(MCCSITEPERSISTORDATA),0));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkSelf)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkMontage)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkLayer)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkSiteOrdering)));
	{
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		CComPtr<IMoniker> mkG;
		RUN(FoolishCreatePointerMoniker(m_group,&mkG));
		RUN(m_mkSiteOrdering->BindToObject(bctx,mkG,riidret(&m_ordering)));
	}
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkControl)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkAmbiance)));
	RUN(CREATE(m_initlist,CLSID_MCCInitList));
	RUN(CComQIPtr<IGroupItem>(m_initlist)->Connect(m_group));
	RUN(CComQIPtr<IPersistStream>(m_initlist)->Load(pStm));
	RUN(pStm->Read(&m_holdClipMode,sizeof(enum MCC_SITE_CLIP_MODE),0));
	return S_OK;
}
STDMETHODIMP SitePersistor::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(SitePersistor,Save);
	if(m_subject) { // realized (not necessarily from a load)
		// update mkSelf from object.  This is a strange operation, because normally
		// non-persistent objects such as sites will not hold monikers.  But since the
		// moniker is always relative to the main group, it is OK here.  But it does have
		// to be translated every time.

		m_mkSelf.Release();
		RUNRET(getSelfMk(m_group,&m_mkSelf));
		_ASSERTE(m_mkSelf);
		RUN(m_subject->get_ClipMode(&m_holdClipMode));
	}
	if(m_cache) {
		RUN(pStm->Write(m_cache,sizeof(MCCSITEPERSISTORDATA),0));
		if(m_cache->state==MCCS_HIDDEN&&m_subject) {
			CComPtr<IPersist> ps;
			RUN(m_subject->GetControl(riidret(&ps)));
			CLSID classid;
			RUN(ps->GetClassID(&classid));
			long tent;
			RUN(m_subject->get_Intent(&tent));
		}
	}
	else {
		MCCSITEPERSISTORDATA msp;
		RUN(fillCache(msp)); // will fail if no subject
		RUN(pStm->Write(&msp,sizeof(MCCSITEPERSISTORDATA),0));
		if(msp.state==MCCS_HIDDEN) {
			CComPtr<IPersist> ps;
			RUN(m_subject->GetControl(riidret(&ps)));
			CLSID classid;
			RUN(ps->GetClassID(&classid));
		}
	}
	RUN(FixedOleSaveToStream(m_mkSelf,pStm));
	RUN(FixedOleSaveToStream(m_mkMontage,pStm));
	RUN(FixedOleSaveToStream(m_mkLayer,pStm));
	RUN(FixedOleSaveToStream(m_mkSiteOrdering,pStm));
	RUN(FixedOleSaveToStream(m_mkControl,pStm));
	RUN(FixedOleSaveToStream(m_mkAmbiance,pStm));
	RUN(CComQIPtr<IPersistStream>(m_initlist)->Save(pStm,fClearDirty));
	RUN(pStm->Write(&m_holdClipMode,sizeof(enum MCC_SITE_CLIP_MODE),0));
	return S_OK;
}
STDMETHODIMP SitePersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(SitePersistor,GetSizeMax);
	CComObjectStack<countStream> fake;
	RUN(Save(&fake,false));
	*pcbSize = fake.m_pos;
	return S_OK;
}
