/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "SiteSet.h"

SiteSet::SiteSet() : m_hermit(0) {
	CONSTRUCTORTRACE(SiteSet);
}
SiteSet::~SiteSet() {
	DESTRUCTORTRACE(SiteSet);
}
// DIMCCSiteSet
STDMETHODIMP SiteSet::Initialize(IUnknown *mkMC) {
    METHTRACE(SiteSet,Initialize);
	if(!mkMC)
		ERR(E_INVALIDARG); // no point to using this class without that!
	if(!m_group)
		ERR(E_PENDING);
	RUNRET(mkMC->QueryInterface(riidret(&m_mkMontage)));
	RUN(m_group->AddStgRef(m_mkMontage));
	// oddness: nothing happens here because this is a persistor-style object.
	// mk gets bound and advise happens in Realize
	return S_OK;
}
// IMontageContainerEvents
STDMETHODIMP SiteSet::Shutdown() {
    METHTRACE(SiteSet,Shutdown);
	return S_OK;
}
STDMETHODIMP SiteSet::Resize(SIZE size) {
    METHTRACE(SiteSet,Resize);
	return S_FALSE;
}
STDMETHODIMP SiteSet::NewExtent(SIZEL ext) {
    METHTRACE(SiteSet,NewExtent);
	return S_FALSE;
}
STDMETHODIMP SiteSet::Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(SiteSet,Viewing);
	return S_FALSE;
}
STDMETHODIMP SiteSet::NewTransform(IMCCTransform *) {
    METHTRACE(SiteSet,NewTransform);
	return S_FALSE;
}
STDMETHODIMP SiteSet::ScaleChanged() {
    METHTRACE(SiteSet,ScaleChanged);
    return S_FALSE;
}
STDMETHODIMP SiteSet::GroupOp(BOOL start) {
    METHTRACE(SiteSet,GroupOp);
	return S_FALSE;
}
STDMETHODIMP SiteSet::SiteDeleted(IMCCSite *val) {
    METHTRACE(SiteSet,SiteDeleted);
	CComPtr<IMoniker> mkSite;
	LOOK(val->get_RelMoniker(&mkSite));
	if(mkSite)
		LOOK(CComQIPtr<IItemSet>(m_aggrSet)->Remove(mkSite)); // may not even be in set
	return S_OK;
}
STDMETHODIMP SiteSet::LayerDeleted(IMCCLayer *val) {
    METHTRACE(SiteSet,LayerDeleted);
	return S_FALSE;
}
STDMETHODIMP SiteSet::Shifting(long oldState,long newState) {
    METHTRACE(SiteSet,Shifting);
	return S_FALSE;
}
// IGroupItem
STDMETHODIMP SiteSet::Connect(IGroup *group) {
    METHTRACE(SiteSet,Connect);
	m_group = group;
	RUN(CComQIPtr<IGroupItem>(m_aggrSet)->Connect(group));
	return S_OK;
}
STDMETHODIMP SiteSet::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
    METHTRACE(SiteSet,Copy);
	if(aggr)
		return CLASS_E_NOAGGREGATION; // aqui no mas
	// copy my ItemSet into a new SiteSet's m_aggrSet, telling it about that new one's unknown
	typedef CComObject<SiteSet> full;
	full *ss2 = new full;
	CComPtr<IUnknown> aggr2 = ss2->GetUnknown(); // vital refcount: if QI below fails or isn't hit, ss2 will be destroyed
	RUN(CComQIPtr<IGroupItem>(m_aggrSet)->Copy(bctx,dest,aggr2,riidret(&ss2->m_aggrSet)));
	RUNRET(aggr2->QueryInterface(riid,ret));
	RUN(CComQIPtr<IGroupItem>(aggr2)->Connect(dest));
	CComPtr<IMoniker> mkMontage2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkMontage,IID_NULL,0,&mkMontage2));
	RUN(CComQIPtr<DIMCCSiteSet>(aggr2)->Initialize(mkMontage2));
	return S_OK;
}
STDMETHODIMP SiteSet::Realize(IBindCtx *bctx) {
    METHTRACE(SiteSet,Realize);
	if(!m_group)
		ERR(E_PENDING);
	if(m_hermit)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	CComPtr<IMoniker> mkG;
	RUN(FoolishCreatePointerMoniker(m_group,&mkG));
	CComPtr<IMontageContainer> mc;
	RUN(m_mkMontage->BindToObject(bctx,mkG,riidret(&mc)));
	RUN(AtlAdvise(mc,GetUnknown(),IID_IMontageContainerEvents,&m_hermit));
	RUN(CComQIPtr<IGroupItem>(m_aggrSet)->Realize(bctx));
	return S_OK;
}
STDMETHODIMP SiteSet::Remove() {
    METHTRACE(SiteSet,Remove);
	if(!m_group)
		ERR(E_PENDING);
	RUN(m_group->ReleaseStgRef(m_mkMontage));
	RUN(CComQIPtr<IGroupItem>(m_aggrSet)->Remove());
	return S_OK;
}
// IPersistStream
STDMETHODIMP SiteSet::GetClassID(CLSID *pClassID) {
    METHTRACE(SiteSet,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP SiteSet::IsDirty(void) {
    METHTRACE(SiteSet,IsDirty);
	return S_OK;
}
STDMETHODIMP SiteSet::Load(IStream *pStm) {
    METHTRACE(SiteSet,Load);
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkMontage)));
	RUN(CComQIPtr<IPersistStream>(m_aggrSet)->Load(pStm));
	return S_OK;
}
STDMETHODIMP SiteSet::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(SiteSet,Save);
	RUN(FixedOleSaveToStream(m_mkMontage,pStm));
	RUN(CComQIPtr<IPersistStream>(m_aggrSet)->Save(pStm,fClearDirty));
	return S_OK;
}
STDMETHODIMP SiteSet::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(SiteSet,GetSizeMax);
	ERR(E_NOTIMPL);
}
