/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "SizeToFitTransform.h"
#include "enums.h" // just for layerenum


SizeToFitTransform::SizeToFitTransform() : m_viewing(false) {
	CONSTRUCTORTRACE(SizeToFitTransform);
	memset(&m_wrect,0,sizeof(RECT)*3+sizeof(SIZE)+1); // hack
}
SizeToFitTransform::~SizeToFitTransform() {
	DESTRUCTORTRACE(SizeToFitTransform);
}
// HACK: the window rect is clipped so that grapples can stick out.
RECT clipRect(SIZE wsize) {
	RECT wrect;
	wrect.left = wrect.top = 0;//5;
	wrect.right = wsize.cx;// - 5;
	wrect.bottom = wsize.cy;// - 5;
	return wrect;
}
// IObjectWithSite
STDMETHODIMP SizeToFitTransform::SetSite(IUnknown * pUnkSite) {
	METHTRACE(SizeToFitTransform,SetSite);
	m_mcRef.Release();
	if(!pUnkSite)
		return S_OK;
	RUN(CComQIPtr<IMontageContainer>(pUnkSite)->get_RefObject(&m_mcRef));
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::GetSite(REFIID riid,void ** ppvSite) {
	METHTRACE(SizeToFitTransform,GetSite);
	*ppvSite = 0;
	if(!m_mcRef)
		ERR(E_FAIL);
	CComPtr<IUnknown> umc;
	m_mcRef->get_Montage(&umc);
	if(!umc)
		ERR(E_FAIL);
	return umc->QueryInterface(riid,ppvSite);
}
// IMCCTransform
STDMETHODIMP SizeToFitTransform::put_TransformedArea(RECT *val) {
    METHTRACE(SizeToFitTransform,put_TransformedArea);
	m_wrect = *val;
	RUN(mc()->TransformChanged(VARIANT_TRUE,VARIANT_TRUE));
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::get_TransformedArea(RECT *ret) {
    METHTRACE(SizeToFitTransform,get_TransformedArea);
	*ret = m_wrect;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::put_Extent(SIZEL *val) {
    METHTRACE(SizeToFitTransform,put_Extent);
	m_extent = *val;
	// upperleft point is reference
	m_crect.right = m_crect.left + m_extent.cx;
	m_crect.bottom = m_crect.top - m_extent.cy;
	RUN(recalc(true));
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::get_Extent(SIZEL *val) {
    METHTRACE(SizeToFitTransform,get_Extent);
	*val = m_extent;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::Viewing(LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
	METHTRACE(SizeToFitTransform,Viewing);
	if(lpcrOut) {
		m_vrect = *(RECT *)lpcrOut;
		m_viewing = true;
		if(lpcrCanv) {
			m_canving = true;
			m_cvrect = *lpcrCanv;
		}
	}
	else m_viewing = false;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::get_IsViewing(VARIANT_BOOL *ret) {
    METHTRACE(SizeToFitTransform,get_IsViewing);
	*ret = m_viewing?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2WS(SIZEL siz,PSIZE psiz) {
	METHTRACE(SizeToFitTransform,C2WS);
	int w = wid(m_crect), h = hei(m_crect);
	(siz.cx *= wid(m_wrect)) /= (w?w:1);
	(siz.cy *= -hei(m_wrect)) /= (h?h:1);
	*psiz = siz;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2VS(SIZEL siz,PSIZE psiz) {
	METHTRACE(SizeToFitTransform,C2VS);
	if(!m_viewing)
		return C2WS(siz,psiz);
	int w = m_canving?wid(m_cvrect):wid(m_crect),
		h = m_canving?hei(m_cvrect):hei(m_crect);
	(siz.cx *= wid(m_vrect)) /= (w?w:1);
	if(m_vrect.top>m_vrect.bottom) // cartesian
		(siz.cy *= hei(m_vrect)) /= (h?h:1);
	else (siz.cy *= -hei(m_vrect)) /= (h?h:1);
	*psiz = siz;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::W2CS(SIZE siz,PSIZEL psiz) {
	METHTRACE(SizeToFitTransform,W2CS);
	int w = wid(m_wrect), h = hei(m_wrect);
	psiz->cx = siz.cx * wid(m_crect)/ (w?w:1);
	psiz->cy = siz.cy *-hei(m_crect)/ (h?h:1);
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2WP(POINTL pt,PPOINT ppt) {
	METHTRACE(SizeToFitTransform,C2WP);
	int w = wid(m_crect), h = hei(m_crect);
	ppt->x = m_wrect.left + (pt.x - m_crect.left)*wid(m_wrect)/ (w?w:1);
	ppt->y = m_wrect.top + (m_crect.top - pt.y)*hei(m_wrect)/ (h?h:1);
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2VP(POINTL pt,PPOINT ppt) {
	METHTRACE(SizeToFitTransform,C2VP);
	if(!m_viewing)
		return C2WP(pt,ppt);
	RECTL &c = m_canving?m_cvrect:m_crect;
	pt.x -= c.left;
	pt.y = c.top - pt.y;
	int w = wid(c), h = hei(c);
	(pt.x *= wid(m_vrect)) /= (w?w:1);
	(pt.y *= hei(m_vrect)) /= (h?h:1);
	if(m_vrect.top>m_vrect.bottom) // cartesian
		*ppt = point(m_vrect.left + pt.x,m_vrect.top - pt.y);
	else *ppt = point(m_vrect.left + pt.x,m_vrect.top + pt.y); // non- (window)
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::W2CP(POINT pt,PPOINTL ppt) {
	METHTRACE(SizeToFitTransform,W2CP);
	int w = wid(m_wrect), h = hei(m_wrect);
	ppt->x = m_crect.left + (pt.x-m_wrect.left)*wid(m_crect)/ (w?w:1);
	ppt->y = m_crect.top - (pt.y-m_wrect.top)*hei(m_crect)/ (h?h:1);
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2WR(LPCRECTL pcr,PRECT prc) {
	METHTRACE(SizeToFitTransform,C2WR);
	*prc = rect(pcr->left - m_crect.left,m_crect.top - pcr->top,
		pcr->right - m_crect.left,m_crect.top - pcr->bottom);
	ScaleRect(*prc,wid(m_wrect),wid(m_crect),hei(m_wrect),hei(m_crect));
	*prc += size(m_wrect.left,m_wrect.top);
	prc->right++; // Windows rectangles are not inclusive
	prc->bottom++;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::C2VR(LPCRECTL pcr,PRECT prc) {
	METHTRACE(SizeToFitTransform,C2VR);
	if(!m_viewing)
		return C2WR(pcr,prc);
	RECTL &c = m_canving?m_cvrect:m_crect;
	RECT r = {pcr->left - c.left,c.top - pcr->top,
		pcr->right - c.left,c.top - pcr->bottom};
	ScaleRect(r,wid(m_vrect),wid(c),hei(m_vrect),hei(c));
	if(m_vrect.top>m_vrect.bottom) { // cartesian
		prc->left = long(r.left) + m_vrect.left;
		prc->top = m_vrect.top - long(r.top);
		prc->right = long(r.right) + m_vrect.left;
		prc->bottom = m_vrect.top - long(r.bottom);
	}
	else *prc = r + size(m_vrect.left,m_vrect.top);
	prc->right++;
	prc->bottom++;
	return S_OK;
}
STDMETHODIMP SizeToFitTransform::W2CR(LPCRECT pcr,PRECTL ret) {
	METHTRACE(SizeToFitTransform,W2CR);
	*ret = *(RECTL*)pcr - size(m_wrect.left,m_wrect.top);
	ScaleRect(*ret,wid(m_crect),wid(m_wrect),hei(m_crect),hei(m_wrect));
	ret->left += m_crect.left;
	ret->right += m_crect.left;
	ret->top = m_crect.top - ret->top; // flip vertically
	ret->bottom = m_crect.top - ret->bottom;
	return S_OK;
}
HRESULT SizeToFitTransform::recalc(bool forcePlace) {
	METHTRACE(SizeToFitTransform,recalc);
	if(!m_bounds.valid()) // no bounds is good bounds
		return S_OK;
	RECTL rc = m_bounds.get();
	bool change = false;
	// if we're crunched and we don't need to be, correct.
	int t;
	if((t=max(m_extent.cx,wid(rc)))<wid(m_crect)) {
		m_crect.right = m_crect.left + t;
		change = true;
	}
	if((t = max(m_extent.cy,hei(rc)))<hei(m_crect)) {
		m_crect.bottom = m_crect.top - t;
		change = true;
	}
	// ... and checks below will "jostle" into place

	// new bounds are pushing out on left
	if(rc.left<m_crect.left) {
		// Is there empty space on right? can we just shift the viewport left?
		if((m_crect.right-rc.right)>=(m_crect.left-rc.left))
			m_crect.right -= (m_crect.left-rc.left);
		else
			m_crect.right = rc.right; // take what we can get.
		// either way, we have to stretch the left side:
		m_crect.left = rc.left;
		change = true;
	}
	// new bounds are pushing out on right, et cetera:
	if(rc.right>m_crect.right) {
		if((rc.left-m_crect.left)>=(rc.right-m_crect.right))
			m_crect.left += (rc.right-m_crect.right);
		else
			m_crect.left = rc.left;
		m_crect.right = rc.right;
		change = true;
	}
	if(rc.top>m_crect.top) {
		if((rc.bottom-m_crect.bottom)>=(rc.top-m_crect.top))
			m_crect.bottom += (rc.top-m_crect.top);
		else
			m_crect.bottom = rc.bottom;
		m_crect.top = rc.top;
		change = true;
	}
	if(rc.bottom<m_crect.bottom) {
		if((m_crect.top-rc.top)>=(m_crect.bottom-rc.bottom))
			m_crect.top -= (m_crect.bottom-rc.bottom);
		else
			m_crect.top = rc.top;
		m_crect.bottom = rc.bottom;
		change = true;
	}
    if(change||forcePlace)
	    return mc()->TransformChanged(VARIANT_TRUE,VARIANT_TRUE);
	else
		return S_FALSE;
}

STDMETHODIMP SizeToFitTransform::AddRect(LPCRECTL r) {
	METHTRACE(SizeToFitTransform,AddRect);
	if(m_bounds.add(*r))
		return recalc();
	else
		return S_OK;
}
STDMETHODIMP SizeToFitTransform::RemoveRect(LPCRECTL r) {
	METHTRACE(SizeToFitTransform,RemoveRect);
	if(m_bounds.remove(*r))
		return recalc();
	else
		return S_OK;
}
// IPersist
STDMETHODIMP SizeToFitTransform::GetClassID(CLSID *pcls) {
	METHTRACE(SizeToFitTransform,GetClassID);
	*pcls = GetObjectCLSID();
	return S_OK;
}
