/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// StickyButton.cpp : Implementation of StickyButton

#include "stdafx.h"
#include "StickyButton.h"
//#include <math.h>
#define DIFF 6

StickyButton::StickyButton() :
	m_button(_T("BUTTON"),this,1),
    m_stuck(false),
    m_sticky(true),
    m_subdown(false)
{
	CONSTRUCTORTRACE(StickyButton);
	m_bWindowOnly = TRUE;

    int bitData = 0x5555aaaa;
    m_fineChecker = CreateBitmap(2,2,1,1,&bitData);
}
StickyButton::~StickyButton() {
	DESTRUCTORTRACE(StickyButton);
    DeleteObject(m_fineChecker);
}
const int BORDER = 2;
RECT StickyButton::childRect() {
    if(m_sticky)
        return rect(BORDER+m_stuck*DIFF,BORDER,m_stuck*DIFF+wid(m_rcPos)-DIFF-BORDER,hei(m_rcPos)-BORDER);
    else
        return rect(0,0,wid(m_rcPos),hei(m_rcPos));
}
void StickyButton::posButton() {
    RECT r = childRect();
	if(m_button)
		m_button.MoveWindow(r.left,r.top,wid(r),hei(r));
}
void StickyButton::setState() {
    m_button.SendMessage(BM_SETSTATE,m_stuck);
}

HWND StickyButton::CreateControlWindow(HWND parent, RECT &pos) {
	USES_CONVERSION;
    HWND res;
	if(!(res=CComControl<StickyButton>::CreateControlWindow(parent,pos)))
        return 0;
    RECT r = childRect();
	HWND hwndButton = CreateWindow("BUTTON",W2T(m_text),WS_CHILD | WS_VISIBLE,
        r.left,r.top,wid(r),hei(r),
		m_hWnd,0,_Module.GetModuleInstance(),0);
    m_button.SubclassWindow(hwndButton);
	return res;
}
STDMETHODIMP StickyButton::InPlaceDeactivate(void) {
	if(m_button) {
		TCHAR buf[200];
		::GetWindowText(m_button,buf,200);
		m_text = buf;
	}
	HRESULT res = CComControl<StickyButton>::IOleInPlaceObject_InPlaceDeactivate();
	m_button = 0; // already destroyed because it was a child
	return res;
}
STDMETHODIMP StickyButton::SetObjectRects(LPCRECT prcPos,LPCRECT prcClip) {
    METHTRACE(StickyButton,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<StickyButton>::SetObjectRects(prcPos,prcClip));
    posButton();
	return S_OK;
}
HRESULT StickyButton::OnDrawAdvanced(ATL_DRAWINFO& di) {
    METHTRACE(StickyButton,OnDrawAdvanced);
    HDC hdc = di.hdcDraw;
    COLORREF clrBack,clrFore;
    LONG_PTR style = GetWindowLongPtr(GWL_STYLE);
    RUN(OleTranslateColor(0x80000000|COLOR_3DFACE,0,&clrBack));
    RUN(OleTranslateColor(0x80000000|((style&WS_DISABLED)?COLOR_GRAYTEXT:COLOR_BTNTEXT),0,&clrFore));
    HBRUSH back = CreateSolidBrush(clrBack);
    FillRect(hdc,(RECT*)di.prcBounds,back);
    DeleteObject(back);
    SelectObject(hdc,GetStockObject(NULL_BRUSH));
    DCObject pen(hdc,CreatePen(PS_INSIDEFRAME,0,clrFore));
    RoundRect(hdc,di.prcBounds->left,di.prcBounds->top,di.prcBounds->right,di.prcBounds->bottom,BORDER+1,BORDER+1);
    HBRUSH checker = CreatePatternBrush(m_fineChecker);
    SetBkColor(hdc,clrBack);
    RECT rc = {di.prcBounds->left+BORDER,di.prcBounds->top+BORDER,di.prcBounds->right-BORDER,di.prcBounds->bottom-BORDER};
    if(!m_stuck) {
        rc.left = di.prcBounds->right-DIFF-BORDER;
        SetTextColor(hdc,clrFore);
    }
    else {
        rc.right = DIFF+BORDER;
        COLORREF clrGrey;
        RUN(OleTranslateColor(0x80000000|COLOR_3DSHADOW,0,&clrGrey));
        SetTextColor(hdc,clrGrey);
    }
    FillRect(hdc,&rc,checker);
    DeleteObject(checker);
    return S_OK;
}

LRESULT StickyButton::ButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(StickyButton,ButtonClick);
    if(m_stuck && !m_heardIt) { // e.g. user pressed a key
		m_stuck = false;
		posButton();
 		fireEvent(GetUnknown(),fireOneParam<IMCCStickyButtonEvents,VARIANT_BOOL>
			(&IMCCStickyButtonEvents::Stick,VARIANT_FALSE));
    }
	else
		fireEvent(GetUnknown(),fireVoid<IMCCStickyButtonEvents>(&IMCCStickyButtonEvents::Click));
	return 0;
}
LRESULT StickyButton::LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(StickyButton,LButtonUp);
    // click on background toggles stuckness
    if(!m_stuck)
		fireEvent(GetUnknown(),fireVoid<IMCCStickyButtonEvents>(&IMCCStickyButtonEvents::Click));
    RUN(put_Stuck(!m_stuck));
    return S_OK;
}
LRESULT StickyButton::SubLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(StickyButton,SubLButtonDown);
    if(m_sticky) {
        // normalize coordinate
        RECT r;
        m_button.GetWindowRect(&r);
        m_xdown = r.left + MAKEPOINT(lParam).x - m_stuck*DIFF;
        m_subdown = true;
        m_wasStuck = m_stuck;
        m_hasChanged = false;
    }
    return m_button.DefWindowProc(uMsg,wParam,lParam);
}
LRESULT StickyButton::SubMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(StickyButton,SubMouseMove);
    if(m_sticky) {
        POINT pt = MAKEPOINT(lParam);
        if(m_subdown) {
            RECT r;
            m_button.GetWindowRect(&r);
            int x = r.left + MAKEPOINT(lParam).x - m_xdown;
            m_stuck = x>DIFF/2;
            if(m_stuck!=m_wasStuck)
                m_hasChanged = true;
            posButton();
            // don't let it get message if it's stuck and mouse moves out!
            m_button.GetClientRect(&r);
            if(m_stuck && !PtInRect(&r,pt))
                return 0;
        }
    }
    return m_button.DefWindowProc(uMsg,wParam,lParam);
}
LRESULT StickyButton::SubLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(StickyButton,SubLButtonUp);
    if(!m_sticky)
        return m_button.DefWindowProc(uMsg,wParam,lParam);
    posButton();
	if(m_hasChanged) {// this is a drag not a click:
        m_heardIt = true; // clicks that aren't from these mouse events cause the button to pop up.
        // send a click in every case except the unstick case
        if(!(m_wasStuck&&!m_stuck))
    		fireEvent(GetUnknown(),fireVoid<IMCCStickyButtonEvents>(&IMCCStickyButtonEvents::Click));
        if(m_stuck!=m_wasStuck)
 			fireEvent(GetUnknown(),fireOneParam<IMCCStickyButtonEvents,VARIANT_BOOL>
				(&IMCCStickyButtonEvents::Stick,m_stuck*VARIANT_TRUE));
	}
    LRESULT lr = m_button.DefWindowProc(uMsg,wParam,lParam);
    m_heardIt = false;
	setState();
    m_subdown = false;
    return lr;
}
LRESULT StickyButton::SubDoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    RUN(put_Stuck(VARIANT_TRUE));
    return 0;
}
LRESULT StickyButton::KeepDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(StickyButton,KeepDown);
    m_heardIt = true; // clicks that aren't from these mouse events cause the button to pop up.
    LRESULT lr = m_button.DefWindowProc(uMsg,wParam,lParam);
    m_heardIt = false;
    if(m_sticky)
	    setState();
    return 0;
}
STDMETHODIMP StickyButton::put_WStyle(DWORD val) {
    METHTRACE(StickyButton,put_WStyle);
    m_wstyle = val | WS_CHILD | WS_VISIBLE;
    if(val&WS_DISABLED) {
        EnableWindow(false);
        m_button.EnableWindow(false);
    }
    else {
        EnableWindow(true);
        m_button.EnableWindow(true);
    }
    InvalidateRect(0,false);
    return S_OK;
}
STDMETHODIMP StickyButton::get_Text(BSTR *ret) {
    METHTRACE(StickyButton,get_Text);
	if(m_button) {
		TCHAR buf[200];
		::GetWindowText(m_button,buf,200);
		*ret = CComBSTR(buf).Detach();
	}
	else
		*ret = m_text.Copy();
	return S_OK;
}
STDMETHODIMP StickyButton::put_Text(BSTR val) {
    METHTRACE(StickyButton,put_Text);
	if(m_button) {
		USES_CONVERSION;
		::SetWindowText(m_button,W2T(val));
	}
	else m_text = val;
	return S_OK;
}
STDMETHODIMP StickyButton::get_Sticky(VARIANT_BOOL *ret) {
    METHTRACE(StickyButton,get_Sticky);
    *ret = m_sticky*VARIANT_TRUE;
    return S_OK;
}
STDMETHODIMP StickyButton::put_Sticky(VARIANT_BOOL val) {
    METHTRACE(StickyButton,put_Sticky);
    if(!(m_sticky = val!=0))
        RUN(put_Stuck(false));
    posButton();
    return S_OK;
}
STDMETHODIMP StickyButton::get_Stuck(VARIANT_BOOL *ret) {
    METHTRACE(StickyButton,get_Stuck);
    *ret = m_stuck*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP StickyButton::put_Stuck(VARIANT_BOOL val) {
    METHTRACE(StickyButton,put_Stuck);
    if(m_stuck != bool(val)) {
        m_stuck = bool(val);
	    setState();
        posButton();
 	    fireEvent(GetUnknown(),fireOneParam<IMCCStickyButtonEvents,VARIANT_BOOL>
		    (&IMCCStickyButtonEvents::Stick,m_stuck*VARIANT_TRUE));
    }
	return S_OK;
}
