/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// StickyButton.h : Declaration of the StickyButton

#ifndef __STICKYBUTTON_H_
#define __STICKYBUTTON_H_


/////////////////////////////////////////////////////////////////////////////
// StickyButton
struct ATL_NO_VTABLE StickyButton :
	CComObjectRoot,
	IMCCStickyButton,
	CComControl<StickyButton>,
	IPersistStreamInitImpl<StickyButton>,
	IOleControlImpl<StickyButton>,
	IOleObjectImpl<StickyButton>,
	IOleInPlaceActiveObjectImpl<StickyButton>,
	IViewObjectExImpl<StickyButton>,
	IOleInPlaceObjectWindowlessImpl<StickyButton>,
	CComCoClass<StickyButton, &CLSID_MCCStickyButton>,
	IConnectionPointContainerImpl<StickyButton>,
	IConnectionPointImpl<StickyButton,&IID_IMCCStickyButtonEvents>
{
	CContainedWindow m_button;
	CComBSTR m_text; // only when inactive
    bool m_sticky,
        m_stuck, //
        m_wasStuck, // whether was stuck at lbuttondown
        m_hasChanged, // whether m_stuck has changed in this lbutton sequence
        m_subdown, // in the middle of subclassed left-button-down operation
        m_heardIt; // sending the WM_LBUTTONUP that causes WM_CLICKED
    int m_xdown;
    HBITMAP m_fineChecker;
    DWORD m_wstyle;

	StickyButton();
	~StickyButton();

    RECT childRect();
    void posButton();
	void setState();

    REGISTER_GENERIC(L"StickyButton")

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(StickyButton)
	COM_INTERFACE_ENTRY(IMCCStickyButton)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(StickyButton)
	CONNECTION_POINT_ENTRY(IID_IMCCStickyButtonEvents)
END_CONNECTION_POINT_MAP()

BEGIN_PROP_MAP(StickyButton)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(StickyButton)
	CHAIN_MSG_MAP(CComControl<StickyButton>)
	COMMAND_CODE_HANDLER(BN_CLICKED,ButtonClick);
    MESSAGE_HANDLER(WM_LBUTTONUP,LButtonUp);
	DEFAULT_REFLECTION_HANDLER()
ALT_MSG_MAP(1)
    MESSAGE_HANDLER(WM_LBUTTONDOWN,SubLButtonDown)
    MESSAGE_HANDLER(WM_MOUSEMOVE,SubMouseMove)
    MESSAGE_HANDLER(WM_LBUTTONUP,SubLButtonUp)
    MESSAGE_HANDLER(WM_LBUTTONDBLCLK,SubDoubleClick)
    MESSAGE_HANDLER(WM_KILLFOCUS,KeepDown)
END_MSG_MAP()


// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	LRESULT ButtonClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT LButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

    LRESULT SubLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT SubMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT SubLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT SubDoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT KeepDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

    HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);

	// disable UI activity that comes with focus
	HRESULT DoVerbUIActivate(LPCRECT lprcPosRect, HWND hwndParent) {
		return DoVerbInPlaceActivate(lprcPosRect,hwndParent);
	}
	// create combo
	HWND CreateControlWindow(HWND parent, RECT &pos);
	// here's where it gets deleted
	STDMETHODIMP InPlaceDeactivate();
	// move combo too
	STDMETHOD(SetObjectRects)(LPCRECT prcPos,LPCRECT prcClip);

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IMCCStickyButton
    STDMETHODIMP put_WStyle(DWORD val);
	STDMETHODIMP get_Text(BSTR *ret);
	STDMETHODIMP put_Text(BSTR val);
    STDMETHODIMP get_State(int *ret) { ERR(E_FAIL); }
    STDMETHODIMP get_Sticky(VARIANT_BOOL *ret);
    STDMETHODIMP put_Sticky(VARIANT_BOOL val);
    STDMETHODIMP get_Stuck(VARIANT_BOOL *ret);
    STDMETHODIMP put_Stuck(VARIANT_BOOL val);
};

#endif //__STICKYBUTTON_H_
