/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Style.h"

Style::Style() : m_newID(1) {
	CONSTRUCTORTRACE(Style);
}
Style::~Style() {
	DESTRUCTORTRACE(Style);
}

// INeedOnePointer
STDMETHODIMP Style::TakePointer(IUnknown *o,IMoniker *m) {
	METHTRACE(Style,TakePointer);
	return put_Defer(CComQIPtr<IDispatchEx>(o));
}

// IMCCStyle
STDMETHODIMP Style::get_Defer(IDispatchEx **ret) {
	METHTRACE(Style,get_Defer);
	m_defer.CopyTo(ret);
	return S_OK;
}
STDMETHODIMP Style::put_Defer(IDispatchEx *val) {
	METHTRACE(Style,put_Defer);
	m_defer = val;
	return S_OK;
}
// DIMCCStyle
STDMETHODIMP Style::get_Defer(IDispatch **ret) {
    METHTRACE(Style,get_Defer);
	return get_Defer((IDispatchEx**)ret);
}
STDMETHODIMP Style::put_Defer(IDispatch *val) {
    METHTRACE(Style,put_Defer);
	CComPtr<IDispatchEx> dex;
	RUN(val->QueryInterface(riidret(&dex)));
	return put_Defer(dex);
}
// IDispatch
STDMETHODIMP Style::GetTypeInfoCount(UINT * pctinfo) {
	METHTRACE(Style,GetTypeInfoCount);
	*pctinfo = 0;
	return S_OK;
}
STDMETHODIMP Style::GetTypeInfo(UINT iTInfo,LCID lcid,ITypeInfo ** ppTInfo) {
	METHTRACE(Style,GetTypeInfo);
	ERR(E_FAIL);
}
STDMETHODIMP Style::GetIDsOfNames(REFIID riid,LPOLESTR * rgszNames,UINT cNames,
	LCID lcid,DISPID * rgDispId) {
	METHTRACE(Style,GetIDsOfNames);
	if(cNames!=1)
		ERR(DISP_E_NONAMEDARGS);
	return GetDispID(*rgszNames,fdexNameCaseInsensitive,rgDispId);
}
STDMETHODIMP Style::Invoke(DISPID dispIdMember,REFIID riid,LCID lcid,WORD wFlags,
	DISPPARAMS * pDispParams,VARIANT * pVarResult,EXCEPINFO * pExcepInfo,UINT * puArgErr) {
	METHTRACE(Style,Invoke);
	return InvokeEx(dispIdMember,lcid,wFlags,pDispParams,pVarResult,pExcepInfo,0);
}
// -Ex
STDMETHODIMP Style::GetDispID(BSTR bstrName,DWORD grfdex,DISPID *pid) {
	METHTRACE(Style,GetDispID);
	HRESULT hr;
	dispmap::iterator i = m_dispmap.find(bstrName);
	if(i==m_dispmap.end()) {
		*pid = DISPID_UNKNOWN;
		// try to fetch the name from the parent.  We have to create an entry no matter
		// what, in case the parent gets disconnected.
		if(m_defer)
			hr = m_defer->GetDispID(bstrName,grfdex&(~fdexNameEnsure),pid); // don't add to parent!
		if(*pid == DISPID_UNKNOWN) {
			if(grfdex&fdexNameEnsure)
				*pid = m_newID++;
			else
				ERR(DISP_E_UNKNOWNNAME);
		}
		value &v = m_data[*pid];
		v.name = bstrName; // (copy)
		v.deleted = false;
		m_dispmap[v.name] = *pid;
		return S_OK;
	}
	value &v = m_data[i->second];
	if(v.deleted) {
		if(grfdex&fdexNameEnsure)
			v.deleted = false;
		else
			ERR(DISP_E_UNKNOWNNAME); //?
	}
	*pid = i->second;
	return S_OK;
}
struct PropSink : CComObjectRoot,
	IPropertyNotifySink
{
	Style *m_style;
	DISPID m_id;
BEGIN_COM_MAP(PropSink)
	COM_INTERFACE_ENTRY(IPropertyNotifySink)
END_COM_MAP()
	STDMETHODIMP OnChanged(DISPID id) {
        METHTRACE(PropSink,OnChanged);
		RUN(fireEvent(m_style->GetUnknown(),fireOneParam<IPropertyNotifySink,DISPID>(&IPropertyNotifySink::OnChanged,m_id)));
		return S_OK;
	}
	STDMETHODIMP OnRequestEdit(DISPID id) {
        METHTRACE(PropSink,OnRequestEdit);
		return S_OK;
	}
};
STDMETHODIMP Style::InvokeEx(DISPID id,LCID lcid,WORD wFlags,DISPPARAMS *pdp,VARIANT *pvarRes,
	EXCEPINFO *pei,IServiceProvider *pspCaller) {
	METHTRACE(Style,InvokeEx);
	HRESULT hr;
	valmap::iterator i = m_data.find(id);
	if(i==m_data.end()) {
		if(wFlags&(DISPATCH_PROPERTYPUT| DISPATCH_PROPERTYPUTREF)) {
			// lots of sanity checks, mostly to make sure I understand how dispatch works.
			if(pdp->cNamedArgs) {
				if(pdp->cNamedArgs!=1)
					ERR(DISP_E_BADPARAMCOUNT);
				if(*pdp->rgdispidNamedArgs!=DISPID_PROPERTYPUT)
					ERR(DISP_E_UNKNOWNNAME);
			}
			if(pdp->cArgs!=1)
				ERR(DISP_E_BADPARAMCOUNT);
			// ID must have come from parent, but put is always local
			i = m_data.insert(make_pair_first_const(id,value())).first;
			value &v = i->second;
			if(!v.name) {// probably already did this on GetDispID
				if(!m_defer)
					ERR(DISP_E_MEMBERNOTFOUND); // because we can't get the name
				RUN(m_defer->GetMemberName(id,&v.name));
			}
			v.deleted = false;
			m_dispmap[v.name] = id;
		}
		else {
			if(m_defer)
				if(SUCCEEDED(hr = m_defer->InvokeEx(id,lcid,wFlags,pdp,pvarRes,pei,pspCaller)))
					return hr;
			ERR(DISP_E_MEMBERNOTFOUND);
		}
	}
	// have to add it again
	if(i->second.deleted)
		ERR(DISP_E_MEMBERNOTFOUND);
	// now i is initialized.
	if(wFlags&(DISPATCH_PROPERTYPUT| DISPATCH_PROPERTYPUTREF)) {
		if(pdp->cNamedArgs) {
			if(pdp->cNamedArgs!=1)
				ERR(DISP_E_BADPARAMCOUNT);
			if(*pdp->rgdispidNamedArgs!=DISPID_PROPERTYPUT)
				ERR(DISP_E_UNKNOWNNAME);
		}
		if(pdp->cArgs!=1)
			ERR(DISP_E_BADPARAMCOUNT);
		for(IUnknown **is = m_vec.begin();is!=m_vec.end();++is)
			if(IPropertyNotifySink *ns = (IPropertyNotifySink*)*is)
				if(ns->OnRequestEdit(id)!=S_OK)
					return S_FALSE;
		i->second.val = *pdp->rgvarg;
		for(is = m_vec.begin();is!=m_vec.end();++is)
			if(IPropertyNotifySink *ns = (IPropertyNotifySink*)*is)
				RUN(ns->OnChanged(id));
		if(i->second.val.vt == VT_DISPATCH || i->second.val.vt == VT_UNKNOWN) {
			CComPtr<PropSink> ps = new CComObject<PropSink>();
			ps->m_id = id;
			ps->m_style = this;
			DWORD ugh;
			RUN(AtlAdvise(i->second.val.punkVal,ps,IID_IPropertyNotifySink,&ugh));
		}
		return S_OK;
	}
	else if(wFlags&DISPATCH_PROPERTYGET) {
		if(pdp) {
			if(pdp->cNamedArgs)
				ERR(DISP_E_NONAMEDARGS);
			if(pdp->cArgs)
				ERR(DISP_E_BADPARAMCOUNT);
		}
		if(!pvarRes)
			ERR(E_INVALIDARG);
		VariantInit(pvarRes); // clients may pass in garbage
		return VariantCopy(pvarRes,&i->second.val);
	}
	else ERR(E_INVALIDARG);
}
STDMETHODIMP Style::DeleteMemberByName(BSTR bstr, DWORD grfdex) {
	METHTRACE(Style,DeleteMemberByName);
	dispmap::iterator i = m_dispmap.find(bstr);
	if(i==m_dispmap.end()) // probably member of parent
		return S_FALSE;
	return DeleteMemberByDispID(i->second);
}
STDMETHODIMP Style::DeleteMemberByDispID(DISPID id) {
	METHTRACE(Style,DeleteMemberByDispID);
	valmap::iterator j = m_data.find(id);
	if(j==m_data.end())
		ERR(E_FAIL);
	j->second.deleted = true;
	return j->second.val.Clear();
}
STDMETHODIMP Style::GetMemberProperties(DISPID id,DWORD grfdexFetch,DWORD *pgrfdex) {
	METHTRACE(Style,GetMemberProperties);
	// first fill in with props of all members.
	*pgrfdex = (fdexPropCanPut|fdexPropCanPutRef|fdexPropNoSideEffects|fdexPropDynamicType|fdexPropCannotConstruct|fdexPropCannotSourceEvents)&grfdexFetch;
	valmap::iterator i = m_data.find(id);
	if(i==m_data.end()) {
		if(m_defer)
			if(SUCCEEDED(m_defer->GetMemberProperties(id,grfdexFetch,pgrfdex))) {
				*pgrfdex |= (fdexPropCanPut|fdexPropCanPutRef)&grfdexFetch;
				return S_OK;
			}
	}
	else *pgrfdex |= fdexPropCanGet&grfdexFetch; // if we have it, you can get it.
	return S_OK;
}
STDMETHODIMP Style::GetMemberName(DISPID id,BSTR *pbstrName) {
	METHTRACE(Style,GetMemberName);
	valmap::iterator i = m_data.find(id);
	if(i==m_data.end()) {
		if(m_defer)
			return m_defer->GetMemberName(id,pbstrName);
		*pbstrName = 0;
		ERR(DISP_E_MEMBERNOTFOUND);
	}
	*pbstrName = SysAllocStringLen(i->second.name,SysStringLen(i->second.name));
	return S_OK;
}
STDMETHODIMP Style::GetNextDispID(DWORD grfdex,DISPID id,DISPID *pid) {
	METHTRACE(Style,GetNextDispID);
	valmap::iterator i;
	if(id==DISPID_STARTENUM)
		i = m_data.begin();
	else {
		i = m_data.find(id);
		if(i!=m_data.end())
			++i;
	}
	if(i==m_data.end()) {
		if(m_defer)
			return m_defer->GetNextDispID(grfdex,id,pid);
		*pid = DISPID_UNKNOWN; //?
		return S_FALSE;
	}
	*pid = i->first;
	return S_OK;
}
STDMETHODIMP Style::GetNameSpaceParent(IUnknown **ppunk) {
	METHTRACE(Style,GetNameSpaceParent);
	// I think this is what this method's for.
	m_defer.CopyTo((IDispatchEx**)ppunk);
	return S_OK;
}
// IPersistStream
STDMETHODIMP Style::GetClassID(CLSID *pClassID) {
	METHTRACE(Style,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP Style::IsDirty(void) {
	METHTRACE(Style,IsDirty);
	return S_FALSE;
}
STDMETHODIMP Style::Load(IStream *pStm) {
	METHTRACE(Style,Load);
	int n;
	RUN(pStm->Read(&n,sizeof(int),0));
	while(n--) {
		DISPID id;
		RUN(pStm->Read(&id,sizeof(DISPID),0));
		CComBSTR readName;
		RUN(readName.ReadFromStream(pStm));
		value &v = m_data[id];
		if(v.name.Length()) { // already exists
			// double-check name is same
			if(wcscmp(v.name,readName))
				ERR(E_FAIL);
		}
		else { // new
			v.name.Empty();
			v.name.Attach(readName.Detach());
			m_dispmap[v.name] = id;
		}
		v.deleted = false;
		v.val.Clear();
		RUN(v.val.ReadFromStream(pStm));
		// random: all fonts should be initialized for lometric
		if(v.val.vt==VT_DISPATCH||v.val.vt==VT_UNKNOWN)
			if(CComQIPtr<IFont> f = v.val.punkVal)
				RUN(f->SetRatio(100,1000));

	}
	return S_OK;
}
STDMETHODIMP Style::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(Style,Save);
	// hold curr pos, save space for n
	ULARGE_INTEGER begpos;
	LARGE_INTEGER zero;
	zero.QuadPart = 0;
	RUN(pStm->Seek(zero,STREAM_SEEK_CUR,&begpos));
	int n = 0;
	RUN(pStm->Write(&n,sizeof(int),0));
	// throw out deleted objects as you go because clients will not
	// expect to cache DISPIDs across incarnations (I don't think).
	for(valmap::iterator i = m_data.begin();i!=m_data.end();i++)
		if(!i->second.deleted) {
			RUN(pStm->Write(&i->first,sizeof(DISPID),0));
			RUN(i->second.name.WriteToStream(pStm));
			RUN(i->second.val.WriteToStream(pStm));
			n++;
		}
	ULARGE_INTEGER endpos;
	RUN(pStm->Seek(zero,STREAM_SEEK_CUR,&endpos));
	RUN(pStm->Seek(*(LARGE_INTEGER*)&begpos,STREAM_SEEK_SET,0));
	RUN(pStm->Write(&n,sizeof(int),0));
	RUN(pStm->Seek(*(LARGE_INTEGER*)&endpos,STREAM_SEEK_SET,0));
	return S_OK;
}
STDMETHODIMP Style::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(Style,GetSizeMax);
	CComObjectStack<countStream> fake;
	RUN(Save(&fake,false));
	*pcbSize = fake.m_pos;
	return S_OK;
}
