/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "StdAfx.h"
#include "TextEdit.h"
#include "defaultFont.h"

TextEdit::TextEdit() : m_start(0),m_pos0(0),m_pos1(0),m_margin(0),
	m_focal(false),m_capture(false),m_opaque(false),m_centerX(false),m_isAmbient(false) {
	CONSTRUCTORTRACE(TextEdit);
    m_bResizeNatural = false;
}

TextEdit::~TextEdit() {
	DESTRUCTORTRACE(TextEdit);
}
HRESULT TextEdit::OnPostVerbInPlaceActivate() {
    METHTRACE(TextEdit,OnPostVerbInPlaceActivate);
	computeLens();
	m_editing = m_text;
	return S_OK;
}
HRESULT TextEdit::setupDC(HDC hdc,POINT LL) {
	CComPtr<IMontageContainer> mc;
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
	// use transform rather than s_sizeExtent:rc ratio to be consistent
	SIZEL fivecm = {5000,5000},
		r5cm = RenderPolicy::inSize(fivecm),
		p5cm;
	RUNRET(t->C2VS(fivecm,&p5cm));
	// lometric
	CHECK(SetMapMode(hdc,MM_ANISOTROPIC));
	CHECK(SetViewportOrgEx(hdc,LL.x,LL.y,0));
	CHECK(SetWindowExtEx(hdc,r5cm.cx,r5cm.cy,0));
	CHECK(SetViewportExtEx(hdc,p5cm.cx,p5cm.cy,0));
	return S_OK;
}
void TextEdit::changed() {
    METHTRACE(TextEdit,changed);
	HDC info;
	info = ::CreateCompatibleDC(0);
	SIZE S = {0,0};
	if(FAILED(setupDC(info,point(0,0))))
        return;
    HFONT hf;
    RUN(m_font->get_hFont(&hf));
    HFONT hfOld = (HFONT)SelectObject(info,hf);
	if(m_editing.size())
		GetTextExtentPoint32(info,m_editing.c_str(),m_editing.size(),&S);
	else {
		GetTextExtentPoint32(info,"Aj",2,&S);
		S.cx = 0;
	}
	SelectObject(info,hfOld);
    ::DeleteDC(info);
	S = RenderPolicy::outSize(S); // lometric -> himetric
	S += size(2*m_margin,2*m_margin);
	// zero bad. .001cm okay.
	if(S.cx==0)
		S.cx = 1;
	if(S.cy==0)
		S.cy = 1;
	if(assign(m_sizeNatural,S)) {
		CComQIPtr<IMCCSite> site(m_spClientSite);
		MCC_POS_INFO mpi;
		mpi.postype = MCC_POS_CANVASSIZE;
		mpi.sl = S;
		HRESULT hr = fireSiteEvent(site,fireMove(&mpi)); // default site owner will fail here if there's no pos
		if(hr!=E_PENDING)
			RUN(hr);
	}
}
void TextEdit::computeLens() {
    METHTRACE(TextEdit,computeLens);
	HDC info;
	RUN(m_spInPlaceSite->GetDC(0,OLEDC_NODRAW,&info));
	CHECK(SaveDC(info));
	RUN(setupDC(info,point(0,0)));
	CComPtr<IMontageContainer> mc;
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
    HFONT hf;
    RUN(m_font->get_hFont(&hf));
    m_lens.compute(m_editing,info,hf,t);
	CHECK(RestoreDC(info,-1));
	RUN(m_spInPlaceSite->ReleaseDC(info));
}
SIZEL TextEdit::margin(bool viewing) {
    METHTRACE(TextEdit,margin);
	CComPtr<IMontageContainer> mc;
	RUN(CComQIPtr<IMCCSite>(m_spClientSite)->GetContainer(riidret(&mc)));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
	SIZE s = {m_margin,m_margin},sv;
	if(viewing)
		RUN(t->C2VS(s,&sv));
	else
		RUN(t->C2WS(s,&sv));
	sv.cy = tabs(sv.cy);
	return sv;
}
void TextEdit::createCursor() {
    METHTRACE(TextEdit,createCursor);
	HWND hWnd;
	RUN(m_spInPlaceSite->GetWindow(&hWnd));
	::CreateCaret(hWnd,0,2,m_lens.m_height);
	positionCursor();
	::ShowCaret(hWnd);
}
void TextEdit::deleteCursor() {
    METHTRACE(TextEdit,deleteCursor);
	DestroyCaret();
}
void TextEdit::positionCursor() {
    METHTRACE(TextEdit,positionCursor);
	int x1 = m_lens.p2x(m_pos1);
	SIZEL m = margin(false);
	::SetCaretPos(m_rcPos.left+m.cx+x1-1,m_rcPos.top+m.cy);
}
int TextEdit::getPos(int x) {
    METHTRACE(TextEdit,getPos);
	return m_lens.x2p(x-margin(false).cx-m_rcPos.left);
}
void TextEdit::focus() {
    METHTRACE(TextEdit,focus);
	RUN(m_spInPlaceSite->SetFocus(true));
	m_focal = true;
    createCursor();
}
void TextEdit::unfocus() {
    METHTRACE(TextEdit,unfocus);
	RUN(m_spInPlaceSite->SetFocus(false));
	m_focal = false;
    deleteCursor();
}
#define SCALE_TEXT
#include <sys/timeb.h>
// IViewObject
STDMETHODIMP TextEdit::Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
		DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdc,LPCRECTL bounds,
		LPCRECTL lprcWBounds,BOOL(STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
		DWORD dwContinue) {
    METHTRACE(TextEdit,Draw);
	if(m_opaque)
		FillRect(hdc,(RECT*)bounds,(HBRUSH)GetStockObject(WHITE_BRUSH));

	RECT rc = *(RECT*)bounds;
	if(m_focal) {
		int x0 = m_lens.p2x(m_pos0),
			x1 = m_lens.p2x(m_pos1);
		if(m_pos0!=m_pos1) {
			RECT rcSel = {rc.left+x0,rc.top,rc.left+x1,rc.top+m_lens.m_height};
			HBRUSH brush = CreateSolidBrush(RGB(255,128,0));
			FillRect(hdc,&rcSel,brush);
			DeleteObject(brush);
		}
	}
	if(!m_editing.empty()) {
		DWORD dctype = GetObjectType(hdc);
		bool isMF = dctype==OBJ_ENHMETADC || dctype==OBJ_METADC;
		RECT rc2;
		/*
		i can't get scaling to work in metafiles,

		so the fonts are kept at the scale that is used for drawing metafiles
		(defined in RenderPolicy)
		*/
		SIZEL m;
		if(isMF) {
			rc2 = rc;
			m = margin(true);
		}
		else {
			CHECK(SaveDC(hdc));
			RUN(setupDC(hdc,point(rc.left,rc.bottom)));
			rc2 = rect(0,RenderPolicy::inY(m_sizeExtent.cy),RenderPolicy::inX(m_sizeExtent.cx),0);
			m = RenderPolicy::inSize(size(m_margin,m_margin));
		}
		FlateRect(rc2,-m.cx,-m.cy);
		CHECK(SetBkMode(hdc,TRANSPARENT));
		SetTextAlign(hdc,TA_CENTER|TA_TOP);
		//CHECK(Rectangle(hdc,rc2.left,rc2.top,rc2.right,rc2.bottom));
		HFONT hf;
		RUN(m_font->get_hFont(&hf));
		HFONT oldF = (HFONT)SelectObject(hdc,hf);

		SIZE ts;
		GetTextExtentPoint32(hdc,m_editing.c_str(),m_editing.size(),&ts);
		SIZE elwid;
		GetTextExtentPoint32(hdc,"",1,&elwid);
		string s2;
		if(ts.cx>(wid(rc2)+elwid.cx/2)) {
			int clip=0;
			GetTextExtentExPoint(hdc,m_editing.c_str(),m_editing.size(),wid(rc2)-elwid.cx,&clip,0,&elwid);
			s2 = string(m_editing,0,clip);
			s2 += "";
		}
		else if(m_editing.size()>8190)
			s2 = string(m_editing,0,8190);
		else
			s2 = m_editing;
		// not satisfactory: don't clip because size isn't always right. argh.
		CHECK(ExtTextOut(hdc,(rc2.left+rc2.right)/2,rc2.top,0,&rc2,s2.c_str(),s2.size(),0));
		SelectObject(hdc,oldF);
		if(!isMF)
			CHECK(RestoreDC(hdc,-1));
	}
	return S_OK;
}
STDMETHODIMP TextEdit::SetClientSite(IOleClientSite *site) {
    METHTRACE(TextEdit,SetClientSite);
	RUN(IOleObjectImpl<TextEdit>::SetClientSite(site));
	if(site) {
		CComDispatchDriver amb = m_spClientSite;
		CComVariant v;
		if(SUCCEEDED(amb.GetPropertyByName(L"Font",&v))&&v.vt==VT_DISPATCH) {
			//RUN(CComQIPtr<IFont>(v.pdispVal)->Clone(&m_font));
			m_font = v.pdispVal;
			m_isAmbient = true;
		}
		else {
			CComPtr<IFontDisp> fd;
			RUN(defaultFont(L"Times New Roman",120000,&fd));
			m_font = fd;
			m_isAmbient = true; // would be ambient if m_font were there
		}
		changed();
	}
	return S_OK;
}
STDMETHODIMP TextEdit::DoVerb(LONG verb,LPMSG msg,IOleClientSite *s,LONG lindex,HWND hwndParent,LPCRECT pos) {
    METHTRACE(TextEdit,DoVerb);
	RUN(IOleObjectImpl<TextEdit>::DoVerb(verb,msg,s,lindex,hwndParent,pos));
	if(m_bInPlaceActive) {
		if(msg) {
			switch(msg->message) {
			case WM_LBUTTONDOWN:
			case WM_LBUTTONUP:
			case WM_LBUTTONDBLCLK:
			case WM_MBUTTONDOWN:
			case WM_MBUTTONUP:
			case WM_MBUTTONDBLCLK:
			case WM_RBUTTONDOWN:
			case WM_RBUTTONUP:
			case WM_RBUTTONDBLCLK:
			{
				// move cursor to mouse pos if known
				POINT pt = msg->pt;
				HWND hwnd;
				RUN(m_spInPlaceSite->GetWindow(&hwnd));
				::ScreenToClient(hwnd,&pt);
				m_pos0 = m_pos1 = getPos(pt.x);
				positionCursor();
				break;
			}
			case WM_KEYDOWN:
			case WM_CHAR:
				// replace text by typing on selection
				if(isprint(msg->wParam)) {
					m_editing.clear();
					m_editing.push_back(msg->wParam);
					m_pos0 = m_pos1 = 1;
					changed();
				}
				break;
			}
		}
		else // otherwise cursor's at end
			m_pos0 = m_pos1 = m_editing.size();
	}
	return S_OK;
}
STDMETHODIMP TextEdit::OnAmbientPropertyChange(DISPID id) {
    METHTRACE(TextEdit,OnAmbientPropertyChange);
	CComDispatchDriver amb = m_spClientSite;
	DISPID idF;
	if(m_isAmbient && SUCCEEDED(amb.GetIDOfName(L"Font",&idF)) && id==idF) {
		CComVariant v;
		RUN(amb.GetProperty(idF,&v));
		_ASSERTE(v.vt==VT_DISPATCH);
		// RUN(CComQIPtr<IFont>(v.pdispVal)->Clone(&m_font));
		m_font = v.pdispVal;
		changed();
	}
	return S_OK;
}
// IOleInPlaceObject
STDMETHODIMP TextEdit::SetObjectRects(LPCRECT pos,LPCRECT clip) {
    METHTRACE(TextEdit,SetObjectRects);
	RUN(IOleInPlaceObjectWindowlessImpl<TextEdit>::SetObjectRects(pos,clip));
    int lastheight = m_lens.m_height;
    // we've been moved or scaled: recalc cursor positions
    computeLens();
    if(m_lens.m_height!=lastheight) {
        deleteCursor();
        createCursor();
    }
    else
	    positionCursor();
	return S_OK;
}

// ICoUIActiveObject
STDMETHODIMP TextEdit::Activate(HMENU,OLEMENUGROUPWIDTHS*) {
    METHTRACE(TextEdit,Activate);
    computeLens();
	focus();
	return S_FALSE;
}
STDMETHODIMP TextEdit::Deactivate(HMENU,OLEMENUGROUPWIDTHS*) {
    METHTRACE(TextEdit,Deactivate);
	unfocus();
	m_text = m_editing;
	RUN(FireOnChanged(0));
	return S_FALSE;
}
// IMCCTextEdit
STDMETHODIMP TextEdit::put_Text(BSTR val) {
    METHTRACE(TextEdit,put_Text);
	USES_CONVERSION;
    if(val)
	    m_text = OLE2T(val);
    else
        m_text.clear();
	m_editing = m_text;
    if(m_spClientSite) {
	    changed();
	    RUN(FireOnChanged(0));
    }
	return S_OK;
}
STDMETHODIMP TextEdit::get_Text(BSTR *ret) {
    METHTRACE(TextEdit,get_Text);
	USES_CONVERSION;
	*ret = SysAllocStringLen(T2OLE(m_text.c_str()),m_text.size());
	return S_OK;
}
// messages
#include <windowsx.h>
LRESULT TextEdit::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,ButtonDown);
	POINT pt = MAKEPOINT(lParam);
	if(!m_focal) {
		computeLens();
		focus();
	}
	m_pos0 = m_pos1 = getPos(pt.x);
	positionCursor();
	RUN(m_spInPlaceSite->SetCapture(true));
	m_capture = true;
	RUN(m_spInPlaceSite->InvalidateRect(0,true));
	return 0;
}
LRESULT TextEdit::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,MouseMove);
	if(!m_capture)
		return 0;
	POINT pt = MAKEPOINT(lParam);
	int p = getPos(pt.x);
	if(p<m_start)
		p = m_start;
	if(m_pos1!=p) {
		m_pos1 = p;
		RUN(m_spInPlaceSite->InvalidateRect(0,true));
		positionCursor();
	}
	return 0;
}
LRESULT TextEdit::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,ButtonUp);
	if(!m_capture) // eg double click
		return 0;
	RUN(m_spInPlaceSite->SetCapture(false));
	m_capture = false;
	return 0;
}
LRESULT TextEdit::DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,DoubleClick);
	selall();
	RUN(m_spInPlaceSite->InvalidateRect(0,true));
	return 0;
}
LRESULT TextEdit::KeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,KeyDown);
	CComQIPtr<IMCCSite> site(m_spClientSite);
	switch(wParam) {
	case VK_RETURN:
		m_text = m_editing;
		RUN(fireSiteEvent(site,fireIntend(MCCS_SHOWN)));
		break;
	case VK_ESCAPE:
		m_editing = m_text;
		changed();
		RUN(fireSiteEvent(site,fireIntend(MCCS_SHOWN)));
		break;
	case VK_BACK:
		if(m_pos0!=m_pos1)
			delsel();
		else if(m_pos0>0) {
			m_editing.erase(m_pos0-1,1);
			m_pos1 = --m_pos0;
		}
		computeLens();
        changed();
		positionCursor();
		RUN(m_spInPlaceSite->InvalidateRect(0,true));
		break;
	case VK_DELETE:
		if(m_pos0!=m_pos1)
			delsel();
		else if(unsigned(m_pos0)<m_editing.size())
			m_editing.erase(m_pos0,1);
		computeLens();
        changed();
		positionCursor();
		RUN(m_spInPlaceSite->InvalidateRect(0,true));
		break;
	case VK_LEFT:
		m_pos1 = m_pos1>0?m_pos1-1:0;
		goto movements;
	case VK_RIGHT:
		m_pos1 = unsigned(m_pos1)<m_editing.size()?m_pos1+1:m_editing.size();
		goto movements;
	case VK_HOME:
		m_pos1 = 0;
		goto movements;
	case VK_END:
		m_pos1 = m_editing.size();
		goto movements;
	movements:
		if(!(GetKeyState(VK_SHIFT)&0x8000))
			m_pos0 = m_pos1;
		positionCursor();
		RUN(m_spInPlaceSite->InvalidateRect(0,true));
		break;
	case 'A':
		if(GetKeyState(VK_CONTROL)&0x8000) {
			selall();
			positionCursor();
			RUN(m_spInPlaceSite->InvalidateRect(0,true));
		}
		break;
	}
	return 0;
}
LRESULT TextEdit::Char(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
    METHTRACE(TextEdit,Char);
	/*
	if(wParam==VK_RETURN) {
		cancelled = false;
		if(owner)
			owner->change();
	}
	else if(wParam==VK_ESCAPE) {
		cancelled = true;
		if(owner)
			owner->change();
	}
	else
	*/
	if(!iscntrl(wParam)) { // control keys handled above in WM_KEYDOWN
		delsel();
		m_editing.insert(m_pos0,1,char(wParam));
		m_pos1 = ++m_pos0;
		computeLens();
        changed();
		positionCursor();
		RUN(m_spInPlaceSite->InvalidateRect(0,true));
	}
	return 0;
}
STDMETHODIMP TextEdit::Save(IStream *stream,BOOL clearDirty) {
    METHTRACE(TextEdit,Save);
	int n = m_text.size();
	RUN(stream->Write(&n,sizeof n,0));
	if(n)
		RUN(stream->Write(m_text.c_str(),n*sizeof(char),0));
	return S_OK;
}
STDMETHODIMP TextEdit::Load(IStream *stream) {
    METHTRACE(TextEdit,Load);
	int n;
	RUN(stream->Read(&n,sizeof n,0));
	m_text.resize(n);
	if(n)
		RUN(stream->Read(&*m_text.begin(),n*sizeof(char),0));
	m_editing = m_text;
	return S_OK;
}
