/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


//#define DRAW_VIA_PATH
#include "RenderPolicy.h"

struct TextEdit : CComObjectRoot,
    IDispatchImpl<IMCCTextEdit>,
    CComCoClass<TextEdit,&CLSID_MCCTextEdit>,
    CComControl<TextEdit>,
	ICoUIActiveObject,
    IPersistStreamInitImpl<TextEdit>,
    IOleObjectImpl<TextEdit>,
	IOleInPlaceObjectWindowlessImpl<TextEdit>,
    IViewObjectExImpl<TextEdit>,
	IOleControlImpl<TextEdit>,
	IDataObjectImpl<TextEdit>,
	IConnectionPointContainerImpl<TextEdit>,
	IPropertyNotifySinkCP<TextEdit>
{
	string m_text,m_editing;
	int m_start, // of editable
		m_pos0,m_pos1, // selection
		m_margin; // all sides (HIMETRIC, argh)
	bool m_focal,m_capture,m_opaque,m_centerX;

	CComQIPtr<IFont> m_font;
	bool m_isAmbient;
	void delsel() {
		if(m_pos0!=m_pos1) {
			if(m_pos0<m_pos1) {
				m_editing.erase(m_pos0,m_pos1-m_pos0);
				m_pos1 = m_pos0;
			}
			else {
				m_editing.erase(m_pos1,m_pos0-m_pos1);
				m_pos0 = m_pos1;
			}
		}
	}
	void selall() {
		m_pos0 = 0;
		m_pos1 = m_editing.size();
	}
	struct charlens {
		vector<int> m_partials;
		int m_height;
		void compute(const string &s, HDC info, HFONT font,IMCCTransform *t) {
			m_partials.resize(s.size());
			SIZE S;
			HFONT oldF = (HFONT)SelectObject(info,font);
			if(s.empty()) {
				GetTextExtentExPoint(info,"Aj",2,0,0,0,&S);
				S.cx = 0;
			}
			else GetTextExtentExPoint(info,s.c_str(),s.size(),0,0,&*m_partials.begin(),&S);
			SelectObject(info,oldF);
			S = RenderPolicy::outSize(S);
			SIZE ws;
			RUN(t->C2WS(S,&ws));
			m_height = -ws.cy;
			for(vector<int>::iterator i = m_partials.begin(); i!=m_partials.end(); ++i) {
				S.cx = RenderPolicy::outX(*i);
				RUN(t->C2WS(S,&ws));
				*i = ws.cx;
			}
		}
		string::size_type x2p(int x) {
			unsigned p = lower_bound(m_partials.begin(),m_partials.end(),x)-m_partials.begin();
			if(p<m_partials.size()) {
				int before = p?m_partials[p-1]:0;
				if(m_partials[p]-x<x-before)
					p++;
			}
			return p;
		}
		int p2x(int p) {
			return p?m_partials[p-1]:0;
		}
	} m_lens;

#ifdef CALC_FONT_VIA_SCREEN
	SIZE figureSize(HDC info,HFONT hf);
	void compute(HDC info);
#endif
	HRESULT setupDC(HDC hdc,POINT LL);
	void changed();
    void computeLens();
	SIZEL margin(bool viewing);
	void createCursor();
	void deleteCursor();
	void positionCursor();
	int getPos(int x);

	TextEdit();
	~TextEdit();

	void focus();
	void unfocus();

	REGISTER_GENERIC(L"TextEdit")

BEGIN_COM_MAP(TextEdit)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IMCCTextEdit)
	COM_INTERFACE_ENTRY(ICoUIActiveObject)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleWindow)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(TextEdit)
	CONNECTION_POINT_ENTRY(IID_IPropertyNotifySink)
END_CONNECTION_POINT_MAP()

BEGIN_PROP_MAP(TextEdit)
END_PROP_MAP()

BEGIN_MSG_MAP(TextEdit)
	MESSAGE_HANDLER(WM_LBUTTONDOWN,ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE,MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP,ButtonUp)
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK,DoubleClick)
	MESSAGE_HANDLER(WM_KEYDOWN,KeyDown)
	MESSAGE_HANDLER(WM_CHAR,Char)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT KeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT Char(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
// CComControlBase override, to run compute() when we've got context
	HRESULT OnPostVerbInPlaceActivate();
// IViewObject
	STDMETHODIMP Draw(DWORD dwDrawAspect,LONG lindex,void *pvAspect,
		DVTARGETDEVICE *ptd,HDC hdcTargetDev,HDC hdc,LPCRECTL bounds,
		LPCRECTL lprcWBounds,BOOL(STDMETHODCALLTYPE *pfnContinue )(DWORD dwContinue),
		DWORD dwContinue);
// IOleObject hooks
	STDMETHODIMP SetClientSite(IOleClientSite *site);
	STDMETHODIMP DoVerb(LONG verb,LPMSG msg,IOleClientSite *s,LONG lindex,HWND hwndParent,LPCRECT pos);
// IOleControl hook
	STDMETHODIMP OnAmbientPropertyChange(DISPID id);
// IOleInPlaceObject hook
	STDMETHODIMP SetObjectRects(LPCRECT pos,LPCRECT clip);
// ICoUIActiveObject
	STDMETHODIMP Activate(HMENU,OLEMENUGROUPWIDTHS*);
	STDMETHODIMP Deactivate(HMENU,OLEMENUGROUPWIDTHS*);
// IMCCTextEdit
	STDMETHODIMP put_Text(BSTR val);
	STDMETHODIMP get_Text(BSTR *ret);
// IPersistStreamInit
	STDMETHODIMP Save(IStream *stream,BOOL clearDirty);
	STDMETHODIMP Load(IStream *stream);
};