/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ToolBack.h"


/////////////////////////////////////////////////////////////////////////////
// ToolBack

ToolBack::ToolBack() : m_gooddrop(false) {
	CONSTRUCTORTRACE(ToolBack);
	m_bWindowOnly = true;
}
ToolBack::~ToolBack() {
	DESTRUCTORTRACE(ToolBack);
}
HRESULT ToolBack::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	METHTRACE(ToolBack,OnDrawAdvanced);
	RECT rc = *(RECT*)di.prcBounds;
	HDC hdc = di.hdcDraw;
	// seems like if you use OnDraw, ATL "corrects" the rect so it extends
	// beyond right, bottom.  But here it includes them, so we have to extend
	// to make fillrect work.
	rc.right++;
	rc.bottom++;
	HBRUSH hbr = GetSysColorBrush(COLOR_3DFACE);
	FillRect(hdc,&rc,hbr);
	return S_OK;
}
// IDropTarget
FORMATETC g_fetz = {0, // cfFormat
	0, // ptd
	DVASPECT_CONTENT, // dwAspect
	-1, // lindex
	TYMED_ISTORAGE // any need for IStream?
};
inline FORMATETC *format(DWORD f) {
	g_fetz.cfFormat = CLIPFORMAT(f);
	return &g_fetz;
}
STDMETHODIMP ToolBack::DragEnter(IDataObject *pDataObj,DWORD grfKeyState,POINTL pt,
								 DWORD *pdwEffect) {
	m_gooddrop = SUCCEEDED(pDataObj->QueryGetData(format(g_clipformats.embedded)))||
		SUCCEEDED(pDataObj->QueryGetData(format(g_clipformats.embedsource)));
	*pdwEffect = m_gooddrop?DROPEFFECT_COPY:DROPEFFECT_NONE;
	return S_OK;
}
STDMETHODIMP ToolBack::DragOver(DWORD grfKeyState,POINTL pt,DWORD *pdwEffect) {
	*pdwEffect = m_gooddrop?DROPEFFECT_COPY:DROPEFFECT_NONE;
	return S_OK;
}
STDMETHODIMP ToolBack::DragLeave() {
	return S_OK;
}
STDMETHODIMP ToolBack::Drop(IDataObject *pDataObj,DWORD grfKeyState,POINTL ptScreen,
							DWORD *pdwEffect) {
    METHTRACE(ToolBack,Drop);
	if(!(*pdwEffect = m_gooddrop?DROPEFFECT_COPY:DROPEFFECT_NONE))
		return S_OK;
	CComPtr<IOleContainer> cont;
	RUN(m_spClientSite->GetContainer(&cont));
	CComQIPtr<IMontageContainer> mc = cont;
	CComPtr<IMCCSitePersistor> sp;
	CComPtr<IItemSet> dataset;
	RUN(CComQIPtr<IOleItemContainer>(cont)->GetObject(L"Data",BINDSPEED_INDEFINITE,BindCtx(),riidret(&dataset)));
	if(FAILED(m_reader->UnpackOne(pDataObj,format(g_clipformats.embedded),0,dataset,&sp)))
		RUN(m_reader->UnpackOne(pDataObj,format(g_clipformats.embedsource),0,dataset,&sp));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));

	// mooched from edit mode: get point in window from screen point.   This kind of sucks!
	HWND hWnd;
	RUN(m_spInPlaceSite->GetWindow(&hWnd));
	POINT pt = *(POINT *)&ptScreen; // why are they using a POINTL for screen coords?
	::ScreenToClient(hWnd,&pt);

	// the following code makes a lot of assumptions about data that should be provided by unpackers.
	MCCSITEPERSISTORDATA data;
	RUN(sp->get_Data(&data));
	if(data.pos.postype!=MCC_POS_NONE) {
		RECT rc;
		if(data.pos.postype==MCC_POS_CANVASRECT) {
			// The unpackers (EmbedUnpacker in this case) place objects at 0,0, which isn't
			// really a position!  Still need to offset by current mouse position and the mouse offset
			// within content.  It's slightly easier and more accurate to do all the calculation in canvas
			// coordinates, even though there isn't really a toolbar canvas.
			POINTL canvPt,mouseOfs;
			RUN(t->W2CP(pt, &canvPt)); // get canvas point of mouse
			SIZEL size, offset;
			RUN(getsizepoint(pDataObj,&size,&mouseOfs));
			// mouseOfs is distance in HIMETRIC from upperleft corner; we want lower left!
			offset.cx = canvPt.x - mouseOfs.x;
			offset.cy = canvPt.y + mouseOfs.y - size.cy;
			data.pos.rcl += offset;
			RUN(t->C2WR(&data.pos.rcl,&rc));
		}
		else if(data.pos.postype==MCC_POS_WINDOWRECT) {
			rc = data.pos.rc + size(pt.x,pt.y);
		}
		else _ASSERTE(false);
		data.pos.postype = MCC_POS_WINDOWRECT;
		data.pos.rc = rc;
	}
	else {
		data.pos.postype = MCC_POS_WINDOWPOINT;
		data.pos.pt = pt;
	}
	RUN(sp->put_Data(&data));
	// tell layout annotators
	RUN(fireEvent(mc,fireTwoParams<IMCCSiteLayoutEvents,IMCCSitePersistor*,LPOLESTR>
		(&IMCCSiteLayoutEvents::Process,sp,0)));
	// realize persistor.  Not actually necessary because ToolLayout does this too,
	// but ToolLayout probably shouldn't.
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(bctx));
	return S_OK;
}
// INeedOnePointer
STDMETHODIMP ToolBack::TakePointer(IUnknown *u,IMoniker *m) {
    METHTRACE(ToolBack,TakePointer);
	m_reader = u;
	return S_OK;
}
