/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ToolLayout.h"
#include "MBasics.h"
#include "createAffixMoniker.h"

ToolLayout::ToolLayout() : m_useBars(true),m_quietBars(false) {
	CONSTRUCTORTRACE(ToolLayout);
}
ToolLayout::~ToolLayout() {
	DESTRUCTORTRACE(ToolLayout);
}

HRESULT ToolLayout::Process(IMCCSitePersistor *sp, LPOLESTR baseName) {
    METHTRACE(ToolLayout,Process);
	CComPtr<IBindCtx> bctx;
	RUN(CreateBindCtx(0,&bctx));
	CComPtr<IMoniker> mkGroup;
	RUN(FoolishCreatePointerMoniker(m_group,&mkGroup));
	CComPtr<IMontagePersistor> montagePersistor;
	RUN(m_mkMontage->BindToObject(bctx,mkGroup,riidret(&montagePersistor)));
	MBasics mb;
	mb.init(montagePersistor);

	// owner
	CComPtr<INeedOnePointer> barown;
	RUN(CREATE(barown,CLSID_MCCToolbarOwner));
	RUN(barown->TakePointer(m_rectEngine,0));

	// get site
	RUN(CComQIPtr<IGroupItem>(sp)->Realize(mb.bctx));
	CComPtr<IUnknown> uBuddy;
	RUN(CComQIPtr<DIPersistor>(sp)->get_Subject(&uBuddy));
	CComPtr<IMCCSite> buddy;
	RUN(uBuddy.QueryInterface(&buddy));
	CComPtr<IMoniker> mkBuddyPer,mkBuddy;
	RUN(buddy->get_RelMoniker(&mkBuddyPer));
	RUN(mkBuddyPer->ComposeWith(mb.goSubject,false,&mkBuddy));

    CComPtr<IMCCSitePersistor> siteper;
    CComPtr<IMoniker> mkToSave;
    if(m_useBars) {
	    // create bar
	    CComPtr<DIMCCToolbar> bar;
	    RUN(CREATE(bar,CLSID_MCCToolbar));
	    RUN(bar->put_Caption(CComBSTR(baseName?baseName:L"")));
        if(m_quietBars) {
            RUN(bar->put_RaiseSelf(VARIANT_FALSE));
            RUN(bar->put_SinkBuddy(VARIANT_FALSE));
            RUN(bar->put_ShowGrips(VARIANT_FALSE));
            RUN(bar->put_AllowMove(VARIANT_FALSE));
        }
        else {
            RUN(bar->put_RaiseSelf(VARIANT_TRUE));
            RUN(bar->put_SinkBuddy(VARIANT_TRUE));
            RUN(bar->put_ShowGrips(VARIANT_TRUE));
            RUN(bar->put_AllowMove(VARIANT_TRUE));
        }
	    // .. site
	    CComPtr<IMCCSite> barsite;
	    RUN(mb.c->new_Site(&barsite));

	    // hook em up
	    RUN(CComQIPtr<INeedOnePointer>(bar)->TakePointer(buddy,0));
	    RUN(barsite->put_Layer(m_barLayer));
	    RUN(barsite->put_Control(bar)); // setsite on control sets pos based on buddy
	    RUN(barsite->put_Intent(MCCS_ACTIVE));

	    // setsite on owner sends pos to engine
	    RUN(CComQIPtr<IObjectWithSite>(barown)->SetSite(barsite));

        // persist bar
	    CComPtr<IMCCSitePersistor> barSP;
	    RUN(CREATE(barSP,CLSID_MCCSitePersistor));
	    CComPtr<IMoniker> barSiteName0, barSiteName;
	    if(baseName)
		    RUN(createAffixMoniker(baseName,L"Bar Site",&barSiteName0));
	    RUN(mb.g->Paste(mb.bctx,GP_SrcIsObj,barSP,barSiteName,IID_NULL,0,&barSiteName));
	    RUN(barsite->put_RelMoniker(barSiteName));
	    CComPtr<IMoniker> mkMontage, mkBarPer,mkBar;
	    RUN(mb.c->get_RelMoniker(&mkMontage));
    	CComPtr<IHoldOnePointer> hop;
	    RUN(CREATE(hop,CLSID_OnePointerPersistor));
	    CComPtr<IMoniker> barName;
	    if(baseName)
		    RUN(createAffixMoniker(baseName,L"Bar",&barName));
	    RUN(mb.g->Paste(mb.bctx,GP_SrcIsObj,hop,barName,IID_NULL,0,&mkBarPer));
	    RUN(hop->put_Moniker(mkBuddy));
	    RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,bar));
	    RUN(mkBarPer->ComposeWith(mb.goSubject,false,&mkBar));
	    // init site persistor
	    RUN(barSP->Initialize(mkMontage,m_mkBarLayer,m_mkBarSiteOrdering,mkBar,0,0));
        siteper = barSP;
    	RUN(CComQIPtr<DIPersistor>(barSP)->SetSubject(true,barsite));
	    CComPtr<IMoniker> fullBarSiteName;
	    RUN(barSiteName->ComposeWith(mb.goSubject,0,&fullBarSiteName));
        mkToSave = fullBarSiteName;
    }
    else { // connect owner directly to content
	    RUN(CComQIPtr<IObjectWithSite>(barown)->SetSite(buddy));
        RUN(mb.bind(mkBuddyPer,&siteper));
        mkToSave = mkBuddy;
    }
	// owner
	CComPtr<IHoldOnePointer> hop;
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	CComPtr<IMoniker> barOwnerName;
	if(baseName)
		RUN(createAffixMoniker(baseName,L"Bar Owner",&barOwnerName));
    CComPtr<IMoniker> mkOwnerPersistor,mkOwner;
	RUN(mb.g->Paste(mb.bctx,GP_SrcIsObj,hop,barOwnerName,IID_NULL,0,&mkOwnerPersistor));
	RUN(hop->put_Moniker(m_mkEngine));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,barown));
	RUN(mkOwnerPersistor->ComposeWith(mb.goSubject,false,&mkOwner));

    // set owner as an initializer of content or bar
	CComPtr<IOrderedItemList> oil;
	RUN(siteper->get_InitList(&oil));
	RUN(oil->SetMoniker(10,mkOwner)); // 10?

	/*
	// (didn't work: tool's site persistor calls out on initlist before it sends position (to itself)
	// and bar needs tool's position
	// hack, sorta: put mk to bar in init list of tool so that it gets dragged along, realized along with.
	// (not a bad idea xept bar already has a mk back.  This circ. reference can't be good.  Maybe it would
	// make more sense to have an owner for the *tool* that sets everything up.)
	// This is OK because the site doesn't support IObjectWithSite or IInitializeOther.
	// it used to be that both bar & tool went on config keeplist (IMontagePersistor::KL) but we're not
	// even returning mkBar.
	// without further ado...
	oil.Release();
	RUN(sp->get_InitList(&oil));
	RUN(oil->SetMoniker(2000,fullBarSiteName));
	*/
	// hack v2.  This works but it's really no good because clients should determine what's kept and what's not.
	// but right now there's no way to delete a toolbar!
	RUN(mb.configset->Add(mkToSave));

	return S_OK;
}
STDMETHODIMP ToolLayout::Initialize(IMoniker *mkMontage, IMoniker *mkEngine, IMoniker *mkBarLayer, IMoniker *mkBarSiteOrdering, VARIANT_BOOL useBars) {
    METHTRACE(ToolLayout,Initialize);
	if(m_mkMontage)
		ERR(E_UNEXPECTED);
	if(!m_group)
		ERR(E_PENDING);
	m_mkMontage = mkMontage;
	m_mkEngine = mkEngine;
	m_mkBarLayer = mkBarLayer;
	m_mkBarSiteOrdering = mkBarSiteOrdering;
	RUN(m_group->AddStgRef(m_mkMontage));
	RUN(m_group->AddStgRef(m_mkEngine));
	RUN(m_group->AddStgRef(m_mkBarLayer));
	RUN(m_group->AddStgRef(m_mkBarSiteOrdering));
    put_UseBars(useBars);
	return S_OK;
}
STDMETHODIMP ToolLayout::put_UseBars(VARIANT_BOOL val) {
    METHTRACE(ToolLayout,put_UseBars);
    m_useBars = val!=0;
    return S_OK;
}
STDMETHODIMP ToolLayout::put_QuietBars(VARIANT_BOOL val) {
    METHTRACE(ToolLayout,put_QuietBars);
    m_quietBars = val!=0;
    return S_OK;
}
// DIMCCToolLayout
STDMETHODIMP ToolLayout::Initialize(IUnknown *mkMontagePer, IUnknown *mkEngine, IUnknown *mkBarLayer, IUnknown *mkBarSiteOrdering,VARIANT_BOOL useBars) {
    METHTRACE(ToolLayout,Initialize);
	CComPtr<IMoniker> m2,e2,bl2,so2;
	if(mkMontagePer)
		RUNRET(mkMontagePer->QueryInterface(riidret(&m2)));
	if(mkEngine)
		RUNRET(mkEngine->QueryInterface(riidret(&e2)));
	if(mkBarLayer)
		RUNRET(mkBarLayer->QueryInterface(riidret(&bl2)));
	if(mkBarSiteOrdering)
		RUNRET(mkBarSiteOrdering->QueryInterface(riidret(&so2)));
	return Initialize(m2,e2,bl2,so2,useBars!=VARIANT_FALSE);
}
//IGroupItem
STDMETHODIMP ToolLayout::Connect(IGroup *group) {
    METHTRACE(ToolLayout,Connect);
	m_group = group;
	return S_OK;
}
STDMETHODIMP ToolLayout::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
    METHTRACE(ToolLayout,Copy);
	if(!m_mkMontage || !m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> mkM2, mkE2, mkBL2, mkBSO2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkMontage,IID_NULL,0,&mkM2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkEngine,IID_NULL,0,&mkE2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkBarLayer,IID_NULL,0,&mkBL2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkBarSiteOrdering,IID_NULL,0,&mkBSO2));
	CComPtr<IMCCToolLayout> copy;
	RUN(CREATE(copy,CLSID_MCCToolLayout));
	RUN(CComQIPtr<IGroupItem>(copy)->Connect(dest));
	RUN(copy->Initialize(mkM2,mkE2,mkBL2,mkBSO2,m_useBars*VARIANT_TRUE));
	return copy.p->QueryInterface(riid,ret);
}
STDMETHODIMP ToolLayout::Realize(IBindCtx *bctx) {
    METHTRACE(ToolLayout,Realize);
	if(!m_mkMontage || !m_group)
		ERR(E_PENDING);
	if(m_barLayer)
		return S_FALSE;
	CComPtr<IBindCtx> bctxRef;
	if(!bctx) {
		RUN(CreateBindCtx(0,&bctx));
		bctxRef.Attach(bctx);
	}
	CComPtr<IMoniker> mkGroup;
	RUN(FoolishCreatePointerMoniker(m_group,&mkGroup));
	CComPtr<IMontagePersistor> montagePersistor;
	RUN(m_mkMontage->BindToObject(bctx,mkGroup,riidret(&montagePersistor)));
	// set up as layout sink (presumably for ToolBack's drops)
	CComPtr<IUnknown> umc;
	RUN(CComQIPtr<DIPersistor>(montagePersistor)->get_Subject(&umc));
	CComPtr<IMontageContainer> mc;
	RUN(umc.QueryInterface(&mc));
	DWORD digestiveBiscuit; // can forget this: layout stays for good (at least for now!)
	RUN(APCAdvise(mc,(IMCCSiteLayoutEvents*)this,IID_IMCCSiteLayoutEvents,&digestiveBiscuit));
	RUN(m_mkEngine->BindToObject(bctx,mkGroup,riidret(&m_rectEngine)));
	RUN(m_mkBarLayer->BindToObject(bctx,mkGroup,riidret(&m_barLayer)));
	return S_OK;
}
STDMETHODIMP ToolLayout::Remove() {
    METHTRACE(ToolLayout,Remove);
	if(!m_mkMontage || !m_group)
		ERR(E_PENDING);
	RUN(m_group->ReleaseStgRef(m_mkMontage));
	RUN(m_group->ReleaseStgRef(m_mkEngine));
	RUN(m_group->ReleaseStgRef(m_mkBarLayer));
	RUN(m_group->ReleaseStgRef(m_mkBarSiteOrdering));
	return S_OK;
}
// IPersistStream
STDMETHODIMP ToolLayout::GetClassID(CLSID *pClassID) {
    METHTRACE(ToolLayout,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP ToolLayout::IsDirty(void) {
    METHTRACE(ToolLayout,IsDirty);
	return S_FALSE;
}
STDMETHODIMP ToolLayout::Load(IStream *pStm) {
    METHTRACE(ToolLayout,Load);
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkMontage)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkEngine)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkBarLayer)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkBarSiteOrdering)));
    RUN(pStm->Read(&m_useBars,sizeof(bool),0));
	return S_OK;
}
STDMETHODIMP ToolLayout::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(ToolLayout,Save);
	RUN(FixedOleSaveToStream(m_mkMontage,pStm));
	RUN(FixedOleSaveToStream(m_mkEngine,pStm));
	RUN(FixedOleSaveToStream(m_mkBarLayer,pStm));
	RUN(FixedOleSaveToStream(m_mkBarSiteOrdering,pStm));
    RUN(pStm->Write(&m_useBars,sizeof(bool),0));
	return S_OK;
}
STDMETHODIMP ToolLayout::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(ToolLayout,GetSizeMax);
	ERR(E_NOTIMPL);
}
