/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ToolLayout.h : Declaration of the DropBack

#ifndef __ToolLayout_H_
#define __ToolLayout_H_

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// ToolLayout
class ATL_NO_VTABLE ToolLayout :
	public CComObjectRoot,
	public CComCoClass<ToolLayout, &CLSID_MCCToolLayout>,
	public IMCCSiteLayoutEvents,
	public IMCCToolLayout,
	public IDispatchImpl<DIMCCToolLayout>,
	public IGroupItem,
	public IPersistStream
{
	// "real" side
	// I had CComPtr<IMontagePersistor> m_montagePersistor as a member, which is a good idea
	// except MP refs Montage refs allpointscontainer refs me....
	CComPtr<IMCCLayer> m_barLayer;
	CComPtr<IRectEngine> m_rectEngine;
	// "persistor" side
	CComPtr<IMoniker> m_mkMontage,m_mkEngine, m_mkBarLayer, m_mkBarSiteOrdering;
	CComPtr<IGroup> m_group;
    bool m_useBars, m_quietBars;
public:
	ToolLayout();
	~ToolLayout();

	REGISTER_GENERIC(L"ToolLayout")

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(ToolLayout)
	COM_INTERFACE_ENTRY(IMCCSiteLayoutEvents)
	COM_INTERFACE_ENTRY(IMCCToolLayout)
	COM_INTERFACE_ENTRY(DIMCCToolLayout)
	COM_INTERFACE_ENTRY(IGroupItem)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IPersistStream)
END_COM_MAP()

// IMCCSiteLayoutEvents
	STDMETHODIMP Process(IMCCSitePersistor *pers, LPOLESTR baseName);
// IMCCToolLayout
	STDMETHODIMP Initialize(IMoniker *mkMontagePer, IMoniker *mkEngine, IMoniker *mkBarLayer, IMoniker *mkBarSiteOrdering, VARIANT_BOOL useBars);
    STDMETHODIMP put_UseBars(VARIANT_BOOL val);
    STDMETHODIMP put_QuietBars(VARIANT_BOOL val);
// DIMCCToolLayout
	STDMETHODIMP Initialize(IUnknown *mkMontagePer, IUnknown *mkEngine, IUnknown *mkBarLayer, IUnknown *mkBarSiteOrdering, VARIANT_BOOL useBars);
// IGroupItem
	STDMETHODIMP Connect(IGroup *group);
	STDMETHODIMP Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret);
	STDMETHODIMP Realize(IBindCtx *bctx);
	STDMETHODIMP Remove();
// IPersistStream
	STDMETHODIMP GetClassID(CLSID *pClassID);
	STDMETHODIMP IsDirty(void);
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
public:
};

#endif //__ToolLayout_H_
