/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Toolbar.h"
#include "cursors.h"

Toolbar::Toolbar() :
	m_caption(L"Toolbar"),
	m_tracking(false),
	m_sigtrack(false),
	m_firstdraw(true),
	m_docked(true),
	m_sinkbuddy(true),
    m_raiseself(true),
    m_allowmove(true),
	m_fudge(0),
	m_blondie(0),
    m_grips(true)
{
	CONSTRUCTORTRACE(Toolbar);
	m_bWindowOnly = TRUE;
	// learned from mfc.
	WORD ray[8]; // twice the size we need, but CreateBitmap wants rows word-aligned
	for(int i=0;i<8;i++)
		ray[i] = WORD(0x5555 << (i&1));
	m_bump = CreateBitmap(8,8,1,1,ray);
	m_checkerbrush = CreatePatternBrush(m_bump);
}
Toolbar::~Toolbar() {
	DESTRUCTORTRACE(Toolbar);
	DeleteObject(m_checkerbrush);
	DeleteObject(m_bump);
}
const int gripsize = 8,
    BORDER = 2;
RECT Toolbar::drect(bool sink) {
	sink = true;
    return rect(m_grips*gripsize+BORDER+sink*m_sinkbuddy,BORDER+sink*m_sinkbuddy,
		-BORDER-sink*m_sinkbuddy,-BORDER-sink*m_sinkbuddy);
}
STDMETHODIMP Toolbar::SetClientSite(IOleClientSite *s) {
	METHTRACE(Toolbar,SetClientSite);
	// TakePointer must get hit first, either manually or through the group
	// this is all quite arbitrary.  Montage used to try to support
	// OLEMISC_SETCLIENTSITEFIRST but that's pretty complicated because
	// it mixes persistence and view-level stuff
	if(!m_buddy)
		ERR(E_PENDING);
	RUN(IOleObjectImpl<Toolbar>::SetClientSite(s));
	if(!s)
		return S_OK;
	MCC_POS_INFO mpi;
	mpi.postype = MCC_POS_WINDOWRECT;
	RUN(m_buddy->get_Rect(&mpi.rc));
    mpi.rc -= drect();
	CComQIPtr<IMCCSite> ms = m_spClientSite;
	return fireSiteEvent(ms,fireMove(&mpi));
}
HRESULT Toolbar::inrect(LPRECT ret) {
	METHTRACE(Toolbar,inrect);
	if(m_docked) {
		CComQIPtr<IMCCSite> s = m_spClientSite;
		if(!s)
			ERR(E_FAIL);
		RUN(s->get_Rect(ret));
        *ret += drect();
	}
	else {
		GetClientRect(ret);
		if(m_sinkbuddy)
			InflateRect(ret,-1,-1);
	}
	return S_OK;
}
HRESULT Toolbar::outrect(LPCRECT input,LPRECT output) {
    *output = *input - drect();
	return S_OK;
}
// there are certainly better ways to do this.
HWND Toolbar::ringWnd() {
	METHTRACE(Toolbar,ringWnd);
	CComPtr<IOleContainer> oc;
	m_spClientSite->GetContainer(&oc);
	if(!oc)
		return 0;
	CComQIPtr<IOleWindow> ow = oc;
	if(!ow)
		return 0;
	HWND ret = 0;
	ow->GetWindow(&ret);
	return ret;
}
void gripHilite(HDC hdc,const RECT &outside,const RECT &inside,int offsetx) {
	MoveToEx(hdc,offsetx+outside.left+BORDER,inside.bottom,0);
	LineTo(hdc,offsetx+outside.left+BORDER,inside.top);
	LineTo(hdc,offsetx+outside.left+BORDER+3,inside.top);
}
void gripShadow(HDC hdc,const RECT &outside,const RECT &inside,int offsetx) {
	MoveToEx(hdc,offsetx+outside.left+BORDER,inside.bottom,0);
    LineTo(hdc,offsetx+outside.left+BORDER+2,inside.bottom);
    LineTo(hdc,offsetx+outside.left+BORDER+2,inside.top);
}
HRESULT Toolbar::OnDraw(ATL_DRAWINFO& di)
{
	METHTRACE(Toolbar,OnDraw);
	const RECT& rc = *(RECT*)di.prcBounds;
	HDC hdc = di.hdcDraw;
    RECT inside = rc+drect(false);
    inside.right--; // we want to use an *inclusive* rect, i.e. right is actual rightmost pixel
    inside.bottom--;


	HBRUSH hbr = CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	FillRect(hdc,&rc,hbr);
	DeleteObject(hbr);

	HPEN hilite = CreatePen(PS_SOLID,0,GetSysColor(COLOR_3DHILIGHT)),
		shadow = CreatePen(PS_SOLID,0,GetSysColor(COLOR_3DSHADOW));
	HPEN old = (HPEN)SelectObject(hdc,hilite);
	if(m_docked) {
		// main hilite
        if(m_raiseself) {
		    MoveToEx(hdc,rc.left,rc.bottom,0);
		    LineTo(hdc,rc.left,rc.top);
		    LineTo(hdc,rc.right,rc.top);
        }
		// grabber hilites
        if(m_grips) {
            gripHilite(hdc,rc,inside,0);
            gripHilite(hdc,rc,inside,4);
        }
		if(m_sinkbuddy) { // sinking border hilite
			MoveToEx(hdc,inside.left,inside.bottom,0);
			LineTo(hdc,inside.right,inside.bottom);
			LineTo(hdc,inside.right,inside.top);
		}
	}
	else if(m_sinkbuddy) { // sinking border hilite along edge of client area
		MoveToEx(hdc,rc.left+1,rc.bottom-1,0);
		LineTo(hdc,rc.right-1,rc.bottom-1);
		LineTo(hdc,rc.right-1,rc.top);
	}
	SelectObject(hdc,shadow);
	if(m_docked) {
		// main shadow
        if(m_raiseself) {
		    MoveToEx(hdc,rc.left+1,rc.bottom-1,0);
		    LineTo(hdc,rc.right-1,rc.bottom-1);
		    LineTo(hdc,rc.right-1,rc.top);
        }
		// grabber shadows
        if(m_grips) {
            gripShadow(hdc,rc,inside,0);
            gripShadow(hdc,rc,inside,4);
        }
		if(m_sinkbuddy) { // sinking border
			MoveToEx(hdc,inside.left,inside.bottom,0);
			LineTo(hdc,inside.left,inside.top);
			LineTo(hdc,inside.right+1,inside.top);
		}
	}
	else if(m_sinkbuddy) { // sinking border shadow along edge of client area
		MoveToEx(hdc,rc.left,rc.bottom,0);
		LineTo(hdc,rc.left,rc.top);
		LineTo(hdc,rc.right,rc.top);
	}
	SelectObject(hdc,old);
	DeleteObject(hilite);
	DeleteObject(shadow);
	return S_OK;
}
HRESULT Toolbar::setBuddyExtent(const RECT &rc) {
    METHTRACE(Toolbar,setBuddyExtent);
    RUN(m_buddy->UpdateExtent(VARIANT_TRUE));

	CComPtr<IOleObject> oo;
	m_buddy->GetControl(riidret(&oo));
	if(!oo)
		return S_FALSE;
	CComPtr<IMontageContainer> mc;
	RUN(m_buddy->GetContainer(riidret(&mc)));
	CComPtr<IMCCTransform> t;
	RUN(mc->get_Transform(&t));
	SIZEL ext;
	RUN(t->W2CS(siz(rc),&ext));
	ext.cy = tabs(ext.cy);
	RUN(oo->SetExtent(DVASPECT_CONTENT,&ext));
	return S_OK;
}
STDMETHODIMP Toolbar::SetObjectRects(LPCRECT pos,LPCRECT clip) {
	METHTRACE(Toolbar,SetObjectRects);
	RECT rc;
	RUN(IOleInPlaceObjectWindowlessImpl<Toolbar>::SetObjectRects(pos,clip));
	RUN(inrect(&rc));
	RUN(setBuddyExtent(rc));
	return m_buddy->put_Rect(&rc);
}
void Toolbar::drophint(LPCRECT lpcr,bool docking) {
	METHTRACE(Toolbar,drophint);
	// translate to screen coords
	HWND ring = ringWnd();
	RECT rc = *lpcr;
	POINT *hack = (POINT *)&rc;
	::ClientToScreen(ring,hack);
	::ClientToScreen(ring,hack+1);
	HDC hdc = ::GetDC(0);
	if(docking) {
		SetROP2(hdc,R2_XORPEN);
		HPEN hpen = CreatePen(PS_DOT,1,RGB(255,255,255)),
			old = (HPEN)SelectObject(hdc,hpen);
		SelectObject(hdc,GetStockObject(NULL_BRUSH));
		Rectangle(hdc,rc.left,rc.top,rc.right,rc.bottom);
		SelectObject(hdc,old);
		DeleteObject(hpen);
	}
	else {
		SetBkColor(hdc,RGB(0,0,0));
		SetTextColor(hdc,RGB(255,255,255));
		HBRUSH old = (HBRUSH)SelectObject(hdc,m_checkerbrush);
		PatBlt(hdc,rc.left,rc.top,3,hei(rc),PATINVERT);
		PatBlt(hdc,rc.left+3,rc.top,wid(rc)-3,3,PATINVERT);
		PatBlt(hdc,rc.right-3,rc.top+3,3,hei(rc)-3,PATINVERT);
		PatBlt(hdc,rc.left+3,rc.bottom-3,wid(rc)-6,3,PATINVERT);
		SelectObject(hdc,old);
	}
	::ReleaseDC(0,hdc);
}
LRESULT Toolbar::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(Toolbar,OnLButtonDown);
    if(!m_allowmove) {
        bHandled = false;
        return 0;
    }
	POINT pt = MAKEPOINT(lParam);
	if(uMsg==WM_NCLBUTTONDOWN)
		ScreenToClient(&pt);
	SetCapture();
	m_down = pt;
	m_tracking = true;
	m_sigtrack = false;
	if(uMsg==WM_NCLBUTTONDOWN)
		m_trackside = wParam;
	else {
		RECT rc;
		GetClientRect(&rc);
		if(pt.x<rc.left+2) {
			if(pt.y<rc.top+2)
				m_trackside = HTTOPLEFT;
			else if(pt.y>rc.bottom-3)
				m_trackside = HTBOTTOMLEFT;
			else
				m_trackside = HTLEFT;
		}
		else if(pt.x>rc.right-3) {
			if(pt.y<rc.top+2)
				m_trackside = HTTOPRIGHT;
			else if(pt.y>rc.bottom-3)
				m_trackside = HTBOTTOMRIGHT;
			else
				m_trackside = HTRIGHT;
		}
		else if(pt.y<rc.top+2)
			m_trackside = HTTOP;
		else if(pt.y>rc.bottom-3)
			m_trackside = HTBOTTOM;
		else
			m_trackside = HTCAPTION;
	}
	return S_OK;
}
SIZE floatextra() {
	SIZE ret = {GetSystemMetrics(SM_CXSIZEFRAME),
		GetSystemMetrics(SM_CYSMCAPTION)+GetSystemMetrics(SM_CYSIZEFRAME)};
	return ret;
}
#define SIGTRACK 2
template<class T>
inline T sqr(T x) { return x*x; }
LRESULT Toolbar::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(Toolbar,OnMouseMove);
	POINT pt = MAKEPOINT(lParam);
	if(!m_tracking) {
		if(m_docked) {
			RECT rc;
			GetClientRect(&rc);
			if(pt.x<rc.left+2) {
				if(pt.y<rc.top+2)
					SetCursor(g_cursors.size[0]);
				else if(pt.y>rc.bottom-3)
					SetCursor(g_cursors.size[2]);
				else
					SetCursor(g_cursors.size[1]);
			}
			else if(pt.x>rc.right-3) {
				if(pt.y<rc.top+2)
					SetCursor(g_cursors.size[2]);
				else if(pt.y>rc.bottom-3)
					SetCursor(g_cursors.size[0]);
				else
					SetCursor(g_cursors.size[1]);
			}
			else if(pt.y<rc.top+2)
				SetCursor(g_cursors.size[3]);
			else if(pt.y>rc.bottom-3)
				SetCursor(g_cursors.size[3]);
			else
				SetCursor(g_cursors.arrow);
		}
		return 0;
	}
	if(!m_sigtrack) {
		if((sqr(pt.x-m_down.x)+sqr(pt.y-m_down.y))<sqr(SIGTRACK))
			return 0;
		m_sigtrack = true;
	}
	// erase old hint
	if(!m_firstdraw)
		drophint(&m_lastrect,m_lastdock);

	// get new rect
	CComQIPtr<IMCCSite> site = m_spClientSite;
	RECT rc; // rect in screen coords
	if(m_docked)
		RUN(site->get_Rect(&rc));
	else {
		HWND ring = ringWnd();
		GetWindowRect(&rc);
		// take off the border room, add the grabber room
		SIZE fext = floatextra();
		rc.bottom -= fext.cy;
		rc.right -= fext.cx - 9;
		POINT *hack = (POINT *)&rc;
		::ScreenToClient(ring,hack);
		::ScreenToClient(ring,hack+1);
	}
	// for the moment, don't worry about natural size of contained!
	switch(m_trackside) {
	case HTTOPLEFT:
		rc.top += pt.y - m_down.y;
	case HTLEFT:
		rc.left += pt.x - m_down.x;
		break;
	case HTTOPRIGHT:
		rc.right += pt.x - m_down.x;
	case HTTOP:
		rc.top += pt.y - m_down.y;
		break;
	case HTBOTTOMRIGHT:
		rc.bottom += pt.y - m_down.y;
	case HTRIGHT:
		rc.right += pt.x - m_down.x;
		break;
	case HTBOTTOMLEFT:
		rc.left += pt.x - m_down.x;
	case HTBOTTOM:
		rc.bottom += pt.y - m_down.y;
		break;
	default:  // move
		rc += (pt - m_down);
	}
	m_firstdraw = false;
	if(m_trackside==HTCAPTION) {
		// we want to dock if any IMontageInclusion sink says so
		m_lastdock = false;
		CComPtr<IConnectionPointContainer> cpc;
		if(SUCCEEDED(site->GetContainer(IID_IConnectionPointContainer,(void**)&cpc)))
			for(connection_iterator i(getSinks(cpc,IID_IMontageInclusion));i!=connection_iterator();++i)
				if(CComQIPtr<IMontageInclusion> mi = i->pUnk)
					m_lastdock |= (mi->ContainsRect(&rc)==S_OK);
        // floating is crashing!
        m_lastdock = true;
		if(!m_lastdock) {
			// adjust rect to float: add the extra that borders and caption need; take grabber size off
			SIZE fext = floatextra();
			rc.bottom += fext.cy;
			rc.right += fext.cx - 9;
		}
		// draw new hint
		drophint(&rc,m_lastdock);
		m_lastrect = rc;
	}
	else {
		if(!m_docked) { // adjust for floatingness
			SIZE fext = floatextra();
			rc.bottom += fext.cy;
			rc.right += fext.cx - 9;
		}
		drophint(&rc,m_docked);
		m_lastdock = m_docked; // sizing can't changed docking
		m_lastrect = rc;
	}
	return 0;
}
LRESULT Toolbar::OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(Toolbar,OnLButtonUp);
	if(!m_tracking) {
		bHandled = false;
		return 0;
	}
	m_tracking = false;
	ReleaseCapture();
	if(!m_sigtrack)
		return 0;
	// erase old hint
	if(!m_firstdraw)
		drophint(&m_lastrect,m_lastdock);
	m_firstdraw = true;

	CComQIPtr<IMCCSite> site = m_spClientSite;

	// act based on last calculated info
	if(m_lastdock) {
		bool wakebuddy = false;
		if(!m_docked) { // dock it!
			// hide floating self
			if(m_buddy) {
				RUN(m_buddy->UseWindow(0));
				wakebuddy = true;
			}
			if(m_hWnd) // IMCCSiteEvents::Shifting may already have triggered
				DestroyWindow();
			// show docking self (but not buddy, yet, because its rect is wrong)
			m_docked = true;
			RUN(fireSiteEvent(site,fireIntend(MCCS_ACTIVE)));
		}
		MCC_POS_INFO mpi;
		mpi.postype = MCC_POS_WINDOWRECT;
		mpi.rc = m_lastrect;
		// ask site's sinks (presumably ToolOwner) to move me
		RUN(fireSiteEvent(site,fireMove(&mpi)));
		if(wakebuddy)
			RUN(m_buddy->put_Intent(MCCS_ACTIVE));
	}
	else {
		if(m_docked) { // float it!
			USES_CONVERSION;
			RUN(fireSiteEvent(site,fireIntend(MCCS_HIDDEN)));
			RUN(m_buddy->put_Intent(MCCS_HIDDEN));
			HWND ring = ringWnd();
			POINT *hack = (POINT *)&m_lastrect;
			::ClientToScreen(ring,hack);
			::ClientToScreen(ring,hack+1);
			Create(0,m_lastrect,OLE2CT(m_caption),
				WS_CLIPSIBLINGS|WS_POPUP|WS_VISIBLE|WS_BORDER|
				WS_THICKFRAME|WS_OVERLAPPED|WS_SYSMENU,
				WS_EX_TOOLWINDOW|WS_EX_WINDOWEDGE);
			m_docked = false;
			RECT rc;
			RUN(inrect(&rc));
			RUN(m_buddy->put_Rect(&rc));
			RUN(m_buddy->UseWindow(*this));
			RUN(m_buddy->put_Intent(MCCS_ACTIVE));
		}
		else {
			HWND ring = ringWnd();
			POINT *hack = (POINT *)&m_lastrect;
			::ClientToScreen(ring,hack);
			::ClientToScreen(ring,hack+1);
			MoveWindow(&m_lastrect);
			if(m_trackside!=HTCAPTION) { // a resize.
				RECT rc;
				RUN(inrect(&rc));
				RUN(m_buddy->put_Rect(&rc));
			}
		}
	}
	return 0;
}

// IMCCToolbar
STDMETHODIMP Toolbar::put_Caption(BSTR strCaption) {
	METHTRACE(Toolbar,put_Caption);
	m_caption = strCaption;
	return S_OK;
}
STDMETHODIMP Toolbar::get_Caption(BSTR* pstrCaption) {
	METHTRACE(Toolbar,get_Caption);
	*pstrCaption = m_caption.Copy();
	return S_OK;
}
STDMETHODIMP Toolbar::get_Buddy(DIMCCSite **ret) {
	METHTRACE(Toolbar,get_Buddy);
	return m_buddy->QueryInterface(riidret(ret));
}
STDMETHODIMP Toolbar::get_AllowMove(VARIANT_BOOL *ret) {
    METHTRACE(Toolbar,get_AllowMove);
    *ret = m_allowmove*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP Toolbar::put_AllowMove(VARIANT_BOOL val) {
    METHTRACE(Toolbar,put_AllowMove);
    m_allowmove = val!=0;
	return S_OK;
}
STDMETHODIMP Toolbar::get_RaiseSelf(VARIANT_BOOL *ret) {
    METHTRACE(Toolbar,get_RaiseSelf);
    *ret = m_raiseself*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP Toolbar::put_RaiseSelf(VARIANT_BOOL val) {
    METHTRACE(Toolbar,put_RaiseSelf);
    m_raiseself = val!=0;
	return S_OK;
}
STDMETHODIMP Toolbar::get_SinkBuddy(VARIANT_BOOL *ret) {
    METHTRACE(Toolbar,get_SinkBuddy);
    *ret = m_sinkbuddy*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP Toolbar::put_SinkBuddy(VARIANT_BOOL val) {
    METHTRACE(Toolbar,put_SinkBuddy);
    m_sinkbuddy = val!=0;
	return S_OK;
}
STDMETHODIMP Toolbar::get_ShowGrips(VARIANT_BOOL *ret) {
    METHTRACE(Toolbar,get_ShowGrips);
    *ret = m_grips*VARIANT_TRUE;
	return S_OK;
}
STDMETHODIMP Toolbar::put_ShowGrips(VARIANT_BOOL val) {
    METHTRACE(Toolbar,put_ShowGrips);
    m_grips = val!=0;
	return S_OK;
}
// INeedOnePointer
STDMETHODIMP Toolbar::TakePointer(IUnknown *buddy,IMoniker *m) {
	METHTRACE(Toolbar,TakePointer);
	if(m_fudge)
		return S_FALSE;
	RUN(buddy->QueryInterface(riidret(&m_buddy)));
	RUN(AtlAdvise(buddy,GetUnknown(),IID_IMCCSiteEvents,&m_fudge));
	RUN(APCAdvise(buddy,GetUnknown(),IID_IMCCSiteOwner,&m_blondie));
	RECT rc;
	if(SUCCEEDED(m_buddy->get_Rect(&rc)))
		RUN(setBuddyExtent(rc));
	RUN(m_buddy->put_ClipMode(MCM_SITE_RECT));
	return S_OK;
}
// IMCCSiteEvents
STDMETHODIMP Toolbar::Moving(LPCRECT winRect) { return S_FALSE; }
STDMETHODIMP Toolbar::Shifting(long oldState, long newState) {
    METHTRACE(Toolbar,Shifting);
	if(newState<MCCS_ACTIVE && m_hWnd)
		DestroyWindow();
	if(newState==MCCS_DELETED) {
		RUN(AtlUnadvise(m_buddy,IID_IMCCSiteEvents,m_fudge));
		RUN(AtlUnadvise(m_buddy,IID_IMCCSiteOwner,m_blondie));
	}
	return S_OK;
}
STDMETHODIMP Toolbar::Layering(IMCCLayer *newLayer) {
    METHTRACE(Toolbar,Layering);
	return S_FALSE;
}
STDMETHODIMP Toolbar::ZChanged() { return S_FALSE; }
STDMETHODIMP Toolbar::Wakeup() { return S_FALSE; }
// IMCCSiteOwner
STDMETHODIMP Toolbar::Move(MCC_POS_INFO *pos) {
    METHTRACE(Toolbar,Move);
	if(pos->postype==MCC_POS_WINDOWRECT) {
		MCC_POS_INFO pos2;
		pos2.postype = MCC_POS_WINDOWRECT;
		RUN(outrect(&pos->rc,&pos2.rc));
		RUN(fireEvent(m_spClientSite,fireOneParam<IMCCSiteOwner,MCC_POS_INFO*>(&IMCCSiteOwner::Move,&pos2)));
		return S_OK;
	}
	return S_FALSE;
}
STDMETHODIMP Toolbar::PutBefore(IMCCSite *other) {
    METHTRACE(Toolbar,PutBefore);
	return S_FALSE;
}
STDMETHODIMP Toolbar::PutBehind(IMCCSite *other) {
    METHTRACE(Toolbar,PutBehind);
	return S_FALSE;
}
STDMETHODIMP Toolbar::Intend(long intent) {
    METHTRACE(Toolbar,Intend);
	return S_FALSE;
}
