/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Toolbar.h : Declaration of the Toolbar

#ifndef __MCCTOOLBAR_H_
#define __MCCTOOLBAR_H_

#include "resource.h"       // main symbols


// how this toolbar floats:
// hides itself in the container (toolbar ring) to destroy the child window
// then creates a new popup window and tells its buddy to UseWindow that.

/////////////////////////////////////////////////////////////////////////////
// Toolbar
class ATL_NO_VTABLE Toolbar :
	public CComObjectRoot,
	public CComCoClass<Toolbar,&CLSID_MCCToolbar>,
	public CComControl<Toolbar>,
	public IDispatchImpl<DIMCCToolbar, &IID_DIMCCToolbar, &LIBID_Montage>,
	public IPersistStreamInitImpl<Toolbar>,
	public IOleControlImpl<Toolbar>,
	public IOleObjectImpl<Toolbar>,
	public IOleInPlaceActiveObjectImpl<Toolbar>,
	public IViewObjectExImpl<Toolbar>,
	public IOleInPlaceObjectWindowlessImpl<Toolbar>,
	public INeedOnePointer,
	public IMCCSiteEvents, // on buddy: to know when to hide floater
	public IMCCSiteOwner // and to deal with tool sizing itself
{
	CComBSTR m_caption;
	CComPtr<IMCCSite> m_buddy;
	DWORD m_fudge,m_blondie;
	POINT m_down;
	RECT m_lastrect;
	bool m_docked,m_tracking,m_sigtrack,m_firstdraw,m_lastdock,m_sinkbuddy,m_grips,m_raiseself,m_allowmove;
	int m_trackside; // bit combination
	HBRUSH m_checkerbrush;
	HBITMAP m_bump; // bitmap for brush

    RECT drect(bool sink=false); // difference between my rect and buddy's
	HRESULT setBuddyExtent(const RECT &rc);
	void drophint(LPCRECT lpcr,bool docking);
	HRESULT inrect(LPRECT ret);
	HRESULT outrect(LPCRECT input,LPRECT output);
	HWND ringWnd();
public:
	Toolbar();
	~Toolbar();

DECLARE_WND_CLASS("Montage Toolbar")

REGGY_RID(IDR_MCCTOOLBAR)

BEGIN_COM_MAP(Toolbar)
	COM_INTERFACE_ENTRY(DIMCCToolbar)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY(INeedOnePointer)
	COM_INTERFACE_ENTRY(IMCCSiteEvents)
	COM_INTERFACE_ENTRY(IMCCSiteOwner)
END_COM_MAP()

BEGIN_PROPERTY_MAP(Toolbar)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(Toolbar)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
	MESSAGE_HANDLER(WM_NCLBUTTONDOWN, OnLButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE,OnMouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP,OnLButtonUp)
	CHAIN_MSG_MAP(CComControl<Toolbar>)
END_MSG_MAP()

	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);

// overriden from CComControl, which does not specify WS_CLIPSIBLINGS
	virtual HWND CreateControlWindow(HWND hWndParent, RECT& rcPos) {
		USES_CONVERSION;
		return Create(hWndParent, rcPos, OLE2CT(m_caption), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS);
	}

// IViewObjectEx
	STDMETHODIMP GetViewStatus(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE;
		return S_OK;
	}
// IViewObjectImpl
	HRESULT OnDraw(ATL_DRAWINFO& di);
// IOleObject(Impl)
	STDMETHODIMP SetClientSite(IOleClientSite *s);

// IOleInPlaceObject
	STDMETHODIMP SetObjectRects(LPCRECT pos,LPCRECT);
// IMCCToolbar
	STDMETHODIMP put_Caption(BSTR strCaption);
	STDMETHODIMP get_Caption(BSTR* pstrCaption);
	STDMETHODIMP get_Buddy(DIMCCSite **ret);
    STDMETHODIMP get_AllowMove(VARIANT_BOOL *ret);
    STDMETHODIMP put_AllowMove(VARIANT_BOOL val);
    STDMETHODIMP get_RaiseSelf(VARIANT_BOOL *ret);
    STDMETHODIMP put_RaiseSelf(VARIANT_BOOL val);
    STDMETHODIMP get_SinkBuddy(VARIANT_BOOL *ret);
    STDMETHODIMP put_SinkBuddy(VARIANT_BOOL val);
    STDMETHODIMP get_ShowGrips(VARIANT_BOOL *ret);
    STDMETHODIMP put_ShowGrips(VARIANT_BOOL val);
// INeedOnePointer
	STDMETHODIMP TakePointer(IUnknown *buddy,IMoniker *m);
// IMCCSiteEvents
	STDMETHODIMP Moving(LPCRECT winRect);
	STDMETHODIMP Shifting(long oldState, long newState);
	STDMETHODIMP Layering(IMCCLayer *newLayer);
	STDMETHODIMP ZChanged();
	STDMETHODIMP Wakeup();
// IMCCSiteOwner
    STDMETHODIMP Move(MCC_POS_INFO *pos);
    STDMETHODIMP PutBefore(IMCCSite *other);
    STDMETHODIMP PutBehind(IMCCSite *other);
	STDMETHODIMP Intend(long intent);
};

#endif //__MCCTOOLBAR_H_
