/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ToolbarOwner.h"

ToolbarOwner::ToolbarOwner() {
	CONSTRUCTORTRACE(ToolbarOwner);
}
ToolbarOwner::~ToolbarOwner() {
	DESTRUCTORTRACE(ToolbarOwner);
}
// IObjectWithSite
STDMETHODIMP ToolbarOwner::SetSite(IUnknown *site) {
	METHTRACE(ToolbarOwner,SetSite);
	m_site = site;
	if(m_site) {
		DWORD sugar;
		RUN(APCAdvise(m_site,(IMCCSiteOwner*)this,IID_IMCCSiteOwner,&sugar));
		RUN(APCAdvise(m_site,(IMCCSiteEvents*)this,IID_IMCCSiteEvents,&sugar));
		// must be called AFTER TakePointer, to follow Montage's annoying
		// "no OLEMISC_SETCLIENTSITEFIRST" policy
		CComQIPtr<IMCCSite> qs = m_site;
		if(!qs) ERR(E_UNEXPECTED);
		RECT rc;
		RUN(qs->get_Rect(&rc));
		RUN(AtlAdvise(m_proxy,(IMoveRect*)this,IID_IMoveRect,&m_doubleStuff));
		return m_proxy->Move(&rc);
	}
	return S_OK;
}
STDMETHODIMP ToolbarOwner::GetSite(REFIID riid, void **ret) {
	METHTRACE(ToolbarOwner,GetSite);
	return m_site->QueryInterface(riid,ret);
}
// INeedOnePointer
STDMETHODIMP ToolbarOwner::TakePointer(IUnknown *toolEngine,IMoniker *m) {
	METHTRACE(ToolbarOwner,TakePointer);
	CComPtr<IRectEngine> engine;
	RUN(toolEngine->QueryInterface(riidret(&engine)));
	RUN(engine->new_Delegate(&m_proxy));
	return S_OK;
}
// IMCCSiteOwner
STDMETHODIMP ToolbarOwner::Move(MCC_POS_INFO *pos) {
	METHTRACE(ToolbarOwner,Move);
	if(!m_proxy)
		return S_FALSE; // don't know anything yet.
	if(pos->postype!=MCC_POS_WINDOWRECT)
		ERR(E_INVALIDARG);
	return m_proxy->Move(&pos->rc);
}
STDMETHODIMP ToolbarOwner::PutBefore(IMCCSite *other) {
	METHTRACE(ToolbarOwner,PutBefore);
	return S_FALSE; // default handling
}
STDMETHODIMP ToolbarOwner::PutBehind(IMCCSite *other) {
	METHTRACE(ToolbarOwner,PutBehind);
	return S_FALSE; // default handling
}
STDMETHODIMP ToolbarOwner::Intend(long val) {
	METHTRACE(ToolbarOwner,Intend);
	if(!m_proxy)
		return S_FALSE; // don't know anything yet.
	CComQIPtr<IMCCSite> s(m_site);
	if(!s) ERR(E_UNEXPECTED);
	long old;
	RUN(s->get_Intent(&old));
	if(old<MCCS_SHOWN && val > MCCS_HIDDEN) {
		RECT rc;
		RUN(s->get_Rect(&rc));
		RUN(m_proxy->Move(&rc));
	}
	else if(old > MCCS_HIDDEN && val < MCCS_SHOWN)
		RUN(m_proxy->Move(0));
	return S_FALSE; // default handling
}
// IMCCSiteEvents
STDMETHODIMP ToolbarOwner::Moving(LPCRECT winRect) {
	METHTRACE(ToolbarOwner,Moving);
	return S_FALSE;
}
STDMETHODIMP ToolbarOwner::Shifting(long oldState, long newState) {
	METHTRACE(ToolbarOwner,Shifting);
	if(newState==MCCS_DELETED)
		RUN(AtlUnadvise(m_proxy,IID_IMoveRect,m_doubleStuff));
	return S_OK;
}
STDMETHODIMP ToolbarOwner::Layering(IMCCLayer *newLayer) {
	return S_FALSE;
}
STDMETHODIMP ToolbarOwner::ZChanged() {
	return S_FALSE;
}
STDMETHODIMP ToolbarOwner::Wakeup() {
	METHTRACE(ToolbarOwner,Wakeup);
	return S_FALSE;
}
// IMoveRect
STDMETHODIMP ToolbarOwner::XMoveRect::Move(LPCRECT lpcr) {
	METHTRACE(ToolbarOwner,XMoveRect);
	CComQIPtr<IMCCSite> s(This()->m_site);
	if(!s) ERR(E_UNEXPECTED);
	if(!lpcr)
		return s->put_Intent(MCCS_HIDDEN);
	else
		return s->put_Rect(lpcr);
}
// IPersist
STDMETHODIMP ToolbarOwner::GetClassID(CLSID *pClassID) {
	METHTRACE(ToolbarOwner,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}

