/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "TransformManager.h"

HRESULT getST(IMontageContainer*mc,IMCCScrollingTransform **ret) {
    FUNTRACE(getST);
	CComPtr<IMCCTransform> t;
	SRUN(mc->get_Transform(&t));
	return t.QueryInterface(ret);
}

TransformManager::TransformManager() : m_showScroll(false),m_useExtra(true),m_locks(0),
	m_tahoe(0), m_amSettingTransform(false),m_amSettingBars(false) {
	CONSTRUCTORTRACE(TransformManager);
}
TransformManager::~TransformManager() {
	DESTRUCTORTRACE(TransformManager);
}
HRESULT TransformManager::getAreas(RECT &retNonExtra,RECT &retNonReserved) {
	CComPtr<IMontageContainer> MC = mc();
	SIZE size;
	RUN(MC->get_Size(&size));
	if(size.cx==0||size.cy==0)
		ERR(E_PENDING);
	RECT client = rect(size),
		extra,reserved;
	RUN(MC->get_ExtraArea(&extra));
	RUN(MC->get_ReservedArea(&reserved));

	retNonExtra = client - extra,
	retNonReserved = client - reserved;
	return S_OK;
}
#define sb2long(x) x
#define long2sb(x) x

HRESULT TransformManager::figureMinMaxPosThumb(int &minX,int &maxX,int &posX,int &thumbX,
											   int &minY,int &maxY,int &posY,int &thumbY) {
    METHTRACE(TransformManager,figureMinMaxPosThumb);
	CComPtr<IMCCScrollingTransform> st;
	if(FAILED(getST(mc(),&st))) {
		// it's not scrolling.
		minX = posX = minY = posY = 0;
		maxX = maxY = thumbX = thumbY = 1;
		ERR(E_FAIL);
	}
	// multiply extent by ratio of nonReserved/nonExtra areas to get
	// actual extent of visible area
	SIZEL ext;
	RUN(mc()->GetZoomedExtent(&ext));
	RECT nonExtra,nonReserved;
	RUN(getAreas(nonExtra,nonReserved));
	(ext.cx*=wid(nonReserved))/=wid(nonExtra);
	(ext.cy*=hei(nonReserved))/=hei(nonExtra);
	RECTL bounds = {0,0,0,0};
	LOOK(st->get_Bounds(&bounds));
	POINTL ofs;
	RUN(st->get_Offset(&ofs));
	minX = long2sb(bounds.left);
	maxX = long2sb(bounds.right);
	if(minX==maxX) maxX = minX+10;
	posX = long2sb(ofs.x);
	thumbX = long2sb(ext.cx);
	if(thumbX>maxX-minX)
		thumbX = maxX-minX+2; // min and max are inclusive
	if(posX<minX)
		posX = minX;
	int maxestX = maxX-thumbX+1;
	if(posX>maxestX)
		posX = maxestX;
	// special case: windows bars demand max > min
	minY = 0;
	maxY = bounds.top-bounds.bottom;
	if(minY==maxY) maxY = minY+10;
	posY = long2sb(bounds.top-ofs.y);
	thumbY = long2sb(ext.cy);
	if(thumbY>maxY-minY)
		thumbY = maxY-minY+1;
	if(posY<minY)
		posY = minY;
	int maxestY = maxY-thumbY+1;
	if(posY>maxestY)
		posY = maxestY;
	return S_OK;
}
HRESULT TransformManager::showBars(bool horz, bool vert) {
    METHTRACE(TransformManager,showBars);
	if(!m_vscroll)
		return S_FALSE;
    CComPtr<IMontageContainer> MC = mc();
	int cx = GetSystemMetrics(SM_CXVSCROLL),
		cy = GetSystemMetrics(SM_CYHSCROLL);
	// ask for extra room around the embedding for the scrollbars
	// ... but since we ask for reserved area anyway, it doesn't matter to us.
    RECT extra = {0,0,0,0};
	if(m_useExtra) {
		RUN(MC->get_ExtraArea(&extra));
		extra.right = vert?cx:0;
		extra.bottom = horz?cy:0;
	}
	RECT reserved;
	RUN(MC->get_ReservedArea(&reserved));
	reserved.right = vert?cx:0;
	reserved.bottom = horz?cy:0;
	RUN(MC->SetAreas(extra,reserved));
	SIZE s;
	RUN(MC->get_Size(&s));
	if(s.cx<cx)
		vert = false;
	if(s.cy<cy)
		horz = false;
	RECT rcV = rect(s),
		rcH = rcV;
	if(vert) {
		rcV.left = rcV.right - cx;
		if(horz)
			rcV.bottom -= cy;
		RUN(m_vscroll->put_Rect(&rcV));
		RUN(m_vscroll->put_Intent(MCCS_ACTIVE));
	}
	else
		RUN(m_vscroll->put_Intent(MCCS_HIDDEN));
	if(horz) {
		rcH.top = rcH.bottom - cy;
		if(vert)
			rcH.right -= cx;
		RUN(m_hscroll->put_Rect(&rcH));
		RUN(m_hscroll->put_Intent(MCCS_ACTIVE));
	}
	else
		RUN(m_hscroll->put_Intent(MCCS_HIDDEN));
	return S_OK;
}
extern CComBSTR g_OleInPlaceBorderWidth;
HRESULT TransformManager::whichBars(bool &horz,bool &vert) {
	int minX,maxX,posX,thumbX,minY,maxY,posY,thumbY;
	RUNRET(figureMinMaxPosThumb(minX,maxX,posX,thumbX,minY,maxY,posY,thumbY));
	horz = thumbX<maxX-minX;
	vert = thumbY<maxY-minY;
	return S_OK;
}
HRESULT TransformManager::setAll() {
    METHTRACE(TransformManager,setAll);
	if(m_amSettingTransform||m_amSettingBars)
		return S_FALSE;

	// check to see if Montage is still booting; if so, don't try now
	RECT nonExtra,nonReserved;
	if(E_PENDING==LOOK(getAreas(nonExtra,nonReserved)))
		return S_FALSE;

	// figure out transform parameters based on montage size & extent,
	// and whether scrollbars
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	CComQIPtr<IMCCScrollingTransform> st = t;
	m_amSettingBars = true;
	bool noBars;
	if(m_showScroll && st) {
		// craziness: must iterate to see if adding or removing scrollbars
		// itself means that scrollbars should be added or removed!
		bool horz=false,vert=false,lastHorz,lastVert;
		RUN(whichBars(horz,vert));
        int oops = 4;
		do {
			lastHorz = horz;
			lastVert = vert;

			RUN(showBars(horz,vert));
			RUN(whichBars(horz,vert));
		}
		while(--oops && (lastHorz!=horz||lastVert!=vert));
		noBars = !horz&&!vert;
	}
	else {
		RUN(showBars(false,false));
		noBars = true;
	}
	RUN(getAreas(nonExtra,nonReserved));
	RUN_NIOK(t->put_TransformedArea(&nonExtra));
	SIZEL ext;
	RUN(mc()->GetZoomedExtent(&ext));
	RUN_NIOK(t->put_Extent(&ext));
	if(noBars) {
		m_amSettingBars = false;
		return S_OK;
	}
	if(!m_vscroll)
		return S_FALSE;
	CComPtr<DIMCCScrollbar> vbar,hbar;
	if(m_vscroll->GetControl(riidret(&vbar))==MONTAGE_E_DELETED) {// container's shut down.
		m_amSettingBars = false;
		return S_FALSE;
	}
	RUN(m_hscroll->GetControl(riidret(&hbar)));
	int minX,maxX,posX,thumbX,minY,maxY,posY,thumbY;
	RUN(figureMinMaxPosThumb(minX,maxX,posX,thumbX,minY,maxY,posY,thumbY));
	RUN(hbar->SetRange(minX,maxX));
	RUN(hbar->put_Pos(posX));
	RUN(hbar->put_Thumb(thumbX));
	RUN(vbar->SetRange(minY,maxY));
	RUN(vbar->put_Pos(posY));
	RUN(vbar->put_Thumb(thumbY));
	m_amSettingBars = false;
	return S_OK;
}
HRESULT TransformManager::placeInd() {
    METHTRACE(TransformManager,placeInd);
	if(!m_ind)
		return S_FALSE;
	SIZE s;
	RUN(mc()->get_Size(&s));

	CComPtr<IMontageContainer> MC = mc();
	SIZE size;
	RUN(MC->get_Size(&size));
	RECT client = rect(size),
		extra;
	RUN(MC->get_ExtraArea(&extra));

	RECT rc = client - extra;
	rc.left = (rc.left+2*rc.right)/3;
	rc.bottom = (rc.top+rc.bottom)/2;
	RUN(m_indSite->put_Rect(&rc));
	return S_OK;
}
HRESULT TransformManager::updateInd() {
    METHTRACE(TransformManager,updateInd);
	if(!m_ind)
		return S_FALSE;
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	SIZEL ext;
	RUN(t->get_Extent(&ext));
	char buf[200];
	if(CComQIPtr<IMCCScrollingTransform> st = t) {
		int minX,maxX,posX,thumbX,minY,maxY,posY,thumbY;
		RUN(figureMinMaxPosThumb(minX,maxX,posX,thumbX,minY,maxY,posY,thumbY));
		RECTL bounds = {0,0,0,0};
		LOOK(st->get_Bounds(&bounds));
		POINTL ofs;
		RUN(st->get_Offset(&ofs));
		sprintf(buf,"extent: %d,%d\nbounds: %d,%d,\n  %d,%d\noffset: %d,%d\n"
			"V: <%d,%d> %d:%d\nH: <%d,%d> %d:%d",ext.cx,ext.cy,
			bounds.left,bounds.top,bounds.right,bounds.bottom,ofs.x,ofs.y,
			minY,maxY,posY,thumbY,
			minX,maxX,posX,thumbX);
	}
	else
		sprintf(buf,"extent: %d,%d",ext.cx,ext.cy);

	RUN(m_ind->put_Text(CComBSTR(buf)));
	return S_OK;
}
// INeedOnePointer
STDMETHODIMP TransformManager::TakePointer(IUnknown *p,IMoniker *m) {
    METHTRACE(TransformManager,TakePointer);
	CComPtr<DIMontageContainer> mc;
	RUNRET(p->QueryInterface(&mc));
	return Initialize(mc);
}
// IMontageContainerEvents
STDMETHODIMP TransformManager::Shutdown() {
    METHTRACE(TransformManager,Shutdown);
	return S_OK;
}
STDMETHODIMP TransformManager::Resize(SIZE s) {
    METHTRACE(TransformManager,Resize);
	RUN(setAll());
	RUN(placeInd());
	RUN(updateInd());
	return S_OK;
}
STDMETHODIMP TransformManager::NewExtent(SIZEL ext) {
    METHTRACE(TransformManager,NewExtent);
	RUN(setAll());
	return S_OK;
}
STDMETHODIMP TransformManager::Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(TransformManager,Viewing);
	return S_FALSE;
}
STDMETHODIMP TransformManager::GroupOp(BOOL whether) {
    METHTRACE(TransformManager,GroupOp);
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	if(CComQIPtr<IMCCScrollingTransform> st = t) {
		if(whether) {
			if(!m_locks++)
				RUN(st->put_DoEvents(VARIANT_FALSE));
		}
		else if(!--m_locks)
			RUN(st->put_DoEvents(VARIANT_TRUE));
	}
	return S_OK;
}
STDMETHODIMP TransformManager::NewTransform(IMCCTransform *old) {
    METHTRACE(TransformManager,NewTransform);
    CComPtr<IMontageContainer> MC = mc();
	CComPtr<IMCCScrollingTransform> st;
	if(m_tahoe && old && SUCCEEDED(old->QueryInterface(riidret(&st))))
		RUN(AtlUnadvise(st,IID_IMCCScrollingTransformEvents,m_tahoe));
	st.Release();
	m_tahoe = 0;
	if(SUCCEEDED(getST(MC,&st))) {
		RUN(AtlAdvise(st,GetUnknown(),IID_IMCCScrollingTransformEvents,&m_tahoe));
		RUN(st->put_DoEvents(m_locks?VARIANT_FALSE:VARIANT_TRUE));
	}
	RUN(setAll());
	return S_OK;
}
STDMETHODIMP TransformManager::ScaleChanged() {
    METHTRACE(TransformManager,ScaleChanged);
    return S_FALSE;
}
STDMETHODIMP TransformManager::SiteDeleted(IMCCSite*) {
    METHTRACE(TransformManager,SiteDeleted);
	return S_FALSE;
}
STDMETHODIMP TransformManager::LayerDeleted(IMCCLayer*) {
    METHTRACE(TransformManager,LayerDeleted);
	return S_FALSE;
}
STDMETHODIMP TransformManager::Shifting(long oldState,long newState) {
    METHTRACE(TransformManager,Shifting);
	return S_FALSE;
}
STDMETHODIMP TransformManager::ShowArea(LPCRECTL r) {
	CComPtr<IMCCTransform> t;
	RUN(mc()->get_Transform(&t));
	if(CComQIPtr<IMCCScrollingTransform> st = t) {
		SIZEL ext;
		RUN(t->get_Extent(&ext));
		POINTL ofs;
		RUN(st->get_Offset(&ofs));
		RECTL curr = {ofs.x,ofs.y,ofs.x+ext.cx,ofs.y-ext.cy};
		int isect = intersectType(*r,curr);
		if(isect==int_inside)
			return S_FALSE;
		POINTL newOffset;
		if(isect==int_none) {
			// could align any of the four corners.. move offset the shortest distance
			POINTL poss[4] = {{r->left,r->top},
				{r->right-ext.cx,r->top},
				{r->left,r->bottom+ext.cy},
				{r->right-ext.cx,r->bottom+ext.cy}};
			int low = 0, lowd = dsqr(ofs,poss[0]);
			for(int i = 1; i<4; ++i) {
				int d = dsqr(ofs,poss[i]);
				if(d<lowd) {
					low = i;
					lowd = d;
				}
			}
			newOffset = poss[low];
		}
		else {
			switch(isect & int_x) {
				case int_rr:
					// there's more to show on right
					newOffset.x = min(r->left,r->right-ext.cx);
					break;
				case int_ll:
					// .. on left
					newOffset.x = max(r->left,r->right-ext.cx);
					break;
				case int_rl: // inside
				case int_lr: // outside
					break;
			}
			switch(isect & int_y) {
				case int_tt: // more above
					newOffset.y = min(r->top,r->bottom+ext.cy);
					break;
				case int_bb: // more below
					newOffset.y = max(r->top,r->bottom+ext.cy);
					break;
				case int_tb: // inside
				case int_bt: // outside
					break;
			}
		}
		RUN(st->put_Offset(&newOffset));
	}
	return S_OK;
}
// IMCCScrollingTransformEvents
STDMETHODIMP TransformManager::BoundsChanged() {
    METHTRACE(TransformManager,BoundsChanged);
	RUN(updateInd());
	RUN(setAll());
	return S_OK;
}
STDMETHODIMP TransformManager::OffsetChanged() {
    METHTRACE(TransformManager,OffsetChanged);
	RUN(updateInd());
	RUN(setAll());
	return S_OK;
}
// DDFlatSBEvents
STDMETHODIMP TransformManager::OnVertPosChange() {
    METHTRACE(TransformManager,OnVertPosChange);
	if(m_amSettingBars)
		return S_OK;
	CComPtr<IMCCScrollingTransform> st;
	RUN(getST(mc(),&st));
	CComPtr<DIMCCScrollbar> vbar;
	RUN(m_vscroll->GetControl(riidret(&vbar)));
	int value;
	RUN(vbar->get_Pos(&value));
	// translate back from special flipped coords
	RECTL bounds = {0,0,0,0};
	LOOK(st->get_Bounds(&bounds));
	value = bounds.top - value;
	POINTL offset;
	RUN(st->get_Offset(&offset));
	offset.y = sb2long(value);

	m_amSettingTransform = true;
	RUN(st->put_Offset(&offset));
	m_amSettingTransform = false;
	return S_OK;
}
STDMETHODIMP TransformManager::OnVertScroll() {
    METHTRACE(TransformManager,OnVertScroll);
	if(m_amSettingBars)
		return S_OK;
	return S_OK;
}
STDMETHODIMP TransformManager::OnHorzPosChange() {
    METHTRACE(TransformManager,OnHorzPosChange);
	if(m_amSettingBars)
		return S_OK;
	CComPtr<DIMCCScrollbar> hbar;
	RUN(m_hscroll->GetControl(riidret(&hbar)));
	int value;
	RUN(hbar->get_Pos(&value));
	CComPtr<IMCCScrollingTransform> st;
	RUN(getST(mc(),&st));
	POINTL offset;
	RUN(st->get_Offset(&offset));
	offset.x = sb2long(value);
	m_amSettingTransform = true;
	RUN(st->put_Offset(&offset));
	m_amSettingTransform = false;
	return S_OK;
}
STDMETHODIMP TransformManager::OnHorzScroll() {
    METHTRACE(TransformManager,OnHorzScroll);
	if(m_amSettingBars)
		return S_OK;
	return S_OK;
}
// DIMCCTransformManager::
STDMETHODIMP TransformManager::Initialize(DIMontageContainer *mc) {
    METHTRACE(TransformManager,Initialize);
	if(m_mref)
		ERR(E_FAIL);
    RUN(CComQIPtr<IMontageContainer>(mc)->get_RefObject(&m_mref));
	RUN(NewTransform(0));
	DWORD gingerSnap; // oops, dropping a cookie or two
	RUN(AtlAdvise(mc,GetUnknown(),IID_IMontageContainerEvents,&gingerSnap));
	if(m_showScroll) { // fake setting ShowScrollbars for first time
		m_showScroll = false;
		RUN(put_ShowScrollbars(VARIANT_TRUE));
	}
	RUN(BoundsChanged());
#ifdef COORD_INDICATOR
	// (debug) display transform coords
	RUN(mc->new_Site(&m_indSite));
	RUN(CREATE(m_ind,CLSID_Indicator));
	RUN(m_indSite->put_Control(m_ind));
	RUN(m_indSite->put_Layer(l));
	RUN(placeInd());
	RUN(m_indSite->put_Intent(MCCS_SHOWN));
	RUN(updateInd());
#endif
	return S_OK;
}
STDMETHODIMP TransformManager::put_ShowScrollbars(VARIANT_BOOL val) {
    METHTRACE(TransformManager,put_ShowScrollbars);
	if(!m_mref)
		ERR(E_PENDING);
	bool newVal = val != 0;
	if(newVal!=m_showScroll) {
		if(newVal) { // create 'em
            CComPtr<IMontageContainer> MC = mc();
			RUN(MC->new_Site(&m_vscroll));
			RUN(MC->new_Site(&m_hscroll));
			// put in front of everything
			CComPtr<IMCCLayer> l;
			RUN(MC->new_Layer(&l));
			RUN(l->PutBehind(0));
			RUN(m_vscroll->put_Layer(l));
			RUN(m_hscroll->put_Layer(l));
			// let them activate in reserved area
			RUN(m_vscroll->put_ClipMode(MCM_FULL));
			RUN(m_hscroll->put_ClipMode(MCM_FULL));
			CComPtr<DIMCCScrollbar> vbar,hbar; // not flat
			RUN(CREATE(vbar,CLSID_MCCScrollbar));
			RUN(CREATE(hbar,CLSID_MCCScrollbar));
			RUN(hbar->put_Vert(VARIANT_FALSE));
			RUN(m_vscroll->put_Control(vbar));
			RUN(m_hscroll->put_Control(hbar));
			m_vsink = new CComObject<ScrollSink>;
			m_hsink = new CComObject<ScrollSink>;
			m_vsink->parent = m_hsink->parent = this;
			m_vsink->vert = !(m_hsink->vert = false);
			DWORD chanukah;
			RUN(AtlAdvise(vbar,m_vsink,IID_IMCCScrollbarEvents,&chanukah));
			RUN(AtlAdvise(hbar,m_hsink,IID_IMCCScrollbarEvents,&chanukah));
		}
		else { // delete 'em
			RUN(m_vscroll->put_Intent(MCCS_DELETED));
			RUN(m_hscroll->put_Intent(MCCS_DELETED));
			m_vscroll.Release();
			m_hscroll.Release();
			m_vsink.Release();
			m_hsink.Release();
		}
		RUN(setAll());
		m_showScroll = newVal;
	}
	return S_OK;
}
STDMETHODIMP TransformManager::get_ShowScrollbars(VARIANT_BOOL *ret) {
    METHTRACE(TransformManager,get_ShowScrollbars);
	*ret = m_showScroll?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
STDMETHODIMP TransformManager::put_UseExtraArea(VARIANT_BOOL val) {
    METHTRACE(TransformManager,put_UseExtraArea);
	if(!m_mref)
		ERR(E_PENDING);
	bool newVal = val != 0;
	if(newVal!=m_useExtra) {
		bool restore = false;
		if(m_showScroll) {
			RUN(put_ShowScrollbars(false));
			restore = true;
		}
		m_useExtra = newVal;
		if(restore)
			RUN(put_ShowScrollbars(true));
	}
	return S_OK;
}
STDMETHODIMP TransformManager::get_UseExtraArea(VARIANT_BOOL *ret) {
    METHTRACE(TransformManager,get_UseExtraArea);
	*ret = m_useExtra?VARIANT_TRUE:VARIANT_FALSE;
	return S_OK;
}
//  IPersistStream
STDMETHODIMP TransformManager::GetClassID(CLSID *pClassID) {
    METHTRACE(TransformManager,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP TransformManager::IsDirty(void) {
    METHTRACE(TransformManager,IsDirty);
	return S_OK;
}
STDMETHODIMP TransformManager::Load(IStream *pStm) {
    METHTRACE(TransformManager,Load);
	RUN(pStm->Read(&m_showScroll,sizeof(bool),0));
	RUN(pStm->Read(&m_useExtra,sizeof(bool),0));
	return S_OK;
}
STDMETHODIMP TransformManager::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(TransformManager,Save);
	RUN(pStm->Write(&m_showScroll,sizeof(bool),0));
	RUN(pStm->Write(&m_useExtra,sizeof(bool),0));
	return S_OK;
}
STDMETHODIMP TransformManager::GetSizeMax(ULARGE_INTEGER *pcbSize) {
    METHTRACE(TransformManager,GetSizeMax);
	ERR(E_NOTIMPL);
}
