/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


const int vertSink=0,horzSink=1;
struct TransformManager :
	CComObjectRoot,
	CComCoClass<TransformManager,&CLSID_MCCTransformManager>,
	IDispatchImpl<DIMCCTransformManager>,
	IMontageContainerEvents,
	IMCCScrollingTransformEvents,
	INeedOnePointer,
	IPersistStream
{
    CComPtr<DIMontageContainerRef> m_mref;
	bool m_showScroll, m_useExtra;
	// hack! avoid recursive event bugs - we care about events except when we've caused them
	bool m_amSettingTransform, m_amSettingBars;
	long m_locks;
	DWORD m_tahoe;
	CComPtr<IMCCSite> m_vscroll,m_hscroll;
	HRESULT getAreas(RECT &retNonExtra,RECT &retNonReserved);
	HRESULT figureMinMaxPosThumb(int &minX,int &maxX,int &poxX,int &thumbX,int &minY,int &maxY,int &posY,int &thumbY);
	HRESULT showBars(bool horz, bool vert);
	HRESULT whichBars(bool &horz,bool &vert);
	HRESULT setAll();

    CComPtr<IMontageContainer> mc() {
        METHTRACE(TransformManager,mc);
        CComPtr<IUnknown> umc;
        RUN(m_mref->get_Montage(&umc));
        return CComQIPtr<IMontageContainer>(umc);
    }

	// debug
	CComPtr<IIndicator> m_ind;
	CComPtr<IMCCSite> m_indSite;
	HRESULT placeInd();
	HRESULT updateInd();

	REGISTER_GENERIC(L"TransformManager")
	TransformManager();
	~TransformManager();
BEGIN_COM_MAP(TransformManager)
	COM_INTERFACE_ENTRY(DIMCCTransformManager)
	COM_INTERFACE_ENTRY(IMontageContainerEvents)
	COM_INTERFACE_ENTRY(IMCCScrollingTransformEvents)
	COM_INTERFACE_ENTRY(INeedOnePointer)
	COM_INTERFACE_ENTRY(IPersistStream)
END_COM_MAP()

	struct ScrollSink : IMCCScrollbarEvents, CComObjectRoot {
		TransformManager *parent;
		bool vert;
	BEGIN_COM_MAP(ScrollSink)
		COM_INTERFACE_ENTRY(IMCCScrollbarEvents)
	END_COM_MAP()
		STDMETHODIMP PosChanged() {
            METHTRACE(ScrollSink,PosChanged);
			if(vert)
				RUN(parent->OnVertPosChange());
			else
				RUN(parent->OnHorzPosChange());
			return S_OK;
		}
	};
	CComPtr<ScrollSink> m_hsink,m_vsink;
	STDMETHODIMP OnVertPosChange();
	STDMETHODIMP OnVertScroll();
	STDMETHODIMP OnHorzPosChange();
	STDMETHODIMP OnHorzScroll();
// INeedOnePointer
	STDMETHODIMP TakePointer(IUnknown *p,IMoniker *m);
// IMontageContainerEvents
	STDMETHODIMP Shutdown();
	STDMETHODIMP Resize(SIZE s);
	STDMETHODIMP NewExtent(SIZEL);
	STDMETHODIMP Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv);
	STDMETHODIMP NewTransform(IMCCTransform*);
	STDMETHODIMP ScaleChanged();
	STDMETHODIMP GroupOp(BOOL);
	STDMETHODIMP SiteDeleted(IMCCSite*);
	STDMETHODIMP LayerDeleted(IMCCLayer*);
	STDMETHODIMP Shifting(long oldState,long newState);
	STDMETHODIMP ShowArea(LPCRECTL r);
    STDMETHODIMP FocusChanged() { return S_FALSE; }
// IMCCScrollingTransformEvents
	STDMETHODIMP BoundsChanged();
	STDMETHODIMP OffsetChanged();
// IMCCScrollbar
// DIMCCTransformManager
	STDMETHODIMP Initialize(DIMontageContainer *mc);
	STDMETHODIMP put_ShowScrollbars(VARIANT_BOOL val);
	STDMETHODIMP get_ShowScrollbars(VARIANT_BOOL *ret);
	STDMETHODIMP put_UseExtraArea(VARIANT_BOOL val);
	STDMETHODIMP get_UseExtraArea(VARIANT_BOOL *ret);
// IPersistStream
	STDMETHODIMP GetClassID(CLSID *pClassID);
	STDMETHODIMP IsDirty(void);
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
	STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
};
