/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"

HRESULT Container::createmenus() {
	METHTRACE(Container,placemenus);
	if(!m_pFrame)
		ERR(E_PENDING);
	_ASSERTE(!m_menu && !m_holeMenu);
	for(int i = 0;i < 6;i++)
		m_widths.width[i] = 0;
	HMENU hmenu = CreateMenu();
	RUN(m_pFrame->InsertMenus(hmenu,&m_widths));
	_ASSERTE(m_hWnd);
	HOLEMENU holemenu = OleCreateMenuDescriptor(hmenu,&m_widths);
	m_menu = hmenu;
	m_holeMenu = holemenu;
	return S_OK;
}
HRESULT Container::killmenus() {
	METHTRACE(Container,killmenus);
	if(m_menu) {
		if(m_pFrame)
			RUN(m_pFrame->RemoveMenus(m_menu));
		DestroyMenu(m_menu);
		m_menu = 0;
	}
	if(m_holeMenu) {
		OleDestroyMenuDescriptor(m_holeMenu);
		m_holeMenu = 0;
	}
	return S_OK;
}
HRESULT Container::installUI() {
	METHTRACE(Container,installUI);
	if(m_haveUI)
		return S_FALSE;
	if(m_pFrame)
		RUN(m_pFrame->SetActiveObject(this,NULL));
	if(m_pDoc)
		RUN(m_pDoc->SetActiveObject(this,NULL));
	if(m_toolpanel && m_pFrame)
		RUN(m_toolpanel->Start(m_pFrame));
	m_haveUI = true;
	RUN(m_pFrame->SetMenu(m_menu,m_holeMenu,m_hWnd));
	return S_OK;
}
HRESULT Container::removeUI() {
	METHTRACE(Container,removeUI);
	// for example, if we skipped UI activation in uiactivate() because we knew a child
	// was just going to grab it, and now the child is calling OnUIActivate
	// to say it's going to steal UI
	if(!m_haveUI)
		return S_FALSE;
	if(m_pFrame) {
		RUN(m_pFrame->SetMenu(0,0,0));
		RUN(m_pFrame->SetActiveObject(0,0));
	}
	if(m_pDoc)
		RUN(m_pDoc->SetActiveObject(0,0));
	if(m_toolpanel)
		RUN(m_toolpanel->Stop());
	m_haveUI = false;
	return S_OK;
}
HRESULT Container::uiactivate() {
	IMETHTRACE(Container,IOleInPlaceActiveObject,uiactivate);
	_ASSERTE(m_state==MCCS_ACTIVE); // internal error
	RUNRET(m_pInPlaceSite->OnUIActivate());
	m_state = MCCS_UIACTIVE;
	if(m_pFrame) {
		// trick: if some child is uiactive, don't install montage UI...  it's inefficient plus
		// it's causing bugs that I think aren't my fault - something about installing
		// and then removing UI all in one call?  You end up with BOTH - very ugly.
		bool childHasUI(false);
		for(layerlist::iterator il = m_layers.begin();il!=m_layers.end();++il)
			for(sitelist::iterator is = (*il)->m_sites.begin();is!=(*il)->m_sites.end();++is)
				if((*is)->m_intent==MCCS_UIACTIVE)
					childHasUI = true;
		RUN(createmenus());
		if(!childHasUI)
			RUN(installUI());
	}
	SetFocus(m_hWnd);
	return pullsites();
}
HRESULT Container::uideactivate() {
	IMETHTRACE(Container,IOleInPlaceActiveObject,uideactivate);
	_ASSERTE(m_state==MCCS_UIACTIVE); // internal error
	m_state = MCCS_ACTIVE;
	if(m_haveUI)
		RUN(removeUI());
	RUN(pullsites()); // (includes UI deactivating child)
	RUN(killmenus());
	LOOK(m_pInPlaceSite->OnUIDeactivate(false)); // We might be in UIDeactivate, which Corel Pres. doesn't like
	return S_OK;
}

