/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ViewMode.cpp : Implementation of ViewMode
#include "stdafx.h"
#include "Montage.h"
#include "ViewMode.h"
#include "CEMPublic.h" // stuff shared with edit mode

ViewMode::ViewMode() : m_macadamia(0) {
	CONSTRUCTORTRACE(ViewMode);
}
ViewMode::~ViewMode() {
	DESTRUCTORTRACE(ViewMode);
}
STDMETHODIMP ViewMode::SetClientSite(IOleClientSite *val) {
	METHTRACE(ViewMode,SetClientSite);
	RUN(IOleObjectImpl<ViewMode>::SetClientSite(val));
	if(!val)
		return S_OK;
	CComPtr<IOleContainer> oc;
	RUN(val->GetContainer(&oc));
	RECT rc;
	rc.left = rc.top = 0;
	RUN(CComQIPtr<IMontageContainer>(oc)->get_Size((SIZE*)&rc.right));
	RUN(CComQIPtr<IMCCSite>(val)->put_Rect(&rc));
	DWORD triscuit; // ran out of cookies
	RUN(AtlAdvise(oc,GetUnknown(),IID_IMontageContainerEvents,&triscuit));
	// delegate may already be active
	if(m_state>=MODE_ACTIVE)
		RUN(Activate(m_state==MODE_ACTIVEONCE));
	return S_OK;
}
HRESULT ViewMode::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	return S_OK;
}
LRESULT ViewMode::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(ViewMode,ButtonDown);
	return 0;
}
LRESULT ViewMode::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(ViewMode,ButtonUp);
	// should return ui-active to active but how?
	if(uMsg==WM_LBUTTONUP) {
		if(m_activator)
			RUN(m_activator->NoUI());
	}
	else if(uMsg==WM_RBUTTONUP) {
		return 0; // Grr.. should do ambient props.  perhaps if this had its own selection?
		// there was a separate ambient-prop message
		POINT pt = MAKEPOINT(lParam);
		HMENU hMenu = CreatePopupMenu();
		g_menuItem.insert(hMenu,g_propertiesName,CMD_PROPERTIES);

		HWND hWnd;
		RUN(m_spInPlaceSite->GetWindow(&hWnd));
		RECT rc;
		::GetWindowRect(hWnd,&rc);
		TrackPopupMenu(hMenu,TPM_LEFTALIGN|TPM_RIGHTBUTTON,rc.left+pt.x,rc.top+pt.y,0,hWnd,NULL);
		DestroyMenu(hMenu);
	}
	return 0;
}
LRESULT ViewMode::DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(ViewMode,DoubleClick);
	return 0;
}
LRESULT ViewMode::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(ViewMode,MouseMove);
	return 0;
}
LRESULT ViewMode::SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
	METHTRACE(ViewMode,SetCursor);
	::SetCursor(LoadCursor(0,IDC_ARROW));
	return 0;
}
// IMCCViewMode
STDMETHODIMP ViewMode::Initialize(DIMCCModeDelegate *del, IItemSet *selection, IMCCItemSetActivator *activator,IMCCLayer *mainl) {
	METHTRACE(ViewMode,Initialize);
	if(m_macadamia)
		return S_FALSE;
	RUN(AtlAdvise(del,GetUnknown(),IID_IMCCMode,&m_macadamia));
	m_selection = selection;
	m_activator = activator;
	m_mainL = mainl;
	RUN(del->get_State(&m_state));
	return S_OK;
}
// DIMCCViewMode
STDMETHODIMP ViewMode::Initialize(DIMCCModeDelegate *del, DIItemSet *selection,
		DIMCCItemSetActivator *activator,DIMCCLayer *mainl) {
    METHTRACE(ViewMode,Initialize);
	CComPtr<IItemSet> selection2;
	if(selection)
		RUNRET(selection->QueryInterface(riidret(&selection2)));
	CComPtr<IMCCItemSetActivator> activator2;
	if(activator)
		RUNRET(activator->QueryInterface(riidret(&activator2)));
	CComPtr<IMCCLayer> mainl2;
	if(mainl)
		RUNRET(mainl->QueryInterface(riidret(&mainl2)));
	return Initialize(del,selection2,activator2,mainl2);
}
// INeedSomePointers
STDMETHODIMP ViewMode::TakePointers(IEnumUnknown *os,IEnumMoniker *ms) {
	METHTRACE(ViewMode,TakePointers);
	CComPtr<IUnknown> u;
	RUN(os->Next(1,&u,0));
	CComQIPtr<DIMCCModeDelegate> del = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IItemSet> sel = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMCCItemSetActivator> ac = u;
	u.Release();
	RUN(os->Next(1,&u,0));
	CComQIPtr<IMCCLayer> ma = u;
	return Initialize(del,sel,ac,ma);
}
// DIMCCMode
typedef enum_iterator<IEnumMCCSites,&IID_IEnumMCCSites,IMCCSite*,_CopyInterface<IMCCSite> >
	itsite;
// select all (for hatching/grapples) and activate
STDMETHODIMP ViewMode::Activate(VARIANT_BOOL once) {
	METHTRACE(ViewMode,Activate);
	if(m_spClientSite)
		RUN(CComQIPtr<IMCCSite>(m_spClientSite)->PutBehind(0));
	CComPtr<IEnumMCCSites> sites;
	RUN(m_mainL->EnumSites(&sites));
	for(itsite i(sites);i!=itsite();++i) {
		CComPtr<IMoniker> mk;
		RUN((*i)->get_RelMoniker(&mk));
		RUN(m_selection->Add(mk));
	}
	return m_activator?m_activator->Start():S_OK;
}
STDMETHODIMP ViewMode::Deactivate() {
	METHTRACE(ViewMode,Deactivate);
	if(m_activator)
		RUN(m_activator->Stop());
	return m_selection->Clear();
}
// IMontageContainerEvents
STDMETHODIMP ViewMode::Shutdown() {
    METHTRACE(ViewMode,Shutdown);
	return S_FALSE;
}
STDMETHODIMP ViewMode::Resize(SIZE s) {
	METHTRACE(ViewMode,Resize);
	if(CComQIPtr<IMCCSite> s2 = m_spClientSite) {
		CComPtr<IMontageContainer> mc;
		RUN(s2->GetContainer(riidret(&mc)));
		RECT extra;
		RUN(mc->get_ExtraArea(&extra));
		RECT rc = rect(s)-extra;
		RUN(s2->put_Rect(&rc));
	}
	return S_OK;
}
STDMETHODIMP ViewMode::NewExtent(SIZEL s) {
	METHTRACE(ViewMode,NewExtent);
	return S_FALSE;
}
STDMETHODIMP ViewMode::ScaleChanged() {
    METHTRACE(ViewMode,ScaleChanged);
    return S_FALSE;
}
STDMETHODIMP ViewMode::Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv) {
    METHTRACE(ViewMode,Viewing);
	return S_FALSE;
}
STDMETHODIMP ViewMode::GroupOp(BOOL) {
	METHTRACE(ViewMode,GroupOp);
	return S_FALSE;
}
STDMETHODIMP ViewMode::NewTransform(IMCCTransform*) {
	METHTRACE(ViewMode,NewTransform);
	return S_FALSE;
}
STDMETHODIMP ViewMode::SiteDeleted(IMCCSite*) {
    METHTRACE(ViewMode,SiteDeleted);
	return S_FALSE;
}
STDMETHODIMP ViewMode::LayerDeleted(IMCCLayer*) {
    METHTRACE(ViewMode,LayerDeleted);
	return S_FALSE;
}
STDMETHODIMP ViewMode::Shifting(long oldState,long newState) {
    METHTRACE(ViewMode,Shifting);
	return S_FALSE;
}

//IPersistStream
STDMETHODIMP ViewMode::Load(IStream *pStm) {
    METHTRACE(ViewMode,Load);
	RUN(IPersistStreamInitImpl<ViewMode>::Load(pStm));
	return S_OK;
}
STDMETHODIMP ViewMode::Save(IStream *pStm,BOOL fClearDirty) {
    METHTRACE(ViewMode,Save);
	RUN(IPersistStreamInitImpl<ViewMode>::Save(pStm,fClearDirty));
	return S_OK;
}
