/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// ViewMode.h : Declaration of the ViewMode

#ifndef __VIEWMODE_H_
#define __VIEWMODE_H_

#include "resource.h"       // main symbols


/////////////////////////////////////////////////////////////////////////////
// ViewMode
class ATL_NO_VTABLE ViewMode :
	public CComObjectRoot,
	public CComCoClass<ViewMode,&CLSID_MCCViewMode>,
	public CComControl<ViewMode>,
	public IPersistStreamInitImpl<ViewMode>,
	public IOleControlImpl<ViewMode>,
	public IOleObjectImpl<ViewMode>,
	public IOleInPlaceActiveObjectImpl<ViewMode>,
	public IViewObjectExImpl<ViewMode>,
	public IOleInPlaceObjectWindowlessImpl<ViewMode>,

	public IMCCViewMode,
	public IDispatchImpl<DIMCCViewMode>,
	public INeedSomePointers,
	public IMCCMode,
	public IMontageContainerEvents
{
	CComPtr<DIMCCModeDelegate> m_delegate;
	DWORD m_macadamia; // IMCCMode event connection ID
	CComPtr<IMCCItemSetActivator> m_activator;
	CComPtr<IItemSet> m_selection;
	CComPtr<IMCCLayer> m_mainL;
	long m_state;
public:
	ViewMode();
	~ViewMode();

	REGISTER_GENERIC(L"ViewMode")

BEGIN_COM_MAP(ViewMode)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject, IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IViewObject2, IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY_IID(IID_IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IOleInPlaceObject, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)

	COM_INTERFACE_ENTRY(IMCCViewMode)
	COM_INTERFACE_ENTRY(DIMCCViewMode)
	COM_INTERFACE_ENTRY(INeedSomePointers)
	COM_INTERFACE_ENTRY(IMCCMode)
	COM_INTERFACE_ENTRY(IMontageContainerEvents)
END_COM_MAP()

BEGIN_PROPERTY_MAP(ViewMode)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROPERTY_MAP()


BEGIN_MSG_MAP(ViewMode)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_MBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_RBUTTONDOWN, ButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE, MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_MBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_RBUTTONUP, ButtonUp)
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK, DoubleClick)
	MESSAGE_HANDLER(WM_SETCURSOR,SetCursor)
	CHAIN_MSG_MAP(CComControl<ViewMode>)
END_MSG_MAP()

	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT DoubleClick(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);

// IViewObjectEx
	STDMETHOD(GetViewStatus)(DWORD* pdwStatus)
	{
		METHTRACE(IViewObjectExImpl,GetViewStatus);
		*pdwStatus = 0;
		return S_OK;
	}
// IOleObject
	STDMETHODIMP SetClientSite(IOleClientSite *val);
public:
	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);

// IMCCViewMode
	STDMETHODIMP Initialize(DIMCCModeDelegate *del, IItemSet *selection, IMCCItemSetActivator *activator, IMCCLayer *main);
// DIMCCViewMode
	STDMETHODIMP Initialize(DIMCCModeDelegate *del, DIItemSet *selection,
		DIMCCItemSetActivator *activator,DIMCCLayer *mainl);
// INeedSomePointers
    STDMETHODIMP TakePointers(IEnumUnknown *os,IEnumMoniker *ms);
// IMCCMode
	STDMETHODIMP Activate(VARIANT_BOOL once);
	STDMETHODIMP Deactivate();
// IPersist
	STDMETHODIMP GetClassID(CLSID *ret) {
		*ret = GetObjectCLSID();
		return S_OK;
	}
// IPersistStream
	STDMETHODIMP Load(IStream *pStm);
	STDMETHODIMP Save(IStream *pStm,BOOL fClearDirty);
// IMontageContainerEvents
	STDMETHODIMP Shutdown();
	STDMETHODIMP Resize(SIZE s);
	STDMETHODIMP NewExtent(SIZEL);
	STDMETHODIMP Viewing(HDC hdc, LPCRECTL lpcrOut, LPCRECTL lpcrCanv);
	STDMETHODIMP NewTransform(IMCCTransform*);
	STDMETHODIMP ScaleChanged();
	STDMETHODIMP GroupOp(BOOL);
	STDMETHODIMP SiteDeleted(IMCCSite*);
	STDMETHODIMP LayerDeleted(IMCCLayer*);
	STDMETHODIMP Shifting(long oldState,long newState);
	STDMETHODIMP ShowArea(LPCRECTL r) { return S_FALSE; }
    STDMETHODIMP FocusChanged() { return S_FALSE; }
};

#endif //__VIEWMODE_H_
