/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Implementation of IViewObjectEx for Container
#include "stdafx.h"
#include "Container.h"

void Container::erase(HDC hdc,RECT *r) {
	COLORREF bkColor = GetSysColor(COLOR_WINDOW);
	if(CComQIPtr<IDispatch> d = m_pSite) {
		VARIANT var;
		if(SUCCEEDED(CComDispatchDriver::GetProperty(d,DISPID_AMBIENT_BACKCOLOR,&var))
			&&(var.vt == VT_I4 || var.vt == VT_UI4))
			OleTranslateColor(var.lVal,0,&bkColor);
	}
	HBRUSH hbr = CreateSolidBrush(bkColor);
	FillRect(hdc,r,hbr);
	DeleteObject(hbr);
}
HRESULT Container::draw(const drawTransformBase &t,HDC hdc,LPCRECTL bounds,LPCRECTL wbounds,LPCRECTL areaHint, site *ref, enum howRefType how) {
	METHTRACE(Container,draw);
	HRESULT hr;
	_ASSERTE(hdc);
	const bool siteCount = false;

	int q = 0;
	bool frontOfRef = false;
	int ln = 0;
	for(layerlist::reverse_iterator i = m_layers.rbegin();i!=m_layers.rend();i++) {
		int ns = (*i)->m_sites.size();
		if(siteCount) {
			char buf[10];
			sprintf(buf,"%d",ns);
			ExtTextOut(hdc,bounds->right-25,bounds->top+ln*20,0,0,buf,strlen(buf),0);
			ln++;
		}
		if(!((*i)->m_amTransformed || t.drawUntransformed))
			continue;
		for(sitelist::reverse_iterator j = (*i)->m_sites.rbegin();j != (*i)->m_sites.rend(); j++) {
			site *s = *j;
			if(s==ref) {
				// we've drawn up to ref
				if(how==upTo)
					return S_OK;
				// we should start drawing now that we've found ref
				if(how==frontOf) {
					frontOfRef = true;
					continue;
				}
			}
			if(how==frontOf && !frontOfRef)
				continue;
			if(s->m_state<MCCS_SHOWN)
				continue;
			RECTL rcl;
			RUN(t.getPos(s,&rcl));

			// don't draw stuff that doesn't intersect with areaHint
			RECT trash;
			if(areaHint && !IntersectRect(&trash,(RECT*)&rcl,(const RECT*)areaHint))
				continue;
//#define CLIP_TO_BOUNDS
#ifdef CLIP_TO_BOUNDS
			HRGN hrgn = CreateRectRgnIndirect((RECT*)&rcl);
			SelectClipRgn(hdc,hrgn);
			DeleteObject(hrgn);
#endif
			//this will fail sometimes when everything's ok?
			int rop2 = GetROP2(hdc);
			int state = SaveDC(hdc);
			hr = s->m_pControl->Draw(DVASPECT_CONTENT,-1,NULL,NULL,NULL,hdc,&rcl,wbounds,NULL,0);
			RestoreDC(hdc,state);
			_ASSERTE(rop2==GetROP2(hdc));
			if(FAILED(hr)) {
				// (when drawing to metafiles, what else?)
				hr = hr;
			}
		}
	}
#ifdef CLIP_TO_BOUNDS
	SelectClipRgn(hdc,0);
#endif
	return S_OK;
}
struct posSameSize : Container::drawTransformBase {
	bool cartesian;
	POINT offsetBounds,offsetSource;
	posSameSize(const RECT &bounds, const RECT &sourceRect) :
		drawTransformBase(true),
		offsetBounds(*(POINT*)&bounds), // sorry
		offsetSource(*(POINT*)&sourceRect),
		cartesian(bounds.top>bounds.bottom) // y-flip
	{}
	HRESULT getPos(site *s,RECTL *ret) const {
		RECT rcCtrl = s->m_windRect;
		OffsetRect(&rcCtrl,-offsetSource.x,-offsetSource.y);
		if(cartesian) {
			ret->top = offsetBounds.y - rcCtrl.top;
			ret->bottom = offsetBounds.y - rcCtrl.bottom;
		}
		else {
			ret->top = offsetBounds.y + rcCtrl.top;
			ret->bottom = offsetBounds.y + rcCtrl.bottom;
		}
		ret->left = offsetBounds.x + rcCtrl.left;
		ret->right = offsetBounds.x + rcCtrl.right;
		return S_OK;
	}
};
struct posTransform : Container::drawTransformBase {
	CComPtr<IMCCTransform> t;
	posTransform(IMCCTransform *t) : t(t),drawTransformBase(false) {}
	HRESULT getPos(site *s,RECTL *ret) const {
		return t->C2VR(&s->m_virtRect,(RECT*)ret);
	}
};

STDMETHODIMP Container::Draw(
     DWORD dwDrawAspect,
     LONG lindex,
     void __RPC_FAR *pvAspect,
     DVTARGETDEVICE __RPC_FAR *ptd,
     HDC hdcTargetDev,
     HDC hdcDraw,
     LPCRECTL lprcBounds,
     LPCRECTL lprcWBounds,
     BOOL (STDMETHODCALLTYPE __RPC_FAR *pfnContinue)(DWORD dwContinue),
     DWORD dwContinue) {
	IMETHTRACE(Container,IViewObject,Draw);
	// let transform know view coords
	// Draw is the root of all non-window drawing,
	// whereas draw gets called by WM_PAINT handler.
	// will not call Viewing when lprcBounds==0 but not windowless yet.
	if(m_transform)
		RUN(m_transform->Viewing(lprcBounds,0));
	RUN(fireEvent((IMontageContainer*)this,fireThreeParams<IMontageContainerEvents,HDC,LPCRECTL,LPCRECTL>(&IMontageContainerEvents::Viewing,hdcDraw,lprcBounds,0)));
//#define SELF_CLIP
#ifdef SELF_CLIP
	// some containers (e.g. Montage, Visio) won't clip IVO::Draw objects.
	// Let's be polite and clip ourselves.
	HRGN clipOld = CreateRectRgn(0,0,0,0),
		clip = CreateRectRgnIndirect((RECT*)lprcBounds);
	int got = GetClipRgn(hdcDraw,clipOld);
	if(got>0) {
		switch(CombineRgn(clip,clip,clipOld,RGN_AND)) {
		case NULLREGION:
			return S_OK; // they didn't really want to draw me.
		case SIMPLEREGION:
		case COMPLEXREGION:
			SelectClipRgn(hdcDraw,clip);
			break;
		case ERROR:
			_ASSERTE(0);
		}
	}
#endif
	/*
	// very weird: this case was causing continuous full-screen repaints
	RECT contRect = contentRect();
	if(wid(*lprcBounds)==wid(contRect) && hei(*lprcBounds)==hei(contRect))
		RUN(draw(posSameSize(*(RECT*)lprcBounds,contRect),hdcDraw,lprcBounds,lprcWBounds));
	else*/
		RUN(draw(posTransform(m_transform),hdcDraw,lprcBounds,lprcWBounds));
#ifdef SELF_CLIP
	if(got>0)
		SelectClipRgn(hdcDraw,clipOld);
	DeleteObject(clipOld);
	DeleteObject(clip);
#endif
	if(m_transform)
		RUN(m_transform->Viewing(0,0));
	RUN(fireEvent((IMontageContainer*)this,fireThreeParams<IMontageContainerEvents,HDC,LPCRECTL,LPCRECTL>(&IMontageContainerEvents::Viewing,0,0,0)));
	return S_OK;
}

STDMETHODIMP Container::GetColorSet(
     DWORD dwDrawAspect,
     LONG lindex,
     void __RPC_FAR *pvAspect,
     DVTARGETDEVICE __RPC_FAR *ptd,
     HDC hicTargetDev,
     LOGPALETTE __RPC_FAR *__RPC_FAR *ppColorSet) {
	IMETHTRACE(Container,IViewObject,GetColorSet);
	*ppColorSet = NULL;
	return S_OK;
}

STDMETHODIMP Container::Freeze(
     DWORD dwDrawAspect,
     LONG lindex,
     void __RPC_FAR *pvAspect,
     DWORD __RPC_FAR *pdwFreeze) {
	METHTRACE(Container,Freeze);
	return S_OK;
}

STDMETHODIMP Container::Unfreeze(
     DWORD dwFreeze) {
	IMETHTRACE(Container,IViewObject,Unfreeze);
	return S_OK;
}

STDMETHODIMP Container::SetAdvise(
     DWORD aspects,
     DWORD advf,
     IAdviseSink __RPC_FAR *pAdvSink) {
	IMETHTRACE(Container,IViewObject,SetAdvise);
	if(aspects!=DVASPECT_CONTENT)
		ERR(E_INVALIDARG);
	if(!(advf&ADVF_ONLYONCE))
		m_pViewSink = pAdvSink;
	if(advf&ADVF_PRIMEFIRST && pAdvSink)
		pAdvSink->OnViewChange(DVASPECT_CONTENT,-1);
	return S_OK;
}

STDMETHODIMP Container::GetAdvise(
     DWORD __RPC_FAR *pAspects,
     DWORD __RPC_FAR *pAdvf,
     IAdviseSink __RPC_FAR *__RPC_FAR *ppAdvSink) {
	IMETHTRACE(Container,IViewObject,GetAdvise);
	if(pAspects)
		*pAspects = DVASPECT_CONTENT; // a little lie
	if(pAdvf)
		*pAdvf = 0; // oo! another
	if(ppAdvSink)
		m_pViewSink.CopyTo(ppAdvSink);
	return S_OK;
}

STDMETHODIMP Container::GetExtent(
     DWORD dwDrawAspect,
     LONG lindex,
     DVTARGETDEVICE __RPC_FAR *ptd,
     LPSIZEL lpsizel) {
	IMETHTRACE(Container,IViewObject2,GetExtent);
	// use IOleObject implementation
	return GetExtent(dwDrawAspect,lpsizel);
}

STDMETHODIMP Container::GetRect(
     DWORD dwAspect,
     LPRECTL pRect) {
	IMETHTRACE(Container,IViewObjectEx,GetRect);
	pRect->left = pRect->top = 0;
	switch(dwAspect) {
	case DVASPECT_CONTENT:
	case DVASPECT_OPAQUE:
	{
		SIZEL szHim;
		GetExtent(dwAspect,&szHim);
		pRect->right = szHim.cx;
		pRect->bottom = szHim.cy;
		return S_OK;
	}
	default:
		ERR(DV_E_DVASPECT);
	}
}

STDMETHODIMP Container::GetViewStatus(
     DWORD __RPC_FAR *pdwStatus) {
	METHTRACE(Container,GetViewStatus);
	*pdwStatus = VIEWSTATUS_OPAQUE|VIEWSTATUS_SOLIDBKGND;
	return S_OK;
}

STDMETHODIMP Container::QueryHitPoint(
     DWORD dwAspect,
     LPCRECT pRectBounds,
     POINT ptlLoc,
     LONG lCloseHint,
     DWORD __RPC_FAR *pHitResult) {
	IMETHTRACE(Container,IViewObjectEx,QueryHitPoint);
	if(smartPtInRect(pRectBounds,ptlLoc))
		*pHitResult = HITRESULT_HIT;
	else
		*pHitResult = HITRESULT_OUTSIDE;
	return S_OK;
}

STDMETHODIMP Container::QueryHitRect(
     DWORD dwAspect,
     LPCRECT pRectBounds,
     LPCRECT pRectLoc,
     LONG lCloseHint,
     DWORD __RPC_FAR *pHitResult) {
	IMETHTRACE(Container,IViewObjectEx,QueryHitRect);
	RECT rc;
	if(IntersectRect(&rc,pRectBounds,pRectLoc))
		*pHitResult = HITRESULT_HIT;
	else
		*pHitResult = HITRESULT_OUTSIDE;
	return S_OK;
}

STDMETHODIMP Container::GetNaturalExtent(
     DWORD dwAspect,
     LONG lindex,
     DVTARGETDEVICE __RPC_FAR *ptd,
     HDC hicTargetDev,
     DVEXTENTINFO __RPC_FAR *pExtentInfo,
     LPSIZEL pSizel) {
	IMETHTRACE(Container,IViewObjectEx,GetNaturalExtent);
	ERR(E_NOTIMPL);
}
