/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// WndProcCC.cpp : WndProc for the Container control

#include "stdafx.h"
#include "Container.h"

#include "keymessname.h"

#define DNU_REENTRY_CHECK
#ifdef DNU_REENTRY_CHECK
map<Container*,bool> reenter;
#endif

pair<bool,LRESULT> relayMessage(site *s,UINT uMsg, WPARAM wParam, LPARAM lParam) {
	if(!s->m_pWindowless) {
		CComQIPtr<IOleWindow> ow = s->m_pControl;
		if(ow) {
			HWND hWnd;
			if(SUCCEEDED(ow->GetWindow(&hWnd)))
				return make_pair(true,SendMessage(hWnd,uMsg,wParam,lParam));
		}
	}
    else {
        LRESULT lr=0;
        return make_pair(S_OK==SLOOK(s->m_pWindowless->OnWindowMessage(uMsg,wParam,lParam,&lr)),lr);
	}
    return make_pair(false,(LRESULT)0);
}
LRESULT CALLBACK Container::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	Container *pThis = (Container *)GetWindowLong(hWnd,0);
	LRESULT lr;
	IOleInPlaceObjectWindowless *pDest = 0; // (doesn't hold refcount)
	FUNTRACE(ContainerWndProc);
	// needs IPointerInactive support
	if(mousemess(uMsg)) {
		// note that any active ctrl we're getting mouse messages for must be windowless!
		POINT pt;
		if(uMsg==WM_SETCURSOR) {
			GetCursorPos(&pt);
			ScreenToClient(hWnd,&pt);
		} else
			pt = MAKEPOINT(lParam);
		// grab focus on button down
        if(uMsg==WM_LBUTTONDOWN || uMsg==WM_MBUTTONDOWN || uMsg==WM_RBUTTONDOWN) {
            SetFocus(hWnd);
			SLOOK(pThis->chstate(MCCS_UIACTIVE));
        }
		// capture, under, container
		if(pThis->m_pCaptureObject)
			if(S_OK==SLOOK(pThis->m_pCaptureObject->m_pWindowless->OnWindowMessage(uMsg,wParam,lParam,&lr)))
				return lr;
		for(layerlist::iterator il=pThis->m_layers.begin();il!=pThis->m_layers.end();il++)
			for(sitelist::iterator is = (*il)->m_sites.begin();is != (*il)->m_sites.end();is++) {
				site *s = *is;
				// the only way we can get messages for an active windowed object
				// is if it is open in a separate window
				if(s->m_state>MCCS_OPEN && s->m_pWindowless && s->hitpoint(pt))
					if(S_OK==SLOOK(s->m_pWindowless->OnWindowMessage(uMsg,wParam,lParam,&lr)))
						return lr;
			}
	}
	else if(keymess(uMsg)) {
		MTrace(KE)("Montage key message 0x%x: '%c'\n",uMsg,(char)wParam);
		// ATLTRACE2("key message: %s, %d ('%c')\n",keyMessName(uMsg),wParam,wParam);
		// Word doesn't seem to call IOleInPlaceActiveObject::TranslateAccelerator (bastards!)
		MSG msg;
		msg.hwnd = hWnd;
		msg.message = uMsg;
		msg.wParam = wParam;
		msg.lParam = lParam;
		msg.time = 0;
		msg.pt.x = msg.pt.y = 0;
		/*
		if(pThis->TranslateAccelerator(&msg)==S_OK)
			return 0;
		*/
		if(pThis->m_pFocusObject&&pThis->m_pFocusObject->m_pWindowless) {
#ifdef _DEBUG
			CLSID clsid;
			SRUN(getObjectCLSID(pThis->m_pFocusObject->m_pWindowless,&clsid));
			LPOLESTR progID;
			SRUN(ProgIDFromCLSID(clsid,&progID));
			MTrace(KE)("key message 0x%x: '%c' goes to object of class %S\n", uMsg,(char)wParam,progID);
			CoTaskMemFree(progID);
            _ASSERTE(pThis->m_pFocusObject->m_pWindowless);
#endif
            if(pThis->m_pFocusObject->m_pWindowless)
			    if(S_OK==SLOOK(pThis->m_pFocusObject->m_pWindowless->OnWindowMessage(uMsg,wParam,lParam,&lr)))
				    return lr;
		}
	}
	switch(uMsg) {
	case WM_COMMAND: {
		menumap::iterator im = pThis->m_menumap.find(LOWORD(wParam));
        if(im!=pThis->m_menumap.end()) {
            pair<bool,LRESULT> res = relayMessage(im->second,uMsg,wParam,lParam);
            if(res.first)
                return res.second;
        }
        UINT code = LOWORD(wParam);
        if(IWindowMessageHandler *handler = pThis->m_commandMap[code])
			if(S_OK==SLOOK(handler->OnWindowMessage(uMsg,wParam,lParam,&lr)))
				return lr;
		break;
	}
    case WM_NOTIFY: {
        UINT code = reinterpret_cast<LPNMHDR>(lParam)->code;
        if(IWindowMessageHandler *handler = pThis->m_notifyMap[code])
			if(S_OK==SLOOK(handler->OnWindowMessage(uMsg,wParam,lParam,&lr)))
				return lr;
		break;
    }
    case WM_TIMER:
        if(site *s = pThis->m_timers[wParam]) {
            pair<bool,LRESULT> res = relayMessage(s,uMsg,wParam,lParam); // not sure why you would use this if you had an HWND
            if(res.first)
                return res.second;
        }
        break;
	case WM_SETCURSOR:
	//case WM_CONTEXTMENU:
	//case WM_HELP:
        SetCursor(LoadCursor(0,IDC_ARROW));
		return 0;
	case WM_KILLFOCUS: {
		if(CComQIPtr<IOleControlSite> pOCSite = pThis->m_pInPlaceSite)
			pOCSite->OnFocus(FALSE);
        pThis->m_haveFocus = false;
        pThis->changeFocus();
		return 0;
	}
	case WM_SETFOCUS: {
		if(CComQIPtr<IOleControlSite> pOCSite = pThis->m_pInPlaceSite)
			pOCSite->OnFocus(TRUE);
        pThis->m_haveFocus = true;
        pThis->changeFocus();
		return 0;
	}
	case WM_PAINT:
		if(pThis->m_skipPaint) {
			pThis->m_skipPaint = false;
			return 0;
		}
		else {
#ifdef DNU_REENTRY_CHECK
			_ASSERTE(!reenter[pThis]);
			reenter[pThis] = true;
#endif
#ifdef TRY_DRAW
            try
#endif
			{
			    RECT rc = pThis->rect();
			    if(wParam) {
				    HDC hdc = (HDC)wParam;
				    hdc = pThis->startBuffer(hdc,&rc,false);
				    pThis->erase(hdc,&rc);
				    pThis->draw(posBounds(),pThis->m_bufferDC,(PRECTL)&rc,(PRECTL)&rc);
				    pThis->blitBuffer();
			    }
			    else {
				    PAINTSTRUCT ps;
				    HDC hdc = BeginPaint(hWnd,&ps);
				    hdc = pThis->startBuffer(hdc,&ps.rcPaint,false);
				    pThis->erase(hdc,&ps.rcPaint);
				    //ATLTRACE2("Paint (%d,%d,%d,%d)\n",ps.rcPaint.left,ps.rcPaint.top,ps.rcPaint.right,ps.rcPaint.bottom);
				    pThis->draw(posBounds(),pThis->m_bufferDC,(PRECTL)&rc,(PRECTL)&rc);
				    pThis->blitBuffer();
				    EndPaint(hWnd,&ps);
			    }
            }
#ifdef TRY_DRAW
            catch(...) {
                BREAK();
            }
#endif
#ifdef DNU_REENTRY_CHECK
			reenter[pThis] = false;
#endif
		}
		return 0;
	case WM_ERASEBKGND:
		if(!pThis)
			break;
		// always handled within WM_PAINT
		return 1;
	case WM_SIZE:
		if(pThis) {
			POINTS newSize = MAKEPOINTS(lParam);
			// apparently using a memory DC w/ createcompatible,
            // even with high-color bitmap selected, will always give you monochrome
			HDC dc = GetDC(hWnd);
			HBITMAP newBitmap = CreateCompatibleBitmap(dc,newSize.x,newSize.y),
				oldBitmap = (HBITMAP)SelectObject(pThis->m_bufferDC,newBitmap);
			ReleaseDC(hWnd,dc);
			DeleteObject(oldBitmap);

            if(pThis->m_intoWnd) {
                RECT rc;
                GetClientRect(pThis->m_hWnd,&rc);
                pThis->setObjectRects(&rc,&rc,true,false,true);
            }
		}
		return 0;
	case MONTAGE_TRIGGER_MESSAGE: {
		for(unsigned i=0; i<pThis->m_triggers.size(); ++i) {
			pThis->m_triggers[i]->Trigger();
			pThis->m_triggers[i]->Release();
		}
		pThis->m_triggers.clear();
		return 0;
	}
	case MONTAGE_NOTICE_CHANGES:
	    SRUN(pThis->sayChange());
		return 0;
	case MONTAGE_CHECK_SITE_SIZE:
		// can't do this stuff during OnDataChange
		site *s = reinterpret_cast<site*>(lParam);
		s->checkSizeChange();
		return 0;
	}
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}
