/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "boot.h"
#include "MBasics.h"

#include "ambiance.h"
#include "defaultFont.h"
#include "createLayer.h"

// the most basic document stuff

STDMETHODIMP bootDocument::InitializeOther(IUnknown *other) {
	METHTRACE(bootDocument,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	// a default type: overload to get the right type displayed in OLE dialogs
	// (the class otherwise only affects this boot stuff, and unless some container
	// has a "create new of this contained item's type" InitNew will never again be
	// called on this doc.)
	RUN(mb.c->put_ClassID(CLSID_MontageManualLayout));
	RUN(mb.p->put_Class(CLSID_MontageManualLayout));

	// initializers to start embedded objects via IOleCache and IRunnableObject
	CComPtr<IInitializeOther> init;
	RUN(CREATE(init,CLSID_MCCCacheInit));
	RUN(mb.add(L"Cache Init",init,keep_configuration,0));
	init.Release();
	RUN(CREATE(init,CLSID_MCCRunInit));
	RUN(mb.add(L"Run Init",init,keep_configuration,0));

	// file im/export
	CComPtr<IMCCFileFormat> port;
	RUN(CREATE(port,CLSID_MCCFilePort));
	RUN(mb.c->put_FilePort(port));
	CComPtr<IMoniker> mkPort;
	RUN(mb.add(L"File Port",port,dont_keep,&mkPort));
	RUN(mb.p->put_FilePortMk(mkPort));

	// ignite the ScrollingTransform
	CComPtr<IMCCTransform> scrolling;
	RUN(CREATE(scrolling,CLSID_MCCScrollingTransform));
	RUN(mb.c->put_Transform(scrolling));
	CComPtr<IMoniker> mkScrolling;
	RUN(mb.add(L"Scrolling Transform",scrolling,dont_keep,&mkScrolling));
	RUN(mb.p->put_TransformMk(mkScrolling));

	// transform manager
	CComPtr<DIMCCTransformManager> tm;
	RUN(CREATE(tm,CLSID_MCCTransformManager));
	RUN(tm->Initialize(CComQIPtr<DIMontageContainer>(mb.c)));
	RUN(tm->put_UseExtraArea(VARIANT_TRUE));
	RUN(tm->put_ShowScrollbars(VARIANT_TRUE));

	CComPtr<IHoldOnePointer> hop;
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	RUN(mb.add(L"Transform Manager",hop,keep_configuration,0));
	RUN(hop->put_Moniker(mb.mk));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,tm));
	hop.Release();

	// the document layers (main and active) need some ambient properties
	CComPtr<IDispatchEx> bien;
	CComPtr<IMoniker> mkAmbiente;
	RUN(ambiance(L"Ambiance",mb,&bien,&mkAmbiente));
	// w/ font
	CComPtr<IFontDisp> fd;
	RUN(defaultFont(L"Times New Roman",120000,&fd));
	CComBSTR font(L"Font");
	DISPID idF; // gets set to DISPID_AMBIENT_FONT
	RUN(bien->GetDispID(font,fdexNameEnsure|fdexNameCaseInsensitive,&idF));
	CComVariant v(fd);
	RUN(CComDispatchDriver::PutProperty(bien,idF,&v));

	CComPtr<IHoldSomePointers> hsp;

	// create & add layer ordering
	CComPtr<IMCCLayerOrdering> layerOrdering;
	RUN(CREATE(layerOrdering,CLSID_MCCLayerOrdering));
	CComPtr<IMoniker> mkLayerOrdering;
	RUN(mb.add(L"Layer Ordering",layerOrdering,dont_keep,&mkLayerOrdering));

	// create layers
	CComPtr<IMCCLayer> mainLayer,grappleLayer,modeLayer,activeLayer;
	CComPtr<IMoniker> mkMainLayer,mkGrappleLayer,mkModeLayer,mkActiveLayer;
	RUN(createLayer(mb,L"Main",0,mkLayerOrdering,bien,mkAmbiente,&mainLayer,&mkMainLayer));
	RUN(createLayer(mb,L"Mode",mainLayer,mkLayerOrdering,0,0,&modeLayer,&mkModeLayer));
	RUN(createLayer(mb,L"Grapple",modeLayer,mkLayerOrdering,0,0,&grappleLayer,&mkGrappleLayer));
	RUN(createLayer(mb,L"Active",grappleLayer,mkLayerOrdering,bien,mkAmbiente,&activeLayer,&mkActiveLayer));

	RUN(mainLayer->put_IsTransformed(true));
	RUN(activeLayer->put_IsTransformed(true));

	// transfer mgrs
	CComPtr<IMCCDataObjectReader> reader;
	RUN(CREATE(reader,CLSID_MCCDataObjectReader));
	CComPtr<IMoniker> mkReader;
	RUN(mb.add(L"DataObject Reader",reader,dont_keep,&mkReader));
	// paste sets.
	CComPtr<IMCCPasteSetUnpacker> native;
	RUN(CREATE(native,CLSID_MCCPasteSetUnpacker));
	RUN(native->Initialize(mb.c,reader));
	hsp.Release();
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Paste Set Unpacker",hsp,keep_configuration,0));
	RUN(hsp->Push(mb.mk));
	RUN(hsp->Push(mkReader));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,native));
	// embed-pastes
	CComPtr<IMCCEmbedUnpacker> unpack2;
	RUN(CREATE(unpack2,CLSID_MCCEmbedUnpacker));
	RUN(unpack2->Initialize(reader,mainLayer));
	hsp.Release();
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Embed Unpacker",hsp,keep_configuration,0));
	RUN(hsp->Push(mkReader));
	RUN(hsp->Push(mkMainLayer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,unpack2));

	// create empty data set & add
	CComPtr<DIMCCSiteSet> dataSet;
	RUN(CREATE(dataSet,CLSID_MCCSiteSet));
	RUN(mb.add(L"Data",dataSet,dont_keep,0));
	RUN(dataSet->Initialize(mb.mk));
	RUN(CComQIPtr<IGroupItem>(dataSet)->Realize(mb.bctx));

	return S_OK;
}
