/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "boot.h"
#include "MBasics.h"

#include "ambiance.h"
#include "defaultFont.h"

// support for multiple modes

STDMETHODIMP bootModeSupport::InitializeOther(IUnknown *other) {
	METHTRACE(bootModeSupport,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	CComPtr<IHoldOnePointer> hop;
	CComPtr<IHoldSomePointers> hsp;

	CComPtr<IMoniker> mkGrappleLayer;
	RUN(ItemMoniker(L"Grapple Layer")->ComposeWith(mb.goSubject,false,&mkGrappleLayer));
	CComPtr<IMCCLayer> grappleLayer;
	RUN(mkGrappleLayer->BindToObject(mb.bctx,mb.mkG,riidret(&grappleLayer)));
	CComPtr<IMoniker> mkMainLayer;
	RUN(ItemMoniker(L"Main Layer")->ComposeWith(mb.goSubject,false,&mkMainLayer));
	CComPtr<IMCCLayer> mainLayer;
	RUN(mkMainLayer->BindToObject(mb.bctx,mb.mkG,riidret(&mainLayer)));
	CComPtr<IMoniker> mkActiveLayer;
	RUN(ItemMoniker(L"Active Layer")->ComposeWith(mb.goSubject,false,&mkActiveLayer));
	CComPtr<IMCCLayer> activeLayer;
	RUN(mkActiveLayer->BindToObject(mb.bctx,mb.mkG,riidret(&activeLayer)));

	// fire up a mode switcher
	CComPtr<IMCCModeSwitcher> switcher;
	RUN(CREATE(switcher,CLSID_MCCModeSwitcher));
	CComPtr<IMoniker> mkSwitcher;
	RUN(mb.add(L"Mode Switcher",switcher,dont_keep,&mkSwitcher));

	// Selection helpers for the modes

	// default IMCCSelectable sink
	CComPtr<INeedOnePointer> nopSelector;
	RUN(CREATE(nopSelector,CLSID_DefaultMCCSelectable));
	RUN(nopSelector->TakePointer(mb.c,0));
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	RUN(mb.add(L"Default IMCCSelectable",hop,keep_configuration,0));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,nopSelector));
	RUN(hop->put_Moniker(mb.mk));
	hop.Release();

	// default IMCCSelectionDraw, a grapple drawer
	CComPtr<INeedOnePointer> nopGrappleDraw;
	RUN(CREATE(nopGrappleDraw,CLSID_DefaultMCCSelectionDraw));
	RUN(nopGrappleDraw->TakePointer(grappleLayer,0));
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	CComPtr<IMoniker> mkSelDraw; // hack: see "Selection Drawer"
	RUN(mb.add(L"Default IMCCSelectionDraw Sink",hop,keep_configuration,&mkSelDraw));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,nopGrappleDraw));
	RUN(hop->put_Moniker(mkGrappleLayer));

	// selection set
	CComPtr<IItemSet> selection;
	RUN(CREATE(selection,CLSID_MCCSiteSet));
	CComPtr<IMoniker> mkSelection;
	RUN(mb.add(L"Selection",selection,dont_keep,&mkSelection));
	RUN(CComQIPtr<DIMCCSiteSet>(selection)->Initialize(mb.mk));
	RUN(CComQIPtr<IGroupItem>(selection)->Realize(mb.bctx));

	// its IItemSetEvents sink that signals for selection indication
	CComPtr<IMCCItemSetDrawer> isd;
	RUN(CREATE(isd,CLSID_MCCItemSetDrawer));
	RUN(isd->Initialize(mb.c,selection));
	RUN(isd->Start());
	hsp.Release();
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Selection Drawer",hsp,keep_configuration,0));
	RUN(hsp->Push(mb.mk));
	RUN(hsp->Push(mkSelection));
	// HACK: MCCItemSetDrawer depends on a default IMCCSelectionDraw sink
	// and there's no way to explicitly order config. loading
	RUN(hsp->Push(mkSelDraw));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,isd));
	hsp.Release();

	// another IItemSetEvents sink keeps the selection activated
	// (between IMCCItemSetActivator::Start and Stop)
	CComPtr<IMCCItemSetActivator> activator;
	CComPtr<IMoniker> mkActivatorP, mkActivator;
	RUN(CREATE(activator,CLSID_MCCItemSetActivator));
	RUN(activator->Initialize(selection,mainLayer,activeLayer));
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Selection Activator",hsp,dont_keep,&mkActivatorP));
	RUN(hsp->Push(mkSelection));
	RUN(hsp->Push(mkMainLayer));
	RUN(hsp->Push(mkActiveLayer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,activator));
	RUN(mkActivatorP->ComposeWith(mb.goSubject,false,&mkActivator));
	hsp.Release();

	return S_OK;
}
STDMETHODIMP bootViewMode::InitializeOther(IUnknown *other) {
	METHTRACE(bootViewMode,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	CComPtr<IHoldSomePointers> hsp;

	ItemMoniker mkSwitcher(L"Mode Switcher"), mkSelection(L"Selection");
	CComPtr<IMCCModeSwitcher> switcher;
	RUN(mkSwitcher->BindToObject(mb.bctx,mb.mkG,riidret(&switcher)));
	CComPtr<IItemSet> selection;
	RUN(mkSelection->BindToObject(mb.bctx,mb.mkG,riidret(&selection)));
	CComPtr<IMoniker> mkActivator;
	RUN(ItemMoniker(L"Selection Activator")->ComposeWith(mb.goSubject,false,&mkActivator));
	CComPtr<IMCCItemSetActivator> activator;
	RUN(mb.bind(mkActivator,&activator));
	CComPtr<IMoniker> mkMainLayer;
	RUN(ItemMoniker(L"Main Layer")->ComposeWith(mb.goSubject,false,&mkMainLayer));
	CComPtr<IMCCLayer> mainLayer;
	RUN(mkMainLayer->BindToObject(mb.bctx,mb.mkG,riidret(&mainLayer)));
	CComPtr<IMoniker> mkModeLayer;
	RUN(ItemMoniker(L"Mode Layer")->ComposeWith(mb.goSubject,false,&mkModeLayer));
	CComPtr<IMCCLayer> modeLayer;
	RUN(mkModeLayer->BindToObject(mb.bctx,mb.mkG,riidret(&modeLayer)));

	// create entry for view mode
	CComPtr<DIMCCModeDelegate> del;
	RUN(switcher->new_Delegate(&del));
	CComBSTR vname(L"View");
	RUN(del->put_Name(vname));
	RUN(del->put_Order(0)); // left
	RUN(del->Show());

	// persist
	CComPtr<DIServerCreationPersistor> scp;
	RUN(CREATE(scp,CLSID_ServerCreationPersistor));
	CComPtr<IMoniker> mkDelP,mkDel;
	RUN(mb.add(L"View Mode Delegate",scp,keep_configuration,&mkDelP));
	RUN(mkDelP->ComposeWith(mb.goSubject,false,&mkDel));
	mkDelP.Release();
	CComPtr<IMoniker> mkDelCreator;
	RUN(mkSwitcher->ComposeWith(ClassMoniker(CLSID_MCCModeDelegate),false,&mkDelCreator));
	RUN(scp->put_ServerMoniker(mkDelCreator));
	RUN(CComQIPtr<DIPersistor>(scp)->SetSubject(true,del));
	scp.Release();

	// view mode
	CComPtr<IMCCViewMode> vm;
	RUN(CREATE(vm,CLSID_MCCViewMode));
	RUN(vm->Initialize(del,selection,activator,mainLayer));
	del.Release();
	// persistor
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkViewModeP,mkViewMode;
	RUN(mb.add(L"View Mode",hsp,dont_keep,&mkViewModeP));
	RUN(mkViewModeP->ComposeWith(mb.goSubject,false,&mkViewMode));
	RUN(hsp->Push(mkDel));
	RUN(hsp->Push(mkSelection));
	RUN(hsp->Push(mkActivator));
	RUN(hsp->Push(mkMainLayer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,vm));
	hsp.Release();
	mkDel.Release();
	// site
	CComPtr<IMCCSite> site;
	RUN(mb.c->new_Site(&site));
	RUN(site->put_Layer(modeLayer));
	RUN(site->put_Control(vm));
	RUN(site->put_Intent(MCCS_ACTIVE));
	// site persistor
	CComPtr<IMCCSitePersistor> msp;
	RUN(CREATE(msp,CLSID_MCCSitePersistor));
	CComPtr<IMoniker> mkSite;
	RUN(mb.add(L"View Mode Site",msp,keep_configuration,&mkSite)); // connect before load/init
	RUN(site->put_RelMoniker(mkSite));
	CComPtr<IMoniker> mkModeSiteOrderer;
	RUN(modeLayer->get_OrderMk(&mkModeSiteOrderer));
	RUN(msp->Initialize(mb.mkP,mkModeLayer,mkModeSiteOrderer,mkViewMode,0,0));
	RUN(CComQIPtr<DIPersistor>(msp)->SetSubject(true,site));
	mkSite.Release();
	msp.Release();
	site.Release();
	return S_OK;
}
STDMETHODIMP bootEditMode::InitializeOther(IUnknown *other) {
	METHTRACE(bootEditMode,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	CComPtr<IHoldSomePointers> hsp;

	ItemMoniker mkSwitcher(L"Mode Switcher"), mkSelection(L"Selection");
	CComPtr<IMCCModeSwitcher> switcher;
	RUN(mkSwitcher->BindToObject(mb.bctx,mb.mkG,riidret(&switcher)));
	CComPtr<IMoniker> mkDelCreator;
	RUN(mkSwitcher->ComposeWith(ClassMoniker(CLSID_MCCModeDelegate),false,&mkDelCreator));
	CComPtr<IItemSet> selection;
	RUN(mkSelection->BindToObject(mb.bctx,mb.mkG,riidret(&selection)));
	CComPtr<IMoniker> mkActivator;
	RUN(ItemMoniker(L"Selection Activator")->ComposeWith(mb.goSubject,false,&mkActivator));
	CComPtr<IMCCItemSetActivator> activator;
	RUN(mb.bind(mkActivator,&activator));
	CComPtr<IMoniker> mkMainLayer;
	RUN(ItemMoniker(L"Main Layer")->ComposeWith(mb.goSubject,false,&mkMainLayer));
	CComPtr<IMCCLayer> mainLayer;
	RUN(mkMainLayer->BindToObject(mb.bctx,mb.mkG,riidret(&mainLayer)));
	CComPtr<IMoniker> mkModeLayer;
	RUN(ItemMoniker(L"Mode Layer")->ComposeWith(mb.goSubject,false,&mkModeLayer));
	CComPtr<IMCCLayer> modeLayer;
	RUN(mkModeLayer->BindToObject(mb.bctx,mb.mkG,riidret(&modeLayer)));
	ItemMoniker mkReader(L"DataObject Reader");
	CComPtr<IMCCDataObjectReader> reader;
	RUN(mb.bind(mkReader,&reader));

	// container edit mode
	CComPtr<IContainerEditMode> cem;
	RUN(CREATE(cem,CLSID_MCCContainerEditMode));
	RUN(cem->Initialize(selection,activator,mainLayer,CComQIPtr<IMCCDataObjectUnpacker>(reader)));

	// ... persistor
	CComPtr<IHoldSomePointers> cemp;
	RUN(CREATE(cemp,CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkCEMP,mkCEM;
	RUN(mb.add(L"Edit Mode",cemp,dont_keep,&mkCEMP));
	RUN(mkCEMP->ComposeWith(mb.goSubject,false,&mkCEM));
	RUN(cemp->Push(mkSelection));
	RUN(cemp->Push(mkActivator));
	RUN(cemp->Push(mkMainLayer));
	RUN(cemp->Push(mkReader));
	RUN(CComQIPtr<DIPersistor>(cemp)->SetSubject(true,cem));

	// ... site
	CComPtr<IMCCSite> site;
	RUN(mb.c->new_Site(&site));
	RUN(site->put_Layer(modeLayer));
	RUN(site->put_Control(cem));

	// create entry in mode switcher's list
	CComPtr<DIMCCModeDelegate> del;
	RUN(switcher->new_Delegate(&del));
	CComBSTR ename(L"Edit");
	RUN(del->put_Name(ename));
	RUN(del->put_Order(100)); // in middle
	RUN(del->Show());
	RUN(del->Activate(false)); // default active mode.

	// persist
	CComPtr<DIServerCreationPersistor> scp;
	RUN(CREATE(scp,CLSID_ServerCreationPersistor));
	CComPtr<IMoniker> mkDelP,mkDel;
	RUN(mb.add(L"Edit Mode Delegate",scp,keep_configuration,&mkDelP));
	RUN(mkDelP->ComposeWith(mb.goSubject,false,&mkDel));
	RUN(scp->put_ServerMoniker(mkDelCreator));
	RUN(CComQIPtr<DIPersistor>(scp)->SetSubject(true,del));

	// ... ControlMode
	CComPtr<DIMCCControlMode> edito;
	RUN(CREATE(edito,CLSID_MCCControlMode));
	RUN(CComQIPtr<IObjectWithSite>(edito)->SetSite(site));
	RUN(edito->Initialize(del));
	RUN(site->put_Intent(MCCS_COUIACTIVE));

	// ... its persistor
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	CComPtr<IMoniker> mkEMOP,mkEMO;
	RUN(mb.add(L"Edit Mode Owner",hsp,dont_keep,&mkEMOP));
	RUN(mkEMOP->ComposeWith(mb.goSubject,false,&mkEMO));
	RUN(hsp->Push(mkDel));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,edito));

	// ... site persistor
	CComPtr<IMCCSitePersistor> msp;
	RUN(CREATE(msp,CLSID_MCCSitePersistor));
	CComPtr<IMoniker> mkSite;
	RUN(mb.add(L"Edit Mode Site",msp,keep_configuration,&mkSite)); // connect before load/init
	RUN(site->put_RelMoniker(mkSite));
	CComPtr<IMoniker> mkModeSiteOrderer;
	RUN(modeLayer->get_OrderMk(&mkModeSiteOrderer));
	RUN(msp->Initialize(mb.mkP,mkModeLayer,mkModeSiteOrderer,mkCEM,0,0));
	// add controlmode to persisted initialization list
	CComPtr<IOrderedItemList> oil;
	RUN(msp->get_InitList(&oil));
	RUN(oil->SetMoniker(10,mkEMO)); // 10?
	RUN(CComQIPtr<DIPersistor>(msp)->SetSubject(true,site));

	return S_OK;
}