/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "boot.h"
#include "MBasics.h"

#include "ambiance.h"
#include "defaultFont.h"

HRESULT bootPalette::InitializeOther(IUnknown *other) {
	METHTRACE(bootPalette,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	// normally montage won't use a margin unless it's a document object
	// (really everything except for a true content embedding should use a margin)
	RUN(mb.c->put_AlwaysMargin(VARIANT_TRUE));

	CComPtr<IInitializeOther> boot;
	RUN(boot.CoCreateInstance(CLSID_MCCBootDocument));
	RUN(boot->InitializeOther(other));
	boot.Release();
	RUN(boot.CoCreateInstance(CLSID_MCCBootModeSupport));
	RUN(boot->InitializeOther(other));
	boot.Release();
	RUN(boot.CoCreateInstance(CLSID_MCCBootEditMode));
	RUN(boot->InitializeOther(other));
	boot.Release();

	ItemMoniker mkSelection(L"Selection");
	CComPtr<IItemSet> selection;
	RUN(mkSelection->BindToObject(mb.bctx,mb.mkG,riidret(&selection)));

	// this grapple layer is behind!
	CComPtr<IMCCLayer> grappleLayer;
	RUN(mb.bind(ComposeMk(ItemMoniker(L"Grapple Layer"),GetSubjectMk()),&grappleLayer));
	RUN(grappleLayer->PutBefore(0));

	// and colored rect drawn instead of grapples
	CComPtr<DIDefaultMCCSelectionDraw> selectionDraw;
	RUN(mb.bind(ComposeMk(ItemMoniker(L"Default IMCCSelectionDraw Sink"),GetSubjectMk()),&selectionDraw));
	RUN(selectionDraw->put_GrappleClass(CComBSTR(L"Montage.ColorBox")));

	// and can't use extra area
	CComPtr<DIMCCTransformManager> tm;
	RUN(mb.bind(ComposeMk(ItemMoniker(L"Transform Manager"),GetSubjectMk()),&tm));
	RUN(tm->put_UseExtraArea(VARIANT_FALSE));

	// create a data object to represent selection (to be able to paste whatever's selected)
	CComPtr<IMCCItemSetRenderer> renderer;
	RUN(CREATE(renderer,CLSID_MCCItemSetRenderer));
	// buffer for efficiency
	CComPtr<IMCCDataObject> renderBuffer;
	RUN(CREATE(renderBuffer,CLSID_MCCDataObjectPersistent));
	RUN(renderer->Initialize(mb.g,selection,CLSID_MontageManualLayout,mb.c,renderBuffer));
	// persist
	CComPtr<IMoniker> mkBuffer;
	RUN(mb.add(L"Data Buffer",renderBuffer,dont_keep,&mkBuffer));
	CComPtr<IHoldSomePointers> hsp;
	RUN(CREATE(hsp,CLSID_SomePointersPersistor));
	RUN(mb.add(L"Selection Data Object",hsp,keep_configuration,0));
	RUN(hsp->Push(mkSelection));
	RUN(hsp->Push(mb.mk));
	RUN(hsp->Push(mkBuffer));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,renderer));

	return S_OK;
}
