/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "boot.h"
#include "MBasics.h"
#include "createLayer.h"
#include "ambiance.h"
#include "Toolbars.h"

STDMETHODIMP bootPanel::InitializeOther(IUnknown *other) {
	METHTRACE(bootPanel,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));

	RUN(mb.c->put_ClassID(CLSID_MCCToolPanel)); // tag

	// create empty data set & add (not sure this actually does anything
	CComPtr<DIMCCSiteSet> dataSet;
	RUN(CREATE(dataSet,CLSID_MCCSiteSet));
	RUN(mb.add(L"Data",dataSet,dont_keep,0));
	RUN(dataSet->Initialize(mb.mk));
	RUN(CComQIPtr<IGroupItem>(dataSet)->Realize(mb.bctx));

	// generally useful and tiny
	CComPtr<IInitializeOther> init;
	RUN(CREATE(init,CLSID_MCCCacheInit));
	RUN(mb.add(L"Cache Init",init,keep_configuration,0));
	init.Release();
	RUN(CREATE(init,CLSID_MCCRunInit));
	RUN(mb.add(L"Run Init",init,keep_configuration,0));

	// create & add layer ordering
	CComPtr<IMCCLayerOrdering> layerOrdering;
	RUN(CREATE(layerOrdering,CLSID_MCCLayerOrdering));
	CComPtr<IMoniker> mkLayerOrdering;
	RUN(mb.add(L"Layer Ordering",layerOrdering,dont_keep,&mkLayerOrdering));

	// install ambiance on tool layer to kill hatching
	CComPtr<IDispatchEx> amb;
	CComPtr<IMoniker> mkAmb;
	RUN(ambiance(L"Ambiance",mb,&amb,&mkAmb));
	// set ShowHatching = FALSE;
	CComBSTR showHatch(L"ShowHatching");
	DISPID idSH; // gets set to DISPID_AMBIENT_SHOWHATCHING
	RUN(amb->GetDispID(showHatch,fdexNameEnsure|fdexNameCaseInsensitive,&idSH));
	CComVariant v(false);
	RUN(CComDispatchDriver::PutProperty(amb,idSH,&v));

	// make layers for the bars & tools (toolbar engine will create another behind)
	CComPtr<IMCCLayer> barLayer,toolLayer;
	CComPtr<IMoniker> mkBarLayer,mkToolLayer;
	RUN(createLayer(mb,L"Bar",0,mkLayerOrdering,0,0,&barLayer,&mkBarLayer));
	RUN(createLayer(mb,L"Tool",barLayer,mkLayerOrdering,amb,mkAmb,&toolLayer,&mkToolLayer));

	// throw in a DefaultTransform for content sizing of toolbars
	CComPtr<IMCCTransform> deftrans;
	RUN(CREATE(deftrans,CLSID_MCCDefaultTransform));
	RUN(mb.c->put_Transform(deftrans));
	CComPtr<IMoniker> mkDefTrans;
	RUN(mb.add(L"Tool Panel Transform",deftrans,dont_keep,&mkDefTrans));
	RUN(mb.p->put_TransformMk(mkDefTrans));

	// fire up toolbar engine & name it
	CComPtr<IRectEngine> eng;
	RUN(CREATE(eng,CLSID_MCCBrutePriceEngine));
	CComPtr<IMoniker> mkEng;
	RUN(mb.add(L"Toolbar Engine",eng,dont_keep,&mkEng));
	RUN(eng->put_OuterFlexible(true));

	// data object handlers
	CComPtr<IMCCDataObjectReader> reader;
	RUN(CREATE(reader,CLSID_MCCDataObjectReader));
	//RUN(reader->Initialize(mb.c,editm));
	CComPtr<IMCCPasteSetUnpacker> unpackGroup;
	RUN(CREATE(unpackGroup,CLSID_MCCPasteSetUnpacker));
	RUN(unpackGroup->Initialize(mb.c,reader));
	CComPtr<IMCCEmbedUnpacker> unpackEmbed;
	RUN(CREATE(unpackEmbed,CLSID_MCCEmbedUnpacker));
	RUN(unpackEmbed->Initialize(reader,toolLayer));
	// persist
	CComPtr<IClassFactory> perFact;
	RUN(CoGetClassObject(CLSID_SomePointersPersistor,CLSCTX_ALL,0,riidret(&perFact)));
	CComPtr<IHoldSomePointers> per;
	CComPtr<IMoniker> mkReader;
	RUN(mb.add(L"Panel Data Reader",reader,dont_keep,&mkReader));
	RUN(perFact->CreateInstance(0,riidret(&per)));
	RUN(mb.add(L"Palette Group Unpacker",per,keep_configuration,0));
	RUN(per->Push(mb.mk));
	RUN(per->Push(mkReader));
	RUN(CComQIPtr<DIPersistor>(per)->SetSubject(true,unpackGroup));
	per.Release();
	RUN(perFact->CreateInstance(0,riidret(&per)));
	RUN(mb.add(L"Palette Embed Unpacker",per,keep_configuration,0));
	RUN(per->Push(mkReader));
	RUN(per->Push(mkToolLayer));
	RUN(CComQIPtr<DIPersistor>(per)->SetSubject(true,unpackEmbed));

	// make ToolLayout for drawing new toolbars
	CComPtr<IMCCToolLayout> toolLayout;
	RUN(CREATE(toolLayout,CLSID_MCCToolLayout));
	RUN(mb.add(L"Toolbar Drawer",toolLayout,keep_configuration,0));
	CComPtr<IMoniker> mkBarOrder;
	RUN(barLayer->get_OrderMk(&mkBarOrder));
	RUN(toolLayout->Initialize(mb.mkP,mkEng,mkBarLayer,mkBarOrder,VARIANT_TRUE));
	RUN(CComQIPtr<IGroupItem>(toolLayout)->Realize(mb.bctx));
	return S_OK;
}
STDMETHODIMP bootRegularPanel::InitializeOther(IUnknown *other) {
    METHTRACE(bootRegularPanel,InitializeOther);
	MBasics mb;
	RUN(mb.init(other));
	CComPtr<IMCCSiteLayoutEvents> toolDraw;
	RUN(ItemMoniker(L"Toolbar Drawer")->BindToObject(mb.bctx,mb.mkG,riidret(&toolDraw)));
	CComPtr<IMoniker> mkToolLayer;
	RUN(ItemMoniker(L"Tool Layer")->ComposeWith(mb.goSubject,false,&mkToolLayer));
	CComPtr<IMCCLayer> toolLayer;
	RUN(mkToolLayer->BindToObject(mb.bctx,mb.mkG,riidret(&toolLayer)));
	ItemMoniker mkOTo(L"Tool Site Ordering");
	CComPtr<IMoniker> mkAmb;
	RUN(ItemMoniker(L"Ambiance")->ComposeWith(mb.goSubject,false,&mkAmb));

	CComPtr<IRectEngine> eng;
	RUN(ItemMoniker(L"Toolbar Engine")->BindToObject(mb.bctx,mb.mkG,riidret(&eng)));
	RUN(eng->put_OuterFlexible(false));


	// get moniker to the switcher and the switcher itself (somewhat of a HACK)
	CComPtr<IMoniker> mkUp,mkSwitcher;
	RUN(mb.g->Up(&mkUp));
	ItemMoniker mkSw1(L"Mode Switcher");
	RUN(mkUp->ComposeWith(mkSw1,false,&mkSwitcher));
	CComPtr<IMCCModeSwitcher> switcher;
	RUN(mkSwitcher->BindToObject(mb.bctx,mb.mkG,riidret(&switcher)));

	// instantiate mode switcher control & site it
	CComPtr<INeedOnePointer> msc;
	RUN(CREATE(msc,CLSID_MCCModeSwitchCtl));
	RUN(msc->TakePointer(switcher,0));
	CComPtr<IHoldOnePointer> hop;
	RUN(CREATE(hop,CLSID_OnePointerPersistor));
	CComPtr<IMoniker> mkSwitchCtlPersistor,mkSwitchCtl,mkSCSiteP,mkSCSite;
	RUN(mb.add(L"ModeSwitcher Control",hop,dont_keep,&mkSwitchCtlPersistor));
	RUN(hop->put_Moniker(mkSwitcher));
	RUN(CComQIPtr<DIPersistor>(hop)->SetSubject(true,msc));
	CComPtr<IMCCSite> mscsite;
	RUN(mb.c->new_Site(&mscsite));
	RECT rcMSC = {0,0,155,25};
	RUN(mscsite->put_Rect(&rcMSC));
	RUN(mscsite->put_Layer(toolLayer));
	RUN(mscsite->put_Control(msc));
	RUN(mscsite->put_Intent(MCCS_ACTIVE));

	// make site persistable
	CComPtr<IMCCSitePersistor> msp;
	RUN(CREATE(msp,CLSID_MCCSitePersistor));
	RUN(mb.add(L"ModeSwitcher Site",msp,keep_configuration,&mkSCSiteP));
	RUN(mkSwitchCtlPersistor->ComposeWith(mb.goSubject,false,&mkSwitchCtl));
	RUN(msp->Initialize(mb.mkP,mkToolLayer,mkOTo,mkSwitchCtl,mkAmb,0));
	RUN(CComQIPtr<DIPersistor>(msp)->SetSubject(true,mscsite));
	RUN(mscsite->put_RelMoniker(mkSCSiteP));
	//RUN(mkSCSiteP->ComposeWith(mb.goSubject,false,&mkSCSite));

	// make bar for it
	//CComPtr<IMCCSitePersistor> modeBar;
	//CComPtr<IMCCSite> modeBS;
	RUN(toolDraw->Process(msp,L"Modes"));
	/*
	RUN(bartool(mb,L"Modes",eng,mkEng,barLayer,mkBarLayer,mkOBa,mscsite,mkSCSite,&modeBS,&modeBar));
	CComPtr<IMoniker> mkModeBar;
	RUN(mb.add(L"ModeBar Site",modeBar,keep_configuration,&mkModeBar));
	RUN(modeBS->put_RelMoniker(mkModeBar));
	*/

	// make palette.

	// When Montage goes into a group, it uses the persistor model (only the
	// persistor is actually named, saved, and loaded).
	CComPtr<IMontagePersistor> palper;
	RUN(CREATE(palper,CLSID_MontagePersistor));

	// create a palette subgroup for clarity
	CComPtr<IGroup> palGroup;
	RUN(CREATE(palGroup,CLSID_Group));
	CComPtr<IMoniker> mkPalGroup;
	RUN(mb.add(L"Palette",palGroup,dont_keep,&mkPalGroup,true));

	// put the persistor in the group, init, and set mkSelf
	ItemMoniker mkPalettePWithin(L"Montage Persistor");// (ie within palgroup)
	RUN(palGroup->Paste(mb.bctx,GP_SrcIsObj,palper,mkPalettePWithin,IID_NULL,0,0));
	RUN(CComQIPtr<IPersistStreamInit>(palper)->InitNew());
	RUN(palper->put_SelfMoniker(mkPalettePWithin));

	// easier to realize the persistor than to connect it and montage
	RUN(CComQIPtr<IGroupItem>(palper)->Realize(mb.bctx));

	// must be initialized explicitly because IPersistStorage::InitNew won't get hit.
	CComPtr<IInitializeOther> bp;
	RUN(CREATE(bp,CLSID_MCCBootPalette));
	RUN(bp->InitializeOther(palper));

	// create site
	CComPtr<IMCCSite> psite;
	RUN(mb.c->new_Site(&psite));
	RECT rcPalette = {300,0,600,60};
	RUN(psite->put_Rect(&rcPalette));
	RUN(psite->put_Layer(toolLayer));
	CComPtr<IUnknown> umcPal;
	RUN(CComQIPtr<DIPersistor>(palper)->get_Subject(&umcPal));
	CComPtr<IMontageContainer> mcPal;
	RUN(umcPal.QueryInterface(&mcPal));
	RUN(psite->put_Control(mcPal));

	// create moniker from outer group in to palette
	CComPtr<IMoniker> mkPaletteWithin,mkPalette;
	RUN(mkPalettePWithin->ComposeWith(mb.goSubject,false,&mkPaletteWithin));
	RUN(mkPalGroup->ComposeWith(mkPaletteWithin,false,&mkPalette));
	RUN(psite->put_Intent(MCCS_ACTIVE));

	// persist site
	msp.Release();
	RUN(CREATE(msp,CLSID_MCCSitePersistor));
	CComPtr<IMoniker> mkPalSiteP,mkPalSite;
	RUN(mb.add(L"Palette Site",msp,keep_configuration,&mkPalSiteP));
	RUN(psite->put_RelMoniker(mkPalSiteP));
	RUN(mkPalSiteP->ComposeWith(mb.goSubject,false,&mkPalSite));
	RUN(msp->Initialize(mb.mkP,mkToolLayer,mkOTo,mkPalette,0,0));
	RUN(CComQIPtr<DIPersistor>(msp)->SetSubject(true,psite));

	// bar it
	RUN(toolDraw->Process(msp,L"Nodes"));

    /*
    // dynagraph's not ready.
	// innovations!
	Toolness toolness(mb,toolLayer,mkToolLayer,toolDraw);

	CComPtr<IMCCButtonSet> playButtons;
	RUN(playButtons.CoCreateInstance(CLSID_MCCButtonSet));
	CComPtr<IMoniker> mkPlayButtons;
	RUN(mb.add(L"Play Buttons",playButtons,keep_configuration,&mkPlayButtons));
	RECT rcPlay = {100,0,196,32};
	RUN(toolness.createToolbar(L"Play Buttons",rcPlay,playButtons,mkPlayButtons));

	CComPtr<IMontageContainer> document;
	RUN(mb.bind(ComposeMk(mkUp,mb.mk),&document));
	CComPtr<IMCCPlayControls> playButtonBrain;
	RUN(playButtonBrain.CoCreateInstance(CLSID_MCCPlayControls));
	RUN(playButtonBrain->Initialize(playButtons,document));

	CComPtr<IHoldSomePointers> hsp;
	RUN(hsp.CoCreateInstance(CLSID_SomePointersPersistor));
	RUN(mb.add(L"Play Button Brain",hsp,keep_configuration,0));
	RUN(hsp->Push(mkPlayButtons));
	RUN(hsp->Push(mkUp));
	RUN(CComQIPtr<DIPersistor>(hsp)->SetSubject(true,playButtonBrain));

    */
	return S_OK;
}
