/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Container.h"
#include "Hatchling.h"

// tells all sites to realign with container's state
HRESULT Container::pullsites() {
	METHTRACE(Container,pullsites);
	for(layerlist::iterator i = m_layers.begin();i!=m_layers.end();i++)
		for(sitelist::iterator j = (*i)->m_sites.begin();j!=(*i)->m_sites.end();j++)
			RUN((*j)->approachstate((*j)->m_intent)); // just tell it to go where it wants to go; it will correct
	return S_OK;
}
// these are very strict about only performing the one step that is their duty.
// (these are not requests but actions, and the things that call them should be
// aware of the state.)

HRESULT Container::hide() {
	IMETHTRACE(Container,IOleObject,hide);
	_ASSERTE(m_state==MCCS_SHOWN); // internal error
	m_state = MCCS_HIDDEN;
	RUN(pullsites());
    // NOT calling LockContainer
	return S_OK;
}
HRESULT Container::show() {
	IMETHTRACE(Container,IOleObject,show);
	_ASSERTE(m_state==MCCS_HIDDEN); // internal error
	m_state = MCCS_SHOWN;
    // NOT calling LockContainer
	RUN(pullsites());
	return S_OK;
}

HRESULT Container::inplaceactivate() {
	IMETHTRACE(Container,IOleObject,inplaceactivate);
	_ASSERTE(m_state==MCCS_SHOWN); // internal error
	_ASSERT(m_pInPlaceSite);
	HRESULT hr = m_pInPlaceSite->CanInPlaceActivate();
	if(hr!=S_OK) {
		MessageBox(0,"Sorry, Montage doesn't yet support Open for Editing in Separate Window","Cannot Open Separately",MB_OK);
		ERR(E_FAIL);
	}
	// see if container supports flicker-free activation
	m_skipPaint = false;
	CComQIPtr<IOleInPlaceSiteEx> pEx = m_pInPlaceSite;
	if(pEx) {
		BOOL b;
		RUN(pEx->OnInPlaceActivateEx(&b,0));
		m_skipPaint = b!=FALSE;
	}
	else
		RUN(m_pInPlaceSite->OnInPlaceActivate());
	RUN(getInPlaceWindowContext()); // if DocObject this has already happened but so what?

	m_state = MCCS_ACTIVE;
	RUN(makewindow(true));
	RUN(pullsites());
	return S_OK;
}
HRESULT Container::inplacedeactivate() {
	IMETHTRACE(Container,IOleObject,inplacedeactivate);
	_ASSERTE(m_state==MCCS_ACTIVE); // internal error
	m_state = MCCS_SHOWN;
	RUN(pullsites());
	if(m_hWnd) {
		RUN(RevokeDragDrop(m_hWnd));
		DestroyWindow(m_hWnd);
		m_hWnd = NULL;
	}
	if(m_pHatchling) {
		delete m_pHatchling;
		m_pHatchling = NULL;
	}
	if(m_pInPlaceSite)
		RUN(m_pInPlaceSite->OnInPlaceDeactivate());
	m_pInPlaceSite.Release();
	m_pFrame.Release();
	m_pDoc.Release();
	return S_OK;
}
HRESULT Container::chstate(long intent) {
	METHTRACE(Container,chstate);
	if(m_state==intent)
		return S_OK;
	long old = m_state;
	HRESULT hr = S_OK; // RUNGOTO reassigns
	if(m_state<intent) {
		if(m_state<0)
			return E_OUTOFMEMORY; // totally bogus code to trick optimizer, which was munging if and switch statements together
		switch(m_state) {
			case MCCS_HIDDEN:
				RUNGOTO(show(),done);
				if(intent==MCCS_SHOWN) break;
			case MCCS_SHOWN:
                if(!m_pInPlaceSite)
                    goto done;
				RUNGOTO(inplaceactivate(),done);
				if(intent==MCCS_ACTIVE) break;
			case MCCS_ACTIVE:
                if(!m_pInPlaceSite)
                    goto done;
				RUNGOTO(uiactivate(),done);
				break;
			default: // MCCS_UIACTIVE
				ERR(E_UNEXPECTED); // some greater state?!
		}
	}
	else switch(m_state) { // > intent
	case MCCS_UIACTIVE:
		RUNGOTO(uideactivate(),done);
		// hack: Visio is calling my InPlaceDeactivate in response to the
		// OnUIDeactivate I am signalling because I am downshifting from
		// UI active to shown in response to -- InPlaceDeactivate!
		if(m_state==MCCS_SHOWN)
			goto already;
		if(intent==MCCS_ACTIVE) break;
	case MCCS_ACTIVE:
		RUNGOTO(inplacedeactivate(),done);
	already:
		if(intent==MCCS_SHOWN) break;
	case MCCS_SHOWN:
		RUNGOTO(hide(),done);
	}
done:
	if(m_state!=old)
		RUN(fireEvent(GetUnknown(),fireTwoParams<IMontageContainerEvents,long,long>(&IMontageContainerEvents::Shifting,old,m_state)));
	//ATLTRACE2("end chstate\n");
	return hr;
}
