/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"

#include <comcat.h>
#include "ChooseFromCatDlg.h"

ChooseFromCatDlg::ChooseFromCatDlg(CATID catid) : m_catid(catid) {
	CONSTRUCTORTRACE(ChooseFromCatDlg);
	USES_CONVERSION;
	CComPtr<ICatInformation> pCatMan;
	CREATE(pCatMan,CLSID_StdComponentCategoriesMgr);
	CComPtr<IEnumCLSID> pClasses;
	RUN(pCatMan->EnumClassesOfCategories(1,&m_catid,0,NULL,&pClasses));
	CRegKey key;
	if(key.Open(HKEY_CLASSES_ROOT,_T("CLSID"),KEY_READ)==ERROR_SUCCESS) {
		classinfo ci;
		while(pClasses->Next(1,&ci.clsid,NULL)==S_OK) {
			OLECHAR guidster[100];
			StringFromGUID2(ci.clsid,guidster,100);
			CRegKey key2;
			if(key2.Open(key,OLE2T(guidster),KEY_READ)==ERROR_SUCCESS) {
				TCHAR szUsable[100];
				DWORD dwCount = 100;
				if(key2.QueryValue(0,0,szUsable,&dwCount)==ERROR_SUCCESS) {
#ifdef UNICODE
					ci.lpszName = new TCHAR[dwCount];
					ATL::AtlW2AHelper(ci.lpszName, szUsable, dwCount);
#else
					ci.lpszName = strdup(szUsable);
#endif
					m_engines.push_back(ci);
				}
			}
		}
	}
}
ChooseFromCatDlg::~ChooseFromCatDlg() {
	DESTRUCTORTRACE(ChooseFromCatDlg);
	for(std::vector<classinfo>::iterator i = m_engines.begin();
			i != m_engines.end(); i++)
		free(i->lpszName);
}

LRESULT ChooseFromCatDlg::Initialize(UINT, WPARAM, LPARAM, BOOL &) {
	METHTRACE(ChooseFromCatDlg,Initialize);
	USES_CONVERSION;
	HWND hListWnd = GetDlgItem(IDC_EngList);
	if(!hListWnd)
		return true;

	for(std::vector<classinfo>::iterator i = m_engines.begin();i!=m_engines.end();i++) {
		int index = ::SendMessage(hListWnd,LB_ADDSTRING,0,(LPARAM)i->lpszName);
		::SendMessage(hListWnd,LB_SETITEMDATA,index,i-m_engines.begin());
	}
	::SendMessage(hListWnd,LB_SETCURSEL,0,0);
	HWND hOkayWnd = GetDlgItem(IDOK);
	::EnableWindow(hOkayWnd,TRUE);
	SetFocus();
	return true;
}
LRESULT ChooseFromCatDlg::Okay(WORD,WORD,HWND,BOOL&) {
	METHTRACE(ChooseFromCatDlg,Okay);
	HWND hListWnd = GetDlgItem(IDC_EngList);
	int index = ::SendMessage(hListWnd,LB_GETCARETINDEX,0,0);
	FinishUp();
	EndDialog(::SendMessage(hListWnd,LB_GETITEMDATA,index,0));
	return true;
}
LRESULT ChooseFromCatDlg::Select(WORD,WORD,HWND,BOOL&) {
	METHTRACE(ChooseFromCatDlg,Select);
	return true;
}
LRESULT ChooseFromCatDlg::Snappy(WORD a,WORD b ,HWND c,BOOL &d) {
	METHTRACE(ChooseFromCatDlg,Snappy);
	return Okay(a,b,c,d);
}
