/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef FactoryActivator_h
#define FactoryActivator_h

// be sure to #define _WIN32_WINNT 0x0400 before #include <objidl.h> to get IClassFactory
// silly way to tack IClassActivator (for class monikers) onto a class.
// simply bounces CreateInstance to derived class with class as param!
// T implements create CI4Factory(CLSID&,IUnknown*,REFIID,void**)
template<class T,const CLSID *pCLSID>
struct IClassFactoryImpl :
	IClassFactory
{
	STDMETHODIMP CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, void** ppvObj) {
		T *mainClass = (T*)this;
		return mainClass->CI4Factory(*pCLSID,pUnkOuter,riid,ppvObj);
	}
	STDMETHODIMP LockServer(BOOL fLock) {
		fLock?_Module.Lock():_Module.Unlock();
		return S_OK;
	}
};
// this could support multiple classes with a map & all, but I don't feel like writing
// that until I need it.  It's just hard-coded to one factory
template<class T,const CLSID *pCLSID>
struct IClassActivatorImpl :
	IClassActivator {
	STDMETHODIMP GetClassObject(
		REFCLSID rclsid,
		DWORD dwClassContext,
		LCID locale,
		REFIID riid,
		void **ppv) {
		if(rclsid!=*pCLSID)
			ERR(E_FAIL);
		T *mainClass = (T*)this;
		typedef IClassFactoryImpl<T,pCLSID> Factory;
		Factory *factory = (Factory*)mainClass;
		if(riid!=IID_IClassFactory)
			ERR(E_NOINTERFACE);
		factory->AddRef();
		*ppv = (IUnknown*)factory;
		return S_OK;
	}
};
#endif FactoryActivator_h
