/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// the documentation claims that CLSID_NULL <=> NULL object, yet both actually
// reject NULL (E_INVALIDARG, E_CLASSNOTREG).  This is a very useful feature,
// esp for persistors and their possibly absent monikers.

// FixedOleSaveToStream also queries for persistence interfaces, to make the call
// easier and to avoid possible errors caused by not querying for all three.  (It's not
// clear whether using IPersist as a no-data IPersistStream is "legal" OLE.  It's useful!)
inline HRESULT FixedOleSaveToStream(IUnknown *o, IStream *s) {
    FUNTRACE(FixedOleSaveToStream);
	// it appears that many things that implement ipersiststream do not expose ipersist.  so I have to
	// go this roundabout way.
	if(CComQIPtr<IPersistStreamInit> psi = o)
		return OleSaveToStream((IPersistStream*)psi.p,s);
	else if(CComQIPtr<IPersistStream> ps = o)
		return OleSaveToStream(ps,s);
	else if(CComQIPtr<IMoniker> mk = o) // classmks too lazy to expose ipersiststream separately
		return OleSaveToStream(mk,s);
	else {
		CLSID clsid;
		if(!o)
			clsid = CLSID_NULL;
		else {
			CComPtr<IPersist> p;
			SRUN(o->QueryInterface(riidret(&p)));
			SRUN(p->GetClassID(&clsid));
		}
		return WriteClassStm(s,clsid);
	}
}
inline HRESULT FixedOleLoadFromStream(IStream *s, REFIID riid, void **ret) {
    FUNTRACE(FixedOleLoadFromStream);
	CLSID clsid;
	SRUN(ReadClassStm(s,&clsid));
	if(clsid==CLSID_NULL) {
		*ret = 0;
		return S_FALSE;
	}
	else {
		CComPtr<IUnknown> o;
		SRUN(CREATE(o,clsid));
		TRACECREATEV(clsid);
		if(CComQIPtr<IPersistStream> ps = o)
			SRUN(ps->Load(s));
		else if(CComQIPtr<IPersistStreamInit> psi = o)
			SRUN(psi->Load(s));
		// else must be IPersist: okay.
		return o->QueryInterface(riid,ret);
	}
}