/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "FoolishCreatePointerMoniker.h"
// whole lot of objects you can (and need to) get from the persistor for configuring.
typedef enum {
	dont_keep,
	keep_configuration,
	keep_data
} keep_type;
struct MBasics {
	CComQIPtr<IMontagePersistor> p;
	CComPtr<IMontageContainer> c;
	CComPtr<IGroup> g;
	CComPtr<IMoniker> goSubject,mk,mkP,mkG;
	CComPtr<IItemSet> configset;//,dataset?
	CComPtr<IBindCtx> bctx;

	HRESULT init(IUnknown *other) {
        METHTRACE(MBasics,init);
		p = other;
		if(!p)
			ERR(E_INVALIDARG);
		CComPtr<IUnknown> umc;
		RUN(CComQIPtr<DIPersistor>(p)->get_Subject(&umc));
		RUN(umc.QueryInterface(&c));
		RUN(c->GetMainGroup(riidret(&g)));
		RUN(c->get_RelMoniker(&mkP));
		RUN(CREATE(goSubject,CLSID_GetSubjectMoniker));
		RUN(mkP->ComposeWith(goSubject,false,&mk));
		RUN(FoolishCreatePointerMoniker(g,&mkG));
		RUN(p->get_KeepSet(&configset));
		RUN(CreateBindCtx(0,&bctx));
		return S_OK;
	}
	HRESULT init(IGroup *g) {
        METHTRACE(MBasics,init);
		CComPtr<IMontagePersistor> mp;
		RUN(ItemMoniker(L"Montage Persistor")->BindToObject(BindCtx(),PointerMoniker(g),riidret(&mp)));
		return init(mp);
	}
	HRESULT init(IMontageContainer *mc) {
        METHTRACE(MBasics,init);
		CComPtr<IGroup> g;
		RUN(mc->GetMainGroup(riidret(&g)));
		return init(g);
	}
	HRESULT add(LPCOLESTR name,IUnknown *o,keep_type keeptype,IMoniker **ret, bool init = false) {
        METHTRACE(MBasics,add);
		CComPtr<IMoniker> mkName,retMk;
		if(name) // if not, anonymous
			RUN(CreateItemMoniker(L"!",name,&mkName));
		RUN(g->Paste(bctx,GP_SrcIsObj|GP_IsMainStg,o,mkName,IID_NULL,0,&retMk));
		if(init) {
			CComPtr<IStorage> stg;
			//RUN(CComQIPtr<IOleItemContainer>(g)->GetObjectStorage(const_cast<LPOLESTR>(name),bctx,riidret(&stg)));
			RUN(retMk->BindToStorage(bctx,mkG,riidret(&stg)));
			CComQIPtr<IPersistStorage> ps = o;
			QISET(ps);
			RUN(ps->InitNew(stg));
		}
		switch(keeptype) {
		case keep_configuration:
			RUN(configset->Add(retMk));
			break;
		/*
		case keep_data:
			RUN(dataset->Add(retMk));
		*/
		}
		if(ret)
			(*ret = retMk)->AddRef();
		return S_OK;
	}
	// pure laziness
	template<typename I>
	HRESULT bind(IMoniker *mk,I **ret) {
        METHTRACE(MBasics,bind);
		return mk->BindToObject(bctx,mkG,riidret(ret));
	}
};

