/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "..\mtrace\mtrace.h"
#include "twoCharCode.h"

#ifdef _DEBUG

inline CComPtr<IMTracer> tracer() {
    static IMTracer *s_tracer; // CComPtr caused destruction problems
    if(!s_tracer) {
        ::CoCreateInstance(CLSID_MTracer,0,CLSCTX_ALL,IID_IMTracer,(void**)&s_tracer);
        s_tracer->Start();
    }
    return s_tracer;
};
// odd syntax MTrace(CD)(fmt,...) because #defines don't support var args
// this all should compile out right?
struct _MTrace {
    int m_code;
    _MTrace(int code) : m_code(code) {}
    bool operator()(char *format,...) {
        char buf[2000];
        va_list args;
	    va_start(args, format);
	    vsprintf(buf, format, args);
	    va_end(args);
        CComPtr<IMTracer> tra = tracer();
        if(!tra)
            return false;
        HRESULT hr = tracer()->Trace(m_code,buf);
        return true;
    }
};
inline void startMTrace() {
    tracer()->Start();
}
#define MTrace(CO) _MTrace(C2C(CO))
inline int callDepth(bool enter) {
    int depth;
    tracer()->CallDepth(enter,&depth);
    return depth;
}
inline int callNumber() {
    int number;
    CComPtr<IMTracer> tra = tracer();
    if(!tra)
        return 0;
    else tra->CallNumber(&number);
    return number;
}
inline void stopMTrace() {
    tracer()->Stop();
}
#else
inline bool _MTrace(char *,...) { return true; }
#define MTrace(CO) _MTrace
#define stopMTrace()
#endif
