/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "regmap.h"

template<const CLSID *montageClass,const CLSID *bootClass>
struct MontageCreator {
	static HRESULT WINAPI CreateInstance(void*pv, REFIID riid, LPVOID* ppv) {
        FUNTRACE(CreateInstance);
		CComPtr<IMontageContainer> mc;
		SRUN(CREATE(mc,*montageClass));
		CComPtr<IInitializeOther> bs;
		SRUN(CREATE(bs,*bootClass));
		SRUN(mc->put_Bootstrapper(bs));
		return mc->QueryInterface(riid,ppv);
	}
};
LPOLESTR progdirString();
template<class T,const CLSID *clsid, class Creator,UINT resID>
struct MontageConfig :
	CComCoClass<MontageConfig,clsid>
{
	static HRESULT WINAPI UpdateRegistry(BOOL bRegister);
	typedef Creator _CreatorClass;
	DECLARE_CLASSFACTORY();
BEGIN_CATEGORY_MAP(MontageConfig)
	IMPLEMENTED_CATEGORY(CATID_Insertable)
	//IMPLEMENTED_CATEGORY(CATID_Control)
	IMPLEMENTED_CATEGORY(CATID_Programmable)
	IMPLEMENTED_CATEGORY(CATID_SafeForScripting)
	IMPLEMENTED_CATEGORY(CATID_SafeForInitializing)
	IMPLEMENTED_CATEGORY(CATID_DocObject)
END_CATEGORY_MAP();

// copy and override this in config's class
BEGIN_REGMAP()
	REGMAP_GUID(L"CLSID",CLSID_MontageControlContainer)
	REGMAP_GUID(L"TypeLib",LIBID_Montage)
	REGMAP_STRING(L"DocName",L"Montage OLE Container")
	REGMAP_STRING(L"AppName",L"Montage")
	REGMAP_STRING(L"Short",L"Container")
	REGMAP_STRING(L"ProgID",L"Montage.Control.1")
	REGMAP_STRING(L"VersIndProgID",L"Montage.Control")
	REGMAP_FUN(L"ProgDir",progdirString)
	REGMAP_STRING(L"DefaultExt",L"")
	REGMAP_STRING(L"Extra",L"")
END_REGMAP()
	// don't know what this does but I don't need it.
	static void WINAPI ObjectMain(bool /* bStarting */) {}
	static HRESULT moreReg() { return S_OK; }
};
template<class T,const CLSID *clsid, class Creator,UINT resID>
HRESULT MontageConfig<T,clsid,Creator,resID>::UpdateRegistry(BOOL bRegister) {
	allocd_regmap *rm = T::getRegMap();
    HRESULT hr =
    __if_exists(_Module) {
    	_Module
    }
    __if_not_exists(_Module) {
        (*ATL::_pAtlModule)
    }
        .UpdateRegistryFromResource(resID, bRegister,rm->map);

	delete rm;
	SRUNRET(hr);
	return T::moreReg();
}
