/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include <common/LGraph-cdt.h>
#include <common/StrAttr.h>

HRESULT writeDString(IStream *str,const DString &s) {
    FUNTRACE(writeDString);
	DString::size_type len = s.length();
	SRUN(str->Write(&len,sizeof DString::size_type,0));
	if(s.size())
		SRUN(str->Write(s.c_str(),len,0));
	return S_OK;
}
HRESULT readDString(IStream *str,DString &out) {
    FUNTRACE(readDString);
	DString::size_type len;
	SRUN(str->Read(&len,sizeof DString::size_type,0));
	if(len) {
		char *buf = new char[len+1];
		SRUN(str->Read(buf,len,0));
		buf[len] = 0;
		out = buf;
		delete [] buf;
	}
	else out.clear();
	return S_OK;
}
HRESULT writeAttrs(IStream *str,const StrAttrs &attrs) {
    FUNTRACE(writeAttrs);
	StrAttrs::size_type N = attrs.size();
	SRUN(str->Write(&N,sizeof StrAttrs::size_type,0));
	for(StrAttrs::const_iterator ai = attrs.begin(); ai!=attrs.end(); ++ai) {
		writeDString(str,ai->first);
		writeDString(str,ai->second);
	}
	return S_OK;
}
HRESULT readAttrs(IStream *str,StrAttrs &out) {
    FUNTRACE(readAttrs);
	out.clear();
	StrAttrs::size_type N;
	SRUN(str->Read(&N,sizeof StrAttrs::size_type,0));
	while(N--) {
		DString attr,val;
		SRUN(readDString(str,attr));
		SRUN(readDString(str,val));
		out[attr] = val;
	}
	return S_OK;
}