/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef RECT_H
#define RECT_H

// helpers
template<class T>
inline T tabs(T x) {
	return (x<T(0)) ? -x : x;
}
template<class O>
inline bool between(O x,O a,O b) {
	if(a<b)
		return (a<=x)&&(x<=b);
	else
		return (b<=x)&&(x<=a);
}

// rects are cool.
inline RECT rect(int left,int top,int right,int bottom) {
	// ie, I wish I could do this besides in initializations:
	RECT ret = {left,top,right,bottom};
	return ret;
}
inline RECTL rectl(int left,int top,int right,int bottom) {
	// ie, I wish I could do this besides in initializations:
	RECTL ret = {left,top,right,bottom};
	return ret;
}

#define R RECT
#include "rect2.h"
#undef R
#define R RECTL
#include "rect2.h"
#undef R

// intersect test
inline bool operator &&(const RECT &a,const RECT &b) {
	return !(a.right < b.left || a.left > b.right ||
	   a.bottom < b.top || a.top > b.bottom);
}
inline bool operator &&(const RECTL &a,const RECTL &b) {
	return !(a.right < b.left || a.left > b.right ||
	   a.bottom > b.top || a.top < b.bottom);
}
// intersectType determines the type of intersection between two rectangles
// the return value is a 5-bit integer.  the top bit, int_none, is set only
// when the two rectangles do not intersect at all
// the lowest bit, int_lr, indicates whether the right side of the first
// argument is to the right of the right side of the second argument
// likewise int_rl indicates whether the left side of the first is to the
// right of the second's, int_tb indicates the first's top is below the second's,
// and int_bt indicates the first's bottom is below the second's
// int_inside and int_outside are supplied for convenience
enum {int_ll = 0,int_lr = 1,int_rl = 2, int_rr = 3, int_x = 3,
	int_tt = 0, int_tb = 4, int_bt = 8, int_bb = 12, int_y = 12,
	int_none=16,
	int_inside = int_rl | int_tb, // first arg inside second
	int_outside = int_lr | int_bt};
inline int intersectType(const RECT &sub,const RECT &ob) {
	if(!(sub&&ob))
		return int_none;
	return (sub.right > ob.right?int_lr:0)|(sub.left > ob.left?int_rl:0)|
		(sub.top > ob.top?int_tb:0)|(sub.bottom > ob.bottom?int_bt:0);
}
inline int intersectType(const RECTL &sub,const RECTL &ob) {
	if(!(sub&&ob))
		return int_none;
	return (sub.right > ob.right?int_lr:0)|(sub.left > ob.left?int_rl:0)|
		(sub.top < ob.top?int_tb:0)|(sub.bottom < ob.bottom?int_bt:0);
}

inline POINT ctr(RECT r) {
	POINT rv = {(r.right+r.left)/2,(r.top+r.bottom)/2};
	return rv;
}
inline POINTL ctr(RECTL r) {
	POINTL rv = {(r.right+r.left)/2,(r.top+r.bottom)/2};
	return rv;
}

// MS PtInRect doesn't recognize up-positive coords
inline bool cartesianPtInRect(const RECTL *r,POINTL pt) {
	if((pt.y<r->bottom)||(pt.y>r->top))
		return false;
	if((pt.x<r->left)||(pt.x>r->right))
		return false;
	return true;
}
inline bool smartPtInRect(const RECT *pcr,POINT pt) {
	if(pcr->top>pcr->bottom) {
		if((pt.y<pcr->bottom)||(pt.y>pcr->top))
			return false;
	}
	else {
		if((pt.y<pcr->top)||(pt.y>pcr->bottom))
			return false;
	}
	// don't see any need to support left-positive, though!
	if((pt.x<pcr->left)||(pt.x>pcr->right))
		return false;
	return true;
}
// general stuff

// related types
inline POINTL pointl(long x,long y) {
	POINTL ret = {x,y};
	return ret;
}
inline POINT point(int x, int y) {
	POINT ret = {x,y};
	return ret;
}
inline SIZE size(int x,int y) {
	SIZE ret = {x,y};
	return ret;
}
inline long mag2(SIZE s) {
	return s.cx*s.cx + s.cy*s.cy;
}
inline SIZE operator |(SIZE s1,SIZE s2) { // union
	return size(max(s1.cx,s2.cx),max(s1.cy,s2.cy));
}
inline POINT &point(POINTL &p) {
	return *(POINT*)&p;
}
inline POINT MAKEPOINT(LPARAM l) {
	return point(MAKEPOINTS(l).x,MAKEPOINTS(l).y);
}
inline POINTL &pointl(POINT &p) {
	return *(POINTL*)&p;
}
inline RECT &rect(RECTL &r) {
	return *(RECT*)&r;
}
inline RECTL &rectl(RECT &r) {
	return *(RECTL*)&r;
}
inline RECT rect(SIZE s) {
	RECT ret = {0,0,s.cx,s.cy};
	return ret;
}
inline RECTL rectl(SIZE s) {
	RECTL ret = {0,s.cy,s.cx,0};
	return ret;
}
inline SIZE operator *(SIZE s,long l) {
	return size(s.cx*l,s.cy*l);
}
inline SIZE operator /(SIZE s,long l) {
	return size(s.cx/l,s.cy/l);
}
// equality, add/subtract
#define P POINT
#define X x
#define Y y
#include "pt_eq.h"
#include "point2.h"
#undef P
#define P POINTL
#include "pt_eq.h"
#include "point2.h"
#undef P
#undef X
#undef Y
#define P SIZE
#define X cx
#define Y cy
#define P_IS_SIZE
#include "pt_eq.h"
#include "point2.h"
#undef P
#undef X
#undef Y
#undef P_IS_SIZE

/* ah for some prettier solution such as
template<class A,class B>
inline A operator +=(A &a,B b) {
	return a = a + b;
}
but that interferes... */
#define PLUSEQUALOP(T1,T2) inline T1 &operator +=(T1 &a,const T2 &b) { return a = a + b; }
#define MINUSEQUALOP(T1,T2) inline T1 &operator -=(T1 &a,const T2 &b) { return a = a - b; }
#define DIVEQUALOP(T1,T2) inline T1 &operator /=(T1 &a,const T2 &b) { return a = a / b; }
#define MULTEQUALOP(T1,T2) inline T1 &operator *=(T1 &a,const T2 &b) { return a = a * b; }

PLUSEQUALOP(RECT,RECT)
MINUSEQUALOP(RECT,RECT)
PLUSEQUALOP(RECTL,RECTL)
MINUSEQUALOP(RECTL,RECTL)
PLUSEQUALOP(RECT,POINT)
PLUSEQUALOP(RECT,POINTL)
PLUSEQUALOP(RECT,SIZE)
MINUSEQUALOP(RECT,SIZE)
PLUSEQUALOP(RECTL,POINT)
PLUSEQUALOP(RECTL,POINTL)
PLUSEQUALOP(RECTL,SIZE)
MINUSEQUALOP(RECTL,SIZE)
PLUSEQUALOP(POINT,SIZE)
MINUSEQUALOP(POINT,SIZE)
PLUSEQUALOP(POINTL,SIZE)
MINUSEQUALOP(POINTL,SIZE)
PLUSEQUALOP(SIZE,SIZE)
MINUSEQUALOP(SIZE,SIZE)
PLUSEQUALOP(POINT,POINT)
MULTEQUALOP(POINT,int)
DIVEQUALOP(POINT,int)
PLUSEQUALOP(POINTL,POINTL)
MULTEQUALOP(POINTL,int)
DIVEQUALOP(POINTL,int)

#endif