/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <comcat.h>
#include <objsafe.h>
HRESULT CreateComponentCategory(CATID catid, LPCOLESTR catDescription)
	{

	ICatRegister* pcr = NULL ;
	HRESULT hr = S_OK ;

	hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
			NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
	if (FAILED(hr))
		return hr;

	// Make sure the HKCR\Component Categories\{..catid...}
	// key is registered
	CATEGORYINFO catinfo;
	catinfo.catid = catid;
	catinfo.lcid = 0x0409 ; // english

	// Make sure the provided description is not too long.
	// Only copy the first 127 characters if it is
	int len = wcslen(catDescription);
	if (len>127)
		len = 127;
	wcsncpy(catinfo.szDescription, catDescription, len);
	// Make sure the description is null terminated
	catinfo.szDescription[len] = '\0';

	hr = pcr->RegisterCategories(1, &catinfo);
	pcr->Release();

	return hr;
}
HRESULT RegisterCLSIDInCategory(REFCLSID clsid, CATID catid)
	{
// Register your component categories information.
	ICatRegister* pcr = NULL ;
	HRESULT hr = S_OK ;
	hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
			NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
	if (SUCCEEDED(hr))
	{
	   // Register this category as being "implemented" by
	   // the class.
	   CATID rgcatid[1] ;
	   rgcatid[0] = catid;
	   hr = pcr->RegisterClassImplCategories(clsid, 1, rgcatid);
	}

	if (pcr != NULL)
		pcr->Release();

	return hr;
}
HRESULT UnRegisterCLSIDInCategory(REFCLSID clsid, CATID catid)
	{
    ICatRegister* pcr = NULL ;
    HRESULT hr = S_OK ;
    hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
			NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (SUCCEEDED(hr))
    {
       // Unregister this category as being "implemented" by
       // the class.
       CATID rgcatid[1] ;
       rgcatid[0] = catid;
       hr = pcr->UnRegisterClassImplCategories(clsid, 1, rgcatid);
    }

    if (pcr != NULL)
        pcr->Release();

	return hr;
}
HRESULT RegisterControlStdCategories(REFCLSID clsid) {
	HRESULT hr;
	hr = CreateComponentCategory(CATID_Insertable, L"Embeddable objects");
	if(FAILED(hr)) return hr;
	hr = CreateComponentCategory(CATID_Control, L"Controls");
	if(FAILED(hr)) return hr;
	hr = CreateComponentCategory(CATID_Programmable, L"Automation objects");
	if(FAILED(hr)) return hr;
	hr = CreateComponentCategory(CATID_SafeForScripting, L"Controls that are safely scriptable");
	if(FAILED(hr)) return hr;
	hr = CreateComponentCategory(CATID_SafeForInitializing, L"Controls safely initializable from persistent data");
	if(FAILED(hr)) return hr;

    ICatRegister* pcr = NULL ;
    hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
			NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (SUCCEEDED(hr)) {
		CATID rgCat[5];
		rgCat[0] = CATID_Insertable;
		rgCat[1] = CATID_Control;
		rgCat[2] = CATID_Programmable;
		rgCat[3] = CATID_SafeForScripting;
		rgCat[4] = CATID_SafeForInitializing;
        hr = pcr->RegisterClassImplCategories(clsid, 5, rgCat);
	}

    if (pcr != NULL)
        pcr->Release();

	return hr;
}
HRESULT UnRegisterControlStdCategories(REFCLSID clsid) {
    ICatRegister* pcr = NULL ;
    HRESULT hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
			NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (SUCCEEDED(hr)) {
		CATID rgCat[5];
		rgCat[0] = CATID_Insertable;
		rgCat[1] = CATID_Control;
		rgCat[2] = CATID_Programmable;
		rgCat[3] = CATID_SafeForScripting;
		rgCat[4] = CATID_SafeForInitializing;
        hr = pcr->UnRegisterClassImplCategories(clsid, 5, rgCat);
	}

    if (pcr != NULL)
        pcr->Release();

	return hr;
}