/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"

// saves a subject, marking its size first, in preparation for caching it on load below
HRESULT saveWithSize(IStream *stream,IPersistStream *object,BOOL fClearDirty) {
    FUNTRACE(saveWithSize);
	ULONGLONG size,start,finish;
	size = 0;
	// remember where we are
	SRUN(stream->Seek(castLarge(size),STREAM_SEEK_CUR,&castULarge(start)));
	// write dummy int where we'll save the size
	SRUN(stream->Write(&size,sizeof(__int64),0));
	SRUN(object->Save(stream,fClearDirty));
	// calculate size
	SRUN(stream->Seek(castLarge(size),STREAM_SEEK_CUR,&castULarge(finish)));
	size = finish - start - sizeof(__int64);
	// go back and write that size
	SRUN(stream->Seek(castLarge(start),STREAM_SEEK_SET,0));
	SRUN(stream->Write(&size,sizeof(__int64),0));
	// and return to after data
	SRUN(stream->Seek(castLarge(finish),STREAM_SEEK_SET,0));
	return S_OK;
}
// often you have to save before you've realized the subject.
HRESULT writeCache(IStream *dest, IStream *source) {
    FUNTRACE(writeCache);
	ULONGLONG writ;
	// save space for size
	SRUN(dest->Seek(castLarge(sizeof(ULONGLONG)),STREAM_SEEK_CUR,0));
	// copy data
	SRUN(source->CopyTo(dest,castULarge(0xffffffffffffffff),0,&castULarge(writ)));
	// rewind to empty space
	SRUN(dest->Seek(castLarge(-LONGLONG(writ)-sizeof(ULONGLONG)),STREAM_SEEK_CUR,0));
	// write size
	SRUN(dest->Write(&writ,sizeof(ULONGLONG),0));
	// return to end of dest
	SRUN(dest->Seek(castLarge(writ),STREAM_SEEK_CUR,0));
	// reset cache (should be default state)
	SRUN(source->Seek(castLarge(0),STREAM_SEEK_SET,0));
	return S_OK;
}

// loads data that's size-prefixed into a memory-based stream.
HRESULT cacheStreamableObject(IStream *stream, IStream **ret) {
    FUNTRACE(cacheStreamableObject);
	if(!ret)
		SERR(E_POINTER);
	ULARGE_INTEGER size;
	SRUN(stream->Read(&size,sizeof(ULARGE_INTEGER),0));
	HGLOBAL hMem = GlobalAlloc(GHND,ULONG(size.QuadPart));
	void *pv = GlobalLock(hMem);
	SRUN(stream->Read(pv,ULONG(size.QuadPart),0));
	GlobalUnlock(hMem);
	SRUN(CreateStreamOnHGlobal(hMem,true,ret));
	return S_OK;
}
