/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// Makes an enumerator looks like an stl container.
// not quite safe:
// - it only supports one normal iterator + the end() iterator
// - getting the begin() iterator resets the enumerator (invalidating any other iterator)
// - values returned are not valid after the iterator has been incremented

// seeing as I only intend to use this to make for() loops more manageable,
// these are reasonable constraints.  Perhaps I could make this a bit safer
// by rephrasing as an STL "input iterator."

struct hrException {
	HRESULT hr;
	hrException(HRESULT hr) : hr(hr) {};
};
// T is the item type
// I is the enumerator interface type
// C is the copy class type (as with ATL enumerators),
// only used to init &destroy items
template<class T, class I, class C>
struct EnumIter;
template<class T, class I, class C>
struct EnumCtr {
	typedef EnumIter<T,I,C> iterator;
	T buf[5];
	ULONG n;
	CComPtr<I> m_object;

	void init() {
		for(int i=0;i<5;i++)
			C::init(buf+i);
	}
	void destroy() {
		for(int i = 0;i<n;i++)
			C::destroy(buf+i);
		n = 0;
	}
	void fetch() {
		destroy();
		init();
		HRESULT hr = m_object->Next(5,buf,&n);
		if(FAILED(hr))
			throw hrException(hr);
	}

	EnumCtr(I *o) {
		n = 0;
		init();
		m_object = o;
	}
	~EnumCtr() {
		destroy();
	}

	EnumIter<T,I,C> begin();
	EnumIter<T,I,C> end();
};
template<class T, class I, class C>
struct EnumIter {
	EnumCtr<T,I,C> *base;
	ULONG n;
	bool stopped;

	EnumIter(EnumCtr<T,I,C> *base,int n = 0, bool stopped = false) : base(base),n(n),stopped(stopped) {}
	// these don't work except comparing end and another iter
	// ... you're not allowed to have 2 iterators on the same enumerator...
	bool operator ==(EnumIter<T,I,C> &e2) {
		return e2.stopped==stopped;
	}
	bool operator !=(EnumIter<T,I,C> &e2) {
		return e2.stopped != stopped;
	}
	T &operator *() {
		if(stopped)
			throw hrException(E_POINTER);
		return base->buf[n];
	}
	T *operator ->() {
		return &**this;
	}
	EnumIter<T,I,C> & operator ++() {
		if(n==4) {
			base->destroy();
			base->fetch();
			n = 0;
			if(base->n==0)
				stopped = true;
		}
		else if(++n==base->n)
			stopped = true;
		return *this;
	}
};

template<class T, class I, class C>
EnumIter<T,I,C> EnumCtr<T,I,C>::begin() {
	METHTRACE(EnumIter,begin);
	HRESULT hr;
	hr = m_object->Reset();
	if(FAILED(hr))
		throw hrException(hr);
	fetch();
	if(n==0)
		return end();
	else return EnumIter<T,I,C>(this,0);
}
template<class T, class I, class C>
EnumIter<T,I,C> EnumCtr<T,I,C>::end() {
	METHTRACE(EnumIter,end);
	return EnumIter<T,I,C>(this,0,true);
}
