/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#pragma once

#define ATL_TRACE_CATEGORY atlTraceQI
#include <atlbase.h>
#if defined(_DEBUG) && defined(TRACE_ERRORS)
// trace returned errors. (these take constant names eg E_FAIL)
#define SERR(e) do {MTrace(ER)("%s(%d): returned " #e "(0x%x)\n",__FILE__,__LINE__,e); return e;} while(0)
#define ERR(e) do {MTrace(ER)("%s(%d): 0x%x returned " #e "(0x%x)\n",__FILE__,__LINE__,this,e); return e;} while(0)
// and errors that are just placed in hr
#define SHRERR(e) do {MTrace(ER)("%s(%d): returning " #e "(0x%x)\n",__FILE__,__LINE__,e); hr = e;} while(0)
#define HRERR(e) do {MTrace(ER)("%s(%d): x0%x returning " #e "(0x%x)\n",__FILE__,__LINE__,this,e); hr = e;} while(0)
// Just print an error (takes int)
inline HRESULT STraceErr(HRESULT hr, char *file, int line) {
	if(FAILED(hr))
		MTrace(ER)("%s(%d): error %x detected\n",file,line,hr);
	return hr;
}
inline HRESULT TraceErr(HRESULT hr, void const *that,char *file, int line) {
	if(FAILED(hr))
		MTrace(ER)("%s(%d): %p error %x detected\n",file,line,that,hr);
	return hr;
}
#define SLOOK(exp) STraceErr(exp,__FILE__,__LINE__)
#define LOOK(exp) TraceErr(exp,this,__FILE__,__LINE__)
#else
inline HRESULT STraceErr(HRESULT hr, char *file, int line) { return hr; }
inline HRESULT TraceErr(HRESULT hr, void const *that,char *file, int line) { return hr; }
#define SERR(e) do {return e;} while(0)
#define ERR(e) do {return e;} while(0)
#define SHRERR(e) do {hr = e;} while(0)
#define HRERR(e) do {hr = e;} while(0)
#define SLOOK(exp) (exp)
#define LOOK(exp) (exp)
#endif
