/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "ezMenu.h"

MenuItem::MenuItem() {
	cbSize = sizeof(MENUITEMINFO);
	fMask = MIIM_ID|MIIM_TYPE;
	fType = MFT_STRING;
	fState = MFS_ENABLED;
	hSubMenu = NULL;
	hbmpChecked = NULL;
	hbmpUnchecked = NULL;
	dwItemData = 0;
}
void MenuItem::insert(HMENU hMenu,const char *s,UINT id, int where,HMENU menu,bool radioCheck) {
	wID = id;
	dwTypeData = const_cast<char*>(s);
	cch = _tcslen(s);
	if(where==-1)
		where = GetMenuItemCount(hMenu);
	if(menu) {
		hSubMenu = menu;
		fMask |= MIIM_SUBMENU;
	}
	if(radioCheck)
		fType |= MFT_RADIOCHECK;
	/*
	if(checked||unchecked) {
		hbmpChecked = checked;
		hbmpUnchecked = unchecked;
		fMask |= MIIM_CHECKMARKS;
	}
	*/
	CHECKR(InsertMenuItem(hMenu,where,true,this));
	fMask &= ~(MIIM_SUBMENU|MIIM_CHECKMARKS);
	fType &= ~MFT_RADIOCHECK;
	hSubMenu = 0;
	//hbmpChecked = hbmpUnchecked = 0;
}
void MenuItem::insertSep(HMENU hMenu,UINT id,int where) {
	wID = id;
	dwTypeData = 0;
	fType = MFT_SEPARATOR;
	if(where==-1)
		where = GetMenuItemCount(hMenu);
	CHECKR(InsertMenuItem(hMenu,where,true,this));
	fType = MFT_STRING;
}
pair<HMENU,int> findMenu(LPTSTR name,HMENU hMenu,int begin,int end) {
	_ASSERTE(hMenu);
	MENUITEMINFO minfo;
	minfo.cbSize = sizeof(MENUITEMINFO);
	minfo.fMask = MIIM_TYPE|MIIM_SUBMENU;
	TCHAR buf[200];
	int len = _tcslen(name);
	for(int i=begin;i<end;i++) {
		minfo.dwTypeData = buf;
		minfo.cch = 200;
		SCHECK(GetMenuItemInfo(hMenu,i,true,&minfo));
		if(!_tcsncmp(minfo.dwTypeData,name,len)) {
			return make_pair(minfo.hSubMenu,i);
		}
	}
	return make_pair(HMENU(0),0);
}
pair<int,int> range(HMENU hMenu,OLEMENUGROUPWIDTHS *widths,int sec) {
	// FIX for excel: has extra entry before &File for window menu which is not accounted for in widths
	// so search out &File for offset
	int n = GetMenuItemCount(hMenu);
	MENUITEMINFO getS;
	getS.cbSize = sizeof getS;
	getS.fMask = MIIM_TYPE;
	TCHAR buf[200];
	for(int i = 0; i<n; ++i) {
		getS.dwTypeData = buf;
		getS.cch = 200;
		GetMenuItemInfo(hMenu,i,true,&getS);
		if(getS.fType==MFT_STRING&&!strcmp(getS.dwTypeData,"&File"))
			break;
	}
	int begin = (i<n)?i:0,end;
	for(i = 0;i<sec;i++)
		begin += widths->width[i];
	end = begin + widths->width[sec];
	return make_pair(begin,end);
}
HMENU findOrAddMenu(LPTSTR name,HMENU hMenu,OLEMENUGROUPWIDTHS *widths,int section,bool atEnd) {
	pair<int,int> sect = range(hMenu,widths,section);
	HMENU ret = findMenu(name,hMenu,sect.first,sect.second).first;
	if(ret)
		return ret;
	ret = CreateMenu();
	MENUITEMINFO minf;
	minf.cbSize = sizeof minf;
	minf.fMask = MIIM_SUBMENU|MIIM_TYPE;
	minf.fType = MFT_STRING;
	minf.hSubMenu = ret;
	minf.dwTypeData = name;
	minf.cch = strlen(name);
	int pos = atEnd?sect.second:sect.first;
	InsertMenuItem(hMenu,pos,TRUE,&minf);
	widths->width[section]++;
	return ret;
}
bool removeEmptyMenu(LPTSTR name,HMENU hMenu,OLEMENUGROUPWIDTHS *widths,int section) {
	pair<int,int> sect = range(hMenu,widths,section);
	pair<HMENU,int> found = findMenu(name,hMenu,sect.first,sect.second);
	if(!found.first)
		return false;
	if(GetMenuItemCount(found.first))
		return false;
	DeleteMenu(hMenu,found.second,MF_BYPOSITION);
	return true;
}
