/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


/*
	fireEvent: fires an event on an object (need not be yours)
Func must have these members:
	const IID &getIID();
	HRESULT operator()(const CONNECTDATA&);
Calling fireEvent is a little inefficient because it causes all sorts of copying
etc.  But it is more safe than firing an event directly for the same reason: you
can never be sure that a sink isn't going to disconnect itself in response to the
event, corrupting the array you're cycling through.

fireFunctors defines Func for 0 and 1 parameter events.
*/

#include "getSinks.h"
template<class Func>
inline HRESULT fireEventOnEC(IEnumConnections *ec,Func f) {
	for(connection_iterator i(ec);i!=connection_iterator();++i)
		SRUNRET(f(*i));
	return S_OK;
}
template<class Func>
inline HRESULT fireEvent(IUnknown *u,Func f) {
 	return fireEventOnEC(getSinks(u,f.getIID()),f);
}

