/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef fireFunctors_h
#define fireFunctors_h

template<class I,const IID *piid = &__uuidof(I)>
struct fireVoid {
	typedef HRESULT (__stdcall I::*func)(void);
	func f;
	fireVoid(func f) : f(f) {}
	const IID &getIID() { return *piid;}
	HRESULT operator()(const CONNECTDATA &cd) {
        METHTRACE(fireVoid,parens);
		CComPtr<I> dest;
		RUN(cd.pUnk->QueryInterface(*piid,(void**)&dest));
		return (dest->*f)();
	}
};
template<class I,class P,const IID *piid = &__uuidof(I)>
struct fireOneParam {
	typedef HRESULT (__stdcall I::*func)(P);
	func f;
	P param;
	fireOneParam(func f,P param) : f(f),param(param) {}
	const IID &getIID() { return *piid;}
	HRESULT operator()(const CONNECTDATA &cd) {
        METHTRACE(fireOneParam,parens);
		CComPtr<I> dest;
		RUN(cd.pUnk->QueryInterface(*piid,(void**)&dest));
		return (dest->*f)(param);
	}
};
template<class I,typename P1,typename P2,const IID *piid = &__uuidof(I)>
struct fireTwoParams {
	typedef HRESULT (__stdcall I::*func)(P1,P2);
	func f;
	P1 param1;
	P2 param2;
	fireTwoParams(func f,P1 param1,P2 param2) : f(f),param1(param1),param2(param2) {}
	const IID &getIID() { return *piid;}
	HRESULT operator()(const CONNECTDATA &cd) {
        METHTRACE(fireTwoParams,parens);
		CComPtr<I> dest;
		RUN(cd.pUnk->QueryInterface(*piid,(void**)&dest));
		return (dest->*f)(param1,param2);
	}
};
template<class I,typename P1,typename P2,typename P3,const IID *piid = &__uuidof(I)>
struct fireThreeParams {
	typedef HRESULT (__stdcall I::*func)(P1,P2,P3);
	func f;
	P1 param1;
	P2 param2;
	P3 param3;
	fireThreeParams(func f,P1 param1,P2 param2, P3 param3) : f(f),param1(param1),param2(param2),param3(param3) {}
	const IID &getIID() { return *piid;}
	HRESULT operator()(const CONNECTDATA &cd) {
        METHTRACE(fireThreeParams,parens);
		CComPtr<I> dest;
		RUN(cd.pUnk->QueryInterface(*piid,(void**)&dest));
		return (dest->*f)(param1,param2,param3);
	}
};

#endif fireFunctors_h