/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#ifndef fireSiteEvent_h
#define fireSiteEvent_h

#include "getSinks.h"

// Func is defined as in fireEvent.h
template<class Func>
HRESULT fireSiteEvent(IMCCSite *s,Func f) {
    FUNTRACE(fireSiteEvent);
	HRESULT hr=S_FALSE;
	// if the site is deleted it has no sinks so don't even try.
	long intent;
	SRUN(s->get_Intent(&intent));
	if(intent==MCCS_DELETED)
		return S_FALSE;
	// try direct sinks
	bool done = false;
	for(connection_iterator i(getSinks(s,f.getIID()));i!=connection_iterator();++i) {
		SRUNRET(hr = f(*i));
		if(hr==S_OK) // if anyone sez definitively "yes, I processed" we don't have to try defaults
			done = true;
	}
	// no answer yet: try default sinks.
	if(!done) {
		CComPtr<IMontageContainer> mc;
		SRUN(s->GetContainer(riidret(&mc)));
		CComPtr<IMCCAllPointsContainer> defaultSinks;
		SRUN(mc->get_DefaultSiteSinks(&defaultSinks));
		for(connection_iterator i(getSinks(defaultSinks,f.getIID()));i!=connection_iterator();++i) {
			if(CComQIPtr<IDefaultSink> ds = i->pUnk)
				ds->StartSink(s);
			SRUNRET(hr = f(*i));
		}
	}
	return hr;
}

#endif fireSiteEvent_h