/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// wraps a forward iterator in an enumerator
// note this is completely unsafe if anything happens to the list while
// the iterator's still around, but I can't resist...
// I think it's possible to create safe set and map enumerators & I should replace
// these.  But for now these are good enough.
// use IterEnum (not iterEnum); it's the full COM object and it's got a handy constructor
template<class IFace, const IID* piid, class T, class Iter, class Copy>
struct iterEnum :
	public CComObjectRoot,
	public IFace
{
	typedef iterEnum<IFace,piid,T,Iter,Copy> _iterEnum;

	Iter i,begin,end;
	iterEnum() {}
BEGIN_COM_MAP(_iterEnum)
	COM_INTERFACE_ENTRY_IID(*piid,IFace)
END_COM_MAP()

	STDMETHODIMP Next(ULONG celt, T *rgelt, ULONG * pceltFetched);
	STDMETHODIMP Skip(ULONG celt);
	STDMETHODIMP Reset(void);
	STDMETHODIMP Clone(IFace ** ppenum);
};
// aside from the constructor, this is basically a templated typedef (which you
// can't do directly).  Learned this trick from Stroustrop's "C++ design and evolution"
// (which is a lot of fun if you dig "topological" concerns like this)
template<class IFace, const IID* piid, class T, class Iter, class Copy>
struct IterEnum : public CComObject<iterEnum<IFace, piid, T, Iter, Copy> >
{
	IterEnum(Iter _begin, Iter _end, Iter _i) {
		begin = _begin;
		end = _end;
		i = _i;
	}
};
template<class IFace, const IID* piid, class T, class Iter, class Copy>
STDMETHODIMP iterEnum<IFace,piid,T,Iter,Copy>::Next(ULONG celt, T *rgelt, ULONG * pceltFetched) {
	METHTRACE(iterEnum,Next);
	T *head = rgelt;
	if(pceltFetched)
		*pceltFetched = 0;
	else if(celt!=1)
		ERR(E_INVALIDARG);
	celt++;
	while(--celt && i!=end) {
		// some ADTs (I ran into hash_set) return const &s;  the second param of
		// ATL's copy functions should be const but it's not.  thus the const_cast
		Copy::copy(head++, const_cast<T*>(&*i)); i++;
		if(pceltFetched)
			(*pceltFetched)++;
	}
	return celt?S_FALSE:S_OK;
}
template<class IFace, const IID* piid, class T, class Iter, class Copy>
STDMETHODIMP iterEnum<IFace,piid,T,Iter,Copy>::Skip(ULONG celt) {
	METHTRACE(iterEnum,Skip);
	while(celt--)
		if(i==end)
			return S_FALSE;
		else i++;
	return S_OK;
}
template<class IFace, const IID* piid, class T, class Iter, class Copy>
STDMETHODIMP iterEnum<IFace,piid,T,Iter,Copy>::Reset(void) {
	METHTRACE(iterEnum,Reset);
	i = begin;
	return S_OK;
}
template<class IFace, const IID* piid, class T, class Iter, class Copy>
STDMETHODIMP iterEnum<IFace,piid,T,Iter,Copy>::Clone(IFace ** ppenum) {
	METHTRACE(iterEnum,Clone);
	(*ppenum = new IterEnum<IFace,piid,T,Iter,Copy>(begin,end,i))->AddRef();
	return S_OK;
}
