/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
// (which must include all dgm idl .h's)
#include "err.h"
#define ATL_TRACE_CATEGORY atlTraceQI
#include "atlbase.h"

CComPtr<IMCCSite> siteFromLayoutConnections(IEnumConnections *ec) {
	connection_iterator ic(ec);
	_ASSERTE(ic!=connection_iterator());
	CComPtr<IMCCSite> ret;
	for(;ic!=connection_iterator();++ic)
		if(CComQIPtr<DIDGMConnector> conn = ic->pUnk) {
			CComQIPtr<IObjectWithSite>(conn)->GetSite(riidret(&ret));
			break;
		}
	return ret;
}
CComPtr<IMCCSite> siteFromNode(IDGDrawnNode *n) {
	return siteFromLayoutConnections(getSinks(n,IID_IDGNodeShadow));
}
CComPtr<IMCCSite> siteFromEdge(IDGDrawnEdge *e) {
	return siteFromLayoutConnections(getSinks(e,IID_IDGEdgeShadow));
}
CComPtr<IMCCSitePersistor> sitePersistor(IMCCSite *s) {
    FUNTRACE(sitePersistor);
	CComPtr<IMoniker> mkSP;
	SRUN(s->get_RelMoniker(&mkSP));
	if(!mkSP)
		return 0;
	// context from site:
	CComPtr<IMontageContainer> mc;
	SRUN(s->GetContainer(riidret(&mc)));
	CComPtr<IMCCSitePersistor> ret;
	SRUN(mkSP->BindToObject(BindCtx(),PointerMoniker(mc),riidret(&ret)));
	return ret;
}
CComPtr<IMoniker> mkConnectorFromSitePersistor(IMCCSitePersistor *sp) {
    FUNTRACE(mkConnectorFromSitePersistor);
	// the persistor of the node connector must be on the init list of the site's persistor, at
	// priority 20 (see Drawer::NewNode).  And that persistor is holding the moniker
	// we want.
	CComPtr<IOrderedItemList> oil;
	SRUN(sp->get_InitList(&oil));

	CComPtr<IMoniker> mkConnector;
	SRUNRET(oil->GetMoniker(20,&mkConnector));
	return mkConnector;
}
CComPtr<DIPersistor> connectorPersistorFromMkConnector(IMoniker *mkConnector,IMoniker *mkGroup) {
    FUNTRACE(connectorPersistorFromMkConnector);
	if(!mkConnector)
		return 0;
	// I happen to know that the GetSubjectMoniker uses anti-monikers as its inverse
	CComPtr<IMoniker> anti;
	SRUN(CreateAntiMoniker(&anti));
	CComPtr<IMoniker> trimmed; // w/o GetSubject moniker on end
	SRUN(mkConnector->ComposeWith(anti,false,&trimmed));
	CComPtr<DIPersistor> per;
	SRUN(trimmed->BindToObject(BindCtx(),mkGroup,riidret(&per)));
	// check to make sure it's the right persistor by seeing
	// if its subject supports DIDGMConnector
	CComPtr<IUnknown> uc;
	SRUN(per->get_Subject(&uc));
	CComPtr<DIDGMConnector> c;
	if(FAILED(uc.QueryInterface(&c)))
		per.Release();
	return per;
}
CComPtr<IMoniker> mkLayoutFromConnectorPersistor(IUnknown *per) {
    FUNTRACE(mkLayoutFromConnectorPersistor);
	CComPtr<IMoniker> ret;
	if(CComQIPtr<IHoldOnePointer> hop = per)
		SRUN(hop->get_Moniker(&ret));
	else if(CComQIPtr<IHoldSomePointers> hsp = per) {
		CComPtr<IEnumMoniker> em;
		SRUN(hsp->Enum(&em));
		SRUN(em->Next(1,&ret,0));
	}
	return ret;
}
CComPtr<IMoniker> mkLayoutFromSitePersistor(IMCCSitePersistor *sp,IMontageContainer *mc) {
    FUNTRACE(mkLayoutFromSitePersistor);
	CComPtr<IMoniker> mkConn = mkConnectorFromSitePersistor(sp);
	if(!mkConn)
		return 0;
	return mkLayoutFromConnectorPersistor(connectorPersistorFromMkConnector(mkConn,PointerMoniker(mc)));
}
CComPtr<IMoniker> mkLayoutFromSite(IMCCSite *s) {
    FUNTRACE(mkLayoutFromSite);
	CComPtr<IMontageContainer> mc;
	SRUN(s->GetContainer(riidret(&mc)));
	return mkLayoutFromSitePersistor(sitePersistor(s),mc);
}