/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


typedef LPOLESTR (*strFun)();
struct GRM_REGMAP_ENTRY {
	const OLECHAR *key;
	enum {isString,isClass,isFun} type;
	const void *dat; // CLSID* if isClass, LPOLESTR if isString, strFun if isFun
};
// keys are const, values are allocd.  whole map is allocd too
struct allocd_regmap {
	_ATL_REGMAP_ENTRY *map;
	allocd_regmap(int size) {
		map = (_ATL_REGMAP_ENTRY*)CoTaskMemAlloc(sizeof(_ATL_REGMAP_ENTRY)*(size+1));
		memset(map,0,sizeof(_ATL_REGMAP_ENTRY)*(size+1));
	}
	~allocd_regmap() {
		for(int i = 0;map[i].szKey;++i)
			;//CoTaskMemFree(const_cast<LPOLESTR>(map[i].szData));
		CoTaskMemFree(map);
	}
};

inline allocd_regmap *grm2arm(GRM_REGMAP_ENTRY *en,int n) {
	allocd_regmap *ret = new allocd_regmap(n);
	for(int i = 0;i<n;++i) {
		ret->map[i].szKey = en[i].key;
		switch(en[i].type) {
			case GRM_REGMAP_ENTRY::isClass:
				StringFromCLSID(*(CLSID*)en[i].dat,const_cast<LPOLESTR*>(&ret->map[i].szData));
				break;
			case GRM_REGMAP_ENTRY::isString:
				ret->map[i].szData = wcsdup((LPOLESTR)en[i].dat);
				break;
			case GRM_REGMAP_ENTRY::isFun:
				ret->map[i].szData = strFun(en[i].dat)();
				break;
			default: _ASSERTE(0);
		}
	}
	return ret;
}
// somewhat inefficient: copies all string values
#define BEGIN_REGMAP() static inline allocd_regmap *getRegMap() { GRM_REGMAP_ENTRY start[] = {
#define REGMAP_STRING(key,value) {key,GRM_REGMAP_ENTRY::isString,value},
#define REGMAP_GUID(key,clsid) {key,GRM_REGMAP_ENTRY::isClass,&clsid},
#define REGMAP_FUN(key,fun) {key,GRM_REGMAP_ENTRY::isFun,fun},
#define END_REGMAP() }; return grm2arm(start,sizeof(start)/sizeof(GRM_REGMAP_ENTRY));}
