/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"

HRESULT siteControl(IBindCtx *bctx,IMoniker *mkEmbed,IMoniker *mkSite,IMCCLayer *layer,MCC_POS_INFO *mpi,int intent,
					int nInits,IMoniker **inits,int *initPriorities,IMoniker **retMkSP,IMCCSitePersistor **ret) {
    FUNTRACE(siteControl);
	CComPtr<IMontageContainer> mc;
	SRUN(layer->GetContainer(riidret(&mc)));
	CComPtr<IGroup> group;
	SRUN(mc->GetMainGroup(riidret(&group)));
	// get all the monikers we need for the site persistor
	CComPtr<IMoniker> mkMontage,mkLayerP,goSubject,mkLayer,mkSiteOrdering;
	SRUN(mc->get_RelMoniker(&mkMontage));
	SRUN(layer->get_RelMoniker(&mkLayerP));
	SRUN(CREATE(goSubject,CLSID_GetSubjectMoniker));
	SRUN(mkLayerP->ComposeWith(goSubject,false,&mkLayer));
	SRUN(layer->get_OrderMk(&mkSiteOrdering));

	// create a site persistor
	CComPtr<IMCCSitePersistor> sp;
	SRUN(CREATE(sp,CLSID_MCCSitePersistor));
	SRUN(CComQIPtr<IGroupItem>(sp)->Connect(group)); // redundant but necessary
	SRUN(sp->Initialize(mkMontage,mkLayer,mkSiteOrdering,mkEmbed,0,0));

	// fill in data
	MCCSITEPERSISTORDATA spd;
	spd.z = 0;
	if(mpi)
		spd.pos = *mpi;
	else
		spd.pos.postype = MCC_POS_NONE;
	spd.state = intent;
	SRUN(sp->put_Data(&spd));

	// initializations
	CComPtr<IOrderedItemList> oil;
	SRUN(sp->get_InitList(&oil));
	for(int i = 0;i<nInits;++i)
		SRUN(oil->SetMoniker(initPriorities[i],inits[i]));

	// paste the persistor
	CComPtr<IMoniker> mkSP;
	SRUN(group->Paste(bctx,GP_SrcIsObj,sp,mkSite,IID_NULL,0,&mkSP));
	if(retMkSP)
		mkSP.CopyTo(retMkSP);
	SRUN(sp->put_RelMoniker(mkSP));

	if(ret)
		(*ret = sp)->AddRef();
	return S_OK;
}