/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "MBasics.h"
#include "toolbars.h"
Toolness::Toolness(MBasics &mb,IMCCLayer *toolLayer,IMoniker *mkToolLayer,IMoniker *mkToolSitesOrdering,IMCCSiteLayoutEvents *toolDrawer)
		: mb(mb),toolLayer(toolLayer),mkToolLayer(mkToolLayer),mkToolSitesOrdering(mkToolSitesOrdering),toolDrawer(toolDrawer) {
    METHTRACE(Toolness,Toolness);
	if(!this->mkToolLayer)
		this->mkToolLayer = ComposeMk(ItemMoniker(L"Tool Layer"),GetSubjectMk());
	if(!this->mkToolSitesOrdering)
		this->mkToolSitesOrdering = ItemMoniker(L"Tool Site Ordering");
	if(!this->toolLayer)
		RUN(mb.bind(this->mkToolLayer,&this->toolLayer));
	if(!this->toolDrawer)
		RUN(mb.bind(ItemMoniker(L"Toolbar Drawer"),&this->toolDrawer));
}
HRESULT Toolness::createToolbar(LPOLESTR desc,const RECT &r,IUnknown *ctrl,IMoniker *mkCtrl,IMCCSite **retSite) {
    METHTRACE(Toolness,createToolbar);
	CComPtr<IMCCSite> site;
	RUN(mb.c->new_Site(&site));

	RUN(site->put_Rect(&r));
	RUN(site->put_Layer(toolLayer));
	RUN(site->put_Control(ctrl));
	RUN(site->put_Intent(MCCS_ACTIVE));

	// make site persistable
	CComPtr<IMCCSitePersistor> msp;
	RUN(CREATE(msp,CLSID_MCCSitePersistor));
	CComPtr<IMoniker> mkEingSiteP;
	OLECHAR buf[90];
	if(desc) {
		wcscpy(buf,desc);
		wcscat(buf,L" Site");
	}
	RUN(mb.add(desc?buf:0,msp,keep_configuration,&mkEingSiteP));
	RUN(msp->Initialize(mb.mkP,mkToolLayer,mkToolSitesOrdering,mkCtrl,0,0));
	RUN(CComQIPtr<DIPersistor>(msp)->SetSubject(true,site));
	RUN(site->put_RelMoniker(mkEingSiteP));

	RUN(toolDrawer->Process(msp,desc));

	if(retSite)
		RUN(site.CopyTo(retSite));
	return S_OK;
}
HRESULT Toolness::createMetabar(LPOLESTR desc,const RECT &r,IMontageContainer **retMeta,IMoniker **retMk) {
    METHTRACE(Toolness,createMetabar);
	CComPtr<IMontageContainer> metabar;
	RUN(metabar.CoCreateInstance(CLSID_MCCToolPanel));
	CComPtr<IInitializeOther> bootPanel;
	RUN(bootPanel.CoCreateInstance(CLSID_MCCBootPanel));
	RUN(metabar->put_Bootstrapper(bootPanel));
	CComPtr<IMoniker> mkMeta;
	RUN(mb.add(desc,metabar,keep_configuration,&mkMeta,true));

	// hack: as with palette, turn off autosizing because montage doesn't know how not to autosize
	RUN(metabar->put_AutoSize(VARIANT_FALSE));
	RUN(createToolbar(desc,r,metabar,mkMeta));

	if(retMeta)
		metabar.CopyTo(retMeta);
	if(retMk)
		mkMeta.CopyTo(retMk);
	return S_OK;
}