/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Frame.h"
#include "moreW2A.h"

#include "getPortFormats.h"

Frame::Frame() : m_wasFile(false), m_useFile(false) {
    CONSTRUCTORTRACE(Frame);
	m_border = rect(0,0,0,0);
	m_frameMenu = LoadMenu(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDR_FrameMenu));
}
Frame::~Frame() {
    DESTRUCTORTRACE(Frame);
	//DestroyMenu(m_frameMenu);
	if(m_hWnd)
		DestroyWindow();
}
LRESULT Frame::OnPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
    METHTRACE(Frame,OnPaint);
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);
	RECT rc = getRect();
	//GetClientRect(&rc);
	InflateRect(&rc,2,2);
	DrawEdge(hdc,&rc,EDGE_SUNKEN,BF_RECT);
	EndPaint(&ps);
	return 0;
}
LRESULT Frame::OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
    METHTRACE(Frame,OnSize);
	if(wParam==SIZE_MINIMIZED) // ignore minimization
		return 0;
	if(m_active) {
		RECT rc;
		GetClientRect(&rc);
		RUN(m_active->ResizeBorder(&rc,this,true));
	}
    if(m_view) {
		RECT rc = getRect();
		RUN(m_view->SetRect(&rc));
	}
	InvalidateRect(0,false);
	return 0;
}
LRESULT Frame::OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
    METHTRACE(Frame,OnDestroy);
	PostQuitMessage(0);
	return 0;
}
LRESULT Frame::OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
    METHTRACE(Frame,OnClose);
	if(SUCCEEDED(close()))
		DestroyWindow();
	return S_OK;
}
LRESULT Frame::OnSetFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled) {
    METHTRACE(Frame,OnSetFocus);
	if(m_content) {
		HWND itsWnd;
		RUN(CComQIPtr<IOleInPlaceObject>(m_content)->GetWindow(&itsWnd));
		::SetFocus(itsWnd);
	}
	return S_OK;
}
LRESULT Frame::FileNew(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileNew);
	if(FAILED(close()))
		return 0;
	RUN(startFromCLSID());
	return 0;
}
LRESULT Frame::FileOpen(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileOpen);
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = m_hWnd;
	ofn.hInstance = 0;
	ofn.lpstrFilter = m_inFormats.c_str();
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	char filename[MAX_PATH] = "";
	if(!m_filename.empty())
		AtlW2AHelper(filename,m_filename.c_str(),m_filename.length()+1);
	ofn.lpstrFile = filename;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFileTitle = 0;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = 0;
	ofn.lpstrTitle = "Open";
	ofn.Flags = 0;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	/*
	char ext[10];
	if(!m_inFormats.empty()) {
		int comma = m_defaultExt.find(',');
		_ASSERTE(comma!=string::npos);
		m_defaultExt.copy(ext,comma-1,1);
		ofn.lpstrDefExt = ext;
	}
	else
	*/
		ofn.lpstrDefExt = 0;
	ofn.lCustData = 0;
	ofn.lpfnHook = 0;
	ofn.lpTemplateName = 0;
	if(GetOpenFileName(&ofn)) {
		if(FAILED(close()))
			return 0;
		CComBSTR bstrFile(filename);
		RUN(StartFromFile(bstrFile));
	}
	return 0;
}
LRESULT Frame::FileClose(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileClose);
	close();
	RUN(setCaption());
	return 0;
}
LRESULT Frame::FileSave(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileSave);
	if(!m_content)
		return 0;
	if(m_useFile) {
		if(m_filename.empty())
			return FileSaveAs(wNotifyCode,wID,hWndCtl,bHandled);
		CComQIPtr<IPersistFile> pf(m_content);
		RUN(pf->Save(0,false));
		RUN(pf->SaveCompleted(m_filename.c_str()));
	}
	else {
		CComQIPtr<IPersistStorage> ps(m_content);
		RUN(ps->Save(m_storage,true));
		RUN(ps->SaveCompleted(0));
	}
	return 0;
}
LRESULT Frame::FileSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileSaveAs);
	if(!m_content)
		return 0;
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = m_hWnd;
	ofn.hInstance = 0;
	ofn.lpstrFilter = m_outFormats.c_str();
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	char filename[MAX_PATH] = "";
	if(!m_filename.empty())
		AtlW2AHelper(filename,m_filename.c_str(),m_filename.length());
	ofn.lpstrFile = filename;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFileTitle = 0;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = 0;
	ofn.lpstrTitle = "Save As";
	ofn.Flags = 0;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = 0;
	ofn.lCustData = 0;
	ofn.lpfnHook = 0;
	ofn.lpTemplateName = 0;
	if(GetSaveFileName(&ofn)) {
		char *namePart = strrchr(filename,'\\');
		_ASSERTE(namePart);
		namePart++;
		if(!strchr(namePart,'.'))
			if(ofn.nFilterIndex) {
				// find wildcard of filter index
				int n = (ofn.nFilterIndex-1)*2+1;
				for(char *i = &m_outFormats[0];n;i+=strlen(i)+1) n--;
				// and concat onto filename
				CHECK(*i++=='*');
				CHECK(*i++=='.');
				if(char *end = strchr(i,';'))
					sprintf(filename+strlen(filename),".%.*s",end-i,i);
				else
					sprintf(filename+strlen(filename),".%s",i);
			}
		// if it's same old file just Save
		USES_CONVERSION;
		if(!m_filename.empty() && !stricmp(filename,OLE2T(m_filename.c_str()))) {
			BOOL junk;
			return FileSave(0,0,0,junk);
		}
		// confirm replace
		FILE *f = fopen(filename,"r");
		if(f) {
			fclose(f);
			char buf[200];
			sprintf(buf,"Replace %s?",filename);
			if(MessageBox(buf,m_appName.c_str(),MB_YESNO)!=IDYES)
				return 0;
		}
		// always use IPersistFile to Save As, to allow exporting.  OLE dox claim you can must choose one
		// IPersist.. interface, but here it seems necessary to switch.
		CComQIPtr<IPersistFile> pf(m_content);
		RUN(pf->Save(T2OLE(filename),m_useFile)); // tell it to remember this filename if was created with StartFromClass
		RUN(pf->SaveCompleted(m_filename.c_str()));
		if(m_useFile) {
			m_filename = T2OLE(filename);
			RUN(setCaption());
		}
		/*
		else { // save a copy as
			CComPtr<IStorage> stg;
			HRESULT hr = StgCreateDocfile(T2OLE(filename),STGM_READWRITE|STGM_SHARE_EXCLUSIVE,0,&stg);
			if(hr==STG_E_FILEALREADYEXISTS) {
			}
			RUN(hr);
			CComQIPtr<IPersistStorage> ps(m_content);
			RUN(WriteClassStg(stg,m_clsid));
			RUN(ps->Save(stg,false));
			RUN(ps->SaveCompleted(0));
		}
		*/
		return 0;
	}
	else return 1;
}
HRESULT Frame::getFormats() {
	CRegKey classKey;
	OLECHAR classString[100];
	RUN(StringFromGUID2(m_clsid,classString,100));
	char classKeyName[200];
	sprintf(classKeyName,"CLSID\\%S",classString);
	CHECK(!classKey.Open(HKEY_CLASSES_ROOT,classKeyName));
	// OLE reg info is default for file format info
	char buf[200];
	DWORD count(200);
	CRegKey extKey;
	string defaultExt;
	if(!extKey.Open(classKey,"DefaultExtension")) {
		CHECK(!extKey.QueryValue(0,0,buf,&count));
		char *comma = strchr(buf,',');
		_ASSERTE(comma);
		defaultExt = comma+1;
		defaultExt.append(1,0);
		defaultExt.append(1,'*');
		defaultExt.append(buf);
		defaultExt.append(2,0);
	}

	// .. but try to read multiple formats from montage
	OLECHAR wfil[2000];
	int len = getPortFormats(m_content,false,wfil);
	if(len) {
		char afil[2000];
		W2AN(afil,2000,wfil,len);
		m_inFormats.assign(afil,len);
	}
	else
		m_inFormats = defaultExt;
	len = getPortFormats(m_content,true,wfil);
	if(len) {
		char afil[2000];
		W2AN(afil,2000,wfil,len);
		m_outFormats.assign(afil,len);
	}
	else
		m_outFormats = defaultExt;
	return S_OK;
}
LRESULT Frame::FileExit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled) {
    METHTRACE(Frame,FileExit);
	SendMessage(WM_CLOSE);
	return 0;
}
HRESULT Frame::setCaption() {
    METHTRACE(Frame,setCaption);
	if(m_filename.empty())
		m_caption = m_appName;
	else {
		char buf2[200];
		int lastSlash = m_filename.rfind('\\');
		sprintf(buf2,"%S - %s",m_filename.c_str()+lastSlash+1,m_appName.c_str());
		m_caption = buf2;
	}
	if(m_hWnd)
		SetWindowText(m_caption.c_str());
	return S_OK;
}
HICON getIcon(char *buf) {
	char *comma = strchr(buf,',');
	*comma = 0;
	int id = atoi(comma+1);
	HICON ike = ExtractIcon(_Module.GetModuleInstance(),buf,id);
	_ASSERTE(ike);
	return ike;
}
HRESULT Frame::show() {
    METHTRACE(Frame,show);
	USES_CONVERSION;
	CRegKey classKey;
	OLECHAR classString[100];
	RUN(StringFromGUID2(m_clsid,classString,100));
	char classKeyName[200];
	sprintf(classKeyName,"CLSID\\%S",classString);
	CHECK(!classKey.Open(HKEY_CLASSES_ROOT,classKeyName));

	// get application name
	CRegKey aux3Key;
	CHECK(!aux3Key.Open(classKey,"AuxUserType\\3"));
	DWORD count = 200;
	char buf[200];
	CHECK(!aux3Key.QueryValue(0,0,buf,&count));
	m_appName = buf;

	// get app icon
	CRegKey iconKey;
	if(!iconKey.Open(classKey,"DefaultIcon")) {
		count = 200;
		CHECK(!iconKey.QueryValue(0,0,buf,&count));
		GetWndClassInfo().m_wc.hIcon = getIcon(buf);
	}
	if(!iconKey.Open(classKey,"ToolboxBitmap32")) {
		count = 200;
		CHECK(!iconKey.QueryValue(0,0,buf,&count));
		GetWndClassInfo().m_wc.hIconSm = getIcon(buf);
	}

	RUN(setCaption());
	if(!m_hWnd)
		CHECK(Create(0,CWindow::rcDefault,m_caption.c_str()));
	ShowWindow(SW_SHOW);
	EnableMenuItem(m_frameMenu,ID_FileSave,MF_ENABLED);
	EnableMenuItem(m_frameMenu,ID_FileSaveAs,MF_ENABLED);
	return S_OK;
}
HRESULT Frame::go() {
    METHTRACE(Frame,go);
	RUN(CComQIPtr<IOleObject>(m_content)->SetClientSite(this));
	RECT rc = getRect();
	RUN(CComQIPtr<IOleObject>(m_content)->DoVerb(OLEIVERB_UIACTIVATE,0,this,-1,m_hWnd,&rc));
	BOOL junk;
	OnSetFocus(0,0,0,junk);
	return S_OK;
}
HRESULT Frame::close() {
    METHTRACE(Frame,close);
	if(!m_view)
		return S_FALSE;
	if(isDirty()) {
		USES_CONVERSION;
		char buf[200];
		BOOL junk(false);
		wsprintf(buf,"Save changes to %s?",m_filename.empty()?"Untitled":OLE2T(m_filename.c_str()));
		switch(MessageBox(buf,m_appName.c_str(),MB_YESNOCANCEL)) {
		case IDYES:
			if(!FileSave(0,0,0,junk))
				break;
			// else was cancelled
		case IDCANCEL:
			ERR(E_FAIL);
		case IDNO:
			break;
		default:
			_ASSERTE(("NO",0));
		}
	}
	RUN(m_view->CloseView(0));
	m_active.Release();
	m_view.Release();
	m_content.Release();
	m_storage.Release();
	m_filename.erase();
	m_border = rect(0,0,0,0);
	CWindow::SetMenu(m_frameMenu);
	EnableMenuItem(m_frameMenu,ID_FileSave,MF_GRAYED);
	EnableMenuItem(m_frameMenu,ID_FileSaveAs,MF_GRAYED);
	return S_OK;
}
// IMontageFrame
STDMETHODIMP Frame::StartFromStorage(IUnknown *storage) {
    METHTRACE(Frame,StartFromStorage);
	RUN(storage->QueryInterface(riidret(&m_storage)));
	if(!m_storage)
		ERR(E_INVALIDARG);
	RUN(ReadClassStg(m_storage,&m_clsid));
	RUN(CREATE(m_content,m_clsid)); TRACECREATEV(m_clsid);
	RUN(CComQIPtr<IPersistStorage>(m_content)->Load(m_storage));
	m_wasFile = false;
	m_useFile = false;
	// change menu items to reflect sitch
	HMENU fileMenu = GetSubMenu(m_frameMenu,0);
	ModifyMenu(fileMenu,ID_FileClose,MF_BYCOMMAND|MF_STRING,ID_FileClose,"&Close && Return");
	ModifyMenu(fileMenu,ID_FileSave,MF_BYCOMMAND|MF_STRING,ID_FileSave,"Updat&e\tCtrl-S");
	ModifyMenu(fileMenu,ID_FileSaveAs,MF_BYCOMMAND|MF_STRING,ID_FileSaveAs,"Save a Copy &As");
	RUNRET(show());
	RUN(getFormats());
	return S_OK;
}
#include "..\dgm\dgm_i.c"
STDMETHODIMP Frame::StartFromFile(BSTR filename) {
    METHTRACE(Frame,StartFromFile);
	m_filename = filename;
	m_wasFile = true;
	m_useFile = true;
	bool isDotFile;
	OLECHAR *lastDot = wcsrchr(filename,L'.');
	if(lastDot)
		isDotFile = wcscmp(lastDot+1,L"dot")==0;
	else
		isDotFile = true;
    CComPtr<IUnknown> content;
	if(isDotFile) { // special case so we don't get Word
		RUN(CREATE(content,m_clsid = CLSID_DynagraphControl));
		RUN(CComQIPtr<IPersistFile>(content)->Load(filename,STGM_READ));
	}
	else {
		// create moniker because file monikers know how to deal with extensions & classes
		CComPtr<IMoniker> mkFile;
		RUN(CreateFileMoniker(filename,&mkFile));
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		SetCursor(LoadCursor(0,IDC_WAIT));
		RUNRET(mkFile->BindToObject(bctx,0,riidret(&content)));
		if(FAILED(getObjectCLSID(content,&m_clsid))) {
			::MessageBox(m_hWnd,"Can't determine file type",m_appName.c_str(),MB_OK);
			return S_FALSE;
		}
	}
	RUN(show());
    // this method is slightly different from the others because it had to create the
    // content before creating the frame in order to get the CLSID for the application name for the titlebar
    // but reactions to show (e.g. OnSize) could try to look at m_content, so that's held til here.
    m_content = content;
	RUN(go());
	RUN(getFormats());
	return S_OK;
}
HRESULT Frame::startFromCLSID() {
    METHTRACE(Frame,startFromCLSID);
	m_wasFile = false;
	m_useFile = true;
	RUN(show());
	RUN(CREATE(m_content,m_clsid)); TRACECREATEV(m_clsid);
	RUN(StgCreateDocfile(0,STGM_SHARE_EXCLUSIVE|STGM_READWRITE,0,&m_storage));
	RUNRET(CComQIPtr<IPersistStorage>(m_content)->InitNew(m_storage));
	RUN(go());
	RUN(getFormats());
	return S_OK;
}
STDMETHODIMP Frame::StartFromClass(BSTR progID) {
    METHTRACE(Frame,StartFromClass);
	RUNRET(CLSIDFromProgID(progID,&m_clsid));
	return startFromCLSID();
}
// IOleDocumentSite
STDMETHODIMP Frame::ActivateMe(IOleDocumentView *pViewToActivate) {
    IMETHTRACE(Frame,IOleDocumentSite,ActivateMe);
	if(pViewToActivate)
		m_view = pViewToActivate;
	else
		RUN(CComQIPtr<IOleDocument>(m_content)->CreateView(this,0,0,&m_view));
	RUN(m_view->SetInPlaceSite(this));
	RUN(m_view->UIActivate(true));
	RECT rc = getRect();
	RUN(m_view->SetRect(&rc));
	RUN(m_view->Show(true));
	return S_OK;
}
// IOleClientSite
STDMETHODIMP Frame::SaveObject(void) {
    IMETHTRACE(Frame,IOleClientSite,SaveObject);
	BOOL junk;
	FileSave(0,0,0,junk);
	return S_OK;
}
STDMETHODIMP Frame::GetMoniker(DWORD dwAssign,DWORD dwWhichMoniker,IMoniker **ppmk) {
    IMETHTRACE(Frame,IOleClientSite,GetMoniker);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Frame::GetContainer(IOleContainer **ppContainer) {
    IMETHTRACE(Frame,IOleClientSite,GetContainer);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Frame::ShowObject(void) {
    IMETHTRACE(Frame,IOleClientSite,ShowObject);
	return S_OK;
}
STDMETHODIMP Frame::OnShowWindow(BOOL fShow) {
    IMETHTRACE(Frame,IOleClientSite,OnShowWindow);
	ERR(E_UNEXPECTED); // we're that window!
}
STDMETHODIMP Frame::RequestNewObjectLayout(void) {
    IMETHTRACE(Frame,IOleClientSite,RequestNewObjectLayout);
	ERR(E_NOTIMPL);
}
// IOleWindow
STDMETHODIMP Frame::GetWindow(HWND *phwnd) {
    IMETHTRACE(Frame,IOleInPlaceSite,GetWindow);
	*phwnd = m_hWnd;
	return S_OK;
}
STDMETHODIMP Frame::ContextSensitiveHelp(BOOL fEnterMode) {
    IMETHTRACE(Frame,IOleInPlaceSite,ContextSensitiveHelp);
	return S_OK;
}
// IOleInPlaceSite
STDMETHODIMP Frame::CanInPlaceActivate(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,CanInPlaceActivate);
	return S_OK;
}
STDMETHODIMP Frame::OnInPlaceActivate(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,OnInPlaceActivate);
	return S_OK;
}
STDMETHODIMP Frame::OnUIActivate(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,OnUIActivate);
	return S_OK;
}
STDMETHODIMP Frame::GetWindowContext(IOleInPlaceFrame **ppFrame,IOleInPlaceUIWindow **ppDoc,LPRECT lprcPosRect,
    	LPRECT lprcClipRect,LPOLEINPLACEFRAMEINFO lpFrameInfo) {
    IMETHTRACE(Frame,IOleInPlaceSite,GetWindowContext);
    (*ppFrame = this)->AddRef();
	*ppDoc = 0;
	*lprcPosRect = getRect();
	*lprcClipRect = *lprcPosRect;
	lpFrameInfo->cAccelEntries = 0;
	lpFrameInfo->fMDIApp = false;
	lpFrameInfo->hwndFrame = m_hWnd;
	lpFrameInfo->haccel = 0;
	lpFrameInfo->cAccelEntries = 0;
	return S_OK;
}
STDMETHODIMP Frame::Scroll(SIZE scrollExtent) {
    IMETHTRACE(Frame,IOleInPlaceSite,Scroll);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Frame::OnUIDeactivate(BOOL fUndoable) {
    IMETHTRACE(Frame,IOleInPlaceSite,OnUIDeactivate);
	return S_OK;
}
STDMETHODIMP Frame::OnInPlaceDeactivate(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,OnInPlaceDeactivate);
	return S_OK;
}
STDMETHODIMP Frame::DiscardUndoState(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,DiscardUndoState);
	return S_OK;
}
STDMETHODIMP Frame::DeactivateAndUndo(void) {
    IMETHTRACE(Frame,IOleInPlaceSite,DeactivateAndUndo);
	return S_OK;
}
STDMETHODIMP Frame::OnPosRectChange(LPCRECT lprcPosRect) {
    IMETHTRACE(Frame,IOleInPlaceSite,OnPosRectChange);
	ERR(E_FAIL);
}
// IOleInPlaceUIWindow
STDMETHODIMP Frame::GetBorder(LPRECT lprectBorder) {
    IMETHTRACE(Frame,IOleInPlaceUIWindow,GetBorder);
	GetClientRect(lprectBorder);
	return S_OK;
}
STDMETHODIMP Frame::RequestBorderSpace(LPCBORDERWIDTHS pborderwidths) {
    IMETHTRACE(Frame,IOleInPlaceUIWindow,RequestBorderSpace);
	return S_OK;
}
STDMETHODIMP Frame::SetBorderSpace(LPCBORDERWIDTHS pborderwidths) {
    IMETHTRACE(Frame,IOleInPlaceUIWindow,SetBorderSpace);
	if(!pborderwidths)
		m_border = rect(0,0,0,0);
	else
		m_border = *pborderwidths;
	RECT rc = getRect();
	RUN(m_view->SetRect(&rc));
	return S_OK;
}
STDMETHODIMP Frame::SetActiveObject(IOleInPlaceActiveObject *pActiveObject,LPCOLESTR pszObjName) {
    IMETHTRACE(Frame,IOleInPlaceUIWindow,SetActiveObject);
	m_active = pActiveObject;
	return S_OK;
}
// IOleInPlaceFrame
STDMETHODIMP Frame::InsertMenus(HMENU hmenuShared,LPOLEMENUGROUPWIDTHS lpMenuWidths) {
    IMETHTRACE(Frame,IOleInPlaceFrame,InsertMenus);
	HMENU fileMenu = GetSubMenu(m_frameMenu,0);
	CHECKR(InsertMenu(hmenuShared,0,MF_BYPOSITION|MF_POPUP|MF_ENABLED|MF_STRING,(UINT)fileMenu,"&File"));
	lpMenuWidths->width[0] = 1;
	return S_OK;
}
STDMETHODIMP Frame::SetMenu(HMENU hmenuShared,HOLEMENU holemenu,HWND hwndActiveObject) {
    IMETHTRACE(Frame,IOleInPlaceFrame,SetMenu);
	CWindow::SetMenu(hmenuShared);
	RUN(OleSetMenuDescriptor(holemenu,m_hWnd,hwndActiveObject,this,m_active));
	return S_OK;
}
STDMETHODIMP Frame::RemoveMenus(HMENU hmenuShared) {
    IMETHTRACE(Frame,IOleInPlaceFrame,RemoveMenus);
	CHECKR(RemoveMenu(hmenuShared,0,MF_BYPOSITION));
	return S_OK;
}
STDMETHODIMP Frame::SetStatusText(LPCOLESTR pszStatusText) {
    IMETHTRACE(Frame,IOleInPlaceFrame,SetStatusText);
	ERR(E_NOTIMPL);
}
STDMETHODIMP Frame::EnableModeless(BOOL fEnable) {
    IMETHTRACE(Frame,IOleInPlaceFrame,EnableModeless);
	return S_OK;
}
STDMETHODIMP Frame::TranslateAccelerator(LPMSG lpmsg,WORD wID) {
    IMETHTRACE(Frame,IOleInPlaceFrame,TranslateAccelerator);
	return S_OK;
}
// IMCCFrameSharer
STDMETHODIMP Frame::get_ActiveObject(IOleInPlaceActiveObject **ret) {
    m_active.CopyTo(ret);
	return S_OK;
}
