/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "dispex.h"
#include "../Group/Group.h"
#include "../Montage/Montage.h" // only for IMCCFrameSharer
#include "resource.h"
// simplest possible Active Document frame
struct Frame : CWindowImpl<Frame,CWindow,CWinTraitsOR<WS_MAXIMIZE,0,CFrameWinTraits> >,
	CComCoClass<Frame,&CLSID_MontageFrame>,
	CComObjectRoot,
	IDispatchImpl<IMontageFrame>,
	IOleDocumentSite,
	IOleClientSite,
	IOleInPlaceSite,
	IOleInPlaceFrame,
    IMCCFrameSharer
{
	string m_appName, // from AuxUserType\3
		m_caption; // "App", "Filename - App", or "Type in Filename"
	HMENU m_frameMenu;
	wstring m_filename;
	string m_inFormats,m_outFormats;
	CComPtr<IStorage> m_storage;
	CLSID m_clsid;
	bool m_wasFile, // true: loaded w/ IPersistFile.  Else IPersistStorage.
		m_useFile; // whether we're using a file (e.g. StartFromCLSID)

	// these three prob. all point to the same o.
	CComPtr<IUnknown> m_content;
	CComPtr<IOleDocumentView> m_view;
	CComPtr<IOleInPlaceActiveObject> m_active;

	// how much toolbar area it's asking for
	RECT m_border;
	RECT getRect() {
		RECT rc;
		GetClientRect(&rc);
		InflateRect(&rc,-2,-2);
		return rect(rc.left+m_border.left,rc.top+m_border.top,rc.right-m_border.right,rc.bottom-m_border.bottom);
	}
	HRESULT setCaption();
	HRESULT getFormats();
	HRESULT show();
	HRESULT startFromCLSID();
	HRESULT go();
	HRESULT close();

	bool isDirty() {
		if(m_wasFile)
			return CComQIPtr<IPersistFile>(m_content)->IsDirty()==S_OK;
		else
			return CComQIPtr<IPersistStorage>(m_content)->IsDirty()==S_OK;
	}

	Frame();
	~Frame();

	REGGY_RID(IDR_MontageFrame);
	//DECLARE_WND_CLASS_EX("MontageFrame",CS_DBLCLKS,COLOR_APPWORKSPACE);
BEGIN_COM_MAP(Frame)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IMontageFrame)
	COM_INTERFACE_ENTRY(IOleDocumentSite)
	COM_INTERFACE_ENTRY(IOleClientSite)
	COM_INTERFACE_ENTRY2(IOleWindow,IOleInPlaceSite)
	COM_INTERFACE_ENTRY(IOleInPlaceSite)
	COM_INTERFACE_ENTRY(IOleInPlaceUIWindow)
	COM_INTERFACE_ENTRY(IOleInPlaceFrame)
    COM_INTERFACE_ENTRY(IMCCFrameSharer)
END_COM_MAP()

BEGIN_MSG_MAP(Frame)
	MESSAGE_HANDLER(WM_PAINT,OnPaint)
	MESSAGE_HANDLER(WM_SIZE,OnSize)
	MESSAGE_HANDLER(WM_DESTROY,OnDestroy)
	MESSAGE_HANDLER(WM_CLOSE,OnClose)
	MESSAGE_HANDLER(WM_SETFOCUS,OnSetFocus)
	MESSAGE_HANDLER(WM_ENABLE,OnSetFocus)
	COMMAND_ID_HANDLER(ID_FileNew,FileNew);
	COMMAND_ID_HANDLER(ID_FileOpen,FileOpen);
	COMMAND_ID_HANDLER(ID_FileClose,FileClose);
	COMMAND_ID_HANDLER(ID_FileSave,FileSave);
	COMMAND_ID_HANDLER(ID_FileSaveAs,FileSaveAs);
	COMMAND_ID_HANDLER(ID_FileExit,FileExit);
END_MSG_MAP()

	LRESULT OnPaint(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);
	LRESULT OnSetFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled);

	LRESULT FileNew(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT FileOpen(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT FileClose(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT FileSave(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT FileSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT FileExit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);

// IMontageFrame
	STDMETHODIMP StartFromStorage(IUnknown *storage);
	STDMETHODIMP StartFromFile(BSTR filename);
	STDMETHODIMP StartFromClass(BSTR progID);
// IOleDocumentSite
	STDMETHODIMP ActivateMe(IOleDocumentView *pViewToActivate);
// IOleClientSite
	STDMETHODIMP SaveObject(void);
	STDMETHODIMP GetMoniker(DWORD dwAssign,DWORD dwWhichMoniker,IMoniker **ppmk);
	STDMETHODIMP GetContainer(IOleContainer **ppContainer);
	STDMETHODIMP ShowObject(void);
	STDMETHODIMP OnShowWindow(BOOL fShow);
	STDMETHODIMP RequestNewObjectLayout(void);
// IOleWindow
	STDMETHODIMP GetWindow(HWND *phwnd);
	STDMETHODIMP ContextSensitiveHelp(BOOL fEnterMode);
// IOleInPlaceSite
	STDMETHODIMP CanInPlaceActivate(void);
	STDMETHODIMP OnInPlaceActivate(void);
	STDMETHODIMP OnUIActivate(void);
	STDMETHODIMP GetWindowContext(IOleInPlaceFrame **ppFrame,IOleInPlaceUIWindow **ppDoc,LPRECT lprcPosRect,
		LPRECT lprcClipRect,LPOLEINPLACEFRAMEINFO lpFrameInfo);
	STDMETHODIMP Scroll(SIZE scrollExtent);
	STDMETHODIMP OnUIDeactivate(BOOL fUndoable);
	STDMETHODIMP OnInPlaceDeactivate(void);
	STDMETHODIMP DiscardUndoState(void);
	STDMETHODIMP DeactivateAndUndo(void);
	STDMETHODIMP OnPosRectChange(LPCRECT lprcPosRect);
// IOleInPlaceUIWindow
	STDMETHODIMP GetBorder(LPRECT lprectBorder);
	STDMETHODIMP RequestBorderSpace(LPCBORDERWIDTHS pborderwidths);
	STDMETHODIMP SetBorderSpace(LPCBORDERWIDTHS pborderwidths);
	STDMETHODIMP SetActiveObject(IOleInPlaceActiveObject *pActiveObject,LPCOLESTR pszObjName);
// IOleInPlaceFrame
	STDMETHODIMP InsertMenus(HMENU hmenuShared,LPOLEMENUGROUPWIDTHS lpMenuWidths);
	STDMETHODIMP SetMenu(HMENU hmenuShared,HOLEMENU holemenu,HWND hwndActiveObject);
	STDMETHODIMP RemoveMenus(HMENU hmenuShared);
	STDMETHODIMP SetStatusText(LPCOLESTR pszStatusText);
	STDMETHODIMP EnableModeless(BOOL fEnable);
	STDMETHODIMP TranslateAccelerator(LPMSG lpmsg,WORD wID);
// IMCCFrameSharer
    STDMETHODIMP get_ActiveObject(IOleInPlaceActiveObject **ret);
};