/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// appFrame.cpp : Implementation of WinMain


// Note: Proxy/Stub Information
//      To build a separate proxy/stub DLL,
//      run nmake -f appFrameps.mk in the project directory.

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include "appFrame.h"

#include "appFrame_i.c"

#include "Frame.h"


const DWORD dwTimeOut = 5000; // time for EXE to be idle before shutting down
const DWORD dwPause = 1000; // time to wait for threads to finish up

// Passed to CreateThread to monitor the shutdown event
static DWORD WINAPI MonitorProc(void* pv)
{
    CExeModule* p = (CExeModule*)pv;
    p->MonitorShutdown();
    return 0;
}

LONG CExeModule::Unlock()
{
    LONG l = CComModule::Unlock();
    if (l == 0)
    {
        bActivity = true;
        SetEvent(hEventShutdown); // tell monitor that we transitioned to zero
    }
    return l;
}

//Monitors the shutdown event
void CExeModule::MonitorShutdown()
{
    while (1)
    {
        WaitForSingleObject(hEventShutdown, INFINITE);
        DWORD dwWait=0;
        do
        {
            bActivity = false;
            dwWait = WaitForSingleObject(hEventShutdown, dwTimeOut);
        } while (dwWait == WAIT_OBJECT_0);
        // timed out
        if (!bActivity && m_nLockCnt == 0) // if no activity let's really bail
        {
#if _WIN32_WINNT >= 0x0400 & defined(_ATL_FREE_THREADED)
            CoSuspendClassObjects();
            if (!bActivity && m_nLockCnt == 0)
#endif
                break;
        }
    }
    CloseHandle(hEventShutdown);
    PostThreadMessage(dwThreadID, WM_QUIT, 0, 0);
}

bool CExeModule::StartMonitor()
{
    hEventShutdown = CreateEvent(NULL, false, false, NULL);
    if (hEventShutdown == NULL)
        return false;
    DWORD dwThreadID;
    HANDLE h = CreateThread(NULL, 0, MonitorProc, this, 0, &dwThreadID);
    return (h != NULL);
}

CExeModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
	OBJECT_ENTRY(CLSID_MontageFrame,Frame)
END_OBJECT_MAP()


LPCTSTR FindOneOf(LPCTSTR p1, LPCTSTR p2)
{
    while (p1 != NULL && *p1 != NULL)
    {
        LPCTSTR p = p2;
        while (p != NULL && *p != NULL)
        {
            if (*p1 == *p)
                return CharNext(p1);
            p = CharNext(p);
        }
        p1 = CharNext(p1);
    }
    return NULL;
}
LPCTSTR nextWord(LPCTSTR start) {
	LPCTSTR i = start;
	if(*i++=='"')
		while(*i++!='"');
	while(*i && *i++!=' ');
	return i;
}
bool findParam(LPCTSTR cmdLine, LPTSTR buf) {
	LPCTSTR i = cmdLine;
	while(*(i = nextWord(i)))
		if(*i!='-' && *i!='/') {
			if(*i=='"') {
				++i;
				LPCTSTR end = strchr(i,'"');
				_ASSERTE(end);
				strncpy(buf,i,end-i);
				return true;
			}
			LPCTSTR end = strchr(i,' ');
			if(end)
				strncpy(buf,i,end-i);
			else
				strcpy(buf,i);
			return true;
		}
	return false;
}

/////////////////////////////////////////////////////////////////////////////
//
extern "C" int WINAPI _tWinMain(HINSTANCE hInstance,
    HINSTANCE /*hPrevInstance*/, LPTSTR lpCmdLine, int /*nShowCmd*/)
{
    lpCmdLine = GetCommandLine(); //this line necessary for _ATL_MIN_CRT

    _Module.Init(ObjectMap, hInstance, &LIBID_MontageFrame);
    _Module.dwThreadID = GetCurrentThreadId();
    TCHAR szTokens[] = _T("-/");

	_CrtSetReportMode(_CRT_ASSERT,_CRTDBG_MODE_WNDW|_CRTDBG_MODE_DEBUG);
    int nRet = 0;
    BOOL bRun = TRUE;
    LPCTSTR lpszToken = FindOneOf(lpCmdLine, szTokens);
    while (lpszToken != NULL)
    {
        if (lstrcmpi(lpszToken, _T("UnregServer"))==0)
        {
            _Module.UpdateRegistryFromResource(IDR_AppFrame, FALSE);
            nRet = _Module.UnregisterServer(TRUE);
            bRun = FALSE;
            break;
        }
        if (lstrcmpi(lpszToken, _T("RegServer"))==0)
        {
            _Module.UpdateRegistryFromResource(IDR_AppFrame, TRUE);
            nRet = _Module.RegisterServer(TRUE);
            bRun = FALSE;
            break;
        }
        lpszToken = FindOneOf(lpszToken, szTokens);
    }

    if (bRun)
    {
	SRUN(OleInitialize(0));
    FUNTRACE(_tWinMain);
        _Module.StartMonitor();
#if _WIN32_WINNT >= 0x0400 & defined(_ATL_FREE_THREADED)
        SRUN(_Module.RegisterClassObjects(CLSCTX_LOCAL_SERVER,
            REGCLS_MULTIPLEUSE | REGCLS_SUSPENDED))
        SRUN(CoResumeClassObjects());
#else
        SRUN(_Module.RegisterClassObjects(CLSCTX_LOCAL_SERVER,
            REGCLS_MULTIPLEUSE));
#endif

		CComPtr<IMontageFrame> frame;
		char param[MAX_PATH];
		HRESULT hr;
		if(findParam(lpCmdLine,param)) {
			// only create with param (otherwise I'm a server)
			SRUN(CREATE(frame,CLSID_MontageFrame));
			CComBSTR bstrParam;
			if(param[0]=='@') { // hmm.
				bstrParam = param+1;
				hr = frame->StartFromClass(bstrParam);
			}
			else {
				bstrParam = param;
				hr = frame->StartFromFile(bstrParam);
			}
		}
		else
			hr = E_INVALIDARG;
		if(FAILED(hr)) {
			char buf[500];
			switch(hr) {
			case REGDB_E_CLASSNOTREG:
				sprintf(buf,"The requested class \"%s\" is not registered.",param+1);
				break;
			case E_INVALIDARG:
				sprintf(buf,"Specify a file or @class, e.g. @DGM.Diagram, @Word.Document");
				break;
			case E_ABORT: // user cancelled a dialog
				goto noerror;
			case MK_E_CANTOPENFILE:
				sprintf(buf,"Couldn't open file %s",param);
				break;
			default:
				sprintf(buf,"An unknown error occured while loading code 0x%x",hr);
			}
			MessageBox(0,buf,"Startup Error",MB_OK);
noerror:
			frame.Release();
			return 0;
		}
        MSG msg;
        while (GetMessage(&msg, 0, 0, 0)) {
			TranslateMessage(&msg);
            DispatchMessage(&msg);
		}

        _Module.RevokeClassObjects();
        Sleep(dwPause); //wait for any threads to finish
    }

    _Module.Term();
    OleUninitialize();
    return nRet;
}