/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "brutePrice.h"

inline int PENALTY(int x) {
	return x>>4;
}
// cheap solution: move other
void pushRight(BPState &s,BPState &next,int border,int box) {
	next.cost += moveRight(s[border].pos,next[box].pos);
}
// more expensive: move border
void yieldRight(BPState &s,BPState &next,int border,int box) {
	if(s[border].pos.right<s[box].pos.left)
		return;
	next.cost += PENALTY((s[border].pos.right-next[box].pos.left+1)*hei(s[border].pos));
	next[border].pos.right = next[box].pos.left-1;
}
void pushLeft(BPState &s,BPState &next,int border,int box) {
	next.cost += moveLeft(s[border].pos,next[box].pos);
}
void yieldLeft(BPState &s,BPState &next,int border,int box) {
	if(s[border].pos.left>s[box].pos.right)
		return;
	next.cost += PENALTY((next[box].pos.right - s[border].pos.left+1)*hei(s[border].pos));
	next[border].pos.left= next[box].pos.right+1;
}
void pushTop(BPState &s,BPState &next,int border,int box) {
	next.cost += moveUp(s[border].pos,next[box].pos);
}
void yieldTop(BPState &s,BPState &next,int border,int box) {
	if(s[border].pos.top>s[box].pos.bottom)
		return;
	next.cost += PENALTY((next[box].pos.bottom - s[border].pos.top+1)*wid(s[border].pos));
	next[border].pos.top = next[box].pos.bottom+1;
}
void pushBottom(BPState &s,BPState &next,int border,int box) {
	next.cost += moveDown(s[border].pos,next[box].pos);
}
void yieldBottom(BPState &s,BPState &next,int border,int box) {
	if(s[border].pos.bottom<s[box].pos.top)
		return;
	next.cost += PENALTY((s[border].pos.bottom - next[box].pos.top +1)*wid(s[border].pos));
	next[border].pos.bottom = next[box].pos.top-1;
}
inline bool share_range(int r0, int r1, int s0, int s1) {
	return r0 <= s1 && r1 >= s0;
}
typedef void (*solver)(BPState &s,BPState &next,int border,int box);
solver g_pushers[4] = {pushLeft,pushTop,pushRight,pushBottom},
	g_yielders[4] = {yieldLeft,yieldTop,yieldRight,yieldBottom};
// push and yield ...
void andStuff(BPState *s,BPGame *g,int border,int box,int solvedir) {
	BPState *next = new BPState(s);
	g_pushers[solvedir](*s,*next,border,box);
	g->push(next);
	next = new BPState(s);
	g_yielders[solvedir](*s,*next,border,box);
	g->push(next);
}
bool border_play(BPBoard::iterator o,BPState *s, BPGame *g) {
	RECT &pos = o->pos;
	int borderN = o-s->begin();
	bool played(false);
	BPBoard::iterator i;

	// first deal with conflicts.
	// cheap trick: throw one state on where we've tried all the most likely yields
	// *to compete with* all the more difficult try-one-at-a-time scenarios
	//dBPState *mostLikelyYield = new BPState(s);
	for(i = upper_bound(s->begin(),s->end(),*o,greater<BPObject>());i!=s->end();++i)
		if(i->valid) {
			int section = intersectType(i->pos,pos);
			if(section==int_none)
				continue;
			int boxN = i-s->begin();
			// try fixing this problem in the two shortest directions
			int dists[4] = {i->pos.right-pos.left,i->pos.bottom-pos.top,
				pos.right-i->pos.left,pos.bottom-i->pos.top};
			int a=0,b=1;
			if(dists[b]<dists[a])
				swap(a,b);
			for(int i = 2; i<4; ++i)
				if(dists[i]<dists[b])
					if(dists[i]<dists[a]) {
						b = a;
						a = i;
					}
					else
						b = i;
			//g_yielders[a](*mostLikelyYield,*mostLikelyYield,borderN,boxN);
			//_ASSERTE(!((*mostLikelyYield)[borderN].pos&&(*mostLikelyYield)[boxN].pos));
			andStuff(s,g,borderN,boxN,a);
			if(dists[b]<2*dists[a])
				andStuff(s,g,borderN,boxN,b);
			return true;
		}
		/*
	if(mostLikelyYield->cost) {
		// hack: if this is obviously first time, make mostLikelyYield free
		if((*s)[0].pos==(*s)[1].pos)
			mostLikelyYield->cost = 0;
		g->push(mostLikelyYield);
		played = true;
	}
	*/
	if(played)
		return true;

	// now look to see if we can push boundaries back
	RECT push = (*s)[0].pos; // start at bounds, find reasons to bring border inward
	for(i = upper_bound(s->begin(),s->end(),*o,greater<BPObject>());i!=s->end();++i)
		if(i->valid) {
			if(share_range(i->pos.top,i->pos.bottom,pos.top,pos.bottom)) {
				if(i->pos.right<pos.left)
					push.left = max(i->pos.right+1,push.left);
				if(i->pos.left>pos.right)
					push.right = min(i->pos.left-1,push.right);
			}
			if(share_range(i->pos.left,i->pos.right,pos.left,pos.right)) {
				if(i->pos.bottom<pos.top)
					push.top = max(i->pos.bottom+1,push.top);
				if(i->pos.top>pos.bottom)
					push.bottom = min(i->pos.top-1,push.bottom);
			}
		}

	_ASSERTE(push.left<=pos.left);
	_ASSERTE(push.top<=pos.top);
	_ASSERTE(push.right>=pos.right);
	_ASSERTE(push.bottom>=pos.bottom);

	// these are individual solutions (it might not work to push two boundaries back at once)
	// so play each possibility
	if(push.left!=pos.left) {
		BPState *next = new BPState(s);
		(*next)[borderN].pos.left = push.left;
		next->cost -= PENALTY((pos.left-push.left)*hei(pos));
		g->push(next);
		played = true;
	}
	if(push.top!=pos.top) {
		BPState *next = new BPState(s);
		(*next)[borderN].pos.top = push.top;
		next->cost -= PENALTY((pos.top - push.top)*wid(pos));
		g->push(next);
		played = true;
	}
	if(push.right!=pos.right) {
		BPState *next = new BPState(s);
		(*next)[borderN].pos.right = push.right;
		next->cost -= PENALTY((push.right - pos.right)*hei(pos));
		g->push(next);
		played = true;
	}
	if(push.bottom!=pos.bottom) {
		BPState *next = new BPState(s);
		(*next)[borderN].pos.bottom = push.bottom;
		next->cost -= PENALTY((push.bottom - pos.bottom)*wid(pos));
		g->push(next);
		played = true;
	}
	return played;
}