/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "brutePrice.h"

bool bounds_play(BPBoard::iterator o,BPState *s, BPGame *g) {
	// replace board with one where no one's out of bounds
	// or expand boundary
	BPState *next = new BPState(s);
	bool played = false;
	if(!g->flexibleBoundary) {
		RECT &pos = o->pos;
		for(BPBoard::iterator i = upper_bound(next->begin(),next->end(),*o,greater<BPObject>());i!=next->end();++i)
			if(i->valid && i->priority<BPP_Bounds) {
				RECT &r = i->pos;

				if(r.right>pos.right) {
					played = true;
					if(i->priority==BPP_Border) { // look for border specifically because it shrinks instead of moving...
						r.right = pos.right;
						if(r.left>r.right)
							r.left = r.right;
					}
					else {
						int w = wid(r);
						if(w<wid(pos)) {
							r.right = pos.right;
							r.left = r.right - w;
						}
					}
				}
				if(r.bottom>pos.bottom) {
					played = true;
					if(i->priority==BPP_Border) {
						r.bottom = pos.bottom;
						if(r.top>r.bottom)
							r.top = r.bottom;
					}
					else {
						int h = hei(r);
						if(h<hei(pos)) {
							r.bottom = pos.bottom;
							r.top = r.bottom - h;
						}
					}
				}
				if(r.left<pos.left) {
					played = true;
					if(i->priority==BPP_Border) {
						r.left = pos.left;
						if(r.right<r.left)
							r.right = r.left;
					}
					else {
						int w = wid(r);
						if(w<wid(pos)) {
							r.left = pos.left;
							r.right = r.left + w;
						}
					}
				}
				if(r.top<pos.top) {
					played = true;
					if(i->priority==BPP_Border) {
						r.top = pos.top;
						if(r.bottom<r.top)
							r.bottom = r.top;
					}
					else {
						int h = hei(r);
						if(h<hei(pos)) {
							r.top = pos.top;
							r.bottom = r.top + h;
						}
					}
				}
			}
	}
	else {
		bool valid=false;
		RECT onion;
		for(BPBoard::iterator i = upper_bound(s->begin(),s->end(),*o,greater<BPObject>());i!=s->end();++i)
			if(i->valid && i->priority<BPP_Bounds) {
				if(valid)
					onion = onion|i->pos;
				else
					onion = i->pos,
					valid = true;
			}
		if(valid) {
			const RECT &pos = o->pos;
			RECT r2;
			if(wid(onion)>wid(pos))
				r2.left = onion.left,r2.right = onion.right;
			else if(onion.right>=pos.right)
				r2.right = onion.right,r2.left = r2.right-wid(pos);
			else if(onion.left<=pos.left)
				r2.left = onion.left,r2.right = r2.left+wid(pos);
			else
				r2.left = pos.left,r2.right = pos.right;
			if(hei(onion)>hei(pos))
				r2.top = onion.top,r2.bottom = onion.bottom;
			else if(onion.top<=pos.top)
				r2.top = onion.top,r2.bottom = r2.top + hei(pos);
			else if(onion.bottom>=pos.bottom)
				r2.bottom = onion.bottom,r2.top = r2.bottom - hei(pos);
			else
				r2.top = pos.top,r2.bottom = pos.bottom;
			if(r2!=pos) {
				(*next)[0].pos = rect(0,0,wid(r2),hei(r2));
				if(r2.left!=0 || r2.top!=0)
					for(size_t i=1; i<next->size(); ++i)
						if((*next)[i].valid)
							(*next)[i].pos.left -= r2.left,
							(*next)[i].pos.top -= r2.top,
							(*next)[i].pos.right -= r2.left,
							(*next)[i].pos.bottom -= r2.top;
				played = true;
			}
		}
	}
	if(played)
		g->push(next); // same cost, will come up again soon
	else
		delete next; // horribly inefficient
	return played;
}