/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "brutePrice.h"

int moveRight(const RECT &sub,RECT &ob) {
	int d = sub.right - ob.left + 1;
	_ASSERTE(d>0);
	ob.left += d;
	ob.right += d;
	return d*hei(ob);
}
int moveDown(const RECT &sub,RECT &ob) {
	int d = sub.bottom - ob.top + 1;
	_ASSERTE(d>0);
	ob.top += d;
	ob.bottom += d;
	return d*wid(ob);
}
int moveLeft(const RECT &sub,RECT &ob) {
	int d = ob.right - sub.left + 1;
	_ASSERTE(d>0);
	ob.left -= d;
	ob.right -= d;
	return d*hei(ob);
}
int moveUp(const RECT &sub,RECT &ob) {
	int d = ob.bottom - sub.top + 1;
	_ASSERTE(d>0);
	ob.top -= d;
	ob.bottom -= d;
	return d*wid(ob);
}
BPGame::stepResult BPGame::step() {
	_ASSERTE(size());
	BPState *curr = top();
	pop();
	// clear up problems from highest priority to lowest
	for(BPState::iterator i = curr->begin();i!=curr->end();++i)
		if(i->valid && i->callOut(i,curr,this))
			return make_pair(true,curr);
	return make_pair(false,curr);
}
BPBoard *BPGame::run(BPBoard *start) {
	push(new BPState(start));
	int count=0;
	while(size()) {
		if(size()>10000) {
			clear();
			return 0;
		}
		stepResult res = step();
		if(!res.first) {// went through whole board without anything happening
			BPBoard *ret = new BPBoard(res.second->begin(),res.second->end());
			clear();
			return ret;
		}
	}
	//MessageBox(0,"what?","couldn't think of anything to do",MB_OK);
	return 0; // (the only things it could figure to do would keep the board the same?)
}

