/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Engine.h"
#include "StringGraph.h"
#include "ComDinoMach.h"

// IDinoNode
STDMETHODIMP ComDinoNode::get_Graph(IDGGraph **ret) {
    IncrLangEvents *hand = gd<DinoMachNode>(m_node).handler;
    DString dinotype = hand->dinotype();
    if(dinotype=="layout")
        (*ret = static_cast<Engine*>(hand))->AddRef();
    else if(dinotype=="abstract")
        (*ret = static_cast<StringGraph<StrGraph>*>(hand))->AddRef();
    else if(dinotype=="dinomachine")
        (*ret = static_cast<ComDinoMach*>(hand))->AddRef();
	return S_OK;
}

// IDinoEdge
STDMETHODIMP ComDinoEdge::Map(IUnknown *src,IEnumUnknown **ret) {
    CComPtr<IDGNode> head;
    RUN(get_HeadNode(&head));
    bool isEdge;
    CComPtr<IDGGraph> destgraph;
    RUN(CComQIPtr<IDinoNode>(head)->get_Graph(&destgraph));
    if(CComQIPtr<IDGNode> n = src)
        isEdge = false;
    else if(CComQIPtr<IDGEdge> e = src)
        isEdge = true;
    else ERR(E_INVALIDARG);
    CComBSTR name;
    RUN(CComQIPtr<IDGStringAttributes>(src)->get_Name(&name));
    USES_CONVERSION;
    NEID_map::tset &s = gd<DinoMachEdge>(m_edge).tailmap()[NEID(isEdge,W2A(name))];
    IUnknown **ray = new IUnknown*[s.size()];
    int i=0;
    for(NEID_map::tset::iterator idi = s.begin(); idi!=s.end(); ++idi)
        if(idi->isEdge)
            RUN(destgraph->get_Edge(CComBSTR(idi->name.c_str()),(IDGEdge**)&ray[i++]));
        else
            RUN(destgraph->get_Node(CComBSTR(idi->name.c_str()),(IDGNode**)&ray[i++]));

    typedef CComObject<CComEnum<IEnumUnknown, &__uuidof(IEnumUnknown), IUnknown*, _CopyInterface<IUnknown> > > enumunk;
    enumunk *eu = new enumunk;
    eu->Init(ray,ray+i,0,AtlFlagTakeOwnership);
    (*ret = eu)->AddRef();
    return S_OK;
}
