/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct Engine;

struct EdgeLabelWrap;

struct Edge :
	CComObjectRoot,
	CComCoClass<Edge,&CLSID_DGEdge>,
	IConnectionPointContainerImpl<Edge>,
	IConnectionPointImpl<Edge, &IID_IDGEdgeShadow>,
	IDGDrawnEdge,
	IDispatchImpl<DIDGDrawnEdge>,
	IPersistStream,
	ChStringHandler<Edge>
{
	Engine *m_engine;
	Layout::Edge *m_edge;
	bool m_destroying,
		m_incrInserted;

	vector<EdgeLabelWrap*> labelz;

	Edge();
	~Edge();

	// for ChStringHandler
	Layout::Edge *sh_object() { return m_edge; }
	void sh_modify(StrAttrs attrs);
	void sh_rename(DString name);

	EdgeGeom &geom() {
		return gd<EdgeGeom>(m_edge);
	}
	EdgeLabels &labels() {
		return gd<EdgeLabels>(m_edge);
	}
	void freeEdge();
	HRESULT mod(Update upd);
// engine call ins
	void show();
	void hide(Engine *engine,Layout::Edge *edge);

BEGIN_COM_MAP(Edge)
	COM_INTERFACE_ENTRY(IDGEdge)
	COM_INTERFACE_ENTRY(IDGDrawnEdge)
	COM_INTERFACE_ENTRY(DIDGDrawnEdge)
	COM_INTERFACE_ENTRY(IDGStringAttributes)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IPersistStream)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(Edge)
   CONNECTION_POINT_ENTRY(IID_IDGEdgeShadow)
END_CONNECTION_POINT_MAP()

// IDGEdge
	STDMETHODIMP get_Graph(IDGGraph **ppOut);
    STDMETHODIMP get_TailNode(IDGNode **pNode);
    STDMETHODIMP get_HeadNode(IDGNode **pNode);
    STDMETHODIMP get_TailPos(PPOINTD ptpos);
    STDMETHODIMP put_TailPos(POINTD tpos);
    STDMETHODIMP get_HeadPos(PPOINTD phpos);
    STDMETHODIMP put_HeadPos(POINTD hpos);
    STDMETHODIMP get_Length(double *pLength);
    STDMETHODIMP put_Length(double Length);
    STDMETHODIMP get_Constraint(VARIANT_BOOL *pConstraint);
    STDMETHODIMP put_Constraint(VARIANT_BOOL Constraint);
	STDMETHODIMP SetPoints(DGSHAPEDESC *val);
	STDMETHODIMP GetPoints(DGSHAPEDESC **ret);
	STDMETHODIMP new_Label(IDGEdgeLabel **ret);
    STDMETHODIMP Insert();
    STDMETHODIMP Delete();
	STDMETHODIMP Optimize(VARIANT_BOOL full);
// DIDGDrawnEdge
	STDMETHODIMP put_BorderColor(OLE_COLOR clr);
	STDMETHODIMP get_BorderColor(OLE_COLOR* pclr);
	STDMETHODIMP put_ForeColor(OLE_COLOR clr);
	STDMETHODIMP get_ForeColor(OLE_COLOR* pclr);
	STDMETHODIMP put_FillColor(OLE_COLOR clr);
	STDMETHODIMP get_FillColor(OLE_COLOR* pclr);
	STDMETHODIMP get_Engine(DIDGLayout **ppEngine);
	STDMETHODIMP get_TailNode(DIDGDrawnNode **pNode);
	STDMETHODIMP get_HeadNode(DIDGDrawnNode **pNode);
	STDMETHODIMP get_TailX(double *ret);
	STDMETHODIMP get_TailY(double *ret);
	STDMETHODIMP SetTailPos(double x,double y);
	STDMETHODIMP get_HeadX(double *ret);
	STDMETHODIMP get_HeadY(double *ret);
	STDMETHODIMP SetHeadPos(double x,double y);
	STDMETHODIMP get_Points(DIDGShape **ret);
// IPersistStream
    STDMETHODIMP GetClassID(CLSID *pClassID);
    STDMETHODIMP IsDirty(void);
    STDMETHODIMP Load(IStream *stream);
    STDMETHODIMP Save(IStream *stream,BOOL fClearDirty);
    STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
};
