/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct Edge;
struct EdgeLabelWrap : IDGEdgeLabel, CComObjectRoot {
	Edge *e;
	unsigned x;

BEGIN_COM_MAP(EdgeLabelWrap)
	COM_INTERFACE_ENTRY(IDGLabel)
	COM_INTERFACE_ENTRY(IDGEdgeLabel)
END_COM_MAP();

	STDMETHODIMP Delete() {
		for(unsigned i = x+1; i<e->labelz.size(); ++i)
			e->labelz[i]->x--;
		e->labelz.erase(e->labelz.begin()+x);
		e->labels().erase(e->labels().begin()+x);
		e = 0;
		Release();
		return S_OK;
	}

	STDMETHODIMP get_Pos1(POINTD *ret) {
		if(!e)
			ERR(E_FAIL);
		*ret = DG2Win(e->m_engine->reorient(e->labels()[x].pos1,false));
		return S_OK;
	}
	STDMETHODIMP put_Pos1(POINTD val) {
		if(!e)
			ERR(E_FAIL);
		e->labels()[x].pos1 = e->m_engine->reorient(Win2DG(val),true);
		e->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_Pos2(POINTD *ret) {
		if(!e)
			ERR(E_FAIL);
		*ret = DG2Win(e->m_engine->reorient(e->labels()[x].pos2,false));
		return S_OK;
	}
	STDMETHODIMP put_Pos2(POINTD val) {
		if(!e)
			ERR(E_FAIL);
		e->labels()[x].pos2 = e->m_engine->reorient(Win2DG(val),true);
		e->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_Where(double *ret) {
		*ret = e->labels()[x].where;
		return S_OK;
	}
	STDMETHODIMP put_Where(double val) {
		if(val<0||val>1)
			ERR(E_INVALIDARG);
		e->labels()[x].where = val;
		e->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_Length(double *ret) {
		*ret = e->labels()[x].length;
		return S_OK;
	}
	STDMETHODIMP put_Length(double val) {
		e->labels()[x].length = val;
		e->mod(DG_UPD_LABEL);
		return S_OK;
	}
};
