/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "EdgePersistor.h"

#include "FixedOlePersist.h"
#include "cacheStreamableObject.h"
#include "countStream.h"

EdgePersistor::EdgePersistor() {
	CONSTRUCTORTRACE(EdgePersistor);
}
EdgePersistor::~EdgePersistor() {
	DESTRUCTORTRACE(EdgePersistor);
}
// IDGEdgePersistor
STDMETHODIMP EdgePersistor::put_Name(BSTR val) {
	m_name = val;
	return S_OK;
}
STDMETHODIMP EdgePersistor::put_Cache(IStream *dgdata) {
	METHTRACE(EdgePersistor,put_Cache);
	m_cache = dgdata;
	return S_OK;
}
STDMETHODIMP EdgePersistor::put_MkEngine(IMoniker *val) {
	METHTRACE(EdgePersistor,put_MkEngine);
	if(!m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> old = m_mkEngine;
	m_mkEngine = val;
	RUN(m_group->AddStgRef(m_mkEngine));
	if(old)
		RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
STDMETHODIMP EdgePersistor::put_MkTailNode(IMoniker *val) {
	METHTRACE(EdgePersistor,put_MkTailNode);
	if(!m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> old = m_mkTailNode;
	m_mkTailNode = val;
	RUN(m_group->AddStgRef(m_mkTailNode));
	if(old)
		RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
STDMETHODIMP EdgePersistor::put_MkHeadNode(IMoniker *val) {
	METHTRACE(EdgePersistor,put_MkHeadNode);
	if(!m_group)
		ERR(E_PENDING);
	CComPtr<IMoniker> old = m_mkHeadNode;
	m_mkHeadNode = val;
	RUN(m_group->AddStgRef(m_mkHeadNode));
	if(old)
		RUN(m_group->ReleaseStgRef(old));
	return S_OK;
}
// IGroupItem
STDMETHODIMP EdgePersistor::Connect(IGroup *group) {
	METHTRACE(EdgePersistor,Connect);
	m_group = group;
	if(!m_group)
		m_subject.Release();
	return S_OK;
}
STDMETHODIMP EdgePersistor::Copy(IBindCtx *bctx, IGroup *dest, IUnknown *aggr, REFIID riid, void **ret) {
	METHTRACE(EdgePersistor,Copy);
	CComPtr<IDGEdgePersistor> ep2;
	RUN(CREATE(ep2,CLSID_DGEdgePersistor));
	CComPtr<IMoniker> mkE2,mkTN2,mkHN2;
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkEngine,IID_NULL,0,&mkE2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkTailNode,IID_NULL,0,&mkTN2));
	RUN(dest->Paste(bctx,GP_SrcIsCont,m_group,m_mkHeadNode,IID_NULL,0,&mkHN2));
	RUN(CComQIPtr<IGroupItem>(ep2)->Connect(dest));
	RUN(ep2->put_Cache(m_cache));
	RUN(ep2->put_MkEngine(mkE2));
	RUN(ep2->put_MkTailNode(mkTN2));
	RUN(ep2->put_MkHeadNode(mkHN2));
	if(m_subject)
		RUN(CComQIPtr<DIPersistor>(ep2)->SetSubject(m_subject!=0, m_subject)); // realized==have subject
	return ep2->QueryInterface(riid,ret);
}
STDMETHODIMP EdgePersistor::Realize(IBindCtx *bctx) {
	METHTRACE(EdgePersistor,Realize);
	if(m_subject)
		return S_FALSE;
	PointerMoniker mkG(m_group);
	CComPtr<IBindCtx> bctx2 = bctx?bctx:BindCtx(); // yick, bctx should really be required
	CComPtr<IDGLayout> eng;
	RUN(m_mkEngine->BindToObject(bctx2,mkG,riidret(&eng)));
	CComPtr<IDGNode> tn,hn;
	RUN(m_mkTailNode->BindToObject(bctx2,mkG,riidret(&tn)));
	RUN(m_mkHeadNode->BindToObject(bctx2,mkG,riidret(&hn)));
	CComPtr<IDGEdge> e;
	RUN(eng->new_Edge(m_name,tn,hn,&e));
	RUN(e.QueryInterface(&m_subject));
	if(m_cache) {
		RUN(CComQIPtr<IPersistStream>(m_subject)->Load(m_cache));
		RUN(m_cache->Seek(castLarge(0),STREAM_SEEK_SET,0)); // in case someone else has this cache
		m_cache.Release();
	}
	return S_OK;
}
STDMETHODIMP EdgePersistor::Remove() {
	METHTRACE(EdgePersistor,Remove);
	RUN(m_group->ReleaseStgRef(m_mkEngine));
	RUN(m_group->ReleaseStgRef(m_mkTailNode));
	RUN(m_group->ReleaseStgRef(m_mkHeadNode));
	return S_OK;
}
// IPersistStream
STDMETHODIMP EdgePersistor::GetClassID(CLSID *pClassID) {
	METHTRACE(EdgePersistor,GetClassID);
	*pClassID = GetObjectCLSID();
	return S_OK;
}
STDMETHODIMP EdgePersistor::IsDirty(void) {
	METHTRACE(EdgePersistor,IsDirty);
	return S_FALSE;
}
STDMETHODIMP EdgePersistor::Load(IStream *pStm) {
	METHTRACE(EdgePersistor,Load);
	RUN(m_name.ReadFromStream(pStm));
	RUN(OleLoadFromStream(pStm,riidret(&m_mkEngine)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkTailNode)));
	RUN(FixedOleLoadFromStream(pStm,riidret(&m_mkHeadNode)));
	return cacheStreamableObject(pStm,&m_cache);
}
STDMETHODIMP EdgePersistor::Save(IStream *pStm,BOOL fClearDirty) {
	METHTRACE(EdgePersistor,Save);
	RUN(m_name.WriteToStream(pStm));
	RUN(OleSaveToStream(m_mkEngine,pStm));
	RUN(FixedOleSaveToStream(m_mkTailNode,pStm));
	RUN(FixedOleSaveToStream(m_mkHeadNode,pStm));
	if(m_subject)
		return saveWithSize(pStm,CComQIPtr<IPersistStream>(m_subject),fClearDirty);
	else {
		if(!m_cache)
			ERR(E_PENDING);
		return writeCache(pStm,m_cache);
	}
}
STDMETHODIMP EdgePersistor::GetSizeMax(ULARGE_INTEGER *pcbSize) {
	METHTRACE(EdgePersistor,GetSizeMax);
	CComObjectStack<countStream> count;
	RUN(Save(&count,false));
	*pcbSize = count.m_pos;
	return S_OK;
}
// IPersistor
STDMETHODIMP EdgePersistor::SetSubject(VARIANT_BOOL realized, IUnknown *pUnkSite) {
	METHTRACE(EdgePersistor,SetSubject);
	// for our purposes, having a subject means being realized (this persistor doesn't support
	// connecting a live object with Realize because there's no demand (yet?)).  So we can ignore
	// realized param.
	QISET(m_subject = pUnkSite);
	return S_OK;
}
STDMETHODIMP EdgePersistor::get_Subject(IUnknown **ret) {
	METHTRACE(EdgePersistor,GetSubject);
	m_subject.CopyTo((IDGDrawnEdge**)ret);
	return S_OK;
}
