/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include <io.h>
#include "IncrClient.h"
#include <list>
#include "includeDynagraph.h"
#include "makePathToBin.h"

typedef void (*fun)(void*);
FILE *errstream;
uintptr_t g_readThread = 0,g_errThread = 0;

IncrClient::IncrClient() :
    m_log(0),m_instream(0),m_outstream(0)
{
    char buf[400];
    makePathToBin("mglog.txt",buf);
	m_log = fopen(buf,"w");
    if(m_log)
	    setvbuf(m_log,0,_IONBF,0);
}
IncrClient::~IncrClient() {
	Stop();
    if(m_log)
	    fclose(m_log);
	m_log = 0;
}

void tellerrors() {
	while(errstream&&!feof(errstream)) {
		char buf[200];
		if(fgets(buf,200,errstream))
			MessageBox(0,buf,"graph server error",MB_OK);
	}
	if(errstream)
		MessageBox(0,"graph server unexpectedly closed","incrclient error",MB_OK);
}
list<string> g_buffer;
void IncrClient::readingThread(void*gs0) {
	IncrClient *gs = reinterpret_cast<IncrClient*>(gs0);
	FILE *f = gs->m_instream;
	int locks = 0,braces = -1;
	g_buffer.push_back(string());
	while(!feof(f)) {
		char buf[1000];
		if(!fgets(buf,1000,f))
			continue;
		g_buffer.back().append(buf);
		if(braces>=0)
			for(char *c = buf; *c; ++c)
				if(*c=='{') braces++;
				else if(*c=='}')
					if(!--braces)
						braces=-1;
		// this is clumsy because it will delay if events on two views overlap
		// when is that going to happen?
		if(!strncmp(buf,"lock",4))
			locks++;
		else if(!strncmp(buf,"unlock",6))
			locks--;
		else if(!strncmp(buf,"segue",5))
			braces = 0;
		if(!locks && braces<0) {
			g_buffer.push_back(string());
			SLOOK(fireEvent(gs->GetUnknown(),fireVoid<IDGIncrClientEvents>(&IDGIncrClientEvents::ReceivedResponse)));
		}
	}
}
void IncrClient::startThreads() {
	// start a thread to read input
	g_readThread = _beginthread(readingThread,0,this);
	// and one to dump errors
	errstream = _fdopen(m_piped.errhand,"r");
	g_errThread = _beginthread((fun)tellerrors,0,0);
}
void IncrClient::stopThreads() {
	/*
	if(g_readThread)
		_endthread(g_readThread),
		g_readThread = 0;
	if(g_errThread)
		_endthread(g_errThread),
		g_errThread = 0;
		*/
}
void IncrClient::parse() {
    try {
	    incr_yyparse();
    }
    catch(IncrError ixep) {
        handleIncrError(ixep);
    }
    catch(DGException xep) {
        handleDGException(xep);
    }
}
FILE *g_inlog = 0;
STDMETHODIMP IncrClient::StartServer(LPCOLESTR cmd,int nargs,LPCOLESTR *args) {
    IMETHTRACE(IncrClient,IDGIncrClient,Start);
	if(m_piped.process)
		RUN(Stop());
	else if(m_outstream)
		fclose(m_outstream),
		m_outstream = 0;
	USES_CONVERSION;
	char **argv = new char*[nargs+1];
	argv[nargs] = 0;
	for(int i=0; i<nargs; ++i) {
		int n = wcslen(args[i])+1;
		argv[i] = new char[n];
		ATLW2AHELPER(argv[i],args[i],n);
	}
	m_piped.Start(W2A(cmd),argv);
    HRESULT hr = S_OK;
	if(!m_piped.process) {
		char buf[600];
		sprintf(buf,"Couldn't start large graph server %s",argv[0]);
		MessageBox(0,buf,"Dynagraph",MB_OK);
		HRERR(E_FAIL);
	}
	for(int i=0; i<nargs; ++i)
		delete [] argv[i];
	delete [] argv;
    if(FAILED(hr))
        return hr;
	m_instream = _fdopen(m_piped.inhand,"r");
	setvbuf(m_instream,0,_IONBF,0);
	m_outstream = _fdopen(m_piped.outhand,"w");
	setvbuf(m_outstream,0,_IONBF,0);
	startThreads();
	return S_OK;
}
STDMETHODIMP IncrClient::FakeIt(LPCOLESTR incrfile,LPCOLESTR viewName) {
	USES_CONVERSION;
    m_instream = tmpfile();
    for(FILE *f = fopen(W2A(incrfile),"r");!feof(f);) {
        char buf[500];
        if(!fgets(buf,500,f))
            break;
        for(char *p = buf,*last = p; p = strstr(buf,"$V$"); last = p+3) {
            *p = 0;
            fputs(last,m_instream);
            fprintf(m_instream,"%S",viewName);
        }
        fputs(last,m_instream);
    }
    fseek(m_instream,0,SEEK_SET);
	_beginthread(readingThread,0,this);
	return S_OK;
}
STDMETHODIMP IncrClient::Write(const void *text, int n) {
	if(m_piped.process) {
	    fwrite(text,1,n,m_outstream);
        if(m_log)
	        fwrite(text,1,n,m_log);
    }
	else {
		FILE *t = tmpfile();
		setvbuf(t,0,_IONBF,0);
		fwrite(text,1,n,t);
		fseek(t,0,SEEK_SET);
		incr_yyin = t;
		parse();
		fclose(t);
	}
	return S_OK;
}
STDMETHODIMP IncrClient::Stop() {
	stopThreads();
	m_piped.Stop();
	if(errstream)
		fclose(errstream),
		errstream = 0;
	m_instream = m_outstream = 0;
	return S_OK;
}
STDMETHODIMP IncrClient::ProcessResponse() {
	if(g_buffer.size()<2)
		return S_FALSE;
	if(!g_inlog) {
        char buf[400];
        makePathToBin("mginlog.txt",buf);
		g_inlog = fopen(buf,"w");
		setvbuf(g_inlog,0,_IONBF,0);
	}
	static bool oops = false;
	_ASSERTE(!oops);
	oops = true;
	FILE *t = tmpfile();
	setvbuf(t,0,_IONBF,0);
    string &buf = g_buffer.front();
	fwrite(buf.c_str(),1,g_buffer.front().length(),t);
	fwrite(buf.c_str(),1,g_buffer.front().length(),g_inlog);
	fseek(t,0,SEEK_SET);
	incr_yyin = t;
	parse();
	fclose(t);
	g_buffer.pop_front();
	oops = false;
	return S_OK;
}
