/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "LayoutChoices.h"
#include "a2olestrdup.h"

LayoutChoices::LayoutChoices() {
    CRegKey options;
	if(!options.Open(HKEY_CURRENT_USER,"Software\\AT&T\\Dynagraph")) {
        TCHAR def[50];
        DWORD n = 50;
        if(!options.QueryValue("Default Layout",0,def,&n))
            m_default = def;
		CRegKey choices;
		if(!choices.Open(options,"Layout Choices")) {
			TCHAR choiceEntry[10],choiceName[50],choiceValue[200];
			int i;
			for(i=0;i<MAX_LAYOUT_CHOICES;++i) {
				wsprintf(choiceEntry,"Choice%d",i+1);
                CRegKey choice;
                if(choice.Open(choices,choiceEntry))
                    continue;
				DWORD n = 50;
                CHECK(!choice.QueryValue(0,0,choiceName,&n));
                n = 200;
                CHECK(!choice.QueryValue("Engines",0,choiceValue,&n));
                m_choices[choiceName] = choiceValue;
			}
		}
	}
}
LayoutChoices::~LayoutChoices() {
	CRegKey options;
	CHECK(!options.Create(HKEY_CURRENT_USER,"Software\\AT&T\\Dynagraph"));
    CHECK(!options.SetValue("Default Layout",REG_SZ,m_default.c_str(),m_default.size()+1));
	CRegKey recent;
	CHECK(!recent.Create(options,"Layout Choices"));
	int j(1);
    for(map<string,string>::iterator chi = m_choices.begin(); j<=MAX_LAYOUT_CHOICES && chi!=m_choices.end(); ++chi) {
        TCHAR choiceEntry[10];
        wsprintf(choiceEntry,"Choice%d",j);
        CRegKey choice;
        CHECK(!choice.Create(recent,choiceEntry));
        CHECK(!choice.SetValue(0,REG_SZ,chi->first.c_str(),chi->first.length()+1));
        CHECK(!choice.SetValue("Engines",REG_SZ,chi->second.c_str(),chi->second.length()+1));
	}
}
STDMETHODIMP LayoutChoices::get_Name(BSTR *ret) {
	USES_CONVERSION;
    *ret = SysAllocStringLen(A2W(m_default.c_str()),m_default.length());
    return S_OK;
}
STDMETHODIMP LayoutChoices::put_Name(BSTR val) {
	USES_CONVERSION;
    m_default = W2A(val);
    return S_OK;
}
STDMETHODIMP LayoutChoices::GetAttr(BSTR name, BSTR *ret) {
	USES_CONVERSION;
    map<string,string>::iterator ci = m_choices.find(W2A(name));
    if(ci==m_choices.end())
        ERR(E_FAIL);
    *ret = SysAllocStringLen(A2W(ci->second.c_str()),ci->second.length());
	return S_OK;
}
STDMETHODIMP LayoutChoices::SetAttr(BSTR name, BSTR value) {
	USES_CONVERSION;
    m_choices[W2A(name)] = W2A(value);
	return S_OK;
}
STDMETHODIMP LayoutChoices::EnumAttrs(IEnumString **ret) {
    // more stupid copied code...
    const size_t N = m_choices.size();
    LPOLESTR *ray = new LPOLESTR[N],*ri=ray;
    for(map<string,string>::iterator ci = m_choices.begin();ci!=m_choices.end(); ++ci)
        *ri++ = a2olestrdup(ci->first);
	typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > en;
	en *e = new en;
	e->Init(ray,ray+N,0,AtlFlagTakeOwnership);
	(*ret = e)->AddRef();
    return S_OK;
}
