/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "StringHandler.h"

struct Engine;

struct NodeLabelWrap;

struct Node :
	CComObjectRoot,
	CComCoClass<Node,&CLSID_DGNode>,
	IConnectionPointContainerImpl<Node>,
	IConnectionPointImpl<Node, &IID_IDGNodeShadow>,
	IDGDrawnNode,
	IDispatchImpl<DIDGDrawnNode>,
	IDispatchImpl<DIDGShapeDefinition>,
	IPersistStream,
	ChStringHandler<Node>
{
	Engine *m_engine;
	Layout::Node *m_node;
	const int secret_code;
	bool m_destroying,
        m_incrInserted; // whether inserted at this moment in the outgoing incr stream

	NodeGeom &geom() {
		return gd<NodeGeom>(m_node);
	}
	NodeLabels &labels() {
		return gd<NodeLabels>(m_node);
	}

	vector<NodeLabelWrap*> labelz;

	Node();
	~Node();

	// for ChStringHandler
	Layout::Node *sh_object() { return m_node; }
	void sh_modify(StrAttrs attrs);
	void sh_rename(DString name);
	HRESULT mod(Update upd);

	HRESULT figureInside(); // find largest
// call-ins from engine
	void show();
	void hide();
BEGIN_COM_MAP(Node)
	COM_INTERFACE_ENTRY(IDGNode)
	COM_INTERFACE_ENTRY(IDGDrawnNode)
	COM_INTERFACE_ENTRY(DIDGDrawnNode)
	COM_INTERFACE_ENTRY2(IDispatch,DIDGDrawnNode)
	COM_INTERFACE_ENTRY(IDGStringAttributes)
	COM_INTERFACE_ENTRY(DIDGShapeDefinition)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IPersist)
	COM_INTERFACE_ENTRY(IPersistStream)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(Node)
   CONNECTION_POINT_ENTRY(IID_IDGNodeShadow)
END_CONNECTION_POINT_MAP()

// IDGNode
	STDMETHODIMP get_Graph(IDGGraph **ppOut);
    STDMETHODIMP get_Edges(IEnumDGEdges **pEdges);
    STDMETHODIMP get_OutEdges(IEnumDGEdges **pEdges);
    STDMETHODIMP get_InEdges(IEnumDGEdges **pEdges);
    STDMETHODIMP get_Pos(PPOINTD ppos);
    STDMETHODIMP put_Pos(POINTD pos);
    STDMETHODIMP put_Shape(DGSHAPESDESC *val);
	STDMETHODIMP get_Shape(DGSHAPESDESC **ret);
	STDMETHODIMP GetRect(RECTD *ret);
	STDMETHODIMP new_Label(IDGNodeLabel **ret);
    STDMETHODIMP Insert();
    STDMETHODIMP Delete();
	STDMETHODIMP Optimize(VARIANT_BOOL full);
// DIDGDrawnNode:
	STDMETHODIMP put_BorderColor(OLE_COLOR clr);
	STDMETHODIMP get_BorderColor(OLE_COLOR* pclr);
	STDMETHODIMP put_ForeColor(OLE_COLOR clr);
	STDMETHODIMP get_ForeColor(OLE_COLOR* pclr);
	STDMETHODIMP put_FillColor(OLE_COLOR clr);
	STDMETHODIMP get_FillColor(OLE_COLOR* pclr);
	STDMETHODIMP get_Engine(DIDGLayout **ppEngine);
	STDMETHODIMP get_Edges(IDispatch **pEdges);
	STDMETHODIMP get_OutEdges(IDispatch **pEdges);
	STDMETHODIMP get_InEdges(IDispatch **pEdges);
	STDMETHODIMP get_X(double *ret);
	STDMETHODIMP get_Y(double *ret);
	STDMETHODIMP SetPos(double x, double y);
	STDMETHODIMP put_Shape(DIDGShapes *shapes);
// DIDGShapeDefinition
	STDMETHODIMP get_IsEllipse(VARIANT_BOOL *ret);
	STDMETHODIMP put_IsEllipse(VARIANT_BOOL val);
	STDMETHODIMP get_Regular(VARIANT_BOOL *ret);
	STDMETHODIMP put_Regular(VARIANT_BOOL val);
	STDMETHODIMP get_Sides(int *ret);
	STDMETHODIMP put_Sides(int val);
	STDMETHODIMP get_Peripheries(int *ret);
	STDMETHODIMP put_Peripheries(int val);
	STDMETHODIMP get_Spacing(double *ret);
	STDMETHODIMP put_Spacing(double val);
	STDMETHODIMP get_Rotation(double *ret);
	STDMETHODIMP put_Rotation(double val);
	STDMETHODIMP get_Skew(double *ret);
	STDMETHODIMP put_Skew(double val);
	STDMETHODIMP get_Distortion(double *ret);
	STDMETHODIMP put_Distortion(double val);
	STDMETHODIMP get_InsideWidth(double *ret);
	STDMETHODIMP get_InsideHeight(double *ret);
	STDMETHODIMP SetInside(double width,double height);
	STDMETHODIMP get_OutsideWidth(double *ret);
	STDMETHODIMP get_OutsideHeight(double *ret);
	STDMETHODIMP SetOutside(double width,double height);
// IPersistStream
    STDMETHODIMP GetClassID(CLSID *pClassID);
    STDMETHODIMP IsDirty(void);
    STDMETHODIMP Load(IStream *stream);
    STDMETHODIMP Save(IStream *stream,BOOL fClearDirty);
    STDMETHODIMP GetSizeMax(ULARGE_INTEGER *pcbSize);
};
