/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


DG_NODELABELALIGN translateAlign(DG_NODELABELALIGN a1,DG_ORIENTATION o1,DG_ORIENTATION o2) {
	if(a1==DGNLA_CENTER || a1==DGNLA_BETTER)
		return a1;
	int d = (o2-o1+4)%4,
		a = ((a1-DGNLA_RIGHT)+d)%4 + DGNLA_RIGHT;
	_ASSERTE(a>=DGNLA_RIGHT && a<=DGNLA_BOTTOM);
	return DG_NODELABELALIGN(a);
/*
	switch(o) {
	case DGO_DOWN:
		return a1;
	case DGO_RIGHT:
		switch(a1) {
		case DGNLA_BOTTOM:
			return DGNLA_LEFT;
		case DGNLA_TOP:
			return DGNLA_RIGHT;
		case DGNLA_RIGHT:
			return DGNLA_BOTTOM;
		case DGNLA_LEFT:
			return DGNLA_TOP;
		}
	}
	return DGNLA_CENTER;
	*/
}
struct Node;
struct NodeLabelWrap : IDGNodeLabel, CComObjectRoot {
	Node *n;
	unsigned x;

BEGIN_COM_MAP(NodeLabelWrap)
	COM_INTERFACE_ENTRY(IDGLabel)
	COM_INTERFACE_ENTRY(IDGNodeLabel)
END_COM_MAP();

	NodeLabel &label() {
		return n->labels()[x];
	}

	STDMETHODIMP Delete() {
		for(unsigned i = x+1; i<n->labelz.size(); ++i)
			n->labelz[i]->x--;
		n->labelz.erase(n->labelz.begin()+x);
		n->labels().erase(n->labels().begin()+x);
		n = 0;
		Release();
		return S_OK;
	}
	STDMETHODIMP get_Size(POINTD *ret) {
		*ret = DG2Win(label().size);
		return S_OK;
	}
	STDMETHODIMP put_Size(POINTD val) {
		label().size = Win2DG(val);
		n->m_engine->GraphLock();
		if(label().align==DGNLA_CENTER)
			n->figureInside();
		n->mod(DG_UPD_LABEL);
		n->m_engine->GraphUnlock();
		return S_OK;
	}
	STDMETHODIMP get_Align(enum DG_NODELABELALIGN *ret) {
		*ret = translateAlign(static_cast<enum DG_NODELABELALIGN>(label().align),n->m_engine->m_orient,DGO_DOWN);
		return S_OK;
	}
	STDMETHODIMP put_Align(enum DG_NODELABELALIGN val) {
		enum DG_NODELABELALIGN val2 = translateAlign(val,DGO_DOWN,n->m_engine->m_orient);
		NodeLabelAlignment nla = static_cast<NodeLabelAlignment>(val2);
		if(label().align!=nla) {
			bool wasCenter = label().align==DGNLA_CENTER;
			label().align = nla;
			n->m_engine->GraphLock();
			if(wasCenter||nla==DGNLA_CENTER)
				n->figureInside();
			n->mod(DG_UPD_LABEL);
			n->m_engine->GraphUnlock();
		}
		return S_OK;
	}
	STDMETHODIMP get_NodeAlign(POINTD *ret) {
		*ret = DG2Win(label().nodeAlign);
		return S_OK;
	}
	STDMETHODIMP put_NodeAlign(POINTD val) {
		label().nodeAlign = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_LabelAlign(POINTD *ret) {
		*ret = DG2Win(label().labelAlign);
		return S_OK;
	}
	STDMETHODIMP put_LabelAlign(POINTD val) {
		label().labelAlign = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_NodeOffset(POINTD *ret) {
		*ret = DG2Win(label().nodeOffset);
		return S_OK;
	}
	STDMETHODIMP put_NodeOffset(POINTD val) {
		label().nodeOffset = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_LabelOffset(POINTD *ret) {
		*ret = DG2Win(label().labelOffset);
		return S_OK;
	}
	STDMETHODIMP put_LabelOffset(POINTD val) {
		label().labelOffset = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_ScaleX(VARIANT_BOOL *ret) {
		*ret = label().scaleX*-1;
		return S_OK;
	}
	STDMETHODIMP put_ScaleX(VARIANT_BOOL val) {
		label().scaleX = val;
		return S_OK;
	}
	STDMETHODIMP get_ScaleY(VARIANT_BOOL *ret) {
		*ret = label().scaleY*-1;
		return S_OK;
	}
	STDMETHODIMP put_ScaleY(VARIANT_BOOL val) {
		label().scaleY = val;
		return S_OK;
	}
	STDMETHODIMP get_NodeAlign2(POINTD *ret) {
		*ret = DG2Win(label().nodeAlign2);
		return S_OK;
	}
	STDMETHODIMP put_NodeAlign2(POINTD val) {
		label().nodeAlign2 = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_LabelAlign2(POINTD *ret) {
		*ret = DG2Win(label().labelAlign2);
		return S_OK;
	}
	STDMETHODIMP put_LabelAlign2(POINTD val) {
		label().labelAlign2 = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_LabelOffset2(POINTD *ret) {
		*ret = DG2Win(label().labelOffset2);
		return S_OK;
	}
	STDMETHODIMP put_LabelOffset2(POINTD val) {
		label().labelOffset2 = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_NodeOffset2(POINTD *ret) {
		*ret = DG2Win(label().nodeOffset2);
		return S_OK;
	}
	STDMETHODIMP put_NodeOffset2(POINTD val) {
		label().nodeOffset2 = Win2DG(val);
		n->mod(DG_UPD_LABEL);
		return S_OK;
	}
	STDMETHODIMP get_Bounds(RECTD *ret) {
		if(!n || !label().bounds.valid)
			ERR(E_PENDING);
		*ret = DG2Win(label().bounds);
		return S_OK;
	}
};
