/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "a2olestrdup.h"

// T implements sh_object() (returns some LGraph node derived from Name & StrAttrs),
// sh_rename(char *) and sh_modify(StrAttrs)
template<typename T>
struct StringHandler : IDGStringAttributes
{
// IDGStringAttributes
	STDMETHODIMP get_Name(BSTR *ret) {
		T *me = static_cast<T*>(this);
		USES_CONVERSION;
		const DString &name = gd<Name>(me->sh_object());
		*ret = SysAllocStringLen(A2W(name.c_str()),name.length());
		return S_OK;
	}
	STDMETHODIMP put_Name(BSTR val) {
		T *me = static_cast<T*>(this);
		USES_CONVERSION;
		me->sh_rename(W2A(val));
		return S_OK;
	}
	STDMETHODIMP GetAttr(BSTR name, BSTR *ret) {
		if(!_wcsicmp(name,L"id"))
			return get_Name(ret);
		T *me = static_cast<T*>(this);
		USES_CONVERSION;
		DString nam = W2A(name);
		const StrAttrs &attrs = gd<StrAttrs>(me->sh_object());
		StrAttrs::const_iterator ai = attrs.find(nam);
		if(ai==attrs.end())
			ERR(E_FAIL);
		*ret = SysAllocStringLen(A2W(ai->second.c_str()),ai->second.length());
		return S_OK;
	}
	STDMETHODIMP SetAttr(BSTR name, BSTR value) {
		T *me = static_cast<T*>(this);
		USES_CONVERSION;
		StrAttrs attrs;
		attrs[W2A(name)] = W2A(value);
		me->sh_modify(attrs);
		return S_OK;
	}
    STDMETHODIMP EnumAttrs(IEnumString **ret) {
		T *me = static_cast<T*>(this);
        USES_CONVERSION;
		const StrAttrs &attrs = gd<StrAttrs>(me->sh_object());
        const size_t N = attrs.size();
        LPOLESTR *ray = new LPOLESTR[N],*ri=ray;
        for(StrAttrs::const_iterator ci = attrs.begin();ci!=attrs.end(); ++ci)
            *ri++ = a2olestrdup(ci->first.c_str());
	    typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > en;
	    en *e = new en;
	    e->Init(ray,ray+N,0,AtlFlagTakeOwnership);
	    (*ret = e)->AddRef();
        return S_OK;
    }
    STDMETHODIMP EnumChangedAttrs(IEnumString **ret) {
		ERR(E_FAIL);
    }
};

template<typename T>
struct ChStringHandler : StringHandler<T> {
    STDMETHODIMP EnumChangedAttrs(IEnumString **ret) {
		T *me = static_cast<T*>(this);
        USES_CONVERSION;
		const StrAttrChanges &changes = gd<StrAttrChanges>(me->sh_object());
        const size_t N = changes.size();
        LPOLESTR *ray = new LPOLESTR[N],*ri=ray;
        for(StrAttrChanges::const_iterator ci = changes.begin();ci!=changes.end(); ++ci)
            *ri++ = a2olestrdup(ci->c_str());
	    typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > en;
	    en *e = new en;
	    e->Init(ray,ray+N,0,AtlFlagTakeOwnership);
	    (*ret = e)->AddRef();
	    return S_OK;
    }
};
