/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "pipeProcess.h"
#include <io.h>
#include <fcntl.h>

void PipedProcess::Start(const char *cmd,const char *const *argv) {
	// direct our stdin and stdout to pipes, spawn the process, and then restore
	// redirecting is a matter of duplicating
	int pipeIn[2],pipeOut[2],pipeErr[2];

	if(_pipe(pipeIn,512,O_BINARY | O_NOINHERIT) == -1)
		throw DupProblem();
	if(_pipe(pipeOut,512,O_BINARY | O_NOINHERIT) == -1)
		throw DupProblem();
	if(_pipe(pipeErr,512,O_BINARY | O_NOINHERIT) == -1)
		throw DupProblem();

	int i = 0,//_fileno(stdin),
		o = 1,//_fileno(stdout);
		e = 2; // stderr

	int stdoutBak = _dup(o),
		stdinBak = _dup(i),
		stderrBak = _dup(e);

	if(_dup2(pipeOut[0],i)!=0)
		throw DupProblem();
	close(pipeOut[0]);
	if(_dup2(pipeIn[1],o)!=0)
		throw DupProblem();
	close(pipeIn[1]);
	if(_dup2(pipeErr[1],e)!=0)
		throw DupProblem();
	close(pipeErr[1]);

	process = _spawnv(P_NOWAIT, cmd, argv);
	if(process==-1)
		process = 0;

	if(_dup2(stdinBak,i)!=0)
		throw DupProblem();
	close(stdinBak);
	if(_dup2(stdoutBak,o)!=0)
		throw DupProblem();
	close(stdoutBak);
	if(_dup2(stderrBak,e)!=0)
		throw DupProblem();
	close(stderrBak);

	inhand = pipeIn[0];
	outhand = pipeOut[1];
	errhand = pipeErr[0];
}
void PipedProcess::Stop() {
	_close(outhand);
	_cwait(0,process,0);
	_close(inhand);
	_close(errhand);
	inhand = outhand = errhand = 0;
	process = 0;
}