/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// clickNodesMode.cpp : Implementation of clickNodesMode

#include "stdafx.h"
#include "ClickNodes.h"
#include "clickNodesMode.h"
#include "createAffixMoniker.h"

#include <commctrl.h>

// dumber tooltip because I kept crashing the computer trying to use
// the real tooltips in a half-assed way
struct foolTip {
	HWND hWnd;
	char line1[200];
	char line2[200];
	static const char *className;
	static bool regged;
	~foolTip() {
		if(hWnd)
			DestroyWindow(hWnd);
		hWnd = 0;
	}
	foolTip() {
		if(!regged) {
			WNDCLASS wc;
			wc.style = CS_SAVEBITS;
			wc.lpfnWndProc = WndProc;
			wc.cbClsExtra = 0;
			wc.cbWndExtra = 4;
			wc.hInstance = _Module.GetModuleInstance();
			wc.hIcon = 0;
			wc.hCursor = 0;
			wc.hbrBackground = HBRUSH(COLOR_INFOBK+1);
			wc.lpszMenuName = 0;
			wc.lpszClassName = className;
			RegisterClass(&wc);
			regged = true;
		}
		hWnd = CreateWindow(className,"clickNodes fooltip",WS_POPUP|WS_BORDER,0,0,0,0,0,0,_Module.GetModuleInstance(),0);
		SetWindowLong(hWnd,0,(LONG)this);
		line1[0] = line2[0] = 0;
	}
	static LRESULT CALLBACK WndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) {
		foolTip *that = (foolTip*)GetWindowLong(hwnd,0);
		switch(msg) {
		case WM_PAINT: {
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd,&ps);
			SetTextAlign(hdc,TA_TOP|TA_LEFT);
			SetTextColor(hdc,GetSysColor(COLOR_INFOTEXT));
			SetBkColor(hdc,GetSysColor(COLOR_INFOBK));
			TextOut(hdc,1,0,that->line1,strlen(that->line1));
			SIZE size1;
			GetTextExtentPoint32(hdc,that->line1,strlen(that->line1),&size1);
			TextOut(hdc,1,size1.cy,that->line2,strlen(that->line2));
			EndPaint(hwnd,&ps);
			return 0;
		}
		default:
			return DefWindowProc(hwnd,msg,wParam,lParam);
		}
	}
	void place(POINT pt) {
		HDC hdcTemp = GetDC(hWnd);
		SIZE size1,size2;
		GetTextExtentPoint32(hdcTemp,line1,strlen(line1),&size1);
		GetTextExtentPoint32(hdcTemp,line2,strlen(line2),&size2);
		SetWindowPos(hWnd,HWND_TOP,pt.x,pt.y+15,__max(size1.cx,size2.cx)+3,size1.cy+size2.cy,SWP_NOACTIVATE);
		ReleaseDC(hWnd,hdcTemp);
	}
	void show(bool whether) {
		ShowWindow(hWnd,whether?SW_SHOWNOACTIVATE:SW_HIDE);
	}
};
const char *foolTip::className = "FoolTip";
bool foolTip::regged = false;

CComPtr<IUnknown> getSubject(IUnknown *persistor) {
	CComPtr<IUnknown> ret;
	SRUN(CComQIPtr<DIPersistor>(persistor)->get_Subject(&ret));
	return ret;
}

///////////
// siteNaming
siteNaming::~siteNaming() {
	iterator j;
	for(iterator i=begin();i!=end();i = j) {
		j = i;
		++j;
		SysFreeString(i->first);
		i->second->Release();
	}
}
/////////////////////////////////////////////////////////////////////////////
// clickNodesMode
clickNodesMode::clickNodesMode() : m_fooltip(0)
{
}
STDMETHODIMP clickNodesMode::SetClientSite(IOleClientSite *s) {
	if(!s) {
		m_dgm.Release();
		m_group.Release();
		delete m_fooltip;
		m_fooltip = 0;
	}
	else {
		RUN(CreateBindCtx(0,&m_bctx));
		CComQIPtr<IMCCSite> s2 = s;
		if(s2==0)
			ERR(E_INVALIDARG);
		RUN(s2->GetContainer(riidret(&m_dgm)));
		// this all violates the group persistence model: clickNodesMode assumes that things will
		// be there but doesn't actually implement IGroupItem to make sure they will be.
		// This is OK for save and load, but if you were to paste the mode somewhere it would
		// get confused and crash.
		RUN(CComQIPtr<IMontageContainer>(m_dgm)->GetMainGroup(riidret(&m_group)));
		CComPtr<IMoniker> mkG;
		RUN(FoolishCreatePointerMoniker(m_group,&mkG));
		CComPtr<IBindCtx> bctx;
		RUN(CreateBindCtx(0,&bctx));
		for(siteNaming::iterator i = m_names.begin();i!=m_names.end();++i) {
			CComPtr<IMoniker> mkSite;
			RUN(createAffixMoniker(i->first,L"Site",&mkSite));
			IMCCSite *s; // hold ref for map
			RUN(ComposeMk(mkSite,GetSubjectMk())->BindToObject(bctx,mkG,riidret(&s)));
			i->second = s;
		}
		// erase active-node indication; the reload of the browser will fire events to re-show,
		// and the same node may not be active.
		if(m_mkActiveSite) {
			CComPtr<IMCCSite> s;
			RUN(ComposeMk(m_mkActiveSite,GetSubjectMk())->BindToObject(bctx,mkG,riidret(&s)));
            CComPtr<DIDGDrawnNode> n;
            RUN(layoutFromSite(s,&n));
			RUN(n->put_FillColor(RGB(255,255,255)));
		}
		// create tooltip
		m_fooltip = new foolTip;
		m_fooltip->show(false);
	}
	return IOleObjectImpl<clickNodesMode>::SetClientSite(s);
}
IUnknown *clickNodesMode::getComponent(LPOLESTR name) {
	CComPtr<IUnknown> ret;
	RUN(m_dgm->GetObject(name,BINDSPEED_INDEFINITE,m_bctx,riidret(&ret)));
	return ret;
}
HRESULT clickNodesMode::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	USES_CONVERSION;
	/*
	if(m_underText.length()) {
		SetTextAlign(di.hdcDraw,TA_BASELINE|TA_CENTER);
		SetBkColor(di.hdcDraw,RGB(212,255,255));
		ExtTextOut(di.hdcDraw,m_ptText.x,m_ptText.y,ETO_OPAQUE,&m_rcText,OLE2T(m_underText),m_underText.length(),0);
	}
	*/
	return S_OK;
}
LRESULT clickNodesMode::SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	::SetCursor(LoadCursor(0,IDC_ARROW));
	return 0;
}
#include "copyDGMLabelInfo.h"
HRESULT firstLabel(IDGMLabelMode *labelMode,IMCCTextEdit **ret) {
	CComPtr<IEnumDGMLabelInfo> labels;
	SRUN(labelMode->EnumLabels(&labels));
	enumLI li(labels);
	if(li!=enumLI())
		return li->content->GetControl(riidret(ret));
	CComPtr<IMCCSite> labelSite;
	CComPtr<IDGLabel> label;
	SRUN(labelMode->NewLabel(0,&labelSite,&label));
	SRUN(CComQIPtr<IDGNodeLabel>(label)->put_Where(DGNLA_CENTER));
	return labelSite->GetControl(riidret(ret));
	return S_OK;
}
HRESULT getFirstLabelTE(IUnknown *site,IMCCTextEdit **ret) {
	CComPtr<IDGMLabelMode> labelMode;
	// dumb way to get label mode
	if(FAILED(getSinkWhichImplements(site,IID_IMCCSiteEvents,&labelMode)))
		SERR(E_FAIL);
	return firstLabel(labelMode,ret);
}
LRESULT clickNodesMode::MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	METHTRACE(clickNodesMode,MouseMove);
	POINT pt = MAKEPOINT(lParam);
	if(m_ptLast.x==pt.x && m_ptLast.y==pt.y)
		return 0;
#define FOOLTIP
#ifdef FOOLTIP
	m_ptLast = pt;
	m_fooltip->show(false);
	CComPtr<IMCCSite> under;
	RUN(m_mainLayer->FindControl(pt,&under));
	//if(under != m_under) {
		/*
		if(m_under)
			RUN(m_spInPlaceSite->InvalidateRect(&m_rcText,true));
		*/
		m_under = under;
		if(m_under) {
			CComPtr<IMoniker> mkSite;
			RUN(under->get_RelMoniker(&mkSite));
			LPOLESTR name;
			RUN(mkSite->GetDisplayName(m_bctx,0,&name));
			// remove " Site", " Label Mode" from end.
			if(WCHAR *space = wcschr(name,L' '))
				*space = 0;
			siteNaming::iterator i = m_names.find(name+1);
			if(i!=m_names.end()) {
				CComBSTR cap;
				CComPtr<IDGMLabelMode> labelMode;
				CComPtr<IMCCTextEdit> textEdit;
				if((SUCCEEDED(under->GetControl(riidret(&labelMode)))&&SUCCEEDED(firstLabel(labelMode,&textEdit))) // hit label mode
					|| SUCCEEDED(under->GetControl(riidret(&textEdit))) //// hit label
					|| SUCCEEDED(getFirstLabelTE(under,&textEdit))) // hit node
					RUN(textEdit->get_Text(&cap));
				int len = cap.Length();
				if(len)
					AtlW2AHelper(m_fooltip->line1,cap,len);
				m_fooltip->line1[len] = 0;
				SysFreeString(cap);
				len = wcslen(name+1);
				if(len)
					AtlW2AHelper(m_fooltip->line2,name+1,len);
				m_fooltip->line2[len] = 0;
				CComPtr<IOleContainer> cont;
				RUN(m_spClientSite->GetContainer(&cont));
				HWND montageWnd;
				RUN(CComQIPtr<IOleWindow>(cont)->GetWindow(&montageWnd));
				::ClientToScreen(montageWnd,&pt);
				m_fooltip->place(pt);
				m_fooltip->show(true);
				//::SetTimer(1,1000);
				m_underText = name+1;
				/*
				HDC hdcScreen = ::GetDC(0);
				SIZE size;
				GetTextExtentPoint32(hdcScreen,OLE2T(name+1),wcslen(name+1),&size);
				RECT rc = {pt.x - size.cx/2,pt.y-size.cy,pt.x+size.cx/2,pt.y+size.cy};
				if(rc.left<m_rcPos.left)
					pt.x += m_rcPos.left-rc.left;
				else if(rc.right>m_rcPos.right)
					pt.x -= rc.right-m_rcPos.right;
				if(rc.bottom > m_rcPos.bottom)
					pt.y -= rc.bottom-m_rcPos.bottom;
				else if(rc.top < m_rcPos.top)
					pt.y += m_rcPos.top - rc.top;
				m_ptText = pt;
				RECT rc2 = {pt.x - size.cx/2,pt.y-size.cy,pt.x+size.cx/2,pt.y+size.cy};
				m_rcText = rc2;
				RUN(m_spInPlaceSite->InvalidateRect(&m_rcText,true));
				*/
			}
			else {
				m_underText.Empty();
				m_under = 0;
			}
			CoTaskMemFree(name);
		}
		else {
			//m_underText = (char*)0;
			//m_fooltip->show(false);
		}
	//}
#endif
	return 0;
}
LRESULT clickNodesMode::ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	return 0;
}
LRESULT clickNodesMode::ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	if(m_underText.Length()) {
		for(IUnknown **i = m_vec.begin();i!=m_vec.end();++i)
			if(CComQIPtr<IClickNodesModeEvents> sink = *i)
				RUN(sink->OnClick(m_underText));
	}
	return 0;
}
/*
LRESULT clickNodesMode::OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled) {
	return S_OK;
}
*/
// IClickNodesMode
STDMETHODIMP clickNodesMode::Initialize(IUnknown *mainLayer) {
	METHTRACE(clickNodesMode,Initialize);
	m_mainLayer = mainLayer;
	return S_OK;
}
STDMETHODIMP clickNodesMode::AddNode(BSTR id, BSTR caption) {
	METHTRACE(clickNodesMode,AddNode);
	CComPtr<IMCCSite> s;
	siteNaming::iterator i = m_names.find(id);
	if(i!=m_names.end())
		s = i->second;
	else {
		CComQIPtr<IDGMDrawer> drawer = getSubject(getComponent(L"Graph Drawer"));
		CComPtr<IMCCSitePersistor> siteper;
		MCC_POS_INFO mpi;
		mpi.postype = MCC_POS_CANVASSIZE;
		mpi.sl = size(5000,500);
		RUN(drawer->NewNode(0,&mpi,0,id,&siteper)); // create default node (eg from palette)

		// map it.
		CComPtr<IUnknown> uS;
		RUN(CComQIPtr<DIPersistor>(siteper)->get_Subject(&uS));
		RUN(uS.QueryInterface(&s));
		s.p->AddRef(); // keep ref for map
		m_names[SysAllocStringLen(id,SysStringLen(id))] = s;
	}
	CComPtr<IMCCTextEdit> textEdit;
	RUN(getFirstLabelTE(s,&textEdit));
	RUN(textEdit->put_Text(caption)); // maybe a refresh
    CComPtr<DIDGDrawnNode> n;
    RUN(layoutFromSite(s,&n));
    RUN(n->put_FillColor(RGB(255,212,128)));
	if(m_active!=0 && m_active != n)
		RUN(m_active->put_FillColor(RGB(255,255,255)));
	m_active = n;
	m_mkActiveSite.Release();
	RUN(s->get_RelMoniker(&m_mkActiveSite));
	return S_OK;
}
STDMETHODIMP clickNodesMode::AddEdge(BSTR idTail, BSTR idHead) {
	METHTRACE(clickNodesMode,AddEdge);
	siteNaming::iterator iT, iH;
	if((iT = m_names.find(idTail))==m_names.end())
		ERR(E_INVALIDARG);
	if((iH = m_names.find(idHead))==m_names.end())
		ERR(E_INVALIDARG);
	CComPtr<IMoniker> mkTail = mkLayoutFromSite(iT->second),
        mkHead = mkLayoutFromSite(iH->second);
	CComPtr<IMoniker> mkDGM;
	RUN(FoolishCreatePointerMoniker(m_dgm,&mkDGM));
	CComPtr<IDGNode> tail,head;
	RUN(mkTail->BindToObject(m_bctx,mkDGM,riidret(&tail)));
	RUN(mkHead->BindToObject(m_bctx,mkDGM,riidret(&head)));
	CComPtr<IDGEdge> edge;
	CComQIPtr<IDGLayout> engine = getComponent(L"Graph Layout Engine");
	RUN(engine->new_Edge(0,tail,head,&edge));
	CComQIPtr<IDGMDrawer> drawer = getSubject(getComponent(L"Graph Drawer"));
	return drawer->NewEdge(CComQIPtr<IDGDrawnEdge>(edge),0,0,0);
}
// IPersistStreamInit overrides
STDMETHODIMP clickNodesMode::Save(IStream *dest,BOOL clearDirty) {
	RUN(IPersistStreamInitImpl<clickNodesMode>::Save(dest,clearDirty));
	siteNaming::size_type n = m_names.size();
	RUN(dest->Write(&n,sizeof(n),0));
	for(siteNaming::iterator i = m_names.begin();i!=m_names.end();++i) {
		CComBSTR helper;
		helper.Attach(i->first);
		helper.WriteToStream(dest);
		helper.Detach();
	}
	RUN(FixedOleSaveToStream(m_mkActiveSite,dest));
	return S_OK;
}
STDMETHODIMP clickNodesMode::Load(IStream *source) {
	RUN(IPersistStreamInitImpl<clickNodesMode>::Load(source));
	siteNaming::size_type n;
	RUN(source->Read(&n,sizeof(n),0));
	while(n--) {
		CComBSTR helper;
		helper.ReadFromStream(source);
		m_names[helper.Detach()] = 0; // for now; bind on SetClientSite b/c we don't have group yet
	}
	RUN(FixedOleLoadFromStream(source,riidret(&m_mkActiveSite)));
	return S_OK;
}
