/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// clickNodesMode.h : Declaration of the clickNodesMode

#ifndef __CLICKNODESMODE_H_
#define __CLICKNODESMODE_H_

#include <atlctl.h>

#include <hash_map>

// stolen from cgroup.h
struct oleHashCompare {
	static const size_t bucket_size = 4,
		min_buckets = 8;
	bool operator ()(BSTR s1, BSTR s2) const {
		return wcscmp(s1,s2)<0;
	}
	size_t operator ()(BSTR s) const {
	  unsigned long h = 0;
	  for ( ; *s; ++s)
		h = 5*h + *s;

	  return size_t(h);
	}
};
// URL->site mapping
struct siteNaming : public std::hash_map<BSTR,IMCCSite*,oleHashCompare> {
	~siteNaming();
};

struct foolTip;
/////////////////////////////////////////////////////////////////////////////
// clickNodesMode
class ATL_NO_VTABLE clickNodesMode :
	public CComObjectRoot,
	public CStockPropImpl<clickNodesMode, IClickNodesMode, &IID_IClickNodesMode, &LIBID_CLICKNODESLib>,
	public CComControl<clickNodesMode>,
	public IPersistStreamInitImpl<clickNodesMode>,
	public IOleControlImpl<clickNodesMode>,
	public IOleObjectImpl<clickNodesMode>,
	public IOleInPlaceActiveObjectImpl<clickNodesMode>,
	public IViewObjectExImpl<clickNodesMode>,
	public IOleInPlaceObjectWindowlessImpl<clickNodesMode>,
	public IConnectionPointContainerImpl<clickNodesMode>,
	public IConnectionPointImpl<clickNodesMode,&IID_IClickNodesModeEvents>,
	public CComCoClass<clickNodesMode, &CLSID_ClickNodesMode>,
	public INeedOnePointer
{
	siteNaming m_names;
	CComPtr<IOleItemContainer> m_dgm;
	CComPtr<IGroup> m_group;
	CComQIPtr<IMCCLayer> m_mainLayer;
	CComPtr<IBindCtx> m_bctx;
	CComPtr<IMCCSite> m_under;
	CComBSTR m_underText;

	POINT m_ptLast;
	/*
	POINT m_ptText;
	RECT m_rcText;
	*/
	CComPtr<DIDGDrawnNode> m_active; //
	CComPtr<IMoniker> m_mkActiveSite; // mk to active just to deactivate on load
	foolTip *m_fooltip; // silly tooltip impl
public:
	clickNodesMode();

	STDMETHODIMP SetClientSite(IOleClientSite *s);

REGGY_RID(IDR_CLICKNODESMODE)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(clickNodesMode)
	COM_INTERFACE_ENTRY(IClickNodesMode)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(INeedOnePointer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(clickNodesMode)
	CONNECTION_POINT_ENTRY(IID_IClickNodesModeEvents)
END_CONNECTION_POINT_MAP()

BEGIN_PROP_MAP(clickNodesMode)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	PROP_ENTRY("BackColor", DISPID_BACKCOLOR, CLSID_StockColorPage)
	PROP_ENTRY("Caption", DISPID_CAPTION, CLSID_NULL)
	PROP_ENTRY("ForeColor", DISPID_FORECOLOR, CLSID_StockColorPage)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(clickNodesMode)
	MESSAGE_HANDLER(WM_SETCURSOR,SetCursor)
	MESSAGE_HANDLER(WM_MOUSEMOVE,MouseMove)
	MESSAGE_HANDLER(WM_LBUTTONDOWN,ButtonDown)
	MESSAGE_HANDLER(WM_LBUTTONUP,ButtonUp)
	//MESSAGE_HANDLER(WM_TIMER,OnTimer)
	CHAIN_MSG_MAP(CComControl<clickNodesMode>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT SetCursor(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT MouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT ButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	//LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);


// fetch group item
	IUnknown *getComponent(LPOLESTR name);

// IViewObjectEx
	DECLARE_VIEW_STATUS(0)

// IClickNodesMode
	STDMETHODIMP Initialize(IUnknown *mainLayer);
	STDMETHODIMP AddNode(BSTR id, BSTR caption);
	STDMETHODIMP AddEdge(BSTR idTail, BSTR idHead);

// IPersistStreamInit overrides
	STDMETHODIMP Save(IStream *dest,BOOL clearDirty);
	STDMETHODIMP Load(IStream *source);
public:

	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);
	OLE_COLOR m_clrBackColor;
	CComBSTR m_bstrCaption;
	OLE_COLOR m_clrForeColor;
// INeedOnePointer
	STDMETHOD(TakePointer)(IUnknown * o,IMoniker *m)
	{
		m_mainLayer = o;
		return S_OK;
	}
};

#endif //__CLICKNODESMODE_H_
