/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// GraphView.cpp : implementation file
//

#include "stdafx.h"
#include "histograph2Doc.h"
#include "GraphView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphView

IMPLEMENT_DYNCREATE(CGraphView, CView)

CGraphView::CGraphView()
{
	EnableAutomation();
}

CGraphView::~CGraphView()
{
}

void CGraphView::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CView::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(CGraphView, CView)
	//{{AFX_MSG_MAP(CGraphView)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CGraphView, CView)
	//{{AFX_DISPATCH_MAP(CGraphView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IGraphView to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the
//  dispinterface in the .ODL file.

// {16A9E42C-C741-11D2-83D8-A589F2CAD60B}
static const IID IID_IGraphView =
{ 0x16a9e42c, 0xc741, 0x11d2, { 0x83, 0xd8, 0xa5, 0x89, 0xf2, 0xca, 0xd6, 0xb } };

BEGIN_INTERFACE_MAP(CGraphView, CView)
	INTERFACE_PART(CGraphView, IID_IGraphView, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphView drawing

void CGraphView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView diagnostics

#ifdef _DEBUG
void CGraphView::AssertValid() const
{
	CView::AssertValid();
}

void CGraphView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
CHistograph2Doc* CGraphView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CHistograph2Doc)));
	return (CHistograph2Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraphView message handlers

void CGraphView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	// Activate graph into self
	CHistograph2Doc* doc = (CHistograph2Doc*) GetDocument();
	if (doc && doc->m_graphItem)
	{
		doc->m_graphItem->Activate(OLEIVERB_INPLACEACTIVATE, this);
		sendSize();
	}
}

// stolen from atlwin.h; I find it friendlier than MFC version.
#define HIMETRIC_PER_INCH   2540
#define MAP_PIX_TO_LOGHIM(x,ppli)   ( (HIMETRIC_PER_INCH*(x) + ((ppli)>>1)) / (ppli) )
inline void AtlPixelToHiMetric(const SIZEL * lpSizeInPix, LPSIZEL lpSizeInHiMetric)
{
	int nPixelsPerInchX;    // Pixels per logical inch along width
	int nPixelsPerInchY;    // Pixels per logical inch along height

	HDC hDCScreen = GetDC(NULL);
	_ASSERTE(hDCScreen != NULL);
	nPixelsPerInchX = GetDeviceCaps(hDCScreen, LOGPIXELSX);
	nPixelsPerInchY = GetDeviceCaps(hDCScreen, LOGPIXELSY);
	ReleaseDC(NULL, hDCScreen);

	lpSizeInHiMetric->cx = MAP_PIX_TO_LOGHIM(lpSizeInPix->cx, nPixelsPerInchX);
	lpSizeInHiMetric->cy = MAP_PIX_TO_LOGHIM(lpSizeInPix->cy, nPixelsPerInchY);
}

// set extent of DG/M control so that graph doesn't get crunched/stretched
void CGraphView::sendSize() {
	CHistograph2Doc* doc = (CHistograph2Doc*) GetDocument();
	if (doc && doc->m_graphItem && doc->m_graphItem->IsInPlaceActive()) {
		CRect rc;
		GetClientRect(rc);
		CSize size(rc.Size()),sizeHI;
		AtlPixelToHiMetric(&size,&sizeHI);
		doc->m_graphItem->SetExtent(sizeHI);
		doc->m_graphItem->SetItemRects(rc);
	}
}
void CGraphView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	sendSize();
}

void CGraphView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used.
   CView::OnDestroy();
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
      pActiveItem->Deactivate();
      ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
}

BOOL CGraphView::PreCreateWindow(CREATESTRUCT& cs)
{
	// I have no idea why this isn't automatic here, and why it appears to be
	// automatic for CHistograph2View.
	cs.style |=	WS_CLIPCHILDREN;
	return CView::PreCreateWindow(cs);
}
