/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// histograph2Doc.cpp : implementation of the CHistograph2Doc class
//

#include "stdafx.h"

#include "histograph2Doc.h"
#include "CntrItem.h"
#include "MainFrm.h"
#include "histograph2View.h"
#include "GraphView.h"

#include "exdisp.h"
#include "mshtml.h"
#include "FoolishCreatePointerMoniker.h"
#include "qi.h"
#include "ItemMoniker.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// defined in clickNodes.idl
PTR(IClickNodesModeEvents);
// defined in group.idl
PTR(IGroup);
PTR(IHoldOnePointer);
PTR(IHoldSomePointers);
PTR(IItemSet);
PTR(IOrderedItemList);
PTR(DIPersistor);
// defined in montage.idl
PTR(DIMCCControlMode);
PTR(IMCCLayer);
PTR(DIMCCModeDelegate);
PTR(IMCCModeSwitcher);
PTR(IMCCSite);
PTR(IMCCSitePersistor);
PTR(IMontageContainer);
PTR(IMontagePersistor);
PTR(DIServerCreationPersistor);
PTR(IHTMLDocument2);

#include "exdispid.h"
// This is really silly, but MFC doesn't seem to support sinking of events from
// controls that aren't in dialogs.  If you know how to do this right, please tell me!
struct DispSink : IDispatch {
	DWORD m_dwRef;
	DispSink() : m_dwRef(0) {}
	ULONG STDMETHODCALLTYPE AddRef() {
		return ++m_dwRef;
	}
	ULONG STDMETHODCALLTYPE Release() {
		if(--m_dwRef==0) {
			delete this;
			return 0;
		}
		return m_dwRef;
	}
	STDMETHODIMP QueryInterface(REFIID riid,void **ret) {
		if(riid!=IID_IDispatch&&riid!=IID_IUnknown)
			ERR(E_NOINTERFACE);
		*ret = (IDispatch*)this;
		AddRef();
		return S_OK;
	}
    STDMETHODIMP GetTypeInfoCount(
        /* [out] */ UINT __RPC_FAR *pctinfo) {
		*pctinfo = 0;
		return S_OK;
	}

    STDMETHODIMP GetTypeInfo(
        /* [in] */ UINT iTInfo,
        /* [in] */ LCID lcid,
        /* [out] */ ITypeInfo __RPC_FAR *__RPC_FAR *ppTInfo) {
		ERR(E_NOTIMPL);
	}

    STDMETHODIMP GetIDsOfNames(
        /* [in] */ REFIID riid,
        /* [size_is][in] */ LPOLESTR __RPC_FAR *rgszNames,
        /* [in] */ UINT cNames,
        /* [in] */ LCID lcid,
        /* [size_is][out] */ DISPID __RPC_FAR *rgDispId) {
		ERR(E_NOTIMPL);
    }
    STDMETHODIMP Invoke(
        /* [in] */ DISPID dispIdMember,
        /* [in] */ REFIID riid,
        /* [in] */ LCID lcid,
        /* [in] */ WORD wFlags,
        /* [out][in] */ DISPPARAMS __RPC_FAR *pDispParams,
        /* [out] */ VARIANT __RPC_FAR *pVarResult,
        /* [out] */ EXCEPINFO __RPC_FAR *pExcepInfo,
        /* [out] */ UINT __RPC_FAR *puArgErr) {
		return S_OK;
	}
};
// specialization to sink DWebBrowserEvents2::NavigateComplete2 and send on to doc.
struct ieSink : DispSink {
	CHistograph2Doc *m_dest;
	ieSink(CHistograph2Doc *dest) : m_dest(dest) {}
    STDMETHODIMP Invoke(
        /* [in] */ DISPID dispIdMember,
        /* [in] */ REFIID riid,
        /* [in] */ LCID lcid,
        /* [in] */ WORD wFlags,
        /* [out][in] */ DISPPARAMS __RPC_FAR *pDispParams,
        /* [out] */ VARIANT __RPC_FAR *pVarResult,
        /* [out] */ EXCEPINFO __RPC_FAR *pExcepInfo,
        /* [out] */ UINT __RPC_FAR *puArgErr) {
		VARIANTARG *rgvarg = pDispParams->rgvarg;
		switch(dispIdMember) {
			case DISPID_NAVIGATECOMPLETE2:
				_ASSERTE(pDispParams->cArgs==2);
				_ASSERTE(rgvarg[1].vt==VT_DISPATCH); // pushed in reverse order
				_ASSERTE(rgvarg[0].vt==(VT_VARIANT|VT_BYREF));
				m_dest->NavigateComplete2(rgvarg[1].pdispVal,rgvarg[0].pvarVal->bstrVal);
				break;
		}
		return S_OK;
	}
};
/////////////////////////////////////////////////////////////////////////////
// CHistograph2Doc

IMPLEMENT_DYNCREATE(CHistograph2Doc, COleDocument)

BEGIN_MESSAGE_MAP(CHistograph2Doc, COleDocument)
	//{{AFX_MSG_MAP(CHistograph2Doc)
	ON_BN_CLICKED(IDC_GO, OnGo)
	ON_BN_CLICKED(IDC_Back, OnBack)
	ON_BN_CLICKED(IDC_Forward, OnForward)
	//}}AFX_MSG_MAP
	// Enable default OLE container implementation
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, COleDocument::OnUpdatePasteMenu)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE_LINK, COleDocument::OnUpdatePasteLinkMenu)
	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_CONVERT, COleDocument::OnUpdateObjectVerbMenu)
	ON_COMMAND(ID_OLE_EDIT_CONVERT, COleDocument::OnEditConvert)
	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_LINKS, COleDocument::OnUpdateEditLinksMenu)
	ON_COMMAND(ID_OLE_EDIT_LINKS, COleDocument::OnEditLinks)
	ON_UPDATE_COMMAND_UI_RANGE(ID_OLE_VERB_FIRST, ID_OLE_VERB_LAST, COleDocument::OnUpdateObjectVerbMenu)
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CHistograph2Doc, COleDocument)
	//{{AFX_DISPATCH_MAP(CHistograph2Doc)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IHistograph2 to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the
//  dispinterface in the .ODL file.

// {16A9E3F9-C741-11D2-83D8-A589F2CAD60B}
static const IID IID_IHistograph2 =
{ 0x16a9e3f9, 0xc741, 0x11d2, { 0x83, 0xd8, 0xa5, 0x89, 0xf2, 0xca, 0xd6, 0xb } };

BEGIN_INTERFACE_MAP(CHistograph2Doc, COleDocument)
	INTERFACE_PART(CHistograph2Doc, IID_IHistograph2, Dispatch)
	INTERFACE_PART(CHistograph2Doc,IID_IClickNodesModeEvents,ClickNodesSink)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHistograph2Doc construction/destruction

CHistograph2Doc::CHistograph2Doc() : m_browserItem(0), m_graphItem(0)
{
	// Use OLE compound files
	EnableCompoundFile();

	// TODO: add one-time construction code here

	EnableAutomation();

	AfxOleLockApp();
}

CHistograph2Doc::~CHistograph2Doc()
{
	AfxOleUnlockApp();
}
HRESULT CHistograph2Doc::SinkEvents() {
	DWORD molasses; // forget cookies: since we own the controls, we never need to stop events.
	// receive IClickNodesModeEvents from click nodes mode
	IConnectionPointPtr point;
	RUN(IConnectionPointContainerPtr(m_clickNodes)->FindConnectionPoint(IID_IClickNodesModeEvents,&point));
	RUN(point->Advise(&m_xClickNodesSink,&molasses));
	// receive IWebBrowserEvents2 from IE
	point.Release();
	RUN(IConnectionPointContainerPtr(m_ie)->FindConnectionPoint(DIID_DWebBrowserEvents2,&point));
	IDispatchPtr disp = new ieSink(this);
	RUN(point->Advise(disp,&molasses));
	return S_OK;
}

#include "FullViewClientItem.h"

// Start up an Internet Explorer DocObject, and
// configure Dynagraph for Montage with the Click Nodes Mode control
BOOL CHistograph2Doc::OnNewDocument()
{
	if (!COleDocument::OnNewDocument())
		return FALSE;

	// start afresh with every document
	m_browserItem = 0;
	m_graphItem = 0;
	m_last = _bstr_t();

	// install IE browser in right docobject frame

	// It gets activated automatically by CHistograph2View::OnInitialUpdate
	m_browserItem = new CHistograph2CntrItem(this);
	if(!m_browserItem->CreateNewItem(CLSID_WebBrowser))
		return -1;
	m_ie = m_browserItem->m_lpObject; // keep pointer around for Navigate()

	// install dg/m control on left; customize with clickNodes mode
	m_graphItem = new CFullViewClientItem(this);
	if(!m_graphItem->CreateNewItem(CLSID_DynagraphControl))
		return -1;

	// add clickNodes mode

	// Get pointers to main Montage interface & group
	IMontageContainerPtr mc = m_graphItem->m_lpObject;
	IGroupPtr g;
	RUN(mc->GetMainGroup(riidret(&g)));

	// get layers from dg/m by creating monikers and binding (we need the mks too)
	//standard moniker setup
	IBindCtxPtr bctx;
	RUN(CreateBindCtx(0,&bctx));
	IMonikerPtr mkMontage,mkMainLayerPersistor,mkModeLayerPersistor,
		mkGoSubject(CLSID_GetSubjectMoniker),
		mkMainLayer,mkModeLayer;
	RUN(FoolishCreatePointerMoniker(mc,&mkMontage));
	// monikers to persistors
	RUN(CreateItemMoniker(L"!",L"Mode Layer",&mkModeLayerPersistor));
	RUN(CreateItemMoniker(L"!",L"Main Layer",&mkMainLayerPersistor));
	// compose with GetSubjectMoniker to get monikers to layers themselves
	RUN(mkMainLayerPersistor->ComposeWith(mkGoSubject,false,&mkMainLayer));
	RUN(mkModeLayerPersistor->ComposeWith(mkGoSubject,false,&mkModeLayer));
	// bind to get layers
	IMCCLayerPtr mainl,model;
	RUN(mkMainLayer->BindToObject(bctx,mkMontage,riidret(&mainl)));
	RUN(mkModeLayer->BindToObject(bctx,mkMontage,riidret(&model)));

	// create click nodes mode & init with the layer it should watch
	RUN(m_clickNodes.CreateInstance(CLSID_ClickNodesMode));
	RUN(m_clickNodes->Initialize(mainl));

	// persist clickNodes control
	//monikers
	IMonikerPtr mkClickNodesPersistor;
	RUN(CreateItemMoniker(L"!",L"Click Nodes Mode Persistor",&mkClickNodesPersistor));
	RUN(mkClickNodesPersistor->ComposeWith(mkGoSubject,false,&m_mkClickNodes));
	//persistor
	DIPersistorPtr p(CLSID_OnePointerPersistor);
	//paste into group
	RUN(g->Paste(bctx,GP_SrcIsObj,p,mkClickNodesPersistor,IID_NULL,0,0));
	RUN(IHoldOnePointerPtr(p)->put_Moniker(mkMainLayer));
	RUN(p->SetSubject(true,m_clickNodes));

	// set mode up.
	IMCCSitePtr site;
	RUN(mc->new_Site(&site));
	RUN(site->put_Layer(model));
	RUN(site->put_Control(m_clickNodes));

	// get mode switcher & connect to click nodes mode's site w/ MCCControlMode
	IMonikerPtr mkSwitcher;
	RUN(CreateItemMoniker(L"!",L"Mode Switcher",&mkSwitcher));
	IMCCModeSwitcherPtr switcher;
	RUN(mkSwitcher->BindToObject(bctx,mkMontage,riidret(&switcher)));
	_bstr_t clickNodesString("Click Nodes");
	DIMCCModeDelegatePtr delegate;
	RUN(switcher->new_Delegate(&delegate));
	RUN(delegate->put_Name(clickNodesString));
	RUN(delegate->Activate(false));
	// take over UI by using MCCControlMode (rect=covering) with no mode switcher
	DIMCCControlModePtr owner(CLSID_MCCControlMode);
	RUN(owner->Initialize(delegate));
	RUN(IObjectWithSitePtr(owner)->SetSite(site)); // sets up position
	// activate
	RUN(site->put_Intent(MCCS_ACTIVE));

	// persist site
	IMCCSitePersistorPtr sitePer(CLSID_MCCSitePersistor);
	IMonikerPtr mkClickNodesSite;
	RUN(CreateItemMoniker(L"!",L"Click Nodes Site",&mkClickNodesSite));
	RUN(g->Paste(bctx,GP_SrcIsObj,sitePer,mkClickNodesSite,IID_NULL,0,0));
	IMonikerPtr mkMontagePer,mkModeOrder;
	RUN(mc->get_RelMoniker(&mkMontagePer));
	RUN(CreateItemMoniker(L"!",L"Mode Site Ordering",&mkModeOrder));
	RUN(sitePer->Initialize(mkMontagePer,mkModeLayer,mkModeOrder,m_mkClickNodes,0,0));
	RUN(DIPersistorPtr(sitePer)->SetSubject(true,site));

	// persist delegate
	DIServerCreationPersistorPtr yugo(CLSID_ServerCreationPersistor);
	IMonikerPtr mkGetSubj(CLSID_GetSubjectMoniker),mkDelP,mkDel;
	RUN(CreateItemMoniker(L"!",L"View Mode Delegate",&mkDelP));
	RUN(g->Paste(bctx,GP_SrcIsObj,yugo,mkDelP,IID_NULL,0,0));
	RUN(mkDelP->ComposeWith(mkGetSubj,false,&mkDel));
	IMonikerPtr mkDelClass,mkDelCreator;
	RUN(CreateClassMoniker(CLSID_MCCModeDelegate,&mkDelClass));
	RUN(mkSwitcher->ComposeWith(mkDelClass,false,&mkDelCreator));
	RUN(yugo->put_ServerMoniker(mkDelCreator));
	RUN(DIPersistorPtr(yugo)->SetSubject(true,delegate));


	// persist ControlMode
	IHoldSomePointersPtr hsp(CLSID_SomePointersPersistor);
	IMonikerPtr mkClickNodesOwnerPersistor,mkClickNodesOwner; // inside dg/m code I abbreviate names but that doesn't help much
	RUN(CreateItemMoniker(L"!",L"Click Nodes Owner",&mkClickNodesOwnerPersistor));
	RUN(g->Paste(bctx,GP_SrcIsObj,hsp,mkClickNodesOwnerPersistor,IID_NULL,0,0));
	RUN(hsp->Push(mkDel));
	RUN(DIPersistorPtr(hsp)->SetSubject(true,owner));
	// All this persistence code gets quite huge.  I want to get it at
	// least to be smaller than the actual set up.
	RUN(mkClickNodesOwnerPersistor->ComposeWith(mkGoSubject,false,&mkClickNodesOwner));
	IOrderedItemListPtr oil;
	RUN(sitePer->get_InitList(&oil));
	// mkClickNodesOwner still reffed from Paste
	RUN(oil->SetMoniker(10,mkClickNodesOwner)); // 10?

	// add site to keepset
	IMonikerPtr mkMP;
	RUN(mc->get_RelMoniker(&mkMP));
	IMontagePersistorPtr mp;
	RUN(mkMP->BindToObject(bctx,mkMontage,riidret(&mp)));
	IItemSetPtr keeps;
	RUN(mp->get_KeepSet(&keeps));
	RUN(keeps->Add(mkClickNodesSite));

	// get into both event streams
	RUN(SinkEvents());

	/*
	// grow graph to right
	CComPtr<IDGLayout> engine;
	RUN(ItemMoniker(L"Graph Layout Engine")->BindToObject(bctx,mkMontage,riidret(&engine)));
	RUN(engine->put_Orientation(DGO_RIGHT));
	*/
	return TRUE;
}
/////////////////////////////////////////////
// CHistograph2Doc IClickNodesModeEvents sink
STDMETHODIMP_(ULONG) CHistograph2Doc::XClickNodesSink::AddRef()
{
    METHOD_PROLOGUE(CHistograph2Doc, ClickNodesSink)
    return pThis->ExternalAddRef();
}

STDMETHODIMP_(ULONG) CHistograph2Doc::XClickNodesSink::Release()
{
    METHOD_PROLOGUE(CHistograph2Doc, ClickNodesSink)
    return pThis->ExternalRelease();
}

STDMETHODIMP CHistograph2Doc::XClickNodesSink::QueryInterface(
    REFIID iid, void FAR* FAR* ppvObj)
{
    METHOD_PROLOGUE(CHistograph2Doc, ClickNodesSink)
    return (HRESULT)pThis->ExternalQueryInterface(&iid, ppvObj);
}

STDMETHODIMP CHistograph2Doc::XClickNodesSink::OnClick(BSTR id)
{
    METHOD_PROLOGUE(CHistograph2Doc, ClickNodesSink)
	pThis->m_last = (char*)0;
	// I'm sure the use of _bstr_t here is horribly inefficient, but it's effective
	pThis->m_ie->Navigate(_bstr_t(id),0,0,0,0);
	return S_OK;
}
/////////////////////////////////////////////////////////////////////////////
// CHistograph2Doc serialization
const LPOLESTR g_monikerName = L"Click Nodes Moniker";
void CHistograph2Doc::Serialize(CArchive& ar)
{
	COleDocument::Serialize(ar);
	// persist pointers to particular items by ID
	DWORD graphID,browserID;
	if (ar.IsStoring())
	{
		ar << m_graphItem->m_dwItemNumber;
		ar << m_browserItem->m_dwItemNumber;
	}
	else
	{
		ar >> graphID;
		ar >> browserID;
	}
	_ASSERTE(m_bCompoundFile);
	if(ar.IsStoring()) {
		IStreamPtr stream;
		RUN(m_lpRootStg->CreateStream(g_monikerName,STGM_WRITE|STGM_SHARE_EXCLUSIVE|STGM_CREATE,0,0,&stream));
		RUN(OleSaveToStream(m_mkClickNodes,stream));
	}
	else {
		IStreamPtr stream;
		RUN(m_lpRootStg->OpenStream(g_monikerName,0,STGM_READ|STGM_SHARE_EXCLUSIVE,0,&stream));
		RUN(OleLoadFromStream(stream,riidret(&m_mkClickNodes)));
	}

	if(ar.IsLoading()) {
		m_browserItem = 0;
		m_graphItem = 0;
		POSITION pos = GetStartPosition();
		while(pos) {
			COleClientItem *item = GetNextClientItem(pos);
			if(item->m_dwItemNumber==graphID)
				m_graphItem = item;
			else if(item->m_dwItemNumber==browserID)
				m_browserItem = (CHistograph2CntrItem*)item;

		}
		_ASSERTE(m_browserItem&&m_graphItem);
		m_ie = m_browserItem->m_lpObject;
		IBindCtxPtr bctx;
		RUN(CreateBindCtx(0,&bctx));
		IMonikerPtr mkMontage;
		RUN(FoolishCreatePointerMoniker(m_graphItem->m_lpObject,&mkMontage));
		RUN(m_mkClickNodes->BindToObject(bctx,mkMontage,riidret(&m_clickNodes)));
		RUN(SinkEvents());
	}
}

/////////////////////////////////////////////////////////////////////////////
// CHistograph2Doc diagnostics

#ifdef _DEBUG
void CHistograph2Doc::AssertValid() const
{
	COleDocument::AssertValid();
}

void CHistograph2Doc::Dump(CDumpContext& dc) const
{
	COleDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHistograph2Doc commands
void CHistograph2Doc::NavigateComplete2(LPDISPATCH pDisp, BSTR URL)
{
	CMainFrame *frame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	// get text of node
	BSTR title;
	IDispatchPtr disp;
	IHTMLDocument2Ptr doc;
	if(SUCCEEDED(m_ie->get_Document(&disp)) && ((doc = disp)!=0))
		RUN(doc->get_title(&title));
	else // say, directory
		RUN(m_ie->get_LocationName(&title));
	frame->SetWindowText(CString(title)+" - Hist-O-Graph ["+GetTitle()+"]");
	if(WCHAR *pound = wcschr(URL,L'#')) // cut out anchors
		*pound = 0;
	if(SysStringLen(title)>38)
		wcscpy(title+34,L"...");
	RUN(m_clickNodes->AddNode(URL,title));
	SysFreeString(title);
	if(m_last.length())
		RUN(m_clickNodes->AddEdge(m_last,URL));
	m_last = URL;
	frame->setURL(CString(URL));
}
void CHistograph2Doc::OnCloseDocument()
{
	if(m_graphItem->IsInPlaceActive())
		m_graphItem->Deactivate();

	COleDocument::OnCloseDocument();
}
void CHistograph2Doc::OnGo()
{
	CString url;
	CMainFrame *frame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	frame->getURL(url);
	BSTR bstr = url.AllocSysString();
	RUN(m_ie->Navigate(bstr,0,0,0,0));
	SysFreeString(bstr);
}

void CHistograph2Doc::OnBack()
{
	m_ie->GoBack();
}

void CHistograph2Doc::OnForward()
{
	m_ie->GoForward();
}
