/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "Central.h"
#include <fstream>
#include <sstream>
#include <common/parsestr.h>
#include "a2olestrdup.h"

StringDict g_stringDict;

Central::Central() {
}
Central::~Central() {
}
void writeCommand(ostream &out,DINOGRAPH_INPUT_TYPES type,Central::Defns::iterator di) {
    out << "define ";
    switch(type) {
    case DINOGRAPH_INPUT_PATTERN:
        out << "pattern";
        break;
    case DINOGRAPH_INPUT_SEARCH:
        out << "search";
        break;
    case DINOGRAPH_INPUT_START:
        out << "input";
        break;
    }
    out << endl;
    out << "digraph " << di->first << " {" << di->second << "}" << endl;
}
STDMETHODIMP Central::writeAll(ostream &out) {
    for(int t = 0; t<3; ++t)
        for(Defns::iterator di = m_defns[t].begin(); di!=m_defns[t].end(); ++di)
            writeCommand(out,(DINOGRAPH_INPUT_TYPES)t,di);
    return S_OK;
}
// IDinographCentral
STDMETHODIMP Central::Initialize(IMCCFilePort *port, IMontageContainer *mc, IDGLayout *dynagraph, IDGIncrClient *incrClient) {
    METHTRACE(Central,Initialize);
	RUN(port->RegisterFileFormat(MFP_IMPORT,L"dinog:Dinograph Definition File",this));
	RUN(port->RegisterFileFormat(MFP_EXPORT,L"dinog:Dinograph Definition File",this));
	RUN(mc->get_RefObject(&m_mref));
	m_dynagraph = dynagraph;
    m_incrClient = incrClient;
    return S_OK;
}
STDMETHODIMP Central::SendAll() {
    METHTRACE(Central,SendAll);
    ostringstream all;
    RUN(writeAll(all));
    string s = all.str();
    RUN(m_incrClient->Write(s.c_str(),int(s.length())));
    return S_OK;
}
STDMETHODIMP Central::ChangeInput(DINOGRAPH_INPUT_TYPES type,LPCOLESTR name,LPCOLESTR value) {
    METHTRACE(Central,ChangeInput);
    if(type<0||type>2)
        ERR(E_INVALIDARG);
    USES_CONVERSION;
    DString name2 = W2A(name);
    Defns::iterator di = m_defns[type].find(name2);
    if(di==m_defns[type].end()) {
        if(value==0)
            return S_FALSE;
        di = m_defns[type].insert(make_pair<DString,string>(name2,string())).first;
        RUN(fireEvent(GetUnknown(),fireTwoParams<IDinographCentralEvents,DINOGRAPH_INPUT_TYPES,LPCOLESTR>
            (&IDinographCentralEvents::InputAdded,type,name)));
    }
    if(value==0) {
        RUN(fireEvent(GetUnknown(),fireTwoParams<IDinographCentralEvents,DINOGRAPH_INPUT_TYPES,LPCOLESTR>
            (&IDinographCentralEvents::InputDeleted,type,name)));
        m_defns[type].erase(di);
    }
    else {
        di->second = W2A(value);
        ostringstream os;
        writeCommand(os,type,di);
        string s = os.str();
        RUN(m_incrClient->Write(s.c_str(),int(s.length())));
    }
	return S_OK;
}
STDMETHODIMP Central::GetInput(DINOGRAPH_INPUT_TYPES type,LPCOLESTR name,LPOLESTR *ret) {
    METHTRACE(Central,GetInput);
    if(type<0||type>2)
        ERR(E_INVALIDARG);
    USES_CONVERSION;
    Defns::iterator di = m_defns[type].find(W2A(name));
    if(di==m_defns[type].end())
        ERR(E_FAIL);
    *ret = a2olestrdup(di->second);
 	return S_OK;
}
STDMETHODIMP Central::EnumExisting(DINOGRAPH_INPUT_TYPES type,IEnumString **ret) {
    METHTRACE(Central,EnumExisting);
    if(type<0||type>2)
        ERR(E_INVALIDARG);
    USES_CONVERSION;
    const size_t N = m_defns[type].size();
    LPOLESTR *ray = new LPOLESTR[N],*ri=ray;
    for(Defns::iterator di = m_defns[type].begin();di!=m_defns[type].end(); ++di)
        *ri++ = a2olestrdup(di->first);
	typedef CComObject<CComEnum<IEnumString,&IID_IEnumString,LPOLESTR,_Copy<LPOLESTR> > > en;
	en *e = new en;
	e->Init(ray,ray+N,0,AtlFlagTakeOwnership);
	(*ret = e)->AddRef();
	return S_OK;
}
// IMCCFileFormat
STDMETHODIMP Central::Import(LPCOLESTR filename) {
    METHTRACE(Central,Import);
    USES_CONVERSION;
    fstream f(W2A(filename),fstream::in);
    try {
        while(!f.eof()) {
            DString define,stype,name;
            f >> must("define") >> stype;
            DINOGRAPH_INPUT_TYPES type;
            if(stype=="pattern")
                type = DINOGRAPH_INPUT_PATTERN;
            else if(stype=="search")
                type = DINOGRAPH_INPUT_SEARCH;
            else if(stype=="input")
                type = DINOGRAPH_INPUT_START;
            else
                ERR(E_FAIL);
            f >> must("digraph") >> name;
            bool isNew = m_defns[type].find(name)==m_defns[type].end();
            f >> match('{','}',m_defns[type][name]);
            f >> ws;
            if(isNew)
                RUN(fireEvent(GetUnknown(),fireTwoParams<IDinographCentralEvents,DINOGRAPH_INPUT_TYPES,LPCOLESTR>
                    (&IDinographCentralEvents::InputAdded,type,A2W(name.c_str()))));
        }
    }
    catch(...) {
        ERR(E_FAIL);
    }
    RUN(SendAll());
    /*
    int N = f.tellg();
    // dnu how to reset stream
    f.close();
    fstream f2(W2A(filename),fstream::in);
    char *all = new char[N];
    f2.read(all,N);
    RUN(m_incrClient->Write(all,N));
    */
    return S_OK;
}
STDMETHODIMP Central::Export(LPCOLESTR filename) {
    METHTRACE(Central,Export);
    USES_CONVERSION;
    fstream f(W2A(filename),fstream::out);
    writeAll(f);
    return S_OK;

}
