/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include <commdlg.h>

#include "DinographBoot.h"

#include "MBasics.h"
#include "toolbars.h"
#include "DinographUI.h"
#include "makePathToBin.h"

// DinographBoot

bool openFileDialog(HWND hwnd,char *title,char *filter,char *filename);

STDMETHODIMP DinographBoot::InitializeOther(IUnknown *montage) {
    METHTRACE(DinographBoot,InitializeOther);
	CComPtr<IInitializeOther> dgmboot;
	dgmboot.CoCreateInstance(CLSID_DGMBoot);
	RUNRET(dgmboot->InitializeOther(montage));
	dgmboot.Release();

	CComPtr<IMoniker> mkTDinPanel,mkToolLayer;
	ItemMoniker mkToolbars(L"Toolbars");

	MBasics mbMain;
	RUN(mbMain.init(montage));

	RUN(mbMain.c->put_ClassID(CLSID_DinographControl)); // re-tag
	RUN(mbMain.p->put_Class(CLSID_DinographControl)); // re-tag

    CComPtr<IGroup> toolGroup;
	RUN(mkToolbars->BindToObject(mbMain.bctx,mbMain.mkG,riidret(&toolGroup)));

	MBasics mbTools;
	RUN(mbTools.init(toolGroup));

	ItemMoniker mkToolDrawer(L"Toolbar Drawer");
	ItemMoniker mkToolLayerP(L"Tool Layer");
	RUN(mkToolLayerP->ComposeWith(mbTools.goSubject,false,&mkToolLayer));

	CComPtr<IMCCSiteLayoutEvents> toolDrawer;
	RUN(mkToolDrawer->BindToObject(mbTools.bctx,mbTools.mkG,riidret(&toolDrawer)));
	CComPtr<IMCCLayer> toolLayer;
	RUN(mkToolLayer->BindToObject(mbTools.bctx,mbTools.mkG,riidret(&toolLayer)));

    // activate Edit mode (draw mode is irrelevant and probably not too stable in dinograph)
    CComPtr<DIMCCModeDelegate> editDel;
    RUN(mbMain.bind(ComposeMk(ItemMoniker(L"Edit Mode Delegate"),GetSubjectMk()),&editDel));
    RUN(editDel->Activate(false));

	Toolness toolness(mbTools);

    CComPtr<IDGLayout> dynagraph;
	RUN(ItemMoniker(L"Graph Layout Engine")->BindToObject(mbMain.bctx,mbMain.mkG,riidret(&dynagraph)));
    CComPtr<IDGIncrClient> incrClient;
	RUN(incrClient.CoCreateInstance(CLSID_DGIncrClient));

	CComPtr<IDinographCentral> central;
	RUN(central.CoCreateInstance(CLSID_DinographCentral));

	CComPtr<IMCCFileFormat> port;
    RUN(mbMain.c->get_FilePort(&port));
    RUN(central->Initialize(CComQIPtr<IMCCFilePort>(port),mbMain.c,dynagraph,incrClient));

	CComPtr<IDGMSpine> spine;
	RUN(ComposeMk(ItemMoniker(L"DGM Spine"),mbMain.goSubject)->BindToObject(mbMain.bctx,mbMain.mkG,riidret(&spine)));

	HWND hwnd;
	RUN(CComQIPtr<IOleWindow>(mbMain.c)->GetWindow(&hwnd)); // null
	char filename[MAX_PATH] = "";
    int i = 0;
	while(!openFileDialog(hwnd,"Open a Graph to Search","DOT Files\0*.dot\0\0",filename))
        if(++i>2)
            ERR(E_FAIL);
	char exepath[400];
	OLECHAR buf0[500],buf1[500];
	LPCOLESTR args[2] = {buf0,buf1};
	CRegKey dynagraphOpts;
	ULONG N = 400;
	if(dynagraphOpts.Open(HKEY_CURRENT_USER,"Software\\AT&T\\Dynagraph") ||
			dynagraphOpts.QueryStringValue("Large Graph Server",exepath,&N))
        makePathToBin("graphsearch.exe",exepath);
	char exepath2[402];
	if(strchr(exepath,' ')) {
		wsprintf(exepath2,"\"%s\"",exepath);
		ATLA2WHELPER(buf0,exepath2,(int)strlen(exepath2)+1);
	}
	else
		ATLA2WHELPER(buf0,exepath,(int)strlen(exepath)+1);
	if(strchr(filename,' ')) { // yeesh
        char buf9[500];
		wsprintf(buf9,"\"%s\"",filename);
		ATLA2WHELPER(buf1,buf9,(int)strlen(buf9)+1);
	}
	else
		ATLA2WHELPER(buf1,filename,(int)strlen(filename)+1);

	RUNRET(incrClient->StartServer(args[0],2,args));
    RUN(central->SendAll());

	// replace the Save As file description
	CComPtr<IMCCFileFormat> fport;
	RUN(mbMain.c->get_FilePort(&fport));
	RUN(CComQIPtr<IMCCFilePort>(fport)->RegisterNativeFileFormat(MFP_IMPORT,L"dinogwks:Dinograph Workspace"));
	RUN(CComQIPtr<IMCCFilePort>(fport)->RegisterNativeFileFormat(MFP_EXPORT,L"dinogwks:Dinograph Workspace"));

    CComObject<DinographUI> *memoryLeak = new CComObject<DinographUI>;
    RUN(memoryLeak->start(central,incrClient,mbMain.p));

    return S_OK;
}






bool openFileDialog(HWND hwnd,char *title,char *filter,char *filename) {
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.hInstance = 0;
	ofn.lpstrFilter = filter;
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 0;
	ofn.lpstrFile = filename;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFileTitle = 0;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = 0;
	ofn.lpstrTitle = title;
	ofn.Flags = 0;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = "dot";
	ofn.lCustData = 0;
	ofn.lpfnHook = 0;
	ofn.lpTemplateName = 0;
	return GetOpenFileName(&ofn)!=0;
}
