/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


struct DinographUI :
    CComObjectRoot,
    IDinographCentralEvents
{
    CComPtr<IDinographCentral> m_central;
    CComPtr<IDGIncrClient> m_incrClient;

    CComPtr<IMCCLayer> topL;
    CComPtr<IMontageContainer> topMC,main,search,pattern,input;
    CComPtr<IDGLayout> mainLayout,searchLayout,patternLayout,inputLayout;
    CComPtr<IDGGraph> mainAbstract,searchAbstract,patternAbstract,inputAbstract;
    CComPtr<IDGGraph> dinoMachine;
   	CComPtr<IItemSet> mainSelection,searchSelection,patternSelection,inputSelection;
    CComPtr<IMCCSite> mainS,searchS,patternS,inputS,
		maintitleS,searchtitleS,patterntitleS,inputtitleS,proptitleS;
    CComQIPtr<IFont> titleFont;
    CComPtr<IMCCPane> columns,leftcol,rightcol,searchspot,patternspot,inputspot,propspot,mainspot,
        maintitlespot,searchtitlespot,patterntitlespot,inputtitlespot,proptitlespot;
	CComPtr<IMCCPaneTitle> maintitle,searchtitle,patterntitle,inputtitle,proptitle;
    CComPtr<IMCCPane> searchwind,patternwind,inputwind,propwind;
    CComPtr<IMCCPropertiesPane> proppane;

    CComPtr<IMCCPane> runcaptionspot,runchoicespot;
    CComPtr<IMCCSite> runchoicesite;
    CComPtr<IMCCComboBox> runchoicectl;
    CComPtr<IMCCPane> searchcaptionspot,searchchoicespot,searchsavespot;
    CComPtr<IMCCSite> searchchoicesite,searchsavesite;
    CComPtr<IMCCComboBox> searchchoicectl;
    CComPtr<IMCCButton> searchsavectl;
    CComPtr<IMCCPane> patterncaptionspot,patternchoicespot,patternsavespot;
    CComPtr<IMCCSite> patternchoicesite,patternsavesite;
    CComPtr<IMCCComboBox> patternchoicectl;
    CComPtr<IMCCButton> patternsavectl;
    CComPtr<IMCCPane> inputcaptionspot,inputchoicespot,inputsavespot,inputgrabspot,inputshowspot;
    CComPtr<IMCCSite> inputchoicesite,inputsavesite,inputgrabsite,inputshowsite;
    CComPtr<IMCCComboBox> inputchoicectl;
    CComPtr<IMCCStickyButton> inputsavectl,inputgrabctl,inputshowctl;
    CComPtr<IMCCPane> propcaptionspot,propscopespot,proptypespot,propsavespot;
    CComPtr<IMCCSite> propscopesite,proptypesite,propsavesite;
    CComPtr<IMCCComboBox> propscopectl,proptypectl;
    CComPtr<IMCCButton> propsavectl;
    CComPtr<IMCCPane> searchtextcheckspot,patterntextcheckspot,inputtextcheckspot;
    CComPtr<IMCCButton> searchtextcheckctl,patterntextcheckctl,inputtextcheckctl;
    CComPtr<IMCCSite> searchtextchecksite,patterntextchecksite,inputtextchecksite,
        searchtextsite,patterntextsite,inputtextsite;
    CComPtr<IMCCEdit> searchtextctl,patterntextctl,inputtextctl;
    bool searchusetext,patternusetext,inputusetext;

    // synched with the sticky buttons on input pane
    bool alwaysSave,alwaysGrab,alwaysShow;
    // the options in the property pane
    bool propNodes,propEdges,propAbstract;


    DinographUI();

    CComPtr<IMCCComboBox> getChooser(int type);
    HRESULT start(IDinographCentral *central,IDGIncrClient *incrClient,IUnknown *mp);
    string readGraph(IDGGraph *sourceDiagram);
    HRESULT sendGraph(LPCOLESTR name,LPCOLESTR text,IDGGraph *destDiagram);
    HRESULT save(int id);
    HRESULT selected(int id,BSTR val);
    HRESULT textCheck(int id,bool val);
    HRESULT grab();
    HRESULT show();
    HRESULT selectChange(int id);
    CComPtr<IMontageContainer> activeMC();
    int getActive();
    HRESULT refreshProperties();
    HRESULT saveProperties();
    CComPtr<IItemSet> activeSelection();
    bool filterThing(IUnknown *u);
    HRESULT getSelected(vector<IDGStringAttributes*> &ret);
    HRESULT setType(BSTR val);
    HRESULT setScope(BSTR val);


BEGIN_COM_MAP(DinographUI)
    COM_INTERFACE_ENTRY(IDinographCentralEvents)
END_COM_MAP()

// IDinographCentralEvents
    STDMETHODIMP InputAdded(DINOGRAPH_INPUT_TYPES type,LPCOLESTR name);
    STDMETHODIMP InputDeleted(DINOGRAPH_INPUT_TYPES type,LPCOLESTR name);
};
