/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


// findLeakObjects.cpp : sifts through debug log for ctor/dtor matchings
//

#include <iostream>
#include <fstream>
#include <map>
#include <vector>
#include <string>
#include <assert.h>
#include <algorithm>

using namespace std;

struct structor_info {
	vector<unsigned> constructed,destroyed;
	structor_info() {}
};

typedef map<string,structor_info> cdtorMap;

int main(int argc, char* argv[])
{
	if(argc!=2) {
		cerr << "findLeakObjects: search mtrace log for COM objects that didn't get destroyed\n";
		cerr << "syntax: findLeakObjects filename\n";
		return 1;
	}
	fstream f(argv[1],fstream::in);
	if(!f.is_open()) {
		cerr << "Couldn't open " << argv[0] << endl;
		return 1;
	}
	f >> hex;
	cdtorMap cdtors;
	while(!f.eof()) {
		string what;
		f >> what;
		if(what=="constructing") {
			unsigned pointer;
			string name;
			f >> pointer >> name;
			cdtors[name].constructed.push_back(pointer);
		}
		else if(what=="destroying") {
			unsigned pointer;
			string name;
			f >> pointer >> name;
			cdtors[name].destroyed.push_back(pointer);
		}
	}
	cout << "Stuck:\n";
	vector<string> okay;
	for(cdtorMap::iterator i = cdtors.begin();i!=cdtors.end();++i) {
		structor_info &slot = i->second;
		if(slot.constructed.size()==slot.destroyed.size())
			okay.push_back(i->first);
		else {
			cout << dec;
			cout << i->first << ": " << int(slot.constructed.size()) << " created, " << int(slot.destroyed.size()) << " released: missed ";
			cout << hex;
			sort(slot.constructed.begin(),slot.constructed.end());
			sort(slot.destroyed.begin(),slot.destroyed.end());
			vector<unsigned>::iterator ic = slot.constructed.begin(),
				id = slot.destroyed.begin();
			while(ic != slot.constructed.end()) {
				pair<vector<unsigned>::iterator,vector<unsigned>::iterator> ii;
				ii = mismatch(id,slot.destroyed.end(),ic);
				if(ii.second==slot.constructed.end())
					break;
				if(ii.first!=slot.destroyed.end())
					assert(*ii.second<*ii.first); // no destroy without construct!
				cout << *ii.second << " ";
				id = ii.first;
				ic = ii.second+1;
			}
			cout << endl;
		}
	}
	cout << endl << "Okay:" << endl;
	for(vector<string>::iterator j(okay.begin());j!=okay.end();++j)
		cout << j->c_str() << "(" << int(cdtors[*j].destroyed.size()) << ") ";
	cout << endl;
	return 0;
}
