/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include "stdafx.h"
#include "mtrace.h"
#include <string>
using namespace std;
#include <windows.h>
#include <crtdbg.h>
#include "MTracer.h"
#include "twoCharCode.h"

MTracer::MTracer() : m_debugOut(0) {
}

STDMETHODIMP MTracer::Start() {
    if(m_debugOut)
        return S_FALSE;
    m_callNumber = 0x1717;
    for(int i = 0; i<1000; ++i)
        m_prefs[i] = DONT;
	char logname[3000];
	DWORD nc = GetModuleFileName(_AtlBaseModule.GetModuleInstance(),logname,3000);
    _ASSERTE(nc);
	char *dotp = strrchr(logname,'.');
	_ASSERTE(dotp);
	// if the file mtrace.dolog exists in install directory, then open mtrace.log
    // and use those m_prefs.  otherwise just dump errors to debug.
	strcpy(dotp,".dolog");
    if(FILE *f = fopen(logname,"r")) {
		how_to state = TO_FILE;
        while(!feof(f)) {
			char s[20];
            fscanf(f,"%20s",s);
			if(!stricmp(s,"don't"))
				state = DONT;
			else if(!stricmp(s,"file"))
				state = TO_FILE;
			else if(!stricmp(s,"debug"))
				state = TO_DEBUGGER;
            else if(strlen(s)==2 && isalpha(s[0]) && isalpha(s[1]))
				m_prefs[twoCharCode(s)] = state;
        }
    	fclose(f);
		strcpy(dotp,".log");
        // try to erase file to see if another process already has it
        // proceed to mtrace2.log etc until remove succeeds or the error is file not found
        // (tried to use plain fopen(logname,"w") but it would allow two processes to open?!)
        int N = 2;
		while(remove(logname)==-1 && errno!=ENOENT && N<50)
            sprintf(dotp,"%d.log",N++);
        if(N==50) // gotta have some limit right?
            MessageBox(0,"Have you got a lot of dg-win processes open?!","Dynagraph",MB_OK);
        else {
            m_debugOut = fopen(logname,"w");
		    setvbuf(m_debugOut,0,_IONBF,0);
        }
    }
    else
        m_prefs[C2C(ER)] = TO_DEBUGGER;
    return S_OK;
}
STDMETHODIMP MTracer::Trace(int stream,LPCSTR text) {
    if(stream<0||stream>999)
        return E_FAIL;
    switch(m_prefs[stream]) {
        case DONT:
            return S_FALSE;
        case TO_FILE:
            if(m_debugOut)
		        fprintf(m_debugOut, text);
            break;
        case TO_DEBUGGER:
#ifdef _DEBUG
    		_CrtDbgReport(_CRT_WARN,0,0,0,text);
#endif
            break;
    }
    return S_OK;
}
STDMETHODIMP MTracer::CallNumber(int *ret) {
    *ret = m_callNumber++;
    return S_OK;
}
STDMETHODIMP MTracer::CallDepth(BOOL enter,int *ret) {
    int &count = m_callDepths[GetCurrentThreadId()];
	*ret = enter?++count:count--;
	return S_OK;
}
STDMETHODIMP MTracer::Stop() {
	if(m_debugOut) {
		fprintf(m_debugOut,"MTrace stopped!!!\n");
		fclose(m_debugOut);
		m_debugOut = 0;
	}
	return S_OK;
}
