/**********************************************************
*      This software is part of the graphviz toolset      *
*                http://www.graphviz.org/                 *
*                                                         *
*            Copyright (c) 1994-2005 AT&T Corp.           *
*                and is licensed under the                *
*            Common Public License, Version 1.0           *
*                      by AT&T Corp.                      *
*                                                         *
*        Information and Software Systems Research        *
*              AT&T Research, Florham Park NJ             *
*                                                         *
*               This version available from               *
*                   http://dynagraph.org                  *
**********************************************************/


#include <iostream>
#include <string>
#include <fstream>
#include <stack>
using namespace std;
int main(int nargs,char *args[]) {
    if(nargs!=2)
        return cerr << "mtraceGraph filename(.log)" << endl,1;
    string filename = args[1];
    fstream f(filename.c_str(),fstream::in);
    if(!f.is_open()) {
        if(filename.find_last_of('.')==string::npos)
            filename.append(".log");
        else
            return cerr << "could not open " << filename << endl,2;
        f.open(filename.c_str(),fstream::in);
        if(!f.is_open())
            return cerr << "could not open " << args[1] << " or " << filename << endl,2;
    }
    cout << "digraph G {" << endl;
	stack<int> Stack;
    while(!f.eof()) {
        string what;
        f >> what;
        if(f.eof())
            break;
		if(what=="called") {
			string type;
			int call;
			f >> type >> call;
			if(!Stack.empty()) // edge
				cout << '\t' << Stack.top() << " -> " << call << ';' << endl;
			Stack.push(call);
			// node
            cout << '\t' << call << " [";
            if(type=="function") {
                string func;
                f >> func;
                cout << "label = " << func << ", ";
                cout << "function = " << func;
            }
            else if(type=="method") {
                string pointer,Class,Method;
                f >> pointer >> Class >> Method;
                cout << "label = \"" << Class << "::" << Method << "\", ";
				// method might be destructor so quote
                cout << "this = \"" << pointer << "\", class = " << Class << ", method = " << '"' << Method << '"';
            }
            else if(type=="imethod") {
                string pointer,Class,Interface,Method;
                f >> pointer >> Class >> Interface >> Method;
                cout << "this = \"" << pointer << "\", class = " << Class << ", interface = " << Interface << ", method = " << Method;
            }
            else
                return cerr << "bad called type " << type << endl, 3;
			string sourcefile;
			int line;
			f >> sourcefile >> line;
            cout << ", sourcefile = \"" << sourcefile << "\", line = " << line << "];" << endl;
        }
		else if(what=="returned") {
			int call;
			f >> call;
			if(call!=Stack.top())
				return cerr << "stack broken: in call " << Stack.top() << ", found \"returned " << call << '"' << endl,4;
			Stack.pop();
		}
        else
            return cerr << "bad log what \"" << what << '"' << endl,3;
    }
    cout << "}" << endl;
    return 0;
}
